@@ -1,22 +1,23 @@
 #!/bin/sh
 
 #   configure the requirements
-AMV="automake (GNU automake) 1.10"
-ACV="autoconf (GNU Autoconf) 2.61"
-LTV="libtoolize (GNU libtool) 1.5.24"
+AMV="automake (GNU automake) 1.11"
+ACV="autoconf (GNU Autoconf) 2.63"
+LTV="libtoolize (GNU libtool) 2.2.6"
 GTT="gettextize (GNU gettext-tools) 0.17"
 USAGE="
 To build RPM from plain CVS sources the following
 installed developer tools are mandatory:
-    GNU automake  1.10
-    GNU autoconf  2.61
-    GNU libtool   1.5.24
-    GNU gettext   0.17
+    $AMV
+    $ACV
+    $LTV
+    $GTT
 "
 
 #   wrapper for running GNU libtool's libtoolize(1)
 libtoolize () {
     _libtoolize=`which glibtoolize 2>/dev/null`
+    _libtoolize_args="$*"
     case "$_libtoolize" in
         /* ) ;;
         *  ) _libtoolize=`which libtoolize 2>/dev/null`
@@ -26,7 +27,11 @@ libtoolize () {
              esac
              ;;
     esac
-    $_libtoolize ${1+"$@"}
+    _libtoolize_version="`$_libtoolize --version | sed -e '1q' | sed -e 's;^[^0-9]*;;'`"
+    case "$_libtoolize_version" in
+        1.* ) _libtoolize_args=`echo "X$_libtoolize_args" | sed -e 's;^X;;' -e 's;--quiet;;' -e 's;--install;;'` ;;
+    esac
+    eval $_libtoolize $_libtoolize_args
 }
 
 #   requirements sanity check
@@ -35,12 +40,18 @@ libtoolize () {
 [ "`libtoolize --version | head -1`" != "$LTV" ] && echo "$USAGE" # && exit 1
 [ "`gettextize --version | head -1 | sed -e 's;^.*/\\(gettextize\\);\\1;'`" != "$GTT" ] && echo "$USAGE" # && exit 1
 
-echo "===> zlib"
-( cd zlib && sh ./autogen.sh --noconfigure "$@" )
-echo "<=== zlib"
+echo "===> pcre"
+( cd pcre && sh ./autogen.sh --noconfigure "$@" )
+echo "<=== pcre"
+echo "===> xz"
+( cd xz && sh ./autogen.sh --noconfigure "$@" )
+echo "<=== xz"
 echo "===> file"
 ( cd file && sh ./autogen.sh --noconfigure "$@" )
 echo "<=== file"
+echo "===> syck"
+( cd syck && sh ./autogen.sh --noconfigure "$@" )
+echo "<=== syck"
 echo "===> xar"
 ( cd xar && sh ./autogen.sh --noconfigure "$@" )
 echo "<=== xar"
@@ -48,7 +59,7 @@ echo "<=== xar"
 echo "===> rpm"
 rm -rf autom4te.cache || true
 echo "---> generate files via GNU libtool (libtoolize)"
-libtoolize --copy --force
+libtoolize --quiet --copy --force --install
 echo "---> generate files via GNU gettext (autopoint)"
 autopoint --force
 echo "---> generate files via GNU autoconf (aclocal, autoheader)"
