@@ -27,7 +27,7 @@
  */
 /*
  * file.h - definitions for file(1) program
- * @(#)$Id: file.h,v 1.72 2005/10/17 15:36:22 christos Exp $
+ * @(#)$File: file.h,v 1.91 2007/03/25 03:13:47 christos Exp $
  */
 
 #ifndef __file_h__
@@ -46,9 +46,13 @@
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
 #endif
+#include <regex.h>
+#include <sys/types.h>
 /* Do this here and now, because struct stat gets re-defined on solaris */
 #include <sys/stat.h>
 
+#define ENABLE_CONDITIONALS
+
 #ifndef MAGIC
 #define MAGIC "/etc/magic"
 #endif
@@ -65,6 +69,28 @@
 #endif
 #define public
 
+#ifndef __GNUC_PREREQ__
+#ifdef __GNUC__
+#define	__GNUC_PREREQ__(x, y)						\
+	((__GNUC__ == (x) && __GNUC_MINOR__ >= (y)) ||			\
+	 (__GNUC__ > (x)))
+#else
+#define	__GNUC_PREREQ__(x, y)	0
+#endif
+#endif
+
+#ifndef f__unused
+#if __GNUC_PREREQ__(2, 7)
+#define	f__unused	__attribute__((__unused__))
+#else
+#define	f__unused	/* delete */
+#endif
+#endif
+
+#ifndef MIN
+#define	MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#endif
+
 #ifndef HOWMANY
 # define HOWMANY (256 * 1024)	/* how much of the file to look at */
 #endif
@@ -73,7 +99,7 @@
 #define MAXstring 32		/* max leng of "string" types */
 
 #define MAGICNO		0xF11E041C
-#define VERSIONNO	2
+#define VERSIONNO	4
 #define FILE_MAGICSIZE	(32 * 4)
 
 #define	FILE_LOAD	0
@@ -85,17 +111,20 @@ struct magic {
 	uint16_t cont_level;	/* level of ">" */
 	uint8_t nospflag;	/* supress space character */
 	uint8_t flag;
-#define INDIR	1		/* if '>(...)' appears,  */
-#define	UNSIGNED 2		/* comparison is unsigned */
-#define OFFADD	4		/* if '>&' appears,  */
-#define INDIROFFADD	8	/* if '>&(' appears,  */
+#define INDIR		1	/* if '(...)' appears */
+#define OFFADD		2	/* if '>&' or '>...(&' appears */
+#define INDIROFFADD	4	/* if '>&(' appears */
+#define	UNSIGNED	8	/* comparison is unsigned */
+
 	/* Word 2 */
 	uint8_t reln;		/* relation (0=eq, '>'=gt, etc) */
 	uint8_t vallen;		/* length of string value, if any */
 	uint8_t type;		/* int, short, long or string. */
 	uint8_t in_type;	/* type of indirrection */
+#define 			FILE_INVALID	0
 #define 			FILE_BYTE	1
 #define				FILE_SHORT	2
+#define				FILE_DEFAULT	3
 #define				FILE_LONG	4
 #define				FILE_STRING	5
 #define				FILE_DATE	6
@@ -113,61 +142,45 @@ struct magic {
 #define				FILE_BESTRING16	18
 #define				FILE_LESTRING16	19
 #define				FILE_SEARCH	20
+#define				FILE_MEDATE	21
+#define				FILE_MELDATE	22
+#define				FILE_MELONG	23
+#define				FILE_QUAD	24
+#define				FILE_LEQUAD	25
+#define				FILE_BEQUAD	26
+#define				FILE_QDATE	27
+#define				FILE_LEQDATE	28
+#define				FILE_BEQDATE	29
+#define				FILE_QLDATE	30
+#define				FILE_LEQLDATE	31
+#define				FILE_BEQLDATE	32
+#define				FILE_NAMES_SIZE	33/* size of array to contain all names */
 
-#define				FILE_FORMAT_NAME	\
-/* 0 */ 			"invalid 0",		\
-/* 1 */				"byte",			\
-/* 2 */ 			"short",		\
-/* 3 */ 			"invalid 3",		\
-/* 4 */ 			"long",			\
-/* 5 */ 			"string",		\
-/* 6 */ 			"date",			\
-/* 7 */ 			"beshort",		\
-/* 8 */ 			"belong",		\
-/* 9 */ 			"bedate",		\
-/* 10 */ 			"leshort",		\
-/* 11 */ 			"lelong",		\
-/* 12 */ 			"ledate",		\
-/* 13 */ 			"pstring",		\
-/* 14 */ 			"ldate",		\
-/* 15 */ 			"beldate",		\
-/* 16 */ 			"leldate",		\
-/* 17 */ 			"regex",		\
-/* 18 */			"bestring16",		\
-/* 19 */			"lestring16",		\
-/* 20 */ 			"search",
+#define IS_STRING(t) \
+	((t) == FILE_STRING || \
+	 (t) == FILE_PSTRING || \
+	 (t) == FILE_BESTRING16 || \
+	 (t) == FILE_LESTRING16 || \
+	 (t) == FILE_REGEX || \
+	 (t) == FILE_SEARCH || \
+	 (t) == FILE_DEFAULT)
 
-#define	FILE_FMT_NUM	"cduxXi"
-#define FILE_FMT_STR	"s"	
-
-#define				FILE_FORMAT_STRING	\
-/* 0 */ 			NULL,			\
-/* 1 */				FILE_FMT_NUM,		\
-/* 2 */ 			FILE_FMT_NUM,		\
-/* 3 */ 			NULL,			\
-/* 4 */ 			FILE_FMT_NUM,		\
-/* 5 */ 			FILE_FMT_STR,		\
-/* 6 */ 			FILE_FMT_STR,		\
-/* 7 */ 			FILE_FMT_NUM,		\
-/* 8 */ 			FILE_FMT_NUM,		\
-/* 9 */ 			FILE_FMT_STR,		\
-/* 10 */ 			FILE_FMT_NUM,		\
-/* 11 */ 			FILE_FMT_NUM,		\
-/* 12 */ 			FILE_FMT_STR,		\
-/* 13 */ 			FILE_FMT_STR,		\
-/* 14 */ 			FILE_FMT_STR,		\
-/* 15 */ 			FILE_FMT_STR,		\
-/* 16 */ 			FILE_FMT_STR,		\
-/* 17 */ 			FILE_FMT_STR,		\
-/* 18 */			FILE_FMT_STR,		\
-/* 19 */			FILE_FMT_STR,		\
-/* 20 */			FILE_FMT_STR,
+#define FILE_FMT_NONE 0
+#define FILE_FMT_NUM  1 /* "cduxXi" */
+#define FILE_FMT_STR  2 /* "s" */
+#define FILE_FMT_QUAD 3 /* "ll" */
 
 	/* Word 3 */
 	uint8_t in_op;		/* operator for indirection */
 	uint8_t mask_op;	/* operator for mask */
+#ifdef ENABLE_CONDITIONALS
+	uint8_t cond;		/* conditional type */
+	uint8_t dummy1;	
+#else
 	uint8_t dummy1;	
 	uint8_t dummy2;	
+#endif
+
 #define				FILE_OPS	"&|^+-*/%"
 #define				FILE_OPAND	0
 #define				FILE_OPOR	1
@@ -177,43 +190,65 @@ struct magic {
 #define				FILE_OPMULTIPLY	5
 #define				FILE_OPDIVIDE	6
 #define				FILE_OPMODULO	7
+#define				FILE_OPS_MASK	0x07 /* mask for above ops */
+#define				FILE_UNUSED_1	0x08
+#define				FILE_UNUSED_2	0x10
+#define				FILE_UNUSED_3	0x20
 #define				FILE_OPINVERSE	0x40
 #define				FILE_OPINDIRECT	0x80
+
+#ifdef ENABLE_CONDITIONALS
+#define				COND_NONE	0
+#define				COND_IF		1
+#define				COND_ELIF	2
+#define				COND_ELSE	3
+#endif /* ENABLE_CONDITIONALS */
+
 	/* Word 4 */
 	uint32_t offset;	/* offset to magic number */
 	/* Word 5 */
 	int32_t in_offset;	/* offset from indirection */
 	/* Word 6 */
-	uint32_t mask;	/* mask before comparison with value */
-	/* Word 7 */
-	uint32_t dummy3;
-	/* Word 8 */
-	uint32_t dummp4;
+	uint32_t lineno;	/* line number in magic file */
+	/* Word 7,8 */
+	union {
+		uint64_t _mask;	/* for use with numeric and date types */
+		struct {
+			uint32_t _count;	/* repeat/line count */
+			uint32_t _flags;	/* modifier flags */
+		} _s;		/* for use with string types */
+	} _u;
+#define num_mask _u._mask
+#define str_count _u._s._count
+#define str_flags _u._s._flags
+
 	/* Words 9-16 */
 	union VALUETYPE {
 		uint8_t b;
 		uint16_t h;
 		uint32_t l;
-		char s[MAXstring];
-		struct {
-/*@relnull@*/
-			char *buf;
-			size_t buflen;
-		} search;
+		uint64_t q;
 		uint8_t hs[2];	/* 2 bytes of a fixed-endian "short" */
 		uint8_t hl[4];	/* 4 bytes of a fixed-endian "long" */
+		uint8_t hq[8];	/* 8 bytes of a fixed-endian "quad" */
+		char s[MAXstring];	/* the search string or regex pattern */
 	} value;		/* either number or string */
 	/* Words 17..31 */
 	char desc[MAXDESC];	/* description */
 };
 
 #define BIT(A)   (1 << (A))
-#define STRING_IGNORE_LOWERCASE		BIT(0)
-#define STRING_COMPACT_BLANK		BIT(1)
-#define STRING_COMPACT_OPTIONAL_BLANK	BIT(2)
-#define CHAR_IGNORE_LOWERCASE		'c'
+#define STRING_COMPACT_BLANK		BIT(0)
+#define STRING_COMPACT_OPTIONAL_BLANK	BIT(1)
+#define STRING_IGNORE_LOWERCASE		BIT(2)
+#define STRING_IGNORE_UPPERCASE		BIT(3)
+#define REGEX_OFFSET_START		BIT(4)
 #define CHAR_COMPACT_BLANK		'B'
 #define CHAR_COMPACT_OPTIONAL_BLANK	'b'
+#define CHAR_IGNORE_LOWERCASE		'c'
+#define CHAR_IGNORE_UPPERCASE		'C'
+#define CHAR_REGEX_OFFSET_START		's'
+#define STRING_IGNORE_CASE		(STRING_IGNORE_LOWERCASE|STRING_IGNORE_UPPERCASE)
 
 
 /* list of magic entries */
@@ -223,58 +258,74 @@ struct mlist {
 	int mapped;  /* allocation type: 0 => apprentice_file
 		      *                  1 => apprentice_map + malloc
 		      *                  2 => apprentice_map + mmap */
-/*@dependent@*/
 	struct mlist *next, *prev;
 };
 
 struct magic_set {
-/*@relnull@*/
+/*@null@*/
     struct mlist *mlist;
     struct cont {
 	size_t len;
-/*@relnull@*/
-	int32_t *off;
+		struct level_info {
+			int32_t off;
+			int got_match;
+#ifdef ENABLE_CONDITIONALS
+			int last_match;
+			int last_cond;	/* used for error checking by parse() */
+#endif
+		} *li;
     } c;
     struct out {
 	/* Accumulation buffer */
-/*@owned@*/ /*@relnull@*/
 	char *buf;
-/*@dependent@*/ /*@relnull@*/
 	char *ptr;
-	size_t len;
+		size_t left;
 	size_t size;
 	/* Printable buffer */
-/*@only@*/
 	char *pbuf;
 	size_t psize;
     } o;
+	uint32_t offset;
     int error;
     int flags;
     int haderr;
     const char *file;
-    size_t line;
+	size_t line;			/* current magic line number */
+
+	/* data for searches */
+	struct {
+		const char *s;		/* start of search in original source */
+		size_t s_len;		/* length of search region */
+		size_t offset;		/* starting offset in source: XXX - should this be off_t? */
+		size_t rm_len;		/* match length */
+	} search;
+
+	union VALUETYPE ms_value;	/* either number or string */
 };
 
 struct stat;
 /*@observer@*/
 protected const char *file_fmttime(uint32_t, int)
 	/*@*/;
-protected int file_buffer(struct magic_set *ms, int, const void *, size_t)
+protected int file_buffer(struct magic_set *ms, int fd, /*@null@*/ const char *inname, const void *buf,
+    size_t nb)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
-protected int file_fsmagic(struct magic_set *ms, /*@null@*/ const char *fn, struct stat *sb)
+protected int file_fsmagic(struct magic_set *ms, const char *, struct stat *sb)
 	/*@modifies ms, sb @*/;
-protected int file_pipe2file(struct magic_set *ms, int fd, const void *startbuf, size_t nbytes)
+protected int file_pipe2file(struct magic_set *ms, int, const void *, size_t)
 	/*@globals errno, fileSystem, internalState @*/
 	/*@modifies ms, errno, fileSystem, internalState @*/;
 protected int file_printf(struct magic_set *ms, const char *, ...)
 	/*@modifies ms @*/;
 protected int file_reset(struct magic_set *ms)
 	/*@modifies ms @*/;
-protected int file_tryelf(struct magic_set *ms, int, const unsigned char *, size_t)
+protected int file_tryelf(struct magic_set *ms, int, const unsigned char *,
+    size_t)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
-protected int file_zmagic(struct magic_set *ms, int, const unsigned char *, size_t)
+protected int file_zmagic(struct magic_set *ms, int fd, /*@null@*/ const char *name,
+    const unsigned char *buf, size_t nbytes)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
 protected int file_ascmagic(struct magic_set *ms, const unsigned char *, size_t)
@@ -288,26 +339,29 @@ protected int file_softmagic(struct magic_set *ms, const unsigned char *, size_t
 protected struct mlist *file_apprentice(struct magic_set *ms, const char *, int)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
-protected uint32_t file_signextend(struct magic_set *ms, struct magic *, uint32_t)
+protected uint64_t file_signextend(struct magic_set *ms, struct magic *,
+    uint64_t)
 	/*@globals fileSystem @*/
 	/*@modifies ms, fileSystem @*/;
-protected void file_delmagic(struct magic *p, int type, size_t entries)
+protected void file_delmagic(/*@only@*/ struct magic *p, int type, size_t entries)
 	/*@globals fileSystem @*/
 	/*@modifies p, fileSystem @*/;
 protected void file_badread(struct magic_set *ms)
 	/*@modifies ms @*/;
 protected void file_badseek(struct magic_set *ms)
 	/*@modifies ms @*/;
-protected void file_oomem(struct magic_set *ms)
+protected void file_oomem(struct magic_set *ms, size_t)
 	/*@modifies ms @*/;
-protected void file_error(struct magic_set *ms, int, /*@null@*/ const char *, ...)
+protected void file_error(struct magic_set *ms, int error, const char *f, ...)
 	/*@modifies ms @*/;
-protected void file_magwarn(struct magic_set *ms, /*@null@*/ const char *, ...)
+protected void file_magerror(struct magic_set *ms, const char *, ...)
+	/*@modifies ms @*/;
+protected void file_magwarn(struct magic_set *ms, const char *, ...)
+	/*@globals stdout, stderr, fileSystem @*/
+	/*@modifies ms, stdout, stderr, fileSystem @*/;
+protected void file_mdump(struct magic *)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
-protected void file_mdump(struct magic *m)
-	/*@globals fileSystem @*/
-	/*@modifies m, fileSystem @*/;
 protected void file_showstr(FILE *fp, const char *, size_t)
 	/*@globals fileSystem @*/
 	/*@modifies fp, fileSystem @*/;
@@ -316,6 +370,15 @@ protected size_t file_mbswidth(const char *)
 /*@observer@*/
 protected const char *file_getbuffer(struct magic_set *ms)
 	/*@modifies ms @*/;
+protected ssize_t sread(int fd, void *buf, size_t n, int canbepipe)
+	/*@modifies buf @*/;
+protected int file_check_mem(struct magic_set *ms, unsigned int)
+	/*@modifies ms @*/;
+
+#ifndef COMPILE_ONLY
+extern const char *file_names[];
+extern const size_t file_nnames;
+#endif
 
 #ifndef HAVE_STRERROR
 extern int sys_nerr;
@@ -329,7 +392,8 @@ extern char *sys_errlist[];
 #endif
 
 #ifndef HAVE_SNPRINTF
-int snprintf(char *, size_t, const char *, ...);
+int snprintf(char *, size_t, const char *, ...)
+	/*@*/;
 #endif
 
 #if defined(HAVE_MMAP) && defined(HAVE_SYS_MMAN_H) && !defined(QUICK)
@@ -341,9 +405,8 @@ int snprintf(char *, size_t, const char *, ...);
 #endif
 
 #define FILE_RCSID(id) \
-static const char *rcsid(const char *p) { \
+static const char *rcsid(const char *p) /*@*/ { \
 	return rcsid(p = id); \
 }
-#else
 
 #endif /* __file_h__ */
