@@ -39,18 +39,15 @@
 #if defined(HAVE_LIMITS_H)
 #include <limits.h>
 #endif
-#ifndef SIZE_T_MAX
-#ifdef __LP64__
-#define SIZE_T_MAX (size_t)0xfffffffffffffffffU
-#else
-#define SIZE_T_MAX (size_t)0xffffffffU
-#endif
-#endif
 
 #ifndef	lint
 FILE_RCSID("@(#)$File: funcs.c,v 1.32 2007/05/24 17:22:27 christos Exp $")
 #endif	/* lint */
 
+#ifndef SIZE_MAX
+#define SIZE_MAX	((size_t)~0)
+#endif
+
 #ifndef HAVE_VSNPRINTF
 int vsnprintf(char *, size_t, const char *, va_list)
 	/*@*/;
@@ -258,7 +255,7 @@ file_getbuffer(struct magic_set *ms)
 
 	len = ms->o.size - ms->o.left;
 	/* * 4 is for octal representation, + 1 is for NUL */
-	if (len > (SIZE_T_MAX - 1) / 4) {
+	if (len > (SIZE_MAX - 1) / 4) {
 		file_oomem(ms, len);
 		return NULL;
 	}
