@@ -29,56 +29,62 @@
 
 #include <sys/types.h>
 
-#define	MAGIC_NONE		0x000	/* No flags */
-#define	MAGIC_DEBUG		0x001	/* Turn on debugging */
-#define	MAGIC_SYMLINK		0x002	/* Follow symlinks */
-#define	MAGIC_COMPRESS		0x004	/* Check inside compressed files */
-#define	MAGIC_DEVICES		0x008	/* Look at the contents of devices */
-#define	MAGIC_MIME		0x010	/* Return a mime string */
-#define	MAGIC_CONTINUE		0x020	/* Return all matches */
-#define	MAGIC_CHECK		0x040	/* Print warnings to stderr */
-#define	MAGIC_PRESERVE_ATIME	0x080	/* Restore access time on exit */
-#define	MAGIC_RAW		0x100	/* Don't translate unprintable chars */
-#define	MAGIC_ERROR		0x200	/* Handle ENOENT etc as real errors */
+#define	MAGIC_NONE		0x000000 /* No flags */
+#define	MAGIC_DEBUG		0x000001 /* Turn on debugging */
+#define	MAGIC_SYMLINK		0x000002 /* Follow symlinks */
+#define	MAGIC_COMPRESS		0x000004 /* Check inside compressed files */
+#define	MAGIC_DEVICES		0x000008 /* Look at the contents of devices */
+#define	MAGIC_MIME		0x000010 /* Return a mime string */
+#define	MAGIC_CONTINUE		0x000020 /* Return all matches */
+#define	MAGIC_CHECK		0x000040 /* Print warnings to stderr */
+#define	MAGIC_PRESERVE_ATIME	0x000080 /* Restore access time on exit */
+#define	MAGIC_RAW		0x000100 /* Don't translate unprintable chars */
+#define	MAGIC_ERROR		0x000200 /* Handle ENOENT etc as real errors */
+#define MAGIC_NO_CHECK_COMPRESS	0x001000 /* Don't check for compressed files */
+#define MAGIC_NO_CHECK_TAR	0x002000 /* Don't check for tar files */
+#define MAGIC_NO_CHECK_SOFT	0x004000 /* Don't check magic entries */
+#define MAGIC_NO_CHECK_APPTYPE	0x008000 /* Don't check application type */
+#define MAGIC_NO_CHECK_ELF	0x010000 /* Don't check for elf details */
+#define MAGIC_NO_CHECK_ASCII	0x020000 /* Don't check for ascii files */
+#define MAGIC_NO_CHECK_TROFF	0x040000 /* Don't check ascii/troff */
+#define MAGIC_NO_CHECK_FORTRAN	0x080000 /* Don't check ascii/fortran */
+#define MAGIC_NO_CHECK_TOKENS	0x100000 /* Don't check ascii/tokens */
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-typedef /*@abstract@*/ struct magic_set *magic_t;
-
+typedef struct magic_set *magic_t;
 /*@null@*/
-magic_t magic_open(int flags)
+magic_t magic_open(int)
 	/*@globals errno @*/
 	/*@modifies errno @*/;
-void magic_close(/*@only@*/ magic_t ms)
+void magic_close(magic_t ms)
 	/*@globals fileSystem @*/
 	/*@modifies ms, fileSystem @*/;
 
-/*@null@*/ /*@observer@*/
+/*@observer@*/ /*@null@*/
 const char *magic_file(magic_t ms, /*@null@*/ const char * inname)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies ms, fileSystem, internalState @*/;
-/*@null@*/ /*@observer@*/
-const char *magic_buffer(magic_t ms, const void * buf, size_t nb)
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies ms, errno, fileSystem, internalState @*/;
+/*@observer@*/ /*@null@*/
+const char *magic_buffer(magic_t ms, const void *, size_t)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
 
 /*@observer@*/
 const char *magic_error(magic_t ms)
 	/*@modifies ms @*/;
-/*@-exportlocal@*/
 int magic_setflags(magic_t ms, int)
 	/*@modifies ms @*/;
-/*@=exportlocal@*/
 
-int magic_load(magic_t ms, const char * magicfile)
+int magic_load(magic_t ms, const char *)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
-int magic_compile(magic_t ms, const char * magicfile)
+int magic_compile(magic_t ms, const char *)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
-int magic_check(magic_t ms, const char * magicfile)
+int magic_check(magic_t ms, const char *)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, fileSystem, internalState @*/;
 int magic_errno(magic_t ms)
