@@ -5,12 +5,12 @@
 #include "system.h"
 
 #include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
+#include <rpmiotypes.h>		/* XXX fnpyKey */
 #include <rpmmacro.h>		/* XXX for %_i18ndomains */
 
 #define	_RPMTAG_INTERNAL
 #include <rpmtag.h>
-#include <rpmlib.h>
+#include <rpmtypes.h>
 
 #define	_RPMEVR_INTERNAL
 #include <rpmds.h>
@@ -62,7 +62,7 @@ static int fssizesTag(Header h, HE_t he)
     rpmTagData fnames = { .ptr = NULL };
     rpmTagData fsizes = { .ptr = NULL };
     rpmTagData p;
-    uint64_t *usages;
+    rpmuint64_t *usages;
     int numFiles;
     int rc = 1;		/* assume error */
     int xx, yy;
@@ -225,7 +225,9 @@ static int missingokTag(Header h, HE_t he)
     size_t nb = 0;
     int i;
 
-assert(ds != NULL);
+    if (ds == NULL)
+	return 1;
+
     /* Collect dependencies marked as hints. */
     ds = rpmdsInit(ds);
     if (ds != NULL)
@@ -252,9 +254,8 @@ assert(ds != NULL);
 	*t++ = '\0';
     }
     av = argvFree(av);
-    ds = rpmdsFree(ds);
-
-    /* XXX perhaps return "(none)" inband if no suggests/enhances <shrug>. */
+    (void)rpmdsFree(ds);
+    ds = NULL;
 
     he->t = RPM_STRING_ARRAY_TYPE;
     he->p.argv = argv;
