@@ -4,6 +4,7 @@
 
 #include "system.h"
 #include <rpmio.h>
+#include <rpmiotypes.h>
 #include <rpmlog.h>
 #include <rpmmacro.h>	/* XXX for rpmGetPath */
 
@@ -110,16 +111,27 @@ static int getFilesystemList(void)
 
 	filesystems[i].mntPoint = fsnames[i] = fsn;
 	
-	if (stat(filesystems[i].mntPoint, &sb)) {
-	    rpmlog(RPMLOG_ERR, _("failed to stat %s: %s\n"), fsnames[i],
+#if defined(RPM_VENDOR_OPENPKG) /* always-skip-proc-filesystem */
+	if (!(strcmp(fsn, "/proc") == 0)) {
+#endif
+	if (Stat(fsn, &sb) < 0) {
+	    switch(errno) {
+	    default:
+		rpmlog(RPMLOG_ERR, _("failed to stat %s: %s\n"), fsn,
 			strerror(errno));
-
-	    rpmFreeFilesystems();
-	    return 1;
+		rpmFreeFilesystems();
+		return 1;
+	    case ENOENT:	/* XXX avoid /proc if leaked into *BSD jails. */
+		sb.st_dev = 0;	/* XXXX make sure st_dev is initialized. */
+		/*@switchbreak@*/ break;
+	    }
 	}
 	
 	filesystems[i].dev = sb.st_dev;
 	filesystems[i].rdonly = rdonly;
+#if defined(RPM_VENDOR_OPENPKG) /* always-skip-proc-filesystem */
+        }
+#endif
 
 	/* goto the next vmount structure: */
 	vm = (struct vmount *)((char *)vm + vm->vmt_length);
@@ -139,7 +151,7 @@ static int getFilesystemList(void)
  * @return		0 on success, 1 on error
  */
 static int getFilesystemList(void)
-	/*@globals filesystems, fsnames, numFilesystems,
+	/*@globals h_errno, filesystems, fsnames, numFilesystems,
 		fileSystem, internalState @*/
 	/*@modifies filesystems, fsnames, numFilesystems,
 		fileSystem, internalState @*/
@@ -213,7 +225,12 @@ static int getFilesystemList(void)
 	    mntdir = mounts[nextMount++].f_mntonname;
 #	endif
 
-	if (stat(mntdir, &sb)) {
+#if defined(RPM_VENDOR_OPENPKG) /* always-skip-proc-filesystem */
+	if (strcmp(mntdir, "/proc") == 0)
+		continue;
+#endif
+
+	if (Stat(mntdir, &sb) < 0) {
 	    switch(errno) {
 	    default:
 		rpmlog(RPMLOG_ERR, _("failed to stat %s: %s\n"), mntdir,
@@ -221,6 +238,7 @@ static int getFilesystemList(void)
 		rpmFreeFilesystems();
 		return 1;
 		/*@notreached@*/ /*@switchbreak@*/ break;
+	    case ENOENT:	/* XXX avoid /proc if leaked into *BSD jails. */
 	    case EACCES:	/* XXX fuse fs #220991 */
 	    case ESTALE:
 		continue;
@@ -268,7 +286,7 @@ static int getFilesystemList(void)
 }
 #endif	/* HAVE_MNTCTL */
 
-int rpmGetFilesystemList(const char *** listptr, uint32_t * num)
+int rpmGetFilesystemList(const char *** listptr, rpmuint32_t * num)
 {
     if (!fsnames) 
 	if (getFilesystemList())
@@ -280,23 +298,26 @@ int rpmGetFilesystemList(const char *** listptr, uint32_t * num)
     return 0;
 }
 
-int rpmGetFilesystemUsage(const char ** fileList, uint32_t * fssizes, int numFiles,
-			  uint64_t ** usagesPtr, /*@unused@*/ int flags)
+int rpmGetFilesystemUsage(const char ** fileList, rpmuint32_t * fssizes,
+		int numFiles, rpmuint64_t ** usagesPtr,
+		/*@unused@*/ int flags)
 {
-    uint64_t * usages;
-    int i, len, j;
+    rpmuint64_t * usages;
+    int i, j;
     char * buf, * dirName;
     char * chptr;
-    int maxLen;
+    size_t maxLen;
+    size_t len;
     char * lastDir;
     const char * sourceDir;
     int lastfs = 0;
-    int lastDev = -1;		/* I hope nobody uses -1 for a st_dev */
+    dev_t lastDev = (dev_t)-1;		/* I hope nobody uses -1 for a st_dev */
     struct stat sb;
+    int rc = 1;		/* assume failure */
 
     if (!fsnames) 
 	if (getFilesystemList())
-	    return 1;
+	    return rc;
 
     usages = xcalloc(numFilesystems, sizeof(*usages));
 
@@ -331,13 +352,15 @@ int rpmGetFilesystemUsage(const char ** fileList, uint32_t * fssizes, int numFil
 	if (strcmp(lastDir, buf)) {
 	    strcpy(dirName, buf);
 	    chptr = dirName + strlen(dirName) - 1;
-	    while (stat(dirName, &sb)) {
-		if (errno != ENOENT) {
+	    while (Stat(dirName, &sb) < 0) {
+		switch(errno) {
+		default:
 		    rpmlog(RPMLOG_ERR, _("failed to stat %s: %s\n"), buf,
 				strerror(errno));
-		    sourceDir = _free(sourceDir);
-		    usages = _free(usages);
-		    return 1;
+		    goto exit;
+		    /*@notreached@*/ /*@switchbreak@*/ break;
+		case ENOENT:	/* XXX paths in empty chroot's don't exist. */
+		    /*@switchbreak@*/ break;
 		}
 
 		/* cut off last directory part, because it was not found. */
@@ -357,9 +380,7 @@ int rpmGetFilesystemUsage(const char ** fileList, uint32_t * fssizes, int numFil
 		if (j == numFilesystems) {
 		    rpmlog(RPMLOG_ERR, 
 				_("file %s is on an unknown device\n"), buf);
-		    sourceDir = _free(sourceDir);
-		    usages = _free(usages);
-		    return 1;
+		    goto exit;
 		}
 
 		lastfs = j;
@@ -370,14 +391,16 @@ int rpmGetFilesystemUsage(const char ** fileList, uint32_t * fssizes, int numFil
 	strcpy(lastDir, buf);
 	usages[lastfs] += fssizes[i];
     }
+    rc = 0;
 
+exit:
     sourceDir = _free(sourceDir);
 
-    if (usagesPtr)
+    if (rc == 0 && usagesPtr)
 	*usagesPtr = usages;
     else
 	usages = _free(usages);
 
-    return 0;
+    return rc;
 }
 /*@=usereleased =onlytrans@*/
