@@ -4,10 +4,10 @@ static const char * DSAsigpem = "DSA.sigpem";
 static const char * DSApem = "DSA.pem";
 static const char * DSApub = "DSA.pub";
 static const char * DSApubpem = "DSA.pubpem";
-static const char * DSApubid = "EC98CFF9C6522696A1BD2295A91653D87B58ED2A";
+static const char * DSApubid = "20EBA3920853A15E770CA466F25F9EB978C1CE98";
 static const char * RSAsig = "RSA.sig";
 static const char * RSAsigpem = "RSA.sigpem";
 static const char * RSApem = "RSA.pem";
 static const char * RSApub = "RSA.pub";
 static const char * RSApubpem = "RSA.pubpem";
-static const char * RSApubid = "0228B2A03C50951749EA5B2C29ED03AB3532351E";
+static const char * RSApubid = "C74DE19B44E32C3E6D7AA40E05C77FA49BEC2A07";
