@@ -8,14 +8,14 @@
 
 #include <rpmio_internal.h>
 #include <rpmcb.h>		/* XXX fnpyKey */
-#include <rpmlib.h>
+
+#include <rpmtag.h>
+#include <rpmtypes.h>
+#include <pkgio.h>
+#include "signature.h"		/* XXX rpmVerifySignature */
 
 #include "rpmts.h"
 
-#include <pkgio.h>
-
-#include "signature.h"		/* XXX rpmVerifySignature */
-
 #include "debug.h"
 
 #define	alloca_strdup(_s)	strcpy(alloca(strlen(_s)+1), (_s))
@@ -23,7 +23,7 @@
 /*@access pgpDig @*/
 /*@access pgpDigParams @*/
 /*@access Header @*/		/* XXX compared with NULL */
-/*@access FD_t @*/		/* XXX stealing digests */
+/*@access FD_t @*/		/* XXX void * */
 
 /*@unchecked@*/
 static int _print_pkts = 0;
@@ -49,7 +49,7 @@ static int pgpStashKeyid(pgpDig dig)
     pgpDigParams sigp = pgpGetSignature(dig);
     const void * sig = pgpGetSig(dig);
     unsigned int keyid;
-    int i;
+    unsigned int i;
 
     if (sig == NULL || dig == NULL || sigp == NULL)
 	return 0;
@@ -77,12 +77,11 @@ static int pgpStashKeyid(pgpDig dig)
 }
 
 /*@-mods@*/
-rpmRC rpmReadPackageFile(rpmts ts, void * _fd, const char * fn, Header * hdrp)
+rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     HE_t she = memset(alloca(sizeof(*she)), 0, sizeof(*she));
     pgpDig dig = rpmtsDig(ts);
-    FD_t fd = _fd;
     char buf[8*BUFSIZ];
     ssize_t count;
     Header sigh = NULL;
@@ -98,18 +97,6 @@ rpmRC rpmReadPackageFile(rpmts ts, void * _fd, const char * fn, Header * hdrp)
 
     if (hdrp) *hdrp = NULL;
 
-#ifdef	DYING
-    {	struct stat st;
-	memset(&st, 0, sizeof(st));
-	(void) fstat(Fileno(fd), &st);
-	/* if fd points to a socket, pipe, etc, st.st_size is *always* zero */
-	if (S_ISREG(st.st_mode) && st.st_size < sizeof(*l)) {
-	    rc = RPMRC_NOTFOUND;
-	    goto exit;
-	}
-    }
-#endif
-
 assert(dig != NULL);
     (void) fdSetDig(fd, dig);
 
@@ -193,7 +180,7 @@ assert(dig != NULL);
 	(void) rpmswEnter(op, 0);
     }
 /*@-type@*/	/* XXX arrow access of non-pointer (FDSTAT_t) */
-    nb = -fd->stats->ops[FDSTAT_READ].bytes;
+    nb = fd->stats->ops[FDSTAT_READ].bytes;
     {	const char item[] = "Header";
 	msg = NULL;
 	rc = rpmpkgRead(item, fd, &h, &msg);
@@ -204,7 +191,7 @@ assert(dig != NULL);
 	}
 	msg = _free(msg);
     }
-    nb += fd->stats->ops[FDSTAT_READ].bytes;
+    nb = fd->stats->ops[FDSTAT_READ].bytes - nb;
 /*@=type@*/
     if (opx > 0 && op != NULL) {
 	(void) rpmswExit(op, nb);
@@ -225,9 +212,9 @@ assert(dig != NULL);
 	rc = RPMRC_FAIL;
 	goto exit;
     }
-/*@-noeffect@*/
+/*@-ownedtrans -noeffect@*/
     xx = pgpSetSig(dig, she->tag, she->t, she->p.ptr, she->c);
-/*@=noeffect@*/
+/*@=ownedtrans =noeffect@*/
 
     switch ((rpmSigTag)she->tag) {
     default:	/* XXX keep gcc quiet. */
@@ -259,12 +246,12 @@ assert(0);
 	(void) headerGetMagic(NULL, &hmagic, &nmagic);
 	op = pgpStatsAccumulator(dig, 10);	/* RPMTS_OP_DIGEST */
 	(void) rpmswEnter(op, 0);
-	dig->hdrmd5ctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
+	dig->hdrctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
 	if (hmagic && nmagic > 0) {
-	    (void) rpmDigestUpdate(dig->hdrmd5ctx, hmagic, nmagic);
+	    (void) rpmDigestUpdate(dig->hdrctx, hmagic, nmagic);
 	    dig->nbytes += nmagic;
 	}
-	(void) rpmDigestUpdate(dig->hdrmd5ctx, uh, uhc);
+	(void) rpmDigestUpdate(dig->hdrctx, uh, uhc);
 	dig->nbytes += uhc;
 	(void) rpmswExit(op, dig->nbytes);
 	op->count--;	/* XXX one too many */
@@ -363,7 +350,8 @@ exit:
 	/* Bump reference count for return. */
 	*hdrp = headerLink(h);
     }
-    h = headerFree(h);
+    (void)headerFree(h);
+    h = NULL;
 
     /* Accumulate time reading package header. */
     (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_READHDR),
@@ -372,7 +360,8 @@ exit:
 		opsave);
 
     rpmtsCleanDig(ts);
-    sigh = headerFree(sigh);
+    (void)headerFree(sigh);
+    sigh = NULL;
     return rc;
 }
 /*@=mods@*/
