@@ -4,44 +4,46 @@
  */
 
 #include "system.h"
-const char *__progname;
+extern const char *__progname;
 
+#if defined(RPM_VENDOR_WINDRIVER)
+const char *__usrlibrpm = USRLIBRPM;
+const char *__etcrpm = SYSCONFIGDIR;
+#endif
+#if defined(ENABLE_NLS) && !defined(__LCLINT__)
+const char *__localedir = LOCALEDIR;
+#endif
+
+#define	_RPMIOB_INTERNAL
 #include <rpmio.h>
-#include <rpmcli.h>
-#include <fs.h>			/* XXX rpmFreeFilesystems() */
-#include <rpmns.h>		/* XXX rpmnsClean() */
+#include <rpmiotypes.h>
 #include <fts.h>
 #include <mire.h>
+#include <poptIO.h>
 
-#define _RPMPGP_INTERNAL
-#if defined(WITH_BEECRYPT)
-#define _RPMBC_INTERNAL
-#include <rpmbc.h>
-#endif
-#if defined(WITH_GCRYPT)
-#define _RPMGC_INTERNAL
-#include <rpmgc.h>
-#endif
-#if defined(WITH_NSS)
-#define _RPMNSS_INTERNAL
-#include <rpmnss.h>
-#endif
-#if defined(WITH_SSL)
-#define _RPMSSL_INTERNAL
-#include <rpmssl.h>
-#endif
+#include <rpmjs.h>
+#include <rpmruby.h>
+
+#include <rpmtag.h>
+#include <rpmtypes.h>
+#include <rpmrc.h>
+#include <rpmversion.h>
+#include <rpmcli.h>
+
+#include <rpmns.h>		/* XXX rpmnsClean() */
+
+#include <fs.h>			/* XXX rpmFreeFilesystems() */
 
 #include "debug.h"
 
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+extern unsigned int * keyids;
+
 #define POPT_SHOWVERSION	-999
 #define POPT_SHOWRC		-998
 #define POPT_QUERYTAGS		-997
 #define POPT_PREDEFINE		-996
-#ifdef  DEAD	/* XXX remember the previous definition however. */
-#define POPT_RCFILE		-995
-#endif
 #define POPT_UNDEFINE		-994
-#define	POPT_CRYPTO		-993
 
 /*@access headerTagIndices @*/		/* XXX rpmcliFini */
 /*@access headerTagTableEntry @*/	/* XXX rpmcliFini */
@@ -53,9 +55,6 @@ static int _debug = 0;
 /*@unchecked@*/
 extern int _rpmds_nopromote;
 
-/*@unchecked@*/
-extern int _cpio_debug;
-
 /*@unchecked@*/
 extern int _fps_debug;
 
@@ -67,15 +66,19 @@ extern int _fsm_threads;
 
 /*@unchecked@*/
 extern int _hdr_debug;
+/*@unchecked@*/
+extern int _hdrqf_debug;
 
 /*@unchecked@*/
-extern int _mire_debug;
+extern int _pkgio_debug;
 
 /*@unchecked@*/
 extern int _print_pkts;
 
 /*@unchecked@*/
 extern int _psm_debug;
+/*@unchecked@*/
+extern rpmioPool _psmPool;
 
 /*@unchecked@*/
 extern int _psm_threads;
@@ -88,41 +91,55 @@ extern int _rpmdb_debug;
 
 /*@unchecked@*/
 extern int _rpmds_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmdsPool;
 
-/* XXX avoid -lrpmbuild linkage. */
 /*@unchecked@*/
        int _rpmfc_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmfcPool;
 
 /*@unchecked@*/
 extern int _rpmfi_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmfiPool;
 
 /*@unchecked@*/
 extern int _rpmgi_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmgiPool;
 
 /*@unchecked@*/
-extern int _rpmmg_debug;
+extern int _rpmmi_debug;
 
 /*@unchecked@*/
 extern int _rpmps_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmpsPool;
 
 /*@unchecked@*/
 extern int _rpmsq_debug;
 
 /*@unchecked@*/
 extern int _rpmsx_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmsxPool;
 
 /*@unchecked@*/
 extern int _rpmte_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmtePool;
+/*@unchecked@*/
+extern rpmioPool _rpmtsiPool;
 
 /*@unchecked@*/
 extern int _rpmts_debug;
+/*@unchecked@*/
+extern rpmioPool _rpmtsPool;
 
 /*@unchecked@*/
 extern int _rpmwf_debug;
 
-/*@unchecked@*/
-extern int _xar_debug;
-
 /*@unchecked@*/
 extern int _rpmts_macros;
 
@@ -132,54 +149,36 @@ extern int _rpmts_stats;
 /*@unchecked@*/
 extern int _hdr_stats;
 
-/*@unchecked@*/
-extern int _tar_debug;
-
-/*@unchecked@*/
-extern int noLibio;
-/*@=exportheadervar@*/
-
-/*@unchecked@*/ /*@null@*/
-const char * rpmcliPipeOutput = NULL;
-
-/*@unchecked@*/
-const char * rpmcliRootDir = "/";
-
 /*@unchecked@*/
 rpmQueryFlags rpmcliQueryFlags;
 
 /*@unchecked@*/ /*@null@*/
 const char * rpmcliTargets = NULL;
 
-/*@-exportheadervar@*/
-/*@unchecked@*/
-extern int _ftp_debug;
-/*@unchecked@*/
-extern int _av_debug;
-/*@unchecked@*/
-extern int _dav_debug;
-
-/*@unchecked@*/
-extern int noLibio;
-
-/*@unchecked@*/
-extern int _rpmio_debug;
-/*@=exportheadervar@*/
-
 /*@unchecked@*/
 static int rpmcliInitialized = -1;
 
+#ifdef WITH_LUA
+/*@unchecked@*/
+extern const char *rpmluaFiles;
+#endif
+
+/*@-readonlytrans@*/	/* argv loading prevents observer, xstrdup needed. */
+/*@unchecked@*/
+static char *rpmpoptfiles = RPMPOPTFILES;
+/*@=readonlytrans@*/
+
 /**
  * Display rpm version.
  */
 static void printVersion(FILE * fp)
-	/*@globals rpmEVR, fileSystem @*/
-	/*@modifies *fp, fileSystem @*/
+	/*@globals rpmEVR, fileSystem, internalState @*/
+	/*@modifies *fp, fileSystem, internalState @*/
 {
     fprintf(fp, _("%s (" RPM_NAME ") %s\n"), __progname, rpmEVR);
     if (rpmIsVerbose())
-	fprintf(fp, "rpmlib 0x%08x,0x%08x,0x%08x\n",
-	    rpmlibVersion(), rpmlibTimestamp(), rpmlibVendor());
+	fprintf(fp, "rpmlib 0x%08x,0x%08x,0x%08x\n", (unsigned)rpmlibVersion(),
+		(unsigned)rpmlibTimestamp(), (unsigned)rpmlibVendor());
 }
 
 void rpmcliConfigured(void)
@@ -204,27 +203,49 @@ void rpmcliConfigured(void)
 	exit(EXIT_FAILURE);
 }
 
+/* ========== all-rpm-modes popt args */
+
+static const char * rpmcliEvalSlurp(const char * arg)
+	/*@*/
+{
+    const char * pre = "";
+    const char * post = "";
+    rpmiob iob = NULL;
+    const char * val = NULL;
+    struct stat sb;
+    int xx;
+
+    if (!strcmp(arg, "-")) {	/* Macros from stdin arg. */
+	xx = rpmiobSlurp(arg, &iob);
+    } else
+    if ((arg[0] == '/' || strchr(arg, ' ') == NULL)
+     && !Stat(arg, &sb)
+     && S_ISREG(sb.st_mode)) {	/* Macros from a file arg. */
+	xx = rpmiobSlurp(arg, &iob);
+    } else {			/* Macros from string arg. */
+	iob = rpmiobAppend(rpmiobNew(strlen(arg)+1), arg, 0);
+    }
+
+    val = rpmExpand(pre, iob->b, post, NULL);
+    iob = rpmiobFree(iob);
+    return val;
+}
+
 /**
  */
 static void rpmcliAllArgCallback(poptContext con,
                 /*@unused@*/ enum poptCallbackReason reason,
                 const struct poptOption * opt, const char * arg,
                 /*@unused@*/ const void * data)
-	/*@globals rpmcliTargets, rpmcliQueryFlags, rpmCLIMacroContext,
+	/*@globals pgpDigVSFlags, rpmcliTargets, rpmcliQueryFlags, rpmCLIMacroContext,
 		rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies con, rpmcliTargets, rpmcliQueryFlags, rpmCLIMacroContext,
+	/*@modifies con, pgpDigVSFlags, rpmcliTargets, rpmcliQueryFlags, rpmCLIMacroContext,
 		rpmGlobalMacroContext, fileSystem, internalState @*/
 {
 
     /* XXX avoid accidental collisions with POPT_BIT_SET for flags */
     if (opt->arg == NULL)
     switch (opt->val) {
-    case 'q':
-	rpmSetVerbosity(RPMLOG_WARNING);
-	break;
-    case 'v':
-	rpmIncreaseVerbosity();
-	break;
     case POPT_PREDEFINE:
 	(void) rpmDefineMacro(NULL, arg, RMIL_CMDLINE);
 	break;
@@ -263,42 +284,15 @@ static void rpmcliAllArgCallback(poptContext con,
 /*@=type@*/
 	s = _free(s);
     }	break;
-    case POPT_CRYPTO:
-	rpmcliConfigured();
-	{   const char *val = rpmExpand(arg, NULL);
-#if defined(WITH_BEECRYPT)
-	    if (!xstrcasecmp(val, "beecrypt") || !xstrcasecmp(val, "bc"))
-		pgpImplVecs = &rpmbcImplVecs;
-#endif
-#if defined(WITH_GCRYPT)
-	    if (!xstrcasecmp(val, "gcrypt") || !xstrcasecmp(val, "gc"))
-		pgpImplVecs = &rpmgcImplVecs;
-#endif
-#if defined(WITH_NSS)
-	    if (!xstrcasecmp(val, "NSS"))
-		pgpImplVecs = &rpmnssImplVecs;
-#endif
-#if defined(WITH_SSL)
-	    if (!xstrcasecmp(val, "OpenSSL") || !xstrcasecmp(val, "ssl"))
-		pgpImplVecs = &rpmsslImplVecs;
-#endif
-	    val = _free(val);
-	}
-	break;
     case 'E':
+assert(arg != NULL);
 	rpmcliConfigured();
-	{   const char *val = rpmExpand(arg, NULL);
-#if defined(RPM_VENDOR_OPENPKG) /* no-extra-terminating-newline-on-eval */
-            size_t val_len;
-            val_len = strlen(val);
-            if (val[val_len - 1] == '\n')
-                fwrite(val, val_len, 1, stdout);
-            else
-#endif
-	    fprintf(stdout, "%s\n", val);
-	    val = _free(val);
-	}
-	break;
+    {	const char * val = rpmcliEvalSlurp(arg);
+	size_t val_len = fwrite(val, strlen(val), 1, stdout);
+	if (val[val_len - 1] != '\n')
+	    fprintf(stdout, "\n");
+	val = _free(val);
+    }	break;
     case POPT_SHOWVERSION:
 	printVersion(stdout);
 /*@i@*/	con = rpmcliFini(con);
@@ -315,10 +309,6 @@ static void rpmcliAllArgCallback(poptContext con,
 /*@i@*/	con = rpmcliFini(con);
 	exit(EXIT_SUCCESS);
 	/*@notreached@*/ break;
-#if defined(POPT_RCFILE)
-    case POPT_RCFILE:		/* XXX FIXME: noop for now */
-	break;
-#endif
     case RPMCLI_POPT_NODIGEST:
 	rpmcliQueryFlags |= VERIFY_DIGEST;
 	pgpDigVSFlags |= _RPMVSF_NODIGESTS;
@@ -350,64 +340,40 @@ static void rpmcliAllArgCallback(poptContext con,
     }
 }
 
-/*@unchecked@*/
-int ftsOpts = 0;
-
-/*@unchecked@*/
-struct poptOption rpmcliFtsPoptTable[] = {
- { "comfollow", '\0', POPT_BIT_SET,	&ftsOpts, FTS_COMFOLLOW,
-	N_("FTS_COMFOLLOW: follow command line symlinks"), NULL },
- { "logical", '\0', POPT_BIT_SET,	&ftsOpts, FTS_LOGICAL,
-	N_("FTS_LOGICAL: logical walk"), NULL },
- { "nochdir", '\0', POPT_BIT_SET,	&ftsOpts, FTS_NOCHDIR,
-	N_("FTS_NOCHDIR: don't change directories"), NULL },
- { "nostat", '\0', POPT_BIT_SET,	&ftsOpts, FTS_NOSTAT,
-	N_("FTS_NOSTAT: don't get stat info"), NULL },
- { "physical", '\0', POPT_BIT_SET,	&ftsOpts, FTS_PHYSICAL,
-	N_("FTS_PHYSICAL: physical walk"), NULL },
- { "seedot", '\0', POPT_BIT_SET,	&ftsOpts, FTS_SEEDOT,
-	N_("FTS_SEEDOT: return dot and dot-dot"), NULL },
- { "xdev", '\0', POPT_BIT_SET,		&ftsOpts, FTS_XDEV,
-	N_("FTS_XDEV: don't cross devices"), NULL },
- { "whiteout", '\0', POPT_BIT_SET,	&ftsOpts, FTS_WHITEOUT,
-	N_("FTS_WHITEOUT: return whiteout information"), NULL },
-   POPT_TABLEEND
-};
-
 /*@unchecked@*/
 int global_depFlags;
 
 /*@unchecked@*/
 struct poptOption rpmcliDepFlagsPoptTable[] = {
  { "aid", '\0', POPT_BIT_SET, &global_depFlags, RPMDEPS_FLAG_ADDINDEPS,
-	N_("add suggested packages to transaction"), NULL },
+	N_("Add suggested packages to transaction"), NULL },
  { "anaconda", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
  	&global_depFlags, RPMDEPS_FLAG_ANACONDA|RPMDEPS_FLAG_DEPLOOPS,
-	N_("use anaconda \"presentation order\""), NULL},
+	N_("Use anaconda \"presentation order\""), NULL},
  { "deploops", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
  	&global_depFlags, RPMDEPS_FLAG_DEPLOOPS,
-	N_("print dependency loops as warning"), NULL},
- { "nosuggest", '\0', POPT_BIT_SET,
+	N_("Print dependency loops as warning"), NULL},
+ { "nosuggest", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE,
 	&global_depFlags, RPMDEPS_FLAG_NOSUGGEST,
-	N_("do not suggest missing dependency resolution(s)"), NULL},
+	N_("Do not suggest missing dependency resolution(s)"), NULL},
  { "noconflicts", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&global_depFlags, RPMDEPS_FLAG_NOCONFLICTS,
-	N_("do not check added package conflicts"), NULL},
- { "nolinktos", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	N_("Do not check added package conflicts"), NULL},
+ { "nolinktos", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&global_depFlags, RPMDEPS_FLAG_NOLINKTOS,
-	N_("ignore added package requires on symlink targets"), NULL},
+	N_("Ignore added package requires on symlink targets"), NULL},
  { "noobsoletes", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&global_depFlags, RPMDEPS_FLAG_NOOBSOLETES,
-	N_("ignore added package obsoletes"), NULL},
- { "noparentdirs", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	N_("Ignore added package obsoletes"), NULL},
+ { "noparentdirs", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&global_depFlags, RPMDEPS_FLAG_NOPARENTDIRS,
-	N_("ignore added package requires on file parent directory"), NULL},
+	N_("Ignore added package requires on file parent directory"), NULL},
  { "norequires", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&global_depFlags, RPMDEPS_FLAG_NOREQUIRES,
-	N_("do not check added package requires"), NULL},
+	N_("Do not check added package requires"), NULL},
  { "noupgrade", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&global_depFlags, RPMDEPS_FLAG_NOUPGRADE,
-	N_("ignore added package upgrades"), NULL},
+	N_("Ignore added package upgrades"), NULL},
    POPT_TABLEEND
 };
 
@@ -420,135 +386,102 @@ struct poptOption rpmcliAllPoptTable[] = {
 /*@=type@*/
 
  { "debug", 'd', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_debug, -1,
-        NULL, NULL },
+	N_("Debug generic operations"), NULL},
 
  { "predefine", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_PREDEFINE,
-	N_("predefine MACRO with value EXPR"),
+	N_("Predefine MACRO with value EXPR"),
 	N_("'MACRO EXPR'") },
+
  { "define", 'D', POPT_ARG_STRING, NULL, 'D',
-	N_("define MACRO with value EXPR"),
+	N_("Define MACRO with value EXPR"),
 	N_("'MACRO EXPR'") },
  { "undefine", '\0', POPT_ARG_STRING, NULL, POPT_UNDEFINE,
-	N_("undefine MACRO"),
+	N_("Undefine MACRO"),
 	N_("'MACRO'") },
  { "eval", 'E', POPT_ARG_STRING, NULL, 'E',
-	N_("print macro expansion of EXPR"),
+	N_("Print macro expansion of EXPR"),
 	N_("'EXPR'") },
  { "macros", '\0', POPT_ARG_STRING, &rpmMacrofiles, 0,
-	N_("read <FILE:...> instead of default file(s)"),
+	N_("Read <FILE:...> instead of default file(s)"),
 	N_("<FILE:...>") },
+#ifdef WITH_LUA
+ { "rpmlua", '\0', POPT_ARG_STRING, &rpmluaFiles, 0,
+	N_("Read <FILE:...> instead of default RPM Lua file(s)"),
+	N_("<FILE:...>") },
+#endif
+ { "rpmpopt", '\0', POPT_ARG_STRING, NULL, 0,
+	N_("Read <FILE:...> instead of default POPT file(s)"),
+	N_("<FILE:...>") },
+
  { "target", '\0', POPT_ARG_STRING, NULL,  RPMCLI_POPT_TARGETPLATFORM,
-        N_("specify target platform"), N_("CPU-VENDOR-OS") },
+        N_("Specify target platform"), N_("CPU-VENDOR-OS") },
 
  { "nodigest", '\0', 0, NULL, RPMCLI_POPT_NODIGEST,
-        N_("don't verify package digest(s)"), NULL },
+        N_("Don't verify package digest(s)"), NULL },
  { "nohdrchk", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, RPMCLI_POPT_NOHDRCHK,
-        N_("don't verify database header(s) when retrieved"), NULL },
-#if defined(HAVE_LIBIO_H) && defined(_G_IO_IO_FILE_VERSION)
- { "nolibio", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &noLibio, 1,
-	N_("disable use of libio(3) API"), NULL},
-#endif
+        N_("Don't verify database header(s) when retrieved"), NULL },
  { "nosignature", '\0', 0, NULL, RPMCLI_POPT_NOSIGNATURE,
-        N_("don't verify package signature(s)"), NULL },
-
- { "pipe", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, &rpmcliPipeOutput, 0,
-	N_("send stdout to CMD"),
-	N_("CMD") },
- { "root", 'r', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &rpmcliRootDir, 0,
-	N_("use ROOT as top level directory"),
-	N_("ROOT") },
+        N_("Don't verify package signature(s)"), NULL },
 
  { "querytags", '\0', 0, NULL, POPT_QUERYTAGS,
-        N_("display known query tags"), NULL },
- { "quiet", '\0', 0, NULL, 'q',
-	N_("provide less detailed output"), NULL},
+        N_("Display known query tags"), NULL },
  { "showrc", '\0', 0, NULL, POPT_SHOWRC,
-	N_("display final rpmrc and macro configuration"), NULL },
- { "verbose", 'v', 0, NULL, 'v',
-	N_("provide more detailed output"), NULL},
- { "version", '\0', 0, NULL, POPT_SHOWVERSION,
-	N_("print the version of rpm being used"), NULL },
-
-#if defined(HAVE_LIBIO_H) && defined(_G_IO_IO_FILE_VERSION)
- { "nolibio", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &noLibio, 1,
-       N_("disable use of libio(3) API"), NULL},
-#endif
+	N_("Display macro and configuration values"), NULL },
+ { "version", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_SHOWVERSION,
+	N_("Print the version"), NULL },
 
  { "promoteepoch", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmds_nopromote, 0,
 	NULL, NULL},
 
- { "usecrypto",'\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_CRYPTO,
-        N_("select cryptography implementation"),
-	N_("CRYPTO") },
-
- { "avdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_av_debug, -1,
-	N_("debug argv collections"), NULL},
- { "cpiodebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_cpio_debug, -1,
-	N_("debug cpio payloads"), NULL},
- { "davdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_dav_debug, -1,
-	N_("debug WebDAV data stream"), NULL},
  { "fpsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fps_debug, -1,
-	NULL, NULL},
+	N_("Debug file FingerPrintS"), NULL},
  { "fsmdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fsm_debug, -1,
-	N_("debug payload file state machine"), NULL},
+	N_("Debug payload File State Machine"), NULL},
  { "fsmthreads", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fsm_threads, -1,
-	N_("use threads for file state machine"), NULL},
- { "ftpdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_ftp_debug, -1,
-	N_("debug FTP/HTTP data stream"), NULL},
+	N_("Use threads for File State Machine"), NULL},
  { "hdrdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_hdr_debug, -1,
 	NULL, NULL},
+ { "hdrqfdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_hdrqf_debug, -1,
+	NULL, NULL},
  { "macrosused", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmts_macros, -1,
+	N_("Display macros used"), NULL},
+ { "pkgiodebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_pkgio_debug, -1,
 	NULL, NULL},
- { "miredebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_mire_debug, -1,
-	NULL, NULL},
-#ifdef	DYING
- { "poptdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_popt_debug, -1,
-	N_("debug option/argument processing"), NULL},
-#endif
  { "prtpkts", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_print_pkts, -1,
-	NULL, NULL},
+	N_("Display OpenPGP (RFC 2440/4880) parsing"), NULL},
  { "psmdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_psm_debug, -1,
-	N_("debug package state machine"), NULL},
+	N_("Debug Package State Machine"), NULL},
  { "psmthreads", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_psm_threads, -1,
-	N_("use threads for package state machine"), NULL},
- { "rpmaldebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmal_debug, -1,
-	NULL, NULL},
+	N_("Use threads for Package State Machine"), NULL},
  { "rpmdbdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmdb_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmdb DataBase"), NULL},
  { "rpmdsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmds_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmds Dependency Set"), NULL},
  { "rpmfcdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmfc_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmfc File Classifier"), NULL},
  { "rpmfidebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmfi_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmfi File Info"), NULL},
  { "rpmgidebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmgi_debug, -1,
-	NULL, NULL},
- { "rpmiodebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmio_debug, -1,
-	N_("debug rpmio I/O"), NULL},
- { "rpmmgdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmmg_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmgi Generalized Iterator"), NULL},
+ { "rpmmidebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmmi_debug, -1,
+	N_("Debug rpmmi Match Iterator"), NULL},
  { "rpmnsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmns_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmns Name Space"), NULL},
  { "rpmpsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmps_debug, -1,
-	NULL, NULL},
- { "rpmsqdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmsq_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmps Problem Set"), NULL},
  { "rpmsxdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmsx_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmsx SELinux Xattrs"), NULL},
  { "rpmtedebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmte_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmte Transaction Element"), NULL},
  { "rpmtsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmts_debug, -1,
-	NULL, NULL},
+	N_("Debug rpmts Transaction Set"), NULL},
  { "rpmwfdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmwf_debug, -1,
-	NULL, NULL},
- { "xardebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_xar_debug, -1,
-	NULL, NULL},
- { "tardebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_tar_debug, -1,
-	N_("debug tar payloads"), NULL},
+	N_("Debug rpmwf Wrapper Format"), NULL},
  { "stats", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmts_stats, -1,
+	N_("Display operation statistics"), NULL},
+
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmioAllPoptTable, 0,
 	NULL, NULL},
- { "urldebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_url_debug, -1,
-	N_("debug URL cache handling"), NULL},
 
    POPT_TABLEEND
 };
@@ -556,17 +489,63 @@ struct poptOption rpmcliAllPoptTable[] = {
 
 poptContext
 rpmcliFini(poptContext optCon)
+	/*@globals keyids @*/
+	/*@modifies keyids @*/
 {
+/*@-nestedextern@*/
+    extern rpmioPool _rpmjsPool;
+    extern rpmioPool _rpmrubyPool;
+    extern rpmioPool _headerPool;
+    extern rpmioPool _rpmmiPool;
+    extern rpmioPool _rpmdbPool;
+    extern rpmioPool _rpmwfPool;
+    extern const char * evr_tuple_order;
+    extern const char * evr_tuple_match;
+    extern miRE evr_tuple_mire;
+/*@=nestedextern@*/
+
+/*@-mods@*/
+    evr_tuple_order = _free(evr_tuple_order);
+    evr_tuple_match = _free(evr_tuple_match);
+    evr_tuple_mire = mireFree(evr_tuple_mire);
+
+/*@-onlyunqglobaltrans@*/
+    /* Realease (and dereference) embedded interpreter global objects first. */
+    _rpmjsI = rpmjsFree(_rpmjsI);
+    _rpmjsPool = rpmioFreePool(_rpmjsPool);
+    _rpmrubyI = rpmrubyFree(_rpmrubyI);
+    _rpmrubyPool = rpmioFreePool(_rpmrubyPool);
+
+    _rpmgiPool = rpmioFreePool(_rpmgiPool);
+    _rpmmiPool = rpmioFreePool(_rpmmiPool);
+
+    _psmPool = rpmioFreePool(_psmPool);
+    _rpmtsiPool = rpmioFreePool(_rpmtsiPool);
+
+    _rpmtsPool = rpmioFreePool(_rpmtsPool);
+    _rpmtePool = rpmioFreePool(_rpmtePool);
+    _rpmpsPool = rpmioFreePool(_rpmpsPool);
+
+    _rpmfcPool = rpmioFreePool(_rpmfcPool);
+    _rpmsxPool = rpmioFreePool(_rpmsxPool);
+
+    rpmnsClean();
+
+    _rpmdsPool = rpmioFreePool(_rpmdsPool);
+    _rpmfiPool = rpmioFreePool(_rpmfiPool);
+
+    _rpmwfPool = rpmioFreePool(_rpmwfPool);
+    _rpmdbPool = rpmioFreePool(_rpmdbPool);
+    _headerPool = rpmioFreePool(_headerPool);
+/*@=onlyunqglobaltrans@*/
+/*@=mods@*/
+
     /* XXX this should be done in the rpmioClean() wrapper. */
     /* keeps memory leak checkers quiet */
     rpmFreeMacros(NULL);
 /*@i@*/	rpmFreeMacros(rpmCLIMacroContext);
 
-    rpmioClean();
-
-    rpmnsClean();
-
-    rpmFreeRpmrc();
+    rpmFreeRpmrc();	/* XXX mireFreeAll(platpat) before rpmioFreePool. */
 
     rpmFreeFilesystems();
 /*@i@*/	rpmcliTargets = _free(rpmcliTargets);
@@ -575,6 +554,8 @@ rpmcliFini(poptContext optCon)
 
     tagClean(NULL);	/* Free header tag indices. */
 
+    rpmioClean();	/* XXX rpmioFreePool()'s after everything else. */
+
     optCon = poptFreeContext(optCon);
 
 #if defined(HAVE_MCHECK_H) && defined(HAVE_MTRACE)
@@ -583,10 +564,13 @@ rpmcliFini(poptContext optCon)
     /*@=noeffect@*/
 #endif
 
+/*@-globstate@*/
     return NULL;
+/*@=globstate@*/
 }
 
 static inline int checkfd(const char * devnull, int fdno, int flags)
+	/*@*/
 {
     struct stat sb;
     int ret = 0;
@@ -596,13 +580,51 @@ static inline int checkfd(const char * devnull, int fdno, int flags)
     return ret;
 }
 
+#if defined(RPM_VENDOR_WINDRIVER)
+void setRuntimeRelocPaths(void)
+{
+    /* 
+     * This is just an example of setting the values using env
+     * variables....  if they're not set, we make sure they get set
+     * for helper apps...  We probably want to escape "%" in the path
+     * to avoid macro expansion.. someone might have a % in a path...
+     */
+
+    __usrlibrpm = getenv("RPM_USRLIBRPM");
+    __etcrpm = getenv("RPM_ETCRPM");
+#if defined(ENABLE_NLS) && !defined(__LCLINT__)
+    __localedir = getenv("RPM_LOCALEDIR");
+#endif
+
+    if ( __usrlibrpm == NULL ) {
+	__usrlibrpm = USRLIBRPM ;
+	setenv("RPM_USRLIBRPM", USRLIBRPM, 0);
+    }
+
+    if ( __etcrpm == NULL ) {
+	__etcrpm = SYSCONFIGDIR ;
+	setenv("RPM_ETCRPM", SYSCONFIGDIR, 0);
+    }
+
+#if defined(ENABLE_NLS) && !defined(__LCLINT__)
+    if ( __localedir == NULL ) {
+	__localedir = LOCALEDIR ;
+	setenv("RPM_LOCALEDIR", LOCALEDIR, 0);
+    }
+#endif
+}
+#endif
+
 /*@-globstate@*/
 poptContext
 rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
+	/*@globals rpmpoptfiles @*/
+	/*@modifies rpmpoptfiles @*/
 {
     poptContext optCon;
-    char *path_buf, *path, *path_next;
     int rc;
+    int xx;
+    int i;
 
 #if defined(HAVE_MCHECK_H) && defined(HAVE_MTRACE)
     /*@-noeffect@*/
@@ -632,9 +654,13 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 #endif
    }
 
+#if defined(RPM_VENDOR_WINDRIVER)
+    (void) setRuntimeRelocPaths();
+#endif
+
 #if defined(ENABLE_NLS) && !defined(__LCLINT__)
     (void) setlocale(LC_ALL, "" );
-    (void) bindtextdomain(PACKAGE, LOCALEDIR);
+    (void) bindtextdomain(PACKAGE, __localedir);
     (void) textdomain(PACKAGE);
 #endif
 
@@ -646,15 +672,31 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
 	return NULL;
     }
 
+    /* read all RPM POPT configuration files */
+    for (i = 1; i < argc; i++) {
+	if (strcmp(argv[i], "--rpmpopt") == 0 && i+1 < argc) {
+	    rpmpoptfiles = argv[i+1];
+	    break;
+	}
+	else if (strncmp(argv[i], "--rpmpopt=", 10) == 0) {
+	    rpmpoptfiles = argv[i]+10;
+	    break;
+	}
+    }
+
 /*@-nullpass -temptrans@*/
     optCon = poptGetContext(__progname, argc, (const char **)argv, optionsTable, 0);
 /*@=nullpass =temptrans@*/
 
-    /* read all RPM POPT configuration files */
-    path_buf = xstrdup(RPMPOPTFILES);
+#if defined(RPM_VENDOR_OPENPKG) /* stick-with-rpm-file-sanity-checking */ || \
+    !defined(POPT_ERROR_BADCONFIG)	/* XXX POPT 1.15 retrofit */
+  { char * path_buf = xstrdup(rpmpoptfiles);
+    char *path;
+    char *path_next;
+
     for (path = path_buf; path != NULL && *path != '\0'; path = path_next) {
         const char **av;
-        int ac, i;
+        int ac;
 
         /* locate start of next path element */
         path_next = strchr(path, ':');
@@ -663,40 +705,61 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
         else
             path_next = path + strlen(path);
 
-#if defined(RPM_VENDOR_OPENPKG) /* security-sanity-check-rpmpopt-and-rpmmacros */
-        if (path[0] == '@' /* attention */) {
-            path++;
-            if (!rpmSecuritySaneFile(path)) {
-                rpmlog(RPMLOG_WARNING, "existing POPT configuration file \"%s\" considered INSECURE -- not loaded\n", path);
-                continue;
-            }
-        }
-#endif
-
         /* glob-expand the path element */
         ac = 0;
         av = NULL;
-        if ((i = rpmGlob(path, &ac, &av)) != 0)
+        if ((xx = rpmGlob(path, &ac, &av)) != 0)
             continue;
 
         /* work-off each resulting file from the path element */
         for (i = 0; i < ac; i++) {
-            (void)poptReadConfigFile(optCon, av[i]);
+	    const char *fn = av[i];
+	    if (fn[0] == '@' /* attention */) {
+		fn++;
+		if (!rpmSecuritySaneFile(fn)) {
+		    rpmlog(RPMLOG_WARNING, "existing POPT configuration file \"%s\" considered INSECURE -- not loaded\n", fn);
+		    /*@innercontinue@*/ continue;
+		}
+	    }
+	    (void) poptReadConfigFile(optCon, fn);
             av[i] = _free(av[i]);
         }
         av = _free(av);
     }
     path_buf = _free(path_buf);
+  }
+#else
+    /* XXX FIXME: better error message is needed. */
+    if ((xx = poptReadConfigFiles(optCon, rpmpoptfiles)) != 0)
+	rpmlog(RPMLOG_WARNING, "existing POPT configuration file \"%s\" considered INSECURE -- not loaded\n", rpmpoptfiles);
+#endif
+
+#if defined(RPM_VENDOR_WINDRIVER)
+    {	const char * poptAliasFn = rpmGetPath(__usrlibrpm, "/rpmpopt", NULL);
+	(void) poptReadConfigFile(optCon, poptAliasFn);
+	poptAliasFn = _free(poptAliasFn);
+    }
+#endif
 
     /* read standard POPT configuration files */
+    /* XXX FIXME: the 2nd arg useEnv flag is UNUSED. */
     (void) poptReadDefaultConfig(optCon, 1);
 
+#if defined(RPM_VENDOR_WINDRIVER)
+    {	const char * poptExecPath = rpmGetPath(__usrlibrpm, NULL);
+	poptSetExecPath(optCon, poptExecPath, 1);
+	poptExecPath = _free(poptExecPath);
+    }
+#else
     poptSetExecPath(optCon, USRLIBRPM, 1);
+#endif
 
     /* Process all options, whine if unknown. */
     while ((rc = poptGetNextOpt(optCon)) > 0) {
 	const char * optArg = poptGetOptArg(optCon);
+/*@-dependenttrans -observertrans@*/	/* Avoid popt memory leaks. */
 	optArg = _free(optArg);
+/*@=dependenttrans =observertrans @*/
 	switch (rc) {
 	default:
 /*@-nullpass@*/
