@@ -6,6 +6,10 @@
 #include "system.h"
 
 #include <rpmio.h>
+#include <rpmiotypes.h>
+#include <rpmcb.h>
+
+#include <rpmtag.h>
 #include <rpmcli.h>
 #include <rpmgi.h>		/* XXX for giFlags */
 
@@ -36,11 +40,15 @@ int specedit = 0;
 #define	POPT_WHATNEEDS		-1038
 #define	POPT_SPECSRPM		-1039
 #define POPT_QUERYBYSOURCEPKGID	-1040
+#define	POPT_WHATCONFLICTS	-1041
+#define	POPT_WHATOBSOLETES	-1042
+#define	POPT_NOPASSWORD		-1043
 
 /* ========== Query/Verify/Signature source args */
 static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
 		/*@unused@*/ enum poptCallbackReason reason,
-		const struct poptOption * opt, /*@unused@*/ const char * arg, 
+		const struct poptOption * opt,
+		/*@unused@*/ const char * arg,
 		/*@unused@*/ const void * data)
 	/*@globals rpmQVKArgs @*/
 	/*@modifies rpmQVKArgs @*/
@@ -65,11 +73,15 @@ static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
     case 'f': qva->qva_source |= RPMQV_PATH; qva->qva_sourceCount++; break;
     case 'g': qva->qva_source |= RPMQV_GROUP; qva->qva_sourceCount++; break;
     case 'p': qva->qva_source |= RPMQV_RPM; qva->qva_sourceCount++; break;
-    case POPT_WHATNEEDS: qva->qva_source |= RPMQV_WHATNEEDS; 
+    case POPT_WHATNEEDS: qva->qva_source |= RPMQV_WHATNEEDS;
 				qva->qva_sourceCount++; break;
-    case POPT_WHATPROVIDES: qva->qva_source |= RPMQV_WHATPROVIDES; 
+    case POPT_WHATPROVIDES: qva->qva_source |= RPMQV_WHATPROVIDES;
 				qva->qva_sourceCount++; break;
-    case POPT_WHATREQUIRES: qva->qva_source |= RPMQV_WHATREQUIRES; 
+    case POPT_WHATREQUIRES: qva->qva_source |= RPMQV_WHATREQUIRES;
+				qva->qva_sourceCount++; break;
+    case POPT_WHATCONFLICTS: qva->qva_source |= RPMQV_WHATCONFLICTS;
+				qva->qva_sourceCount++; break;
+    case POPT_WHATOBSOLETES: qva->qva_source |= RPMQV_WHATOBSOLETES;
 				qva->qva_sourceCount++; break;
     case POPT_TRIGGEREDBY: qva->qva_source |= RPMQV_TRIGGEREDBY;
 				qva->qva_sourceCount++; break;
@@ -99,9 +111,14 @@ static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
 	qva->qva_sourceCount++;
 	break;
     case POPT_QUERYBYNUMBER:
-	qva->qva_source |= RPMQV_DBOFFSET; 
+	qva->qva_source |= RPMQV_DBOFFSET;
 	qva->qva_sourceCount++;
 	break;
+
+    case POPT_NOPASSWORD:
+	qva->nopassword = 1;
+	break;
+
     }
 }
 
@@ -111,7 +128,7 @@ static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
 /*@unchecked@*/
 struct poptOption rpmQVSourcePoptTable[] = {
 /*@-type@*/ /* FIX: cast? */
- { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA, 
+ { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA,
 	rpmQVSourceArgCallback, 0, NULL, NULL },
 /*@=type@*/
  { "all", 'a', 0, NULL, 'a',
@@ -149,18 +166,22 @@ struct poptOption rpmQVSourcePoptTable[] = {
 	N_("query source metadata from spec file parse"), N_("<spec>") },
  { "tid", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_QUERYBYTID,
 	N_("query/verify package(s) from install transaction"), "TID" },
- { "triggeredby", '\0', 0, NULL, POPT_TRIGGEREDBY, 
+ { "triggeredby", '\0', 0, NULL, POPT_TRIGGEREDBY,
 	N_("query the package(s) triggered by the package"), "PACKAGE" },
  { "verify", 'V', POPT_ARGFLAG_DOC_HIDDEN, NULL, 'V',
 	N_("rpm verify mode"), NULL },
- { "whatrequires", '\0', 0, NULL, POPT_WHATREQUIRES, 
+ { "whatrequires", '\0', 0, NULL, POPT_WHATREQUIRES,
 	N_("query/verify the package(s) which require a dependency"), "CAPABILITY" },
- { "whatneeds", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_WHATNEEDS, 
+ { "whatneeds", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_WHATNEEDS,
 	N_("query/verify the package(s) which require any contained provide"),
 	"CAPABILITY" },
 
- { "whatprovides", '\0', 0, NULL, POPT_WHATPROVIDES, 
+ { "whatprovides", '\0', 0, NULL, POPT_WHATPROVIDES,
 	N_("query/verify the package(s) which provide a dependency"), "CAPABILITY" },
+ { "whatconflicts", '\0', 0, NULL, POPT_WHATCONFLICTS,
+	N_("query/verify the package(s) which conflict with a dependency"), "CAPABILITY" },
+ { "whatobsoletes", '\0', 0, NULL, POPT_WHATOBSOLETES,
+	N_("query/verify the package(s) which obsolete a dependency"), "CAPABILITY" },
 
  { "transaction", 'T', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN, &giFlags, (RPMGI_TSADD|RPMGI_TSORDER),
 	N_("create transaction set"), NULL},
@@ -180,14 +201,16 @@ struct poptOption rpmQVSourcePoptTable[] = {
 /* ========== Query specific popt args */
 
 static void queryArgCallback(poptContext con,
-		/*@unused@*/enum poptCallbackReason reason,
-		const struct poptOption * opt, const char * arg, 
+		/*@unused@*/ enum poptCallbackReason reason,
+		const struct poptOption * opt, const char * arg,
 		/*@unused@*/ const void * data)
 	/*@globals rpmQVKArgs @*/
 	/*@modifies con, rpmQVKArgs @*/
 {
     QVA_t qva = &rpmQVKArgs;
 
+    /* XXX avoid accidental collisions with POPT_BIT_SET for flags */
+    if (opt->arg == NULL)
     switch (opt->val) {
     case 'c': qva->qva_flags |= QUERY_FOR_CONFIG | QUERY_FOR_LIST; break;
     case 'd': qva->qva_flags |= QUERY_FOR_DOCS | QUERY_FOR_LIST; break;
@@ -200,6 +223,30 @@ static void queryArgCallback(poptContext con,
     case POPT_QUERYFORMAT:
 	if (arg) {
 	    char * qf = (char *)qva->qva_queryFormat;
+	    char * b = NULL;
+	    size_t nb = 0;
+
+	    /* Read queryformat from file. */
+	    if (arg[0] == '/') {
+		const char * fn = arg;
+		int rc;
+
+		rc = poptReadFile(fn, &b, &nb, POPT_READFILE_TRIMNEWLINES);
+		if (rc != 0)
+		    goto _qfexit;
+		if (b == NULL || nb == 0)	/* XXX can't happen */
+		    goto _qfexit;
+		/* XXX trim double quotes */
+		if (*b == '"') {
+		    while (nb > 0 && b[nb] != '"')
+			b[nb--] = '\0';
+		    b[nb] = '\0';
+		    arg = b + 1;
+		} else
+		    arg = b;
+	    }
+
+	    /* Append to existing queryformat. */
 	    if (qf) {
 		size_t len = strlen(qf) + strlen(arg) + 1;
 		qf = xrealloc(qf, len);
@@ -209,6 +256,9 @@ static void queryArgCallback(poptContext con,
 		strcpy(qf, arg);
 	    }
 	    qva->qva_queryFormat = qf;
+
+	_qfexit:
+	    b = _free(b);
 	}
 	break;
 
@@ -266,7 +316,7 @@ static void queryArgCallback(poptContext con,
 /*@unchecked@*/
 struct poptOption rpmQueryPoptTable[] = {
 /*@-type@*/ /* FIX: cast? */
- { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE, 
+ { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE,
 	queryArgCallback, 0, NULL, NULL },
 /*@=type@*/
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmQVSourcePoptTable, 0,
@@ -287,25 +337,25 @@ struct poptOption rpmQueryPoptTable[] = {
 	N_("add suggested packages to transaction"), NULL },
 
  /* Duplicate file attr flags from packages into command line options. */
- { "noconfig", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ { "noconfig", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_CONFIG,
         N_("skip %%config files"), NULL },
- { "nodoc", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ { "nodoc", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_DOC,
         N_("skip %%doc files"), NULL },
- { "noghost", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ { "noghost", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_GHOST,
         N_("skip %%ghost files"), NULL },
 #ifdef	NOTEVER		/* XXX there's hardly a need for these */
- { "nolicense", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ { "nolicense", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_LICENSE,
         N_("skip %%license files"), NULL },
- { "noreadme", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ { "noreadme", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_README,
         N_("skip %%readme files"), NULL },
 #endif
 
- { "qf", '\0', POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN, 0, 
+ { "qf", '\0', POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN, 0,
 	POPT_QUERYFORMAT, NULL, NULL },
  { "queryformat", '\0', POPT_ARG_STRING, 0, POPT_QUERYFORMAT,
 	N_("use the following query format"), N_("QUERYFORMAT") },
@@ -323,13 +373,13 @@ struct poptOption rpmQueryPoptTable[] = {
  */
 struct poptOption rpmVerifyPoptTable[] = {
 /*@-type@*/ /* FIX: cast? */
- { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE, 
+ { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE,
 	queryArgCallback, 0, NULL, NULL },
 /*@=type@*/
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmQVSourcePoptTable, 0,
 	NULL, NULL },
 
- { "aid", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ { "aid", '\0', POPT_BIT_SET|POPT_ARGFLAG_TOGGLE|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.depFlags, RPMDEPS_FLAG_ADDINDEPS,
 	N_("add suggested packages to transaction"), NULL },
 
@@ -443,15 +493,18 @@ struct poptOption rpmSignPoptTable[] = {
 	N_("sign package(s) (identical to --addsign)"), NULL },
  { "sign", '\0', POPT_ARGFLAG_DOC_HIDDEN, &rpmQVKArgs.sign, 0,
 	N_("generate signature"), NULL },
- /* XXX perhaps POPT_ARG_INT instead of callback. */
+
  { "trust", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0,  POPT_TRUST,
-        N_("specify trust metric"), "TRUST" },
+        N_("specify trust metric"), N_("TRUST") },
  { "trusted", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.trust, 1,
         N_("set ultimate trust when importing pubkey(s)"), NULL },
  { "untrusted", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.trust, -1,
         N_("unset ultimate trust when importing pubkey(s)"), NULL },
+ { "nopassword", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0,  POPT_NOPASSWORD,
+        N_("disable password challenge"), NULL },
+ /* XXX perhaps POPT_ARG_INT instead of callback. */
 
  { "nodigest", '\0', POPT_BIT_SET, &rpmQVKArgs.qva_flags, VERIFY_DIGEST,
         N_("don't verify package digest(s)"), NULL },
