@@ -12,27 +12,30 @@
 #endif
 
 #include <rpmio.h>
-#define	_RPMTAG_INTERNAL
+#include <rpmiotypes.h>
+#include <poptIO.h>
+
 #include <rpmtag.h>
-#include <rpmcli.h>
-
 #include "rpmdb.h"
-#include "rpmfi.h"
 
-#include "rpmgi.h"
+#include "rpmfi.h"
+#define	_RPMTS_INTERNAL		/* XXX for ts->rdb */
 #include "rpmts.h"
+#include "rpmgi.h"
 
 #include "manifest.h"
 #include "misc.h"	/* XXX for currentDirectory() */
 
+#include <rpmcli.h>
+
 #include "debug.h"
 
-/*@access headerSprintfExtension@*/	/* XXX cast, increment */
+/*@access rpmts @*/	/* XXX cast */
 
 /**
  */
 static void printFileInfo(char * te, const char * name,
-			  unsigned int size, unsigned short mode,
+			  size_t size, unsigned short mode,
 			  unsigned int mtime,
 			  unsigned short rdev, unsigned int nlink,
 			  const char * owner, const char * group,
@@ -49,7 +52,7 @@ static void printFileInfo(char * te, const char * name,
     char ownerfield[8+1], groupfield[8+1];
 #endif
     char timefield[100];
-    time_t when = mtime;  /* important if sizeof(uint32_t) ! sizeof(time_t) */
+    time_t when = mtime;  /* important if sizeof(rpmuint32_t) ! sizeof(time_t) */
     struct tm * tm;
     static time_t now;
     static struct tm nowtm;
@@ -74,9 +77,9 @@ static void printFileInfo(char * te, const char * name,
     /* In verbose file listing output, give the owner and group fields
        more width and at the same time reduce the nlink and size fields
        more to typical sizes within OpenPKG. */
-    sprintf(sizefield, "%8u", size);
+    sprintf(sizefield, "%8u", (unsigned)size);
 #else
-    sprintf(sizefield, "%12u", size);
+    sprintf(sizefield, "%12u", (unsigned)size);
 #endif
 
     /* this knows too much about dev_t */
@@ -122,17 +125,19 @@ static void printFileInfo(char * te, const char * name,
        more width and at the same time reduce the nlink and size fields
        more to typical sizes within OpenPKG. */
     sprintf(te, "%s %d %-13s %-13s %8s %s %s", perms,
+	(int)nlink, ownerfield, groupfield, sizefield, timefield, namefield);
 #else
     sprintf(te, "%s %4d %-7s %-8s %10s %s %s", perms,
-#endif
 	(int)nlink, ownerfield, groupfield, sizefield, timefield, namefield);
+#endif
     perms = _free(perms);
 }
 
 /**
  */
 static inline /*@null@*/ const char * queryHeader(Header h, const char * qfmt)
-	/*@*/
+	/*@globals internalState @*/
+	/*@modifies h, internalState @*/
 {
     const char * errstr = "(unkown error)";
     const char * str;
@@ -182,7 +187,12 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
     *te = '\0';
 
     if (qva->qva_queryFormat != NULL) {
-	const char * str = queryHeader(h, qva->qva_queryFormat);
+	const char * str;
+/*@-type@*/	/* FIX rpmtsGetRDB()? */
+	(void) headerSetRpmdb(h, ts->rdb);
+/*@=type@*/
+	str = queryHeader(h, qva->qva_queryFormat);
+	(void) headerSetRpmdb(h, NULL);
 	if (str) {
 	    size_t tx = (te - t);
 
@@ -223,7 +233,7 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	const char * fuser;
 	const char * fgroup;
 	const char * flink;
-	uint32_t fnlink;
+	rpmuint32_t fnlink;
 
 	fflags = rpmfiFFlags(fi);
 	fmode = rpmfiFMode(fi);
@@ -237,7 +247,7 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	    size_t dlen = 0;
 	    const unsigned char * digest = rpmfiDigest(fi, &dalgo, &dlen);
 	    char * p;
-	    int j;
+	    size_t j;
 	    fdigest = p = xcalloc(1, ((2 * dlen) + 1));
 	    for (j = 0; j < dlen; j++) {
 		unsigned k = *digest++;
@@ -387,6 +397,10 @@ static int rpmgiShowMatches(QVA_t qva, rpmts ts)
 	Header h;
 	int rc;
 
+#ifdef	NOTYET	/* XXX exiting here will leave stale locks. */
+	(void) rpmdbCheckSignals();
+#endif
+
 	h = rpmgiHeader(gi);
 	if (h == NULL)		/* XXX perhaps stricter break instead? */
 	    continue;
@@ -406,7 +420,7 @@ int rpmcliShowMatches(QVA_t qva, rpmts ts)
     int ec = 1;
 
     qva->qva_showFAIL = qva->qva_showOK = 0;
-    while ((h = rpmdbNextIterator(qva->qva_mi)) != NULL) {
+    while ((h = rpmmiNext(qva->qva_mi)) != NULL) {
 	ec = qva->qva_showPackage(qva, ts, h);
 	if (ec)
 	    qva->qva_showFAIL++;
@@ -415,7 +429,7 @@ int rpmcliShowMatches(QVA_t qva, rpmts ts)
 	if (qva->qva_source == RPMQV_DBOFFSET)
 	    break;
     }
-    qva->qva_mi = rpmdbFreeIterator(qva->qva_mi);
+    qva->qva_mi = rpmmiFree(qva->qva_mi);
     return ec;
 }
 
@@ -449,6 +463,22 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 	return 1;
 
     switch (qva->qva_source) {
+#ifdef	NOTYET
+    default:
+#endif
+    case RPMQV_GROUP:
+    case RPMQV_TRIGGEREDBY:
+    case RPMQV_WHATCONFLICTS:
+    case RPMQV_WHATOBSOLETES:
+	qva->qva_mi = rpmtsInitIterator(ts, qva->qva_source, arg, 0);
+	if (qva->qva_mi == NULL) {
+	    rpmlog(RPMLOG_NOTICE, _("key \"%s\" not found in %s table\n"),
+			arg, tagName((rpmTag)qva->qva_source));
+	    res = 1;
+	} else
+	    res = rpmcliShowMatches(qva, ts);
+	break;
+
     case RPMQV_RPM:
 	res = rpmgiShowMatches(qva, ts);
 	break;
@@ -471,6 +501,7 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 		? qva->qva_specQuery(ts, qva, arg) : 1);
 	break;
 
+#ifdef	DYING
     case RPMQV_GROUP:
 	qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_GROUP, arg, 0);
 	if (qva->qva_mi == NULL) {
@@ -489,12 +520,13 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 	} else
 	    res = rpmcliShowMatches(qva, ts);
 	break;
+#endif
 
     case RPMQV_SOURCEPKGID:
     case RPMQV_PKGID:
     {	unsigned char MD5[16];
 	unsigned char * t;
-	uint32_t tag;
+	rpmuint32_t tag;
 
 	for (i = 0, s = arg; *s && isxdigit(*s); s++, i++)
 	    {};
@@ -622,7 +654,7 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 
 	if (*s == '\0') {
 	    char fnbuf[PATH_MAX];
-	    fn = realpath(arg, fnbuf);
+	    fn = Realpath(arg, fnbuf);
 	    fn = xstrdup( (fn != NULL ? fn : arg) );
 	} else if (*arg != '/') {
 	    const char *curDir = currentDirectory();
@@ -702,6 +734,8 @@ assert(fn != NULL);
 }
 
 int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
+	/*@globals rpmioFtsOpts @*/
+	/*@modifies rpmioFtsOpts @*/
 {
     rpmRC rpmrc = RPMRC_NOTFOUND;
     int ec = 0;
@@ -709,7 +743,7 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
     switch (qva->qva_source) {
     case RPMQV_ALL:
 	qva->qva_gi = rpmgiNew(ts, RPMDBI_PACKAGES, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, RPMGI_NONE);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, rpmioFtsOpts, RPMGI_NONE);
 
 	if (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD)	/* Load the ts with headers. */
 	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
@@ -724,7 +758,7 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
 	break;
     case RPMQV_RPM:
 	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, rpmioFtsOpts, giFlags);
 
 	if (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD)	/* Load the ts with headers. */
 	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
@@ -739,7 +773,7 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
 	break;
     case RPMQV_HDLIST:
 	qva->qva_gi = rpmgiNew(ts, RPMDBI_HDLIST, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, rpmioFtsOpts, giFlags);
 
 	if (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD)	/* Load the ts with headers. */
 	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
@@ -753,10 +787,10 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
 	rpmtsEmpty(ts);
 	break;
     case RPMQV_FTSWALK:
-	if (ftsOpts == 0)
-	    ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+	if (rpmioFtsOpts == 0)
+	    rpmioFtsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
 	qva->qva_gi = rpmgiNew(ts, RPMDBI_FTSWALK, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, rpmioFtsOpts, giFlags);
 
 	if (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD)	/* Load the ts with headers. */
 	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
@@ -772,7 +806,7 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
     default:
       if (giFlags & RPMGI_TSADD) {
 	qva->qva_gi = rpmgiNew(ts, RPMDBI_LABEL, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts,
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, rpmioFtsOpts,
 		(giFlags | (RPMGI_NOGLOB               )));
 	if (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD)	/* Load the ts with headers. */
 	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
@@ -786,7 +820,7 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
 	rpmtsEmpty(ts);
       } else {
 	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts,
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, rpmioFtsOpts,
 		(giFlags | (RPMGI_NOGLOB|RPMGI_NOHEADER)));
 	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK) {
 	    const char * path;
