@@ -5,21 +5,19 @@
 #include "system.h"
 
 #include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
-#include <rpmlib.h>
+#include <rpmiotypes.h>		/* XXX fnpyKey */
+
+#include <rpmtag.h>
+#include <rpmtypes.h>
 
-#include "rpmal.h"
 #define	_RPMDS_INTERNAL
-#include "rpmds.h"
-#include "rpmfi.h"
+#include <rpmds.h>
+#include <rpmal.h>
 
 #include "debug.h"
 
 typedef /*@abstract@*/ struct availablePackage_s * availablePackage;
 
-/*@unchecked@*/
-int _rpmal_debug = 0;
-
 /*@access alKey @*/
 /*@access alNum @*/
 /*@access rpmal @*/
@@ -37,7 +35,7 @@ struct availablePackage_s {
 /*@refcounted@*/ /*@null@*/
     rpmfi fi;			/*!< File info set. */
 
-    uint32_t tscolor;		/*!< Transaction color bits. */
+    rpmuint32_t tscolor;	/*!< Transaction color bits. */
 
 /*@exposed@*/ /*@dependent@*/ /*@null@*/
     fnpyKey key;		/*!< Associated file name/python object */
@@ -84,9 +82,9 @@ typedef /*@abstract@*/ struct fileIndexEntry_s *	fileIndexEntry;
 struct fileIndexEntry_s {
 /*@dependent@*/ /*@relnull@*/
     const char * baseName;	/*!< File basename. */
-    int baseNameLen;
+    size_t baseNameLen;
     alNum pkgNum;		/*!< Containing package index. */
-    uint32_t ficolor;
+    rpmuint32_t ficolor;
 };
 
 typedef /*@abstract@*/ struct dirInfo_s *		dirInfo;
@@ -98,7 +96,7 @@ typedef /*@abstract@*/ struct dirInfo_s *		dirInfo;
 struct dirInfo_s {
 /*@owned@*/ /*@relnull@*/
     const char * dirName;	/*!< Directory path (+ trailing '/'). */
-    int dirNameLen;		/*!< No. bytes in directory path. */
+    size_t dirNameLen;		/*!< No. bytes in directory path. */
 /*@owned@*/
     fileIndexEntry files;	/*!< Array of files in directory. */
     int numFiles;		/*!< No. files in directory. */
@@ -114,7 +112,7 @@ struct rpmal_s {
     int delta;			/*!< Delta for pkg list reallocation. */
     int size;			/*!< No. of pkgs in list. */
     int alloced;		/*!< No. of pkgs allocated for list. */
-    uint32_t tscolor;		/*!< Transaction color. */
+    rpmuint32_t tscolor;	/*!< Transaction color. */
     int numDirs;		/*!< No. of directories. */
 /*@owned@*/ /*@null@*/
     dirInfo dirs;		/*!< Set of directories. */
@@ -134,25 +132,13 @@ static void rpmalFreeIndex(rpmal al)
     }
 }
 
-#ifdef	DYING
-/**
- * Return number of packages in list.
- * @param al		available list
- * @return		no. of packages in list
- */
-static int alGetSize(/*@null@*/ const rpmal al)
-	/*@*/
-{
-    return (al != NULL ? al->size : 0);
-}
-#endif
-
 static inline alNum alKey2Num(/*@unused@*/ /*@null@*/ const rpmal al,
 		/*@null@*/ alKey pkgKey)
 	/*@*/
 {
     /*@-nullret -temptrans -retalias @*/
     union { alKey key; alNum num; } u;
+    u.num = 0;
     u.key = pkgKey;
     return u.num;
     /*@=nullret =temptrans =retalias @*/
@@ -164,34 +150,12 @@ static inline alKey alNum2Key(/*@unused@*/ /*@null@*/ const rpmal al,
 {
     /*@-nullret -temptrans -retalias @*/
     union { alKey key; alNum num; } u;
+    u.key = 0;
     u.num = pkgNum;
     return u.key;
     /*@=nullret =temptrans =retalias @*/
 }
 
-#ifdef	DYING
-/**
- * Return available package.
- * @param al		available list
- * @param pkgKey	available package key
- * @return		available package pointer
- */
-/*@dependent@*/ /*@null@*/
-static availablePackage alGetPkg(/*@null@*/ const rpmal al,
-		/*@null@*/ alKey pkgKey)
-	/*@*/
-{
-    alNum pkgNum = alKey2Num(al, pkgKey);
-    availablePackage alp = NULL;
-
-    if (al != NULL && pkgNum >= 0 && pkgNum < alGetSize(al)) {
-	if (al->list != NULL)
-	    alp = al->list + pkgNum;
-    }
-    return alp;
-}
-#endif
-
 rpmal rpmalCreate(int delta)
 {
     rpmal al = xcalloc(1, sizeof(*al));
@@ -221,7 +185,8 @@ rpmal rpmalFree(rpmal al)
 
     if ((alp = al->list) != NULL)
     for (i = 0; i < al->size; i++, alp++) {
-	alp->provides = rpmdsFree(alp->provides);
+	(void)rpmdsFree(alp->provides);
+	alp->provides = NULL;
 	alp->fi = rpmfiFree(alp->fi);
     }
 
@@ -253,7 +218,7 @@ static int dieCompare(const void * one, const void * two)
     const dirInfo a = (const dirInfo) one;
     const dirInfo b = (const dirInfo) two;
     /*@=castexpose@*/
-    int lenchk = a->dirNameLen - b->dirNameLen;
+    int lenchk = (int)a->dirNameLen - (int)b->dirNameLen;
 
     if (lenchk || a->dirNameLen == 0)
 	return lenchk;
@@ -278,7 +243,7 @@ static int fieCompare(const void * one, const void * two)
     const fileIndexEntry a = (const fileIndexEntry) one;
     const fileIndexEntry b = (const fileIndexEntry) two;
     /*@=castexpose@*/
-    int lenchk = a->baseNameLen - b->baseNameLen;
+    int lenchk = (int)a->baseNameLen - (int)b->baseNameLen;
 
     if (lenchk)
 	return lenchk;
@@ -286,19 +251,6 @@ static int fieCompare(const void * one, const void * two)
     if (a->baseName == NULL || b->baseName == NULL)
 	return lenchk;
 
-#ifdef	NOISY
-/*@-modfilesys@*/
-if (_rpmal_debug) {
-fprintf(stderr, "\t\tstrcmp(%p:%p, %p:%p)", a, a->baseName, b, b->baseName);
-#if 0
-fprintf(stderr, " a %s", a->baseName);
-#endif
-fprintf(stderr, " b %s", a->baseName);
-fprintf(stderr, "\n");
-}
-/*@=modfilesys@*/
-#endif
-
     return strcmp(a->baseName, b->baseName);
 }
 
@@ -313,11 +265,6 @@ void rpmalDel(rpmal al, alKey pkgKey)
 
     alp = al->list + pkgNum;
 
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "*** del %p[%d]\n", al->list, pkgNum);
-/*@=modfilesys@*/
-
     /* Delete directory/file info entries from added package list. */
     if ((fi = alp->fi) != NULL)
     if (rpmfiFC(fi) > 0) {
@@ -348,11 +295,6 @@ fprintf(stderr, "*** del %p[%d]\n", al->list, pkgNum);
 	    if (die == NULL)
 		continue;
 
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "--- die[%5d] %p [%3d] %s\n", (int)(die - al->dirs), die, die->dirNameLen, die->dirName);
-/*@=modfilesys@*/
-
 	    last = die->numFiles;
 	    fie = die->files + last - 1;
 	    for (i = last - 1; i >= 0; i--, fie--) {
@@ -360,18 +302,8 @@ fprintf(stderr, "--- die[%5d] %p [%3d] %s\n", (int)(die - al->dirs), die, die->d
 		    /*@innercontinue@*/ continue;
 		die->numFiles--;
 
-		if (i < die->numFiles) {
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "\t%p[%3d] memmove(%p:%p,%p:%p,0x%x) %s <- %s\n", die->files, die->numFiles, fie, fie->baseName, fie+1, (fie+1)->baseName, (unsigned) ((die->numFiles - i) * sizeof(*fie)), fie->baseName, (fie+1)->baseName);
-/*@=modfilesys@*/
-
+		if (i < die->numFiles)
 		    memmove(fie, fie+1, (die->numFiles - i) * sizeof(*fie));
-		}
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "\t%p[%3d] memset(%p,0,0x%x) %p [%3d] %s\n", die->files, die->numFiles, die->files + die->numFiles, (unsigned)sizeof(*fie), fie->baseName, fie->baseNameLen, fie->baseName);
-/*@=modfilesys@*/
 		memset(die->files + die->numFiles, 0, sizeof(*fie)); /* overkill */
 
 	    }
@@ -384,19 +316,9 @@ fprintf(stderr, "\t%p[%3d] memset(%p,0,0x%x) %p [%3d] %s\n", die->files, die->nu
 	    die->files = _free(die->files);
 	    die->dirName = _free(die->dirName);
 	    al->numDirs--;
-	    if ((die - al->dirs) < al->numDirs) {
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "    die[%5d] memmove(%p,%p,0x%x)\n", (int)(die - al->dirs), die, die+1, (unsigned)((al->numDirs - (die - al->dirs)) * sizeof(*die)));
-/*@=modfilesys@*/
-
+	    if ((die - al->dirs) < al->numDirs)
 		memmove(die, die+1, (al->numDirs - (die - al->dirs)) * sizeof(*die));
-	    }
 
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "    die[%5d] memset(%p,0,0x%x)\n", al->numDirs, al->dirs + al->numDirs, (unsigned)sizeof(*die));
-/*@=modfilesys@*/
 	    memset(al->dirs + al->numDirs, 0, sizeof(*al->dirs)); /* overkill */
 	}
 
@@ -408,7 +330,8 @@ fprintf(stderr, "    die[%5d] memset(%p,0,0x%x)\n", al->numDirs, al->dirs + al->
 	}
     }
 
-    alp->provides = rpmdsFree(alp->provides);
+    (void)rpmdsFree(alp->provides);
+    alp->provides = NULL;
     alp->fi = rpmfiFree(alp->fi);
 
     memset(alp, 0, sizeof(*alp));	/* XXX trash and burn */
@@ -416,7 +339,7 @@ fprintf(stderr, "    die[%5d] memset(%p,0,0x%x)\n", al->numDirs, al->dirs + al->
 }
 
 alKey rpmalAdd(rpmal * alistp, alKey pkgKey, fnpyKey key,
-		rpmds provides, rpmfi fi, uint32_t tscolor)
+		rpmds provides, rpmfi fi, rpmuint32_t tscolor)
 {
     alNum pkgNum;
     rpmal al;
@@ -446,15 +369,14 @@ alKey rpmalAdd(rpmal * alistp, alKey pkgKey, fnpyKey key,
     alp->key = key;
     alp->tscolor = tscolor;
 
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "*** add %p[%d] 0x%x\n", al->list, pkgNum, tscolor);
-/*@=modfilesys@*/
-
+/*@-assignexpose -castexpose @*/
     alp->provides = rpmdsLink(provides, "Provides (rpmalAdd)");
     alp->fi = rpmfiLink(fi, "Files (rpmalAdd)");
+/*@=assignexpose =castexpose @*/
 
+/*@-castexpose@*/
     fi = rpmfiLink(alp->fi, "Files index (rpmalAdd)");
+/*@=castexpose@*/
     fi = rpmfiInit(fi, 0);
     if (rpmfiFC(fi) > 0) {
 	dirInfo dieNeedle =
@@ -522,10 +444,6 @@ fprintf(stderr, "*** add %p[%d] 0x%x\n", al->list, pkgNum, tscolor);
 		die->dirNameLen = dieNeedle->dirNameLen;
 		die->files = NULL;
 		die->numFiles = 0;
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "+++ die[%5d] %p [%3d] %s\n", al->numDirs, die, die->dirNameLen, die->dirName);
-/*@=modfilesys@*/
 
 		al->numDirs++;
 	    }
@@ -549,13 +467,6 @@ fprintf(stderr, "+++ die[%5d] %p [%3d] %s\n", al->numDirs, die, die->dirNameLen,
 
 	    fie = die->files + die->numFiles;
 
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "    die[%5d] %p->files [%p[%d],%p) -> [%p[%d],%p)\n", dirMapping[dx], die,
-die->files, die->numFiles, die->files+die->numFiles,
-fie, (next - first), fie + (next - first));
-/*@=modfilesys@*/
-
 	    /* Rewind to first file, generate file index entry for each file. */
 	    fi = rpmfiInit(fi, first);
 	    while ((first = rpmfiNext(fi)) >= 0 && first < next) {
@@ -565,10 +476,6 @@ fie, (next - first), fie + (next - first));
 		fie->baseNameLen = (fie->baseName ? strlen(fie->baseName) : 0);
 		fie->pkgNum = pkgNum;
 		fie->ficolor = rpmfiFColor(fi);
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "\t%p[%3d] %p:%p[%2d] %s\n", die->files, die->numFiles, fie, fie->baseName, fie->baseNameLen, rpmfiFN(fi));
-/*@=modfilesys@*/
 
 		die->numFiles++;
 		fie++;
@@ -611,9 +518,9 @@ static int indexcmp(const void * one, const void * two)
     return strcmp(a->entry, b->entry);
 }
 
-void rpmalAddProvides(rpmal al, alKey pkgKey, rpmds provides, uint32_t tscolor)
+void rpmalAddProvides(rpmal al, alKey pkgKey, rpmds provides, rpmuint32_t tscolor)
 {
-    uint32_t dscolor;
+    rpmuint32_t dscolor;
     const char * Name;
     alNum pkgNum = alKey2Num(al, pkgKey);
     availableIndex ai = &al->index;
@@ -643,7 +550,7 @@ void rpmalAddProvides(rpmal al, alKey pkgKey, rpmds provides, uint32_t tscolor)
 /*@-assignexpose@*/
 	aie->entry = Name;
 /*@=assignexpose@*/
-	aie->entryLen = strlen(Name);
+	aie->entryLen = (unsigned short)strlen(Name);
 	ix = rpmdsIx(provides);
 
 /* XXX make sure that element index fits in unsigned short */
@@ -686,8 +593,8 @@ void rpmalMakeIndex(rpmal al)
 fnpyKey *
 rpmalAllFileSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 {
-    uint32_t tscolor;
-    uint32_t ficolor;
+    rpmuint32_t tscolor;
+    rpmuint32_t ficolor;
     int found = 0;
     const char * dirName;
     const char * baseName;
@@ -738,11 +645,6 @@ rpmalAllFileSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 	 die++)
     {
 
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "==> die %p %s\n", die, (die->dirName ? die->dirName : "(nil)"));
-/*@=modfilesys@*/
-
 /*@-observertrans@*/
 	fieNeedle->baseName = baseName;
 /*@=observertrans@*/
@@ -752,11 +654,6 @@ fprintf(stderr, "==> die %p %s\n", die, (die->dirName ? die->dirName : "(nil)"))
 	if (fie == NULL)
 	    continue;	/* XXX shouldn't happen */
 
-/*@-modfilesys@*/
-if (_rpmal_debug)
-fprintf(stderr, "==> fie %p %s\n", fie, (fie->baseName ? fie->baseName : "(nil)"));
-/*@=modfilesys@*/
-
 	alp = al->list + fie->pkgNum;
 
         /* Ignore colored files not in our rainbow. */
@@ -816,7 +713,7 @@ rpmalAllSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
     /*@-assignexpose -temptrans@*/
     needle->entry = KName;
     /*@=assignexpose =temptrans@*/
-    needle->entryLen = strlen(needle->entry);
+    needle->entryLen = (unsigned short)strlen(needle->entry);
 
     match = bsearch(needle, ai->index, ai->size, sizeof(*ai->index), indexcmp);
     if (match == NULL)
