@@ -6,21 +6,24 @@
 #include "system.h"
 
 #include <rpmio_internal.h>
+#include <poptIO.h>
 #include <rpmbc.h>		/* XXX beecrypt base64 */
 #include <rpmtag.h>
-#include <rpmcli.h>
+#include <rpmtypes.h>
 #define	_RPMEVR_INTERNAL	/* XXX RPMSENSE_KEYRING */
 #include <rpmevr.h>
+#include <rpmdb.h>
+#include <rpmxar.h>
+#include <pkgio.h>
+#include "signature.h"
 
 #include <rpmts.h>
 
-#include "rpmdb.h"
 #include "rpmgi.h"
 
-#include <rpmxar.h>
-#include <pkgio.h>
+#include <rpmversion.h>
+#include <rpmcli.h>
 
-#include "signature.h"
 #include "debug.h"
 
 /*@access FD_t @*/		/* XXX stealing digests */
@@ -74,8 +77,10 @@ static int manageFile(/*@out@*/ FD_t *fdp,
 	}
 	if (fnp != NULL)
 	    *fnp = fn;
+/*@-refcounttrans@*/	/* FIX: XfdLink/XfdFree annotation */
 	*fdp = fdLink(fd, "manageFile return");
 	fd = fdFree(fd, "manageFile return");
+/*@=refcounttrans@*/
 	return 0;
     }
 
@@ -108,7 +113,7 @@ static int copyFile(FD_t *sfdp, const char **sfnp,
 
     while ((count = Fread(buf, sizeof(buf[0]), sizeof(buf), *sfdp)) > 0)
     {
-	if (Fwrite(buf, sizeof(buf[0]), count, *tfdp) != count) {
+	if (Fwrite(buf, sizeof(buf[0]), count, *tfdp) != (size_t)count) {
 	    rpmlog(RPMLOG_ERR, _("%s: Fwrite failed: %s\n"), *tfnp,
 		Fstrerror(*tfdp));
 	    goto exit;
@@ -139,7 +144,7 @@ exit:
  * @retval signid	signer fingerprint
  * @return		0 on success
  */
-static int getSignid(Header sigh, int sigtag, unsigned char * signid)
+static int getSignid(Header sigh, rpmSigTag sigtag, unsigned char * signid)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies *signid, fileSystem, internalState @*/
 {
@@ -147,7 +152,7 @@ static int getSignid(Header sigh, int sigtag, unsigned char * signid)
     int rc = 1;
     int xx;
 
-    he->tag = sigtag;
+    he->tag = (rpmTag) sigtag;
     xx = headerGet(sigh, he, 0);
     if (xx && he->p.ptr != NULL) {
 	pgpDig dig = pgpDigNew(0);
@@ -158,7 +163,7 @@ static int getSignid(Header sigh, int sigtag, unsigned char * signid)
 	}
      
 	he->p.ptr = _free(he->p.ptr);
-	dig = pgpDigFree(dig);
+	dig = pgpDigFree(dig, "getSignid");
     }
     return rc;
 }
@@ -191,21 +196,22 @@ static int rpmReSign(/*@unused@*/ rpmts ts,
     int deleting = (qva->qva_mode == RPMSIGN_DEL_SIGNATURE);
     rpmRC rc;
     int xx;
+    int i;
     
     tmprpm[0] = '\0';
 
     if (argv)
  {       /* start-of-arg-iteration */
-    uint32_t tag = (qva->qva_source == RPMQV_FTSWALK)
+    rpmuint32_t tag = (qva->qva_source == RPMQV_FTSWALK)
 	? RPMDBI_FTSWALK : RPMDBI_ARGLIST;
     rpmgiFlags _giFlags = RPMGI_NONE;
 
     gi = rpmgiNew(ts, tag, NULL, 0);
 /*@-mods@*/
-    if (ftsOpts == 0)
-	ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+    if (rpmioFtsOpts == 0)
+	rpmioFtsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
 /*@=mods@*/
-    rc = rpmgiSetArgs(gi, argv, ftsOpts, (_giFlags|RPMGI_NOHEADER));
+    rc = rpmgiSetArgs(gi, argv, rpmioFtsOpts, (_giFlags|RPMGI_NOHEADER));
 
     while (rpmgiNext(gi) == RPMRC_OK) {
 	const char * fn = rpmgiHdrPath(gi);
@@ -283,11 +289,14 @@ static int rpmReSign(/*@unused@*/ rpmts ts,
 		}
 	    }
 	    hi = headerFini(hi);
-	    oh = headerFree(oh);
+	    (void)headerFree(oh);
+	    oh = NULL;
 
-	    sigh = headerFree(sigh);
+	    (void)headerFree(sigh);
+	    sigh = NULL;
 	    sigh = headerLink(nh);
-	    nh = headerFree(nh);
+	    (void)headerFree(nh);
+	    nh = NULL;
 	}
 
 if (sigh != NULL) {
@@ -302,27 +311,28 @@ if (sigh != NULL) {
 	xx = headerDel(sigh, he, 0);
 
 	/* Toss and recalculate header+payload size and digests. */
-	he->tag = (rpmTag)RPMSIGTAG_SIZE;
-	xx = headerDel(sigh, he, 0);
-	xx = rpmAddSignature(sigh, sigtarget, RPMSIGTAG_SIZE, qva->passPhrase);
-	he->tag = (rpmTag)RPMSIGTAG_MD5;
-	xx = headerDel(sigh, he, 0);
-	xx = rpmAddSignature(sigh, sigtarget, RPMSIGTAG_MD5, qva->passPhrase);
-	he->tag = (rpmTag)RPMSIGTAG_SHA1;
-	xx = headerDel(sigh, he, 0);
-	xx = rpmAddSignature(sigh, sigtarget, RPMSIGTAG_SHA1, qva->passPhrase);
+	{   static const rpmuint32_t sigs[] =
+		{ RPMSIGTAG_SIZE, RPMSIGTAG_MD5, RPMSIGTAG_SHA1 };
+	    size_t nsigs = sizeof(sigs) / sizeof(sigs[0]);
+	    for (i = 0; i < (int)nsigs; i++) {
+		he->tag = (rpmTag)sigs[i];
+		xx = headerDel(sigh, he, 0);
+		xx = rpmAddSignature(sigh, sigtarget, (rpmSigTag) he->tag, qva->passPhrase);
+		if (xx)
+		    goto exit;
+	    }
+	}
 
-	if (deleting) {	/* Nuke all the signature tags. */
-	    he->tag = (rpmTag)RPMSIGTAG_GPG;
-	    xx = headerDel(sigh, he, 0);
-	    he->tag = (rpmTag)RPMSIGTAG_PGP5;
-	    xx = headerDel(sigh, he, 0);
-	    he->tag = (rpmTag)RPMSIGTAG_PGP;
-	    xx = headerDel(sigh, he, 0);
-	    he->tag = (rpmTag)RPMSIGTAG_DSA;
-	    xx = headerDel(sigh, he, 0);
-	    he->tag = (rpmTag)RPMSIGTAG_RSA;
-	    xx = headerDel(sigh, he, 0);
+	if (deleting) {
+	    /* Nuke all the signature tags. */
+	    static const rpmuint32_t sigs[] =
+		{ RPMSIGTAG_GPG, RPMSIGTAG_PGP5, RPMSIGTAG_PGP,
+		  RPMSIGTAG_DSA, RPMSIGTAG_RSA };
+	    size_t nsigs = sizeof(sigs) / sizeof(sigs[0]);
+	    for (i = 0; i < (int)nsigs; i++) {
+		he->tag = (rpmTag)sigs[i];
+		xx = headerDel(sigh, he, 0);
+	    }
 	} else {		/* If gpg/pgp is configured, replace the signature. */
 	  int addsig = 0;
 	  sigtag = RPMSIGTAG_GPG;
@@ -347,6 +357,8 @@ if (sigh != NULL) {
 		xx = headerDel(sigh, he, 0);
 		/*@switchbreak@*/ break;
 	    case RPMSIGTAG_GPG:
+		he->tag = (rpmTag)RPMSIGTAG_PGP;
+		xx = headerDel(sigh, he, 0);
 		he->tag = (rpmTag)RPMSIGTAG_DSA;
 		xx = headerDel(sigh, he, 0);
 		/*@fallthrough@*/
@@ -360,6 +372,8 @@ if (sigh != NULL) {
 	    he->tag = (rpmTag)sigtag;
 	    xx = headerDel(sigh, he, 0);
 	    xx = rpmAddSignature(sigh, sigtarget, sigtag, qva->passPhrase);
+	    if (xx)
+		goto exit;
 
 	    /* If package was previously signed, check for same signer. */
 	    memset(newsignid, 0, sizeof(newsignid));
@@ -445,7 +459,8 @@ exit:
     if (ofd)	(void) manageFile(&ofd, NULL, 0, res);
 
     lead = _free(lead);
-    sigh = headerFree(sigh);
+    (void)headerFree(sigh);
+    sigh = NULL;
 
     gi = rpmgiFree(gi);
 
@@ -470,8 +485,8 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
     const char * group = "Public Keys";
     const char * license = "pubkey";
     const char * buildhost = "localhost";
-    uint32_t pflags = (RPMSENSE_KEYRING|RPMSENSE_EQUAL);
-    uint32_t zero = 0;
+    rpmuint32_t pflags = (RPMSENSE_KEYRING|RPMSENSE_EQUAL);
+    rpmuint32_t zero = 0;
     pgpDig dig = NULL;
     pgpDigParams pubp = NULL;
     const char * d = NULL;
@@ -547,37 +562,44 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
     he->t = RPM_STRING_TYPE;
     he->c = 1;
     he->tag = RPMTAG_NAME;
-    he->p.str = "gpg-pubkey";
+    he->p.str = xstrdup("gpg-pubkey");
     xx = headerPut(h, he, 0);
+    he->p.ptr = _free(he->p.ptr);
     he->tag = RPMTAG_VERSION;
     he->p.str = v+8;
     xx = headerPut(h, he, 0);
     he->tag = RPMTAG_RELEASE;
-    he->p.str = r;
+    he->p.str = xstrdup(r);
     xx = headerPut(h, he, 0);
+    he->p.ptr = _free(he->p.ptr);
 
     /* Add Summary/Description/Group. */
     he->tag = RPMTAG_DESCRIPTION;
-    he->p.str = d;
+    he->p.str = xstrdup(d);
 #if defined(SUPPORT_IMPLICIT_TAG_DATA_TYPES)
     xx = headerAddI18NString(h, he->tag, he->p.ptr, "C");
 #else
     xx = headerPut(h, he, 0);
 #endif
+    he->p.ptr = _free(he->p.ptr);
+
     he->tag = RPMTAG_GROUP;
-    he->p.str = group;
+    he->p.str = xstrdup(group);
 #if defined(SUPPORT_IMPLICIT_TAG_DATA_TYPES)
     xx = headerAddI18NString(h, he->tag, he->p.ptr, "C");
 #else
     xx = headerPut(h, he, 0);
 #endif
+    he->p.ptr = _free(he->p.ptr);
+
     he->tag = RPMTAG_SUMMARY;
-    he->p.str = u;
+    he->p.str = xstrdup(u);
 #if defined(SUPPORT_IMPLICIT_TAG_DATA_TYPES)
     xx = headerAddI18NString(h, he->tag, he->p.ptr, "C");
 #else
     xx = headerPut(h, he, 0);
 #endif
+    he->p.ptr = _free(he->p.ptr);
 
 #ifdef	NOTYET	/* XXX can't erase pubkeys with "pubkey" arch. */
     /* Add a "pubkey" arch/os to avoid missing value NULL ptrs. */
@@ -590,8 +612,9 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
 #endif
 
     he->tag = RPMTAG_LICENSE;
-    he->p.str = license;
+    he->p.str = xstrdup(license);
     xx = headerPut(h, he, 0);
+    he->p.ptr = _free(he->p.ptr);
 
     he->tag = RPMTAG_SIZE;
     he->t = RPM_UINT32_TYPE;
@@ -601,6 +624,7 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
 
     he->append = 1;
 
+    /* Provides: gpg(IDENTITY) = PUBKEYVERSIONTYPE:PUBKEYID-CREATION */
     he->tag = RPMTAG_PROVIDENAME;
     he->t = RPM_STRING_ARRAY_TYPE;
     he->p.argv = &u;
@@ -617,6 +641,7 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
     he->c = 1;
     xx = headerPut(h, he, 0);
 
+    /* Provides: gpg(PUBKEYID) = PUBKEYVERSION:PUBKEYID-CREATION */
     he->tag = RPMTAG_PROVIDENAME;
     he->t = RPM_STRING_ARRAY_TYPE;
     he->p.argv = &n;
@@ -637,23 +662,26 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
 
     he->tag = RPMTAG_RPMVERSION;
     he->t = RPM_STRING_TYPE;
-    he->p.str = RPMVERSION;
+    he->p.str = xstrdup(RPMVERSION);
     he->c = 1;
     xx = headerPut(h, he, 0);
+    he->p.ptr = _free(he->p.ptr);
 
-    /* XXX W2DO: tag value inheirited from parent? */
+    /* XXX W2DO: tag value inherited from parent? */
     he->tag = RPMTAG_BUILDHOST;
     he->t = RPM_STRING_TYPE;
-    he->p.str = buildhost;
+    he->p.str = xstrdup(buildhost);
     he->c = 1;
     xx = headerPut(h, he, 0);
-    {   uint32_t tid = rpmtsGetTid(ts);
+    he->p.ptr = _free(he->p.ptr);
+
+    {   rpmuint32_t tid = rpmtsGetTid(ts);
 	he->tag = RPMTAG_INSTALLTIME;
 	he->t = RPM_UINT32_TYPE;
 	he->p.ui32p = &tid;
 	he->c = 1;
 	xx = headerPut(h, he, 0);
-	/* XXX W2DO: tag value inheirited from parent? */
+	/* XXX W2DO: tag value inherited from parent? */
 	he->tag = RPMTAG_BUILDTIME;
 	he->t = RPM_UINT32_TYPE;
 	he->p.ui32p = &tid;
@@ -662,7 +690,7 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
     }
 
 #ifdef	NOTYET
-    /* XXX W2DO: tag value inheirited from parent? */
+    /* XXX W2DO: tag value inherited from parent? */
     he->tag = RPMTAG_SOURCERPM;
     he->t = RPM_STRING_TYPE;
     he->p.str = fn;
@@ -678,8 +706,9 @@ rpmRC rpmcliImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktl
 
 exit:
     /* Clean up. */
-    h = headerFree(h);
-    dig = pgpDigFree(dig);
+    (void)headerFree(h);
+    h = NULL;
+    dig = pgpDigFree(dig, "rpmcliImportPubkey");
     n = _free(n);
     u = _free(u);
     v = _free(v);
@@ -702,13 +731,13 @@ exit:
 static int rpmcliImportPubkeys(const rpmts ts,
 		/*@unused@*/ QVA_t qva,
 		/*@null@*/ const char ** argv)
-	/*@globals RPMVERSION, rpmGlobalMacroContext, h_errno,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
 {
     const char * fn;
-    const unsigned char * pkt = NULL;
+    rpmuint8_t * pkt = NULL;
     size_t pktlen = 0;
     char * t = NULL;
     int res = 0;
@@ -801,7 +830,8 @@ pgpDig dig = fdGetDig(fd);
 	    he->tag = RPMTAG_HEADERIMMUTABLE;
 	    xx = headerGet(h, he, 0);
 	    if (!xx || he->p.ptr == NULL) {
-		h = headerFree(h);
+		(void)headerFree(h);
+		h = NULL;
 		rpmlog(RPMLOG_ERR, "%s: %s: %s\n", fn, _("headerGet failed"),
 			_("failed to retrieve original header\n"));
 		rc = RPMRC_FAIL;
@@ -812,13 +842,14 @@ pgpDig dig = fdGetDig(fd);
 	    if (hmagic && nmagic > 0)
 		(void) rpmDigestUpdate(dig->hdrsha1ctx, hmagic, nmagic);
 	    (void) rpmDigestUpdate(dig->hdrsha1ctx, he->p.ptr, he->c);
-	    dig->hdrmd5ctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
+	    dig->hdrctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
 	    if (hmagic && nmagic > 0)
-		(void) rpmDigestUpdate(dig->hdrmd5ctx, hmagic, nmagic);
-	    (void) rpmDigestUpdate(dig->hdrmd5ctx, he->p.ptr, he->c);
+		(void) rpmDigestUpdate(dig->hdrctx, hmagic, nmagic);
+	    (void) rpmDigestUpdate(dig->hdrctx, he->p.ptr, he->c);
 	    he->p.ptr = _free(he->p.ptr);
 	}
-	h = headerFree(h);
+	(void)headerFree(h);
+	h = NULL;
     }
 
     if (xar != NULL) {
@@ -849,12 +880,13 @@ exit:
     return rc;
 }
 
-int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
-		const char * fn)
+int rpmVerifySignatures(QVA_t qva, rpmts ts, void * _fd, const char * fn)
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     HE_t she = memset(alloca(sizeof(*she)), 0, sizeof(*she));
-    int res2, res3;
+/*@-castexpose@*/
+    FD_t fd = (FD_t)_fd;
+/*@=castexpose@*/
     char result[1024];
     char buf[8192], * b;
     char missingKeys[7164], * m;
@@ -866,20 +898,17 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
     const char * msg = NULL;
     int res = 0;
     int xx;
-    rpmRC rc;
+    rpmRC rc, sigres;
+    int failed;
     int nodigests = !(qva->qva_flags & VERIFY_DIGEST);
     int nosignatures = !(qva->qva_flags & VERIFY_SIGNATURE);
 
-    if ((rc = rpmtsOpenDB(ts, rpmtsDBMode(ts))) != RPMRC_OK) {
-	/* XXX rpmtsOpenDB has reported the error. */
-	res++;
-	goto exit;
-    }
-
     {
 	{   const char item[] = "Lead";
 	    msg = NULL;
+/*@-mods@*/	/* LCL: avoid void * _fd annotation for now. */
 	    rc = rpmpkgRead(item, fd, NULL, &msg);
+/*@=mods@*/
 	    if (rc != RPMRC_OK) {
 		rpmlog(RPMLOG_ERR, "%s: %s: %s\n", fn, item, msg);
 		msg = _free(msg);
@@ -891,7 +920,9 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 
 	{   const char item[] = "Signature";
 	    msg = NULL;
+/*@-mods@*/	/* LCL: avoid void * _fd annotation for now. */
 	    rc = rpmpkgRead(item, fd, &sigh, &msg);
+/*@=mods@*/
 	    switch (rc) {
 	    default:
 		rpmlog(RPMLOG_ERR, "%s: %s: %s\n", fn, item,
@@ -927,7 +958,9 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 	}
 
 	dig = rpmtsDig(ts);
+/*@-mods@*/	/* LCL: avoid void * _fd annotation for now. */
 	(void) fdSetDig(fd, dig);
+/*@=mods@*/
 	sigp = pgpGetSignature(dig);
 
 	/* XXX RSA needs the hash_algo, so decode early. */
@@ -938,18 +971,22 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 	    he->p.ptr = _free(he->p.ptr);
 	}
 
+/*@-mods@*/	/* LCL: avoid void * _fd annotation for now. */
 	if (headerIsEntry(sigh, (rpmTag)RPMSIGTAG_MD5))
 	    fdInitDigest(fd, PGPHASHALGO_MD5, 0);
+/*@=mods@*/
 
 	/* Read the file, generating digest(s) on the fly. */
+/*@-mods@*/	/* LCL: avoid void * _fd annotation for now. */
 	if (dig == NULL || sigp == NULL
 	 || readFile(fd, fn) != RPMRC_OK)
 	{
 	    res++;
 	    goto exit;
 	}
+/*@=mods@*/
 
-	res2 = 0;
+	failed = 0;
 	b = buf;		*b = '\0';
 	m = missingKeys;	*m = '\0';
 	u = untrustedKeys;	*u = '\0';
@@ -967,9 +1004,9 @@ assert(she->p.ptr != NULL);
 	    /* Clean up parameters from previous she->tag. */
 	    pgpDigClean(dig);
 
-/*@-noeffect@*/
+/*@-ownedtrans -noeffect@*/
 	    xx = pgpSetSig(dig, she->tag, she->t, she->p.ptr, she->c);
-/*@=noeffect@*/
+/*@=ownedtrans =noeffect@*/
 
 	    switch ((rpmSigTag)she->tag) {
 	    case RPMSIGTAG_RSA:
@@ -1003,64 +1040,56 @@ assert(she->p.ptr != NULL);
 		/*@notreached@*/ /*@switchbreak@*/ break;
 	    }
 
-	    res3 = rpmVerifySignature(dig, result);
+	    sigres = rpmVerifySignature(dig, result);
 
-	    if (res3) {
-		if (rpmIsVerbose()) {
+	    if (sigres) {
+		failed = 1;
+		if (rpmIsVerbose())
 		    b = stpcpy( stpcpy( stpcpy(b, "    "), result), "\n");
-		    res2 = 1;
-		} else {
-		    switch ((rpmSigTag)she->tag) {
-		    case RPMSIGTAG_SIZE:
-			b = stpcpy(b, "SIZE ");
-			res2 = 1;
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_SHA1:
-			b = stpcpy(b, "SHA1 ");
-			res2 = 1;
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_MD5:
-			b = stpcpy(b, "MD5 ");
-			res2 = 1;
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_RSA:
-			b = stpcpy(b, "RSA ");
-			res2 = 1;
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_DSA:
-			b = stpcpy(b, "(SHA1) DSA ");
-			res2 = 1;
-			/*@switchbreak@*/ break;
-		    default:
-			b = stpcpy(b, "?UnknownSignatureType? ");
-			res2 = 1;
-			/*@switchbreak@*/ break;
-		    }
+		else
+		switch ((rpmSigTag)she->tag) {
+		case RPMSIGTAG_SIZE:
+		    b = stpcpy(b, "SIZE ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_SHA1:
+		    b = stpcpy(b, "SHA1 ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_MD5:
+		    b = stpcpy(b, "MD5 ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_RSA:
+		    b = stpcpy(b, "RSA ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_DSA:
+		    b = stpcpy(b, "(SHA1) DSA ");
+		    /*@switchbreak@*/ break;
+		default:
+		    b = stpcpy(b, "?UnknownSignatureType? ");
+		    /*@switchbreak@*/ break;
 		}
 	    } else {
-		if (rpmIsVerbose()) {
+		if (rpmIsVerbose())
 		    b = stpcpy( stpcpy( stpcpy(b, "    "), result), "\n");
-		} else {
-		    switch ((rpmSigTag)she->tag) {
-		    case RPMSIGTAG_SIZE:
-			b = stpcpy(b, "size ");
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_SHA1:
-			b = stpcpy(b, "sha1 ");
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_MD5:
-			b = stpcpy(b, "md5 ");
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_RSA:
-			b = stpcpy(b, "rsa ");
-			/*@switchbreak@*/ break;
-		    case RPMSIGTAG_DSA:
-			b = stpcpy(b, "(sha1) dsa ");
-			/*@switchbreak@*/ break;
-		    default:
-			b = stpcpy(b, "??? ");
-			/*@switchbreak@*/ break;
-		    }
+		else
+		switch ((rpmSigTag)she->tag) {
+		case RPMSIGTAG_SIZE:
+		    b = stpcpy(b, "size ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_SHA1:
+		    b = stpcpy(b, "sha1 ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_MD5:
+		    b = stpcpy(b, "md5 ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_RSA:
+		    b = stpcpy(b, "rsa ");
+		    /*@switchbreak@*/ break;
+		case RPMSIGTAG_DSA:
+		    b = stpcpy(b, "(sha1) dsa ");
+		    /*@switchbreak@*/ break;
+		default:
+		    b = stpcpy(b, "??? ");
+		    /*@switchbreak@*/ break;
 		}
 	    }
 	}
@@ -1070,14 +1099,14 @@ assert(she->p.ptr != NULL);
 	xx = pgpSetSig(dig, 0, 0, NULL, 0);
 /*@=noeffect@*/
 
-	res += res2;
+	res += failed;
 
-	if (res2) {
+	if (failed) {
 	    if (rpmIsVerbose()) {
 		rpmlog(RPMLOG_NOTICE, "%s", buf);
 	    } else {
 		rpmlog(RPMLOG_NOTICE, "%s%s%s%s%s%s%s%s\n", buf,
-			_("NOT OK"),
+			_("NOT_OK"),
 			(missingKeys[0] != '\0') ? _(" (MISSING KEYS:") : "",
 			missingKeys,
 			(missingKeys[0] != '\0') ? _(") ") : "",
@@ -1105,11 +1134,14 @@ assert(she->p.ptr != NULL);
 
 exit:
     rpmtsCleanDig(ts);
-    sigh = headerFree(sigh);
+    (void)headerFree(sigh);
+    sigh = NULL;
     return res;
 }
 
 int rpmcliSign(rpmts ts, QVA_t qva, const char ** argv)
+	/*@globals rpmioFtsOpts @*/
+	/*@modifies rpmioFtsOpts @*/
 {
     int res = 0;
 
@@ -1140,11 +1172,9 @@ int rpmcliSign(rpmts ts, QVA_t qva, const char ** argv)
     rpmgiFlags _giFlags = RPMGI_NONE;
     rpmRC rc;
 
-/*@-mods@*/
-    if (ftsOpts == 0)
-	ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
-/*@=mods@*/
-    rc = rpmgiSetArgs(gi, argv, ftsOpts, (_giFlags|RPMGI_NOHEADER));
+    if (rpmioFtsOpts == 0)
+	rpmioFtsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+    rc = rpmgiSetArgs(gi, argv, rpmioFtsOpts, (_giFlags|RPMGI_NOHEADER));
     while (rpmgiNext(gi) == RPMRC_OK) {
 	const char * fn = rpmgiHdrPath(gi);
 	FD_t fd;
