@@ -5,15 +5,23 @@
  * \file lib/rpmcli.h
  */
 
-#include "popt.h"
-#include "argv.h"
-#include "rpmcb.h"
-#include "rpmmacro.h"
-#include "rpmurl.h"
-#include "rpmlib.h"
-#include "rpmps.h"
-#include "rpmte.h"
-#include "rpmts.h"
+#include <popt.h>
+#include <rpmmacro.h>
+#include <rpmtypes.h>
+#include <rpmtag.h>
+#include <rpmps.h>
+#include <rpmrc.h>
+#include <rpmfi.h>	/* XXX rpmfileAttrs */
+#include <rpmts.h>	/* XXX rpmdepFlags */
+
+/**
+ * Table of query format extensions.
+ * @note Chains *headerCompoundFormats -> *headerDefaultFormats.
+ */
+/*@-redecl@*/
+/*@unchecked@*/
+extern headerSprintfExtension rpmHeaderFormats;
+/*@=redecl@*/
 
 /** \ingroup rpmcli
  * Should version 3 packages be produced?
@@ -33,24 +41,12 @@ extern "C" {
 /*@unchecked@*/
 extern struct poptOption		rpmcliAllPoptTable[];
 
-/*@unchecked@*/
-extern int ftsOpts;
-
-/*@unchecked@*/
-extern struct poptOption		rpmcliFtsPoptTable[];
-
 /*@unchecked@*/
 extern int global_depFlags;
 
 /*@unchecked@*/
 extern struct poptOption		rpmcliDepFlagsPoptTable[];
 
-/*@unchecked@*/ /*@observer@*/ /*@null@*/
-extern const char * rpmcliPipeOutput;
-
-/*@unchecked@*/ /*@observer@*/ /*@null@*/
-extern const char * rpmcliRootDir;
-
 /*@unchecked@*/ /*@observer@*/ /*@null@*/
 extern const char * rpmcliTargets;
 /*@=redecl@*/
@@ -118,10 +114,10 @@ typedef enum rpmQVSources_e {
     RPMQV_PATH,		/*!< ... from file path db search. */
     RPMQV_ALL,		/*!< ... from each installed package. */
     RPMQV_RPM, 		/*!< ... from reading binary rpm package. */
-    RPMQV_GROUP,	/*!< ... from group db search. */
+    RPMQV_GROUP		= RPMTAG_GROUP,
     RPMQV_WHATPROVIDES,	/*!< ... from provides db search. */
     RPMQV_WHATREQUIRES,	/*!< ... from requires db search. */
-    RPMQV_TRIGGEREDBY,	/*!< ... from trigger db search. */
+    RPMQV_TRIGGEREDBY	= RPMTAG_TRIGGERNAME,
     RPMQV_DBOFFSET,	/*!< ... from database header instance. */
     RPMQV_SPECFILE,	/*!< ... from spec file parse (query only). */
     RPMQV_PKGID,	/*!< ... from package id (header+payload MD5). */
@@ -132,7 +128,9 @@ typedef enum rpmQVSources_e {
     RPMQV_FTSWALK,	/*!< ... from fts(3) walk. */
     RPMQV_WHATNEEDS,	/*!< ... from requires using contained provides. */
     RPMQV_SPECSRPM,	/*!< ... srpm from spec file parse (query only). */
-    RPMQV_SOURCEPKGID	/*!< ... from source package id (header+payload MD5). */
+    RPMQV_SOURCEPKGID,	/*!< ... from source package id (header+payload MD5). */
+    RPMQV_WHATCONFLICTS	= RPMTAG_CONFLICTNAME,
+    RPMQV_WHATOBSOLETES	= RPMTAG_OBSOLETENAME
 } rpmQVSources;
 
 /** \ingroup rpmcli
@@ -321,7 +319,7 @@ int rpmcliShowMatches(QVA_t qva, rpmts ts)
  * @param qva		parsed query/verify options
  * @param ts		transaction set
  * @param arg		name of source to query/verify
- * @return		showPackage() result, 1 if rpmdbInitIterator() is NULL
+ * @return		showPackage() result, 1 if rpmmiInit() is NULL
  */
 int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 	/*@globals rpmGlobalMacroContext, h_errno,
@@ -348,10 +346,10 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
  * @param argv		query argument(s) (or NULL)
  * @return		0 on success, else no. of failures
  */
-int rpmcliArgIter(rpmts ts, QVA_t qva, /*@null@*/ ARGV_t argv)
-	/*@globals ftsOpts, rpmGlobalMacroContext, h_errno,
+int rpmcliArgIter(rpmts ts, QVA_t qva, /*@null@*/ const char ** argv)
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
-	/*@modifies ts, qva, ftsOpts, rpmGlobalMacroContext,
+	/*@modifies ts, qva, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
 /** \ingroup rpmcli
@@ -368,23 +366,6 @@ int rpmcliQuery(rpmts ts, QVA_t qva, /*@null@*/ const char ** argv)
 	/*@modifies ts, qva, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
-/** \ingroup rpmcli
- * Verify file attributes (including file digest).
- * @todo gnorpm and python bindings prevent this from being static.
- * @param ts		transaction set
- * @param fi		file info (with linked header and current file index)
- * @retval *res		bit(s) returned to indicate failure
- * @param omitMask	bit(s) to disable verify checks
- * @return		0 on success (or not installed), 1 on error
- */
-/*@-incondefs@*/
-int rpmVerifyFile(const rpmts ts, rpmfi fi,
-		/*@out@*/ rpmVerifyAttrs * res, rpmVerifyAttrs omitMask)
-	/*@globals h_errno, fileSystem, internalState @*/
-	/*@modifies ts, fi, *res, fileSystem, internalState @*/
-	/*@requires maxSet(res) >= 0 @*/;
-/*@=incondefs@*/
-
 /** \ingroup rpmcli
  * Display results of package verify.
  * @param qva		parsed query/verify options
@@ -400,14 +381,13 @@ int showVerifyPackage(QVA_t qva, rpmts ts, Header h)
  * Check package and header signatures.
  * @param qva		parsed query/verify options
  * @param ts		transaction set
- * @param fd		package file handle
+ * @param _fd		package file handle
  * @param fn		package file name
  * @return		0 on success, 1 on failure
  */
-int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd, const char * fn)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies qva, ts, fd, rpmGlobalMacroContext, h_errno,
-		fileSystem, internalState @*/;
+int rpmVerifySignatures(QVA_t qva, rpmts ts, void * _fd, const char * fn)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies qva, ts, fileSystem, internalState @*/;
 
 /** \ingroup rpmcli
  * Verify package install.
@@ -455,9 +435,9 @@ extern int rpmcliHashesCurrent;
 /*@unchecked@*/
 extern int rpmcliHashesTotal;
 /*@unchecked@*/
-extern uint64_t rpmcliProgressCurrent;
+extern rpmuint64_t rpmcliProgressCurrent;
 /*@unchecked@*/
-extern uint64_t rpmcliProgressTotal;
+extern rpmuint64_t rpmcliProgressTotal;
 
 /** \ingroup rpmcli
  * The rpm CLI generic transaction callback handler.
@@ -478,16 +458,16 @@ extern uint64_t rpmcliProgressTotal;
 /*@null@*/
 void * rpmShowProgress(/*@null@*/ const void * arg,
 		const rpmCallbackType what,
-		const uint64_t amount,
-		const uint64_t total,
+		const rpmuint64_t amount,
+		const rpmuint64_t total,
 		/*@null@*/ fnpyKey key,
 		/*@null@*/ void * data)
 	/*@globals rpmcliHashesCurrent,
 		rpmcliProgressCurrent, rpmcliProgressTotal,
-		h_errno, fileSystem, internalState @*/
-	/*@modifies rpmcliHashesCurrent,
+		h_errno, rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@modifies arg, rpmcliHashesCurrent,
 		rpmcliProgressCurrent, rpmcliProgressTotal,
-		fileSystem, internalState @*/;
+		rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmcli
  * Install source rpm package.
@@ -513,8 +493,8 @@ int rpmInstallSource(rpmts ts, const char * arg,
  * @return		no. of (added) packages
  */
 int rpmcliInstallProblems(rpmts ts, /*@null@*/ const char * msg, int rc)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies ts, fileSystem, internalState @*/;
 
 /** \ingroup rpmcli
  * Report packages(if any) that satisfy unresolved dependencies.
@@ -522,8 +502,8 @@ int rpmcliInstallProblems(rpmts ts, /*@null@*/ const char * msg, int rc)
  * @return		0 always
  */
 int rpmcliInstallSuggests(rpmts ts)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@globals internalState @*/
+	/*@modifies ts, internalState @*/;
 
 /** \ingroup rpmcli
  * Check package element dependencies in a transaction set, reporting problems.
@@ -580,114 +560,6 @@ int rpmErase(rpmts ts, QVA_t ia, /*@null@*/ const char ** argv)
 	/*@modifies ts, ia, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
-/**
- * A rollback transaction id element.
- */
-/*@-fielduse@*/
-typedef /*@abstract@*/ struct IDT_s * IDT;
-#if !defined(SWIG)
-struct IDT_s {
-    int done;			/*!< package processed? */
-    unsigned int instance;	/*!< installed package transaction id. */
-/*@owned@*/ /*@null@*/
-    const char * key;		/*! removed package file name. */
-    Header h;			/*!< removed package header. */
-    union {
-	uint32_t u32;		/*!< install/remove transaction id */
-    } val;
-};
-#endif
-/*@=fielduse@*/
-
-/**
- * A rollback transaction id index.
- */
-typedef /*@abstract@*/ struct IDTindex_s * IDTX;
-#if !defined(SWIG)
-struct IDTindex_s {
-    int delta;			/*!< no. elements to realloc as a chunk. */
-    int size;			/*!< size of id index element. */
-    int alloced;		/*!< current number of elements allocated. */
-    int nidt;			/*!< current number of elements initialized. */
-/*@only@*/ /*@null@*/
-    IDT idt;			/*!< id index elements. */
-};
-#endif
-
-/**
- * Destroy id index.
- * @param idtx		id index
- * @return		NULL always
- */
-/*@null@*/
-IDTX IDTXfree(/*@only@*/ /*@null@*/ IDTX idtx)
-	/*@modifies idtx @*/;
-
-/**
- * Create id index.
- * @return		new id index
- */
-/*@only@*/
-IDTX IDTXnew(void)
-	/*@*/;
-
-/**
- * Insure that index has room for "need" elements.
- * @param idtx		id index
- * @param need		additional no. of elements needed
- * @return 		id index (with room for "need" elements)
- */
-/*@only@*/ /*@null@*/
-IDTX IDTXgrow(/*@only@*/ /*@null@*/ IDTX idtx, int need)
-	/*@modifies idtx @*/;
-
-/**
- * Sort tag (instance,value) pairs.
- * @param idtx		id index
- * @return 		id index
- */
-/*@only@*/ /*@null@*/
-IDTX IDTXsort(/*@only@*/ /*@null@*/ IDTX idtx)
-	/*@modifies idtx @*/;
-
-/**
- * Load tag (instance,value) pairs from rpm databse, and return sorted id index.
- * @param ts		transaction set
- * @param tag		rpm tag
- * @param rbtid		rollback goal
- * @return 		id index
- */
-/*@only@*/ /*@null@*/
-IDTX IDTXload(rpmts ts, rpmTag tag, uint32_t rbtid)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState  @*/;
-
-/**
- * Load tag (instance,value) pairs from packages, and return sorted id index.
- * @param ts		transaction set
- * @param globstr	glob expression
- * @param tag		rpm tag
- * @param rbtid		rollback goal
- * @return 		id index
- */
-/*@only@*/ /*@null@*/
-IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag, uint32_t rbtid)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
-
-/** \ingroup rpmcli
- * Rollback transactions, erasing new, reinstalling old, package(s).
- * @param ts		transaction set
- * @param ia		mode flags and parameters
- * @param argv		array of arguments (NULL terminated)
- * @return		0 on success
- */
-int rpmRollback(rpmts ts, QVA_t ia, /*@null@*/ const char ** argv)
-	/*@globals rpmcliPackagesTotal, rpmGlobalMacroContext, h_errno,
-		fileSystem, internalState @*/
-	/*@modifies ts, ia, rpmcliPackagesTotal, rpmGlobalMacroContext,
-		fileSystem, internalState @*/;
-
 /** \ingroup rpmcli
  */
 /*@unchecked@*/
@@ -718,7 +590,7 @@ extern struct poptOption rpmDatabasePoptTable[];
  */
 rpmRC rpmcliImportPubkey(const rpmts ts,
 		const unsigned char * pkt, ssize_t pktlen)
-	/*@globals RPMVERSION, rpmGlobalMacroContext, h_errno,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
@@ -752,7 +624,7 @@ extern struct poptOption rpmSignPoptTable[];
  * @return		0 on success
  */
 int rpmcliSign(rpmts ts, QVA_t qva, /*@null@*/ const char ** argv)
-	/*@globals RPMVERSION, rpmGlobalMacroContext, h_errno,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies ts, qva, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
@@ -769,7 +641,7 @@ struct rpmQVKArguments_s {
     rpmQueryFlags qva_flags;	/*!< Bit(s) to control operation. */
     rpmfileAttrs qva_fflags;	/*!< Bit(s) to filter on attribute. */
 /*@only@*/ /*@null@*/
-    rpmdbMatchIterator qva_mi;	/*!< Match iterator on selected headers. */
+    rpmmi qva_mi;		/*!< Match iterator on selected headers. */
 /*@refccounted@*/ /*@relnull@*/
     rpmgi qva_gi;		/*!< Generalized iterator on args. */
     rpmRC qva_rc;		/*!< Current return code. */
@@ -785,6 +657,7 @@ struct rpmQVKArguments_s {
 /*@only@*/ /*@null@*/
     const char * qva_queryFormat;/*!< Format for headerSprintf(). */
     int sign;			/*!< Is a passphrase needed? */
+    int nopassword;
     int trust;			/*!< Trust metric when importing pubkeys. */
 /*@observer@*/
     const char * passPhrase;	/*!< Pass phrase. */
@@ -807,16 +680,16 @@ struct rpmQVKArguments_s {
     rpmtransFlags transFlags;
     rpmprobFilterFlags probFilter;
     rpmInstallInterfaceFlags installInterfaceFlags;
-    uint32_t arbtid;		/*!< from --arbgoal */
-    uint32_t rbtid;		/*!< from --rollback */
-    uint32_t *rbtidExcludes;	/*!< from --rollback */
+    rpmuint32_t arbtid;		/*!< from --arbgoal */
+    rpmuint32_t rbtid;		/*!< from --rollback */
+    rpmuint32_t *rbtidExcludes;	/*!< from --rollback */
     int numrbtidExcludes;	/*!< from --rollback */
-    int numRelocations;
     int noDeps;
     int incldocs;
     int no_rollback_links;
-/*@owned@*/ /*@null@*/
+/*@owned@*/ /*@relnull@*/
     rpmRelocation relocations;
+    int nrelocations;
 
     /* database mode arguments */
     int init;			/*!< from --initdb */
@@ -864,6 +737,7 @@ struct rpmBuildArguments_s {
     int noLang;			/*!< from --nolang */
     int shortCircuit;		/*!< from --short-circuit */
     int sign;			/*!< from --sign */
+    int nopassword;
     int useCatalog;		/*!< from --usecatalog */
     char buildMode;		/*!< Build mode (one of "btBC") */
     char buildChar;		/*!< Build stage (one of "abcilps ") */
