@@ -20,7 +20,7 @@ extern int _rpmds_debug;
 
 /** \ingroup rpmds
  */
-/*@unchecked@*/ /*@observer@*/ /*@owned@*/ /*@relnull@*/
+/*@unchecked@*/ /*@observer@*/ /*@owned@*/ /*@null@*/
 extern const char *_sysinfo_path;
 
 /** \ingroup rpmds
@@ -31,10 +31,13 @@ extern int _rpmds_nopromote;
 /*@=exportlocal@*/
 
 #if defined(_RPMDS_INTERNAL)
+#include "mire.h"
+
 /** \ingroup rpmds
  * A dependency set.
  */
 struct rpmds_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
 /*@observer@*/
     const char * Type;		/*!< Tag name. */
 /*@only@*/ /*@null@*/
@@ -48,26 +51,34 @@ struct rpmds_s {
 /*@only@*/ /*@relnull@*/
     evrFlags * Flags;		/*!< Bit(s) identifying context/comparison. */
 /*@only@*/ /*@null@*/
-    uint32_t * Color;		/*!< Bit(s) calculated from file color(s). */
+    rpmuint32_t * Color;	/*!< Bit(s) calculated from file color(s). */
 /*@only@*/ /*@null@*/
-    uint32_t * Refs;		/*!< No. of file refs. */
+    rpmuint32_t * Refs;		/*!< No. of file refs. */
 /*@only@*/ /*@null@*/
-    int32_t * Result;		/*!< Dependency check result. */
+    rpmint32_t * Result;	/*!< Dependency check result. */
 /*@null@*/
     int (*EVRparse) (const char *evrstr, EVR_t evr);	 /* EVR parsing. */
     int (*EVRcmp) (const char *a, const char *b);	 /* EVR comparison. */
     struct rpmns_s ns;		/*!< Name (split). */
+/*@only@*/ /*@null@*/
+    miRE exclude;		/*!< Iterator exclude patterns. */
+    int nexclude;		/*!< No. of exclude patterns. */
+/*@only@*/ /*@null@*/
+    miRE include;		/*!< Iterator include patterns. */
+    int ninclude;		/*!< No. of include patterns. */
 /*@only@*/ /*@null@*/
     const char * A;		/*!< Arch (from containing package). */
-    uint32_t BT;		/*!< Package build time tie breaker. */
+    rpmuint32_t BT;		/*!< Package build time tie breaker. */
     rpmTag tagN;		/*!< Header tag. */
-    uint32_t Count;		/*!< No. of elements */
+    rpmuint32_t Count;		/*!< No. of elements */
     int i;			/*!< Element index. */
     unsigned l;			/*!< Low element (bsearch). */
     unsigned u;			/*!< High element (bsearch). */
     int nopromote;		/*!< Don't promote Epoch: in rpmdsCompare()? */
+#if defined(__LCLINT__)
 /*@refs@*/
-    int nrefs;			/*!< Reference count. */
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 #endif	/* _RPMDS_INTERNAL */
 
@@ -76,6 +87,7 @@ struct rpmds_s {
  * Container for provides/requires/conflicts/obsoletes dependency set(s).
  */
 struct rpmPRCO_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
 /*@dependent@*/ /*@relnull@*/
     rpmds * Pdsp;		/*!< Provides: collector. */
 /*@dependent@*/ /*@relnull@*/
@@ -106,6 +118,10 @@ struct rpmPRCO_s {
     rpmds D;		/*!< Dirnames */
 /*@refcounted@*/ /*@null@*/
     rpmds L;		/*!< Linktos */
+#if defined(__LCLINT__)
+/*@refs@*/
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 #endif	/* _RPMPRCO_INTERNAL */
 
@@ -115,25 +131,19 @@ extern "C" {
 
 /** \name RPMDS */
 /*@{*/
+
 /** \ingroup rpmds
  * Unreference a dependency set instance.
  * @param ds		dependency set
  * @param msg
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@unused@*/ /*@null@*/
 rpmds rpmdsUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmds ds,
 		/*@null@*/ const char * msg)
 	/*@modifies ds @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@-exportlocal@*/
-/*@null@*/
-rpmds XrpmdsUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmds ds,
-		/*@null@*/ const char * msg, const char * fn, unsigned ln)
-	/*@modifies ds @*/;
-/*@=exportlocal@*/
-#define	rpmdsUnlink(_ds, _msg)	XrpmdsUnlink(_ds, _msg, __FILE__, __LINE__)
+#define	rpmdsUnlink(_ds, _msg)	\
+    ((rpmds)rpmioUnlinkPoolItem((rpmioItem)(_ds), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmds
  * Reference a dependency set instance.
@@ -144,22 +154,20 @@ rpmds XrpmdsUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmds ds,
 /*@unused@*/ /*@newref@*/ /*@null@*/
 rpmds rpmdsLink (/*@null@*/ rpmds ds, /*@null@*/ const char * msg)
 	/*@modifies ds @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@newref@*/ /*@null@*/
-rpmds XrpmdsLink (/*@null@*/ rpmds ds, /*@null@*/ const char * msg,
-		const char * fn, unsigned ln)
-        /*@modifies ds @*/;
-#define	rpmdsLink(_ds, _msg)	XrpmdsLink(_ds, _msg, __FILE__, __LINE__)
+#define	rpmdsLink(_ds, _msg)	\
+    ((rpmds)rpmioLinkPoolItem((rpmioItem)(_ds), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmds
  * Destroy a dependency set.
  * @param ds		dependency set
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@null@*/
-rpmds rpmdsFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmds ds)
+rpmds rpmdsFree(/*@killref@*/ /*@null@*/ rpmds ds)
 	/*@modifies ds @*/;
+#define	rpmdsFree(_ds)	\
+    ((rpmds)rpmioFreePoolItem((rpmioItem)(_ds), __FUNCTION__, __FILE__, __LINE__))
+
 /** \ingroup rpmds
  * Create and load a dependency set.
  * @param h		header
@@ -201,7 +209,8 @@ char * rpmdsNewDNEVR(const char * dspfx, rpmds ds)
  */
 /*@null@*/
 rpmds rpmdsThis(Header h, rpmTag tagN, evrFlags Flags)
-	/*@*/;
+	/*@globals internalState @*/
+	/*@modifies internalState @*/;
 
 /** \ingroup rpmds
  * Create, load and initialize a dependency set of size 1.
@@ -370,7 +379,7 @@ void * rpmdsSetEVRcmp(/*@null@*/ rpmds ds,
  * @param ds		dependency set
  * @return		current dependency color (0 if not set)
  */
-uint32_t rpmdsColor(/*@null@*/ const rpmds ds)
+rpmuint32_t rpmdsColor(/*@null@*/ const rpmds ds)
 	/*@*/;
 
 /** \ingroup rpmds
@@ -379,15 +388,49 @@ uint32_t rpmdsColor(/*@null@*/ const rpmds ds)
  * @param color		new dependency color
  * @return		previous dependency color
  */
-uint32_t rpmdsSetColor(/*@null@*/ const rpmds ds, uint32_t color)
+rpmuint32_t rpmdsSetColor(/*@null@*/ const rpmds ds, rpmuint32_t color)
 	/*@modifies ds @*/;
 
+/** \ingroup rpmds
+ * Return dependency exclude patterns.
+ * @param ds		dependency set
+ * @return		dependency exclude patterns (NULL if not set)
+ */
+/*@null@*/
+void * rpmdsExclude(/*@null@*/ const rpmds ds)
+	/*@*/;
+
+/** \ingroup rpmds
+ * Return no. of dependency exclude patterns.
+ * @param ds		dependency set
+ * @return		dependency exclude patterns (0 if not set)
+ */
+int rpmdsNExclude(/*@null@*/ const rpmds ds)
+	/*@*/;
+
+/** \ingroup rpmds
+ * Return dependency include patterns.
+ * @param ds		dependency set
+ * @return		dependency include patterns (NULL if not set)
+ */
+/*@null@*/
+void * rpmdsInclude(/*@null@*/ const rpmds ds)
+	/*@*/;
+
+/** \ingroup rpmds
+ * Return no. of dependency include patterns.
+ * @param ds		dependency set
+ * @return		dependency include patterns (0 if not set)
+ */
+int rpmdsNInclude(/*@null@*/ const rpmds ds)
+	/*@*/;
+
 /** \ingroup rpmds
  * Return current dependency file refs.
  * @param ds		dependency set
  * @return		current dependency file refs (0 if not set)
  */
-uint32_t rpmdsRefs(/*@null@*/ const rpmds ds)
+rpmuint32_t rpmdsRefs(/*@null@*/ const rpmds ds)
 	/*@*/;
 
 /** \ingroup rpmds
@@ -396,7 +439,7 @@ uint32_t rpmdsRefs(/*@null@*/ const rpmds ds)
  * @param refs		new dependency refs
  * @return		previous dependency refs
  */
-uint32_t rpmdsSetRefs(/*@null@*/ const rpmds ds, uint32_t refs)
+rpmuint32_t rpmdsSetRefs(/*@null@*/ const rpmds ds, rpmuint32_t refs)
 	/*@modifies ds @*/;
 
 /** \ingroup rpmds
@@ -404,7 +447,7 @@ uint32_t rpmdsSetRefs(/*@null@*/ const rpmds ds, uint32_t refs)
  * @param ds		dependency set
  * @return		current dependency result (0 if not set)
  */
-int32_t rpmdsResult(/*@null@*/ const rpmds ds)
+rpmint32_t rpmdsResult(/*@null@*/ const rpmds ds)
 	/*@*/;
 
 /** \ingroup rpmds
@@ -413,7 +456,7 @@ int32_t rpmdsResult(/*@null@*/ const rpmds ds)
  * @param result	new dependency result
  * @return		previous dependency result
  */
-int32_t rpmdsSetResult(/*@null@*/ const rpmds ds, int32_t result)
+rpmint32_t rpmdsSetResult(/*@null@*/ const rpmds ds, rpmint32_t result)
 	/*@modifies ds @*/;
 
 /** \ingroup rpmds
@@ -474,6 +517,11 @@ int rpmdsMerge(/*@null@*/ /*@out@*/ rpmds * dsp, /*@null@*/ rpmds ods)
 int rpmdsSearch(/*@null@*/ rpmds ds, /*@null@*/ rpmds ods)
 	/*@modifies ds, ods @*/;
 
+/**
+ */
+/*@unchecked@*/ /*@null@*/
+extern const char * _cpuinfo_path;
+
 /** \ingroup rpmds
  * Load /proc/cpuinfo provides into a dependency set.
  * @retval *dsp		(loaded) depedency set
@@ -637,6 +685,15 @@ int rpmdsPipe(rpmds * dsp, rpmTag tagN, /*@null@*/ const char * cmd)
 int rpmdsCompare(const rpmds A, const rpmds B)
 	/*@*/;
 
+/** \ingroup rpmds
+ * Compare A against every member of B, looking for 1st match.
+ * @param A		1st dependency
+ * @param B		2nd dependency
+ * @return		1 if some dependency overlaps, 0 otherwise
+ */
+int rpmdsMatch(const rpmds A, rpmds B)
+	/*@modifies B @*/;
+
 /** \ingroup rpmds
  * Report a Requires: or Conflicts: dependency problem.
  * @param ps		transaction set problems
@@ -670,7 +727,8 @@ int rpmdsAnyMatchesDep (const Header h, const rpmds req, int nopromote)
  * @return		1 if dependency overlaps, 0 otherwise
  */
 int rpmdsNVRMatchesDep(const Header h, const rpmds req, int nopromote)
-	/*@*/;
+	/*@globals internalState @*/
+	/*@modifies internalState @*/;
 
 /** \ingroup rpmds
  * Negate return code for negated comparisons.
@@ -681,29 +739,16 @@ int rpmdsNVRMatchesDep(const Header h, const rpmds req, int nopromote)
 int rpmdsNegateRC(const rpmds ds, int rc)
 	/*@*/;
 
-#if !defined(SWIG)
 /** \ingroup rpmds
  * Return current dependency type name.
  * @param ds		dependency set
  * @return		current dependency type name
  */
-/*@unused@*/ static inline /*@observer@*/
-const char * rpmdsTagName(/*@null@*/ const rpmds ds)
-	/*@*/
-{
-    rpmTag tagN = rpmdsTagN(ds);
-
-    switch (tagN) {
-    case RPMTAG_PROVIDENAME:	return "Provides";	/*@notreached@*/ break;
-    case RPMTAG_REQUIRENAME:	return "Requires";	/*@notreached@*/ break;
-    case RPMTAG_CONFLICTNAME:	return "Conflicts";	/*@notreached@*/ break;
-    case RPMTAG_OBSOLETENAME:	return "Obsoletes";	/*@notreached@*/ break;
-    case RPMTAG_TRIGGERNAME:	return "Triggers";	/*@notreached@*/ break;
-    case RPMTAG_DIRNAMES:	return "Dirnames";	/*@notreached@*/ break;
-    }
-    return tagName(tagN);
-}
+/*@observer@*/
+const char * rpmdsType(/*@null@*/ const rpmds ds)
+	/*@*/;
 
+#if !defined(SWIG)
 /** \ingroup rpmds
  * Print current dependency set contents.
  * @param ds		dependency set
@@ -719,7 +764,7 @@ int rpmdsPrint(/*@null@*/ rpmds ds, /*@null@*/ FILE * fp)
 	fp = stderr;
     ds = rpmdsInit(ds);
     while (rpmdsNext(ds) >= 0)
-        fprintf(fp, "%6d\t%s: %s\n", rpmdsIx(ds), rpmdsTagName(ds), rpmdsDNEVR(ds)+2);
+        fprintf(fp, "%6d\t%s: %s\n", rpmdsIx(ds), rpmdsType(ds), rpmdsDNEVR(ds)+2);
     return 0;
 }
 
@@ -738,10 +783,10 @@ int rpmdsPrintResults(/*@null@*/ rpmds ds, /*@null@*/ FILE * fp)
 	fp = stderr;
     ds = rpmdsInit(ds);
     while (rpmdsNext(ds) >= 0) {
-	int rc = rpmdsResult(ds);
+	rpmint32_t rc = rpmdsResult(ds);
 	if (rc > 0)
 	    continue;
-        fprintf(fp, "%6d\t%s: %s\n", rpmdsIx(ds), rpmdsTagName(ds), rpmdsDNEVR(ds)+2);
+        fprintf(fp, "%6d\t%s: %s\n", rpmdsIx(ds), rpmdsType(ds), rpmdsDNEVR(ds)+2);
     }
     return 0;
 }
