@@ -3,19 +3,24 @@
 #include <signal.h>	/* getOutputFrom() */
 
 #include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
+#include <rpmiotypes.h>		/* XXX fnpyKey */
+#include <rpmlog.h>
+#include <rpmurl.h>
+#include <rpmmg.h>
+#include <argv.h>
+#define	_MIRE_INTERNAL
+#include <mire.h>
+
 #include <rpmtag.h>
 #define	_RPMEVR_INTERNAL
 #include <rpmbuild.h>
-#include <argv.h>
-#include <rpmmg.h>
-
-#define	_RPMFC_INTERNAL
-#include <rpmfc.h>
 
 #define	_RPMNS_INTERNAL
 #include <rpmns.h>
 
+#define	_RPMFC_INTERNAL
+#include <rpmfc.h>
+
 #define	_RPMDS_INTERNAL
 #include <rpmds.h>
 #include <rpmfi.h>
@@ -23,6 +28,12 @@
 #include "debug.h"
 
 /*@access rpmds @*/
+/*@access miRE @*/
+
+/*@unchecked@*/
+static int _filter_values = 1;
+/*@unchecked@*/
+static int _filter_execs = 1;
 
 /**
  */
@@ -44,7 +55,7 @@ static int rpmfcExpandAppend(/*@out@*/ ARGV_t * argvp, const ARGV_t av)
     return 0;
 }
 
-/** \ingroup rpmbuild
+/**
  * Return output from helper script.
  * @todo Use poll(2) rather than select(2), if available.
  * @param dir		directory to run in (or NULL)
@@ -55,18 +66,18 @@ static int rpmfcExpandAppend(/*@out@*/ ARGV_t * argvp, const ARGV_t av)
  * @return		buffered stdout from script, NULL on error
  */     
 /*@null@*/
-static StringBuf getOutputFrom(/*@null@*/ const char * dir, ARGV_t argv,
-                        const char * writePtr, int writeBytesLeft,
-                        int failNonZero)
-        /*@globals h_errno, fileSystem, internalState@*/
-        /*@modifies fileSystem, internalState@*/
+static rpmiob getOutputFrom(/*@null@*/ const char * dir, ARGV_t argv,
+			const char * writePtr, size_t writeBytesLeft,
+			int failNonZero)
+	/*@globals h_errno, fileSystem, internalState@*/
+	/*@modifies fileSystem, internalState@*/
 {
     pid_t child, reaped;
     int toProg[2];
     int fromProg[2];
     int status;
     void *oldhandler;
-    StringBuf readBuff;
+    rpmiob iob = NULL;
     int done;
 
     /*@-type@*/ /* FIX: cast? */
@@ -74,9 +85,11 @@ static StringBuf getOutputFrom(/*@null@*/ const char * dir, ARGV_t argv,
     /*@=type@*/
 
     toProg[0] = toProg[1] = 0;
-    (void) pipe(toProg);
     fromProg[0] = fromProg[1] = 0;
-    (void) pipe(fromProg);
+    if (pipe(toProg) < 0 || pipe(fromProg) < 0) {
+	rpmlog(RPMLOG_ERR, _("Couldn't create pipe for %s: %m\n"), argv[0]);
+	return NULL;
+    }
     
     if (!(child = fork())) {
 	(void) close(toProg[1]);
@@ -93,7 +106,7 @@ static StringBuf getOutputFrom(/*@null@*/ const char * dir, ARGV_t argv,
 	}
 	
 	rpmlog(RPMLOG_DEBUG, D_("\texecv(%s) pid %d\n"),
-                        argv[0], (unsigned)getpid());
+			argv[0], (unsigned)getpid());
 
 	unsetenv("MALLOC_CHECK_");
 	(void) execvp(argv[0], (char *const *)argv);
@@ -115,12 +128,14 @@ static StringBuf getOutputFrom(/*@null@*/ const char * dir, ARGV_t argv,
     (void) fcntl(fromProg[0], F_SETFL, O_NONBLOCK);
     (void) fcntl(toProg[1], F_SETFL, O_NONBLOCK);
     
-    readBuff = newStringBuf();
+    iob = rpmiobNew(0);
 
     do {
 	fd_set ibits, obits;
 	struct timeval tv;
-	int nfd, nbw, nbr;
+	int nfd;
+	ssize_t nbr;
+	ssize_t nbw;
 	int rc;
 
 	done = 0;
@@ -145,9 +160,10 @@ top:
 
 	/* Write any data to program */
 	if (toProg[1] >= 0 && FD_ISSET(toProg[1], &obits)) {
-          if (writePtr && writeBytesLeft > 0) {
+	  if (writePtr && writeBytesLeft > 0) {
 	    if ((nbw = write(toProg[1], writePtr,
-		    (1024<writeBytesLeft) ? 1024 : writeBytesLeft)) < 0) {
+		    ((size_t)1024<writeBytesLeft) ? (size_t)1024 : writeBytesLeft)) < 0)
+	    {
 	        if (errno != EAGAIN) {
 		    perror("getOutputFrom()");
 	            exit(EXIT_FAILURE);
@@ -166,7 +182,7 @@ top:
 	{   char buf[BUFSIZ+1];
 	    while ((nbr = read(fromProg[0], buf, sizeof(buf)-1)) > 0) {
 		buf[nbr] = '\0';
-		appendStringBuf(readBuff, buf);
+		iob = rpmiobAppend(iob, buf, 0);
 	    }
 	}
 
@@ -180,14 +196,14 @@ top:
     	(void) close(toProg[1]);
     if (fromProg[0] >= 0)
 	(void) close(fromProg[0]);
-    /*@-type@*/ /* FIX: cast? */
+/*@-type@*/ /* FIX: cast? */
     (void) signal(SIGPIPE, oldhandler);
-    /*@=type@*/
+/*@=type@*/
 
     /* Collect status from prog */
     reaped = waitpid(child, &status, 0);
     rpmlog(RPMLOG_DEBUG, D_("\twaitpid(%d) rc %d status %x\n"),
-        (unsigned)child, (unsigned)reaped, status);
+	(unsigned)child, (unsigned)reaped, status);
 
     if (failNonZero && (!WIFEXITED(status) || WEXITSTATUS(status))) {
 	const char *cmd = argvJoin(argv);
@@ -195,16 +211,18 @@ top:
 
 	rpmlog(RPMLOG_ERR, _("Command \"%s\" failed, exit(%d)\n"), cmd, rc);
 	cmd = _free(cmd);
+	iob = rpmiobFree(iob);
 	return NULL;
     }
     if (writeBytesLeft) {
 	rpmlog(RPMLOG_ERR, _("failed to write all data to %s\n"), argv[0]);
+	iob = rpmiobFree(iob);
 	return NULL;
     }
-    return readBuff;
+    return iob;
 }
 
-int rpmfcExec(ARGV_t av, StringBuf sb_stdin, StringBuf * sb_stdoutp,
+int rpmfcExec(ARGV_t av, rpmiob iob_stdin, rpmiob * iob_stdoutp,
 		int failnonzero)
 {
     const char * s = NULL;
@@ -212,13 +230,13 @@ int rpmfcExec(ARGV_t av, StringBuf sb_stdin, StringBuf * sb_stdoutp,
     ARGV_t pav = NULL;
     int pac = 0;
     int ec = -1;
-    StringBuf sb = NULL;
+    rpmiob iob = NULL;
     const char * buf_stdin = NULL;
-    int buf_stdin_len = 0;
+    size_t buf_stdin_len = 0;
     int xx;
 
-    if (sb_stdoutp)
-	*sb_stdoutp = NULL;
+    if (iob_stdoutp)
+	*iob_stdoutp = NULL;
     if (!(av && *av))
 	goto exit;
 
@@ -239,23 +257,23 @@ int rpmfcExec(ARGV_t av, StringBuf sb_stdin, StringBuf * sb_stdoutp,
     if (av[1])
 	xx = rpmfcExpandAppend(&xav, av + 1);
 
-    if (sb_stdin != NULL) {
-	buf_stdin = getStringBuf(sb_stdin);
-	buf_stdin_len = strlen(buf_stdin);
+    if (iob_stdin != NULL) {
+	buf_stdin = rpmiobStr(iob_stdin);
+	buf_stdin_len = rpmiobLen(iob_stdin);
     }
 
     /* Read output from exec'd helper. */
-    sb = getOutputFrom(NULL, xav, buf_stdin, buf_stdin_len, failnonzero);
+    iob = getOutputFrom(NULL, xav, buf_stdin, buf_stdin_len, failnonzero);
 
-    if (sb_stdoutp != NULL) {
-	*sb_stdoutp = sb;
-	sb = NULL;	/* XXX don't free */
+    if (iob_stdoutp != NULL) {
+	*iob_stdoutp = iob;
+	iob = NULL;	/* XXX don't free */
     }
 
     ec = 0;
 
 exit:
-    sb = freeStringBuf(sb);
+    iob = rpmiobFree(iob);
     xav = argvFree(xav);
     pav = _free(pav);	/* XXX popt mallocs in single blob. */
     s = _free(s);
@@ -279,18 +297,20 @@ static int rpmfcSaveArg(/*@out@*/ ARGV_t * argvp, const char * key)
 
 /**
  */
-static char * rpmfcFileDep(/*@returned@*/ char * buf, int ix,
+static char * rpmfcFileDep(/*@returned@*/ char * buf, size_t ix,
 		/*@null@*/ rpmds ds)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies buf, fileSystem, internalState @*/
+	/*@globals internalState @*/
+	/*@modifies buf, internalState @*/
 	/*@requires maxSet(buf) >= 0 @*/
-	/*@ensures maxRead(buf) == 0 @*/
 {
-    uint32_t tagN = rpmdsTagN(ds);
+    rpmTag tagN = rpmdsTagN(ds);
     char deptype = 'X';
 
     buf[0] = '\0';
     switch (tagN) {
+    default:
+assert(0);
+	/*@notreached@*/ break;
     case RPMTAG_PROVIDENAME:
 	deptype = 'P';
 	break;
@@ -300,12 +320,82 @@ static char * rpmfcFileDep(/*@returned@*/ char * buf, int ix,
     }
 /*@-nullpass@*/
     if (ds != NULL)
-	sprintf(buf, "%08d%c %s %s 0x%08x", ix, deptype,
+	sprintf(buf, "%08u%c %s %s 0x%08x", (unsigned)ix, deptype,
 		rpmdsN(ds), rpmdsEVR(ds), rpmdsFlags(ds));
 /*@=nullpass@*/
     return buf;
 };
 
+/*@null@*/
+static void * rpmfcExpandRegexps(const char * str, int * nmirep)
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies *nmirep, rpmGlobalMacroContext, internalState @*/
+{
+    ARGV_t av = NULL;
+    int ac = 0;
+    miRE mire = NULL;
+    int nmire = 0;
+    const char * s;
+    int xx;
+    int i;
+
+    s = rpmExpand(str, NULL);
+    if (s) {
+    	xx = poptParseArgvString(s, &ac, (const char ***)&av);
+	s = _free(s);
+    }
+    if (ac == 0 || av == NULL || *av == NULL)
+	goto exit;
+
+    for (i = 0; i < ac; i++) {
+	xx = mireAppend(RPMMIRE_REGEX, 0, av[i], NULL, &mire, &nmire);
+	/* XXX add REG_NOSUB? better error msg?  */
+	if (xx) {
+	    rpmlog(RPMLOG_NOTICE, 
+			_("Compilation of pattern '%s'"
+		        " (expanded from '%s') failed. Skipping ...\n"),
+			av[i], str);
+	    nmire--;	/* XXX does this actually skip?!? */
+	}
+    }
+    if (nmire == 0)
+	mire = mireFree(mire);
+
+exit:
+    av = _free(av);
+    if (nmirep)
+	*nmirep = nmire;
+    return mire;
+}
+
+static int rpmfcMatchRegexps(void * mires, int nmire,
+		const char * str, char deptype)
+	/*@modifies mires @*/
+{
+    miRE mire = mires;
+    int xx;
+    int i;
+
+    for (i = 0; i < nmire; i++) {
+	rpmlog(RPMLOG_DEBUG, D_("Checking %c: '%s'\n"), deptype, str);
+	if ((xx = mireRegexec(mire + i, str, 0)) < 0)
+	    continue;
+	rpmlog(RPMLOG_NOTICE, _("Skipping %c: '%s'\n"), deptype, str);
+	return 1;
+    }
+    return 0;
+}
+
+/*@null@*/
+static void * rpmfcFreeRegexps(/*@only@*/ void * mires, int nmire)
+	/*@modifies mires @*/
+{
+    miRE mire = mires;
+/*@-refcounttrans@*/
+    return mireFreeAll(mire, nmire);
+/*@=refcounttrans@*/
+}
+
 /**
  * Run per-interpreter dependency helper.
  * @param fc		file classifier
@@ -317,10 +407,12 @@ static int rpmfcHelper(rpmfc fc, unsigned char deptype, const char * nsdep)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies fc, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
+    miRE mire = NULL;
+    int nmire = 0;
     const char * fn = fc->fn[fc->ix];
     char buf[BUFSIZ];
-    StringBuf sb_stdout = NULL;
-    StringBuf sb_stdin;
+    rpmiob iob_stdout = NULL;
+    rpmiob iob_stdin;
     const char *av[2];
     rpmds * depsp, ds;
     const char * N;
@@ -345,6 +437,8 @@ static int rpmfcHelper(rpmfc fc, unsigned char deptype, const char * nsdep)
 	depsp = &fc->provides;
 	dsContext = RPMSENSE_FIND_PROVIDES;
 	tagN = RPMTAG_PROVIDENAME;
+	mire = fc->Pmires;
+	nmire = fc->Pnmire;
 	break;
     case 'R':
 	if (fc->skipReq)
@@ -353,21 +447,23 @@ static int rpmfcHelper(rpmfc fc, unsigned char deptype, const char * nsdep)
 	depsp = &fc->requires;
 	dsContext = RPMSENSE_FIND_REQUIRES;
 	tagN = RPMTAG_REQUIRENAME;
+	mire = fc->Rmires;
+	nmire = fc->Rnmire;
 	break;
     }
     buf[sizeof(buf)-1] = '\0';
     av[0] = buf;
     av[1] = NULL;
 
-    sb_stdin = newStringBuf();
-    appendLineStringBuf(sb_stdin, fn);
-    sb_stdout = NULL;
-    xx = rpmfcExec(av, sb_stdin, &sb_stdout, 0);
-    sb_stdin = freeStringBuf(sb_stdin);
+    iob_stdin = rpmiobNew(0);
+    iob_stdin = rpmiobAppend(iob_stdin, fn, 1);
+    iob_stdout = NULL;
+    xx = rpmfcExec(av, iob_stdin, &iob_stdout, 0);
+    iob_stdin = rpmiobFree(iob_stdin);
 
-    if (xx == 0 && sb_stdout != NULL) {
+    if (xx == 0 && iob_stdout != NULL) {
 	pav = NULL;
-	xx = argvSplit(&pav, getStringBuf(sb_stdout), " \t\n\r");
+	xx = argvSplit(&pav, rpmiobStr(iob_stdout), " \t\n\r");
 	pac = argvCount(pav);
 	if (pav)
 	for (i = 0; i < pac; i++) {
@@ -397,6 +493,8 @@ assert(*s != '\0');
 assert(EVR != NULL);
 	    }
 
+	    if (_filter_values && rpmfcMatchRegexps(mire, nmire, N, deptype))
+		continue;
 
 	    /* Add tracking dependency for versioned Provides: */
 	    if (!fc->tracked && deptype == 'P' && *EVR != '\0') {
@@ -404,7 +502,8 @@ assert(EVR != NULL);
 			"rpmlib(VersionedDependencies)", "3.0.3-1",
 			RPMSENSE_RPMLIB|(RPMSENSE_LESS|RPMSENSE_EQUAL));
 		xx = rpmdsMerge(&fc->requires, ds);
-		ds = rpmdsFree(ds);
+		(void)rpmdsFree(ds);
+		ds = NULL;
 		fc->tracked = 1;
 	    }
 
@@ -416,18 +515,20 @@ assert(EVR != NULL);
 	    /* Add to file dependencies. */
 	    xx = rpmfcSaveArg(&fc->ddict, rpmfcFileDep(buf, fc->ix, ds));
 
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	}
 
 	pav = argvFree(pav);
     }
-    sb_stdout = freeStringBuf(sb_stdout);
+    iob_stdout = rpmiobFree(iob_stdout);
 
     return 0;
 }
 
 /**
  */
+/*@-nullassign@*/
 /*@unchecked@*/ /*@observer@*/
 static struct rpmfcTokens_s rpmfcTokens[] = {
   { "directory",		RPMFC_DIRECTORY|RPMFC_INCLUDE },
@@ -469,7 +570,7 @@ static struct rpmfcTokens_s rpmfcTokens[] = {
 
   { "Java ",			RPMFC_JAVA|RPMFC_INCLUDE },
 
-  { "PE executable",            RPMFC_MONO|RPMFC_INCLUDE },
+  { "Mono/.Net assembly",	RPMFC_MONO|RPMFC_INCLUDE },
 
   { "current ar archive",	RPMFC_STATIC|RPMFC_LIBRARY|RPMFC_ARCHIVE|RPMFC_INCLUDE },
 
@@ -527,6 +628,7 @@ static struct rpmfcTokens_s rpmfcTokens[] = {
 
   { NULL,			RPMFC_BLACK }
 };
+/*@=nullassign@*/
 
 int rpmfcColoring(const char * fmstr)
 {
@@ -549,10 +651,10 @@ void rpmfcPrint(const char * msg, rpmfc fc, FILE * fp)
     int ndx;
     int cx;
     int dx;
-    int fx;
+    size_t fx;
 
-int nprovides;
-int nrequires;
+unsigned nprovides;
+unsigned nrequires;
 
     if (fp == NULL) fp = stderr;
 
@@ -569,7 +671,7 @@ assert(fx < fc->fcdictx->nvals);
 assert(fx < fc->fcolor->nvals);
 	fcolor = fc->fcolor->vals[fx];
 
-	fprintf(fp, "%3d %s", fx, fc->fn[fx]);
+	fprintf(fp, "%3d %s", (int)fx, fc->fn[fx]);
 	if (fcolor != RPMFC_BLACK)
 		fprintf(fp, "\t0x%x", fc->fcolor->vals[fx]);
 	else
@@ -620,37 +722,6 @@ assert(ix < nrequires);
     }
 }
 
-rpmfc rpmfcFree(rpmfc fc)
-{
-    if (fc) {
-	fc->fn = argvFree(fc->fn);
-	fc->fcolor = argiFree(fc->fcolor);
-	fc->fcdictx = argiFree(fc->fcdictx);
-	fc->fddictx = argiFree(fc->fddictx);
-	fc->fddictn = argiFree(fc->fddictn);
-	fc->cdict = argvFree(fc->cdict);
-	fc->ddict = argvFree(fc->ddict);
-	fc->ddictx = argiFree(fc->ddictx);
-
-	fc->provides = rpmdsFree(fc->provides);
-	fc->requires = rpmdsFree(fc->requires);
-
-	fc->sb_java = freeStringBuf(fc->sb_java);
-	fc->sb_perl = freeStringBuf(fc->sb_perl);
-	fc->sb_python = freeStringBuf(fc->sb_python);
-	fc->sb_php = freeStringBuf(fc->sb_php);
-
-    }
-    fc = _free(fc);
-    return NULL;
-}
-
-rpmfc rpmfcNew(void)
-{
-    rpmfc fc = xcalloc(1, sizeof(*fc));
-    return fc;
-}
-
 /**
  * Extract script dependencies.
  * @param fc		file classifier
@@ -674,7 +745,7 @@ static int rpmfcSCRIPT(rpmfc fc)
     {	struct stat sb, * st = &sb;
 	if (stat(fn, st) != 0)
 	    return -1;
-	is_executable = (st->st_mode & (S_IXUSR|S_IXGRP|S_IXOTH));
+	is_executable = (int)(st->st_mode & (S_IXUSR|S_IXGRP|S_IXOTH));
     }
 
     fp = fopen(fn, "r");
@@ -708,6 +779,9 @@ static int rpmfcSCRIPT(rpmfc fc)
 	*se = '\0';
 	se++;
 
+	if (!_filter_values
+	 || (!fc->skipReq
+	  && !rpmfcMatchRegexps(fc->Rmires, fc->Rnmire, s, 'R')))
 	if (is_executable) {
 	    /* Add to package requires. */
 	    ds = rpmdsSingle(RPMTAG_REQUIRENAME, s, "", RPMSENSE_FIND_REQUIRES);
@@ -716,7 +790,8 @@ static int rpmfcSCRIPT(rpmfc fc)
 	    /* Add to file requires. */
 	    xx = rpmfcSaveArg(&fc->ddict, rpmfcFileDep(se, fc->ix, ds));
 
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	}
 
 	/* Set color based on interpreter name. */
@@ -785,7 +860,6 @@ static int rpmfcSCRIPT(rpmfc fc)
     return 0;
 }
 
-
 /**
  * Merge provides/requires dependencies into a rpmfc container.
  * @param context	merge dependency set(s) container
@@ -798,28 +872,39 @@ static int rpmfcMergePR(void * context, rpmds ds)
 {
     rpmfc fc = context;
     char buf[BUFSIZ];
-    int rc = -1;
+    int rc = 0;
 
 if (_rpmfc_debug < 0)
 fprintf(stderr, "*** rpmfcMergePR(%p, %p) %s\n", context, ds, tagName(rpmdsTagN(ds)));
     switch(rpmdsTagN(ds)) {
     default:
+	rc = -1;
 	break;
     case RPMTAG_PROVIDENAME:
-	/* Add to package provides. */
-	rc = rpmdsMerge(&fc->provides, ds);
+	if (!_filter_values
+	 || (!fc->skipProv
+	  && !rpmfcMatchRegexps(fc->Pmires, fc->Pnmire, ds->N[0], 'P')))
+	{
+	    /* Add to package provides. */
+	    rc = rpmdsMerge(&fc->provides, ds);
 
-	/* Add to file dependencies. */
-	buf[0] = '\0';
-	rc = rpmfcSaveArg(&fc->ddict, rpmfcFileDep(buf, fc->ix, ds));
+	    /* Add to file dependencies. */
+	    buf[0] = '\0';
+	    rc = rpmfcSaveArg(&fc->ddict, rpmfcFileDep(buf, fc->ix, ds));
+	}
 	break;
     case RPMTAG_REQUIRENAME:
-	/* Add to package requires. */
-	rc = rpmdsMerge(&fc->requires, ds);
+	if (!_filter_values
+	 || (!fc->skipReq
+	  && !rpmfcMatchRegexps(fc->Rmires, fc->Rnmire, ds->N[0], 'R')))
+	{
+	    /* Add to package requires. */
+	    rc = rpmdsMerge(&fc->requires, ds);
 
-	/* Add to file dependencies. */
-	buf[0] = '\0';
-	rc = rpmfcSaveArg(&fc->ddict, rpmfcFileDep(buf, fc->ix, ds));
+	    /* Add to file dependencies. */
+	    buf[0] = '\0';
+	    rc = rpmfcSaveArg(&fc->ddict, rpmfcFileDep(buf, fc->ix, ds));
+	}
 	break;
     }
     return rc;
@@ -853,12 +938,14 @@ typedef struct rpmfcApplyTbl_s {
 /**
  * XXX Having two entries for rpmfcSCRIPT may be unnecessary duplication.
  */
+/*@-nullassign@*/
 /*@unchecked@*/
 static struct rpmfcApplyTbl_s rpmfcApplyTable[] = {
     { rpmfcELF,		RPMFC_ELF },
     { rpmfcSCRIPT,	(RPMFC_SCRIPT|RPMFC_PERL|RPMFC_PYTHON|RPMFC_LIBTOOL|RPMFC_PKGCONFIG|RPMFC_BOURNE|RPMFC_JAVA|RPMFC_PHP|RPMFC_MONO) },
     { NULL, 0 }
 };
+/*@=nullassign@*/
 
 rpmRC rpmfcApply(rpmfc fc)
 {
@@ -866,6 +953,7 @@ rpmRC rpmfcApply(rpmfc fc)
     const char * s;
     char * se;
     rpmds ds;
+    const char * fn;
     const char * N;
     const char * EVR;
     evrFlags Flags;
@@ -879,6 +967,25 @@ rpmRC rpmfcApply(rpmfc fc)
     int xx;
     int skipping;
 
+    miRE mire;
+    int skipProv = fc->skipProv;
+    int skipReq = fc->skipReq;
+    int j;
+
+    if (_filter_execs) {
+	fc->Pnmire = 0;
+	fc->PFnmire = 0;
+	fc->Rnmire = 0;
+	fc->RFnmire = 0;
+    
+	fc->PFmires = rpmfcExpandRegexps("%{__noautoprovfiles}", &fc->PFnmire);
+	fc->RFmires = rpmfcExpandRegexps("%{__noautoreqfiles}", &fc->RFnmire);
+	fc->Pmires = rpmfcExpandRegexps("%{__noautoprov}", &fc->Pnmire);
+	fc->Rmires = rpmfcExpandRegexps("%{__noautoreq}", &fc->Rnmire);
+	rpmlog(RPMLOG_DEBUG, D_("%i _noautoprov patterns.\n"), fc->Pnmire);
+	rpmlog(RPMLOG_DEBUG, D_("%i _noautoreq patterns.\n"), fc->Rnmire);
+    }
+
 /* Make sure something didn't go wrong previously! */
 assert(fc->fn != NULL);
     /* Generate package and per-file dependencies. */
@@ -886,7 +993,7 @@ assert(fc->fn != NULL);
 
 	/* XXX Insure that /usr/lib{,64}/python files are marked RPMFC_PYTHON */
 	/* XXX HACK: classification by path is intrinsically stupid. */
-	{   const char *fn = strstr(fc->fn[fc->ix], "/usr/lib");
+	{   fn = strstr(fc->fn[fc->ix], "/usr/lib");
 	    if (fn) {
 		fn += sizeof("/usr/lib")-1;
 		if ((fn[0] == '3' && fn[1] == '2') || 
@@ -901,10 +1008,45 @@ assert(fc->fn != NULL);
 	for (fcat = rpmfcApplyTable; fcat->func != NULL; fcat++) {
 	    if (!(fc->fcolor->vals[fc->ix] & fcat->colormask))
 		/*@innercontinue@*/ continue;
+
+	    if (_filter_execs) {
+		fc->skipProv = skipProv;
+		fc->skipReq = skipReq;
+		if ((mire = fc->PFmires) != NULL)
+		for (j = 0; j < fc->PFnmire; j++, mire++) {
+		    fn = fc->fn[fc->ix] + fc->brlen;
+		    if ((xx = mireRegexec(mire, fn, 0)) < 0)
+			/*@innercontinue@*/ continue;
+		    rpmlog(RPMLOG_NOTICE, _("skipping %s provides detection\n"),
+				fn);
+		    fc->skipProv = 1;
+		    /*@innerbreak@*/ break;
+		}
+		if ((mire = fc->RFmires) != NULL)
+		for (j = 0; j < fc->RFnmire; j++, mire++) {
+		    fn = fc->fn[fc->ix] + fc->brlen;
+		    if ((xx = mireRegexec(mire, fn, 0)) < 0)
+			/*@innercontinue@*/ continue;
+		    rpmlog(RPMLOG_NOTICE, _("skipping %s requires detection\n"),
+				fn);
+		    fc->skipReq = 1;
+		    /*@innerbreak@*/ break;
+		}
+	    }
+
 	    xx = (*fcat->func) (fc);
 	}
     }
 
+    if (_filter_execs) {
+	fc->PFmires = rpmfcFreeRegexps(fc->PFmires, fc->PFnmire);
+	fc->RFmires = rpmfcFreeRegexps(fc->RFmires, fc->RFnmire);
+	fc->Pmires = rpmfcFreeRegexps(fc->Pmires, fc->Pnmire);
+	fc->Rmires = rpmfcFreeRegexps(fc->Rmires, fc->Rnmire);
+    }
+    fc->skipProv = skipProv;
+    fc->skipReq = skipReq;
+
     /* Generate per-file indices into package dependencies. */
     nddict = argvCount(fc->ddict);
     previx = -1;
@@ -935,13 +1077,15 @@ assert(se != NULL);
 	    skipping = fc->skipProv;
 	    ds = rpmdsSingle(RPMTAG_PROVIDENAME, N, EVR, Flags);
 	    dix = rpmdsFind(fc->provides, ds);
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	    /*@switchbreak@*/ break;
 	case 'R':
 	    skipping = fc->skipReq;
 	    ds = rpmdsSingle(RPMTAG_REQUIRENAME, N, EVR, Flags);
 	    dix = rpmdsFind(fc->requires, ds);
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	    /*@switchbreak@*/ break;
 	}
 
@@ -967,7 +1111,7 @@ assert(dix >= 0);
     return RPMRC_OK;
 }
 
-rpmRC rpmfcClassify(rpmfc fc, ARGV_t argv, uint16_t * fmode)
+rpmRC rpmfcClassify(rpmfc fc, ARGV_t argv, rpmuint16_t * fmode)
 {
     ARGV_t fcav = NULL;
     ARGV_t dav;
@@ -1000,7 +1144,7 @@ assert(mg != NULL);	/* XXX figger a proper return path. */
     for (fc->ix = 0; fc->ix < fc->nfiles; fc->ix++) {
 	const char * ftype;
 	int freeftype;
-	uint16_t mode = (fmode ? fmode[fc->ix] : 0);
+	rpmuint16_t mode = (fmode ? fmode[fc->ix] : 0);
 	int urltype;
 
 	ftype = "";	freeftype = 0;
@@ -1063,7 +1207,9 @@ assert(ftype != NULL);	/* XXX never happens, rpmmgFile() returns "" */
 	}
 
 	se = ftype;
-        rpmlog(RPMLOG_DEBUG, "%s: %s\n", s, se);
+
+if (_rpmfc_debug)	/* XXX noisy */
+	rpmlog(RPMLOG_DEBUG, "%s: %s\n", s, se);
 
 	/* Save the path. */
 	xx = argvAdd(&fc->fn, s);
@@ -1073,7 +1219,7 @@ assert(ftype != NULL);	/* XXX never happens, rpmmgFile() returns "" */
 
 	/* Add (filtered) entry to sorted class dictionary. */
 	fcolor = rpmfcColoring(se);
-	xx = argiAdd(&fc->fcolor, fc->ix, fcolor);
+	xx = argiAdd(&fc->fcolor, (int)fc->ix, fcolor);
 
 	if (fcolor != RPMFC_WHITE && (fcolor & RPMFC_INCLUDE))
 	    xx = rpmfcSaveArg(&fc->cdict, se);
@@ -1092,10 +1238,10 @@ assert(se != NULL);
 
 	dav = argvSearch(fc->cdict, se, NULL);
 	if (dav) {
-	    xx = argiAdd(&fc->fcdictx, fc->ix, (dav - fc->cdict));
+	    xx = argiAdd(&fc->fcdictx, (int)fc->ix, (dav - fc->cdict));
 	    fc->fknown++;
 	} else {
-	    xx = argiAdd(&fc->fcdictx, fc->ix, 0);
+	    xx = argiAdd(&fc->fcdictx, (int)fc->ix, 0);
 	    fc->fwhite++;
 	}
     }
@@ -1103,6 +1249,9 @@ assert(se != NULL);
     fcav = argvFree(fcav);
 
     mg = rpmmgFree(mg);
+    rpmlog(RPMLOG_DEBUG,
+		D_("categorized %d files into %d classes (using %s).\n"),
+		fc->nfiles, argvCount(fc->cdict), magicfile);
     magicfile = _free(magicfile);
 
     return RPMRC_OK;
@@ -1128,6 +1277,7 @@ struct DepMsg_s {
 
 /**
  */
+/*@-nullassign@*/
 /*@unchecked@*/
 static struct DepMsg_s depMsgs[] = {
   { "Provides",		{ "%{?__find_provides}", NULL, NULL, NULL },
@@ -1165,6 +1315,7 @@ static struct DepMsg_s depMsgs[] = {
 	0, -1 },
   { NULL,		{ NULL, NULL, NULL, NULL },	0, 0, 0, 0, 0 }
 };
+/*@=nullassign@*/
 
 /*@unchecked@*/
 static DepMsg_t DepMsgs = depMsgs;
@@ -1185,8 +1336,9 @@ static void printDeps(Header h)
     int bingo = 0;
 
     for (dm = DepMsgs; dm->msg != NULL; dm++) {
-	if (dm->ntag != -1) {
-	    ds = rpmdsFree(ds);
+	if ((int)dm->ntag != -1) {
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	    ds = rpmdsNew(h, dm->ntag, flags);
 	}
 	if (dm->ftag == 0)
@@ -1214,17 +1366,18 @@ static void printDeps(Header h)
 	if (bingo)
 	    rpmlog(RPMLOG_NOTICE, "\n");
     }
-    ds = rpmdsFree(ds);
+    (void)rpmdsFree(ds);
+    ds = NULL;
 }
 
 /**
  */
 static rpmRC rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-        /*@modifies fi, rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@modifies fi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
-    StringBuf sb_stdin;
-    StringBuf sb_stdout;
+    rpmiob iob_stdin;
+    rpmiob iob_stdout;
     DepMsg_t dm;
     int failnonzero = 0;
     rpmRC rc = RPMRC_OK;
@@ -1232,14 +1385,15 @@ static rpmRC rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
     /*
      * Create file manifest buffer to deliver to dependency finder.
      */
-    sb_stdin = newStringBuf();
+    iob_stdin = rpmiobNew(0);
     fi = rpmfiInit(fi, 0);
     if (fi != NULL)
     while (rpmfiNext(fi) >= 0)
-	appendLineStringBuf(sb_stdin, rpmfiFN(fi));
+	iob_stdin = rpmiobAppend(iob_stdin, rpmfiFN(fi), 1);
 
     for (dm = DepMsgs; dm->msg != NULL; dm++) {
-	int tag, tagflags;
+	rpmTag tag;
+	rpmsenseFlags tagflags;
 	char * s;
 	int xx;
 
@@ -1265,7 +1419,7 @@ static rpmRC rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 	    /*@notreached@*/ /*@switchbreak@*/ break;
 	}
 
-	xx = rpmfcExec(dm->argv, sb_stdin, &sb_stdout, failnonzero);
+	xx = rpmfcExec(dm->argv, iob_stdin, &iob_stdout, failnonzero);
 	if (xx == -1)
 	    continue;
 
@@ -1274,7 +1428,7 @@ static rpmRC rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 		(s ? s : ""));
 	s = _free(s);
 
-	if (sb_stdout == NULL) {
+	if (iob_stdout == NULL) {
 	    rpmlog(RPMLOG_ERR, _("Failed to find %s:\n"), dm->msg);
 	    rc = RPMRC_FAIL;
 	    break;
@@ -1282,9 +1436,9 @@ static rpmRC rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 
 	/* Parse dependencies into header */
 	if (spec->_parseRCPOT)
-	    rc = spec->_parseRCPOT(spec, pkg, getStringBuf(sb_stdout), tag,
+	    rc = spec->_parseRCPOT(spec, pkg, rpmiobStr(iob_stdout), tag,
 				0, tagflags);
-	sb_stdout = freeStringBuf(sb_stdout);
+	iob_stdout = rpmiobFree(iob_stdout);
 
 	if (rc) {
 	    rpmlog(RPMLOG_ERR, _("Failed to find %s:\n"), dm->msg);
@@ -1292,13 +1446,14 @@ static rpmRC rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 	}
     }
 
-    sb_stdin = freeStringBuf(sb_stdin);
+    iob_stdin = rpmiobFree(iob_stdin);
 
     return rc;
 }
 
 /**
  */
+/*@-nullassign@*/
 /*@unchecked@*/
 static struct DepMsg_s scriptMsgs[] = {
   { "Requires(pre)",	{ "%{?__scriptlet_requires}", NULL, NULL, NULL },
@@ -1315,6 +1470,7 @@ static struct DepMsg_s scriptMsgs[] = {
 	RPMSENSE_SCRIPT_POSTUN, 0 },
   { NULL,		{ NULL, NULL, NULL, NULL },	0, 0, 0, 0, 0 }
 };
+/*@=nullassign@*/
 
 /*@unchecked@*/
 static DepMsg_t ScriptMsgs = scriptMsgs;
@@ -1323,11 +1479,11 @@ static DepMsg_t ScriptMsgs = scriptMsgs;
  */
 static int rpmfcGenerateScriptletDeps(const Spec spec, Package pkg)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-        /*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
-    StringBuf sb_stdin = newStringBuf();
-    StringBuf sb_stdout = NULL;
+    rpmiob iob_stdin = rpmiobNew(0);
+    rpmiob iob_stdout = NULL;
     DepMsg_t dm;
     int failnonzero = 0;
     int rc = 0;
@@ -1355,22 +1511,22 @@ static int rpmfcGenerateScriptletDeps(const Spec spec, Package pkg)
 	xx = headerGet(pkg->header, he, 0);
 	if (!xx || he->p.str == NULL)
 	    continue;
-	truncStringBuf(sb_stdin);
-	appendLineStringBuf(sb_stdin, he->p.str);
-	stripTrailingBlanksStringBuf(sb_stdin);
+	iob_stdin = rpmiobEmpty(iob_stdin);
+	iob_stdin = rpmiobAppend(iob_stdin, he->p.str, 1);
+	iob_stdin = rpmiobRTrim(iob_stdin);
 	he->p.ptr = _free(he->p.ptr);
 
-	xx = rpmfcExec(dm->argv, sb_stdin, &sb_stdout, failnonzero);
+	xx = rpmfcExec(dm->argv, iob_stdin, &iob_stdout, failnonzero);
 	if (xx == -1)
 	    continue;
 
 	/* Parse dependencies into header */
-	s = getStringBuf(sb_stdout);
+	s = rpmiobStr(iob_stdout);
 	if (s != NULL && *s != '\0') {
 	    char * se = s;
 	    /* XXX Convert "executable(/path/to/file)" to "/path/to/file". */
 	    while ((se = strstr(se, "executable(/")) != NULL) {
-/*@-modobserver@*/	/* FIX: getStringBuf should not be observer */
+/*@-modobserver@*/	/* FIX: rpmiobStr should not be observer */
 		se = stpcpy(se,     "           ");
 		*se = '/';	/* XXX stpcpy truncates the '/' */
 /*@=modobserver@*/
@@ -1382,11 +1538,11 @@ static int rpmfcGenerateScriptletDeps(const Spec spec, Package pkg)
 	    if (spec->_parseRCPOT)
 		rc = spec->_parseRCPOT(spec, pkg, s, tag, 0, tagflags);
 	}
-	sb_stdout = freeStringBuf(sb_stdout);
+	iob_stdout = rpmiobFree(iob_stdout);
 
     }
 
-    sb_stdin = freeStringBuf(sb_stdin);
+    iob_stdin = rpmiobFree(iob_stdin);
 
     return rc;
 }
@@ -1401,7 +1557,7 @@ rpmRC rpmfcGenerateDepends(void * specp, void * pkgp)
     rpmds ds;
     int flags = 0x2;	/* XXX no filtering, !scareMem */
     ARGV_t av;
-    uint16_t * fmode;
+    rpmuint16_t * fmode;
     int ac = rpmfiFC(fi);
     char buf[BUFSIZ];
     const char * N;
@@ -1470,7 +1626,8 @@ rpmRC rpmfcGenerateDepends(void * specp, void * pkgp)
     if (!fc->skipProv) {
 	ds = rpmdsNew(pkg->header, RPMTAG_PROVIDENAME, flags);
 	xx = rpmdsMerge(&fc->provides, ds);
-	ds = rpmdsFree(ds);
+	(void)rpmdsFree(ds);
+	ds = NULL;
 	he->tag = RPMTAG_PROVIDENAME;
 	xx = headerDel(pkg->header, he, 0);
 	he->tag = RPMTAG_PROVIDEVERSION;
@@ -1488,14 +1645,16 @@ assert(EVR != NULL);
 	    ds = rpmdsSingle(RPMTAG_PROVIDENAME, buf, EVR,
 			(RPMSENSE_EQUAL|RPMSENSE_CONFIG));
 	    xx = rpmdsMerge(&fc->provides, ds);
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	}
     }
 
     if (!fc->skipReq) {
 	ds = rpmdsNew(pkg->header, RPMTAG_REQUIRENAME, flags);
 	xx = rpmdsMerge(&fc->requires, ds);
-	ds = rpmdsFree(ds);
+	(void)rpmdsFree(ds);
+	ds = NULL;
 	he->tag = RPMTAG_REQUIRENAME;
 	xx = headerDel(pkg->header, he, 0);
 	he->tag = RPMTAG_REQUIREVERSION;
@@ -1513,7 +1672,8 @@ assert(EVR != NULL);
 	    ds = rpmdsSingle(RPMTAG_REQUIRENAME, buf, EVR,
 			(RPMSENSE_EQUAL|RPMSENSE_CONFIG));
 	    xx = rpmdsMerge(&fc->requires, ds);
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	}
     }
 
@@ -1528,12 +1688,12 @@ assert(EVR != NULL);
     he->t = RPM_UINT32_TYPE;
     he->p.ui32p = argiData(fc->fcolor);
     he->c = argiCount(fc->fcolor);
-assert(ac == he->c);
+assert(ac == (int)he->c);
     if (he->p.ptr != NULL && he->c > 0) {
-	uint32_t * fcolors = he->p.ui32p;
+	rpmuint32_t * fcolors = he->p.ui32p;
 
 	/* XXX Make sure only primary (i.e. Elf32/Elf64) colors are added. */
-	for (i = 0; i < he->c; i++)
+	for (i = 0; i < (int)he->c; i++)
 	    fcolors[i] &= 0x0f;
 
 	xx = headerPut(pkg->header, he, 0);
@@ -1553,7 +1713,7 @@ assert(ac == he->c);
     he->t = RPM_UINT32_TYPE;
     he->p.ui32p = argiData(fc->fcdictx);
     he->c = argiCount(fc->fcdictx);
-assert(ac == he->c);
+assert(ac == (int)he->c);
     if (he->p.ptr != NULL && he->c > 0) {
 	xx = headerPut(pkg->header, he, 0);
     }
@@ -1577,7 +1737,7 @@ assert(he->p.ptr != NULL);
 
 	he->tag = RPMTAG_PROVIDEFLAGS;
 	he->t = RPM_UINT32_TYPE;
-	he->p.ui32p = (uint32_t *) fc->provides->Flags;
+	he->p.ui32p = (rpmuint32_t *) fc->provides->Flags;
 assert(he->p.ptr != NULL);
 	xx = headerPut(pkg->header, he, 0);
 /*@=nullpass@*/
@@ -1603,7 +1763,7 @@ assert(he->p.ptr != NULL);
 
 	he->tag = RPMTAG_REQUIREFLAGS;
 	he->t = RPM_UINT32_TYPE;
-	he->p.ui32p = (uint32_t *) fc->requires->Flags;
+	he->p.ui32p = (rpmuint32_t *) fc->requires->Flags;
 assert(he->p.ptr != NULL);
 	xx = headerPut(pkg->header, he, 0);
 /*@=nullpass@*/
@@ -1623,7 +1783,7 @@ assert(he->p.ptr != NULL);
     he->t = RPM_UINT32_TYPE;
     he->p.ui32p = argiData(fc->fddictx);
     he->c = argiCount(fc->fddictx);
-assert(ac == he->c);
+assert(ac == (int)he->c);
     if (he->p.ptr != NULL) {
 	xx = headerPut(pkg->header, he, 0);
     }
@@ -1632,7 +1792,7 @@ assert(ac == he->c);
     he->t = RPM_UINT32_TYPE;
     he->p.ui32p = argiData(fc->fddictn);
     he->c = argiCount(fc->fddictn);
-assert(ac == he->c);
+assert(ac == (int)he->c);
     if (he->p.ptr != NULL) {
 	xx = headerPut(pkg->header, he, 0);
     }
@@ -1641,7 +1801,7 @@ assert(ac == he->c);
 
 if (fc != NULL && _rpmfc_debug) {
 char msg[BUFSIZ];
-sprintf(msg, "final: files %d cdict[%d] %d%% ddictx[%d]", fc->nfiles, argvCount(fc->cdict), ((100 * fc->fknown)/fc->nfiles), argiCount(fc->ddictx));
+sprintf(msg, "final: files %u cdict[%d] %u%% ddictx[%d]", (unsigned int)fc->nfiles, argvCount(fc->cdict), (unsigned int)((100 * fc->fknown)/fc->nfiles), argiCount(fc->ddictx));
 rpmfcPrint(msg, fc, NULL);
 }
 
@@ -1652,3 +1812,54 @@ rpmfcPrint(msg, fc, NULL);
 
     return rc;
 }
+
+/*@-mustmod@*/
+static void rpmfcFini(void * _fc)
+	/*@modifies _fc @*/
+{
+    rpmfc fc = _fc;
+
+    fc->fn = argvFree(fc->fn);
+    fc->fcolor = argiFree(fc->fcolor);
+    fc->fcdictx = argiFree(fc->fcdictx);
+    fc->fddictx = argiFree(fc->fddictx);
+    fc->fddictn = argiFree(fc->fddictn);
+    fc->cdict = argvFree(fc->cdict);
+    fc->ddict = argvFree(fc->ddict);
+    fc->ddictx = argiFree(fc->ddictx);
+
+    (void)rpmdsFree(fc->provides);
+    fc->provides = NULL;
+    (void)rpmdsFree(fc->requires);
+    fc->requires = NULL;
+
+    fc->iob_java = rpmiobFree(fc->iob_java);
+    fc->iob_perl = rpmiobFree(fc->iob_perl);
+    fc->iob_python = rpmiobFree(fc->iob_python);
+    fc->iob_php = rpmiobFree(fc->iob_php);
+}
+/*@=mustmod@*/
+
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _rpmfcPool = NULL;
+
+static rpmfc rpmfcGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _rpmfcPool, fileSystem, internalState @*/
+	/*@modifies pool, _rpmfcPool, fileSystem, internalState @*/
+{
+    rpmfc fc;
+
+    if (_rpmfcPool == NULL) {
+	_rpmfcPool = rpmioNewPool("fc", sizeof(*fc), -1, _rpmfc_debug,
+			NULL, NULL, rpmfcFini);
+	pool = _rpmfcPool;
+    }
+    return (rpmfc) rpmioGetPool(pool, sizeof(*fc));
+}
+
+rpmfc rpmfcNew(void)
+{
+    rpmfc fc = rpmfcGetPool(_rpmfcPool);
+    return rpmfcLink(fc);
+}
+
