@@ -8,7 +8,7 @@ extern int _rpmfc_debug;
 
 /**
  */
-typedef /*@abstract@*/ struct rpmfc_s * rpmfc;
+typedef /*@abstract@*/ /*@refcounted@*/ struct rpmfc_s * rpmfc;
 
 /**
  */
@@ -70,10 +70,11 @@ enum FCOLOR_e {
 /**
  */
 struct rpmfc_s {
-    int nfiles;		/*!< no. of files */
-    int fknown;		/*!< no. of classified files */
-    int fwhite;		/*!< no. of "white" files */
-    int ix;		/*!< current file index */
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
+    size_t nfiles;	/*!< no. of files */
+    size_t fknown;	/*!< no. of classified files */
+    size_t fwhite;	/*!< no. of "white" files */
+    size_t ix;		/*!< current file index */
     int skipProv;	/*!< Don't auto-generate Provides:? */
     int skipReq;	/*!< Don't auto-generate Requires:? */
     int tracked;	/*!< Versioned Provides: tracking dependency added? */
@@ -93,10 +94,23 @@ struct rpmfc_s {
 /*@relnull@*/
     rpmds requires;	/*!< (no. requires) package requires */
 
-    StringBuf sb_java;	/*!< concatenated list of java colored files. */
-    StringBuf sb_perl;	/*!< concatenated list of perl colored files. */
-    StringBuf sb_python;/*!< concatenated list of python colored files. */
-    StringBuf sb_php;	/*!< concatenated list of php colored files. */
+    rpmiob iob_java;	/*!< concatenated list of java colored files. */
+    rpmiob iob_perl;	/*!< concatenated list of perl colored files. */
+    rpmiob iob_python;	/*!< concatenated list of python colored files. */
+    rpmiob iob_php;	/*!< concatenated list of php colored files. */
+
+/*@null@*/
+    void * Pmires;	/*!< Filter patterns from %{__noautoprov} */
+    int Pnmire;
+/*@null@*/
+    void * PFmires;	/*!< Filter patterns from %{__noautoprov} */
+    int PFnmire;
+/*@null@*/
+    void * Rmires;	/*!< Filter patterns from %{__noautoreq} */
+    int Rnmire;
+/*@null@*/
+    void * RFmires;	/*!< Filter patterns from %{__noautoreqfile} */
+    int RFnmire;
 
 };
 
@@ -116,16 +130,16 @@ extern "C" {
 /**
  * Return helper output.
  * @param av		helper argv (with possible macros)
- * @param sb_stdin	helper input
- * @retval *sb_stdoutp	helper output
+ * @param iob_stdin	helper input
+ * @retval *iob_stdoutp	helper output
  * @param failnonzero	Is non-zero helper exit status a failure?
  */
-int rpmfcExec(ARGV_t av, StringBuf sb_stdin, /*@out@*/ StringBuf * sb_stdoutp,
+int rpmfcExec(const char ** av, rpmiob iob_stdin, /*@out@*/ rpmiob * iob_stdoutp,
 		int failnonzero)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies *sb_stdoutp, rpmGlobalMacroContext,
+	/*@modifies *iob_stdoutp, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
-        /*@requires maxSet(sb_stdoutp) >= 0 @*/;
+        /*@requires maxSet(iob_stdoutp) >= 0 @*/;
 
 /**
  * Return file color given file(1) string.
@@ -150,26 +164,6 @@ void rpmfcPrint(/*@null@*/ const char * msg, rpmfc fc, /*@null@*/ FILE * fp)
 	/*@modifies *fp, fc, fileSystem @*/;
 /*@=exportlocal@*/
 
-/**
- * Destroy a file classifier.
- * @param fc		file classifier
- * @return		NULL always
- */
-/*@-exportlocal@*/
-/*@null@*/
-rpmfc rpmfcFree(/*@only@*/ /*@null@*/ rpmfc fc)
-	/*@modifies fc @*/;
-/*@=exportlocal@*/
-
-/**
- * Create a file classifier.
- * @return		new file classifier
- */
-/*@-exportlocal@*/
-rpmfc rpmfcNew(void)
-	/*@*/;
-/*@=exportlocal@*/
-
 /**
  * Build file class dictionary and mappings.
  * @param fc		file classifier
@@ -178,7 +172,7 @@ rpmfc rpmfcNew(void)
  * @return		RPMRC_OK on success
  */
 /*@-exportlocal@*/
-rpmRC rpmfcClassify(rpmfc fc, ARGV_t argv, /*@null@*/ uint16_t * fmode)
+rpmRC rpmfcClassify(rpmfc fc, const char ** argv, /*@null@*/ rpmuint16_t * fmode)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies fc, rpmGlobalMacroContext, fileSystem, internalState @*/;
 /*@=exportlocal@*/
@@ -190,7 +184,8 @@ rpmRC rpmfcClassify(rpmfc fc, ARGV_t argv, /*@null@*/ uint16_t * fmode)
  */
 /*@-exportlocal@*/
 rpmRC rpmfcApply(rpmfc fc)
-	/*@modifies fc @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies fc, rpmGlobalMacroContext, internalState @*/;
 /*@=exportlocal@*/
 
 /**
@@ -201,8 +196,49 @@ rpmRC rpmfcApply(rpmfc fc)
  */
 rpmRC rpmfcGenerateDepends(void * specp, void * pkgp)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies *pkgp,
-		rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/**
+ * Unreference a file classifier instance.
+ * @param ds		dependency set
+ * @return		NULL if free'd
+ */
+/*@unused@*/ /*@null@*/
+rpmfc rpmfcUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmfc fc)
+	/*@modifies fc @*/;
+#define	rpmfcUnlink(_fc)	\
+	((rpmfc)rpmioUnlinkPoolItem((rpmioItem)(_fc), __FUNCTION__, __FILE__, __LINE__))
+
+/**
+ * Reference a file classifier instance.
+ * @param ds		file classifier
+ * @return		new file classifier reference
+ */
+/*@unused@*/ /*@newref@*/ /*@null@*/
+rpmfc rpmfcLink (/*@null@*/ rpmfc fc)
+	/*@modifies fc @*/;
+#define	rpmfcLink(_fc)	\
+	((rpmfc)rpmioLinkPoolItem((rpmioItem)(_fc), __FUNCTION__, __FILE__, __LINE__))
+
+/**
+ * Destroy a file classifier.
+ * @param fc		file classifier
+ * @return		NULL if free'd
+ */
+/*@null@*/
+rpmfc rpmfcFree(/*@only@*/ /*@null@*/ rpmfc fc)
+	/*@modifies fc @*/;
+#define	rpmfcFree(_fc)	\
+	((rpmfc)rpmioFreePoolItem((rpmioItem)(_fc), __FUNCTION__, __FILE__, __LINE__))
+
+/**
+ * Create a file classifier.
+ * @return		new file classifier
+ */
+/*@-exportlocal@*/
+rpmfc rpmfcNew(void)
+	/*@*/;
+/*@=exportlocal@*/
 
 #ifdef __cplusplus
 }
