@@ -6,11 +6,19 @@
 #include "system.h"
 
 #include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
+#include <rpmiotypes.h>	/* XXX fnpyKey */
+#include <rpmlog.h>
 #include <rpmurl.h>	/* XXX urlGetPath */
-#include <rpmlib.h>
+#include <rpmmacro.h>	/* XXX rpmCleanPath */
+#include <ugid.h>
 
-#include "cpio.h"	/* XXX CPIO_FOO */
+#define	_RPMAV_INTERNAL	/* XXX avOpendir */
+#include <rpmdav.h>
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
+
+#define	_IOSM_INTERNAL
 #define	_RPMFI_INTERNAL
 #include "fsm.h"	/* XXX newFSM() */
 #include "legacy.h"	/* XXX dodigest */
@@ -21,37 +29,46 @@
 #include "rpmte.h"
 #include "rpmts.h"
 
-#include "misc.h"	/* XXX stripTrailingChar */
-#include "rpmmacro.h"	/* XXX rpmCleanPath */
+#include <rpmcli.h>	/* XXX rpmHeaderFormats */
 
 #include "debug.h"
 
+/*@access IOSM_t @*/	/* XXX cast */
+
 /*@access rpmte @*/
+/*@access rpmts @*/	/* XXX cast */
+
 /*@access FSM_t @*/	/* XXX fsm->repackaged */
+/*@access DIR @*/
+
+/**
+ */
+struct rpmRelocation_s {
+/*@only@*/ /*@null@*/
+    const char * oldPath;	/*!< NULL here evals to RPMTAG_DEFAULTPREFIX, */
+/*@only@*/ /*@null@*/
+    const char * newPath;	/*!< NULL means to omit the file completely! */
+};
 
 /*@unchecked@*/
 int _rpmfi_debug = 0;
 
-rpmfi XrpmfiUnlink(rpmfi fi, const char * msg, const char * fn, unsigned ln)
+/**
+ * Remove occurences of trailing character from string.
+ * @param s		string
+ * @param c		character to strip
+ * @return 		string
+ */
+static /*@only@*/
+char * stripTrailingChar(/*@only@*/ char * s, char c)
+	/*@modifies *s */
 {
-    if (fi == NULL) return NULL;
-/*@-modfilesys@*/
-if (_rpmfi_debug && msg != NULL)
-fprintf(stderr, "--> fi %p -- %d %s at %s:%u\n", fi, fi->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-    fi->nrefs--;
-    return NULL;
-}
-
-rpmfi XrpmfiLink(rpmfi fi, const char * msg, const char * fn, unsigned ln)
-{
-    if (fi == NULL) return NULL;
-    fi->nrefs++;
-/*@-modfilesys@*/
-if (_rpmfi_debug && msg != NULL)
-fprintf(stderr, "--> fi %p ++ %d %s at %s:%u\n", fi, fi->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-    /*@-refcounttrans@*/ return fi; /*@=refcounttrans@*/
+    char * t;
+/*@-boundswrite@*/
+    for (t = s + strlen(s) - 1; *t == c && t >= s; t--)
+	*t = '\0';
+/*@=boundswrite@*/
+    return s;
 }
 
 int rpmfiFC(rpmfi fi)
@@ -79,7 +96,7 @@ int rpmfiSetFX(rpmfi fi, int fx)
 {
     int i = -1;
 
-    if (fi != NULL && fx >= 0 && fx < fi->fc) {
+    if (fi != NULL && fx >= 0 && fx < (int)fi->fc) {
 	i = fi->i;
 	fi->i = fx;
 	fi->j = fi->dil[fi->i];
@@ -96,7 +113,7 @@ int rpmfiSetDX(rpmfi fi, int dx)
 {
     int j = -1;
 
-    if (fi != NULL && dx >= 0 && dx < fi->dc) {
+    if (fi != NULL && dx >= 0 && dx < (int)fi->dc) {
 	j = fi->j;
 	fi->j = dx;
     }
@@ -112,7 +129,7 @@ const char * rpmfiBN(rpmfi fi)
 {
     const char * BN = NULL;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->bnl != NULL)
 	    BN = fi->bnl[fi->i];
     }
@@ -123,7 +140,7 @@ const char * rpmfiDN(rpmfi fi)
 {
     const char * DN = NULL;
 
-    if (fi != NULL && fi->j >= 0 && fi->j < fi->dc) {
+    if (fi != NULL && fi->j >= 0 && fi->j < (int)fi->dc) {
 	if (fi->dnl != NULL)
 	    DN = fi->dnl[fi->j];
     }
@@ -134,11 +151,11 @@ const char * rpmfiFN(rpmfi fi)
 {
     const char * FN = "";
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	const char *dn;
 	char * t;
 	if (fi->fn == NULL)
-	    fi->fn = xmalloc(fi->fnlen);
+	    fi->fn = xmalloc(fi->fnlen + 1);
 	FN = t = fi->fn;
 	(void) urlPath(fi->dnl[fi->dil[fi->i]], &dn);
 	*t = '\0';
@@ -148,59 +165,66 @@ const char * rpmfiFN(rpmfi fi)
     return FN;
 }
 
-uint32_t rpmfiFFlags(rpmfi fi)
+size_t rpmfiFNMaxLen(rpmfi fi)
 {
-    uint32_t FFlags = 0;
+    if (fi != NULL)
+	return fi->fnlen;
+    return 0;
+}
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+rpmuint32_t rpmfiFFlags(rpmfi fi)
+{
+    rpmuint32_t FFlags = 0;
+
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fflags != NULL)
 	    FFlags = fi->fflags[fi->i];
     }
     return FFlags;
 }
 
-uint32_t rpmfiSetFFlags(rpmfi fi, uint32_t FFlags)
+rpmuint32_t rpmfiSetFFlags(rpmfi fi, rpmuint32_t FFlags)
 {
-    uint32_t oFFlags = 0;
+    rpmuint32_t oFFlags = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fflags != NULL && fi->h == NULL) {
 	    oFFlags = fi->fflags[fi->i];
-	    *((uint32_t *)(fi->fflags + fi->i)) = FFlags;
+	    *((rpmuint32_t *)(fi->fflags + fi->i)) = FFlags;
 	}
     }
     return oFFlags;
 }
 
-uint32_t rpmfiVFlags(rpmfi fi)
+rpmuint32_t rpmfiVFlags(rpmfi fi)
 {
-    uint32_t VFlags = 0;
+    rpmuint32_t VFlags = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->vflags != NULL)
 	    VFlags = fi->vflags[fi->i];
     }
     return VFlags;
 }
 
-uint32_t rpmfiSetVFlags(rpmfi fi, uint32_t VFlags)
+rpmuint32_t rpmfiSetVFlags(rpmfi fi, rpmuint32_t VFlags)
 {
-    uint32_t oVFlags = 0;
+    rpmuint32_t oVFlags = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->vflags != NULL && fi->h == NULL) {
 	    oVFlags = fi->vflags[fi->i];
-	    *((uint32_t *)(fi->vflags + fi->i)) = VFlags;
+	    *((rpmuint32_t *)(fi->vflags + fi->i)) = VFlags;
 	}
     }
     return oVFlags;
 }
 
-uint16_t rpmfiFMode(rpmfi fi)
+rpmuint16_t rpmfiFMode(rpmfi fi)
 {
-    uint16_t fmode = 0;
+    rpmuint16_t fmode = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fmodes != NULL)
 	    fmode = fi->fmodes[fi->i];
     }
@@ -211,7 +235,7 @@ rpmfileState rpmfiFState(rpmfi fi)
 {
     rpmfileState fstate = RPMFILE_STATE_MISSING;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fstates != NULL)
 	    fstate = fi->fstates[fi->i];
     }
@@ -220,9 +244,9 @@ rpmfileState rpmfiFState(rpmfi fi)
 
 rpmfileState rpmfiSetFState(rpmfi fi, rpmfileState fstate)
 {
-    uint32_t ofstate = 0;
+    rpmuint32_t ofstate = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fstates != NULL) {
 	    ofstate = fi->fstates[fi->i];
 	    fi->fstates[fi->i] = fstate;
@@ -235,7 +259,7 @@ const unsigned char * rpmfiDigest(rpmfi fi, int * algop, size_t * lenp)
 {
     unsigned char * digest = NULL;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->digests != NULL) {
 	    digest = fi->digests + (fi->digestlen * fi->i);
 	    if (algop != NULL)
@@ -252,49 +276,49 @@ const char * rpmfiFLink(rpmfi fi)
 {
     const char * flink = NULL;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->flinks != NULL)
 	    flink = fi->flinks[fi->i];
     }
     return flink;
 }
 
-uint32_t rpmfiFSize(rpmfi fi)
+rpmuint32_t rpmfiFSize(rpmfi fi)
 {
-    uint32_t fsize = 0;
+    rpmuint32_t fsize = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fsizes != NULL)
 	    fsize = fi->fsizes[fi->i];
     }
     return fsize;
 }
 
-uint16_t rpmfiFRdev(rpmfi fi)
+rpmuint16_t rpmfiFRdev(rpmfi fi)
 {
-    uint16_t frdev = 0;
+    rpmuint16_t frdev = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->frdevs != NULL)
 	    frdev = fi->frdevs[fi->i];
     }
     return frdev;
 }
 
-uint32_t rpmfiFInode(rpmfi fi)
+rpmuint32_t rpmfiFInode(rpmfi fi)
 {
-    uint32_t finode = 0;
+    rpmuint32_t finode = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->finodes != NULL)
 	    finode = fi->finodes[fi->i];
     }
     return finode;
 }
 
-uint32_t rpmfiColor(rpmfi fi)
+rpmuint32_t rpmfiColor(rpmfi fi)
 {
-    uint32_t color = 0;
+    rpmuint32_t color = 0;
 
     if (fi != NULL)
 	/* XXX ignore all but lsnibble for now. */
@@ -302,11 +326,11 @@ uint32_t rpmfiColor(rpmfi fi)
     return color;
 }
 
-uint32_t rpmfiFColor(rpmfi fi)
+rpmuint32_t rpmfiFColor(rpmfi fi)
 {
-    uint32_t fcolor = 0;
+    rpmuint32_t fcolor = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fcolors != NULL)
 	    /* XXX ignore all but lsnibble for now. */
 	    fcolor = (fi->fcolors[fi->i] & 0x0f);
@@ -317,11 +341,10 @@ uint32_t rpmfiFColor(rpmfi fi)
 const char * rpmfiFClass(rpmfi fi)
 {
     const char * fclass = NULL;
-    int cdictx;
 
-    if (fi != NULL && fi->fcdictx != NULL && fi->i >= 0 && fi->i < fi->fc) {
-	cdictx = fi->fcdictx[fi->i];
-	if (fi->cdict != NULL && cdictx >= 0 && cdictx < fi->ncdict)
+    if (fi != NULL && fi->fcdictx != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
+	int cdictx = fi->fcdictx[fi->i];
+	if (fi->cdict != NULL && cdictx >= 0 && cdictx < (int)fi->ncdict)
 	    fclass = fi->cdict[cdictx];
     }
     return fclass;
@@ -331,25 +354,25 @@ const char * rpmfiFContext(rpmfi fi)
 {
     const char * fcontext = NULL;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fcontexts != NULL)
 	    fcontext = fi->fcontexts[fi->i];
     }
     return fcontext;
 }
 
-uint32_t rpmfiFDepends(rpmfi fi, const uint32_t ** fddictp)
+rpmuint32_t rpmfiFDepends(rpmfi fi, const rpmuint32_t ** fddictp)
 {
     int fddictx = -1;
     int fddictn = 0;
-    const uint32_t * fddict = NULL;
+    const rpmuint32_t * fddict = NULL;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fddictn != NULL)
 	    fddictn = fi->fddictn[fi->i];
 	if (fddictn > 0 && fi->fddictx != NULL)
 	    fddictx = fi->fddictx[fi->i];
-	if (fi->ddict != NULL && fddictx >= 0 && (fddictx+fddictn) <= fi->nddict)
+	if (fi->ddict != NULL && fddictx >= 0 && (fddictx+fddictn) <= (int)fi->nddict)
 	    fddict = fi->ddict + fddictx;
     }
 /*@-dependenttrans -onlytrans @*/
@@ -359,18 +382,18 @@ uint32_t rpmfiFDepends(rpmfi fi, const uint32_t ** fddictp)
     return fddictn;
 }
 
-uint32_t rpmfiFNlink(rpmfi fi)
+rpmuint32_t rpmfiFNlink(rpmfi fi)
 {
-    uint32_t nlink = 0;
+    rpmuint32_t nlink = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	/* XXX rpm-2.3.12 has not RPMTAG_FILEINODES */
 	if (fi->finodes && fi->frdevs) {
-	    uint32_t finode = fi->finodes[fi->i];
-	    uint16_t frdev = fi->frdevs[fi->i];
+	    rpmuint32_t finode = fi->finodes[fi->i];
+	    rpmuint16_t frdev = fi->frdevs[fi->i];
 	    int j;
 
-	    for (j = 0; j < fi->fc; j++) {
+	    for (j = 0; j < (int)fi->fc; j++) {
 		if (fi->frdevs[j] == frdev && fi->finodes[j] == finode)
 		    nlink++;
 	    }
@@ -379,11 +402,11 @@ uint32_t rpmfiFNlink(rpmfi fi)
     return nlink;
 }
 
-uint32_t rpmfiFMtime(rpmfi fi)
+rpmuint32_t rpmfiFMtime(rpmfi fi)
 {
-    uint32_t fmtime = 0;
+    rpmuint32_t fmtime = 0;
 
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fmtimes != NULL)
 	    fmtime = fi->fmtimes[fi->i];
     }
@@ -395,7 +418,7 @@ const char * rpmfiFUser(rpmfi fi)
     const char * fuser = NULL;
 
     /* XXX add support for ancient RPMTAG_FILEUIDS? */
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fuser != NULL)
 	    fuser = fi->fuser[fi->i];
     }
@@ -407,19 +430,39 @@ const char * rpmfiFGroup(rpmfi fi)
     const char * fgroup = NULL;
 
     /* XXX add support for ancient RPMTAG_FILEGIDS? */
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+    if (fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
 	if (fi->fgroup != NULL)
 	    fgroup = fi->fgroup[fi->i];
     }
     return fgroup;
 }
 
+void * rpmfiExclude(const rpmfi fi)
+{
+    return (fi != NULL ? fi->exclude : NULL);
+}
+
+int rpmfiNExclude(const rpmfi fi)
+{
+    return (fi != NULL ? fi->nexclude : 0);
+}
+
+void * rpmfiInclude(const rpmfi fi)
+{
+    return (fi != NULL ? fi->include : NULL);
+}
+
+int rpmfiNInclude(const rpmfi fi)
+{
+    return (fi != NULL ? fi->ninclude : 0);
+}
+
 int rpmfiNext(rpmfi fi)
 {
     int i = -1;
 
     if (fi != NULL && ++fi->i >= 0) {
-	if (fi->i < fi->fc) {
+	if (fi->i < (int)fi->fc) {
 	    i = fi->i;
 	    if (fi->dil != NULL)
 		fi->j = fi->dil[fi->i];
@@ -439,7 +482,7 @@ fprintf(stderr, "*** fi %p\t%s[%d] %s%s\n", fi, (fi->Type ? fi->Type : "?Type?")
 rpmfi rpmfiInit(rpmfi fi, int fx)
 {
     if (fi != NULL) {
-	if (fx >= 0 && fx < fi->fc) {
+	if (fx >= 0 && fx < (int)fi->fc) {
 	    fi->i = fx - 1;
 	    fi->j = -1;
 	}
@@ -455,7 +498,7 @@ int rpmfiNextD(rpmfi fi)
     int j = -1;
 
     if (fi != NULL && ++fi->j >= 0) {
-	if (fi->j < fi->dc)
+	if (fi->j < (int)fi->dc)
 	    j = fi->j;
 	else
 	    fi->j = -1;
@@ -473,7 +516,7 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, (fi->Type ? fi->Type : "?Type?"), j);
 rpmfi rpmfiInitD(rpmfi fi, int dx)
 {
     if (fi != NULL) {
-	if (dx >= 0 && dx < fi->fc)
+	if (dx >= 0 && dx < (int)fi->fc)
 	    fi->j = dx - 1;
 	else
 	    fi = NULL;
@@ -511,7 +554,7 @@ const char * rpmfiFtstring (rpmFileTypes ft)
  * @param mode		file mode bits (from header)
  * @return		file type
  */
-static rpmFileTypes rpmfiWhatis(uint16_t mode)
+static rpmFileTypes rpmfiWhatis(rpmuint16_t mode)
 	/*@*/
 {
     if (S_ISDIR(mode))	return XDIR;
@@ -559,7 +602,7 @@ int rpmfiCompare(const rpmfi afi, const rpmfi bfi)
     return 0;
 }
 
-fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
+int rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 {
     const char * fn = rpmfiFN(nfi);
     int newFlags = rpmfiFFlags(nfi);
@@ -582,7 +625,7 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 	}
     }
 
-    diskWhat = rpmfiWhatis((uint16_t)sb.st_mode);
+    diskWhat = rpmfiWhatis((rpmuint16_t)sb.st_mode);
     dbWhat = rpmfiWhatis(rpmfiFMode(ofi));
     newWhat = rpmfiWhatis(rpmfiFMode(nfi));
 
@@ -632,7 +675,7 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 	const char * oFLink, * nFLink;
 	oFLink = rpmfiFLink(ofi);
 	if (diskWhat == LINK) {
-	    if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
+	    if (Readlink(fn, buffer, sizeof(buffer) - 1) == -1)
 		return FA_CREATE;	/* assume file has been removed */
 	    buffer[sizeof(buffer)-1] = '\0';
 	    if (oFLink && !strcmp(oFLink, buffer))
@@ -678,7 +721,7 @@ const char * rpmfiTypeString(rpmfi fi)
  */
 static
 Header relocateFileList(const rpmts ts, rpmfi fi,
-		Header origH, fileAction * actions)
+		Header origH, iosmFileAction * actions)
 	/*@globals rpmGlobalMacroContext, h_errno,
 		internalState @*/
 	/*@modifies ts, fi, origH, actions, rpmGlobalMacroContext,
@@ -695,21 +738,21 @@ Header relocateFileList(const rpmts ts, rpmfi fi,
     int numValid;
     const char ** baseNames;
     const char ** dirNames;
-    uint32_t * dirIndexes;
-    uint32_t fileCount;
-    uint32_t dirCount;
-    uint32_t mydColor = rpmExpandNumeric("%{?_autorelocate_dcolor}");
-    uint32_t * fFlags = NULL;
-    uint32_t * fColors = NULL;
-    uint32_t * dColors = NULL;
-    uint16_t * fModes = NULL;
+    rpmuint32_t * dirIndexes;
+    rpmuint32_t fileCount;
+    rpmuint32_t dirCount;
+    rpmuint32_t mydColor = rpmExpandNumeric("%{?_autorelocate_dcolor}");
+    rpmuint32_t * fFlags = NULL;
+    rpmuint32_t * fColors = NULL;
+    rpmuint32_t * dColors = NULL;
+    rpmuint16_t * fModes = NULL;
     Header h;
     int nrelocated = 0;
-    int fileAlloced = 0;
+    size_t fileAlloced = 0;
     char * fn = NULL;
     int haveRelocatedFile = 0;
     int reldel = 0;
-    int len;
+    size_t len;
     int i, j;
     int xx;
 
@@ -746,10 +789,14 @@ assert(p != NULL);
 	    validRelocations = _free(validRelocations);
 	}
 	/* XXX FIXME multilib file actions need to be checked. */
+/*@-castexpose@*/
 	return headerLink(origH);
+/*@=castexpose@*/
     }
 
+/*@-castexpose@*/
     h = headerLink(origH);
+/*@=castexpose@*/
 
     relocations = alloca(sizeof(*relocations) * numRelocations);
 
@@ -795,7 +842,7 @@ assert(p != NULL);
 		ps = rpmpsFree(ps);
 	    }
 	    del =
-		strlen(relocations[i].newPath) - strlen(relocations[i].oldPath);
+		(int)strlen(relocations[i].newPath) - (int)strlen(relocations[i].oldPath);
 	    /*@=nullpass@*/
 
 	    if (del > reldel)
@@ -910,7 +957,7 @@ assert(p != NULL);
 
     for (i = fileCount - 1; i >= 0; i--) {
 	rpmFileTypes ft;
-	int fnlen;
+	size_t fnlen;
 
 	len = reldel +
 		strlen(dirNames[dirIndexes[i]]) + strlen(baseNames[i]) + 1;
@@ -925,7 +972,7 @@ assert(fn != NULL);		/* XXX can't happen */
 
 if (fColors != NULL) {
 /* XXX pkgs may not have unique dirNames, so color all dirNames that match. */
-for (j = 0; j < dirCount; j++) {
+for (j = 0; j < (int)dirCount; j++) {
 if (strcmp(dirNames[dirIndexes[i]], dirNames[j])) /*@innercontinue@*/ continue;
 dColors[j] |= fColors[i];
 }
@@ -968,7 +1015,7 @@ dColors[j] |= fColors[i];
 	if (relocations[j].newPath == NULL) {
 	    if (ft == XDIR) {
 		/* Start with the parent, looking for directory to exclude. */
-		for (j = dirIndexes[i]; j < dirCount; j++) {
+		for (j = dirIndexes[i]; j < (int)dirCount; j++) {
 		    len = strlen(dirNames[j]) - 1;
 		    while (len > 0 && dirNames[j][len-1] == '/') len--;
 		    if (fnlen != len)
@@ -1009,7 +1056,7 @@ dColors[j] |= fColors[i];
 	}
 
 	/* Does this directory already exist in the directory list? */
-	for (j = 0; j < dirCount; j++) {
+	for (j = 0; j < (int)dirCount; j++) {
 	    if (fnlen != strlen(dirNames[j]))
 		/*@innercontinue@*/ continue;
 	    if (strncmp(fn, dirNames[j], fnlen))
@@ -1017,7 +1064,7 @@ dColors[j] |= fColors[i];
 	    /*@innerbreak@*/ break;
 	}
 	
-	if (j < dirCount) {
+	if (j < (int)dirCount) {
 	    dirIndexes[i] = j;
 	    continue;
 	}
@@ -1028,7 +1075,7 @@ dColors[j] |= fColors[i];
 
 	    haveRelocatedFile = 1;
 	    newDirList = xmalloc((dirCount + 1) * sizeof(*newDirList));
-	    for (j = 0; j < dirCount; j++)
+	    for (j = 0; j < (int)dirCount; j++)
 		newDirList[j] = alloca_strdup(dirNames[j]);
 	    dirNames = _free(dirNames);
 	    dirNames = newDirList;
@@ -1116,7 +1163,9 @@ dColors[j] |= fColors[i];
 	xx = headerMod(h, he, 0);
 	fi->bnl = _free(fi->bnl);
 	xx = headerGet(h, he, 0);
+/*@-dependenttrans@*/
 	fi->bnl = he->p.argv;
+/*@=dependenttrans@*/
 	fi->fc = he->c;
 
 	he->tag = RPMTAG_DIRNAMES;
@@ -1136,7 +1185,9 @@ dColors[j] |= fColors[i];
 	xx = headerMod(h, he, 0);
 	fi->dil = _free(fi->dil);
 	xx = headerGet(h, he, 0);
+/*@-dependenttrans@*/
 	fi->dil = he->p.ui32p;
+/*@=dependenttrans@*/
     }
 
     baseNames = _free(baseNames);
@@ -1150,20 +1201,27 @@ dColors[j] |= fColors[i];
     fn = _free(fn);
 /*@=dependenttrans@*/
 
+/*@-retalias@*/
     return h;
+/*@=retalias@*/
 }
 
-rpmfi rpmfiFree(rpmfi fi)
+int rpmfiSetHeader(rpmfi fi, Header h)
 {
-    if (fi == NULL) return NULL;
+    if (fi->h != NULL)
+	(void)headerFree(fi->h);
+    fi->h = NULL;
+/*@-assignexpose -castexpose @*/
+    if (h != NULL)
+	fi->h = headerLink(h);
+/*@=assignexpose =castexpose @*/
+    return 0;
+}
 
-    if (fi->nrefs > 1)
-	return rpmfiUnlink(fi, fi->Type);
-
-/*@-modfilesys@*/
-if (_rpmfi_debug < 0)
-fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, fi->Type, fi->fc);
-/*@=modfilesys@*/
+static void rpmfiFini(void * _fi)
+	/*@modifies *_fi @*/
+{
+    rpmfi fi = _fi;
 
     /* Free pre- and post-transaction script and interpreter strings. */
     fi->pretrans = _free(fi->pretrans);
@@ -1205,7 +1263,12 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, fi->Type, fi->fc);
 	fi->fddictn = _free(fi->fddictn);
     }
 
+/*@-globs@*/	/* Avoid rpmGlobalMacroContext */
     fi->fsm = freeFSM(fi->fsm);
+/*@=globs@*/
+
+    fi->exclude = mireFreeAll(fi->exclude, fi->nexclude);
+    fi->include = mireFreeAll(fi->include, fi->ninclude);
 
     fi->fn = _free(fi->fn);
     fi->apath = _free(fi->apath);
@@ -1219,15 +1282,25 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, fi->Type, fi->fc);
     fi->actions = _free(fi->actions);
     fi->replacedSizes = _free(fi->replacedSizes);
 
-    fi->h = headerFree(fi->h);
+    (void)headerFree(fi->h);
+    fi->h = NULL;
+}
 
-    /*@-nullstate -refcounttrans -usereleased@*/
-    (void) rpmfiUnlink(fi, fi->Type);
-    memset(fi, 0, sizeof(*fi));		/* XXX trash and burn */
-    fi = _free(fi);
-    /*@=nullstate =refcounttrans =usereleased@*/
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _rpmfiPool;
 
-    return NULL;
+static rpmfi rpmfiGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _rpmfiPool, fileSystem, internalState @*/
+	/*@modifies pool, _rpmfiPool, fileSystem, internalState @*/
+{
+    rpmfi fi;
+
+    if (_rpmfiPool == NULL) {
+	_rpmfiPool = rpmioNewPool("fi", sizeof(*fi), -1, _rpmfi_debug,
+			NULL, NULL, rpmfiFini);
+	pool = _rpmfiPool;
+    }
+    return (rpmfi) rpmioGetPool(pool, sizeof(*fi));
 }
 
 /**
@@ -1257,16 +1330,18 @@ static inline unsigned char nibble(char c)
     xx = headerGet((_h), he, 0); \
     _data = he->p.ptr;
 
-rpmfi rpmfiNew(const rpmts ts, Header h, rpmTag tagN, int flags)
+rpmfi rpmfiNew(const void * _ts, Header h, rpmTag tagN, int flags)
 {
+/*@-castexpose@*/
+    const rpmts ts = (const rpmts) _ts;
+/*@=castexpose@*/
     int scareMem = (flags & 0x1);
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     rpmte p;
     rpmfi fi = NULL;
     const char * Type;
-    int dnlmax, bnlmax;
     unsigned char * t;
-    int len;
+    pgpHashAlgo dalgo;
     int xx;
     int i;
 
@@ -1278,7 +1353,7 @@ assert(scareMem == 0);		/* XXX always allocate memory */
 	goto exit;
     }
 
-    fi = xcalloc(1, sizeof(*fi));
+    fi = rpmfiGetPool(_rpmfiPool);
     if (fi == NULL)	/* XXX can't happen */
 	goto exit;
 
@@ -1290,12 +1365,13 @@ assert(scareMem == 0);		/* XXX always allocate memory */
     fi->h = NULL;
     fi->isSource =
 	(headerIsEntry(h, RPMTAG_SOURCERPM) == 0 &&
+	 headerIsEntry(h, RPMTAG_RPMVERSION) != 0 &&
 	 headerIsEntry(h, RPMTAG_ARCH) != 0);
 
     if (fi->fsm == NULL)
 	fi->fsm = newFSM();
 
-    fi->fsm->repackaged = (headerIsEntry(h, RPMTAG_REMOVETID) ? 1 : 0);
+    ((FSM_t)fi->fsm)->repackaged = (headerIsEntry(h, RPMTAG_REMOVETID) ? 1 : 0);
 
     /* 0 means unknown */
     he->tag = RPMTAG_ARCHIVESIZE;
@@ -1314,6 +1390,11 @@ assert(scareMem == 0);		/* XXX always allocate memory */
 
     he->tag = RPMTAG_BASENAMES;
     xx = headerGet(h, he, 0);
+    /* XXX 3.0.x SRPM's can be used, relative fn's at RPMTAG_OLDFILENAMES. */
+    if (xx == 0 && fi->isSource) {
+	he->tag = RPMTAG_OLDFILENAMES;
+	xx = headerGet(h, he, 0);
+    }
     fi->bnl = he->p.argv;
     fi->fc = he->c;
     if (!xx) {
@@ -1323,7 +1404,15 @@ assert(scareMem == 0);		/* XXX always allocate memory */
     }
     _fdupedata(h, RPMTAG_DIRNAMES, fi->dnl);
     fi->dc = he->c;
-    _fdupedata(h, RPMTAG_DIRINDEXES, fi->dil);
+    /* XXX 3.0.x SRPM's can be used, relative fn's at RPMTAG_OLDFILENAMES. */
+    if (fi->dc == 0 && fi->isSource) {
+	fi->dc = 1;
+	fi->dnl = xcalloc(3, sizeof(*fi->dnl));
+	fi->dnl[0] = (const char *)&fi->dnl[2];
+	fi->dil = xcalloc(fi->fc, sizeof(*fi->dil));
+    } else {
+	_fdupedata(h, RPMTAG_DIRINDEXES, fi->dil);
+    }
     _fdupedata(h, RPMTAG_FILEMODES, fi->fmodes);
     _fdupedata(h, RPMTAG_FILEFLAGS, fi->fflags);
     _fdupedata(h, RPMTAG_FILEVERIFYFLAGS, fi->vflags);
@@ -1332,7 +1421,7 @@ assert(scareMem == 0);		/* XXX always allocate memory */
     _fdupedata(h, RPMTAG_FILECOLORS, fi->fcolors);
     fi->color = 0;
     if (fi->fcolors != NULL)
-    for (i = 0; i < fi->fc; i++)
+    for (i = 0; i < (int)fi->fc; i++)
 	fi->color |= fi->fcolors[i];
     _fdupedata(h, RPMTAG_CLASSDICT, fi->cdict);
     fi->ncdict = he->c;
@@ -1353,48 +1442,58 @@ assert(scareMem == 0);		/* XXX always allocate memory */
 if (fi->actions == NULL)
 	fi->actions = xcalloc(fi->fc, sizeof(*fi->actions));
 
-    /* XXX TR_REMOVED needs CPIO_MAP_{ABSOLUTE,ADDDOT} CPIO_ALL_HARDLINKS */
+    /* XXX TR_REMOVED needs IOSM_MAP_{ABSOLUTE,ADDDOT} IOSM_ALL_HARDLINKS */
     fi->mapflags =
-		CPIO_MAP_PATH | CPIO_MAP_MODE | CPIO_MAP_UID | CPIO_MAP_GID;
+		IOSM_MAP_PATH | IOSM_MAP_MODE | IOSM_MAP_UID | IOSM_MAP_GID;
 
     _fdupedata(h, RPMTAG_FILELINKTOS, fi->flinks);
     _fdupedata(h, RPMTAG_FILELANGS, fi->flangs);
 
-    fi->digestalgo = PGPHASHALGO_MD5;
-    fi->digestlen = 16;
+    dalgo = PGPHASHALGO_ERROR;
     fi->fdigestalgos = NULL;
     _fdupedata(h, RPMTAG_FILEDIGESTALGOS, fi->fdigestalgos);
     if (fi->fdigestalgos) {
-	int dalgo = 0;
 	/* XXX Insure that all algorithms are either 0 or constant. */
-	for (i = 0; i < fi->fc; i++) {
+	for (i = 0; i < (int)fi->fc; i++) {
 	    if (fi->fdigestalgos[i] == 0)
 		continue;
-	    if (dalgo == 0)
-		dalgo = fi->fdigestalgos[i];
+	    if (dalgo == PGPHASHALGO_ERROR)
+		dalgo = (fi->fdigestalgos[i] & 0xff);
 	    else
-assert(dalgo == fi->fdigestalgos[i]);
-	}
-	fi->digestalgo = dalgo;
-	switch (dalgo) {
-	case PGPHASHALGO_MD5:		fi->digestlen = 128/8;	break;
-	case PGPHASHALGO_SHA1:		fi->digestlen = 160/8;	break;
-	case PGPHASHALGO_RIPEMD128:	fi->digestlen = 128/8;	break;
-	case PGPHASHALGO_RIPEMD160:	fi->digestlen = 160/8;	break;
-	case PGPHASHALGO_SHA256:	fi->digestlen = 256/8;	break;
-	case PGPHASHALGO_SHA384:	fi->digestlen = 384/8;	break;
-	case PGPHASHALGO_SHA512:	fi->digestlen = 512/8;	break;
-	case PGPHASHALGO_CRC32:		fi->digestlen = 32/8;	break;
+assert(dalgo == (pgpHashAlgo)fi->fdigestalgos[i]);
 	}
 	fi->fdigestalgos = _free(fi->fdigestalgos);
+    } else {
+	he->tag = RPMTAG_FILEDIGESTALGO;
+	xx = headerGet(h, he, 0);
+	if (xx)
+	    dalgo = he->p.ui32p[0];
+	he->p.ptr = _free(he->p.ptr);
     }
 
-    _fdupedata(h, RPMTAG_FILEDIGESTS, fi->fdigests);
+    switch (dalgo) {
+    default: dalgo = PGPHASHALGO_MD5;	fi->digestlen = 128/8; break;
+    case PGPHASHALGO_MD2:	fi->digestlen = 128/8;	break;
+    case PGPHASHALGO_MD5:	fi->digestlen = 128/8;	break;
+    case PGPHASHALGO_SHA1:	fi->digestlen = 160/8;	break;
+    case PGPHASHALGO_RIPEMD128:	fi->digestlen = 128/8;	break;
+    case PGPHASHALGO_RIPEMD160:	fi->digestlen = 160/8;	break;
+    case PGPHASHALGO_RIPEMD256:	fi->digestlen = 256/8;	break;
+    case PGPHASHALGO_RIPEMD320:	fi->digestlen = 320/8;	break;
+    case PGPHASHALGO_SHA224:	fi->digestlen = 224/8;	break;
+    case PGPHASHALGO_SHA256:	fi->digestlen = 256/8;	break;
+    case PGPHASHALGO_SHA384:	fi->digestlen = 384/8;	break;
+    case PGPHASHALGO_SHA512:	fi->digestlen = 512/8;	break;
+    case PGPHASHALGO_CRC32:	fi->digestlen = 32/8;	break;
+    }
+    fi->digestalgo = dalgo;
+
     fi->digests = NULL;
+    _fdupedata(h, RPMTAG_FILEDIGESTS, fi->fdigests);
     if (fi->fdigests) {
 	t = xmalloc(fi->fc * fi->digestlen);
 	fi->digests = t;
-	for (i = 0; i < fi->fc; i++) {
+	for (i = 0; i < (int)fi->fc; i++) {
 	    const char * fdigests;
 	    int j;
 
@@ -1404,13 +1503,13 @@ assert(dalgo == fi->fdigestalgos[i]);
 		t += fi->digestlen;
 		continue;
 	    }
-	    for (j = 0; j < fi->digestlen; j++, t++, fdigests += 2)
+	    for (j = 0; j < (int)fi->digestlen; j++, t++, fdigests += 2)
 		*t = (nibble(fdigests[0]) << 4) | nibble(fdigests[1]);
 	}
 	fi->fdigests = _free(fi->fdigests);
     }
 
-    /* XXX TR_REMOVED doesn;t need fmtimes, frdevs, finodes, or fcontexts */
+    /* XXX TR_REMOVED doesn't need fmtimes, frdevs, finodes, or fcontexts */
     _fdupedata(h, RPMTAG_FILEMTIMES, fi->fmtimes);
     _fdupedata(h, RPMTAG_FILERDEVS, fi->frdevs);
     _fdupedata(h, RPMTAG_FILEINODES, fi->finodes);
@@ -1468,11 +1567,13 @@ assert(dalgo == fi->fdigestalgos[i]);
 if (fi->actions == NULL)
 	fi->actions = xcalloc(fi->fc, sizeof(*fi->actions));
 	/*@-compdef@*/ /* FIX: fi->digests undefined */
-	foo = relocateFileList(ts, fi, h, fi->actions);
+	foo = relocateFileList(ts, fi, h, (iosmFileAction *) fi->actions);
 	/*@=compdef@*/
-	fi->h = headerFree(fi->h);
+	(void)headerFree(fi->h);
+	fi->h = NULL;
 	fi->h = headerLink(foo);
-	foo = headerFree(foo);
+	(void)headerFree(foo);
+	foo = NULL;
     }
 
     if (fi->isSource && fi->dc == 1 && *fi->dnl[0] == '\0') {
@@ -1492,13 +1593,13 @@ if (fi->actions == NULL)
 	fi->dnl = _free(fi->dnl);
 	fi->dc = 4;
 	nb = fi->dc * sizeof(*av);
-	for (i = 0; i < fi->dc; i++)
+	for (i = 0; i < (int)fi->dc; i++)
 	    nb += strlen(av[i]) + sizeof("/");
 
 	fi->dnl = xmalloc(nb);
 	te = (char *) (&fi->dnl[fi->dc]);
 	*te = '\0';
-	for (i = 0; i < fi->dc; i++) {
+	for (i = 0; i < (int)fi->dc; i++) {
 	    fi->dnl[i] = te;
 	    te = stpcpy( stpcpy(te, av[i]), "/");
 	    *te++ = '\0';
@@ -1506,7 +1607,7 @@ if (fi->actions == NULL)
 	av = argvFree(av);
 
 	/* Map basenames to appropriate directories. */
-	for (i = 0; i < fi->fc; i++) {
+	for (i = 0; i < (int)fi->fc; i++) {
 	    if (fi->fflags[i] & RPMFILE_SOURCE)
 		fi->dil[i] = 0;
 	    else if (fi->fflags[i] & RPMFILE_SPECFILE)
@@ -1525,20 +1626,16 @@ if (fi->actions == NULL)
     }
 
     if (!scareMem)
-	fi->h = headerFree(fi->h);
+	(void)headerFree(fi->h);
+    fi->h = NULL;
 
-    dnlmax = -1;
-    for (i = 0; i < fi->dc; i++) {
-	if ((len = strlen(fi->dnl[i])) > dnlmax)
-	    dnlmax = len;
-    }
-    bnlmax = -1;
-    for (i = 0; i < fi->fc; i++) {
-	if ((len = strlen(fi->bnl[i])) > bnlmax)
-	    bnlmax = len;
-    }
-    fi->fnlen = dnlmax + bnlmax + 1;
     fi->fn = NULL;
+    fi->fnlen = 0;
+    for (i = 0; i < (int)fi->fc; i++) {
+	size_t fnlen = strlen(fi->dnl[fi->dil[i]]) + strlen(fi->bnl[i]);
+	if (fnlen > fi->fnlen)
+	    fi->fnlen = fnlen;
+    }
 
     fi->dperms = 0755;
     fi->fperms = 0644;
@@ -1554,8 +1651,160 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, Type, (fi ? fi->fc : 0));
     /*@=compdef =nullstate@*/
 }
 
+int rpmfiAddRelocation(rpmRelocation * relp, int * nrelp,
+		const char * oldPath, const char * newPath)
+{
+/*@-unqualifiedtrans@*/
+    *relp = xrealloc(*relp, sizeof(**relp) * ((*nrelp) + 1));
+/*@=unqualifiedtrans@*/
+    (*relp)[*nrelp].oldPath = (oldPath ? xstrdup(oldPath) : NULL);
+    (*relp)[*nrelp].newPath = (newPath ? xstrdup(newPath) : NULL);
+    (*nrelp)++;
+    return 0;
+}
+
+rpmRelocation rpmfiFreeRelocations(rpmRelocation relocs)
+{
+    if (relocs) {
+	rpmRelocation r;
+	for (r = relocs; (r->oldPath || r->newPath); r++) {
+	    r->oldPath = _free(r->oldPath);
+	    r->newPath = _free(r->newPath);
+	}
+	relocs = _free(relocs);
+    }
+    return NULL;
+}
+
+rpmRelocation rpmfiDupeRelocations(rpmRelocation relocs, int * nrelocsp)
+{
+    rpmRelocation newr = NULL;
+    int nrelocs = 0;
+
+    if (relocs) {
+	rpmRelocation r;
+	int i;
+
+	for (r = relocs; r->oldPath || r->newPath; r++)
+	    nrelocs++;
+	newr = xmalloc((nrelocs + 1) * sizeof(*relocs));
+
+	for (i = 0, r = relocs; r->oldPath || r->newPath; i++, r++) {
+	    newr[i].oldPath = r->oldPath ? xstrdup(r->oldPath) : NULL;
+	    newr[i].newPath = r->newPath ? xstrdup(r->newPath) : NULL;
+	}
+	newr[i].oldPath = NULL;
+	newr[i].newPath = NULL;
+    }
+    if (nrelocsp)
+	*nrelocsp = nrelocs;
+    return newr;
+}
+
+int rpmfiFStat(rpmfi fi, struct stat * st)
+{
+    int rc = -1;
+
+    if (st != NULL && fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc) {
+	memset(st, 0, sizeof(*st));
+	st->st_dev =
+	st->st_rdev = fi->frdevs[fi->i];
+	st->st_ino = fi->finodes[fi->i];
+	st->st_mode = fi->fmodes[fi->i];
+	st->st_nlink = rpmfiFNlink(fi) + (int)S_ISDIR(st->st_mode);
+	if (unameToUid(fi->fuser[fi->i], &st->st_uid) == -1)
+	    st->st_uid = 0;		/* XXX */
+	if (gnameToGid(fi->fgroup[fi->i], &st->st_gid) == -1)
+	    st->st_gid = 0;		/* XXX */
+	st->st_size = fi->fsizes[fi->i];
+	st->st_blksize = 4 * 1024;	/* XXX */
+	st->st_blocks = (st->st_size + (st->st_blksize - 1)) / st->st_blksize;
+	st->st_atime =
+	st->st_ctime =
+	st->st_mtime = fi->fmtimes[fi->i];
+	rc = 0;
+    }
+
+    return rc;
+}
+
+int rpmfiStat(rpmfi fi, const char * path, struct stat * st)
+{
+    size_t pathlen = strlen(path);
+    int rc = -1;
+    int i;
+
+    while (pathlen > 0 && path[pathlen-1] == '/')
+	pathlen--;
+
+    /* If not actively iterating, initialize. */
+    if (!(fi != NULL && fi->i >= 0 && fi->i < (int)fi->fc))
+	fi = rpmfiInit(fi, 0);
+
+    while ((i = rpmfiNext(fi)) >= 0) {
+	const char * fn = rpmfiFN(fi);
+	size_t fnlen = strlen(fn);
+
+	if (pathlen != fnlen || strncmp(path, fn, fnlen))
+	    continue;
+	rc = rpmfiFStat(fi, st);
+	break;
+    }
+
+/*@-modfilesys@*/
+if (_rpmfi_debug)
+fprintf(stderr, "*** rpmfiStat(%p, %s, %p) rc %d\n", fi, path, st, rc);
+/*@=modfilesys@*/
+
+    return rc;
+}
+
+DIR * rpmfiOpendir(rpmfi fi, const char * name)
+{
+    const char * dn = name;
+    size_t dnlen = strlen(dn);
+    const char ** fnames = NULL;
+    rpmuint16_t * fmodes = NULL;
+    DIR * dir;
+    int xx;
+    int i, j;
+
+    j = 0;
+    fmodes = xcalloc(fi->fc, sizeof(*fmodes));
+
+    /* XXX todo full iteration is pig slow, fi->dil can be used for speedup. */
+    fi = rpmfiInit(fi, 0);
+    while ((i = rpmfiNext(fi)) >= 0) {
+	const char * fn = rpmfiFN(fi);
+	size_t fnlen = strlen(fn);
+
+	if (fnlen <= dnlen)
+	    continue;
+	if (strncmp(dn, fn, dnlen) || fn[dnlen] != '/')
+	    continue;
+
+	/* XXX todo basename, or orphandir/.../basname, needs to be used. */
+	/* Trim the directory part of the name. */
+	xx = argvAdd(&fnames, fn + dnlen + 1);
+	fmodes[j++] = fi->fmodes[i];
+    }
+
+    /* Add "." & ".." to the argv array. */
+    dir = (DIR *) avOpendir(name, fnames, fmodes);
+
+    fnames = argvFree(fnames);
+    fmodes = _free(fmodes);
+
+/*@-modfilesys +voidabstract @*/
+if (_rpmfi_debug)
+fprintf(stderr, "*** rpmfiOpendir(%p, %s) dir %p\n", fi, name, dir);
+/*@=modfilesys =voidabstract @*/
+
+    return dir;
+}
+
 void rpmfiBuildFClasses(Header h,
-	/*@out@*/ const char *** fclassp, /*@out@*/ uint32_t * fcp)
+	/*@out@*/ const char *** fclassp, /*@out@*/ rpmuint32_t * fcp)
 {
     int scareMem = 0;
     rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
@@ -1607,7 +1856,7 @@ exit:
 }
 
 void rpmfiBuildFContexts(Header h,
-	/*@out@*/ const char *** fcontextp, /*@out@*/ uint32_t * fcp)
+	/*@out@*/ const char *** fcontextp, /*@out@*/ rpmuint32_t * fcp)
 {
     int scareMem = 0;
     rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
@@ -1659,7 +1908,7 @@ exit:
 }
 
 void rpmfiBuildFSContexts(Header h,
-	/*@out@*/ const char *** fcontextp, /*@out@*/ uint32_t * fcp)
+	/*@out@*/ const char *** fcontextp, /*@out@*/ rpmuint32_t * fcp)
 {
     int scareMem = 0;
     rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
@@ -1685,7 +1934,7 @@ void rpmfiBuildFSContexts(Header h,
     if (fi != NULL)
     while (rpmfiNext(fi) >= 0) {
 	const char *fn;
-	security_context_t scon;
+	security_context_t scon = NULL;
 
 	fn = rpmfiFN(fi);
 	fcnb[ac] = lgetfilecon(fn, &scon);
@@ -1727,7 +1976,7 @@ exit:
 }
 
 void rpmfiBuildREContexts(Header h,
-	/*@out@*/ const char *** fcontextp, /*@out@*/ uint32_t * fcp)
+	/*@out@*/ const char *** fcontextp, /*@out@*/ rpmuint32_t * fcp)
 {
     int scareMem = 0;
     rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
@@ -1812,7 +2061,7 @@ exit:
 }
 
 void rpmfiBuildFDeps(Header h, rpmTag tagN,
-	/*@out@*/ const char *** fdepsp, /*@out@*/ uint32_t * fcp)
+	/*@out@*/ const char *** fdepsp, /*@out@*/ rpmuint32_t * fcp)
 {
     int scareMem = 0;
     rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
@@ -1824,7 +2073,7 @@ void rpmfiBuildFDeps(Header h, rpmTag tagN,
     char deptype = 'R';
     char mydt;
     const char * DNEVR;
-    const uint32_t * ddict;
+    const rpmuint32_t * ddict;
     unsigned ix;
     int ndx;
 
@@ -1898,7 +2147,8 @@ void rpmfiBuildFDeps(Header h, rpmTag tagN,
 
 exit:
     fi = rpmfiFree(fi);
-    ds = rpmdsFree(ds);
+    (void)rpmdsFree(ds);
+    ds = NULL;
     if (fdepsp)
 	*fdepsp = av;
     else
