@@ -11,7 +11,7 @@
 extern int _rpmfi_debug;
 /*@=exportlocal@*/
 
-/** \rpmfi
+/** \ingroup rpmfi
  * File types.
  * These are the file types used internally by rpm. The file
  * type is determined by applying stat(2) macros like S_ISDIR to
@@ -65,38 +65,27 @@ typedef	enum rpmfileAttrs_e {
     RPMFILE_TYPED	= (1 << 15),	/*!< (unimplemented) from %%spook */
     RPMFILE_SOURCE	= (1 << 16),	/*!< from SourceN: (srpm only). */
     RPMFILE_PATCH	= (1 << 17),	/*!< from PatchN: (srpm only). */
-    RPMFILE_OPTIONAL	= (1 << 18)	/*!< from %%optional. */
+    RPMFILE_OPTIONAL	= (1 << 18),	/*!< from %%optional. */
+    RPMFILE_REMOVE	= (1 << 19)	/*!< remove file (after %post). */
 } rpmfileAttrs;
 
 #define	RPMFILE_SPOOK	(RPMFILE_GHOST|RPMFILE_TYPED)
 #define	RPMFILE_ALL	~(RPMFILE_NONE)
 
-/**
- * File disposition(s) during package install/erase transaction.
+/** \ingroup rpmfi
+ * File info tag sets from a header, so that a header can be discarded early.
  */
-typedef enum fileAction_e {
-    FA_UNKNOWN = 0,	/*!< initial action for file ... */
-    FA_CREATE,		/*!< ... copy in from payload. */
-    FA_COPYIN,		/*!< ... copy in from payload. */
-    FA_COPYOUT,		/*!< ... copy out to payload. */
-    FA_BACKUP,		/*!< ... renamed with ".rpmorig" extension. */
-    FA_SAVE,		/*!< ... renamed with ".rpmsave" extension. */
-    FA_SKIP, 		/*!< ... already replaced, don't remove. */
-    FA_ALTNAME,		/*!< ... create with ".rpmnew" extension. */
-    FA_ERASE,		/*!< ... to be removed. */
-    FA_SKIPNSTATE,	/*!< ... untouched, state "not installed". */
-    FA_SKIPNETSHARED,	/*!< ... untouched, state "netshared". */
-    FA_SKIPCOLOR	/*!< ... untouched, state "wrong color". */
-} fileAction;
+typedef /*@abstract@*/ /*@refcounted@*/ struct rpmfi_s * rpmfi;
 
 #if defined(_RPMFI_INTERNAL)
-#define XFA_SKIPPING(_a)	\
-    ((_a) == FA_SKIP || (_a) == FA_SKIPNSTATE || (_a) == FA_SKIPNETSHARED || (_a) == FA_SKIPCOLOR)
+
+#include "mire.h"
 
 /** \ingroup rpmfi
  * A package filename set.
  */
 struct rpmfi_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
     int i;			/*!< Current file index. */
     int j;			/*!< Current directory index. */
 
@@ -107,34 +96,34 @@ struct rpmfi_s {
 /*@refcounted@*/ /*@null@*/
     Header h;			/*!< Header for file info set (or NULL) */
 
-/*@only@*/ /*?null?*/
+/*@only@*/ /*@relnull@*/
     const char ** bnl;		/*!< Base name(s) (from header) */
-/*@only@*/ /*?null?*/
+/*@only@*/ /*@relnull@*/
     const char ** dnl;		/*!< Directory name(s) (from header) */
 
 /*@only@*/ /*@relnull@*/
     const char ** fdigests;	/*!< File digest(s) (from header) */
 /*@only@*/ /*@null@*/
-    uint32_t * fdigestalgos;	/*!< File digest algorithm(s) (from header) */
+    rpmuint32_t * fdigestalgos;	/*!< File digest algorithm(s) (from header) */
 /*@only@*/ /*@relnull@*/
     const char ** flinks;	/*!< File link(s) (from header) */
 /*@only@*/ /*@null@*/
     const char ** flangs;	/*!< File lang(s) (from header) */
 
 /*@only@*/ /*@relnull@*/
-          uint32_t * dil;	/*!< Directory indice(s) (from header) */
-/*@only@*/ /*?null?*/
-    const uint32_t * fflags;	/*!< File flag(s) (from header) */
-/*@only@*/ /*?null?*/
-    const uint32_t * fsizes;	/*!< File size(s) (from header) */
-/*@only@*/ /*?null?*/
-    const uint32_t * fmtimes;	/*!< File modification time(s) (from header) */
-/*@only@*/ /*?null?*/
-          uint16_t * fmodes;	/*!< File mode(s) (from header) */
-/*@only@*/ /*?null?*/
-    const uint16_t * frdevs;	/*!< File rdev(s) (from header) */
-/*@only@*/ /*?null?*/
-    const uint32_t * finodes;	/*!< File inodes(s) (from header) */
+          rpmuint32_t * dil;	/*!< Directory indice(s) (from header) */
+/*@only@*/ /*@relnull@*/
+    const rpmuint32_t * fflags;	/*!< File flag(s) (from header) */
+/*@only@*/ /*@relnull@*/
+    const rpmuint32_t * fsizes;	/*!< File size(s) (from header) */
+/*@only@*/ /*@relnull@*/
+    const rpmuint32_t * fmtimes;/*!< File modification time(s) (from header) */
+/*@only@*/ /*@relnull@*/
+          rpmuint16_t * fmodes;	/*!< File mode(s) (from header) */
+/*@only@*/ /*@relnull@*/
+    const rpmuint16_t * frdevs;	/*!< File rdev(s) (from header) */
+/*@only@*/ /*@relnull@*/
+    const rpmuint32_t * finodes;/*!< File inodes(s) (from header) */
 
 /*@only@*/ /*@null@*/
     const char ** fuser;	/*!< File owner(s) (from header) */
@@ -142,45 +131,55 @@ struct rpmfi_s {
     const char ** fgroup;	/*!< File group(s) (from header) */
 
 /*@only@*/ /*@null@*/
-    uint8_t * fstates;		/*!< File state(s) (from header) */
+    rpmuint8_t * fstates;	/*!< File state(s) (from header) */
 
 /*@only@*/ /*@null@*/
-    const uint32_t * fcolors;	/*!< File color bits (header) */
+    const rpmuint32_t * fcolors;/*!< File color bits (header) */
 
 /*@only@*/ /*@null@*/
-    const char ** fcontexts;	/*! FIle security contexts. */
+    const char ** fcaps;	/*! File capabilities. */
+
+/*@only@*/ /*@null@*/
+    const char ** fcontexts;	/*! File security contexts. */
 
 /*@only@*/ /*@null@*/
     const char ** cdict;	/*!< File class dictionary (header) */
-    uint32_t ncdict;		/*!< No. of class entries. */
+    rpmuint32_t ncdict;		/*!< No. of class entries. */
 /*@only@*/ /*@null@*/
-    const uint32_t * fcdictx;	/*!< File class dictionary index (header) */
+    const rpmuint32_t * fcdictx;/*!< File class dictionary index (header) */
 
 /*@only@*/ /*@null@*/
-    const uint32_t * ddict;	/*!< File depends dictionary (header) */
-    uint32_t nddict;		/*!< No. of depends entries. */
+    const rpmuint32_t * ddict;	/*!< File depends dictionary (header) */
+    rpmuint32_t nddict;		/*!< No. of depends entries. */
 /*@only@*/ /*@null@*/
-    const uint32_t * fddictx;	/*!< File depends dictionary start (header) */
+    const rpmuint32_t * fddictx;/*!< File depends dictionary start (header) */
 /*@only@*/ /*@null@*/
-    const uint32_t * fddictn;	/*!< File depends dictionary count (header) */
+    const rpmuint32_t * fddictn;/*!< File depends dictionary count (header) */
 
-/*@only@*/ /*?null?*/
-    const uint32_t * vflags;	/*!< File verify flag(s) (from header) */
+/*@only@*/ /*@relnull@*/
+    const rpmuint32_t * vflags;	/*!< File verify flag(s) (from header) */
 
-    uint32_t dc;		/*!< No. of directories. */
-    uint32_t fc;		/*!< No. of files. */
+    rpmuint32_t dc;		/*!< No. of directories. */
+    rpmuint32_t fc;		/*!< No. of files. */
 
 /*=============================*/
 /*@dependent@*/ /*@relnull@*/
-    rpmte te;
+    void * te;
+
+/*@only@*/ /*@null@*/
+    miRE exclude;		/*!< Iterator exclude patterns. */
+    int nexclude;		/*!< No. of exclude patterns. */
+/*@only@*/ /*@null@*/
+    miRE include;		/*!< Iterator include patterns. */
+    int ninclude;		/*!< No. of include patterns. */
 
 /*-----------------------------*/
     uid_t uid;			/*!< File uid (default). */
     gid_t gid;			/*!< File gid (default). */
-    uint32_t flags;		/*!< File flags (default). */
-    fileAction action;		/*!< File disposition (default). */
+    rpmuint32_t flags;		/*!< File flags (default). */
+    int action;			/*!< File disposition (default). */
 /*@owned@*/ /*@relnull@*/
-    fileAction * actions;	/*!< File disposition(s). */
+    int * actions;		/*!< File disposition(s). */
 /*@owned@*/
     struct fingerPrint_s * fps;	/*!< File fingerprint(s). */
 /*@owned@*/
@@ -188,12 +187,12 @@ struct rpmfi_s {
 /*@owned@*/
     const char ** odnl;		/*!< Original dirname(s) (from header) */
 /*@unused@*/
-    uint32_t * odil;		/*!< Original dirindex(s) (from header) */
+    rpmuint32_t * odil;		/*!< Original dirindex(s) (from header) */
 
 /*@only@*/ /*@relnull@*/
     unsigned char * digests;	/*!< File digest(s) in binary. */
-    uint32_t digestalgo;	/*!< File digest algorithm. */
-    uint32_t digestlen;		/*!< No. bytes in binary digest. */
+    rpmuint32_t digestalgo;	/*!< File digest algorithm. */
+    rpmuint32_t digestlen;	/*!< No. bytes in binary digest. */
 
 /*@only@*/ /*@relnull@*/
     const char * pretrans;
@@ -209,8 +208,8 @@ struct rpmfi_s {
     const char * verifyscriptprog;
 
 /*@only@*/ /*@null@*/
-    char * fn;			/*!< File name buffer. */
-    size_t fnlen;			/*!< File name buffer length. */
+    char * fn;			/*!< File name buffer, fnlen + 1 bytes. */
+    size_t fnlen;		/*!< Maximum file name length (without '\0'). */
 
     size_t astriplen;
     size_t striplen;
@@ -224,20 +223,22 @@ struct rpmfi_s {
 /*@owned@*/ /*@null@*/
     int * fmapflags;
 /*@owned@*/
-    FSM_t fsm;			/*!< File state machine data. */
-    uint32_t color;		/*!< Color bit(s) from file color union. */
+    void * fsm;			/*!< File state machine data. */
+    rpmuint32_t color;		/*!< Color bit(s) from file color union. */
 
     int isSource;		/*!< Is this a SRPM? */
 
 /*@owned@*/
-    uint32_t * replacedSizes;	/*!< (TR_ADDED) */
+    rpmuint32_t * replacedSizes;/*!< (TR_ADDED) */
 
     unsigned int record;	/*!< (TR_REMOVED) */
     int magic;
 #define	RPMFIMAGIC	0x09697923
 /*=============================*/
-
-/*@refs@*/ int nrefs;		/*!< Reference count. */
+#if defined(__LCLINT__)
+/*@refs@*/
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 
 #endif	/* _RPMFI_INTERNAL */
@@ -246,6 +247,7 @@ struct rpmfi_s {
 extern "C" {
 #endif
 
+#if !defined(_RPMFI_NOMETHODS)
 /** \name RPMFI */
 /*@{*/
 
@@ -253,27 +255,14 @@ extern "C" {
  * Unreference a file info set instance.
  * @param fi		file info set
  * @param msg
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@unused@*/ /*@null@*/
 rpmfi rpmfiUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi,
 		/*@null@*/ const char * msg)
 	/*@modifies fi @*/;
-
-/** @todo Remove debugging entry from the ABI.
- * @param fi		file info set
- * @param msg
- * @param fn
- * @param ln
- * @return		NULL always
- */
-/*@-exportlocal@*/
-/*@null@*/
-rpmfi XrpmfiUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi,
-		/*@null@*/ const char * msg, const char * fn, unsigned ln)
-	/*@modifies fi @*/;
-/*@=exportlocal@*/
-#define	rpmfiUnlink(_fi, _msg) XrpmfiUnlink(_fi, _msg, __FILE__, __LINE__)
+#define	rpmfiUnlink(_fi, _msg)	\
+    ((rpmfi) rpmioUnlinkPoolItem((rpmioItem)(_fi), _msg, __FILE__, __LINE__))
 
 /**
  * Reference a file info set instance.
@@ -284,19 +273,8 @@ rpmfi XrpmfiUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi,
 /*@unused@*/ /*@null@*/
 rpmfi rpmfiLink (/*@null@*/ rpmfi fi, /*@null@*/ const char * msg)
 	/*@modifies fi @*/;
-
-/** @todo Remove debugging entry from the ABI.
- * @param fi		file info set
- * @param msg
- * @param fn
- * @param ln
- * @return		NULL always
- */
-/*@null@*/
-rpmfi XrpmfiLink (/*@null@*/ rpmfi fi, /*@null@*/ const char * msg,
-		const char * fn, unsigned ln)
-        /*@modifies fi @*/;
-#define	rpmfiLink(_fi, _msg)	XrpmfiLink(_fi, _msg, __FILE__, __LINE__)
+#define	rpmfiLink(_fi, _msg)	\
+    ((rpmfi) rpmioLinkPoolItem((rpmioItem)(_fi), _msg, __FILE__, __LINE__))
 
 /**
  * Return file count from file info set.
@@ -385,12 +363,20 @@ extern const char * rpmfiDN(/*@null@*/ rpmfi fi)
 extern const char * rpmfiFN(/*@null@*/ rpmfi fi)
 	/*@modifies fi @*/;
 
+/**
+ * Return maximum file name length from file info set.
+ * @param fi		file info set
+ * @return		maximum file name length (not including '\0')
+ */
+extern size_t rpmfiFNMaxLen(/*@null@*/ rpmfi fi)
+	/*@*/;
+
 /**
  * Return current file flags from file info set.
  * @param fi		file info set
  * @return		current file flags, 0 on invalid
  */
-uint32_t rpmfiFFlags(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiFFlags(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -399,7 +385,7 @@ uint32_t rpmfiFFlags(/*@null@*/ rpmfi fi)
  * @param FFlags	new file flags
  * @return		previous file flags, 0 on invalid
  */
-uint32_t rpmfiSetFFlags(/*@null@*/ rpmfi fi, uint32_t FFlags)
+rpmuint32_t rpmfiSetFFlags(/*@null@*/ rpmfi fi, rpmuint32_t FFlags)
 	/*@modifies fi @*/;
 
 /**
@@ -407,7 +393,7 @@ uint32_t rpmfiSetFFlags(/*@null@*/ rpmfi fi, uint32_t FFlags)
  * @param fi		file info set
  * @return		current file verify flags, 0 on invalid
  */
-uint32_t rpmfiVFlags(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiVFlags(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -416,7 +402,7 @@ uint32_t rpmfiVFlags(/*@null@*/ rpmfi fi)
  * @param VFlags	new file verify flags
  * @return		previous file verify flags, 0 on invalid
  */
-uint32_t rpmfiSetVFlags(/*@null@*/ rpmfi fi, uint32_t VFlags)
+rpmuint32_t rpmfiSetVFlags(/*@null@*/ rpmfi fi, rpmuint32_t VFlags)
 	/*@modifies fi @*/;
 
 /**
@@ -424,7 +410,7 @@ uint32_t rpmfiSetVFlags(/*@null@*/ rpmfi fi, uint32_t VFlags)
  * @param fi		file info set
  * @return		current file mode, 0 on invalid
  */
-uint16_t rpmfiFMode(/*@null@*/ rpmfi fi)
+rpmuint16_t rpmfiFMode(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -471,7 +457,7 @@ extern const char * rpmfiFLink(/*@null@*/ rpmfi fi)
  * @param fi		file info set
  * @return		current file size, 0 on invalid
  */
-uint32_t rpmfiFSize(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiFSize(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -479,7 +465,7 @@ uint32_t rpmfiFSize(/*@null@*/ rpmfi fi)
  * @param fi		file info set
  * @return		current file rdev, 0 on invalid
  */
-uint16_t rpmfiFRdev(/*@null@*/ rpmfi fi)
+rpmuint16_t rpmfiFRdev(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -487,7 +473,7 @@ uint16_t rpmfiFRdev(/*@null@*/ rpmfi fi)
  * @param fi		file info set
  * @return		current file inode, 0 on invalid
  */
-uint32_t rpmfiFInode(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiFInode(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -495,7 +481,7 @@ uint32_t rpmfiFInode(/*@null@*/ rpmfi fi)
  * @param fi		file info set
  * @return		current color
  */
-uint32_t rpmfiColor(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiColor(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -503,7 +489,7 @@ uint32_t rpmfiColor(/*@null@*/ rpmfi fi)
  * @param fi		file info set
  * @return		current file color
  */
-uint32_t rpmfiFColor(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiFColor(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -534,8 +520,8 @@ extern const char * rpmfiFContext(/*@null@*/ rpmfi fi)
  * @retval *fddictp	file depends dictionary array (or NULL)
  * @return		no. of file depends entries, 0 on invalid
  */
-uint32_t rpmfiFDepends(/*@null@*/ rpmfi fi,
-		/*@out@*/ /*@null@*/ const uint32_t ** fddictp)
+rpmuint32_t rpmfiFDepends(/*@null@*/ rpmfi fi,
+		/*@out@*/ /*@null@*/ const rpmuint32_t ** fddictp)
 	/*@modifies *fddictp @*/;
 
 /**
@@ -543,7 +529,7 @@ uint32_t rpmfiFDepends(/*@null@*/ rpmfi fi,
  * @param fi		file info set
  * @return		current file nlink count, 0 on invalid
  */
-uint32_t rpmfiFNlink(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiFNlink(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -551,7 +537,7 @@ uint32_t rpmfiFNlink(/*@null@*/ rpmfi fi)
  * @param fi		file info set
  * @return		current file modify time, 0 on invalid
  */
-uint32_t rpmfiFMtime(/*@null@*/ rpmfi fi)
+rpmuint32_t rpmfiFMtime(/*@null@*/ rpmfi fi)
 	/*@*/;
 
 /**
@@ -572,6 +558,40 @@ extern const char * rpmfiFUser(/*@null@*/ rpmfi fi)
 extern const char * rpmfiFGroup(/*@null@*/ rpmfi fi)
 	/*@*/;
 
+/** \ingroup rpmfi
+ * Return file info exclude patterns.
+ * @param fi		file info set
+ * @return		file info exclude patterns (NULL if not set)
+ */
+/*@null@*/
+void * rpmfiExclude(/*@null@*/ const rpmfi fi)
+	/*@*/;
+
+/** \ingroup rpmfi
+ * Return no. of file info exclude patterns.
+ * @param fi		file info set
+ * @return		file info exclude patterns (0 if not set)
+ */
+int rpmfiNExclude(/*@null@*/ const rpmfi fi)
+	/*@*/;
+
+/** \ingroup rpmfi
+ * Return file info include patterns.
+ * @param fi		file info set
+ * @return		file info include patterns (NULL if not set)
+ */
+/*@null@*/
+void * rpmfiInclude(/*@null@*/ const rpmfi fi)
+	/*@*/;
+
+/** \ingroup rpmfi
+ * Return no. of file info include patterns.
+ * @param fi		file info set
+ * @return		file info include patterns (0 if not set)
+ */
+int rpmfiNInclude(/*@null@*/ const rpmfi fi)
+	/*@*/;
+
 /**
  * Return next file iterator index.
  * @param fi		file info set
@@ -609,28 +629,69 @@ int rpmfiNextD(/*@null@*/ rpmfi fi)
 rpmfi rpmfiInitD(/*@null@*/ rpmfi fi, int dx)
 	/*@modifies fi @*/;
 
+/**
+ * Link a header to a file info set.
+ * @param fi		file info set
+ * @param h		header
+ * @return		0 always
+ */
+int rpmfiSetHeader(rpmfi fi, /*@null@*/ Header h)
+	/*@modifies fi, h @*/;
+
 /**
  * Destroy a file info set.
  * @param fi		file info set
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@null@*/
 rpmfi rpmfiFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi)
 	/*@globals fileSystem @*/
 	/*@modifies fi, fileSystem @*/;
+#define	rpmfiFree(_fi)	\
+    ((rpmfi) rpmioFreePoolItem((rpmioItem)(_fi), __FUNCTION__, __FILE__, __LINE__))
 
 /**
  * Create and load a file info set.
- * @param ts		transaction set (NULL skips path relocation)
+ * @param _ts		transaction set (NULL skips path relocation)
  * @param h		header
  * @param tagN		RPMTAG_BASENAMES
  * @param flags		scareMem(0x1), nofilter(0x2)
  * @return		new file info set
  */
 /*@null@*/
-rpmfi rpmfiNew(/*@null@*/ const rpmts ts, Header h, rpmTag tagN, int flags)
+rpmfi rpmfiNew(/*@null@*/ const void * _ts, Header h, rpmTag tagN, int flags)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@modifies _ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/**
+ * Return current stat(2) data from file info set.
+ * @param fi		file info set
+ * @retval *st		stat(2) data
+ * @return		-1 on error, 0 on success
+ */
+int rpmfiFStat(rpmfi fi, /*@out@*/ struct stat * st)
+	/*@modifies *st @*/;
+
+/**
+ * Return lstat(2) data of path from file info set.
+ * @param fi		file info set
+ * @param path		file path
+ * @retval *st		stat(2) data
+ * @return		-1 on error, 0 on success
+ */
+int rpmfiStat(rpmfi fi, const char * path, /*@out@*/ struct stat * st)
+	/*@modifies fi, *st @*/;
+
+/**
+ * Return directory stream onto file info set.
+ * @param fi		file info set
+ * @param name		directory path
+ * @return		NULL on error
+ */
+/*@null@*/
+DIR * rpmfiOpendir(rpmfi fi, const char * name)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fi, fileSystem, internalState @*/;
 
 /**
  * Retrieve file classes from header.
@@ -642,7 +703,7 @@ rpmfi rpmfiNew(/*@null@*/ const rpmts ts, Header h, rpmTag tagN, int flags)
  * @retval *fcp		number of files
  */
 void rpmfiBuildFClasses(Header h,
-		/*@out@*/ const char *** fclassp, /*@out@*/ uint32_t * fcp)
+		/*@out@*/ const char *** fclassp, /*@out@*/ rpmuint32_t * fcp)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies h, *fclassp, *fcp, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
@@ -657,7 +718,7 @@ void rpmfiBuildFClasses(Header h,
  * @retval *fcp		number of files
  */
 void rpmfiBuildFContexts(Header h,
-		/*@out@*/ const char *** fcontextp, /*@out@*/ uint32_t * fcp)
+		/*@out@*/ const char *** fcontextp, /*@out@*/ rpmuint32_t * fcp)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies h, *fcontextp, *fcp, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
@@ -672,7 +733,7 @@ void rpmfiBuildFContexts(Header h,
  * @retval *fcp		number of files
  */
 void rpmfiBuildFSContexts(Header h,
-		/*@out@*/ const char *** fcontextp, /*@out@*/ uint32_t * fcp)
+		/*@out@*/ const char *** fcontextp, /*@out@*/ rpmuint32_t * fcp)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies h, *fcontextp, *fcp, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
@@ -687,7 +748,7 @@ void rpmfiBuildFSContexts(Header h,
  * @retval *fcp		number of files
  */
 void rpmfiBuildREContexts(Header h,
-		/*@out@*/ const char *** fcontextp, /*@out@*/ uint32_t * fcp)
+		/*@out@*/ const char *** fcontextp, /*@out@*/ rpmuint32_t * fcp)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies h, *fcontextp, *fcp, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
@@ -703,7 +764,7 @@ void rpmfiBuildREContexts(Header h,
  * @retval *fcp		number of files
  */
 void rpmfiBuildFDeps(Header h, rpmTag tagN,
-		/*@out@*/ const char *** fdepsp, /*@out@*/ uint32_t * fcp)
+		/*@out@*/ const char *** fdepsp, /*@out@*/ rpmuint32_t * fcp)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies h, *fdepsp, *fcp,
 		rpmGlobalMacroContext, fileSystem, internalState @*/;
@@ -724,7 +785,7 @@ int rpmfiCompare(const rpmfi afi, const rpmfi bfi)
  * @param skipMissing	OK to skip missing files?
  * @return		file dispostion
  */
-fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
+int rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies nfi, fileSystem, internalState @*/;
 
@@ -739,7 +800,39 @@ const char * rpmfiTypeString(rpmfi fi)
 	/*@*/;
 /*@=redef@*/
 
+/**
+ * Free relocation array.
+ * @param relocs	relocation array
+ * @return		NULL always
+ */
+/*@null@*/
+rpmRelocation rpmfiFreeRelocations(/*@only@*/ rpmRelocation relocs)
+	/*@modifies relocs@*/;
+
+/**
+ * Duplicate a relocation array.
+ * @param relocs	relocation array
+ * @retval *nrelocsp	no. of elements
+ * @return		duplicate relocation array
+ */
+/*@only@*/ /*@null@*/
+rpmRelocation rpmfiDupeRelocations(rpmRelocation relocs, int * nrelocsp)
+	/*@modifies *nrelocsp @*/;
+
+/**
+ * Add relocation element to array.
+ * @retval *relocations		relocation array
+ * @retval *nrelocations	no. of elements
+ * @param oldPath		old path
+ * @param newPath		new path
+ * @return			0 on success
+ */
+int rpmfiAddRelocation(rpmRelocation * relp, int * nrelp,
+		const char * oldPath, const char * newPath)
+	/*@modifies *relp, *nrelp @*/;
+
 /*@}*/
+#endif	/* _RPMFI_NOMETHODS */
 
 #ifdef __cplusplus
 }
