@@ -5,9 +5,13 @@
 #include "system.h"
 
 #include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
+#include <rpmiotypes.h>		/* XXX fnpyKey */
+#include <rpmcb.h>
 #include <rpmmacro.h>		/* XXX rpmExpand */
-#include <rpmlib.h>
+#include <rpmtypes.h>
+#include <rpmtag.h>
+#include <rpmdb.h>
+
 #include <rpmte.h>		/* XXX rpmElementType */
 #include <pkgio.h>		/* XXX rpmElementType */
 
@@ -15,14 +19,15 @@
 #define	_RPMTS_INTERNAL		/* XXX ts->probs et al */
 #include <rpmgi.h>
 
-#include <rpmdb.h>
 #include "manifest.h"
 
+#include <rpmlib.h>
+
 #include "debug.h"
 
 /*@access FD_t @*/		/* XXX void * arg */
 /*@access fnpyKey @*/
-/*@access rpmdbMatchIterator @*/
+/*@access rpmmi @*/
 /*@access rpmts @*/
 /*@access rpmps @*/
 
@@ -125,16 +130,7 @@ static rpmRC rpmgiLoadManifest(rpmgi gi, const char * path)
     return rpmrc;
 }
 
-/**
- * Return header from package.
- * @param gi		generalized iterator
- * @param path		file path
- * @return		header (NULL on failure)
- */
-/*@null@*/
-static Header rpmgiReadHeader(rpmgi gi, const char * path)
-	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
-	/*@modifies gi, rpmGlobalMacroContext, h_errno, internalState @*/
+Header rpmgiReadHeader(rpmgi gi, const char * path)
 {
     FD_t fd = rpmgiOpen(path, "r%{?_rpmgio}");
     Header h = NULL;
@@ -150,7 +146,8 @@ static Header rpmgiReadHeader(rpmgi gi, const char * path)
 	    /* XXX Read a package manifest. Restart ftswalk on success. */
 	case RPMRC_FAIL:
 	default:
-	    h = headerFree(h);
+	    (void)headerFree(h);
+	    h = NULL;
 	    break;
 	case RPMRC_NOTTRUSTED:
 	case RPMRC_NOKEY:
@@ -203,7 +200,8 @@ static rpmRC rpmgiLoadReadHeader(rpmgi gi)
 	const char * fn;	/* XXX gi->hdrPath? */
 
 	fn = gi->argv[gi->i];
-	if (!(gi->flags & RPMGI_NOHEADER)) {
+	/* XXX Skip +bing -bang =boom special arguments. */
+	if (strchr("-+=", *fn) == NULL && !(gi->flags & RPMGI_NOHEADER)) {
 	    h = rpmgiReadHeader(gi, fn);
 	    if (h != NULL)
 		rpmrc = RPMRC_OK;
@@ -218,6 +216,7 @@ static rpmRC rpmgiLoadReadHeader(rpmgi gi)
 	/* Not a header, so try for a manifest. */
 	gi->argv[gi->i] = NULL;		/* Mark the insertion point */
 	rpmrc = rpmgiLoadManifest(gi, fn);
+	/* XXX its unclear if RPMRC_NOTFOUND should fail or continue here. */
 	if (rpmrc != RPMRC_OK) {
 	    gi->argv[gi->i] = fn;	/* Manifest failed, restore fn */
 	    break;
@@ -228,7 +227,8 @@ static rpmRC rpmgiLoadReadHeader(rpmgi gi)
 
     if (rpmrc == RPMRC_OK && h != NULL)
 	gi->h = headerLink(h);
-    h = headerFree(h);
+    (void)headerFree(h);
+    h = NULL;
 
     return rpmrc;
 }
@@ -258,6 +258,8 @@ rpmlog(RPMLOG_DEBUG, "FTS_%s\t%*s %s%s\n", ftsInfoStr(fts->fts_info),
     case FTS_DP:	/* postorder directory */
 	break;
     case FTS_F:		/* regular file */
+	if ((size_t)fts->fts_namelen <= sizeof(".rpm"))
+	    break;
 	/* Ignore all but *.rpm files. */
 	s = fts->fts_name + fts->fts_namelen + 1 - sizeof(".rpm");
 	if (strcmp(s, ".rpm"))
@@ -313,7 +315,8 @@ static rpmRC rpmgiWalkReadHeader(rpmgi gi)
 	}
 	if (h != NULL) {
 	    gi->h = headerLink(h);
-	    h = headerFree(h);
+	    (void)headerFree(h);
+	    h = NULL;
 /*@-noeffectuncon@*/
 	    if (gi->stash != NULL)
 		(void) (*gi->stash) (gi, gi->h);
@@ -436,7 +439,7 @@ fprintf(stderr, "*** gi %p key %p[%d]\tmi %p\n", gi, gi->keyp, (int)gi->keylen,
 	    if (!res) {
 if (_rpmgi_debug  < 0)
 fprintf(stderr, "\tav %p[%d]: \"%s\" -> %s ~= \"%s\"\n", gi->argv, (int)(av - gi->argv), *av, tagName(tag), pat);
-		res = rpmdbSetIteratorRE(gi->mi, tag, RPMMIRE_DEFAULT, pat);
+		res = rpmmiAddPattern(gi->mi, tag, RPMMIRE_DEFAULT, pat);
 	    }
 	    a = _free(a);
 	}
@@ -444,7 +447,7 @@ fprintf(stderr, "\tav %p[%d]: \"%s\" -> %s ~= \"%s\"\n", gi->argv, (int)(av - gi
 	if (res == 0)
 	    continue;
 
-	gi->mi = rpmdbFreeIterator(gi->mi);	/* XXX odd side effect? */
+	gi->mi = rpmmiFree(gi->mi);	/* XXX odd side effect? */
 	rpmrc = RPMRC_FAIL;
 	break;
     }
@@ -452,77 +455,65 @@ fprintf(stderr, "\tav %p[%d]: \"%s\" -> %s ~= \"%s\"\n", gi->argv, (int)(av - gi
     return rpmrc;
 }
 
-rpmgi XrpmgiUnlink(rpmgi gi, const char * msg, const char * fn, unsigned ln)
+/*@-mustmod@*/
+static void rpmgiFini(void * _gi)
+	/*@globals rpmGlobalMacroContext @*/
+	/*@modifies _gi, rpmGlobalMacroContext @*/
 {
-    if (gi == NULL) return NULL;
-
-if (_rpmgi_debug && msg != NULL)
-fprintf(stderr, "--> gi %p -- %d %s(%s) at %s:%u\n", gi, gi->nrefs, msg, tagName(gi->tag), fn, ln);
-
-    gi->nrefs--;
-    return NULL;
-}
-
-rpmgi XrpmgiLink(rpmgi gi, const char * msg, const char * fn, unsigned ln)
-{
-    if (gi == NULL) return NULL;
-    gi->nrefs++;
-
-if (_rpmgi_debug && msg != NULL)
-fprintf(stderr, "--> gi %p ++ %d %s(%s) at %s:%u\n", gi, gi->nrefs, msg, tagName(gi->tag), fn, ln);
-
-    /*@-refcounttrans@*/ return gi; /*@=refcounttrans@*/
-}
-
-rpmgi rpmgiFree(rpmgi gi)
-{
-    if (gi == NULL)
-	return NULL;
-
-    if (gi->nrefs > 1)
-	return rpmgiUnlink(gi, "rpmgiFree");
-
-    (void) rpmgiUnlink(gi, "rpmgiFree");
-
-/*@-usereleased@*/
+    rpmgi gi = _gi;
+    int xx;
 
     gi->hdrPath = _free(gi->hdrPath);
-    gi->h = headerFree(gi->h);
+    (void)headerFree(gi->h);
+    gi->h = NULL;
 
     gi->argv = argvFree(gi->argv);
 
     if (gi->ftsp != NULL) {
-	int xx;
 	xx = Fts_close(gi->ftsp);
 	gi->ftsp = NULL;
 	gi->fts = NULL;
     }
     if (gi->fd != NULL) {
-	(void) Fclose(gi->fd);
+	xx = Fclose(gi->fd);
 	gi->fd = NULL;
     }
     gi->tsi = rpmtsiFree(gi->tsi);
-    gi->mi = rpmdbFreeIterator(gi->mi);
-    gi->ts = rpmtsFree(gi->ts);
+    gi->mi = rpmmiFree(gi->mi);
+    (void)rpmtsFree(gi->ts); 
+    gi->ts = NULL;
+}
+/*@=mustmod@*/
 
-    memset(gi, 0, sizeof(*gi));		/* XXX trash and burn */
-/*@-refcounttrans@*/
-    gi = _free(gi);
-/*@=refcounttrans@*/
-/*@=usereleased@*/
-    return NULL;
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _rpmgiPool;
+
+static rpmgi rpmgiGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _rpmgiPool, fileSystem, internalState @*/
+	/*@modifies pool, _rpmgiPool, fileSystem, internalState @*/
+{
+    rpmgi gi;
+
+    if (_rpmgiPool == NULL) {
+	_rpmgiPool = rpmioNewPool("gi", sizeof(*gi), -1, _rpmgi_debug,
+			NULL, NULL, rpmgiFini);
+	pool = _rpmgiPool;
+    }
+    return (rpmgi) rpmioGetPool(pool, sizeof(*gi));
 }
 
 rpmgi rpmgiNew(rpmts ts, int tag, const void * keyp, size_t keylen)
 {
-    rpmgi gi = xcalloc(1, sizeof(*gi));
+    rpmgi gi = rpmgiGetPool(_rpmgiPool);
 
-    if (gi == NULL)
+    if (gi == NULL)	/* XXX can't happen */
 	return NULL;
 
+/*@-assignexpose -castexpose @*/
     gi->ts = rpmtsLink(ts, "rpmgiNew");
+/*@=assignexpose =castexpose @*/
     gi->tsOrder = rpmtsOrder;
-    gi->tag = tag;
+    gi->tag = (rpmTag) tag;
 /*@-assignexpose@*/
     gi->keyp = keyp;
 /*@=assignexpose@*/
@@ -565,7 +556,8 @@ if (_rpmgi_debug)
 fprintf(stderr, "*** rpmgiNext(%p) tag %s\n", gi, tagName(gi->tag));
 
     /* Free header from previous iteration. */
-    gi->h = headerFree(gi->h);
+    (void)headerFree(gi->h);
+    gi->h = NULL;
     gi->hdrPath = _free(gi->hdrPath);
     hnum[0] = '\0';
 
@@ -579,7 +571,7 @@ nextkey:
 		goto enditer;
 	    rpmrc = rpmgiInitFilter(gi);
 	    if (rpmrc != RPMRC_OK || gi->mi == NULL) {
-		gi->mi = rpmdbFreeIterator(gi->mi);	/* XXX unnecessary */
+		gi->mi = rpmmiFree(gi->mi);	/* XXX unnecessary */
 		gi->i++;
 		goto nextkey;
 	    }
@@ -587,18 +579,18 @@ nextkey:
 	    gi->active = 1;
 	}
 	if (gi->mi != NULL) {	/* XXX unnecessary */
-	    Header h = rpmdbNextIterator(gi->mi);
+	    Header h = rpmmiNext(gi->mi);
 	    if (h != NULL) {
 		if (!(gi->flags & RPMGI_NOHEADER))
 		    gi->h = headerLink(h);
-		sprintf(hnum, "%u", rpmdbGetIteratorOffset(gi->mi));
+		sprintf(hnum, "%u", rpmmiInstance(gi->mi));
 		gi->hdrPath = rpmExpand("rpmdb h# ", hnum, NULL);
 		rpmrc = RPMRC_OK;
 		/* XXX header reference held by iterator, so no headerFree */
 	    }
 	}
 	if (rpmrc != RPMRC_OK) {
-	    gi->mi = rpmdbFreeIterator(gi->mi);
+	    gi->mi = rpmmiFree(gi->mi);
 	    goto nextkey;
 	}
 	break;
@@ -606,25 +598,25 @@ nextkey:
 	if (!gi->active) {
 	    rpmrc = rpmgiInitFilter(gi);
 	    if (rpmrc != RPMRC_OK) {
-		gi->mi = rpmdbFreeIterator(gi->mi);	/* XXX unnecessary */
+		gi->mi = rpmmiFree(gi->mi);	/* XXX unnecessary */
 		goto enditer;
 	    }
 	    rpmrc = RPMRC_NOTFOUND;	/* XXX hack */
 	    gi->active = 1;
 	}
 	if (gi->mi != NULL) {	/* XXX unnecessary */
-	    Header h = rpmdbNextIterator(gi->mi);
+	    Header h = rpmmiNext(gi->mi);
 	    if (h != NULL) {
 		if (!(gi->flags & RPMGI_NOHEADER))
 		    gi->h = headerLink(h);
-		sprintf(hnum, "%u", rpmdbGetIteratorOffset(gi->mi));
+		sprintf(hnum, "%u", rpmmiInstance(gi->mi));
 		gi->hdrPath = rpmExpand("rpmdb h# ", hnum, NULL);
 		rpmrc = RPMRC_OK;
 		/* XXX header reference held by iterator, so no headerFree */
 	    }
 	}
 	if (rpmrc != RPMRC_OK) {
-	    gi->mi = rpmdbFreeIterator(gi->mi);
+	    gi->mi = rpmmiFree(gi->mi);
 	    goto enditer;
 	}
 	break;
@@ -650,7 +642,8 @@ nextkey:
 		sprintf(hnum, "%u", (unsigned)gi->i);
 		gi->hdrPath = rpmExpand("%s h# ", teTypeString, hnum, NULL);
 		rpmrc = RPMRC_OK;
-		h = headerFree(h);
+		(void)headerFree(h);
+		h = NULL;
 	    }
 	}
 	if (rpmrc != RPMRC_OK) {
@@ -687,7 +680,8 @@ nextkey:
 		sprintf(hnum, "%u", (unsigned)gi->i);
 		gi->hdrPath = rpmExpand("hdlist h# ", hnum, NULL);
 		rpmrc = RPMRC_OK;
-		h = headerFree(h);
+		(void)headerFree(h);
+		h = NULL;
 	    }
 	}
 	if (rpmrc != RPMRC_OK) {
@@ -766,9 +760,9 @@ enditer:
 
 	/* Permit access to indices used for depsolving. */
 	if (!(gi->flags & RPMGI_ERASING)) {
-	    xx = rpmdbBlockDBI(rpmtsGetRdb(ts), +RPMTAG_PROVIDENAME);
-	    xx = rpmdbBlockDBI(rpmtsGetRdb(ts), +RPMTAG_BASENAMES);
-	    xx = rpmdbBlockDBI(rpmtsGetRdb(ts), +RPMDBI_DEPENDS);
+	    xx = rpmdbBlockDBI(rpmtsGetRdb(ts), RPMTAG_PROVIDENAME);
+	    xx = rpmdbBlockDBI(rpmtsGetRdb(ts), RPMTAG_BASENAMES);
+	    xx = rpmdbBlockDBI(rpmtsGetRdb(ts), RPMDBI_DEPENDS);
 	}
 
 	/* XXX query/verify will need the glop added to a buffer instead. */
@@ -811,7 +805,8 @@ enditer:
 
     }
 
-    gi->h = headerFree(gi->h);
+    (void)headerFree(gi->h);
+    gi->h = NULL;
     gi->hdrPath = _free(gi->hdrPath);
     gi->i = -1;
     gi->active = 0;
