@@ -5,12 +5,12 @@
  * \file lib/rpmgi.h
  */
 
-#include <rpmlib.h>
+#include <fts.h>
+#include <argv.h>
+#include <rpmtypes.h>
 #include <rpmds.h>
 #include <rpmte.h>
 #include <rpmts.h>
-#include <fts.h>
-#include <argv.h>
 
 /**
  */
@@ -40,10 +40,11 @@ extern rpmgiFlags giFlags;
 /** \ingroup rpmgi
  */
 struct rpmgi_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
 /*@refcounted@*/
     rpmts ts;			/*!< Iterator transaction set. */
     int (*tsOrder) (rpmts ts);	/*!< Iterator transaction ordering. */
-    int tag;			/*!< Iterator type. */
+    rpmTag tag;			/*!< Iterator type. */
 /*@kept@*/ /*@relnull@*/
     const void * keyp;		/*!< Iterator key. */
     size_t keylen;		/*!< Iterator key length. */
@@ -60,7 +61,7 @@ struct rpmgi_s {
     rpmtsi tsi;
 
 /*@null@*/
-    rpmdbMatchIterator mi;
+    rpmmi mi;
 
 /*@refcounted@*/ /*@relnull@*/
     FD_t fd;
@@ -78,8 +79,10 @@ struct rpmgi_s {
 /*@null@*/
     rpmRC (*stash) (rpmgi gi, Header h);
 
+#if defined(__LCLINT__)
 /*@refs@*/
-    int nrefs;			/*!< Reference count. */
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 #endif
 
@@ -94,21 +97,14 @@ extern "C" {
  * Unreference a generalized iterator instance.
  * @param gi		generalized iterator
  * @param msg
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@unused@*/ /*@null@*/
 rpmgi rpmgiUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmgi gi,
 		/*@null@*/ const char * msg)
 	/*@modifies gi @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@-exportlocal@*/
-/*@null@*/
-rpmgi XrpmgiUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmgi gi,
-		/*@null@*/ const char * msg, const char * fn, unsigned ln)
-	/*@modifies gi @*/;
-/*@=exportlocal@*/
-#define	rpmgiUnlink(_gi, _msg)	XrpmgiUnlink(_gi, _msg, __FILE__, __LINE__)
+#define	rpmgiUnlink(_gi, _msg)	\
+    ((rpmgi)rpmioUnlinkPoolItem((rpmioItem)(_gi), _msg, __FILE__, __LINE__))
 
 /**
  * Reference a generalized iterator instance.
@@ -119,24 +115,19 @@ rpmgi XrpmgiUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ rpmgi gi,
 /*@unused@*/ /*@newref@*/ /*@null@*/
 rpmgi rpmgiLink (/*@null@*/ rpmgi gi, /*@null@*/ const char * msg)
 	/*@modifies gi @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@-exportlocal@*/
-/*@newref@*/ /*@null@*/
-rpmgi XrpmgiLink (/*@null@*/ rpmgi gi, /*@null@*/ const char * msg,
-		const char * fn, unsigned ln)
-        /*@modifies gi @*/;
-/*@=exportlocal@*/
-#define	rpmgiLink(_gi, _msg)	XrpmgiLink(_gi, _msg, __FILE__, __LINE__)
+#define	rpmgiLink(_gi, _msg)	\
+    ((rpmgi)rpmioLinkPoolItem((rpmioItem)(_gi), _msg, __FILE__, __LINE__))
 
 /** Destroy a generalized iterator.
  * @param gi		generalized iterator
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@null@*/
 rpmgi rpmgiFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmgi gi)
 	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
         /*@modifies gi, rpmGlobalMacroContext, h_errno, internalState @*/;
+#define	rpmgiFree(_gi)	\
+    ((rpmgi)rpmioFreePoolItem((rpmioItem)(_gi), __FUNCTION__, __FILE__, __LINE__))
 
 /**
  * Return a generalized iterator.
@@ -217,6 +208,17 @@ rpmRC rpmgiSetArgs(/*@null@*/ rpmgi gi, /*@null@*/ ARGV_t argv,
 	/*@globals internalState @*/
 	/*@modifies gi, internalState @*/;
 
+/**
+ * Return header from package.
+ * @param gi		generalized iterator
+ * @param path		file path
+ * @return		header (NULL on failure)
+ */
+/*@null@*/
+Header rpmgiReadHeader(rpmgi gi, const char * path)
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies gi, rpmGlobalMacroContext, h_errno, internalState @*/;
+
 /*@}*/
 
 #ifdef __cplusplus
