@@ -5,23 +5,35 @@
 #include "system.h"
 
 #include <rpmio.h>
-#include <rpmtag.h>
-#define	_RPMTS_INTERNAL		/* XXX ts->suggests */
-#include <rpmcli.h>
+#include <rpmiotypes.h>
+#include <poptIO.h>
 
+#include <rpmtag.h>
+#include <rpmevr.h>
 #include "rpmdb.h"
 #ifdef	NOTYET
 #include "rpmds.h"		/* XXX ts->suggests, +foo -foo =foo args */
 #endif
 
 #include "rpmte.h"		/* XXX rpmtsPrint() */
+#define	_RPMTS_INTERNAL		/* XXX ts->suggests */
+#include <rpmts.h>
 
 #include "manifest.h"
+#define	_RPMGI_INTERNAL		/* XXX "+bing" args need gi->h. */
 #include "rpmgi.h"
+
+#include <rpmlib.h>
+
+#include <rpmcli.h>
+#define	_RPMROLLBACK_INTERNAL
+#include <rpmrollback.h>
+
 #include "debug.h"
 
 /*@access FD_t @*/	/* XXX void * arg */
 /*@access rpmts @*/	/* XXX ts->suggests */
+/*@access rpmgi @*/	/* XXX gi->h */
 /*@access fnpyKey @*/	/* XXX cast */
 
 /*@unchecked@*/
@@ -31,9 +43,9 @@ int rpmcliHashesCurrent = 0;
 /*@unchecked@*/
 int rpmcliHashesTotal = 0;
 /*@unchecked@*/
-uint64_t rpmcliProgressCurrent = 0;
+rpmuint64_t rpmcliProgressCurrent = 0;
 /*@unchecked@*/
-uint64_t rpmcliProgressTotal = 0;
+rpmuint64_t rpmcliProgressTotal = 0;
 
 /**
  * Print a CLI progress bar.
@@ -41,7 +53,7 @@ uint64_t rpmcliProgressTotal = 0;
  * @param amount	current
  * @param total		final
  */
-static void printHash(const uint64_t amount, const uint64_t total)
+static void printHash(const rpmuint64_t amount, const rpmuint64_t total)
 	/*@globals rpmcliHashesCurrent, rpmcliHashesTotal,
 		rpmcliProgressCurrent, fileSystem @*/
 	/*@modifies rpmcliHashesCurrent, rpmcliHashesTotal,
@@ -90,19 +102,19 @@ static void printHash(const uint64_t amount, const uint64_t total)
 
 void * rpmShowProgress(/*@null@*/ const void * arg,
 			const rpmCallbackType what,
-			const uint64_t amount,
-			const uint64_t total,
+			const rpmuint64_t amount,
+			const rpmuint64_t total,
 			/*@null@*/ fnpyKey key,
 			/*@null@*/ void * data)
 	/*@globals rpmcliHashesCurrent, rpmcliProgressCurrent, rpmcliProgressTotal,
-		fileSystem @*/
+		rpmGlobalMacroContext, fileSystem @*/
 	/*@modifies rpmcliHashesCurrent, rpmcliProgressCurrent, rpmcliProgressTotal,
-		fileSystem @*/
+		rpmGlobalMacroContext, fileSystem @*/
 {
 /*@-abstract -castexpose @*/
     Header h = (Header) arg;
 /*@=abstract =castexpose @*/
-    char * s;
+    const char * s;
     int flags = (int) ((long)data);
     void * rc = NULL;
 /*@-abstract -assignexpose @*/
@@ -115,7 +127,17 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
     case RPMCALLBACK_INST_OPEN_FILE:
 	if (filename == NULL || filename[0] == '\0')
 	    return NULL;
-	fd = Fopen(filename, "r.fdio");
+	fd = Fopen(filename, "r%{?_rpmgio}");
+
+	/* XXX Retry once to handle http:// server timeout reopen's. */
+	if (Ferror(fd)) {
+	    int ut = urlPath(filename, NULL);
+	    if (ut == URL_IS_HTTP || ut == URL_IS_HTTPS) {
+		/* XXX HACK: Fclose(fd) no workie here. */
+		fd = Fopen(filename, "r%{?_rpmgio}");
+	    }
+	}
+
 	/*@-type@*/ /* FIX: still necessary? */
 	if (fd == NULL || Ferror(fd)) {
 	    rpmlog(RPMLOG_ERR, _("open of %s failed: %s\n"), filename,
@@ -157,11 +179,12 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
 	    (void) fflush(stdout);
 	    s = _free(s);
 	} else {
-	    s = headerSprintf(h, "%{NAME}-%{VERSION}-%{RELEASE}",
-				  NULL, rpmHeaderFormats, NULL);
+	    char * t = rpmExpand("%{?___NVRA}%{!?___NVRA:%%{NAME}-%%{VERSION}-%%{RELEASE}}", NULL);
+	    s = headerSprintf(h, t, NULL, rpmHeaderFormats, NULL);
 	    fprintf(stdout, "%s\n", s);
 	    (void) fflush(stdout);
 	    s = _free(s);
+	    t = _free(t);
 	}
 	break;
 
@@ -250,7 +273,7 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
     }
 
     return rc;
-}	
+}
 
 int rpmcliInstallProblems(rpmts ts, const char * msg, int rc)
 	/*@globals fileSystem @*/
@@ -306,6 +329,167 @@ int rpmcliInstallRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 /*@=evalorder@*/
 }
 
+static rpmRC rpmcliEraseElement(rpmts ts, const char * arg)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    rpmmi mi;
+    Header h;
+    rpmRC rc = RPMRC_OK;
+    int xx;
+
+    mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
+    if (mi == NULL)
+	return RPMRC_NOTFOUND;
+
+    while ((h = rpmmiNext(mi)) != NULL) {
+	unsigned int recOffset = rpmmiInstance(mi);
+
+	if (recOffset == 0) {	/* XXX can't happen. */
+	    rc = RPMRC_FAIL;
+	    break;
+	}
+	xx = rpmtsAddEraseElement(ts, h, recOffset);
+    }
+    mi = rpmmiFree(mi);
+
+    return 0;
+}
+
+static const char * rpmcliWalkFirst(ARGV_t av, miRE mire)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies mire, fileSystem, internalState @*/
+{
+    /* XXX use global ftsOpts? */
+    /* XXX changing FTS_LOGICAL to FTS_PHYSICAL prevents symlink follow. */
+    /* XXX FTS_NOCHDIR is automatically assumed for URI's */
+    int _ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+    FTS * ftsp = NULL;
+    FTSENT * fts;
+    const char * fn = NULL;
+    int fts_level = 1;
+    int xx;
+
+    if (av != NULL && av[0] != NULL)
+	ftsp = Fts_open((char *const *)av, _ftsOpts, NULL);
+    if (ftsp != NULL)
+    while((fts = Fts_read(ftsp)) != NULL) {
+	switch (fts->fts_info) {
+	/* No-op conditions. */
+	case FTS_D:		/* preorder directory */
+	case FTS_DP:		/* postorder directory */
+	    /* XXX Don't recurse downwards, all elements should be files. */
+	    if (fts_level > 0 && fts->fts_level >= fts_level)
+		xx = Fts_set(ftsp, fts, FTS_SKIP);
+	    /*@fallthrough@*/
+	case FTS_DOT:		/* dot or dot-dot */
+	    continue;
+	    /*@notreached@*/ /*@switchbreak@*/ break;
+	case FTS_F:		/* regular file */
+	    if (mireRegexec(mire, fts->fts_accpath, 0) < 0)
+		continue;
+	    /*@switchbreak@*/ break;
+	/* Error conditions. */
+	case FTS_NS:		/* stat(2) failed */
+	case FTS_DNR:		/* unreadable directory */
+	case FTS_ERR:		/* error; errno is set */
+	case FTS_DC:		/* directory that causes cycles */
+	case FTS_DEFAULT:	/* none of the above */
+	case FTS_INIT:		/* initialized only */
+	case FTS_NSOK:		/* no stat(2) requested */
+	case FTS_SL:		/* symbolic link */
+	case FTS_SLNONE:	/* symbolic link without target */
+	case FTS_W:		/* whiteout object */
+	default:
+	    goto exit;
+	    /*@notreached@*/ /*@switchbreak@*/ break;
+	}
+
+	/* Stop on first file that matches. */
+	fn = xstrdup(fts->fts_accpath);
+	break;
+    }
+
+exit:
+    xx = Fts_close(ftsp);
+    return fn;
+}
+
+static const char * rpmcliInstallElementPath(/*@unused@*/ rpmts ts,
+		const char * arg)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    /* A glob pattern list to match repository directories. */
+    const char * fn = rpmExpand(
+	"%{?_rpmgi_pattern_glob}"
+	"%{!?_rpmgi_pattern_glob:.}",
+	NULL
+    );
+    /* A regex pattern list to match candidate *.rpm files. */
+    const char * mirePattern = rpmExpand(
+        "%{?_rpmgi_pattern_regex:%{_rpmgi_pattern_regex ", arg, "}}"
+        "%{!?_rpmgi_pattern_regex:", arg, "-[^-]+-[^-]+\\.[^.]+\\.rpm$}",
+        NULL
+    );
+    miRE mire = mireNew(RPMMIRE_REGEX, 0);
+    ARGV_t dav = NULL;
+    int dac = 0;
+    ARGV_t av = NULL;
+    int xx = mireRegcomp(mire, mirePattern);
+    int i;
+
+    /* Get list of candidate repository patterns. */
+    xx = argvSplit(&dav, fn, ":");
+    fn = _free(fn);
+    if (xx || dav == NULL)
+	goto exit;
+
+    dac = argvCount(dav);
+    for (i = 0; i < dac; i++) {
+	ARGV_t nav = NULL;
+	int nac = 0;
+
+	/* Insure only directory paths are matched. */
+	fn = rpmGetPath(dav[i], "/", NULL);
+	xx = rpmGlob(fn, &nac, &nav);
+
+	/* Insure that final directory paths have trailing '/' */
+	if (nav != NULL)
+	for (i = 0; i < nac; i++) {
+	    char * t = rpmExpand(nav[i], "/", NULL);
+	    nav[i] = _free(nav[i]);
+	    nav[i] = t;
+	}
+
+	/* Append matches to list of repository directories. */
+	if (nac > 0 && nav != NULL)
+	    xx = argvAppend(&av, nav);
+	nav = argvFree(nav);
+	nac = 0;
+	fn = _free(fn);
+    }
+
+    /* Walk (possibly multi-root'd) directories, until 1st match is found. */
+    fn = rpmcliWalkFirst(av, mire);
+
+exit:
+    av = argvFree(av);
+    dav = argvFree(dav);
+    mire = mireFree(mire);
+    mirePattern = _free(mirePattern);
+
+    return fn;
+}
+
+/*@-redef@*/	/* XXX Add rpmfi methods to make rpmRelocation opaque. */
+struct rpmRelocation_s {
+/*@only@*/ /*@null@*/
+    const char * oldPath;       /*!< NULL here evals to RPMTAG_DEFAULTPREFIX, */
+/*@only@*/ /*@null@*/
+    const char * newPath;       /*!< NULL means to omit the file completely! */
+};
+/*@=redef@*/
 
 /** @todo Generalize --freshen policies. */
 int rpmcliInstall(rpmts ts, QVA_t ia, const char ** argv)
@@ -317,6 +501,7 @@ int rpmcliInstall(rpmts ts, QVA_t ia, const char ** argv)
     int numRPMS = 0;
     rpmRelocation relocations = NULL;
     rpmVSFlags vsflags, ovsflags;
+    rpmRC rpmrc;
     int rc;
     int xx;
 
@@ -328,16 +513,6 @@ int rpmcliInstall(rpmts ts, QVA_t ia, const char ** argv)
     if (rpmExpandNumeric("%{?_repackage_all_erasures}"))
 	ia->transFlags |= RPMTRANS_FLAG_REPACKAGE;
 
-    /* Initialize security context patterns (if not already done). */
-    if (rpmtsSELinuxEnabled(ts) && !(ia->transFlags & RPMTRANS_FLAG_NOCONTEXTS))
-    {
-	const char *fn = rpmGetPath("%{?_install_file_context_path}", NULL);
-/*@-moduncon@*/
-	if (fn != NULL && *fn != '\0')
-	    xx = matchpathcon_init(fn);
-/*@=moduncon@*/
-	fn = _free(fn);
-    }
     (void) rpmtsSetFlags(ts, ia->transFlags);
     (void) rpmtsSetDFlags(ts, ia->depFlags);
 
@@ -345,10 +520,10 @@ int rpmcliInstall(rpmts ts, QVA_t ia, const char ** argv)
     if (rpmExpandNumeric("%{?_rollback_transaction_on_failure}")) {
 	if (ia->arbtid) {
 	    time_t ttid = (time_t)ia->arbtid;
-	    rpmlog(RPMLOG_DEBUG, D_("Autorollback Goal: %-24.24s (0x%08x)\n"), 
+	    rpmlog(RPMLOG_DEBUG, D_("Autorollback Goal: %-24.24s (0x%08x)\n"),
 		ctime(&ttid), ia->arbtid);
 	    rpmtsSetARBGoal(ts, ia->arbtid);
-	}	
+	}
     }
 
     if (ia->installInterfaceFlags & INSTALL_UPGRADE)
@@ -376,50 +551,66 @@ int rpmcliInstall(rpmts ts, QVA_t ia, const char ** argv)
 	    relocations = NULL;
     }
 
-#if defined(REFERENCE_FORNOW)
-if (fileURL[0] == '=') {
-    rpmds this = rpmdsSingle(RPMTAG_REQUIRENAME, fileURL+1, NULL, 0);
-
-    xx = rpmtsSolve(ts, this, NULL);
-    if (ts->suggests && ts->nsuggests > 0) {
-	fileURL = _free(fileURL);
-	fileURL = ts->suggests[0];
-	ts->suggests[0] = NULL;
-	while (ts->nsuggests-- > 0) {
-	    if (ts->suggests[ts->nsuggests] == NULL)
-		continue;
-	    ts->suggests[ts->nsuggests] = _free(ts->suggests[ts->nsuggests]);
-	}
-	ts->suggests = _free(ts->suggests);
-	rpmlog(RPMLOG_DEBUG, D_("Adding goal: %s\n"), fileURL);
-	pkgURL[pkgx] = fileURL;
-	fileURL = NULL;
-	pkgx++;
-    }
-    this = rpmdsFree(this);
-} else
-#endif
-
  {	/* start-of-transaction-build */
     int tag = (ia->qva_source == RPMQV_FTSWALK)
 	? RPMDBI_FTSWALK : RPMDBI_ARGLIST;
     rpmgi gi = rpmgiNew(ts, tag, NULL, 0);
     rpmgiFlags _giFlags = RPMGI_NONE;
+    const char * fn = NULL;;
 
 /*@-mods@*/
-    if (ftsOpts == 0)
-	ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+    if (rpmioFtsOpts == 0)
+	rpmioFtsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
 /*@=mods@*/
-    rc = rpmgiSetArgs(gi, argv, ftsOpts, _giFlags);
-    while (rpmgiNext(gi) == RPMRC_OK) {
-	Header h = rpmgiHeader(gi);
-	const char * fn;
+    rc = rpmgiSetArgs(gi, argv, rpmioFtsOpts, _giFlags);
+    while ((rpmrc = rpmgiNext(gi)) == RPMRC_OK) {
+	Header h;
 
+	fn = _free(fn);
+	fn = xstrdup(rpmgiHdrPath(gi));
+
+        /* === Check for "+bing" lookaside paths within install transaction. */
+        if (fn[0] == '+') {
+	    const char * nfn;
+	    addMacro(NULL, "NEVRA", NULL, &fn[1], RMIL_GLOBAL);
+	    nfn = rpmcliInstallElementPath(ts, &fn[1]);
+	    delMacro(NULL, "NEVRA");
+	    if (nfn == NULL) {
+		rpmlog(RPMLOG_ERR, _("package \"%s\" cannot be found\n"), fn);
+		numFailed++;	/* XXX multiple erasures? */
+		continue;
+	    }
+	    fn = _free(fn);
+	    fn = nfn;
+	    /* XXX hack into rpmgi innards for now ... */
+	    h = rpmgiReadHeader(gi, fn);
+	    if (h != NULL)
+		gi->h = headerLink(h);
+	    (void)headerFree(h);
+	    h = NULL;
+        }
+
+	/* === Check for "-bang" erasures within install transaction. */
+	if (fn[0] == '-') {
+	    switch (rpmcliEraseElement(ts, &fn[1])) {
+	    case RPMRC_OK:
+		numRPMS++;	/* XXX multiple erasures? */
+		/*@switchbreak@*/ break;
+	    case RPMRC_NOTFOUND:
+	    default:
+		rpmlog(RPMLOG_ERR, _("package \"%s\" cannot be erased\n"), fn);
+		numFailed++;	/* XXX multiple erasures? */
+		goto exit;
+		/*@notreached@*/ /*@switchbreak@*/ break;
+	    }
+	    continue;
+	}
+
+	h = rpmgiHeader(gi);
 	if (h == NULL) {
 	    numFailed++;
 	    continue;
 	}
-	fn = rpmgiHdrPath(gi);
 
 	/* === Check for relocatable package. */
 	if (relocations) {
@@ -443,7 +634,7 @@ if (fileURL[0] == '=') {
 
 	/* === On --freshen, verify package is installed and newer. */
 	if (ia->installInterfaceFlags & INSTALL_FRESHEN) {
-	    rpmdbMatchIterator mi;
+	    rpmmi mi;
 	    Header oldH;
 	    int count;
 
@@ -452,15 +643,15 @@ if (fileURL[0] == '=') {
 assert(xx != 0 && he->p.str != NULL);
 	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, he->p.str, 0);
 	    he->p.ptr = _free(he->p.ptr);
-	    count = rpmdbGetIteratorCount(mi);
-	    while ((oldH = rpmdbNextIterator(mi)) != NULL) {
+	    count = rpmmiCount(mi);
+	    while ((oldH = rpmmiNext(mi)) != NULL) {
 		if (rpmVersionCompare(oldH, h) < 0)
 		    /*@innercontinue@*/ continue;
 		/* same or newer package already installed */
 		count = 0;
 		/*@innerbreak@*/ break;
 	    }
-	    mi = rpmdbFreeIterator(mi);
+	    mi = rpmmiFree(mi);
 	    if (count == 0)
 		continue;
 	    /* Package is newer than those currently installed. */
@@ -478,10 +669,13 @@ assert(xx != 0 && he->p.str != NULL);
 	numRPMS++;
     }
 
+    fn = _free(fn);
     gi = rpmgiFree(gi);
 
  }	/* end-of-transaction-build */
 
+    /* XXX exit if the iteration failed. */
+    if (rpmrc == RPMRC_FAIL) numFailed = numRPMS;
     if (numFailed) goto exit;
 
     if (numRPMS) {
@@ -498,7 +692,8 @@ assert(xx != 0 && he->p.str != NULL);
 	/* Drop added/available package indices and dependency sets. */
 	rpmtsClean(ts);
 
-	if (numFailed == 0
+	/* XXX Avoid empty transaction msg, run iff there are elements. */
+	if (numFailed == 0 && rpmtsNElements(ts) > 0
 	 && (rc = rpmcliInstallRun(ts, NULL, ia->probFilter)) != 0)
 	    numFailed += (rc < 0 ? numRPMS : rc);
     }
@@ -548,10 +743,10 @@ int rpmErase(rpmts ts, QVA_t ia, const char ** argv)
     if (rpmExpandNumeric("%{?_rollback_transaction_on_failure}")) {
 	if (ia->arbtid) {
 	    time_t ttid = (time_t)ia->arbtid;
-	    rpmlog(RPMLOG_DEBUG, D_("Autorollback Goal: %-24.24s (0x%08x)\n"), 
+	    rpmlog(RPMLOG_DEBUG, D_("Autorollback Goal: %-24.24s (0x%08x)\n"),
 		ctime(&ttid), ia->arbtid);
 	    rpmtsSetARBGoal(ts, ia->arbtid);
-	}	
+	}
     }
 
 #ifdef	NOTYET	/* XXX no callbacks on erase yet */
@@ -565,7 +760,7 @@ int rpmErase(rpmts ts, QVA_t ia, const char ** argv)
     (void) rpmtsSetGoal(ts, TSM_ERASE);
 
     for (arg = argv; *arg; arg++) {
-	rpmdbMatchIterator mi;
+	rpmmi mi;
 
 	/* XXX HACK to get rpmdbFindByLabel out of the API */
 	mi = rpmtsInitIterator(ts, RPMDBI_LABEL, *arg, 0);
@@ -575,8 +770,8 @@ int rpmErase(rpmts ts, QVA_t ia, const char ** argv)
 	} else {
 	    Header h;	/* XXX iterator owns the reference */
 	    count = 0;
-	    while ((h = rpmdbNextIterator(mi)) != NULL) {
-		unsigned int recOffset = rpmdbGetIteratorOffset(mi);
+	    while ((h = rpmmiNext(mi)) != NULL) {
+		unsigned int recOffset = rpmmiInstance(mi);
 
 		if (!(count++ == 0 || (ia->installInterfaceFlags & INSTALL_ALLMATCHES))) {
 		    rpmlog(RPMLOG_ERR, _("\"%s\" specifies multiple packages\n"),
@@ -590,7 +785,7 @@ int rpmErase(rpmts ts, QVA_t ia, const char ** argv)
 		}
 	    }
 	}
-	mi = rpmdbFreeIterator(mi);
+	mi = rpmmiFree(mi);
     }
 
     if (numFailed == 0 && numRPMS > 0) {
@@ -623,7 +818,7 @@ int rpmInstallSource(rpmts ts, const char * arg,
     FD_t fd;
     int rc;
 
-    fd = Fopen(arg, "r.fdio");
+    fd = Fopen(arg, "r%{?_rpmgio}");
     if (fd == NULL || Ferror(fd)) {
 	rpmlog(RPMLOG_ERR, _("cannot open %s: %s\n"), arg, Fstrerror(fd));
 	if (fd != NULL) (void) Fclose(fd);
