@@ -5,8 +5,9 @@
 #include "system.h"
 
 #include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
-#include <rpmlib.h>
+#include <rpmiotypes.h>		/* XXX fnpyKey */
+#include <rpmtypes.h>
+#include <yarn.h>
 
 #define	_RPMPS_INTERNAL
 #include "rpmps.h"
@@ -19,30 +20,21 @@
 /*@unchecked@*/
 int _rpmps_debug = 0;
 
-rpmps XrpmpsUnlink(rpmps ps, const char * msg,
-		const char * fn, unsigned ln)
-{
-/*@-modfilesys@*/
-if (_rpmps_debug && msg != NULL)
-fprintf(stderr, "--> ps %p -- %d %s at %s:%u\n", ps, ps->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-    ps->nrefs--;
-/*@-refcounttrans@*/
-    return ps;
-/*@=refcounttrans@*/
-}
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _rpmpsPool;
 
-rpmps XrpmpsLink(rpmps ps, const char * msg,
-		const char * fn, unsigned ln)
+static rpmps rpmpsGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _rpmpsPool, fileSystem, internalState @*/
+	/*@modifies pool, _rpmpsPool, fileSystem, internalState @*/
 {
-    ps->nrefs++;
-/*@-modfilesys@*/
-if (_rpmps_debug && msg != NULL)
-fprintf(stderr, "--> ps %p ++ %d %s at %s:%u\n", ps, ps->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-/*@-refcounttrans@*/
-    return ps;
-/*@=refcounttrans@*/
+    rpmps ps;
+
+    if (_rpmpsPool == NULL) {
+	_rpmpsPool = rpmioNewPool("ps", sizeof(*ps), -1, _rpmps_debug,
+			NULL, NULL, NULL);
+	pool = _rpmpsPool;
+    }
+    return (rpmps) rpmioGetPool(pool, sizeof(*ps));
 }
 
 int rpmpsNumProblems(rpmps ps)
@@ -58,7 +50,9 @@ rpmpsi rpmpsInitIterator(rpmps ps)
     rpmpsi psi = NULL;
     if (ps != NULL) {
 	psi = xcalloc(1, sizeof(*psi));
+/*@-assignexpose -castexpose @*/
 	psi->ps = rpmpsLink(ps, "iter ref");
+/*@=assignexpose =castexpose @*/
 	psi->ix = -1;
     }
     return psi;
@@ -98,35 +92,43 @@ rpmProblem rpmpsProblem(rpmpsi psi)
 
 rpmps rpmpsCreate(void)
 {
-    rpmps ps = xcalloc(1, sizeof(*ps));
+#ifdef	BUGGY
+    rpmps ps = rpmpsGetPool(_rpmpsPool);
+#else
+    rpmps ps = rpmpsGetPool(NULL);
+#endif
     return rpmpsLink(ps, "create");
 }
 
 rpmps rpmpsFree(rpmps ps)
 {
     if (ps == NULL) return NULL;
-    ps = rpmpsUnlink(ps, "dereference");
-    if (ps->nrefs > 0)
-	return NULL;
-	
-    if (ps->probs) {
-	int i;
-	for (i = 0; i < ps->numProblems; i++) {
-	    rpmProblem p = ps->probs + i;
-	    p->pkgNEVR = _free(p->pkgNEVR);
-	    p->altNEVR = _free(p->altNEVR);
-	    p->str1 = _free(p->str1);
+    yarnPossess(ps->_item.use);
+/*@-modfilesys@*/
+if (_rpmps_debug)
+fprintf(stderr, "--> ps %p -- %ld %s at %s:%u\n", ps, yarnPeekLock(ps->_item.use), "rpmpsFree", __FILE__, __LINE__);
+/*@=modfilesys@*/
+    if (yarnPeekLock(ps->_item.use) <= 1L) {
+	if (ps->probs) {
+	    int i;
+	    for (i = 0; i < ps->numProblems; i++) {
+		rpmProblem p = ps->probs + i;
+		p->pkgNEVR = _free(p->pkgNEVR);
+		p->altNEVR = _free(p->altNEVR);
+		p->str1 = _free(p->str1);
+	    }
+	    ps->probs = _free(ps->probs);
 	}
-	ps->probs = _free(ps->probs);
-    }
-    ps = _free(ps);
+	ps = (rpmps) rpmioPutPool((rpmioItem)ps);
+    } else
+	yarnTwist(ps->_item.use, BY, -1);
     return NULL;
 }
 
 void rpmpsAppend(rpmps ps, rpmProblemType type,
 		const char * pkgNEVR, fnpyKey key,
 		const char * dn, const char * bn,
-		const char * altNEVR, uint64_t ulong1)
+		const char * altNEVR, rpmuint64_t ulong1)
 {
     rpmProblem p;
     char *t;
@@ -388,37 +390,42 @@ void rpmpsPrint(FILE *fp, rpmps ps)
 
 rpmProblem rpmpsGetProblem(rpmps ps, int num)
 {
-    if (ps == NULL || num > ps->numProblems)
+    if (ps == NULL)
         return(NULL);
-    return(ps->probs + num);
+    /* XXX Retrieve last problem with negative index. */
+    if (num < 0)
+	num = ps->numProblems - 1;
+    if (num >= ps->numProblems)
+        return(NULL);
+    return (ps->probs + num);
 }
 
 char * rpmProblemGetPkgNEVR(rpmProblem prob)
 {
-    return(prob->pkgNEVR);
+    return prob->pkgNEVR;
 }
 
 char * rpmProblemGetAltNEVR(rpmProblem prob)
 {
-    return(prob->altNEVR);
+    return prob->altNEVR;
 }
 
 char * rpmProblemGetStr(rpmProblem prob)
 {
-    return(prob->str1);
+    return prob->str1;
 }
 
-unsigned long long rpmProblemGetLong(rpmProblem prob)
+rpmuint64_t rpmProblemGetDiskNeed(rpmProblem prob)
 {
-    return(prob->ulong1);
+    return prob->ulong1;
 }
 
 rpmProblemType rpmProblemGetType(rpmProblem prob)
 {
-    return(prob->type);
+    return prob->type;
 }
 
 fnpyKey rpmProblemKey(rpmProblem prob)
 {
-    return(prob->key);
+    return prob->key;
 }
