@@ -8,14 +8,28 @@
 #define __power_pc() 0
 #endif
 
+#define	_RPMIOB_INTERNAL	/* XXX for rpmiobSlurp */
+#include <rpmio.h>
+#include <rpmcb.h>
 #define _MIRE_INTERNAL
-#include <rpmio_internal.h> /* for rpmioSlurp() */
+#include <mire.h>
+#include <argv.h>
 #include <rpmlua.h>
 #include <rpmluaext.h>
 #include <rpmmacro.h>
-#include <rpmcli.h>
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
+#define _RPMEVR_INTERNAL
+#include <rpmevr.h>
+
+#define _RPMDS_INTERNAL
 #include <rpmds.h>
 
+#include <rpmcli.h>
+
+#include <rpmsyck.h>
+
 #include "debug.h"
 
 /*@access miRE@*/
@@ -25,11 +39,29 @@ static const char * configTarget = NULL;
 
 /*@observer@*/ /*@unchecked@*/
 static const char * platform = SYSCONFIGDIR "/platform";
+
 /*@only@*/ /*@relnull@*/ /*@unchecked@*/
 void * platpat = NULL;
 /*@unchecked@*/
 int nplatpat = 0;
 
+extern rpmds cpuinfoP;
+
+
+
+/** \ingroup rpmrc
+ * Build and install arch/os table identifiers.
+ * @deprecated Eliminate from API.
+ * @todo	Eliminate in rpm-5.1.
+ */
+enum rpm_machtable_e {
+    RPM_MACHTABLE_INSTARCH	= 0,	/*!< Install platform architecture. */
+    RPM_MACHTABLE_INSTOS	= 1,	/*!< Install platform operating system. */
+    RPM_MACHTABLE_BUILDARCH	= 2,	/*!< Build platform architecture. */
+    RPM_MACHTABLE_BUILDOS	= 3	/*!< Build platform operating system. */
+};
+#define	RPM_MACHTABLE_COUNT	4	/*!< No. of arch/os tables. */
+
 typedef /*@owned@*/ const char * cptr_t;
 
 typedef struct machCacheEntry_s {
@@ -84,7 +116,7 @@ typedef struct tableType_s {
 /*@-fullinitblock@*/
 /*@unchecked@*/
 static struct tableType_s tables[RPM_MACHTABLE_COUNT] = {
-    { "arch", 1, 0 },
+    { "arch", 1, 0  },
     { "os", 1, 0 },
     { "buildarch", 0, 1 },
     { "buildos", 0, 1 }
@@ -242,7 +274,7 @@ static void addMacroDefault(const char * macroname,
 }
 
 static void setPathDefault(const char * macroname, const char * subdir)
-	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@globals rpmGlobalMacroContext, internalState @*/
 	/*@modifies rpmGlobalMacroContext, internalState @*/
 {
     if (macroname != NULL) {
@@ -273,11 +305,24 @@ export RPM_PACKAGE_NAME RPM_PACKAGE_VERSION RPM_PACKAGE_RELEASE\n\
 export RPM_BUILD_ROOT\n}\
 ";
 
+#if defined(RPM_VENDOR_WINDRIVER)
+/*@unchecked@*/
+extern const char * __usrlibrpm;
+/*@unchecked@*/
+extern const char * __etcrpm;
+#endif
+
 static void setDefaults(void)
-	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@globals rpmGlobalMacroContext, internalState @*/
 	/*@modifies rpmGlobalMacroContext, internalState @*/
 {
 
+#if defined(RPM_VENDOR_WINDRIVER)
+    addMacro(NULL, "_usrlibrpm", NULL, __usrlibrpm, RMIL_DEFAULT);
+    addMacro(NULL, "_etcrpm", NULL, __etcrpm, RMIL_DEFAULT);
+    addMacro(NULL, "_vendor", NULL, "%{?_host_vendor}%{!?_host_vendor:wrs}", RMIL_DEFAULT);
+#endif
+
     addMacro(NULL, "_usr", NULL, USRPREFIX, RMIL_DEFAULT);
     addMacro(NULL, "_var", NULL, VARPREFIX, RMIL_DEFAULT);
     addMacro(NULL, "_prefix", NULL, "%{_usr}", RMIL_DEFAULT);
@@ -377,57 +422,6 @@ static int parseCVOG(const char * str, CVOG_t *cvogp)
     return 0;
 }
 
-/**
- * Destroy platform patterns.
- * @param mire		platform pattern array
- * @param nre		no of patterns in array
- * @return		NULL always 
- */
-/*@-onlytrans@*/	/* XXX miRE array, not refcounted. */
-/*@null@*/
-static void * mireFreeAll(/*@only@*/ /*@null@*/ miRE mire, int nre)
-	/*@modifies mire@*/
-{
-    if (mire != NULL) {
-	int i;
-	for (i = 0; i < nre; i++)
-	    (void) mireClean(mire + i);
-	mire = _free(mire);
-    }
-    return NULL;
-}
-/*@=onlytrans@*/
-
-/**
- * Append pattern to array.
- * @param mode		type of pattern match
- * @param tag		identifier (like an rpmTag)
- * @param pattern	pattern to compile
- * @retval *mi_rep	platform pattern array
- * @retval *mi_nrep	no. of patterns in array
- */
-/*@-onlytrans@*/	/* XXX miRE array, not refcounted. */
-/*@null@*/
-static int mireAppend(rpmMireMode mode, int tag, const char * pattern,
-		miRE * mi_rep, int * mi_nrep)
-	/*@modifies *mi_rep, *mi_nrep @*/
-{
-    miRE mire;
-
-    mire = (*mi_rep);
-/*@-refcounttrans@*/
-    mire = xrealloc(mire, ((*mi_nrep) + 1) * sizeof(*mire));
-/*@=refcounttrans@*/
-    (*mi_rep) = mire;
-    mire += (*mi_nrep);
-    (*mi_nrep)++;
-    memset(mire, 0, sizeof(*mire));
-    mire->mode = mode;
-    mire->tag = tag;
-    return mireRegcomp(mire, pattern);
-}
-/*@=onlytrans@*/
-
 /**
  * Read and configure /etc/rpm/platform patterns.
  * @param platform	path to platform patterns
@@ -441,8 +435,7 @@ static rpmRC rpmPlatform(const char * platform)
 		rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     CVOG_t cvog = NULL;
-    uint8_t * b = NULL;
-    ssize_t blen = 0;
+    rpmiob iob = NULL;
     int init_platform = 0;
     miRE mi_re = NULL;
     int mi_nre = 0;
@@ -450,14 +443,14 @@ static rpmRC rpmPlatform(const char * platform)
     rpmRC rc;
     int xx;
 
-    rc = rpmioSlurp(platform, &b, &blen);
+    rc = rpmiobSlurp(platform, &iob);
 
-    if (rc || b == NULL || blen <= 0) {
+    if (rc || iob == NULL) {
 	rc = RPMRC_FAIL;
 	goto exit;
     }
 
-    p = (char *)b;
+    p = (char *)iob->b;
     for (pe = p; p && *p; p = pe) {
 	pe = strchr(p, '\n');
 	if (pe)
@@ -472,8 +465,9 @@ static rpmRC rpmPlatform(const char * platform)
 	    char * t = p + strlen(p);
 	    while (--t > p && xisspace(*t))
 		*t = '\0';
-	    if (t > p)
-		xx = mireAppend(RPMMIRE_REGEX, 0, p, &mi_re, &mi_nre);
+	    if (t > p) {
+		xx = mireAppend(RPMMIRE_REGEX, 0, p, NULL, &mi_re, &mi_nre);
+	    }
 	    continue;
 	}
 
@@ -491,7 +485,7 @@ static rpmRC rpmPlatform(const char * platform)
 		(cvog && *cvog->gnu ? "-" : NULL),
 		(cvog ? cvog->gnu : NULL), NULL);
 #endif
-	xx = mireAppend(RPMMIRE_STRCMP, 0, p, &mi_re, &mi_nre);
+	xx = mireAppend(RPMMIRE_STRCMP, 0, p, NULL, &mi_re, &mi_nre);
 	p = _free(p);
 	
 	init_platform++;
@@ -503,9 +497,7 @@ exit:
 	cvog->str = _free(cvog->str);
 	cvog = _free(cvog);
     }
-/*@-modobserver@*/
-    b = _free(b);
-/*@=modobserver@*/
+    iob = rpmiobFree(iob);
     if (rc == RPMRC_OK) {
 	platpat = mireFreeAll(platpat, nplatpat);
 	platpat = mi_re;
@@ -515,6 +507,139 @@ exit:
 }
 /*@=onlytrans@*/
 
+#if defined(WITH_CPUINFO) && defined(WITH_SYCK)
+static inline int rpmCpuinfoMatch(const char * feature, const char * EVR, rpmds cpuinfo)
+{
+    rpmds cpufeature = rpmdsSingle(RPMTAG_REQUIRENAME, feature, EVR, RPMSENSE_PROBE);
+    int ret = rpmdsMatch(cpufeature, cpuinfo);
+
+    (void)rpmdsFree(cpufeature);
+    cpufeature = NULL;
+    return ret;
+}
+
+static rpmRC rpmCpuinfo(void)
+{
+    rpmRC rc = RPMRC_FAIL;
+    const char *cpu, *_cpuinfo_path;
+    miRE mi_re = NULL;
+    int mi_nre = 0, xx, i;
+    CVOG_t cvog = NULL;
+    struct stat st;
+    char *yaml;
+    rpmsyck_node *tmp, node;
+    rpmSyck cpuinfoYaml;
+    FD_t fd;
+
+    _cpuinfo_path = rpmGetPath("%{?_rpmhome}%{!?_rpmhome:" USRLIBRPM "}/cpuinfo.yaml", NULL);
+    if(Stat(_cpuinfo_path, &st))
+	return rc;
+
+    fd = Fopen(_cpuinfo_path, "r");
+    _cpuinfo_path = _free(_cpuinfo_path);
+    yaml = xcalloc(st.st_size+1, 1);
+    Fread(yaml, 1, st.st_size, fd);
+    Fclose(fd);
+
+    xx = rpmdsCpuinfo(&cpuinfoP, NULL);
+    cpuinfoYaml = rpmSyckLoad(yaml);
+    yaml = _free(yaml);
+    htGetEntry(cpuinfoYaml->firstNode->value.map, "cpuinfo", &tmp, NULL, NULL);
+    node = tmp[0]->value.seq;
+
+    /* TODO: cleanup.. */
+    for(i = 0; node[i].type != T_END; i++) {
+	if(node[i].type == T_MAP) {
+	    rpmsyck_node *tmp;
+	    if(htHasEntry(node[i].value.map, "Family")) {
+		htGetEntry(node[i].value.map, "Family", &tmp, NULL, NULL);
+		const char *family = tmp[0]->value.key;
+		int j;
+		hashTable cpus = NULL; 
+		if(rpmCpuinfoMatch(family, "", cpuinfoP)) {
+		    if(htHasEntry(node[i].value.map, "Arch")) {
+			htGetEntry(node[i].value.map, "Arch", &tmp, NULL, NULL);
+			rpmsyck_node arch = tmp[0]->value.seq;
+			for(j = 0; arch[j].type != T_END; j++);
+			cpus = htCreate(j*2, 0, 0, NULL, NULL);
+			for(j = 0; arch[j].type != T_END; j++) {
+			    if(htHasEntry(arch[j].value.map, "Extends")) {
+				if(htGetEntry(arch[j].value.map, "Extends", &tmp, NULL, NULL) &&
+					tmp[0]->type == T_STR && !htHasEntry(cpus, tmp[0]->value.key))
+				    continue;
+			    }
+			    if(htHasEntry(arch[j].value.map, "Features")) {
+				htGetEntry(arch[j].value.map, "Features", &tmp, NULL, NULL);
+				rpmsyck_node features = tmp[0]->value.seq;
+				int k, match = 0;
+				for(k = 0; features[k].type != T_END; k++)
+				    if(features[k].type == T_STR && !(match = rpmCpuinfoMatch(features[k].value.key, "", cpuinfoP))) break;
+				if(!match) continue;
+			    }
+			    if(htHasEntry(arch[j].value.map, "Name")) {
+				htGetEntry(arch[j].value.map, "Name", &tmp, NULL, NULL);
+				if(tmp[0]->type != T_STR) continue;
+				const char *name = tmp[0]->value.key;
+				rpmsyck_node alias = NULL;
+				if(htHasEntry(arch[j].value.map, "Alias")) {
+				    htGetEntry(arch[j].value.map, "Alias", &tmp, NULL, NULL);
+				    alias = tmp[0]->value.seq;
+				}
+				htAddEntry(cpus, name, alias);
+			    }
+			}
+		    }
+		    if(htHasEntry(node[i].value.map, "Priority")) {
+			htGetEntry(node[i].value.map, "Priority", &tmp, NULL, NULL);
+			rpmsyck_node priority = tmp[0]->value.seq;
+			int j;
+			for(j = 0; priority[j].type != T_END; j++)
+			    if(htHasEntry(cpus, priority[j].value.key)) {
+				xx = mireAppend(RPMMIRE_REGEX, 0, priority[j].value.key, NULL, &mi_re, &mi_nre);
+				htGetEntry(cpus, priority[j].value.key, &tmp, NULL, NULL);
+				if(tmp[0]) {
+				    rpmsyck_node alias = tmp[0];
+				    int k;
+				    for(k = 0; alias[k].type != T_END; k++)
+					xx = mireAppend(RPMMIRE_REGEX, 0, alias[k].value.key, NULL, &mi_re, &mi_nre);
+				}
+			    }
+		    }
+		}
+		if(cpus) cpus = htFree(cpus);
+	    }
+	}
+    }
+
+    cpuinfoYaml = rpmSyckFree(cpuinfoYaml);
+
+    xx = mireAppend(RPMMIRE_REGEX, 0, "noarch", NULL, &mi_re, &mi_nre);
+
+    cpu = mi_re[0].pattern;
+    if(cpu != NULL)
+    {
+	if (!parseCVOG(cpu, &cvog) && cvog != NULL) {
+	    addMacro(NULL, "_host_cpu", NULL, cvog->cpu, -1);
+	    addMacro(NULL, "_host_vendor", NULL, cvog->vendor, -1);
+	    addMacro(NULL, "_host_os", NULL, cvog->os, -1);
+	}
+	if (cvog) {
+	    cvog->str = _free(cvog->str);
+	    cvog = _free(cvog);
+	}
+
+	rc = RPMRC_OK;
+	if (rc == RPMRC_OK) {
+	    platpat = mireFreeAll(platpat, nplatpat);
+	    platpat = mi_re;
+	    nplatpat = mi_nre;
+	}
+
+    }
+    return rc;
+}
+#endif
+
 /*@-onlytrans@*/	/* XXX miRE array, not refcounted. */
 int rpmPlatformScore(const char * platform, void * mi_re, int mi_nre)
 {
@@ -528,7 +653,7 @@ int rpmPlatformScore(const char * platform, void * mi_re, int mi_nre)
 
     if ((mire = mi_re) != NULL)
     for (i = 0; i < mi_nre; i++) {
-	if (!mireRegexec(mire + i, platform))
+	if (mireRegexec(mire + i, platform, 0) >= 0)
 	    return (i + 1);
     }
     return 0;
@@ -560,6 +685,11 @@ static void defaultMachine(/*@out@*/ const char ** arch,
     int rc;
 
     while (!gotDefaults) {
+#if defined(RPM_VENDOR_WINDRIVER)
+	const char * _platform = rpmGetPath(__etcrpm, "/platform", NULL);
+#else
+	const char * _platform = platform;
+#endif
 	CVOG_t cvog = NULL;
 #if defined(RPM_VENDOR_OPENPKG) /* larger-utsname */
 	const char *cp;
@@ -612,10 +742,12 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 	/* allow the path to the "platforms" file be overridden under run-time */
 	cp = rpmExpand("%{?__platform}", NULL);
 	if (cp == NULL || cp[0] == '\0')
-	    cp = platform;
+	    cp = _platform;
 	if (rpmPlatform(cp) == RPMRC_OK) {
+#elif defined(WITH_CPUINFO) && defined(WITH_SYCK)
+	if (rpmPlatform(_platform) == RPMRC_OK || rpmCpuinfo() == RPMRC_OK) {
 #else
-	if (rpmPlatform(platform) == RPMRC_OK) {
+	if (rpmPlatform(_platform) == RPMRC_OK) {
 #endif
 	    const char * s;
 	    gotDefaults = 1;
@@ -635,9 +767,12 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 
 #if defined(RPM_VENDOR_OPENPKG) /* explicit-platform */
 	/* cleanup after above processing */
-	if (cp != NULL && cp != platform)
+	if (cp != NULL && cp != _platform)
 	    cp = _free(cp);
 #endif
+#if defined(RPM_VENDOR_WINDRIVER)
+	_platform = _free(_platform);
+#endif
 
 	if (configTarget && !parseCVOG(configTarget, &cvog) && cvog != NULL) {
 	    gotDefaults = 1;
@@ -662,9 +797,18 @@ static void defaultMachine(/*@out@*/ const char ** arch,
     if (os) *os = un.sysname;
 }
 
-void rpmSetTables(int archTable, int osTable)
-	/*@globals currTables @*/
-	/*@modifies currTables @*/
+/** \ingroup rpmrc
+ * @deprecated Use addMacro to set _target_* macros.
+ * @todo	Eliminate in rpm-5.1.
+ # @note Only used by build code.
+ * @param archTable
+ * @param osTable
+ */
+static void rpmSetTables(int archTable, int osTable)
+	/*@globals currTables, rpmGlobalMacroContext, h_errno,
+		fileSystem, internalState @*/
+	/*@modifies currTables, rpmGlobalMacroContext,
+		fileSystem, internalState @*/
 {
     const char * arch, * os;
 
@@ -741,7 +885,28 @@ static void getMachineInfo(int type, /*@null@*/ /*@out@*/ const char ** name,
 	if (name) *name = canon->short_name;
     } else {
 	if (num) *num = 255;
+#if defined(WITH_CPUINFO)
+	if (name)
+	{
+	    if(type == ARCH)
+	    {
+		int i, j, n;
+	    	ARGV_t archs = NULL;
+		char *pref = rpmExpand("%{?_prefer_target_cpu}", NULL);
+
+		(void) argvSplit(&archs, pref, " ");
+		for(i = 0, n = argvCount(archs); (i < n && !*name); i++)
+    		    if((j = rpmPlatformScore(archs[i], platpat, nplatpat)) > 0)
+    			*name = ((miRE)platpat)[j-1].pattern;
+
+		archs = argvFree(archs);
+		pref = _free(pref);
+	    }
+	    if(!*name) *name = current[type];
+	}
+#else
 	if (name) *name = current[type];
+#endif
     }
 }
 
@@ -794,7 +959,7 @@ static void rpmRebuildTargetVars(const char ** target, const char ** canontarget
     }
     if (ca != NULL)
     for (x = 0; ca[x] != '\0'; x++)
-	ca[x] = xtolower(ca[x]);
+	ca[x] = (char)xtolower(ca[x]);
 
     if (co == NULL) {
 	const char *o = NULL;
@@ -803,7 +968,7 @@ static void rpmRebuildTargetVars(const char ** target, const char ** canontarget
     }
     if (co != NULL)
     for (x = 0; co[x] != '\0'; x++)
-	co[x] = xtolower(co[x]);
+	co[x] = (char)xtolower(co[x]);
 
     /* XXX For now, set canonical target to arch-os */
     if (ct == NULL) {
@@ -838,7 +1003,8 @@ void rpmFreeRpmrc(void)
 {
     int i, j, k;
 
-    platpat = mireFreeAll(platpat, nplatpat);
+    (void)mireFreeAll(platpat, nplatpat);
+    platpat = NULL;
     nplatpat = 0;
 
     for (i = 0; i < RPM_MACHTABLE_COUNT; i++) {
@@ -896,8 +1062,8 @@ void rpmFreeRpmrc(void)
  * Read macro configuration file(s).
  * @return		0 on success
  */
-static int rpmReadRC(void)
-	/*@globals defaultsInitialized, rpmMacrofiles,
+static int rpmReadRC(const char *macrofiles)
+	/*@globals defaultsInitialized,
 		rpmGlobalMacroContext, rpmCLIMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies defaultsInitialized, rpmGlobalMacroContext,
@@ -911,7 +1077,7 @@ static int rpmReadRC(void)
     }
 
     /* Read macro files. */
-    {	const char *mfpath = rpmExpand(rpmMacrofiles, NULL);
+    {	const char *mfpath = rpmExpand(macrofiles, NULL);
 	    
 	if (mfpath != NULL) {
 	    rpmInitMacros(NULL, mfpath);
@@ -922,12 +1088,17 @@ static int rpmReadRC(void)
     return rc;
 }
 
-int rpmReadConfigFiles(/*@unused@*/ const char * file, const char * target)
-	/*@globals configTarget @*/
+int rpmReadConfigFiles(/*@unused@*/ const char * file,
+		const char * target)
+	/*@globals configTarget, rpmMacrofiles @*/
 	/*@modifies configTarget @*/
 {
     mode_t mode = 0022;
 
+#ifdef PREMACROFILES
+    if (rpmReadRC(PREMACROFILES)) return -1;
+#endif
+
     /* Reset umask to its default umask(2) value. */
     mode = umask(mode);
 
@@ -939,7 +1110,7 @@ int rpmReadConfigFiles(/*@unused@*/ const char * file, const char * target)
 
     /* Read the files */
 /*@-globs@*/
-    if (rpmReadRC()) return -1;
+    if (rpmReadRC(rpmMacrofiles)) return -1;
 /*@=globs@*/
 
     /* Reset target macros */
@@ -950,6 +1121,7 @@ int rpmReadConfigFiles(/*@unused@*/ const char * file, const char * target)
     {	const char *cpu = rpmExpand("%{_target_cpu}", NULL);
 	const char *os = rpmExpand("%{_target_os}", NULL);
 	rpmSetMachine(cpu, os);
+
 	cpu = _free(cpu);
 	os = _free(os);
     }
@@ -972,6 +1144,7 @@ int rpmShowRC(FILE * fp)
     int i;
     machEquivTable equivTable;
     int xx;
+    miRE mire;
 
     /* the caller may set the build arch which should be printed here */
     fprintf(fp, "ARCHITECTURE AND OS:\n");
@@ -995,9 +1168,8 @@ int rpmShowRC(FILE * fp)
     fprintf(fp, "install os            : %s\n", current[OS]);
 
     fprintf(fp, "compatible archs      :");
-    equivTable = &tables[RPM_MACHTABLE_INSTARCH].equiv;
-    for (i = 0; i < equivTable->count; i++)
-	fprintf(fp," %s", equivTable->list[i].name);
+    for (mire = platpat, i = 0; i < nplatpat; i++)
+	fprintf(fp, " %s", mire[i].pattern);
     fprintf(fp, "\n");
 
     fprintf(fp, "compatible os's       :");
@@ -1009,10 +1181,25 @@ int rpmShowRC(FILE * fp)
     {	const char * s = rpmExpand("%{?optflags}", NULL);
 	fprintf(fp, "%-21s : %s\n", "optflags", ((s && *s) ? s : "(not set)"));
 	s = _free(s);
+
+#ifdef	WITH_LUA
+	fprintf(fp, "\nLUA MODULES:\n");
+/*@-globs@*/
+	s = rpmExpand(rpmluaFiles, NULL);
+/*@=globs@*/
+	fprintf(fp, "%-21s : %s\n", "luafiles", ((s && *s) ? s : "(not set)"));
+	s = _free(s);
+/*@-globs@*/
+	s = rpmExpand(rpmluaPath, NULL);
+/*@=globs@*/
+	fprintf(fp, "%-21s : %s\n", "luapath", ((s && *s) ? s : "(not set)"));
+	s = _free(s);
+#endif
+
+	fprintf(fp, "\nMACRO DEFINITIONS:\n");
 /*@-globs@*/
 	s = rpmExpand(rpmMacrofiles, NULL);
 /*@=globs@*/
-	fprintf(fp, "\nMACRO DEFINITIONS:\n");
 	fprintf(fp, "%-21s : %s\n", "macrofiles", ((s && *s) ? s : "(not set)"));
 	s = _free(s);
     }
@@ -1022,14 +1209,16 @@ int rpmShowRC(FILE * fp)
 	xx = rpmdsSysinfo(PRCO, NULL);
 	ds = rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME);
 	if (ds != NULL) {
-	    fprintf(fp, _("Configured system provides (from /etc/rpm/sysinfo):\n"));
+	    const char * fn = (_sysinfo_path ? _sysinfo_path : "/etc/rpm/sysinfo");
+	    fprintf(fp, _("Configured system provides (from %s):\n"), fn);
 	    ds = rpmdsInit(ds);
 	    while (rpmdsNext(ds) >= 0) {
 		const char * DNEVR = rpmdsDNEVR(ds);
 		if (DNEVR != NULL)
 		    fprintf(fp, "    %s\n", DNEVR+2);
 	    }
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	    fprintf(fp, "\n");
 	}
 	PRCO = rpmdsFreePRCO(PRCO);
@@ -1044,23 +1233,34 @@ int rpmShowRC(FILE * fp)
 	    if (DNEVR != NULL)
 		fprintf(fp, "    %s\n", DNEVR+2);
 	}
-	ds = rpmdsFree(ds);
+	(void)rpmdsFree(ds);
+	ds = NULL;
 	fprintf(fp, "\n");
 
-	xx = rpmdsCpuinfo(&ds, NULL);
-	if (ds != NULL) {
+	if(cpuinfoP == NULL)
+	    xx = rpmdsCpuinfo(&cpuinfoP, NULL);
+	if (cpuinfoP != NULL) {
+#if defined(WITH_CPUINFO)
+	    const char * fn = "libcpuinfo";
+#else
+	    const char * fn = (_cpuinfo_path ? _cpuinfo_path : "/proc/cpuinfo");
+#endif
 	    fprintf(fp,
-		_("Features provided by current cpuinfo (from /proc/cpuinfo):\n"));
-	    ds = rpmdsInit(ds);
-	    while (rpmdsNext(ds) >= 0) {
-		const char * DNEVR = rpmdsDNEVR(ds);
+		_("Features provided by current cpuinfo (from %s):\n"), fn);
+	    cpuinfoP = rpmdsInit(cpuinfoP);
+	    while (rpmdsNext(cpuinfoP) >= 0) {
+		const char * DNEVR = rpmdsDNEVR(cpuinfoP);
 		if (DNEVR != NULL)
 		    fprintf(fp, "    %s\n", DNEVR+2);
 	    }
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(cpuinfoP);
+	    cpuinfoP = NULL;
+
 	    fprintf(fp, "\n");
 	}
+    }
 
+    if (rpmIsDebug()) {
 	xx = rpmdsGetconf(&ds, NULL);
 	if (ds != NULL) {
 	    fprintf(fp,
@@ -1071,7 +1271,8 @@ int rpmShowRC(FILE * fp)
 		if (DNEVR != NULL)
 		    fprintf(fp, "    %s\n", DNEVR+2);
 	    }
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	    fprintf(fp, "\n");
 	}
 
@@ -1085,7 +1286,8 @@ int rpmShowRC(FILE * fp)
 		if (DNEVR != NULL)
 		    fprintf(fp, "    %s\n", DNEVR+2);
 	    }
-	    ds = rpmdsFree(ds);
+	    (void)rpmdsFree(ds);
+	    ds = NULL;
 	    fprintf(fp, "\n");
 	}
     }
