@@ -5,17 +5,28 @@
 #include "system.h"
 
 #include <rpmio.h>
+#include <rpmiotypes.h>
+#include <rpmcb.h>
+#include <argv.h>
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
+#include <pkgio.h>
+#include <rpmdb.h>
+
+#include <rpmds.h>
+#include "manifest.h"
+#include "misc.h"		/* XXX rpmGlob() */
+
 #define	_RPMTE_INTERNAL		/* XXX findErases needs rpmte internals. */
 #define	_RPMTS_INTERNAL		/* XXX ts->teErase, ts->probs */
 #define _RPMTS_PRINT
+#include <rpmgi.h>		/* XXX rpmgiEscapeSpaces */
+
 #include <rpmcli.h>
+#define	_RPMROLLBACK_INTERNAL
+#include <rpmrollback.h>
 
-#include "rpmdb.h"
-#include "rpmds.h"
-
-#include "manifest.h"
-#include "misc.h"		/* XXX rpmGlob() */
-#include "rpmgi.h"		/* XXX rpmgiEscapeSpaces */
 #include "debug.h"
 
 /*@access FD_t @*/	/* XXX void * arg */
@@ -44,7 +55,8 @@ IDTX IDTXfree(IDTX idtx)
 	if (idtx->idt)
 	for (i = 0; i < idtx->nidt; i++) {
 	    IDT idt = idtx->idt + i;
-	    idt->h = headerFree(idt->h);
+	    (void)headerFree(idt->h);
+	    idt->h = NULL;
 	    idt->key = _free(idt->key);
 	}
 	idtx->idt = _free(idtx->idt);
@@ -57,7 +69,7 @@ IDTX IDTXnew(void)
 {
     IDTX idtx = xcalloc(1, sizeof(*idtx));
     idtx->delta = 10;
-    idtx->size = sizeof(*((IDT)0));
+    idtx->size = (int)sizeof(*((IDT)0));
     return idtx;
 }
 
@@ -85,20 +97,20 @@ IDTX IDTXsort(IDTX idtx)
     return idtx;
 }
 
-IDTX IDTXload(rpmts ts, rpmTag tag, uint32_t rbtid)
+IDTX IDTXload(rpmts ts, rpmTag tag, rpmuint32_t rbtid)
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     IDTX idtx = NULL;
-    rpmdbMatchIterator mi;
+    rpmmi mi;
     Header h;
-    uint32_t tid;
+    rpmuint32_t tid;
     int xx;
 
     mi = rpmtsInitIterator(ts, tag, NULL, 0);
 #ifdef	NOTYET
-    (void) rpmdbSetIteratorRE(mi, RPMTAG_NAME, RPMMIRE_DEFAULT, '!gpg-pubkey');
+    (void) rpmmiAddPattern(mi, RPMTAG_NAME, RPMMIRE_DEFAULT, '!gpg-pubkey');
 #endif
-    while ((h = rpmdbNextIterator(mi)) != NULL) {
+    while ((h = rpmmiNext(mi)) != NULL) {
 	he->tag = tag;
 	xx = headerGet(h, he, 0);
 	if (!xx || he->p.ui32p == NULL)
@@ -106,7 +118,7 @@ IDTX IDTXload(rpmts ts, rpmTag tag, uint32_t rbtid)
 	tid = (he->p.ui32p ? he->p.ui32p[0] : 0);
 	he->p.ptr = _free(he->p.ptr);
 
-	if (tid == 0 || tid == -1)
+	if (tid == 0 || tid == 0xffffffff)
 	    continue;
 
 	/* Don't bother with headers installed prior to the rollback goal. */
@@ -124,22 +136,22 @@ IDTX IDTXload(rpmts ts, rpmTag tag, uint32_t rbtid)
 	    idt->done = 0;
 	    idt->h = headerLink(h);
 	    idt->key = NULL;
-	    idt->instance = rpmdbGetIteratorOffset(mi);
+	    idt->instance = rpmmiInstance(mi);
 	    idt->val.u32 = tid;
 	}
 	idtx->nidt++;
     }
-    mi = rpmdbFreeIterator(mi);
+    mi = rpmmiFree(mi);
 
     return IDTXsort(idtx);
 }
 
-IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag, uint32_t rbtid)
+IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag, rpmuint32_t rbtid)
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     IDTX idtx = NULL;
     Header h;
-    uint32_t tid;
+    rpmuint32_t tid;
     FD_t fd;
     const char ** av = NULL;
     const char * fn;
@@ -212,7 +224,8 @@ assert(!strcmp(av[i], origin));
 	}
 	idtx->nidt++;
 bottom:
-	h = headerFree(h);
+	(void)headerFree(h);
+	h = NULL;
     }
 
     for (i = 0; i < ac; i++)
@@ -297,7 +310,7 @@ static int findErases(rpmts ts, /*@null@*/ rpmte p, unsigned thistid,
 	    const char ** flinkPkgid = NULL;
 	    const char ** flinkHdrid = NULL;
 	    const char ** flinkNEVRA = NULL;
-	    uint32_t pn, hn, nn;
+	    rpmuint32_t pn, hn, nn;
 	    int bingo;
 
 	    he->tag = RPMTAG_BLINKPKGID;
@@ -398,7 +411,7 @@ int rpmRollback(rpmts ts, QVA_t ia, const char ** argv)
     int vsflags, ovsflags;
     int numAdded;
     int numRemoved;
-    int _unsafe_rollbacks = 0;
+    unsigned int _unsafe_rollbacks = 0;
     rpmtransFlags transFlags = ia->transFlags;
     rpmdepFlags depFlags = ia->depFlags;
     int xx;
@@ -488,7 +501,7 @@ int rpmRollback(rpmts ts, QVA_t ia, const char ** argv)
 	/* Is this transaction excluded from the rollback? */
 	if (ia->rbtidExcludes != NULL && ia->numrbtidExcludes > 0)
 	{
-	    uint32_t *excludedTID;
+	    rpmuint32_t *excludedTID;
 	    int excluded = 0;
 	    for(excludedTID = ia->rbtidExcludes; 
 		excludedTID < ia->rbtidExcludes + ia->numrbtidExcludes;
@@ -529,7 +542,6 @@ int rpmRollback(rpmts ts, QVA_t ia, const char ** argv)
 	(void) rpmtsSetFlags(ts, transFlags);
 	(void) rpmtsSetDFlags(ts, depFlags);
 	ts->probs = rpmpsFree(ts->probs);
-	ts->probs = rpmpsCreate();
 
 	/* Install the previously erased packages for this transaction. 
 	 */
@@ -555,7 +567,8 @@ int rpmRollback(rpmts ts, QVA_t ia, const char ** argv)
 		if (rc < 0)
 		    goto exit;
 #ifdef	NOTYET
-	    	rp->h = headerFree(rp->h);
+	    	(void)headerFree(rp->h);
+		rpm->h = NULL;
 #endif
 		rp->done = 1;
 	    }
@@ -582,7 +595,7 @@ assert(ip->done || ia->no_rollback_links);
 	    if (!(ip->done || ia->no_rollback_links)) {
 		numRemoved++;
 
-		if (_unsafe_rollbacks)
+		if (_unsafe_rollbacks != 0)
 		    rpmcliPackagesTotal++;
 
 		if (!(ia->installInterfaceFlags & ifmask))
