@@ -5,7 +5,7 @@
  * \file lib/rpmte.h
  * Structures used for an "rpmte" transaction element.
  */
-#include <argv.h>
+#include <rpmfi.h>
 
 /**
  */
@@ -22,7 +22,7 @@ typedef /*@abstract@*/ struct tsortInfo_s *		tsortInfo;
 /** \ingroup rpmte
  * Transaction element iterator.
  */
-typedef /*@abstract@*/ struct rpmtsi_s *		rpmtsi;
+typedef /*@abstract@*/ /*@refcounted@*/ struct rpmtsi_s *rpmtsi;
 
 /** \ingroup rpmte
  * Transaction element type.
@@ -33,6 +33,9 @@ typedef enum rpmElementType_e {
 } rpmElementType;
 
 #if	defined(_RPMTE_INTERNAL)
+#include <argv.h>
+#include <rpmal.h>
+
 /** \ingroup rpmte
  * Dependncy ordering information.
  */
@@ -76,16 +79,17 @@ typedef struct sharedFileInfo_s *		sharedFileInfo;
  * Replaced file cross reference.
  */
 struct sharedFileInfo_s {
-    int pkgFileNum;
-    int otherFileNum;
-    int otherPkg;
-    int isRemoved;
+    rpmuint32_t pkgFileNum;
+    rpmuint32_t otherFileNum;
+    rpmuint32_t otherPkg;
+    rpmuint32_t isRemoved;
 };
 
 /** \ingroup rpmte
  * A single package instance to be installed/removed atomically.
  */
 struct rpmte_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
     rpmElementType type;	/*!< Package disposition (installed/removed). */
 
 /*@refcounted@*/ /*@relnull@*/
@@ -98,14 +102,20 @@ struct rpmte_s {
     const char * pkgid;		/*!< Package identifier (header+payload md5). */
 /*@only@*/ /*@relnull@*/
     const char * hdrid;		/*!< Package header identifier (header sha1). */
-/*@owned@*/
+/*@only@*/ /*@null@*/
+    const char * sourcerpm;	/*!< Source package. */
+/*@only@*/
     const char * name;		/*!< Name: */
 /*@only@*/ /*@null@*/
     char * epoch;
-/*@dependent@*/ /*@null@*/
+/*@only@*/ /*@null@*/
     char * version;		/*!< Version: */
-/*@dependent@*/ /*@null@*/
+/*@only@*/ /*@null@*/
     char * release;		/*!< Release: */
+#ifdef	RPM_VENDOR_MANDRIVA
+/*@only@*/ /*@null@*/
+    char * distepoch;
+#endif
 /*@only@*/ /*@null@*/
     const char * arch;		/*!< Architecture hint. */
 /*@only@*/ /*@null@*/
@@ -128,8 +138,10 @@ struct rpmte_s {
 /*@refcounted@*/ /*@null@*/
     rpmfi fi;			/*!< File information. */
 
-    uint32_t color;		/*!< Color bit(s) from package dependencies. */
-    uint32_t pkgFileSize;	/*!< No. of bytes in package file (approx). */
+    rpmuint32_t depFlags;	/*!< Package depFlags mask. */
+    rpmuint32_t transFlags;	/*!< Package transFlags mask. */
+    rpmuint32_t color;		/*!< Color bit(s) from package dependencies. */
+    rpmuint32_t pkgFileSize;	/*!< No. of bytes in package file (approx). */
 
 /*@exposed@*/ /*@dependent@*/ /*@null@*/
     fnpyKey key;		/*!< (TR_ADDED) Retrieval key. */
@@ -148,6 +160,8 @@ struct rpmte_s {
     struct rpmChainLink_s flink;/*!< Forward link info to installed element. */
     int linkFailed;		/*!< Did the linked element upgrade succeed? */
     int done;			/*!< Has the element been installed/erased? */
+    rpmuint32_t originTid[2];	/*!< Transaction id of first install. */
+    rpmuint32_t originTime[2];	/*!< Time that package was first installed. */
 
     int installed;		/*!< Was the header installed? */
     int downgrade;		/*!< Adjust package count on downgrades. */
@@ -161,18 +175,26 @@ struct rpmte_s {
 	    int dboffset;
 	} removed;
     } u;
-
+#if defined(__LCLINT__)
+/*@refs@*/
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 
 /** \ingroup rpmte
  * Iterator across transaction elements, forward on install, backward on erase.
  */
 struct rpmtsi_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
 /*@refcounted@*/
     rpmts ts;		/*!< transaction set. */
     int reverse;	/*!< reversed traversal? */
     int ocsave;		/*!< last returned iterator index. */
     int oc;		/*!< iterator index. */
+#if defined(__LCLINT__)
+/*@refs@*/
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 
 #endif	/* _RPMTE_INTERNAL */
@@ -181,15 +203,18 @@ struct rpmtsi_s {
 extern "C" {
 #endif
 
+#if	defined(_RPMTE_INTERNAL)
 /** \ingroup rpmte
  * Destroy a transaction element.
  * @param te		transaction element
- * @return		NULL always
+ * @return		NULL on last dereference
  */
-/*@null@*/
+/*@unused@*/ /*@null@*/
 rpmte rpmteFree(/*@only@*/ /*@null@*/ rpmte te)
 	/*@globals fileSystem @*/
 	/*@modifies te, fileSystem @*/;
+#define rpmteFree(_te)  \
+    ((rpmte) rpmioFreePoolItem((rpmioItem)(_te), __FUNCTION__, __FILE__, __LINE__))
 
 /** \ingroup rpmte
  * Create a transaction element.
@@ -210,6 +235,7 @@ rpmte rpmteNew(const rpmts ts, Header h, rpmElementType type,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ alKey pkgKey)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
+#endif	/* _RPMTE_INTERNAL */
 
 /** \ingroup rpmte
  * Retrieve header from transaction element.
@@ -272,6 +298,17 @@ extern const char * rpmteV(rpmte te)
 extern const char * rpmteR(rpmte te)
 	/*@*/;
 
+#ifdef	RPM_VENDOR_MANDRIVA
+/** \ingroup rpmte
+ * Retrieve distepoch string of transaction element.
+ * @param te		transaction element
+ * @return		distepoch string
+ */
+/*@observer@*/ /*@null@*/
+extern const char * rpmteD(rpmte te)
+	/*@*/;
+#endif
+
 /** \ingroup rpmte
  * Retrieve arch string of transaction element.
  * @param te		transaction element
@@ -303,7 +340,7 @@ extern int rpmteIsSource(rpmte te)
  * @param te		transaction element
  * @return		color bits
  */
-uint32_t rpmteColor(rpmte te)
+rpmuint32_t rpmteColor(rpmte te)
 	/*@*/;
 
 /** \ingroup rpmte
@@ -312,7 +349,7 @@ uint32_t rpmteColor(rpmte te)
  * @param color		new color bits
  * @return		previous color bits
  */
-uint32_t rpmteSetColor(rpmte te, uint32_t color)
+rpmuint32_t rpmteSetColor(rpmte te, rpmuint32_t color)
 	/*@modifies te @*/;
 
 /** \ingroup rpmte
@@ -338,7 +375,25 @@ void rpmteSetDBInstance(rpmte te, unsigned int instance)
  * @param te		transaction element
  * @return		size in bytes of package file.
  */
-uint32_t rpmtePkgFileSize(rpmte te)
+rpmuint32_t rpmtePkgFileSize(rpmte te)
+	/*@*/;
+
+/** \ingroup rpmte
+ * Retrieve transaction start time that package was first installed.
+ * @param te		transaction element
+ * @return		origin time
+ */
+/*@observer@*/
+rpmuint32_t * rpmteOriginTid(rpmte te)
+	/*@*/;
+
+/** \ingroup rpmte
+ * Retrieve time that package was first installed.
+ * @param te		transaction element
+ * @return		origin time
+ */
+/*@observer@*/
+rpmuint32_t * rpmteOriginTime(rpmte te)
 	/*@*/;
 
 /** \ingroup rpmte
@@ -475,6 +530,7 @@ void rpmteNewTSI(rpmte te)
 void rpmteCleanDS(rpmte te)
 	/*@modifies te @*/;
 
+#if	defined(_RPMTE_INTERNAL)
 /** \ingroup rpmte
  * Retrieve pkgKey of TR_ADDED transaction element.
  * @param te		transaction element
@@ -494,6 +550,7 @@ alKey rpmteAddedKey(rpmte te)
 alKey rpmteSetAddedKey(rpmte te,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ alKey npkgKey)
 	/*@modifies te @*/;
+#endif	/* _RPMTE_INTERNAL */
 
 /** \ingroup rpmte
  * Retrieve rpmdb instance of TR_REMOVED transaction element.
@@ -545,6 +602,17 @@ extern const char * rpmteHdrid(rpmte te)
 	/*@*/;
 /*@=exportlocal@*/
 
+/** \ingroup rpmte
+ * Retrieve sourcerpm string from transaction element.
+ * @param te		transaction element
+ * @return		sourcerpm string
+ */
+/*@-exportlocal@*/
+/*@observer@*/ /*@null@*/
+extern const char * rpmteSourcerpm(rpmte te)
+	/*@*/;
+/*@=exportlocal@*/
+
 /** \ingroup rpmte
  * Retrieve file handle from transaction element.
  * @param te		transaction element
@@ -599,7 +667,8 @@ void rpmteColorDS(rpmte te, rpmTag tag)
  * @return		0 on success
  */
 int rpmteChain(rpmte p, rpmte q, Header oh, /*@null@*/ const char * msg)
-	/*@modifies p, q, oh @*/;
+	/*@globals internalState @*/
+	/*@modifies p, q, oh, internalState @*/;
 
 #define	RPMTE_CHAIN_END	"CHAIN END"	/*!< End of chain marker. */
 
@@ -614,26 +683,14 @@ int rpmtsiOc(rpmtsi tsi)
 /** \ingroup rpmte
  * Destroy transaction element iterator.
  * @param tsi		transaction element iterator
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@unused@*/ /*@null@*/
-rpmtsi rpmtsiFree(/*@only@*//*@null@*/ rpmtsi tsi)
+rpmtsi rpmtsiFree(/*@killref@*//*@null@*/ rpmtsi tsi)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
-
-/** \ingroup rpmte
- * Destroy transaction element iterator.
- * @param tsi		transaction element iterator
- * @param fn
- * @param ln
- * @return		NULL always
- */
-/*@null@*/
-rpmtsi XrpmtsiFree(/*@only@*//*@null@*/ rpmtsi tsi,
-		const char * fn, unsigned int ln)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-#define	rpmtsiFree(_tsi)	XrpmtsiFree(_tsi, __FILE__, __LINE__)
+#define	rpmtsiFree(_tsi)	\
+	((rpmtsi)rpmioFreePoolItem((rpmioItem)(_tsi), __FUNCTION__, __FILE__, __LINE__))
 
 /** \ingroup rpmte
  * Create transaction element iterator.
@@ -667,8 +724,8 @@ rpmtsi XrpmtsiInit(rpmts ts,
 rpmte rpmtsiNext(rpmtsi tsi, rpmElementType type)
         /*@modifies tsi @*/;
 
+#if	defined(DYING)
 #if !defined(SWIG)
-#if	defined(_RPMTE_INTERNAL)
 /** \ingroup rpmte
  */
 static inline void rpmtePrintID(rpmte p)
