@@ -4,21 +4,22 @@
  */
 #include "system.h"
 
-#include <rpmio_internal.h>
-#include <rpmbc.h>		/* XXX beecrypt base64 */
 #include <rpmio.h>
+#include <rpmiotypes.h>		/* XXX fnpyKey */
+#include <rpmlog.h>
+#include <iosm.h>		/* XXX iosmFileAction */
 #include <rpmurl.h>
 #include <rpmpgp.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
 #include <rpmmacro.h>		/* XXX rpmtsOpenDB() needs rpmGetPath */
-#include <rpmlib.h>
+#include <rpmkeyring.h>
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
+#include <pkgio.h>
 
 #define	_RPMDB_INTERNAL		/* XXX almost opaque sigh */
 #include "rpmdb.h"		/* XXX stealing db->db_mode. */
 
-#define	_RPMEVR_INTERNAL	/* XXX RPMSENSE_KEYRING */
-#include "rpmevr.h"
-
 #include "rpmal.h"
 #include "rpmds.h"
 #include "rpmfi.h"
@@ -31,6 +32,8 @@
 #define	_RPMTS_INTERNAL
 #include "rpmts.h"
 
+#include <rpmcli.h>
+
 #include "fs.h"
 
 /* XXX FIXME: merge with existing (broken?) tests in system.h */
@@ -66,8 +69,9 @@ extern int statvfs (const char * file, /*@out@*/ struct statvfs * buf)
 /*@access FD_t @*/		/* XXX void * arg */
 /*@access rpmdb @*/		/* XXX db->db_chrootDone, NULL */
 
-/*@access rpmps @*/
 /*@access rpmDiskSpaceInfo @*/
+/*@access rpmKeyring @*/
+/*@access rpmps @*/
 /*@access rpmsx @*/
 /*@access rpmte @*/
 /*@access rpmtsi @*/
@@ -84,26 +88,6 @@ int _rpmts_stats = 0;
 /*@unchecked@*/
 int _rpmts_macros = 0;
 
-rpmts XrpmtsUnlink(rpmts ts, const char * msg, const char * fn, unsigned ln)
-{
-/*@-modfilesys@*/
-if (_rpmts_debug)
-fprintf(stderr, "--> ts %p -- %d %s at %s:%u\n", ts, ts->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-    ts->nrefs--;
-    return NULL;
-}
-
-rpmts XrpmtsLink(rpmts ts, const char * msg, const char * fn, unsigned ln)
-{
-    ts->nrefs++;
-/*@-modfilesys@*/
-if (_rpmts_debug)
-fprintf(stderr, "--> ts %p ++ %d %s at %s:%u\n", ts, ts->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-    /*@-refcounttrans@*/ return ts; /*@=refcounttrans@*/
-}
-
 int rpmtsCloseDB(rpmts ts)
 {
     int rc = 0;
@@ -175,10 +159,10 @@ int rpmtsVerifyDB(rpmts ts)
 }
 
 /*@-compdef@*/ /* keyp might not be defined. */
-rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
+rpmmi rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 			const void * keyp, size_t keylen)
 {
-    rpmdbMatchIterator mi;
+    rpmmi mi;
     const char * arch = NULL;
     int xx;
 
@@ -198,12 +182,12 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	while ((c = *s++) != '\0') {
 	    switch (c) {
 	    default:
-		*t++ = c;
+		*t++ = (char)c;
 		/*@switchbreak@*/ break;
 	    case '(':
 		/* XXX Fail if nested parens. */
 		if (level++ != 0) {
-		    rpmlog(RPMLOG_ERR, _("extra '(' in package label: %s\n"), keyp);
+		    rpmlog(RPMLOG_ERR, _("extra '(' in package label: %s\n"), (char *)keyp);
 		    return NULL;
 		}
 		/* Parse explicit epoch. */
@@ -221,7 +205,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	    case ')':
 		/* XXX Fail if nested parens. */
 		if (--level != 0) {
-		    rpmlog(RPMLOG_ERR, _("missing '(' in package label: %s\n"), keyp);
+		    rpmlog(RPMLOG_ERR, _("missing '(' in package label: %s\n"), (char *)keyp);
 		    return NULL;
 		}
 		/* Don't copy trailing ')' */
@@ -229,7 +213,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	    }
 	}
 	if (level) {
-	    rpmlog(RPMLOG_ERR, _("missing ')' in package label: %s\n"), keyp);
+	    rpmlog(RPMLOG_ERR, _("missing ')' in package label: %s\n"), (char *)keyp);
 	    return NULL;
 	}
 	*t = '\0';
@@ -242,249 +226,19 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	}
     }
 
-    mi = rpmdbInitIterator(ts->rdb, rpmtag, keyp, keylen);
+    mi = rpmmiInit(ts->rdb, rpmtag, keyp, keylen);
 
     /* Verify header signature/digest during retrieve (if not disabled). */
     if (mi && !(rpmtsVSFlags(ts) & RPMVSF_NOHDRCHK))
-	(void) rpmdbSetHdrChk(mi, ts);
+	(void) rpmmiSetHdrChk(mi, ts);
 
     /* Select specified arch only. */
     if (arch != NULL)
-	xx = rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT, arch);
+	xx = rpmmiAddPattern(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT, arch);
     return mi;
 }
 /*@=compdef@*/
 
-rpmRC rpmtsImportPubkey(const rpmts ts, const unsigned char * pkt, ssize_t pktlen)
-{
-    HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
-    static unsigned char zeros[] =
-	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-    const char * afmt = "%{pubkeys:armor}";
-    const char * group = "Public Keys";
-    const char * license = "pubkey";
-    const char * buildhost = "localhost";
-    uint32_t pflags = (RPMSENSE_KEYRING|RPMSENSE_EQUAL);
-    uint32_t zero = 0;
-    pgpDig dig = NULL;
-    pgpDigParams pubp = NULL;
-    const char * d = NULL;
-    const char * enc = NULL;
-    const char * n = NULL;
-    const char * u = NULL;
-    const char * v = NULL;
-    const char * r = NULL;
-    const char * evr = NULL;
-    Header h = NULL;
-    rpmRC rc = RPMRC_FAIL;		/* assume failure */
-    char * t;
-    int xx;
-
-    if (pkt == NULL || pktlen <= 0)
-	return RPMRC_FAIL;
-    if (rpmtsOpenDB(ts, (O_RDWR|O_CREAT)))
-	return RPMRC_FAIL;
-
-/*@-moduncon@*/
-    if ((enc = b64encode(pkt, pktlen)) == NULL)
-	goto exit;
-/*@=moduncon@*/
-
-    dig = pgpDigNew(0);
-
-    /* Build header elements. */
-    (void) pgpPrtPkts(pkt, pktlen, dig, 0);
-    pubp = pgpGetPubkey(dig);
-
-    if (!memcmp(pubp->signid, zeros, sizeof(pubp->signid))
-     || !memcmp(pubp->time, zeros, sizeof(pubp->time))
-     || pubp->userid == NULL)
-	goto exit;
-
-    v = t = xmalloc(16+1);
-    t = stpcpy(t, pgpHexStr(pubp->signid, sizeof(pubp->signid)));
-
-    r = t = xmalloc(8+1);
-    t = stpcpy(t, pgpHexStr(pubp->time, sizeof(pubp->time)));
-
-    n = t = xmalloc(sizeof("gpg()")+8);
-    t = stpcpy( stpcpy( stpcpy(t, "gpg("), v+8), ")");
-
-    /*@-nullpass@*/ /* FIX: pubp->userid may be NULL */
-    u = t = xmalloc(sizeof("gpg()")+strlen(pubp->userid));
-    t = stpcpy( stpcpy( stpcpy(t, "gpg("), pubp->userid), ")");
-    /*@=nullpass@*/
-
-    evr = t = xmalloc(sizeof("4X:-")+strlen(v)+strlen(r));
-    t = stpcpy(t, (pubp->version == 4 ? "4:" : "3:"));
-    t = stpcpy( stpcpy( stpcpy(t, v), "-"), r);
-
-    /* Check for pre-existing header. */
-
-    /* Build pubkey header. */
-    h = headerNew();
-
-    he->append = 1;
-
-    he->tag = RPMTAG_PUBKEYS;
-    he->t = RPM_STRING_ARRAY_TYPE;
-    he->p.argv = &enc;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-
-    he->append = 0;
-
-    d = headerSprintf(h, afmt, NULL, rpmHeaderFormats, NULL);
-    if (d == NULL)
-	goto exit;
-
-    he->t = RPM_STRING_TYPE;
-    he->c = 1;
-    he->tag = RPMTAG_NAME;
-    he->p.str = "gpg-pubkey";
-    xx = headerPut(h, he, 0);
-    he->tag = RPMTAG_VERSION;
-    he->p.str = v+8;
-    xx = headerPut(h, he, 0);
-    he->tag = RPMTAG_RELEASE;
-    he->p.str = r;
-    xx = headerPut(h, he, 0);
-
-    /* Add Summary/Description/Group. */
-    he->tag = RPMTAG_DESCRIPTION;
-    he->p.str = d;
-#if defined(SUPPORT_IMPLICIT_TAG_DATA_TYPES)
-    xx = headerAddI18NString(h, he->tag, he->p.ptr, "C");
-#else
-    xx = headerPut(h, he, 0);
-#endif
-    he->tag = RPMTAG_GROUP;
-    he->p.str = group;
-#if defined(SUPPORT_IMPLICIT_TAG_DATA_TYPES)
-    xx = headerAddI18NString(h, he->tag, he->p.ptr, "C");
-#else
-    xx = headerPut(h, he, 0);
-#endif
-    he->tag = RPMTAG_SUMMARY;
-    he->p.str = u;
-#if defined(SUPPORT_IMPLICIT_TAG_DATA_TYPES)
-    xx = headerAddI18NString(h, he->tag, he->p.ptr, "C");
-#else
-    xx = headerPut(h, he, 0);
-#endif
-
-#ifdef	NOTYET	/* XXX can't erase pubkeys with "pubkey" arch. */
-    /* Add a "pubkey" arch/os to avoid missing value NULL ptrs. */
-    he->tag = RPMTAG_ARCH;
-    he->p.str = "pubkey";
-    xx = headerPut(h, he, 0);
-    he->tag = RPMTAG_OS;
-    he->p.str = "pubkey";
-    xx = headerPut(h, he, 0);
-#endif
-
-    he->tag = RPMTAG_LICENSE;
-    he->p.str = license;
-    xx = headerPut(h, he, 0);
-
-    he->tag = RPMTAG_SIZE;
-    he->t = RPM_UINT32_TYPE;
-    he->p.ui32p = &zero;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-
-    he->append = 1;
-
-    he->tag = RPMTAG_PROVIDENAME;
-    he->t = RPM_STRING_ARRAY_TYPE;
-    he->p.argv = &u;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-    he->tag = RPMTAG_PROVIDEVERSION;
-    he->t = RPM_STRING_ARRAY_TYPE;
-    he->p.argv = &evr;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-    he->tag = RPMTAG_PROVIDEFLAGS;
-    he->t = RPM_UINT32_TYPE;
-    he->p.ui32p = &pflags;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-
-    he->tag = RPMTAG_PROVIDENAME;
-    he->t = RPM_STRING_ARRAY_TYPE;
-    he->p.argv = &n;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-    he->tag = RPMTAG_PROVIDEVERSION;
-    he->t = RPM_STRING_ARRAY_TYPE;
-    he->p.argv = &evr;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-    he->tag = RPMTAG_PROVIDEFLAGS;
-    he->t = RPM_UINT32_TYPE;
-    he->p.ui32p = &pflags;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-
-    he->append = 0;
-
-    he->tag = RPMTAG_RPMVERSION;
-    he->t = RPM_STRING_TYPE;
-    he->p.str = RPMVERSION;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-
-    /* XXX W2DO: tag value inheirited from parent? */
-    he->tag = RPMTAG_BUILDHOST;
-    he->t = RPM_STRING_TYPE;
-    he->p.str = buildhost;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-    {   uint32_t tid = rpmtsGetTid(ts);
-	he->tag = RPMTAG_INSTALLTIME;
-	he->t = RPM_UINT32_TYPE;
-	he->p.ui32p = &tid;
-	he->c = 1;
-	xx = headerPut(h, he, 0);
-	/* XXX W2DO: tag value inheirited from parent? */
-	he->tag = RPMTAG_BUILDTIME;
-	he->t = RPM_UINT32_TYPE;
-	he->p.ui32p = &tid;
-	he->c = 1;
-	xx = headerPut(h, he, 0);
-    }
-
-#ifdef	NOTYET
-    /* XXX W2DO: tag value inheirited from parent? */
-    he->tag = RPMTAG_SOURCERPM;
-    he->t = RPM_STRING_TYPE;
-    he->p.str = fn;
-    he->c = 1;
-    xx = headerPut(h, he, 0);
-#endif
-
-    /* Add header to database. */
-    xx = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), h, NULL);
-    if (xx != 0)
-	goto exit;
-    rc = RPMRC_OK;
-
-exit:
-    /* Clean up. */
-    h = headerFree(h);
-    dig = pgpDigFree(dig);
-    n = _free(n);
-    u = _free(u);
-    v = _free(v);
-    r = _free(r);
-    evr = _free(evr);
-    enc = _free(enc);
-    d = _free(d);
-    
-    return rc;
-}
-
 int rpmtsCloseSDB(rpmts ts)
 {
     int rc = 0;
@@ -551,7 +305,7 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
     const char * errstr;
     const char * str = NULL;
     const char * qfmt;
-    rpmdbMatchIterator mi;
+    rpmmi mi;
     Header bh = NULL;
     Header h = NULL;
     size_t bhnamelen = 0;
@@ -588,8 +342,8 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 
     /* Look for a matching Provides: in suggested universe. */
     rpmtag = (*keyp == '/' ? RPMTAG_BASENAMES : RPMTAG_PROVIDENAME);
-    mi = rpmdbInitIterator(ts->sdb, rpmtag, keyp, keylen);
-    while ((h = rpmdbNextIterator(mi)) != NULL) {
+    mi = rpmmiInit(ts->sdb, rpmtag, keyp, keylen);
+    while ((h = rpmmiNext(mi)) != NULL) {
 	size_t hnamelen;
 	time_t htime;
 
@@ -615,12 +369,13 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 	    continue;
 
 	/* Save new "best" candidate. */
-	bh = headerFree(bh);
+	(void)headerFree(bh);
+	bh = NULL;
 	bh = headerLink(h);
 	bhtime = htime;
 	bhnamelen = hnamelen;
     }
-    mi = rpmdbFreeIterator(mi);
+    mi = rpmmiFree(mi);
 
     /* Is there a suggested resolution? */
     if (bh == NULL)
@@ -631,7 +386,8 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
     if (qfmt == NULL || *qfmt == '\0')
 	goto exit;
     str = headerSprintf(bh, qfmt, NULL, rpmHeaderFormats, &errstr);
-    bh = headerFree(bh);
+    (void)headerFree(bh);
+    bh = NULL;
     qfmt = _free(qfmt);
     if (str == NULL) {
 	rpmlog(RPMLOG_ERR, _("incorrect solve path format: %s\n"), errstr);
@@ -671,7 +427,8 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 	    break;
 	}
 	str = _free(str);
-	h = headerFree(h);
+	(void)headerFree(h);
+	h = NULL;
 	goto exit;
     }
 
@@ -745,10 +502,14 @@ int rpmtsSetSolveCallback(rpmts ts,
 
 rpmps rpmtsProblems(rpmts ts)
 {
+    static const char msg[] = "rpmtsProblems";
     rpmps ps = NULL;
     if (ts) {
-	if (ts->probs)
-	    ps = rpmpsLink(ts->probs, "rpmtsProblems");
+	if (ts->probs == NULL)
+	    ts->probs = rpmpsCreate();
+/*@-castexpose@*/
+	ps = rpmpsLink(ts->probs, msg);
+/*@=castexpose@*/
     }
     return ps;
 }
@@ -862,16 +623,20 @@ static void rpmtsPrintStats(rpmts ts)
 /*@=globstate@*/
 }
 
-rpmts rpmtsFree(rpmts ts)
+static void rpmtsFini(void * _ts)
+	/*@modifies _ts @*/
 {
-    if (ts == NULL)
-	return NULL;
-
-    if (ts->nrefs > 1)
-	return rpmtsUnlink(ts, "tsCreate");
+    rpmts ts = _ts;
 
 /*@-nullstate@*/	/* FIX: partial annotations */
+    /* XXX there's a recursion here ... release and reacquire the lock */
+#ifndef	BUGGY
+    yarnRelease(ts->_item.use);	/* XXX hack-o-round */
+#endif
     rpmtsEmpty(ts);
+#ifndef	BUGGY
+    yarnPossess(ts->_item.use);	/* XXX hack-o-round */
+#endif
 /*@=nullstate@*/
 
     ts->PRCO = rpmdsFreePRCO(ts->PRCO);
@@ -890,7 +655,9 @@ rpmts rpmtsFree(rpmts ts)
     ts->dsi = _free(ts->dsi);
 
     if (ts->scriptFd != NULL) {
-	ts->scriptFd = fdFree(ts->scriptFd, "rpmtsFree");
+/*@-refcounttrans@*/	/* FIX: XfdFree annotation */
+	ts->scriptFd = fdFree(ts->scriptFd, __FUNCTION__);
+/*@=refcounttrans@*/
 	ts->scriptFd = NULL;
     }
     ts->rootDir = _free(ts->rootDir);
@@ -901,6 +668,7 @@ rpmts rpmtsFree(rpmts ts)
 /*@=type =voidabstract @*/
     ts->orderAlloced = 0;
 
+    ts->keyring = rpmKeyringFree(ts->keyring);
     ts->pkpkt = _free(ts->pkpkt);
     ts->pkpktlen = 0;
     memset(ts->pksignid, 0, sizeof(ts->pksignid));
@@ -910,26 +678,87 @@ rpmts rpmtsFree(rpmts ts)
 
     if (_rpmts_macros) {
 	const char ** av = NULL;
+/*@-globs@*/	/* Avoid rpmGlobalMcroContext et al. */
 	(void)rpmGetMacroEntries(NULL, NULL, 1, &av);
+/*@=globs@*/
 	argvPrint("macros used", av, NULL);
 	av = argvFree(av);
     }
-
-    (void) rpmtsUnlink(ts, "tsCreate");
-
-    /*@-refcounttrans -usereleased @*/
-    ts = _free(ts);
-    /*@=refcounttrans =usereleased @*/
-
-    return NULL;
 }
 
-rpmVSFlags rpmtsVSFlags(/*@unused@*/rpmts ts)
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _rpmtsPool;
+
+static rpmts rpmtsGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _rpmtsPool, fileSystem, internalState @*/
+	/*@modifies pool, _rpmtsPool, fileSystem, internalState @*/
+{
+    rpmts ts;
+
+    if (_rpmtsPool == NULL) {
+	_rpmtsPool = rpmioNewPool("ts", sizeof(*ts), -1, _rpmts_debug,
+			NULL, NULL, rpmtsFini);
+	pool = _rpmtsPool;
+    }
+    return (rpmts) rpmioGetPool(pool, sizeof(*ts));
+}
+
+void * rpmtsGetKeyring(rpmts ts, /*@unused@*/ int autoload)
+{
+    rpmKeyring keyring = NULL;
+    if (ts) {
+#ifdef	NOTYET
+	if (ts->keyring == NULL && autoload)
+	    loadKeyring(ts);
+	keyring = rpmKeyringLink(ts->keyring);
+#else
+	keyring = ts->keyring;
+#endif
+    }
+/*@-refcounttrans@*/
+    return (void *)keyring;
+/*@=refcounttrans@*/
+}
+
+int rpmtsSetKeyring(rpmts ts, void * _keyring)
+{
+    rpmKeyring keyring = _keyring;
+
+    if (ts == NULL)
+       return -1;
+
+#ifdef	NOTYET
+    /*
+     * Should we permit switching keyring on the fly? For now, require
+     * rpmdb isn't open yet (fairly arbitrary limitation)...
+     */
+    if (rpmtsGetRdb(ts) != NULL)
+	return -1;
+#endif
+
+/*@-modnomods@*/
+    ts->keyring = rpmKeyringFree(ts->keyring);
+/*@=modnomods@*/
+
+#ifdef	NOTYET
+    ts->keyring = rpmKeyringLink(keyring);
+#else
+/*@-assignexpose -newreftrans @*/
+/*@i@*/    ts->keyring = keyring;
+/*@=assignexpose =newreftrans @*/
+#endif
+
+    return 0;
+}
+
+rpmVSFlags rpmtsVSFlags(/*@unused@*/ rpmts ts)
 {
     return pgpDigVSFlags;
 }
 
-rpmVSFlags rpmtsSetVSFlags(/*@unused@*/rpmts ts, rpmVSFlags vsflags)
+rpmVSFlags rpmtsSetVSFlags(/*@unused@*/ rpmts ts, rpmVSFlags vsflags)
+	/*@globals pgpDigVSFlags @*/
+	/*@modifies pgpDigVSFlags @*/
 {
     rpmVSFlags ovsflags;
     ovsflags = pgpDigVSFlags;
@@ -963,12 +792,12 @@ void rpmtsSetType(rpmts ts, rpmTSType type)
 	ts->type = type;
 }
 
-uint32_t rpmtsARBGoal(rpmts ts)
+rpmuint32_t rpmtsARBGoal(rpmts ts)
 {
     return ((ts != NULL) ?  ts->arbgoal : 0);
 }
 
-void rpmtsSetARBGoal(rpmts ts, uint32_t goal)
+void rpmtsSetARBGoal(rpmts ts, rpmuint32_t goal)
 {
     if (ts != NULL)
 	ts->arbgoal = goal;
@@ -1068,19 +897,24 @@ void rpmtsSetScriptFd(rpmts ts, FD_t scriptFd)
 
     if (ts != NULL) {
 	if (ts->scriptFd != NULL) {
+/*@-assignexpose@*/
 	    ts->scriptFd = fdFree(ts->scriptFd, "rpmtsSetScriptFd");
+/*@=assignexpose@*/
 	    ts->scriptFd = NULL;
 	}
-/*@+voidabstract@*/
+/*@-assignexpose -castexpose @*/
 	if (scriptFd != NULL)
 	    ts->scriptFd = fdLink((void *)scriptFd, "rpmtsSetScriptFd");
-/*@=voidabstract@*/
+/*@=assignexpose =castexpose @*/
     }
 }
 
 int rpmtsSELinuxEnabled(rpmts ts)
 {
-    return (ts != NULL ? (ts->selinuxEnabled > 0) : 0);
+    int selinuxEnabled = 0;
+    if (ts)
+	selinuxEnabled = (ts->selinuxEnabled > 0);
+    return selinuxEnabled;
 }
 
 int rpmtsChrootDone(rpmts ts)
@@ -1100,46 +934,46 @@ int rpmtsSetChrootDone(rpmts ts, int chrootDone)
     return ochrootDone;
 }
 
-rpmsx rpmtsREContext(rpmts ts)
+rpmuint32_t rpmtsGetTid(rpmts ts)
 {
-    return ( (ts && ts->sx ? rpmsxLink(ts->sx, __func__) : NULL) );
-}
-
-int rpmtsSetREContext(rpmts ts, rpmsx sx)
-{
-    int rc = -1;
+    rpmuint32_t tid = 0;	/* XXX -1 is time(2) error return. */
     if (ts != NULL) {
-	ts->sx = rpmsxFree(ts->sx);
-	ts->sx = rpmsxLink(sx, __func__);
-	if (ts->sx != NULL)
-	    rc = 0;
-    }
-    return rc;
-}
-
-uint32_t rpmtsGetTid(rpmts ts)
-{
-    uint32_t tid = 0;	/* XXX -1 is time(2) error return. */
-    if (ts != NULL) {
-	tid = ts->tid;
+	tid = ts->tid[0];
     }
     return tid;
 }
 
-uint32_t rpmtsSetTid(rpmts ts, uint32_t tid)
+rpmuint32_t rpmtsSetTid(rpmts ts, rpmuint32_t tid)
 {
-    uint32_t otid = 0;	/* XXX -1 is time(2) error return. */
+    rpmuint32_t otid = 0;	/* XXX -1 is time(2) error return. */
     if (ts != NULL) {
-	otid = ts->tid;
-	ts->tid = tid;
+	otid = ts->tid[0];
+	ts->tid[0] = tid;
+	ts->tid[1] = 0;
     }
     return otid;
 }
 
 rpmPRCO rpmtsPRCO(rpmts ts)
 {
+    rpmPRCO PRCO = NULL;
+
+    if (ts != NULL) {
+	static int oneshot = 0;
+	if (!oneshot) {
+	    const char * fn = rpmGetPath("%{?_rpmds_sysinfo_path}", NULL);
+	    int xx;
+
+	    ts->PRCO = rpmdsNewPRCO(NULL);
+	    if (fn && *fn != '\0' && !rpmioAccess(fn, NULL, R_OK))
+		xx = rpmdsSysinfo(ts->PRCO, NULL);
+	    fn = _free(fn);
+	    oneshot++;
+	}
+	PRCO = ts->PRCO;
+    }
 /*@-compdef -retexpose -usereleased @*/
-    return (ts != NULL ? ts->PRCO : NULL);
+    return PRCO;
 /*@=compdef =retexpose =usereleased @*/
 }
 
@@ -1148,7 +982,7 @@ int rpmtsInitDSI(const rpmts ts)
     rpmDiskSpaceInfo dsi;
     struct stat sb;
     int rc;
-    int i;
+    size_t i;
 
     if (rpmtsFilterFlags(ts) & RPMPROB_FILTER_DISKSPACE)
 	return 0;
@@ -1206,31 +1040,34 @@ int rpmtsInitDSI(const rpmts ts)
 	dsi->f_fsid = sfb.f_fsid;
 #endif
 	dsi->f_flag = sfb.f_flag;
-	dsi->f_favail = sfb.f_favail;
+	dsi->f_favail = (long long) sfb.f_favail;
 	dsi->f_namemax = sfb.f_namemax;
 #elif defined(__APPLE__) && defined(__MACH__) && !defined(_SYS_STATVFS_H_)
 	dsi->f_fsid = 0; /* "Not meaningful in this implementation." */
 	dsi->f_namemax = pathconf(ts->filesystems[i], _PC_NAME_MAX);
+#elif defined(__OpenBSD__)
+	dsi->f_fsid = 0; /* sfb.f_fsid is a structure on OpenBSD */
+	dsi->f_namemax = pathconf(ts->filesystems[i], _PC_NAME_MAX);
 #else
 	dsi->f_fsid = sfb.f_fsid;
 	dsi->f_namemax = sfb.f_namelen;
 #endif
 
 	dsi->f_bsize = sfb.f_bsize;
-	dsi->f_blocks = sfb.f_blocks;
-	dsi->f_bfree = sfb.f_bfree;
-	dsi->f_files = sfb.f_files;
-	dsi->f_ffree = sfb.f_ffree;
+	dsi->f_blocks = (unsigned long long)sfb.f_blocks;
+	dsi->f_bfree = (unsigned long long)sfb.f_bfree;
+	dsi->f_files = (unsigned long long)sfb.f_files;
+	dsi->f_ffree = (unsigned long long)sfb.f_ffree;
 
 	dsi->bneeded = 0;
 	dsi->ineeded = 0;
 #ifdef STATFS_HAS_F_BAVAIL
-	dsi->f_bavail = sfb.f_bavail ? sfb.f_bavail : 1;
+	dsi->f_bavail = (long long)(sfb.f_bavail ? sfb.f_bavail : 1);
 	if (sfb.f_ffree > 0 && sfb.f_files > 0 && sfb.f_favail > 0)
-	    dsi->f_favail = sfb.f_favail;
+	    dsi->f_favail = (long long)sfb.f_favail;
 	else	/* XXX who knows what evil lurks here? */
 	    dsi->f_favail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
-				? sfb.f_ffree : -1;
+				?  (signed long long) sfb.f_ffree : -1;
 #else
 /* FIXME: the statfs struct doesn't have a member to tell how many blocks are
  * available for non-superusers.  f_blocks - f_bfree is probably too big, but
@@ -1245,8 +1082,8 @@ int rpmtsInitDSI(const rpmts ts)
 #if !defined(ST_RDONLY)
 #define	ST_RDONLY	1
 #endif
-	rpmlog(RPMLOG_DEBUG, "%5d 0x%08x %8u %12ld %12ld %s %s\n",
-		i, (unsigned) dsi->dev, (unsigned) dsi->f_bsize,
+	rpmlog(RPMLOG_DEBUG, "%5u 0x%08x %8u %12ld %12ld %s %s\n",
+		(unsigned)i, (unsigned) dsi->dev, (unsigned) dsi->f_bsize,
 		(signed long) dsi->f_bavail, (signed long) dsi->f_favail,
 		((dsi->f_flag & ST_RDONLY) ? "ro" : "rw"),
 		ts->filesystems[i]);
@@ -1255,11 +1092,12 @@ int rpmtsInitDSI(const rpmts ts)
 }
 
 void rpmtsUpdateDSI(const rpmts ts, dev_t dev,
-		uint32_t fileSize, uint32_t prevSize, uint32_t fixupSize,
-		fileAction action)
+		rpmuint32_t fileSize, rpmuint32_t prevSize, rpmuint32_t fixupSize,
+		int _action)
 {
+    iosmFileAction action = _action;
     rpmDiskSpaceInfo dsi;
-    uint64_t bneeded;
+    rpmuint64_t bneeded;
 
     dsi = ts->dsi;
     if (dsi) {
@@ -1309,7 +1147,7 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
     rpmDiskSpaceInfo dsi;
     rpmps ps;
     int fc;
-    int i;
+    size_t i;
 
     if (ts->filesystems == NULL || ts->filesystemCount == 0)
 	return;
@@ -1348,15 +1186,26 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
 }
 
 void * rpmtsNotify(rpmts ts, rpmte te,
-		rpmCallbackType what, uint64_t amount, uint64_t total)
+		rpmCallbackType what, rpmuint64_t amount, rpmuint64_t total)
 {
     void * ptr = NULL;
-    if (ts && ts->notify && te) {
-assert(!(te->type == TR_ADDED && te->h == NULL));
+    if (ts && ts->notify) {
+	Header h;
+	fnpyKey cbkey;
 	/*@-type@*/ /* FIX: cast? */
 	/*@-noeffectuncon @*/ /* FIX: check rc */
-	ptr = ts->notify(te->h, what, amount, total,
-			rpmteKey(te), ts->notifyData);
+	if (te) {
+/*@-castexpose -mods@*/	/* XXX noisy in transaction.c */
+	    h = headerLink(te->h);
+/*@=castexpose =mods@*/
+	    cbkey = rpmteKey(te);
+	} else {
+	    h = NULL;
+	    cbkey = NULL;
+	}
+	ptr = ts->notify(h, what, amount, total, cbkey, ts->notifyData);
+	(void)headerFree(h);
+	h = NULL;
 	/*@=noeffectuncon @*/
 	/*@=type@*/
     }
@@ -1391,7 +1240,15 @@ rpmprobFilterFlags rpmtsFilterFlags(rpmts ts)
 
 rpmtransFlags rpmtsFlags(rpmts ts)
 {
-    return (ts != NULL ? ts->transFlags : 0);
+    rpmtransFlags transFlags = 0;
+    if (ts != NULL) {
+	transFlags = ts->transFlags;
+	if (rpmtsSELinuxEnabled(ts) > 0)
+	    transFlags &= ~RPMTRANS_FLAG_NOCONTEXTS;
+	else
+	    transFlags |= RPMTRANS_FLAG_NOCONTEXTS;
+    }
+    return transFlags;
 }
 
 rpmtransFlags rpmtsSetFlags(rpmts ts, rpmtransFlags transFlags)
@@ -1399,6 +1256,10 @@ rpmtransFlags rpmtsSetFlags(rpmts ts, rpmtransFlags transFlags)
     rpmtransFlags otransFlags = 0;
     if (ts != NULL) {
 	otransFlags = ts->transFlags;
+	if (rpmtsSELinuxEnabled(ts) > 0)
+	    transFlags &= ~RPMTRANS_FLAG_NOCONTEXTS;
+	else
+	    transFlags |= RPMTRANS_FLAG_NOCONTEXTS;
 	ts->transFlags = transFlags;
     }
     return otransFlags;
@@ -1481,14 +1342,14 @@ int rpmtsSetDBMode(rpmts ts, int dbmode)
     return odbmode;
 }
 
-uint32_t rpmtsColor(rpmts ts)
+rpmuint32_t rpmtsColor(rpmts ts)
 {
     return (ts != NULL ? ts->color : 0);
 }
 
-uint32_t rpmtsSetColor(rpmts ts, uint32_t color)
+rpmuint32_t rpmtsSetColor(rpmts ts, rpmuint32_t color)
 {
-    uint32_t ocolor = 0;
+    rpmuint32_t ocolor = 0;
     if (ts != NULL) {
 	ocolor = ts->color;
 	ts->color = color;
@@ -1496,7 +1357,7 @@ uint32_t rpmtsSetColor(rpmts ts, uint32_t color)
     return ocolor;
 }
 
-uint32_t rpmtsPrefColor(rpmts ts)
+rpmuint32_t rpmtsPrefColor(rpmts ts)
 {
     return (ts != NULL ? ts->prefcolor : 0);
 }
@@ -1513,10 +1374,9 @@ int rpmtsSetNotifyCallback(rpmts ts,
 
 rpmts rpmtsCreate(void)
 {
-    rpmts ts;
+    rpmts ts = rpmtsGetPool(_rpmtsPool);
     int xx;
 
-    ts = xcalloc(1, sizeof(*ts));
     memset(&ts->ops, 0, sizeof(ts->ops));
     (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_TOTAL), -1);
     ts->type = RPMTRANS_TYPE_NORMAL;
@@ -1530,12 +1390,7 @@ rpmts rpmtsCreate(void)
     ts->nsuggests = 0;
     ts->suggests = NULL;
 
-    ts->PRCO = rpmdsNewPRCO(NULL);
-    {	const char * fn = rpmGetPath("%{?_rpmds_sysinfo_path}", NULL);
-	if (fn && *fn != '\0' && !rpmioAccess(fn, NULL, R_OK))
-	   xx = rpmdsSysinfo(ts->PRCO, NULL);
-	fn = _free(fn);
-    }
+    ts->PRCO = NULL;
 
     ts->sdb = NULL;
     ts->sdbmode = O_RDONLY;
@@ -1544,7 +1399,11 @@ rpmts rpmtsCreate(void)
     ts->dbmode = O_RDONLY;
 
     ts->scriptFd = NULL;
-    ts->tid = (uint32_t) time(NULL);
+    {	struct timeval tv;
+	xx = gettimeofday(&tv, NULL);
+	ts->tid[0] = (rpmuint32_t) tv.tv_sec;
+	ts->tid[1] = (rpmuint32_t) tv.tv_usec;
+    }
     ts->delta = 5;
 
     ts->color = rpmExpandNumeric("%{?_transaction_color}");
@@ -1579,6 +1438,7 @@ rpmts rpmtsCreate(void)
 
     ts->probs = NULL;
 
+    ts->keyring = NULL;
     ts->pkpkt = NULL;
     ts->pkpktlen = 0;
     memset(ts->pksignid, 0, sizeof(ts->pksignid));
@@ -1587,7 +1447,5 @@ rpmts rpmtsCreate(void)
     /* Set autorollback goal to the end of time. */
     ts->arbgoal = 0xffffffff;
 
-    ts->nrefs = 0;
-
     return rpmtsLink(ts, "tsCreate");
 }
