@@ -7,7 +7,6 @@
  */
 
 #include "rpmps.h"
-#include "rpmfi.h"
 #include "rpmsw.h"
 #include "rpmsx.h"
 #include <rpmpgp.h>		/* XXX pgpVSFlags */
@@ -59,9 +58,7 @@ typedef enum rpmtransFlags_e {
     RPMTRANS_FLAG_NOTRIGGERS	= (1 <<  4),	/*!< from --notriggers */
     RPMTRANS_FLAG_NODOCS	= (1 <<  5),	/*!< from --excludedocs */
     RPMTRANS_FLAG_ALLFILES	= (1 <<  6),	/*!< from --allfiles */
-/*@-enummemuse@*/
-    RPMTRANS_FLAG_KEEPOBSOLETE	= (1 <<  7),	/*!< @todo Document. */
-/*@=enummemuse@*/
+	/* 7 unused */
     RPMTRANS_FLAG_NOCONTEXTS	= (1 <<  8),	/*!< from --nocontexts */
     RPMTRANS_FLAG_DIRSTASH	= (1 <<  9),	/*!< from --dirstash */
     RPMTRANS_FLAG_REPACKAGE	= (1 << 10),	/*!< from --repackage */
@@ -74,7 +71,7 @@ typedef enum rpmtransFlags_e {
 /*@-enummemuse@*/
     RPMTRANS_FLAG_UNDO		= (1 << 14),
 /*@=enummemuse@*/
-    /* 15 unused */
+    RPMTRANS_FLAG_APPLYONLY	= (1 << 25),
 
     RPMTRANS_FLAG_NOTRIGGERPREIN= (1 << 16),	/*!< from --notriggerprein */
     RPMTRANS_FLAG_NOPRE		= (1 << 17),	/*!< from --nopre */
@@ -87,20 +84,22 @@ typedef enum rpmtransFlags_e {
 /*@-enummemuse@*/
     RPMTRANS_FLAG_NOPAYLOAD	= (1 << 24),
 /*@=enummemuse@*/
-    RPMTRANS_FLAG_APPLYONLY	= (1 << 25),
-
-    /* 26 unused */
+    RPMTRANS_FLAG_NORPMDB	= (1 << 25),	/*!< from --norpmdb */
+	/* 26 unused */
     RPMTRANS_FLAG_NOFDIGESTS	= (1 << 27),	/*!< from --nofdigests */
-    /* 28-29 unused */
+    RPMTRANS_FLAG_NOPRETRANS	= (1 << 28),	/*!< from --nopretrans */
+    RPMTRANS_FLAG_NOPOSTTRANS	= (1 << 29),	/*!< from --noposttrans */
     RPMTRANS_FLAG_NOCONFIGS	= (1 << 30),	/*!< from --noconfigs */
-    /* 31 unused */
+	/* 31 unused */
 } rpmtransFlags;
 
 #define	_noTransScripts		\
-  ( RPMTRANS_FLAG_NOPRE |	\
+  ( RPMTRANS_FLAG_NOPRETRANS |	\
+    RPMTRANS_FLAG_NOPRE |	\
     RPMTRANS_FLAG_NOPOST |	\
     RPMTRANS_FLAG_NOPREUN |	\
-    RPMTRANS_FLAG_NOPOSTUN	\
+    RPMTRANS_FLAG_NOPOSTUN |	\
+    RPMTRANS_FLAG_NOPOSTTRANS	\
   )
 
 #define	_noTransTriggers	\
@@ -158,6 +157,7 @@ typedef enum tsStage_e {
 #if defined(_RPMTS_INTERNAL)
 
 #include "rpmhash.h"	/* XXX hashTable */
+#include "rpmkeyring.h"
 #include "rpmal.h"	/* XXX availablePackage/relocateFileList ,*/
 
 /*@unchecked@*/
@@ -201,6 +201,7 @@ struct diskspaceInfo_s {
  * The set of packages to be installed/removed atomically.
  */
 struct rpmts_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
     rpmdepFlags depFlags;	/*!< Bit(s) to control rpmtsCheck(). */
     rpmtransFlags transFlags;	/*!< Bit(s) to control rpmtsRun(). */
     tsmStage goal;		/*!< Transaction goal (i.e. mode) */
@@ -231,7 +232,7 @@ struct rpmts_s {
     rpmprobFilterFlags ignoreSet;
 				/*!< Bits to filter current problems. */
 
-    uint32_t filesystemCount;	/*!< No. of mounted filesystems. */
+    rpmuint32_t filesystemCount;/*!< No. of mounted filesystems. */
 /*@dependent@*/ /*@null@*/
     const char ** filesystems;	/*!< Mounted filesystem names. */
 /*@only@*/ /*@relnull@*/
@@ -276,9 +277,9 @@ struct rpmts_s {
     int ntrees;			/*!< No. of dependency trees. */
     int maxDepth;		/*!< Maximum depth of dependency tree(s). */
 
-/*@dependent@*/
+/*@dependent@*/ /*@relnull@*/
     rpmte teInstall;		/*!< current rpmtsAddInstallElement element. */
-/*@dependent@*/
+/*@dependent@*/ /*@relnull@*/
     rpmte teErase;		/*!< current rpmtsAddEraseElement element. */
 
     int selinuxEnabled;		/*!< Is SE linux enabled? */
@@ -290,18 +291,20 @@ struct rpmts_s {
 /*@null@*/
     FD_t scriptFd;		/*!< Scriptlet stdout/stderr. */
     int delta;			/*!< Delta for reallocation. */
-    uint32_t tid;			/*!< Transaction id. */
+    rpmuint32_t tid[2];		/*!< Transaction id. */
 
-    uint32_t color;		/*!< Transaction color bits. */
-    uint32_t prefcolor;		/*!< Preferred file color. */
+    rpmuint32_t color;		/*!< Transaction color bits. */
+    rpmuint32_t prefcolor;	/*!< Preferred file color. */
 
 /*@observer@*/ /*@dependent@*/ /*@null@*/
     const char * fn;		/*!< Current package fn. */
 
-/*@only@*/ /*@relnull@*/
-    const unsigned char * pkpkt;/*!< Current pubkey packet. */
+/*@refcounted@*/ /*@relnull@*/
+    rpmKeyring keyring;		/*!< Keyring in use. */
+/*@relnull@*/
+    rpmuint8_t * pkpkt;		/*!< Current pubkey packet. */
     size_t pkpktlen;		/*!< Current pubkey packet length. */
-    unsigned char pksignid[8];	/*!< Current pubkey fingerprint. */
+    rpmuint8_t pksignid[8];	/*!< Current pubkey fingerprint. */
 
     struct rpmop_s ops[RPMTS_OP_MAX];
 
@@ -311,10 +314,12 @@ struct rpmts_s {
 /*@null@*/
     Spec spec;			/*!< Spec file control structure. */
 
-    uint32_t arbgoal;		/*!< Autorollback goal */
+    rpmuint32_t arbgoal;	/*!< Autorollback goal */
 
+#if defined(__LCLINT__)
 /*@refs@*/
-    int nrefs;			/*!< Reference count. */
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 #endif	/* _RPMTS_INTERNAL */
 
@@ -394,21 +399,14 @@ rpmRC rpmtsRollback(rpmts rbts, rpmprobFilterFlags ignoreSet,
  * Unreference a transaction instance.
  * @param ts		transaction set
  * @param msg
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@unused@*/ /*@null@*/
 rpmts rpmtsUnlink (/*@killref@*/ /*@only@*/ rpmts ts,
 		const char * msg)
 	/*@modifies ts @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@-exportlocal@*/
-/*@null@*/
-rpmts XrpmtsUnlink (/*@killref@*/ /*@only@*/ rpmts ts,
-		const char * msg, const char * fn, unsigned ln)
-	/*@modifies ts @*/;
-/*@=exportlocal@*/
-#define	rpmtsUnlink(_ts, _msg)	XrpmtsUnlink(_ts, _msg, __FILE__, __LINE__)
+#define	rpmtsUnlink(_ts, _msg)	\
+	((rpmts) rpmioUnlinkPoolItem((rpmioItem)(_ts), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmts
  * Reference a transaction set instance.
@@ -416,15 +414,11 @@ rpmts XrpmtsUnlink (/*@killref@*/ /*@only@*/ rpmts ts,
  * @param msg
  * @return		new transaction set reference
  */
-/*@unused@*/
+/*@unused@*/ /*@newref@*/
 rpmts rpmtsLink (rpmts ts, const char * msg)
 	/*@modifies ts @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-rpmts XrpmtsLink (rpmts ts,
-		const char * msg, const char * fn, unsigned ln)
-        /*@modifies ts @*/;
-#define	rpmtsLink(_ts, _msg)	XrpmtsLink(_ts, _msg, __FILE__, __LINE__)
+#define	rpmtsLink(_ts, _msg)	\
+	((rpmts) rpmioLinkPoolItem((rpmioItem)(_ts), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmts
  * Close the database used by the transaction.
@@ -483,7 +477,7 @@ int rpmtsVerifyDB(rpmts ts)
  * @return		NULL on failure
  */
 /*@only@*/ /*@null@*/
-rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
+rpmmi rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 			/*@null@*/ const void * keyp, size_t keylen)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
@@ -500,21 +494,6 @@ rpmRC rpmtsFindPubkey(rpmts ts, /*@null@*/ void * _dig)
 	/*@modifies ts, _dig, rpmGlobalMacroContext, fileSystem, internalState */;
 /*@=exportlocal@*/
 
-/** \ingroup rpmcli
- * Import public key packet(s).
- * @todo Implicit --update policy for gpg-pubkey headers.
- * @param ts		transaction set
- * @param pkt		pgp pubkey packet(s)
- * @param pktlen	pgp pubkey length
- * @return		RPMRC_OK/RPMRC_FAIL
- */
-rpmRC rpmtsImportPubkey(const rpmts ts,
-		const unsigned char * pkt, ssize_t pktlen)
-	/*@globals RPMVERSION, rpmGlobalMacroContext, h_errno,
-		fileSystem, internalState @*/
-	/*@modifies ts, rpmGlobalMacroContext,
-		fileSystem, internalState @*/;
-
 /** \ingroup rpmts
  * Close the database used by the transaction to solve dependencies.
  * @param ts		transaction set
@@ -601,7 +580,7 @@ void rpmtsSetType(rpmts ts, rpmTSType type)
  * @param ts		transaction set
  * @return		autorollback goal
  */
-uint32_t rpmtsARBGoal(rpmts ts)
+rpmuint32_t rpmtsARBGoal(rpmts ts)
 	/*@*/;
 
 /** \ingroup rpmts
@@ -609,7 +588,7 @@ uint32_t rpmtsARBGoal(rpmts ts)
  * @param ts		transaction set
  * @param goal		autorollback goal
  */
-void rpmtsSetARBGoal(rpmts ts, uint32_t goal)
+void rpmtsSetARBGoal(rpmts ts, rpmuint32_t goal)
 	/*@modifies ts @*/;
 
 /** \ingroup rpmts
@@ -626,7 +605,8 @@ rpmps rpmtsProblems(rpmts ts)
  * @param ts		transaction set
  */
 void rpmtsCleanDig(rpmts ts)
-	/*@modifies ts @*/;
+	/*@globals fileSystem @*/
+	/*@modifies ts, fileSystem @*/;
 
 /** \ingroup rpmts
  * Free memory needed only for dependency checks and ordering.
@@ -647,12 +627,32 @@ void rpmtsEmpty(rpmts ts)
 /** \ingroup rpmts
  * Destroy transaction set, closing the database as well.
  * @param ts		transaction set
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@null@*/
-rpmts rpmtsFree(/*@killref@*/ /*@only@*//*@null@*/ rpmts ts)
+rpmts rpmtsFree(/*@killref@*/ /*@null@*/ rpmts ts)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ts, fileSystem, internalState @*/;
+#define	rpmtsFree(_ts)	\
+	((rpmts) rpmioFreePoolItem((rpmioItem)(_ts), __FUNCTION__, __FILE__, __LINE__))
+
+/** \ingroup rpmts
+ * Get transaction keyring.
+ * @param ts		transaction set
+ * @param autoload	Should keyring be loaded? (unimplmented)
+ * @return		transaction keyring
+ */
+void * rpmtsGetKeyring(rpmts ts, int autoload)
+	/*@*/;
+
+/** \ingroup rpmts
+ * Set transaction keyring.
+ * @param ts		transaction set
+ * @param _keyring	new transaction keyring
+ * @return		0 on success
+ */
+int rpmtsSetKeyring(rpmts ts, void * _keyring)
+	/*modifies ts, _keyring @*/;
 
 /** \ingroup rpmts
  * Get verify signatures flag(s).
@@ -714,6 +714,7 @@ extern const char * rpmtsCurrDir(rpmts ts)
 void rpmtsSetCurrDir(rpmts ts, /*@null@*/ const char * currDir)
 	/*@modifies ts @*/;
 
+#if defined(_RPMTS_INTERNAL)	/* XXX avoid FD_t in API. */
 /** \ingroup rpmts
  * Get transaction script file handle, i.e. stdout/stderr on scriptlet execution
  * @param ts		transaction set
@@ -729,7 +730,9 @@ FD_t rpmtsScriptFd(rpmts ts)
  * @param scriptFd	new script file handle (or NULL)
  */
 void rpmtsSetScriptFd(rpmts ts, /*@null@*/ FD_t scriptFd)
-	/*@modifies ts, scriptFd @*/;
+	/*@globals fileSystem @*/
+	/*@modifies ts, scriptFd, fileSystem @*/;
+#endif
 
 /** \ingroup rpmts
  * Get selinuxEnabled flag, i.e. is SE linux enabled?
@@ -756,30 +759,12 @@ int rpmtsChrootDone(rpmts ts)
 int rpmtsSetChrootDone(rpmts ts, int chrootDone)
 	/*@modifies ts @*/;
 
-/** \ingroup rpmts
- * Get file security context patterns.
- * @param ts		transaction set
- * @return		file security context patterns
- */
-/*@null@*/
-rpmsx rpmtsREContext(const rpmts ts)
-	/*@modifies ts @*/;
-
-/** \ingroup rpmts
- * Get file security context patterns.
- * @param ts		transaction set
- * @param sx		security context patterns
- * @return		0 on success
- */
-int rpmtsSetREContext(rpmts ts, rpmsx sx)
-	/*@modifies ts, sx @*/;
-
 /** \ingroup rpmts
  * Get transaction id, i.e. transaction time stamp.
  * @param ts		transaction set
  * @return		transaction id
  */
-uint32_t rpmtsGetTid(rpmts ts)
+rpmuint32_t rpmtsGetTid(rpmts ts)
 	/*@*/;
 
 /** \ingroup rpmts
@@ -788,7 +773,7 @@ uint32_t rpmtsGetTid(rpmts ts)
  * @param tid		new transaction id
  * @return		previous transaction id
  */
-uint32_t rpmtsSetTid(rpmts ts, uint32_t tid)
+rpmuint32_t rpmtsSetTid(rpmts ts, rpmuint32_t tid)
 	/*@modifies ts @*/;
 
 /** \ingroup rpmts
@@ -797,7 +782,8 @@ uint32_t rpmtsSetTid(rpmts ts, uint32_t tid)
  * @return		signature/pubkey constants.
  */
 pgpDig rpmtsDig(rpmts ts)
-	/*@*/;
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/;
 
 /** \ingroup rpmts
  * Return OpenPGP pubkey constants.
@@ -807,7 +793,8 @@ pgpDig rpmtsDig(rpmts ts)
 /*@-exportlocal@*/
 /*@exposed@*/ /*@null@*/
 pgpDigParams rpmtsPubkey(const rpmts ts)
-	/*@*/;
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/;
 /*@=exportlocal@*/
 
 /** \ingroup rpmts
@@ -826,7 +813,8 @@ rpmdb rpmtsGetRdb(rpmts ts)
  */
 /*@null@*/
 rpmPRCO rpmtsPRCO(rpmts ts)
-	/*@*/;
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmts
  * Initialize disk space info for each and every mounted file systems.
@@ -844,11 +832,11 @@ int rpmtsInitDSI(const rpmts ts)
  * @param fileSize	file size
  * @param prevSize	previous file size (if upgrading)
  * @param fixupSize	size difference (if
- * @param action	file disposition
+ * @param _action	file disposition
  */
 void rpmtsUpdateDSI(const rpmts ts, dev_t dev,
-		uint32_t fileSize, uint32_t prevSize, uint32_t fixupSize,
-		fileAction action)
+		rpmuint32_t fileSize, rpmuint32_t prevSize, rpmuint32_t fixupSize,
+		int _action)
 	/*@modifies ts @*/;
 
 /** \ingroup rpmts
@@ -872,8 +860,8 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
  */
 /*@null@*/
 void * rpmtsNotify(rpmts ts, rpmte te,
-                rpmCallbackType what, uint64_t amount, uint64_t total)
-	/*@*/;
+                rpmCallbackType what, rpmuint64_t amount, rpmuint64_t total)
+	/*@modifies te @*/;
 
 /** \ingroup rpmts
  * Return number of (ordered) transaction set elements.
@@ -1012,7 +1000,7 @@ int rpmtsSetDBMode(rpmts ts, int dbmode)
  * @param ts		transaction set
  * @return		color bits
  */
-uint32_t rpmtsColor(rpmts ts)
+rpmuint32_t rpmtsColor(rpmts ts)
 	/*@*/;
 
 /** \ingroup rpmts
@@ -1020,7 +1008,7 @@ uint32_t rpmtsColor(rpmts ts)
  * @param ts		transaction set
  * @return		color bits
  */
-uint32_t rpmtsPrefColor(rpmts ts)
+rpmuint32_t rpmtsPrefColor(rpmts ts)
 	/*@*/;
 
 /** \ingroup rpmts
@@ -1029,7 +1017,7 @@ uint32_t rpmtsPrefColor(rpmts ts)
  * @param color		new color bits
  * @return		previous color bits
  */
-uint32_t rpmtsSetColor(rpmts ts, uint32_t color)
+rpmuint32_t rpmtsSetColor(rpmts ts, rpmuint32_t color)
 	/*@modifies ts @*/;
 
 /** \ingroup rpmts
@@ -1064,8 +1052,8 @@ int rpmtsSetNotifyCallback(rpmts ts,
  */
 /*@newref@*/
 rpmts rpmtsCreate(void)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies rpmGlobalMacroContext, internalState @*/;
 
 /*@-redecl@*/
 /*@unchecked@*/
@@ -1117,7 +1105,7 @@ int rpmtsPrint(/*@null@*/ rpmts ts, /*@null@*/ FILE * fp)
 	/*@globals fileSystem @*/
 	/*@modifies ts, *fp, fileSystem @*/
 {
-    int tid = rpmtsGetTid(ts);
+    rpmuint32_t tid = rpmtsGetTid(ts);
     time_t ttid = tid;
     rpmtsi tsi;
     rpmte te;
