@@ -5,42 +5,56 @@
 #include "system.h"
 
 #include <rpmio.h>
+#include <rpmiotypes.h>
+#include <rpmlog.h>
 #include <rpmmacro.h>	/* XXX for rpmExpand */
+#include "fprint.h"
+
+#include <rpmtypes.h>
 #include <rpmtag.h>
-
-#define	_RPMFI_INTERNAL
-#define	_RPMTE_INTERNAL
-#define	_RPMTS_INTERNAL
-#include "rpmcli.h"	/* IDTX prototypes */
-
-#include "fsm.h"
-#define	_RPMSQ_INTERNAL
-#include "psm.h"
+#include <pkgio.h>
 
 #define	_RPMDB_INTERNAL	/* XXX for dbiIndexFoo() */
-#include "rpmdb.h"
+#include <rpmdb.h>
+#include "legacy.h"	/* XXX dodigest */
+
+#define	_RPMFI_INTERNAL
+#include <rpmfi.h>
+#include "fsm.h"
+
+#define	_RPMTE_INTERNAL
+#include "rpmte.h"
+#define	_RPMTS_INTERNAL
+#include "rpmts.h"
+
+#define	_RPMSQ_INTERNAL
+#include "psm.h"
 
 #include "rpmds.h"
 
 #include "rpmlock.h"
 
-#include "cpio.h"
-#include "fprint.h"
-#include "legacy.h"	/* XXX dodigest */
-#include "misc.h" /* XXX (free)splitString, currentDirectory */
+#include "misc.h" /* XXX currentDirectory */
+
+#if defined(RPM_VENDOR_MANDRIVA)
+#include "filetriggers.h" /* XXX mayAddToFilesAwaitingFiletriggers, rpmRunFileTriggers */
+#endif
+
+#include <rpmcli.h>	/* XXX QVA_t INSTALL_FOO flags */
+#include <rpmrollback.h>	/* IDTX prototypes */
 
 #include "debug.h"
 
-/*@access Header @*/		/* XXX ts->notify arg1 is void ptr */
-/*@access rpmps @*/	/* XXX need rpmProblemSetOK() */
 /*@access dbiIndexSet @*/
 
-/*@access rpmpsm @*/
-
-/*@access alKey @*/
 /*@access fnpyKey @*/
 
+/*@access alKey @*/
+/*@access rpmdb @*/	/* XXX cast */
+
 /*@access rpmfi @*/
+/*@access rpmps @*/	/* XXX need rpmProblemSetOK() */
+/*@access rpmpsm @*/
 
 /*@access rpmte @*/
 /*@access rpmtsi @*/
@@ -67,12 +81,14 @@ static int sharedCmp(const void * one, const void * two)
 }
 
 /**
+ * handleInstInstalledFiles.
  * @param ts		transaction set
  * @param p		current transaction element
  * @param fi		file info set
  * @param shared	shared file info
  * @param sharedCount	no. of shared elements
  * @param reportConflicts
+ * @return		0 on success
  *
  * XXX only ts->{probs,rpmdb} modified
  */
@@ -85,23 +101,23 @@ static int handleInstInstalledFiles(const rpmts ts,
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     const char * altNVRA = NULL;
-    uint32_t tscolor = rpmtsColor(ts);
-    uint32_t prefcolor = rpmtsPrefColor(ts);
-    uint32_t otecolor, tecolor;
-    uint32_t oFColor, FColor;
-    uint32_t oFFlags, FFlags;
+    rpmuint32_t tscolor = rpmtsColor(ts);
+    rpmuint32_t prefcolor = rpmtsPrefColor(ts);
+    rpmuint32_t otecolor, tecolor;
+    rpmuint32_t oFColor, FColor;
+    rpmuint32_t oFFlags, FFlags;
     rpmfi otherFi = NULL;
     rpmps ps;
     int xx;
     int i;
 
-    {	rpmdbMatchIterator mi;
+    {	rpmmi mi;
 	Header h;
 	int scareMem = 0;
 
 	mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES,
 			&shared->otherPkg, sizeof(shared->otherPkg));
-	while ((h = rpmdbNextIterator(mi)) != NULL) {
+	while ((h = rpmmiNext(mi)) != NULL) {
 	    he->tag = RPMTAG_NVRA;
 	    xx = headerGet(h, he, 0);
 assert(he->p.str != NULL);
@@ -109,7 +125,7 @@ assert(he->p.str != NULL);
 	    otherFi = rpmfiNew(ts, h, RPMTAG_BASENAMES, scareMem);
 	    break;
 	}
-	mi = rpmdbFreeIterator(mi);
+	mi = rpmmiFree(mi);
     }
 
     /* Compute package color. */
@@ -152,14 +168,14 @@ assert(he->p.str != NULL);
 	    continue;
 #endif
 
-	if (XFA_SKIPPING(fi->actions[fileNum]))
+	if (iosmFileActionSkipped(fi->actions[fileNum]))
 	    continue;
 
 	/* Remove setuid/setgid bits on other (possibly hardlinked) files. */
-	if (!(fi->mapflags & CPIO_SBIT_CHECK)) {
-	    uint16_t omode = rpmfiFMode(otherFi);
+	if (!(fi->mapflags & IOSM_SBIT_CHECK)) {
+	    rpmuint16_t omode = rpmfiFMode(otherFi);
 	    if (S_ISREG(omode) && (omode & 06000) != 0)
-		fi->mapflags |= CPIO_SBIT_CHECK;
+		fi->mapflags |= IOSM_SBIT_CHECK;
 	}
 
 	if (((FFlags | oFFlags) & RPMFILE_GHOST))
@@ -181,6 +197,27 @@ assert(he->p.str != NULL);
 		    rConflicts = 0;
 		}
 	    }
+#if defined(RPM_VENDOR_MANDRIVA) /* no-doc-conflicts */
+	    /* HACK: always install latest (arch-independent) man
+	       pages and gtk/gnome html doc files. */
+	    if (rConflicts && tscolor != 0 && FColor == 0 && oFColor == 0) {
+		const char *ignorelist[] = {
+		    "/usr/share/man/",
+		    "/usr/share/gtk-doc/html/",
+		    "/usr/share/gnome/html/",
+		    NULL
+		};
+		const char *fn = rpmfiFN(fi);
+		const char **dnp;
+		for (dnp = ignorelist; *dnp != NULL; dnp++) {
+		    if (strstr(fn, *dnp) == fn) {
+			fi->actions[fileNum] = FA_CREATE;
+			rConflicts = 0;
+			break;
+		    }
+		}
+	    }
+#endif
 
 	    if (rConflicts) {
 		rpmpsAppend(ps, RPMPROB_FILE_CONFLICT,
@@ -191,7 +228,7 @@ assert(he->p.str != NULL);
 	    }
 
 	    /* Save file identifier to mark as state REPLACED. */
-	    if ( !(((FFlags | oFFlags) & RPMFILE_CONFIG) || XFA_SKIPPING(fi->actions[fileNum])) ) {
+	    if ( !(((FFlags | oFFlags) & RPMFILE_CONFIG) || iosmFileActionSkipped(fi->actions[fileNum])) ) {
 		/*@-assignexpose@*/
 		if (!shared->isRemoved)
 		    p->replaced[p->nreplaced++] = *shared;
@@ -203,7 +240,7 @@ assert(he->p.str != NULL);
 	if (((FFlags | oFFlags) & RPMFILE_CONFIG)) {
 	    int skipMissing =
 		((rpmtsFlags(ts) & RPMTRANS_FLAG_ALLFILES) ? 0 : 1);
-	    fileAction action = rpmfiDecideFate(otherFi, fi, skipMissing);
+	    iosmFileAction action = rpmfiDecideFate(otherFi, fi, skipMissing);
 	    fi->actions[fileNum] = action;
 	}
 	fi->replacedSizes[fileNum] = rpmfiFSize(otherFi);
@@ -233,13 +270,13 @@ static int handleRmvdInstalledFiles(const rpmts ts, rpmfi fi,
     const unsigned char * otherStates;
     int i, xx;
 
-    rpmdbMatchIterator mi;
+    rpmmi mi;
 
     mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES,
 			&shared->otherPkg, sizeof(shared->otherPkg));
-    h = rpmdbNextIterator(mi);
+    h = rpmmiNext(mi);
     if (h == NULL) {
-	mi = rpmdbFreeIterator(mi);
+	mi = rpmmiFree(mi);
 	return 1;
     }
 
@@ -260,7 +297,7 @@ static int handleRmvdInstalledFiles(const rpmts ts, rpmfi fi,
 	fi->actions[fileNum] = FA_SKIP;
     }
     he->p.ptr = _free(he->p.ptr);
-    mi = rpmdbFreeIterator(mi);
+    mi = rpmmiFree(mi);
 
     return 0;
 }
@@ -362,7 +399,7 @@ static void handleOverlappedFiles(const rpmts ts,
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fi, fileSystem, internalState @*/
 {
-    uint32_t fixupSize = 0;
+    rpmuint32_t fixupSize = 0;
     rpmps ps;
     const char * fn;
     int i, j;
@@ -371,18 +408,18 @@ static void handleOverlappedFiles(const rpmts ts,
     fi = rpmfiInit(fi, 0);
     if (fi != NULL)
     while ((i = rpmfiNext(fi)) >= 0) {
-	uint32_t tscolor = rpmtsColor(ts);
-	uint32_t prefcolor = rpmtsPrefColor(ts);
-	uint32_t oFColor, FColor;
+	rpmuint32_t tscolor = rpmtsColor(ts);
+	rpmuint32_t prefcolor = rpmtsPrefColor(ts);
+	rpmuint32_t oFColor, FColor;
 	struct fingerPrint_s * fiFps;
 	int otherPkgNum, otherFileNum;
 	rpmfi otherFi;
-	uint32_t FFlags;
-	uint16_t FMode;
+	rpmuint32_t FFlags;
+	rpmuint16_t FMode;
 	const rpmfi * recs;
 	int numRecs;
 
-	if (XFA_SKIPPING(fi->actions[i]))
+	if (iosmFileActionSkipped(fi->actions[i]))
 	    continue;
 
 	fn = rpmfiFN(fi);
@@ -484,7 +521,7 @@ assert(otherFi != NULL);
 		if (tscolor != 0) {
 		    if (FColor & prefcolor) {
 			/* ... last file of preferred colour is installed ... */
-			if (!XFA_SKIPPING(fi->actions[i])) {
+			if (!iosmFileActionSkipped(fi->actions[i])) {
 			    /* XXX static helpers are order dependent. Ick. */
 			    if (strcmp(fn, "/usr/sbin/libgcc_post_upgrade")
 			     && strcmp(fn, "/usr/sbin/glibc_post_upgrade"))
@@ -495,7 +532,7 @@ assert(otherFi != NULL);
 		    } else
 		    if (oFColor & prefcolor) {
 			/* ... first file of preferred colour is installed ... */
-			if (XFA_SKIPPING(fi->actions[i]))
+			if (iosmFileActionSkipped(fi->actions[i]))
 			    otherFi->actions[otherFileNum] = FA_CREATE;
 			fi->actions[i] = FA_SKIPCOLOR;
 			rConflicts = 0;
@@ -543,7 +580,7 @@ assert(otherFi != NULL);
 		/* Here is an overlapped removed file: skip in previous. */
 		otherFi->actions[otherFileNum] = FA_SKIP;
 	    }
-	    if (XFA_SKIPPING(fi->actions[i]))
+	    if (iosmFileActionSkipped(fi->actions[i]))
 		/*@switchbreak@*/ break;
 	    if (rpmfiFState(fi) != RPMFILE_STATE_NORMAL)
 		/*@switchbreak@*/ break;
@@ -589,20 +626,24 @@ assert(digest != NULL);
 /*@-nullpass@*/
 static int ensureOlder(rpmts ts,
 		const rpmte p, const Header h)
-	/*@modifies ts @*/
+	/*@globals internalState @*/
+	/*@modifies ts, internalState @*/
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
-    uint32_t reqFlags = (RPMSENSE_LESS | RPMSENSE_EQUAL);
+    rpmuint32_t reqFlags = (RPMSENSE_LESS | RPMSENSE_EQUAL);
     const char * reqEVR;
     rpmds req;
     char * t;
-    int nb;
+    size_t nb;
     int rc;
 
     if (p == NULL || h == NULL)
 	return 1;
 
     nb = strlen(rpmteNEVR(p)) + (rpmteE(p) != NULL ? strlen(rpmteE(p)) : 0) + 1;
+#ifdef	RPM_VENDOR_MANDRIVA
+    nb += (rpmteD(p) != NULL ? strlen(rpmteD(p)) + 1 : 0);
+#endif
     t = alloca(nb);
     *t = '\0';
     reqEVR = t;
@@ -610,10 +651,14 @@ static int ensureOlder(rpmts ts,
     if (rpmteV(p) != NULL)	t = stpcpy(t, rpmteV(p));
     *t++ = '-';
     if (rpmteR(p) != NULL)	t = stpcpy(t, rpmteR(p));
+#ifdef RPM_VENDOR_MANDRIVA
+    if (rpmteD(p) != NULL)	*t++ = ':', t = stpcpy(t, rpmteD(p));
+#endif
 
     req = rpmdsSingle(RPMTAG_REQUIRENAME, rpmteN(p), reqEVR, reqFlags);
     rc = rpmdsNVRMatchesDep(h, req, _rpmds_nopromote);
-    req = rpmdsFree(req);
+    (void)rpmdsFree(req);
+    req = NULL;
 
     if (rc == 0) {
 	rpmps ps = rpmtsProblems(ts);
@@ -646,19 +691,21 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
 	/*@modifies fi, rpmGlobalMacroContext, internalState @*/
 {
-    uint32_t tscolor = rpmtsColor(ts);
-    uint32_t FColor;
+    rpmuint32_t tscolor = rpmtsColor(ts);
+    rpmuint32_t FColor;
     int noConfigs = (rpmtsFlags(ts) & RPMTRANS_FLAG_NOCONFIGS);
     int noDocs = (rpmtsFlags(ts) & RPMTRANS_FLAG_NODOCS);
-    char ** netsharedPaths = NULL;
-    const char ** languages;
+    ARGV_t netsharedPaths = NULL;
+    ARGV_t languages = NULL;
     const char * dn, * bn;
-    int dnlen, bnlen, ix;
+    size_t dnlen, bnlen;
+    int ix;
     const char * s;
     int * drc;
     char * dff;
     int dc;
     int i, j;
+    int xx;
 
 #if defined(RPM_VENDOR_OPENPKG) /* allow-excludedocs-default */
     /* The "%_excludedocs" macro is intended to set the _default_ if
@@ -671,20 +718,19 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	noDocs = rpmExpandNumeric("%{_excludedocs}");
 #endif
 
-    {	const char *tmpPath = rpmExpand("%{_netsharedpath}", NULL);
-	if (tmpPath && *tmpPath != '%')
-	    netsharedPaths = splitString(tmpPath, strlen(tmpPath), ':');
+    {	const char *tmpPath = rpmExpand("%{?_netsharedpath}", NULL);
+	if (tmpPath && *tmpPath)
+	    xx = argvSplit(&netsharedPaths, tmpPath, ":");
 	tmpPath = _free(tmpPath);
     }
 
-    s = rpmExpand("%{_install_langs}", NULL);
-    if (!(s && *s != '%'))
+    s = rpmExpand("%{?_install_langs}", NULL);
+    if (!(s && *s))
 	s = _free(s);
     if (s) {
-	languages = (const char **) splitString(s, strlen(s), ':');
+	xx = argvSplit(&languages, s, ":");
 	s = _free(s);
-    } else
-	languages = NULL;
+    }
 
     /* Compute directory refcount, skip directory if now empty. */
     dc = rpmfiDC(fi);
@@ -697,7 +743,7 @@ static void skipFiles(const rpmts ts, rpmfi fi)
     if (fi != NULL)	/* XXX lclint */
     while ((i = rpmfiNext(fi)) >= 0)
     {
-	char ** nsp;
+	ARGV_t nsp;
 
 	bn = rpmfiBN(fi);
 	bnlen = strlen(bn);
@@ -710,7 +756,7 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	drc[ix]++;
 
 	/* Don't bother with skipped files */
-	if (XFA_SKIPPING(fi->actions[i])) {
+	if (iosmFileActionSkipped(fi->actions[i])) {
 	    drc[ix]--; dff[ix] = 1;
 	    continue;
 	}
@@ -729,7 +775,7 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	 * they do need to take package relocations into account).
 	 */
 	for (nsp = netsharedPaths; nsp && *nsp; nsp++) {
-	    int len;
+	    size_t len;
 
 	    len = strlen(*nsp);
 	    if (dnlen >= len) {
@@ -767,7 +813,8 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	 * Skip i18n language specific files.
 	 */
 	if (languages != NULL && fi->flangs != NULL && *fi->flangs[i]) {
-	    const char **lang, *l, *le;
+	    ARGV_t lang;
+	    const char *l, *le;
 	    for (lang = languages; *lang != NULL; lang++) {
 		if (!strcmp(*lang, "all"))
 		    /*@innerbreak@*/ break;
@@ -834,9 +881,9 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	if (fi != NULL)		/* XXX lclint */
 	while ((i = rpmfiNext(fi)) >= 0) {
 	    const char * fdn, * fbn;
-	    uint16_t fFMode;
+	    rpmuint16_t fFMode;
 
-	    if (XFA_SKIPPING(fi->actions[i]))
+	    if (iosmFileActionSkipped(fi->actions[i]))
 		/*@innercontinue@*/ continue;
 
 	    fFMode = rpmfiFMode(fi);
@@ -860,8 +907,8 @@ static void skipFiles(const rpmts ts, rpmfi fi)
     }
 
 /*@-dependenttrans@*/
-    if (netsharedPaths) freeSplitString(netsharedPaths);
-    if (languages) freeSplitString((char **)languages);
+    netsharedPaths = argvFree(netsharedPaths);
+    languages = argvFree(languages);
 /*@=dependenttrans@*/
 }
 /*@=nullpass@*/
@@ -923,7 +970,7 @@ static rpmRC _processFailedPackage(rpmts ts, rpmte p)
 assert(psm != NULL);
 	psm->stepName = "failed";	/* XXX W2DO? */
 	rc = rpmpsmStage(psm, PSM_RPMDB_ADD);
-	psm = rpmpsmFree(psm);
+	psm = rpmpsmFree(psm, "_processFailedPackage");
     }
     return rc;
 }
@@ -936,7 +983,7 @@ rpmRC rpmtsRollback(rpmts rbts, rpmprobFilterFlags ignoreSet, int running, rpmte
 {
     const char * semfn = NULL;
     rpmRC rc = 0;
-    uint32_t arbgoal = rpmtsARBGoal(rbts);
+    rpmuint32_t arbgoal = rpmtsARBGoal(rbts);
     QVA_t ia = memset(alloca(sizeof(*ia)), 0, sizeof(*ia));
     time_t ttid;
     int xx;
@@ -1112,11 +1159,10 @@ static int markLinkedFailed(rpmts ts, rpmte p)
     return 0;
 }
 
-#define	NOTIFY(_ts, _al) /*@i@*/ if ((_ts)->notify) (void) (_ts)->notify _al
-
 int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 {
-    uint32_t tscolor = rpmtsColor(ts);
+    static const char msg[] = "rpmtsRun";
+    rpmuint32_t tscolor = rpmtsColor(ts);
     int i, j;
     int ourrc = 0;
     int totalFileCount = 0;
@@ -1133,6 +1179,7 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     int numRemoved;
     int rollbackFailures = 0;
     void * lock = NULL;
+    void * ptr;
     int xx;
 
     /* XXX programmer error segfault avoidance. */
@@ -1171,8 +1218,21 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     if (rpmtsFlags(ts) & RPMTRANS_FLAG_JUSTDB)
 	(void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | _noTransScripts | _noTransTriggers));
 
+    /* if SELinux isn't enabled or init fails, don't bother... */
+    if (!rpmtsSELinuxEnabled(ts))
+	(void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | RPMTRANS_FLAG_NOCONTEXTS));
+
+    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOCONTEXTS)) {
+	const char * fn = rpmGetPath("%{?_install_file_context_path}", NULL);
+/*@-moduncon@*/
+	int xx = matchpathcon_init(fn);
+/*@=moduncon@*/
+        if (xx == -1)
+	    (void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | RPMTRANS_FLAG_NOCONTEXTS));
+	fn = _free(fn);
+    }
+
     ts->probs = rpmpsFree(ts->probs);
-    ts->probs = rpmpsCreate();
 
     /* XXX Make sure the database is open RDWR for package install/erase. */
     {	int dbmode = O_RDONLY;
@@ -1203,7 +1263,7 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     (void) rpmtsSetChrootDone(ts, 0);
 
     /* XXX rpmtsCreate() sets the transaction id, but apps may not honor. */
-    {	uint32_t tid = (uint32_t) time(NULL);
+    {	rpmuint32_t tid = (rpmuint32_t) time(NULL);
 	(void) rpmtsSetTid(ts, tid);
     }
 
@@ -1224,7 +1284,7 @@ rpmlog(RPMLOG_DEBUG, D_("sanity checking %d elements\n"), rpmtsNElements(ts));
     pi = rpmtsiInit(ts);
     /* XXX Only added packages need be checked. */
     while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
-	rpmdbMatchIterator mi;
+	rpmmi mi;
 	int fc;
 
 	if (p->isSource) continue;
@@ -1235,34 +1295,38 @@ rpmlog(RPMLOG_DEBUG, D_("sanity checking %d elements\n"), rpmtsNElements(ts));
 	if (!(rpmtsFilterFlags(ts) & RPMPROB_FILTER_OLDPACKAGE)) {
 	    Header h;
 	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(p), 0);
-	    while ((h = rpmdbNextIterator(mi)) != NULL)
+	    while ((h = rpmmiNext(mi)) != NULL)
 		xx = ensureOlder(ts, p, h);
-	    mi = rpmdbFreeIterator(mi);
+	    mi = rpmmiFree(mi);
 	}
 
 	if (!(rpmtsFilterFlags(ts) & RPMPROB_FILTER_REPLACEPKG)) {
 	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(p), 0);
-	    xx = rpmdbSetIteratorRE(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP,
+	    xx = rpmmiAddPattern(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP,
 				rpmteE(p));
-	    xx = rpmdbSetIteratorRE(mi, RPMTAG_VERSION, RPMMIRE_STRCMP,
+	    xx = rpmmiAddPattern(mi, RPMTAG_VERSION, RPMMIRE_STRCMP,
 				rpmteV(p));
-	    xx = rpmdbSetIteratorRE(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP,
+	    xx = rpmmiAddPattern(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP,
 				rpmteR(p));
+#ifdef	RPM_VENDOR_MANDRIVA
+	    xx = rpmmiAddPattern(mi, RPMTAG_DISTEPOCH, RPMMIRE_STRCMP,
+				rpmteD(p));
+#endif
 	    if (tscolor) {
-		xx = rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_STRCMP,
+		xx = rpmmiAddPattern(mi, RPMTAG_ARCH, RPMMIRE_STRCMP,
 				rpmteA(p));
-		xx = rpmdbSetIteratorRE(mi, RPMTAG_OS, RPMMIRE_STRCMP,
+		xx = rpmmiAddPattern(mi, RPMTAG_OS, RPMMIRE_STRCMP,
 				rpmteO(p));
 	    }
 
-	    while (rpmdbNextIterator(mi) != NULL) {
+	    while (rpmmiNext(mi) != NULL) {
 		rpmpsAppend(ps, RPMPROB_PKG_INSTALLED,
 			rpmteNEVR(p), rpmteKey(p),
 			NULL, NULL,
 			NULL, 0);
 		/*@innerbreak@*/ break;
 	    }
-	    mi = rpmdbFreeIterator(mi);
+	    mi = rpmmiFree(mi);
 	}
 
 	/* Count no. of files (if any). */
@@ -1289,9 +1353,10 @@ rpmlog(RPMLOG_DEBUG, D_("sanity checking %d elements\n"), rpmtsNElements(ts));
 
     /* Run pre-transaction scripts, but only if there are no known
      * problems up to this point. */
-    if (!((rpmtsFlags(ts) & (RPMTRANS_FLAG_BUILD_PROBS|RPMTRANS_FLAG_TEST))
+    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPRETRANS) &&
+       (!((rpmtsFlags(ts) & (RPMTRANS_FLAG_BUILD_PROBS|RPMTRANS_FLAG_TEST))
      	  || (rpmpsNumProblems(ts->probs) &&
-		(okProbs == NULL || rpmpsTrim(ts->probs, okProbs)))))
+		(okProbs == NULL || rpmpsTrim(ts->probs, okProbs))))))
     {
 	rpmlog(RPMLOG_DEBUG, D_("running pre-transaction scripts\n"));
 	pi = rpmtsiInit(ts);
@@ -1304,9 +1369,8 @@ rpmlog(RPMLOG_DEBUG, D_("sanity checking %d elements\n"), rpmtsNElements(ts));
 	    if (fi->pretrans == NULL)
 		continue;
 
-	    p->fd = ts->notify(p->h, RPMCALLBACK_INST_OPEN_FILE, 0, 0,
-			    rpmteKey(p), ts->notifyData);
 	    p->h = NULL;
+	    p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_OPEN_FILE, 0, 0);
 	    if (rpmteFd(p) != NULL) {
 		rpmVSFlags ovsflags = rpmtsVSFlags(ts);
 		rpmVSFlags vsflags = ovsflags | RPMVSF_NEEDPAYLOAD;
@@ -1317,11 +1381,7 @@ rpmlog(RPMLOG_DEBUG, D_("sanity checking %d elements\n"), rpmtsNElements(ts));
 		vsflags = rpmtsSetVSFlags(ts, ovsflags);
 		switch (rpmrc) {
 		default:
-		    /*@-noeffectuncon@*/ /* FIX: notify annotations */
-		    p->fd = ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE,
-				    0, 0,
-				    rpmteKey(p), ts->notifyData);
-		    /*@=noeffectuncon@*/
+		    p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_CLOSE_FILE, 0, 0);
 		    p->fd = NULL;
 		    /*@switchbreak@*/ break;
 		case RPMRC_NOTTRUSTED:
@@ -1346,14 +1406,14 @@ assert(psm != NULL);
 		psm->scriptTag = RPMTAG_PRETRANS;
 		psm->progTag = RPMTAG_PRETRANSPROG;
 		xx = rpmpsmStage(psm, PSM_SCRIPT);
-		psm = rpmpsmFree(psm);
+		psm = rpmpsmFree(psm, msg);
 
-/*@-noeffectuncon -compdef -usereleased @*/
-		(void) ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
-				  rpmteKey(p), ts->notifyData);
-/*@=noeffectuncon =compdef =usereleased @*/
+/*@-compdef -usereleased @*/
+		p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_CLOSE_FILE, 0, 0);
+/*@=compdef =usereleased @*/
 		p->fd = NULL;
-		p->h = headerFree(p->h);
+		(void)headerFree(p->h);
+		p->h = NULL;
 	    }
 	}
 	pi = rpmtsiFree(pi);
@@ -1438,7 +1498,7 @@ rpmlog(RPMLOG_DEBUG, D_("computing %d file fingerprints\n"), totalFileCount);
  	fi = rpmfiInit(fi, 0);
  	if (fi != NULL)		/* XXX lclint */
 	while ((i = rpmfiNext(fi)) >= 0) {
-	    if (XFA_SKIPPING(fi->actions[i]))
+	    if (iosmFileActionSkipped(fi->actions[i]))
 		/*@innercontinue@*/ continue;
 	    /*@-dependenttrans@*/
 	    htAddEntry(ts->ht, fi->fps + i, (void *) fi);
@@ -1449,8 +1509,7 @@ rpmlog(RPMLOG_DEBUG, D_("computing %d file fingerprints\n"), totalFileCount);
     }
     pi = rpmtsiFree(pi);
 
-    NOTIFY(ts, (NULL, RPMCALLBACK_TRANS_START, 6, ts->orderCount,
-	NULL, ts->notifyData));
+    ptr = rpmtsNotify(ts, NULL, RPMCALLBACK_TRANS_START, 6, ts->orderCount);
 
     /* ===============================================
      * Compute file disposition for each package in transaction set.
@@ -1471,8 +1530,8 @@ rpmlog(RPMLOG_DEBUG, D_("computing file dispositions\n"));
 	    continue;	/* XXX can't happen */
 	fc = rpmfiFC(fi);
 
-	NOTIFY(ts, (NULL, RPMCALLBACK_TRANS_PROGRESS, rpmtsiOc(pi),
-			ts->orderCount, NULL, ts->notifyData));
+	ptr = rpmtsNotify(ts, NULL, RPMCALLBACK_TRANS_PROGRESS, rpmtsiOc(pi),
+			ts->orderCount);
 
 	if (fc == 0) continue;
 
@@ -1499,7 +1558,7 @@ rpmlog(RPMLOG_DEBUG, D_("computing file dispositions\n"));
  	fi = rpmfiInit(fi, 0);
 	while ((i = rpmfiNext(fi)) >= 0) {
 	    struct stat sb, *st = &sb;
-	    uint32_t FFlags = rpmfiFFlags(fi);
+	    rpmuint32_t FFlags = rpmfiFFlags(fi);
 	    numShared += dbiIndexSetCount(matches[i]);
 	    if (!(FFlags & RPMFILE_CONFIG))
 		/*@innercontinue@*/ continue;
@@ -1520,7 +1579,7 @@ rpmlog(RPMLOG_DEBUG, D_("computing file dispositions\n"));
 	     * Take care not to mark files as replaced in packages that will
 	     * have been removed before we will get here.
 	     */
-	    for (j = 0; j < dbiIndexSetCount(matches[i]); j++) {
+	    for (j = 0; j < (int)dbiIndexSetCount(matches[i]); j++) {
 		int ro;
 		ro = dbiIndexRecordOffset(matches[i], j);
 		knownBad = 0;
@@ -1565,7 +1624,7 @@ rpmlog(RPMLOG_DEBUG, D_("computing file dispositions\n"));
 	    beingRemoved = 0;
 	    if (ts->removedPackages != NULL)
 	    for (j = 0; j < ts->numRemovedPackages; j++) {
-		if (ts->removedPackages[j] != shared->otherPkg)
+		if (ts->removedPackages[j] != (int)shared->otherPkg)
 		    /*@innercontinue@*/ continue;
 		beingRemoved = 1;
 		/*@innerbreak@*/ break;
@@ -1618,8 +1677,7 @@ rpmlog(RPMLOG_DEBUG, D_("computing file dispositions\n"));
 	    xx = Chdir(currDir);
     }
 
-    NOTIFY(ts, (NULL, RPMCALLBACK_TRANS_STOP, 6, ts->orderCount,
-	NULL, ts->notifyData));
+    ptr = rpmtsNotify(ts, NULL, RPMCALLBACK_TRANS_STOP, 6, ts->orderCount);
 
     /* ===============================================
      * Free unused memory as soon as possible.
@@ -1654,9 +1712,8 @@ rpmlog(RPMLOG_DEBUG, D_("computing file dispositions\n"));
      * Save removed files before erasing.
      */
     if (rpmtsFlags(ts) & (RPMTRANS_FLAG_DIRSTASH | RPMTRANS_FLAG_REPACKAGE)) {
-	int progress;
+	int progress = 0;
 
-	progress = 0;
 	pi = rpmtsiInit(ts);
 	while ((p = rpmtsiNext(pi, 0)) != NULL) {
 
@@ -1672,26 +1729,26 @@ rpmlog(RPMLOG_DEBUG, D_("computing file dispositions\n"));
 		if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_REPACKAGE))
 		    /*@switchbreak@*/ break;
 		if (!progress)
-		    NOTIFY(ts, (NULL, RPMCALLBACK_REPACKAGE_START,
-				7, numRemoved, NULL, ts->notifyData));
+		    ptr = rpmtsNotify(ts, NULL, RPMCALLBACK_REPACKAGE_START,
+				7, numRemoved);
 
-		NOTIFY(ts, (NULL, RPMCALLBACK_REPACKAGE_PROGRESS, progress,
-			numRemoved, NULL, ts->notifyData));
+		ptr = rpmtsNotify(ts, NULL, RPMCALLBACK_REPACKAGE_PROGRESS,
+				progress, numRemoved);
 		progress++;
 
 		(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_REPACKAGE), 0);
 
-	/* XXX TR_REMOVED needs CPIO_MAP_{ABSOLUTE,ADDDOT} CPIO_ALL_HARDLINKS */
-		fi->mapflags |= CPIO_MAP_ABSOLUTE;
-		fi->mapflags |= CPIO_MAP_ADDDOT;
-		fi->mapflags |= CPIO_ALL_HARDLINKS;
+	/* XXX TR_REMOVED needs IOSM_MAP_{ABSOLUTE,ADDDOT} IOSM_ALL_HARDLINKS */
+		fi->mapflags |= IOSM_MAP_ABSOLUTE;
+		fi->mapflags |= IOSM_MAP_ADDDOT;
+		fi->mapflags |= IOSM_ALL_HARDLINKS;
 		psm = rpmpsmNew(ts, p, fi);
 assert(psm != NULL);
 		xx = rpmpsmStage(psm, PSM_PKGSAVE);
-		psm = rpmpsmFree(psm);
-		fi->mapflags &= ~CPIO_MAP_ABSOLUTE;
-		fi->mapflags &= ~CPIO_MAP_ADDDOT;
-		fi->mapflags &= ~CPIO_ALL_HARDLINKS;
+		psm = rpmpsmFree(psm, msg);
+		fi->mapflags &= ~IOSM_MAP_ABSOLUTE;
+		fi->mapflags &= ~IOSM_MAP_ADDDOT;
+		fi->mapflags &= ~IOSM_ALL_HARDLINKS;
 
 		(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_REPACKAGE), 0);
 
@@ -1699,10 +1756,9 @@ assert(psm != NULL);
 	    }
 	}
 	pi = rpmtsiFree(pi);
-	if (progress) {
-	    NOTIFY(ts, (NULL, RPMCALLBACK_REPACKAGE_STOP, 7, numRemoved,
-			NULL, ts->notifyData));
-	}
+	if (progress)
+	    ptr = rpmtsNotify(ts, NULL, RPMCALLBACK_REPACKAGE_STOP,
+				7, numRemoved);
     }
 
     /* ===============================================
@@ -1722,8 +1778,10 @@ assert(psm != NULL);
 	
 	psm = rpmpsmNew(ts, p, fi);
 assert(psm != NULL);
-	psm->unorderedSuccessor =
-		(rpmtsiOc(pi) >= rpmtsUnorderedSuccessors(ts, -1) ? 1 : 0);
+	if (rpmtsiOc(pi) >= rpmtsUnorderedSuccessors(ts, -1))
+	    psm->flags |= RPMPSM_FLAGS_UNORDERED;
+	else
+	    psm->flags &= ~RPMPSM_FLAGS_UNORDERED;
 
 	switch (rpmteType(p)) {
 	case TR_ADDED:
@@ -1737,10 +1795,7 @@ assert(psm != NULL);
 	    p->h = NULL;
 	    /*@-type@*/ /* FIX: rpmte not opaque */
 	    {
-		/*@-noeffectuncon@*/ /* FIX: notify annotations */
-		p->fd = ts->notify(p->h, RPMCALLBACK_INST_OPEN_FILE, 0, 0,
-				rpmteKey(p), ts->notifyData);
-		/*@=noeffectuncon@*/
+		p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_OPEN_FILE, 0, 0);
 		if (rpmteFd(p) != NULL) {
 		    rpmVSFlags ovsflags = rpmtsVSFlags(ts);
 		    rpmVSFlags vsflags = ovsflags | RPMVSF_NEEDPAYLOAD;
@@ -1753,11 +1808,8 @@ assert(psm != NULL);
 
 		    switch (rpmrc) {
 		    default:
-			/*@-noeffectuncon@*/ /* FIX: notify annotations */
-			p->fd = ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE,
-					0, 0,
-					rpmteKey(p), ts->notifyData);
-			/*@=noeffectuncon@*/
+			p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_CLOSE_FILE,
+					0, 0);
 			p->fd = NULL;
 			ourrc++;
 			/*@innerbreak@*/ break;
@@ -1767,6 +1819,9 @@ assert(psm != NULL);
 			/*@innerbreak@*/ break;
 		    }
 		    if (rpmteFd(p) != NULL) gotfd = 1;
+		} else {
+		    ourrc++;
+		    xx = markLinkedFailed(ts, p);
 		}
 	    }
 	    /*@=type@*/
@@ -1778,8 +1833,8 @@ assert(psm != NULL);
 		 */
 		psm->fi = rpmfiFree(psm->fi);
 		{
-		    uint8_t * fstates = fi->fstates;
-		    fileAction * actions = fi->actions;
+		    rpmuint8_t * fstates = fi->fstates;
+		    iosmFileAction * actions = (iosmFileAction *) fi->actions;
 		    int mapflags = fi->mapflags;
 		    rpmte savep;
 		    int scareMem = 0;
@@ -1799,9 +1854,9 @@ assert(psm != NULL);
 			fi->fstates = _free(fi->fstates);
 			fi->fstates = fstates;
 			fi->actions = _free(fi->actions);
-			fi->actions = actions;
-			if (mapflags & CPIO_SBIT_CHECK)
-			    fi->mapflags |= CPIO_SBIT_CHECK;
+			fi->actions = (int *) actions;
+			if (mapflags & IOSM_SBIT_CHECK)
+			    fi->mapflags |= IOSM_SBIT_CHECK;
 			p->fi = fi;
 		    }
 		}
@@ -1810,21 +1865,22 @@ assert(psm != NULL);
 		if ((xx = rpmpsmStage(psm, PSM_PKGINSTALL)) != 0) {
 		    ourrc++;
 		    xx = markLinkedFailed(ts, p);
-		} else
+		}
+#if defined(RPM_VENDOR_MANDRIVA)
+		else {
+		    if(!rpmteIsSource(fi->te))
+	    		xx = mayAddToFilesAwaitingFiletriggers(rpmtsRootDir(ts), psm->fi, 1);
 		    p->done = 1;
+		}
+#endif
 
 	    } else {
 		ourrc++;
 	    }
 
 	    if (gotfd) {
-		/*@-noeffectuncon @*/ /* FIX: check rc */
-		(void) ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
-			rpmteKey(p), ts->notifyData);
-		/*@=noeffectuncon @*/
-		/*@-type@*/
+		p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_CLOSE_FILE, 0, 0);
 		p->fd = NULL;
-		/*@=type@*/
 	    }
 
 	    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_INSTALL), 0);
@@ -1839,10 +1895,16 @@ assert(psm != NULL);
 
 	    /* If linked element install failed, then don't erase. */
 	    if (p->linkFailed == 0) {
-		if ((xx != rpmpsmStage(psm, PSM_PKGERASE)) != 0) {
+		if ((xx = rpmpsmStage(psm, PSM_PKGERASE)) != 0) {
 		    ourrc++;
-		} else
+		}
+#if defined(RPM_VENDOR_MANDRIVA)
+		else {
+		    if(!rpmteIsSource(fi->te))
+			xx = mayAddToFilesAwaitingFiletriggers(rpmtsRootDir(ts), psm->fi, 0);
 		    p->done = 1;
+		}
+#endif
 	    } else
 		ourrc++;
 
@@ -1854,13 +1916,15 @@ assert(psm != NULL);
 	/* Would have freed header above in TR_ADD portion of switch
 	 * but needed the header to add it to the autorollback transaction.
 	 */
-	if (rpmteType(p) == TR_ADDED)
-	    p->h = headerFree(p->h);
+	if (rpmteType(p) == TR_ADDED) {
+	    (void)headerFree(p->h);
+	    p->h = NULL;
+	}
 
 	xx = rpmdbSync(rpmtsGetRdb(ts));
 
 /*@-nullstate@*/ /* FIX: psm->fi may be NULL */
-	psm = rpmpsmFree(psm);
+	psm = rpmpsmFree(psm, msg);
 /*@=nullstate@*/
 
 	/* If we received an error, lets break out and rollback, provided
@@ -1874,7 +1938,15 @@ assert(psm != NULL);
 /*@=nullpass@*/
     pi = rpmtsiFree(pi);
 
-    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)) {
+    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPOSTTRANS) &&
+	!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST))
+    {
+
+#if defined(RPM_VENDOR_MANDRIVA)
+	if ((rpmtsFlags(ts) & _noTransTriggers) != _noTransTriggers)
+	    rpmRunFileTriggers(rpmtsRootDir(ts));
+#endif
+
 	rpmlog(RPMLOG_DEBUG, D_("running post-transaction scripts\n"));
 	pi = rpmtsiInit(ts);
 	while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
@@ -1883,16 +1955,15 @@ assert(psm != NULL);
 	    if ((fi = rpmtsiFi(pi)) == NULL)
 	    	continue;	/* XXX can't happen */
 
-	    haspostscript = (fi->posttrans != NULL ? 1 : 0);
+	    haspostscript = (fi->posttrans || fi->posttransprog ? 1 : 0);
 	    p->fi = rpmfiFree(p->fi);
 
 	    /* If no post-transaction script, then don't bother. */
 	    if (!haspostscript)
 	    	continue;
 
-	    p->fd = ts->notify(p->h, RPMCALLBACK_INST_OPEN_FILE, 0, 0,
-	    	    	rpmteKey(p), ts->notifyData);
 	    p->h = NULL;
+	    p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_OPEN_FILE, 0, 0);
 	    if (rpmteFd(p) != NULL) {
 	    	rpmVSFlags ovsflags = rpmtsVSFlags(ts);
 	    	rpmVSFlags vsflags = ovsflags | RPMVSF_NEEDPAYLOAD;
@@ -1903,8 +1974,8 @@ assert(psm != NULL);
 	    	vsflags = rpmtsSetVSFlags(ts, ovsflags);
 	    	switch (rpmrc) {
 	    	default:
-	    	    p->fd = ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE,
-	    	    		0, 0, rpmteKey(p), ts->notifyData);
+		    p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_CLOSE_FILE,
+					0, 0);
 	    	    p->fd = NULL;
 	    	    /*@switchbreak@*/ break;
 	    	case RPMRC_NOTTRUSTED:
@@ -1928,21 +1999,26 @@ assert(psm != NULL);
 	    	psm->scriptTag = RPMTAG_POSTTRANS;
 	    	psm->progTag = RPMTAG_POSTTRANSPROG;
 	    	xx = rpmpsmStage(psm, PSM_SCRIPT);
-	    	psm = rpmpsmFree(psm);
+	    	psm = rpmpsmFree(psm, msg);
 
-/*@-noeffectuncon -compdef -usereleased @*/
-	    	(void) ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
-	    	    	      rpmteKey(p), ts->notifyData);
-/*@=noeffectuncon =compdef =usereleased @*/
+/*@-compdef -usereleased @*/
+		p->fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_CLOSE_FILE, 0, 0);
+/*@=compdef =usereleased @*/
 	    	p->fd = NULL;
 	    	p->fi = rpmfiFree(p->fi);
-	    	p->h = headerFree(p->h);
+	    	(void)headerFree(p->h);
+		p->h = NULL;
 	    }
 /*@=nullpass@*/
 	}
 	pi = rpmtsiFree(pi);
     }
 
+/*@-moduncon -noeffectuncon @*/
+    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOCONTEXTS))
+	matchpathcon_fini();
+/*@=moduncon =noeffectuncon @*/
+
     lock = rpmtsFreeLock(lock);
 
     /*@-nullstate@*/ /* FIX: ts->flList may be NULL */
