@@ -6,28 +6,48 @@
 #include "system.h"
 
 #include <rpmio.h>
-#define	_RPMFI_INTERNAL
-#define	_RPMPS_INTERNAL	/* XXX rpmps needs iterator. */
-#include <rpmcli.h>
+#include <rpmiotypes.h>
+#include <rpmcb.h>
+#include "ugid.h"
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
+
+#include <rpmfi.h>
 
 #define	_RPMSQ_INTERNAL
 #include "psm.h"
 
 #include "legacy.h"	/* XXX dodigest(), uidToUname(), gnameToGid */
-#include "ugid.h"
+
+#define	_RPMPS_INTERNAL	/* XXX rpmps needs iterator. */
+#define	_RPMTS_INTERNAL	/* XXX expose rpmtsSetScriptFd */
+#include <rpmcli.h>
+
 #include "debug.h"
 
-/*@access rpmfi @*/
+/*@access rpmts @*/	/* XXX cast */
 /*@access rpmpsm @*/	/* XXX for %verifyscript through rpmpsmStage() */
-/*@access rpmProblem @*/
 
 #define S_ISDEV(m) (S_ISBLK((m)) || S_ISCHR((m)))
 
 /*@unchecked@*/
 extern int _rpmds_unspecified_epoch_noise;
 
-int rpmVerifyFile(const rpmts ts, const rpmfi fi,
-		rpmVerifyAttrs * res, rpmVerifyAttrs omitMask)
+/** \ingroup rpmcli
+ * Verify file attributes (including file digest).
+ * @todo gnorpm and python bindings prevent this from being static.
+ * @param ts		transaction set
+ * @param fi		file info (with linked header and current file index)
+ * @retval *res		bit(s) returned to indicate failure
+ * @param omitMask	bit(s) to disable verify checks
+ * @return		0 on success (or not installed), 1 on error
+ */
+static int rpmVerifyFile(const rpmts ts, const rpmfi fi,
+		/*@out@*/ rpmVerifyAttrs * res, rpmVerifyAttrs omitMask)
+	/*@globals h_errno, fileSystem, internalState @*/
+	/*@modifies fi, *res, fileSystem, internalState @*/
+	/*@requires maxSet(res) >= 0 @*/
 {
     unsigned short fmode = rpmfiFMode(fi);
     rpmfileAttrs fileAttrs = rpmfiFFlags(fi);
@@ -183,15 +203,15 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	{
 	    *res |= RPMVERIFY_RDEV;
 	} else if (S_ISDEV(fmode) && S_ISDEV(sb.st_mode)) {
-	    uint16_t st_rdev = (sb.st_rdev & 0xffff);
-	    uint16_t frdev = (rpmfiFRdev(fi) & 0xffff);
+	    rpmuint16_t st_rdev = (rpmuint16_t)(sb.st_rdev & 0xffff);
+	    rpmuint16_t frdev = (rpmuint16_t)(rpmfiFRdev(fi) & 0xffff);
 	    if (st_rdev != frdev)
 		*res |= RPMVERIFY_RDEV;
 	}
     }
 
     if (flags & RPMVERIFY_MTIME) {
-	if (sb.st_mtime != rpmfiFMtime(fi))
+	if ((rpmuint32_t)sb.st_mtime != rpmfiFMtime(fi))
 	    *res |= RPMVERIFY_MTIME;
     }
 
@@ -249,7 +269,7 @@ static int rpmVerifyScript(/*@unused@*/ QVA_t qva, rpmts ts,
     if (scriptFd != NULL)
 	rpmtsSetScriptFd(psm->ts, NULL);
 
-    psm = rpmpsmFree(psm);
+    psm = rpmpsmFree(psm, "rpmVerifyScript");
 
     return rc;
 }
@@ -263,7 +283,7 @@ static int rpmVerifyScript(/*@unused@*/ QVA_t qva, rpmts ts,
  */
 static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	/*@globals h_errno, fileSystem, internalState @*/
-	/*@modifies ts, fi, fileSystem, internalState  @*/
+	/*@modifies fi, fileSystem, internalState  @*/
 {
     rpmVerifyAttrs verifyResult = 0;
     /*@-type@*/ /* FIX: union? */
@@ -277,7 +297,9 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
     te = t = buf;
     *te = '\0';
 
+/*@-castexpose@*/
     fi = rpmfiLink(fi, "verifyHeader");
+/*@=castexpose@*/
     fi = rpmfiInit(fi, 0);
     if (fi != NULL)	/* XXX lclint */
     while ((i = rpmfiNext(fi)) >= 0) {
@@ -301,6 +323,7 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 
 	rc = rpmVerifyFile(ts, fi, &verifyResult, omitMask);
 	if (rc) {
+	    if (qva->qva_mode != 'v')	/* XXX no output w verify(...) probe. */
 	    if (!(fflags & (RPMFILE_MISSINGOK|RPMFILE_GHOST)) || rpmIsVerbose()) {
 		sprintf(te, _("missing   %c %s"),
 			((fflags & RPMFILE_CONFIG)	? 'c' :
@@ -323,8 +346,10 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	    /*@observer@*/ static const char *const aok = ".";
 	    /*@observer@*/ static const char *const unknown = "?";
 
-	    ec = 1;
+	    if (!ec)
+		ec = (verifyResult != 0);
 
+	    if (qva->qva_mode != 'v') {	/* XXX no output w verify(...) probe. */
 #define	_verify(_RPMVERIFY_F, _C)	\
 	((verifyResult & _RPMVERIFY_F) ? _C : aok)
 #define	_verifylink(_RPMVERIFY_F, _C)	\
@@ -334,21 +359,21 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	((verifyResult & RPMVERIFY_READFAIL) ? unknown : \
 	 (verifyResult & _RPMVERIFY_F) ? _C : aok)
 	
-	    digest = _verifyfile(RPMVERIFY_FDIGEST, "5");
-	    size = _verify(RPMVERIFY_FILESIZE, "S");
-	    link = _verifylink(RPMVERIFY_LINKTO, "L");
-	    mtime = _verify(RPMVERIFY_MTIME, "T");
-	    rdev = _verify(RPMVERIFY_RDEV, "D");
-	    user = _verify(RPMVERIFY_USER, "U");
-	    group = _verify(RPMVERIFY_GROUP, "G");
-	    mode = _verify(RPMVERIFY_MODE, "M");
+		digest = _verifyfile(RPMVERIFY_FDIGEST, "5");
+		size = _verify(RPMVERIFY_FILESIZE, "S");
+		link = _verifylink(RPMVERIFY_LINKTO, "L");
+		mtime = _verify(RPMVERIFY_MTIME, "T");
+		rdev = _verify(RPMVERIFY_RDEV, "D");
+		user = _verify(RPMVERIFY_USER, "U");
+		group = _verify(RPMVERIFY_GROUP, "G");
+		mode = _verify(RPMVERIFY_MODE, "M");
 
 #undef _verifyfile
 #undef _verifylink
 #undef _verify
 
-	    sprintf(te, "%s%s%s%s%s%s%s%s  %c %s",
-		size, mode, digest, rdev, link, user, group, mtime,
+		sprintf(te, "%s%s%s%s%s%s%s%s  %c %s",
+		    size, mode, digest, rdev, link, user, group, mtime,
 			((fflags & RPMFILE_CONFIG)	? 'c' :
 			 (fflags & RPMFILE_DOC)	? 'd' :
 			 (fflags & RPMFILE_GHOST)	? 'g' :
@@ -356,9 +381,11 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 			 (fflags & RPMFILE_PUBKEY)	? 'P' :
 			 (fflags & RPMFILE_README)	? 'r' : ' '),
 			rpmfiFN(fi));
-	    te += strlen(te);
+		te += strlen(te);
+	    }
 	}
 
+	if (qva->qva_mode != 'v')	/* XXX no output w verify(...) probe. */
 	if (te > t) {
 	    *te++ = '\n';
 	    *te = '\0';
@@ -406,17 +433,18 @@ static int verifyDependencies(/*@unused@*/ QVA_t qva, rpmts ts,
 	const char * altNEVR;
 	const char * pkgNEVR = NULL;
 	rpmpsi psi;
-	rpmProblem p;
+	rpmProblem prob;
 	char * t, * te;
 	int nb = 512;
 
 	psi = rpmpsInitIterator(ps);
 	while (rpmpsNextIterator(psi) >= 0) {
-	    p = rpmpsProblem(psi);
+	    prob = rpmpsProblem(psi);
 	    if (pkgNEVR == NULL)
-		pkgNEVR = rpmProblemGetPkgNEVR(p);
+		pkgNEVR = rpmProblemGetPkgNEVR(prob);
 
-	    altNEVR = rpmProblemGetAltNEVR(p);
+	    altNEVR = rpmProblemGetAltNEVR(prob);
+assert(altNEVR != NULL);
 	    if (altNEVR[0] == 'R' && altNEVR[1] == ' ')
 		nb += sizeof("\tRequires: ")-1;
 	    if (altNEVR[0] == 'C' && altNEVR[1] == ' ')
@@ -433,9 +461,10 @@ static int verifyDependencies(/*@unused@*/ QVA_t qva, rpmts ts,
 
 	psi = rpmpsInitIterator(ps);
 	while (rpmpsNextIterator(psi) >= 0) {
-	    p = rpmpsProblem(psi);
+	    prob = rpmpsProblem(psi);
 
-	    altNEVR = (p->altNEVR ? p->altNEVR : "? ?altNEVR?");
+	    if ((altNEVR = rpmProblemGetAltNEVR(prob)) == NULL)
+		altNEVR = "? ?altNEVR?";
 	    if (altNEVR[0] == 'R' && altNEVR[1] == ' ')
 		te = stpcpy(te, "\tRequires: ");
 	    if (altNEVR[0] == 'C' && altNEVR[1] == ' ')
@@ -491,13 +520,12 @@ int showVerifyPackage(QVA_t qva, rpmts ts, Header h)
 	{
 	    FD_t fdo = fdDup(STDOUT_FILENO);
 
-/* XXX hackery to assert(!scaremem) in rpmfiNew. */
-if (fi->h == NULL) fi->h = headerLink(h);
+	    rc = rpmfiSetHeader(fi, h);
 	    if ((rc = rpmVerifyScript(qva, ts, fi, fdo)) != 0)
 		ec = rc;
 	    if (fdo != NULL)
 		rc = Fclose(fdo);
-fi->h = headerFree(fi->h);
+	    rc = rpmfiSetHeader(fi, NULL);
 	}
 
 	fi = rpmfiFree(fi);
