@@ -5,6 +5,8 @@
 * 05 Nov 2003 22:09:10
 */
 
+#include "system.h" /* setenv/unsetenv/glob/etc */
+
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -401,6 +403,7 @@ static int pushresult(lua_State *L, int i, const char *info)
 static void badoption(lua_State *L, /*@unused@*/ int i, const char *what, int option)
 	/*@modifies L @*/
 {
+	i = i;		/* XXX keep gcc happy. */
 	luaL_argerror(L, 2,
 		lua_pushfstring(L, "unknown %s option `%c'", what, option));
 }
@@ -415,7 +418,7 @@ static uid_t mygetuid(lua_State *L, int i)
 	else if (lua_isstring(L, i))
 	{
 		struct passwd *p=getpwnam(lua_tostring(L, i));
-		return (p==NULL) ? -1 : p->pw_uid;
+		return (p==NULL) ? (uid_t)-1 : p->pw_uid;
 	}
 	else
 		return luaL_typerror(L, i, "string or number");
@@ -431,7 +434,7 @@ static gid_t mygetgid(lua_State *L, int i)
 	else if (lua_isstring(L, i))
 	{
 		struct group *g=getgrnam(lua_tostring(L, i));
-		return (g==NULL) ? -1 : g->gr_gid;
+		return (g==NULL) ? (gid_t)-1 : g->gr_gid;
 	}
 	else
 		return luaL_typerror(L, i, "string or number");
@@ -535,6 +538,26 @@ static int Pmkdir(lua_State *L)			/** mkdir(path) */
 	return pushresult(L, mkdir(path, 0777), path);
 }
 
+#ifdef	NOTYET
+static int Pmkdtemp(lua_State *L)		/** mkdtemp(template) */
+	/*@globals fileSystem @*/
+	/*@modifies L, fileSystem @*/
+{
+	const char *template = luaL_checkstring(L, 1);
+	const char *path;
+	char buf[MYBUFSIZ];
+	(void) strncpy(buf, template, sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
+	if ((path = mkdtemp(buf)) == NULL)
+		return pusherror(L, template);
+	else
+	{
+		lua_pushstring(L, path);
+		return 1;
+	}
+}
+#endif
+
 
 static int Pchdir(lua_State *L)			/** chdir(path) */
 	/*@globals internalState @*/
@@ -687,7 +710,6 @@ static int Pputenv(lua_State *L)		/** putenv(string) */
 }
 
 
-#ifdef linux
 static int Psetenv(lua_State *L)		/** setenv(name,value,[over]) */
 	/*@modifies L @*/
 {
@@ -705,7 +727,6 @@ static int Punsetenv(lua_State *L)		/** unsetenv(name) */
 	unsetenv(name);
 	return 0;
 }
-#endif
 
 
 static int Pgetenv(lua_State *L)		/** getenv([name]) */
@@ -812,6 +833,7 @@ static int Putime(lua_State *L)			/** utime(path,[mtime,atime]) */
 static int FgetID(lua_State *L, int i, /*@unused@*/ const void *data)
 	/*@modifies L @*/
 {
+	data = data;		/* XXX keep gcc happy. */
 	switch (i)
 	{
 		case 0:	lua_pushnumber(L, getegid());	break;
@@ -837,6 +859,29 @@ static int Pgetprocessid(lua_State *L)		/** getprocessid([selector]) */
 	return doselection(L, 1, SgetID, FgetID, NULL);
 }
 
+static int Pglob(lua_State *L)  /** glob(pattern) */
+{
+    const char *pattern;
+    glob_t g;
+    int i;
+    int rc;
+
+    pattern = luaL_optstring(L, 1, ".");
+    rc = glob(pattern, 0, NULL, &g);
+    if (rc == 0) {
+        lua_newtable(L);
+        for (i = 1; i <= (int)g.gl_pathc; i++)
+            storeindex(L, i, g.gl_pathv[i - 1]);
+        globfree(&g);
+        return 1;
+    }
+    else if (rc == GLOB_NOMATCH) {
+        lua_newtable(L);
+        return 1;
+    }
+    else
+        return pusherror(L, pattern);
+}
 
 static int Pttyname(lua_State *L)		/** ttyname(fd) */
 	/*@modifies L @*/
@@ -1109,6 +1154,7 @@ static const int Ksysconf[] =
 static int Fsysconf(lua_State *L, int i, /*@unused@*/ const void *data)
 	/*@modifies L @*/
 {
+	data = data;		/* XXX keep gcc happy. */
 	lua_pushnumber(L, sysconf(Ksysconf[i]));
 	return 1;
 }
@@ -1127,6 +1173,38 @@ static int Psysconf(lua_State *L)		/** sysconf([selector]) */
 	return doselection(L, 1, Ssysconf, Fsysconf, NULL);
 }
 
+static int Pmkstemp(lua_State *L)
+	/*@modifies L @*/
+{
+	const char *path;
+	char *dynpath;
+	int fd;
+	FILE **f;
+
+	path = luaL_checkstring(L, 1);
+	if (path == NULL)
+		return 0;
+	dynpath = strdup(path);
+	fd = mkstemp(dynpath);
+	f = (FILE**)lua_newuserdata(L, sizeof(FILE*));
+	if (f == NULL) {
+		close(fd);
+		free(dynpath);
+		return 0;
+	}
+	*f = fdopen(fd, "a+");
+	lua_pushstring(L, dynpath);
+	free(dynpath);
+	luaL_getmetatable(L, "FILE*");
+	if (lua_isnil(L, -1)) {
+		lua_pop(L, 1);
+		luaL_error(L, "FILE* metatable not available "
+				"(io not loaded?)");
+	} else {
+		lua_setmetatable(L, -3);
+	}
+	return 2;
+}
 
 /*@-readonlytrans@*/
 /*@observer@*/ /*@unchecked@*/
@@ -1148,10 +1226,15 @@ static const luaL_reg R[] =
 	{"getlogin",		Pgetlogin},
 	{"getpasswd",		Pgetpasswd},
 	{"getprocessid",	Pgetprocessid},
+	{"glob",		Pglob},
 	{"kill",		Pkill},
 	{"link",		Plink},
 	{"mkdir",		Pmkdir},
+#ifdef	NOTYET
+	{"mkdtemp",		Pmkdtemp},
+#endif
 	{"mkfifo",		Pmkfifo},
+	{"mkstemp",		Pmkstemp},
 	{"pathconf",		Ppathconf},
 	{"putenv",		Pputenv},
 	{"readlink",		Preadlink},
@@ -1169,11 +1252,8 @@ static const luaL_reg R[] =
 	{"unlink",		Punlink},
 	{"utime",		Putime},
 	{"wait",		Pwait},
-
-#ifdef linux
 	{"setenv",		Psetenv},
 	{"unsetenv",		Punsetenv},
-#endif
 	{NULL,			NULL}
 };
 /*@=readonlytrans@*/
