@@ -68,10 +68,6 @@
 #include "strstr.c"
 #endif
 
-#include "fnmatch.h"
-#include "fnmatch.c"
-#ifndef __CYGWIN__
-#include "glob.h"
-#include "glob.c"
+#if !defined(HAVE_MKDTEMP)
+#include "mkdtemp.c"
 #endif
-
