@@ -24,6 +24,7 @@ use vars qw/@ISA @EXPORT/;
     reset_macros
     dump_macros
     platformscore
+    installsrpm
 );
 
 bootstrap RPM;
@@ -48,6 +49,13 @@ the installed RPM database as well as files on the filesystem.
 
 =head1 FUNCTIONS
 
+=head2 installsrpm($filename)
+
+Install a source rpm and return spec file path and its cookies.
+Returns undef if install is impossible.
+
+see L<RPM::Spec>->new() for more information about cookies.
+
 =head2 GENERICS FUNCTIONS
 
 =head3 rpmversion
@@ -143,6 +151,7 @@ DEPRECATED ?
 
 DEPRECATED ?
 
+
 =head1 TODO
 
 Make package installation and removal better (-;.
