@@ -5,16 +5,19 @@
 #include "system.h"
 
 #include "rpmio_internal.h"
+#include <rpmcb.h>
 
 #include "legacy.h"
 #define	_RPMTAG_INTERNAL
 #include "header_internal.h"	/* XXX HEADERFLAG_ALLOCATED */
-#include "rpmcli.h"	/* XXX for rpmCheckSig */
+#include "rpmtypes.h"
+#define	_RPMEVR_INTERNAL
+#include "rpmevr.h"
 #include "pkgio.h"		/* XXX rpmpkgRead */
 
 #include "rpmts.h"	/* XXX rpmtsCreate/rpmtsFree */
-#define	_RPMEVR_INTERNAL
-#include "rpmevr.h"
+
+#include "rpmcli.h"
 
 #include "header-py.h"
 #include "rpmds-py.h"
@@ -92,7 +95,7 @@
  * \code
  * 	import os, rpm
  *
- *	ts = rpm.TranssactionSet()
+ *	ts = rpm.TransactionSet()
  * 	fdno = os.open("/tmp/foo-1.0-1.i386.rpm", os.O_RDONLY)
  * 	hdr = ts.hdrFromFdno(fdno)
  *	os.close(fdno)
@@ -206,14 +209,16 @@ static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords)
     /* XXX this legacy switch is a hack, needs to be removed. */
     if (legacy) {
 	h = headerCopy(s->h);	/* XXX strip region tags, etc */
-	headerFree(s->h);
+	(void)headerFree(s->h);
+	s->h = NULL;
     }
     {	size_t len;
 	buf = headerUnload(h, &len);
 	nb = len;
 	nb -= 8;	/* XXX HEADER_MAGIC_NO */
     }
-    h = headerFree(h);
+    (void)headerFree(h);
+    h = NULL;
 
     if (buf == NULL || nb == 0) {
 	PyErr_SetString(pyrpmError, "can't unload bad header\n");
@@ -331,22 +336,23 @@ static void hdr_dealloc(hdrObject * s)
 	/*@*/
 {
     if (s->h) headerFree(s->h);
+    s->h = NULL;
     PyObject_Del(s);
 }
 
 /** \ingroup py_c
  */
-long tagNumFromPyObject (PyObject *item)
+rpmTag tagNumFromPyObject (PyObject *item)
 {
     char * str;
 
     if (PyInt_Check(item)) {
-	return PyInt_AsLong(item);
+	return (rpmTag) PyInt_AsLong(item);
     } else if (PyString_Check(item) || PyUnicode_Check(item)) {
 	str = PyString_AsString(item);
 	return tagValue(str);
     }
-    return -1;
+    return (rpmTag)0xffffffff;
 }
 
 /** \ingroup py_c
@@ -355,8 +361,8 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 	/*@*/
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
-    uint32_t tag = 0xffffffff;
-    int i;
+    rpmTag tag = (rpmTag)0xffffffff;
+    uint32_t i;
     PyObject * o, * metao;
     int forceArray = 0;
     const struct headerSprintfExtension_s * ext = NULL;
@@ -367,7 +373,7 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     else
 	tag = tagNumFromPyObject (item);
 
-    if (tag == 0xffffffff && (PyString_Check(item) || PyUnicode_Check(item))) {
+    if (tag == (rpmTag)0xffffffff && (PyString_Check(item) || PyUnicode_Check(item))) {
 	const struct headerSprintfExtension_s * extensions = rpmHeaderFormats;
 	char * str;
 	/* if we still don't have the tag, go looking for the header
@@ -388,7 +394,7 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     if (ext) {
         ext->u.tagFunction(s->h, he);
     } else {
-        if (tag == 0xffffffff) {
+        if (tag == (rpmTag)0xffffffff) {
             PyErr_SetString(PyExc_KeyError, "unknown header tag");
             return NULL;
         }
@@ -402,6 +408,7 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 	    case RPMTAG_NAME:
 	    case RPMTAG_VERSION:
 	    case RPMTAG_RELEASE:
+	    case RPMTAG_DISTEPOCH:
 	    case RPMTAG_ARCH:
 	    case RPMTAG_OS:
 		Py_INCREF(Py_None);
@@ -523,7 +530,10 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 	break;
 
     case RPM_STRING_TYPE:
-	o = PyString_FromString(he->p.str);
+	if (he->p.str != NULL)
+	    o = PyString_FromString(he->p.str);
+	else
+	    o = PyString_FromString("");
 	break;
 
     default:
@@ -540,9 +550,9 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
  */
 /*@unchecked@*/ /*@observer@*/
 static PyMappingMethods hdr_as_mapping = {
-	(inquiry) 0,			/* mp_length */
+	(lenfunc) 0,			/* mp_length */
 	(binaryfunc) hdr_subscript,	/* mp_subscript */
-	(objobjargproc)0,		/* mp_ass_subscript */
+	(objobjargproc) 0,		/* mp_ass_subscript */
 };
 
 static PyObject * hdr_getattro(hdrObject * o, PyObject * n)
@@ -657,7 +667,8 @@ PyObject * hdrLoad(PyObject * self, PyObject * args, PyObject * kwds)
     headerAllocated(h);
 
     hdr = hdr_Wrap(h);
-    h = headerFree(h);	/* XXX ref held by hdr */
+    (void)headerFree(h);	/* XXX ref held by hdr */
+    h = NULL;
 
     return (PyObject *) hdr;
 }
@@ -680,8 +691,12 @@ PyObject * rpmReadHeaders (FD_t fd)
     {   const char item[] = "Header";
 	const char * msg = NULL;
 	rpmRC rc = rpmpkgRead(item, fd, &h, &msg);
-	if (rc != RPMRC_OK)
-	    rpmlog(RPMLOG_ERR, "%s: %s: %s\n", "rpmpkgRead", item, msg);
+	if(rc == RPMRC_NOTFOUND) {
+		Py_INCREF(Py_None);
+		list = Py_None;
+	}
+	else if (rc != RPMRC_OK)
+	    rpmlog(RPMLOG_ERR, "%s: %s: %s : error code: %d\n", "rpmpkgRead", item, msg, rc);
 	msg = _free(msg);
     }
     Py_END_ALLOW_THREADS
@@ -695,14 +710,19 @@ PyObject * rpmReadHeaders (FD_t fd)
 	}
 	Py_DECREF(hdr);
 
-	h = headerFree(h);	/* XXX ref held by hdr */
+	(void)headerFree(h);	/* XXX ref held by hdr */
+	h = NULL;
 
 	Py_BEGIN_ALLOW_THREADS
 	{   const char item[] = "Header";
 	    const char * msg = NULL;
 	    rpmRC rc = rpmpkgRead(item, fd, &h, &msg);
-	    if (rc != RPMRC_OK && rc != RPMRC_NOTFOUND)
-		rpmlog(RPMLOG_ERR, "%s: %s: %s\n", "rpmpkgRead", item, msg);
+	    if(rc == RPMRC_NOTFOUND) {
+		    Py_INCREF(Py_None);
+		    list = Py_None;
+	    }
+	    else if (rc != RPMRC_OK)
+		rpmlog(RPMLOG_ERR, "%s: %s: %s : error code: %d\n", "rpmpkgRead", item, msg, rc);
 	    msg = _free(msg);
 	}
 	Py_END_ALLOW_THREADS
@@ -713,7 +733,8 @@ PyObject * rpmReadHeaders (FD_t fd)
 
 /**
  */
-PyObject * rpmHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
+PyObject * rpmHeaderFromFD(PyObject * self, PyObject * args,
+		PyObject * kwds)
 {
     FD_t fd;
     int fileno;
@@ -733,7 +754,8 @@ PyObject * rpmHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
 
 /**
  */
-PyObject * rpmHeaderFromFile(PyObject * self, PyObject * args, PyObject *kwds)
+PyObject * rpmHeaderFromFile(PyObject * self, PyObject * args,
+		PyObject *kwds)
 {
     char * filespec;
     FD_t fd;
@@ -759,7 +781,8 @@ PyObject * rpmHeaderFromFile(PyObject * self, PyObject * args, PyObject *kwds)
 /**
  */
 PyObject *
-rpmSingleHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
+rpmSingleHeaderFromFD(PyObject * self, PyObject * args,
+		PyObject * kwds)
 {
     FD_t fd;
     int fileno;
@@ -784,8 +807,12 @@ rpmSingleHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
     {   const char item[] = "Header";
 	const char * msg = NULL;
 	rpmRC rc = rpmpkgRead(item, fd, &h, &msg);
-	if (rc != RPMRC_OK)
-	    rpmlog(RPMLOG_ERR, "%s: %s: %s\n", "rpmpkgRead", item, msg);
+	if(rc == RPMRC_NOTFOUND) {
+		Py_INCREF(Py_None);
+		tuple = Py_None;
+	}
+	else if (rc != RPMRC_OK)
+	    rpmlog(RPMLOG_ERR, "%s: %s: %s : error code: %d\n", "rpmpkgRead", item, msg, rc);
 	msg = _free(msg);
     }
     Py_END_ALLOW_THREADS
@@ -797,7 +824,8 @@ rpmSingleHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
     if (h && tuple) {
 	PyTuple_SET_ITEM(tuple, 0, (PyObject *) hdr_Wrap(h));
 	PyTuple_SET_ITEM(tuple, 1, PyLong_FromLong(offset));
-	h = headerFree(h);
+	(void)headerFree(h);
+	h = NULL;
     } else {
 	Py_INCREF(Py_None);
 	Py_INCREF(Py_None);
@@ -810,7 +838,82 @@ rpmSingleHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
 
 /**
  */
-PyObject * versionCompare (PyObject * self, PyObject * args, PyObject * kwds)
+PyObject * rpmWriteHeaders (PyObject * list, FD_t fd)
+{
+    int count;
+
+    if (!fd) {
+	PyErr_SetFromErrno(pyrpmError);
+	return NULL;
+    }
+
+    for(count = 0; count < PyList_Size(list); count++){
+	Py_BEGIN_ALLOW_THREADS
+	const char item[] = "Header";
+	const char * msg = NULL;
+	hdrObject * hdr = (hdrObject *)PyList_GetItem(list, count);
+	rpmRC rc = rpmpkgWrite(item, fd, hdr->h, &msg);
+	if (rc != RPMRC_OK)
+	    rpmlog(RPMLOG_ERR, "%s: %s: %s : error code: %d\n", "rpmpkgWrite", item, msg, rc);
+	msg = _free(msg);
+	Py_END_ALLOW_THREADS
+    }
+    
+    Py_RETURN_TRUE;
+}
+
+/**
+ */
+PyObject * rpmHeaderToFD(PyObject * self, PyObject * args,
+		PyObject * kwds)
+{
+    FD_t fd;
+    int fileno;
+    PyObject * list;
+    PyObject * ret;
+    char * kwlist[] = {"headers", "fd", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "Oi", kwlist, &list, &fileno))
+	return NULL;
+
+    fd = fdDup(fileno);
+
+    ret = rpmWriteHeaders (list, fd);
+    Fclose(fd);
+
+    return list;
+}
+
+/**
+ */
+PyObject * rpmHeaderToFile(PyObject * self, PyObject * args,
+		PyObject *kwds)
+{
+    char * filespec;
+    FD_t fd;
+    PyObject * list;
+    PyObject * ret;
+    char * kwlist[] = {"headers", "file", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "Os", kwlist, &list, &filespec))
+	return NULL;
+
+    fd = Fopen(filespec, "w.fdio");
+    if (!fd) {
+	PyErr_SetFromErrno(pyrpmError);
+	return NULL;
+    }
+
+    ret = rpmWriteHeaders (list, fd);
+    Fclose(fd);
+
+    return ret; 
+} 
+
+/**
+ */
+PyObject * versionCompare (PyObject * self, PyObject * args,
+		PyObject * kwds)
 {
     hdrObject * h1, * h2;
     char * kwlist[] = {"version0", "version1", NULL};
@@ -824,22 +927,36 @@ PyObject * versionCompare (PyObject * self, PyObject * args, PyObject * kwds)
 
 PyObject * labelCompare (PyObject * self, PyObject * args)
 {
-    EVR_t A = memset(alloca(sizeof(*A)), 0, sizeof(*A));
-    EVR_t B = memset(alloca(sizeof(*B)), 0, sizeof(*B));
+    EVR_t a = rpmEVRnew(RPMSENSE_EQUAL, 1);
+    EVR_t b = rpmEVRnew(RPMSENSE_EQUAL, 1);
     int rc;
+    PyObject *aTuple, *bTuple;
 
-    if (!PyArg_ParseTuple(args, "(zzz)(zzz)",
-			&A->E, &A->V, &A->R, &B->E, &B->V, &B->R))
+    if (!PyArg_ParseTuple(args, "OO", &aTuple, &bTuple) ||
+	    !PyArg_ParseTuple(aTuple, "zzz|z",
+		&a->F[RPMEVR_E], &a->F[RPMEVR_V], &a->F[RPMEVR_R], &a->F[RPMEVR_D]) ||
+	    !PyArg_ParseTuple(bTuple, "zzz|z",
+		&b->F[RPMEVR_E], &b->F[RPMEVR_V], &b->F[RPMEVR_R], &b->F[RPMEVR_D]))
+    {
+	a = rpmEVRfree(a);
+	b = rpmEVRfree(b);
 	return NULL;
+    }
 
-    if (A->E == NULL)	A->E = "0";
-    if (B->E == NULL)	B->E = "0";
-    if (A->V == NULL)	A->E = "";
-    if (B->V == NULL)	B->E = "";
-    if (A->R == NULL)	A->E = "";
-    if (B->R == NULL)	B->E = "";
+    /* XXX HACK: postpone committing to single "missing" value for now. */
+    if (a->F[RPMEVR_E] == NULL)	a->F[RPMEVR_E] = "0";
+    if (b->F[RPMEVR_E] == NULL)	b->F[RPMEVR_E] = "0";
+    if (a->F[RPMEVR_V] == NULL)	a->F[RPMEVR_V] = "";
+    if (b->F[RPMEVR_V] == NULL)	b->F[RPMEVR_V] = "";
+    if (a->F[RPMEVR_R] == NULL)	a->F[RPMEVR_R] = "";
+    if (b->F[RPMEVR_R] == NULL)	b->F[RPMEVR_R] = "";
+    if (a->F[RPMEVR_D] == NULL)	a->F[RPMEVR_D] = "";
+    if (b->F[RPMEVR_D] == NULL)	b->F[RPMEVR_D] = "";
 
-    rc = rpmEVRcompare(A, B);
+    rc = rpmEVRcompare(a, b);
+
+    a = rpmEVRfree(a);
+    b = rpmEVRfree(b);
 
     return Py_BuildValue("i", rc);
 }
