@@ -37,7 +37,7 @@ Header hdrGetHeader(hdrObject * h)
 
 /** \ingroup py_c
  */
-long tagNumFromPyObject (PyObject *item)
+rpmTag tagNumFromPyObject (PyObject *item)
 	/*@*/;
 
 /** \ingroup py_c
@@ -65,11 +65,26 @@ PyObject * rpmHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
 PyObject * rpmSingleHeaderFromFD(PyObject * self, PyObject * args, PyObject * kwds)
 	/*@*/;
 
+/** \ingroup py_c
+ */
+PyObject * rpmHeaderToFile(PyObject * self, PyObject * args, PyObject * kwds)
+	/*@*/;
+
+/** \ingroup py_c
+ */
+PyObject * rpmHeaderToFD(PyObject * self, PyObject * args, PyObject * kwds)
+	/*@*/;
+
 /** \ingroup py_c
  */
 PyObject * rpmReadHeaders (FD_t fd)
 	/*@*/;
 
+/** \ingroup py_c
+ */
+PyObject * rpmWriteHeaders (PyObject * list, FD_t fd)
+	/*@*/;
+
 /** \ingroup py_c
  */
 PyObject * rhnLoad(PyObject * self, PyObject * args, PyObject * kwds)
