@@ -6,7 +6,8 @@
 
 #include <rpmio.h>
 #include <rpmcb.h>		/* XXX fnpyKey */
-#include <rpmlib.h>
+#include <rpmtypes.h>
+#include <rpmtag.h>
 
 #include "rpmdb-py.h"
 #include "rpmmi-py.h"
@@ -128,7 +129,7 @@ rpmdb_Match (rpmdbObject * s, PyObject * args, PyObject * kwds)
 	return NULL;
     }
 
-    return rpmmi_Wrap( rpmdbInitIterator(s->db, tag, key, len) );
+    return rpmmi_Wrap( rpmmiInit(s->db, tag, key, len) );
 }
 
 /*@}*/
@@ -152,13 +153,13 @@ rpmdb_length(rpmdbObject * s)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
-    rpmdbMatchIterator mi;
+    rpmmi mi;
     int count = 0;
 
-    mi = rpmdbInitIterator(s->db, RPMDBI_PACKAGES, NULL, 0);
-    while (rpmdbNextIterator(mi) != NULL)
+    mi = rpmmiInit(s->db, RPMDBI_PACKAGES, NULL, 0);
+    while (rpmmiNext(mi) != NULL)
 	count++;
-    mi = rpmdbFreeIterator(mi);
+    mi = rpmmiFree(mi);
 
     return count;
 }
@@ -174,7 +175,7 @@ rpmdb_subscript(rpmdbObject * s, PyObject * key)
     int offset;
     hdrObject * ho;
     Header h;
-    rpmdbMatchIterator mi;
+    rpmmi mi;
 
     if (!PyInt_Check(key)) {
 	PyErr_SetString(PyExc_TypeError, "integer expected");
@@ -183,15 +184,16 @@ rpmdb_subscript(rpmdbObject * s, PyObject * key)
 
     offset = (int) PyInt_AsLong(key);
 
-    mi = rpmdbInitIterator(s->db, RPMDBI_PACKAGES, &offset, sizeof(offset));
-    if (!(h = rpmdbNextIterator(mi))) {
-	mi = rpmdbFreeIterator(mi);
+    mi = rpmmiInit(s->db, RPMDBI_PACKAGES, &offset, sizeof(offset));
+    if (!(h = rpmmiNext(mi))) {
+	mi = rpmmiFree(mi);
 	PyErr_SetString(pyrpmError, "cannot read rpmdb entry");
 	return NULL;
     }
 
     ho = hdr_Wrap(h);
-    h = headerFree(h);
+    (void)headerFree(h);
+    h = NULL;
 
     return ho;
 }
@@ -200,7 +202,7 @@ rpmdb_subscript(rpmdbObject * s, PyObject * key)
  */
 /*@unchecked@*/ /*@observer@*/
 static PyMappingMethods rpmdb_as_mapping = {
-	(inquiry) rpmdb_length,		/* mp_length */
+	(lenfunc) rpmdb_length,		/* mp_length */
 	(binaryfunc) rpmdb_subscript,	/* mp_subscript */
 	(objobjargproc)0,		/* mp_ass_subscript */
 };
