@@ -5,8 +5,9 @@
 #include "system.h"
 
 #include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
-#include <rpmlib.h>
+#include <rpmiotypes.h>		/* XXX fnpyKey */
+#include <rpmtypes.h>
+#include <rpmtag.h>
 
 #include "header-py.h"
 #include "rpmds-py.h"
@@ -154,8 +155,8 @@ rpmds_iternext(rpmdsObject * s)
     if (rpmdsNext(s->ds) >= 0) {
 	const char * N = rpmdsN(s->ds);
 	const char * EVR = rpmdsEVR(s->ds);
-	int tagN = rpmdsTagN(s->ds);
-	int Flags = rpmdsFlags(s->ds);
+	rpmTag tagN = rpmdsTagN(s->ds);
+	rpmsenseFlags Flags = rpmdsFlags(s->ds);
 
 /*@-branchstate@*/
 	if (N != NULL) N = xstrdup(N);
@@ -192,7 +193,8 @@ rpmds_Next(rpmdsObject * s)
 
 /*@null@*/
 static PyObject *
-rpmds_Debug(/*@unused@*/ rpmdsObject * s, PyObject * args, PyObject * kwds)
+rpmds_Debug(/*@unused@*/ rpmdsObject * s, PyObject * args,
+		PyObject * kwds)
 	/*@globals _Py_NoneStruct @*/
 	/*@modifies _Py_NoneStruct @*/
 {
@@ -336,7 +338,8 @@ rpmds_Sort(rpmdsObject * s)
     rpmds nds = NULL;
 
     if (rpmdsMerge(&nds, s->ds) >= 0) {
-	s->ds = rpmdsFree(s->ds);
+	(void)rpmdsFree(s->ds);
+	s->ds = NULL;
 	s->ds = nds;
     }
     Py_INCREF(Py_None);
@@ -590,7 +593,8 @@ rpmds_dealloc(rpmdsObject * s)
 	/*@modifies s @*/
 {
     if (s) {
-	s->ds = rpmdsFree(s->ds);
+	(void)rpmdsFree(s->ds);
+	s->ds = NULL;
 	PyObject_Del(s);
     }
 }
@@ -648,7 +652,7 @@ rpmds_subscript(rpmdsObject * s, PyObject * key)
 }
 
 static PyMappingMethods rpmds_as_mapping = {
-        (inquiry) rpmds_length,		/* mp_length */
+        (lenfunc) rpmds_length,		/* mp_length */
         (binaryfunc) rpmds_subscript,	/* mp_subscript */
         (objobjargproc)0,		/* mp_ass_subscript */
 };
@@ -661,8 +665,8 @@ static int rpmds_init(rpmdsObject * s, PyObject *args, PyObject *kwds)
 {
     hdrObject * ho = NULL;
     PyObject * to = NULL;
-    int tagN = RPMTAG_REQUIRENAME;
-    int flags = 0;
+    rpmTag tagN = RPMTAG_REQUIRENAME;
+    rpmsenseFlags flags = 0;
     char * kwlist[] = {"header", "tag", "flags", NULL};
 
 if (_rpmds_debug < 0)
@@ -674,7 +678,7 @@ fprintf(stderr, "*** rpmds_init(%p,%p,%p)\n", s, args, kwds);
 
     if (to != NULL) {
 	tagN = tagNumFromPyObject(to);
-	if (tagN == -1) {
+	if (tagN == (rpmTag)0xffffffff) {
 	    PyErr_SetString(PyExc_KeyError, "unknown header tag");
 	    return -1;
 	}
@@ -692,7 +696,8 @@ static void rpmds_free(/*@only@*/ rpmdsObject * s)
 {
 if (_rpmds_debug)
 fprintf(stderr, "%p -- ds %p\n", s, s->ds);
-    s->ds = rpmdsFree(s->ds);
+    (void)rpmdsFree(s->ds);
+    s->ds = NULL;
 
     PyObject_Del((PyObject *)s);
 }
@@ -810,10 +815,10 @@ rpmdsObject *
 rpmds_Single(/*@unused@*/ PyObject * s, PyObject * args, PyObject * kwds)
 {
     PyObject * to = NULL;
-    int tagN = RPMTAG_PROVIDENAME;
+    rpmTag tagN = RPMTAG_PROVIDENAME;
     const char * N;
     const char * EVR = NULL;
-    int Flags = 0;
+    rpmsenseFlags Flags = 0;
     char * kwlist[] = {"to", "name", "evr", "flags", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "Os|si:Single", kwlist,
@@ -822,7 +827,7 @@ rpmds_Single(/*@unused@*/ PyObject * s, PyObject * args, PyObject * kwds)
 
     if (to != NULL) {
 	tagN = tagNumFromPyObject(to);
-	if (tagN == -1) {
+	if (tagN == (rpmTag)0xffffffff) {
 	    PyErr_SetString(PyExc_KeyError, "unknown header tag");
 	    return NULL;
 	}
@@ -847,7 +852,7 @@ hdr_dsFromHeader(PyObject * s, PyObject * args, PyObject * kwds)
 
     if (to != NULL) {
 	tagN = tagNumFromPyObject(to);
-	if (tagN == -1) {
+	if (tagN == (rpmTag)0xffffffff) {
 	    PyErr_SetString(PyExc_KeyError, "unknown header tag");
 	    return NULL;
 	}
