@@ -15,7 +15,7 @@
 
 #include "rpmfts-py.h"
 
-#include <rpmio.h>	/* XXX _free */
+#include <rpmiotypes.h>
 
 #include "debug.h"
 
@@ -63,7 +63,7 @@ rpmfts_debug (const char * msg, rpmftsObject * s)
     if (msg)
 	fprintf(stderr, "*** %s(%p)", msg, s);
     if (s)
-	fprintf(stderr, " %d %d ftsp %p fts %p\n", s->ob_refcnt, s->active, s->ftsp, s->fts);
+	fprintf(stderr, " %u %d ftsp %p fts %p\n", (unsigned) s->ob_refcnt, s->active, s->ftsp, s->fts);
 }
 
 static int
@@ -177,7 +177,8 @@ rpmfts_debug("rpmfts_step", s);
 /*@{*/
 /*@null@*/
 static PyObject *
-rpmfts_Debug(/*@unused@*/ rpmftsObject * s, PyObject * args, PyObject * kwds)
+rpmfts_Debug(/*@unused@*/ rpmftsObject * s, PyObject * args,
+		PyObject * kwds)
 	/*@globals _Py_NoneStruct @*/
 	/*@modifies _Py_NoneStruct @*/
 {
@@ -369,7 +370,7 @@ static void rpmfts_free(/*@only@*/ PyObject * s)
     _PyObject_GC_Del(s);
 }
 
-static PyObject * rpmfts_alloc(PyTypeObject * type, int nitems)
+static PyObject * rpmfts_alloc(PyTypeObject * type, Py_ssize_t nitems)
 	/*@*/
 {
     return PyType_GenericAlloc(type, nitems);
@@ -515,7 +516,8 @@ static int rpmfts_traverse(rpmftsObject * s, visitproc visit, void * arg)
     return 0;
 }
 
-static int rpmfts_print(rpmftsObject * s,  FILE * fp, /*@unused@*/ int flags)
+static int rpmfts_print(rpmftsObject * s,  FILE * fp,
+		/*@unused@*/ int flags)
 	/*@globals fileSystem @*/
 	/*@modifies fp, fileSystem @*/
 {
