@@ -6,8 +6,9 @@
 
 #include <rpmio.h>
 #include <rpmcb.h>		/* XXX fnpyKey */
+#include <rpmtypes.h>
+#include <rpmtag.h>
 #include <rpmdb.h>
-#include <rpmlib.h>
 
 #include "rpmmi-py.h"
 #include "header-py.h"
@@ -83,8 +84,8 @@ rpmmi_iternext(rpmmiObject * s)
 {
     Header h;
 
-    if (s->mi == NULL || (h = rpmdbNextIterator(s->mi)) == NULL) {
-	s->mi = rpmdbFreeIterator(s->mi);
+    if (s->mi == NULL || (h = rpmmiNext(s->mi)) == NULL) {
+	s->mi = rpmmiFree(s->mi);
 	return NULL;
     }
     return (PyObject *) hdr_Wrap(h);
@@ -124,7 +125,7 @@ rpmmi_Instance(rpmmiObject * s)
     int rc = 0;
 
     if (s->mi != NULL)
-	rc = rpmdbGetIteratorOffset(s->mi);
+	rc = rpmmiInstance(s->mi);
 
     return Py_BuildValue("i", rc);
 }
@@ -139,7 +140,7 @@ rpmmi_Count(rpmmiObject * s)
     int rc = 0;
 
     if (s->mi != NULL)
-	rc = rpmdbGetIteratorCount(s->mi);
+	rc = rpmmiCount(s->mi);
 
     return Py_BuildValue("i", rc);
 }
@@ -156,18 +157,18 @@ rpmmi_Pattern(rpmmiObject * s, PyObject * args, PyObject * kwds)
     int type;
     char * pattern;
     rpmTag tag;
-    char * kwlist[] = {"tag", "type", "patern", NULL};
+    char * kwlist[] = {"tag", "type", "pattern", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "Ois:Pattern", kwlist,
 	    &TagN, &type, &pattern))
 	return NULL;
 
-    if ((tag = tagNumFromPyObject (TagN)) == -1) {
+    if ((tag = tagNumFromPyObject (TagN)) == (rpmTag)-1) {
 	PyErr_SetString(PyExc_TypeError, "unknown tag type");
 	return NULL;
     }
 
-    rpmdbSetIteratorRE(s->mi, tag, type, pattern);
+    rpmmiAddPattern(s->mi, tag, type, pattern);
 
     Py_INCREF (Py_None);
     return Py_None;
@@ -202,7 +203,7 @@ static void rpmmi_dealloc(/*@only@*/ /*@null@*/ rpmmiObject * s)
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
     if (s) {
-	s->mi = rpmdbFreeIterator(s->mi);
+	s->mi = rpmmiFree(s->mi);
 	PyObject_Del(s);
     }
 }
@@ -275,7 +276,7 @@ PyTypeObject rpmmi_Type = {
 };
 /*@=fullinitblock@*/
 
-rpmmiObject * rpmmi_Wrap(rpmdbMatchIterator mi)
+rpmmiObject * rpmmi_Wrap(rpmmi mi)
 {
     rpmmiObject * mio = (rpmmiObject *) PyObject_New(rpmmiObject, &rpmmi_Type);
 
