@@ -5,14 +5,23 @@
 #include "system.h"
 
 #include <rpmio_internal.h>	/* XXX for fdSetOpen */
+#include <rpmcb.h>
 
 #define	_RPMPS_INTERNAL	/* XXX almost (but not quite) opaque. */
-#include <rpmcli.h>
 #include <rpmpgp.h>
-#include <rpmdb.h>
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
 #include <pkgio.h>		/* XXX headerCheck() */
+#include <rpmdb.h>
+
+#define	_RPMTS_INTERNAL		/* XXX expose rpmtsSetScriptFd */
 #include <rpmbuild.h>
 
+#include <rpmcli.h>
+#define	_RPMROLLBACK_INTERNAL	/* XXX IDTX et al */
+#include <rpmrollback.h>
+
 #include "header-py.h"
 #include "rpmds-py.h"	/* XXX for rpmdsNew */
 #include "rpmfi-py.h"	/* XXX for rpmfiNew */
@@ -25,6 +34,8 @@
 
 #include "debug.h"
 
+#define rpmtsfree() rpmioFreePoolItem()
+
 /*@unchecked@*/
 /*@-shadow@*/
 extern int _rpmts_debug;
@@ -164,10 +175,25 @@ struct rpmtsCallbackType_s {
     PyObject * data;
     rpmtsObject * tso;
     rpmdsObject * dso;
-    int pythonError;
     PyThreadState *_save;
 };
 
+/**
+ */
+/*@exits@*/
+static void rpmts_Die(PyObject *cb)
+	/*@*/
+{
+    PyObject * r = PyObject_Repr(cb);
+    char *pyfn = (r != NULL ? PyString_AsString(r) : "???");
+
+    if (PyErr_Occurred())
+	PyErr_Print();
+    rpmlog(RPMLOG_ERR, _("python callback %s failed, aborting!\n"), pyfn);
+    rpmdbCheckTerminate(1);
+    exit(EXIT_FAILURE);
+}
+
 /**
  */
 static int
@@ -182,7 +208,6 @@ if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_SolveCallback(%p,%p,%p) \"%s\"\n", ts, ds, data, rpmdsDNEVR(ds));
 
     if (cbInfo->tso == NULL) return res;
-    if (cbInfo->pythonError) return res;
     if (cbInfo->cb == Py_None) return res;
 
     PyEval_RestoreThread(cbInfo->_save);
@@ -195,7 +220,8 @@ fprintf(stderr, "*** rpmts_SolveCallback(%p,%p,%p) \"%s\"\n", ts, ds, data, rpmd
     Py_DECREF(args);
 
     if (!result) {
-	cbInfo->pythonError = 1;
+	rpmts_Die(cbInfo->cb);
+	/*@notreached@*/
     } else {
 	if (PyInt_Check(result))
 	    res = PyInt_AsLong(result);
@@ -212,8 +238,8 @@ fprintf(stderr, "*** rpmts_SolveCallback(%p,%p,%p) \"%s\"\n", ts, ds, data, rpmd
 /*@null@*/
 static void *
 rpmtsCallback(/*@unused@*/ const void * hd, const rpmCallbackType what,
-		         const uint64_t amount, const uint64_t total,
-	                 const void * pkgKey, rpmCallbackData data)
+		         const rpmuint64_t amount, const rpmuint64_t total,
+	                 fnpyKey pkgKey, rpmCallbackData data)
 	/*@globals _Py_NoneStruct @*/
 	/*@modifies _Py_NoneStruct @*/
 {
@@ -225,11 +251,8 @@ rpmtsCallback(/*@unused@*/ const void * hd, const rpmCallbackType what,
     PyObject * oh = NULL;
     const char * origin = NULL;
     PyObject * args, * result;
-    unsigned long oamount = amount;
-    unsigned long ototal = total;
     static FD_t fd;
 
-    if (cbInfo->pythonError) return NULL;
     if (cbInfo->cb == Py_None) return NULL;
 
     /* Synthesize a python object for callback (if necessary). */
@@ -260,24 +283,22 @@ rpmtsCallback(/*@unused@*/ const void * hd, const rpmCallbackType what,
 
     PyEval_RestoreThread(cbInfo->_save);
 
-    args = Py_BuildValue("(illOO)", what, oamount, ototal, pkgObj, cbInfo->data);
+    args = Py_BuildValue("(iLLOO)", what, amount, total, pkgObj, cbInfo->data);
     result = PyEval_CallObject(cbInfo->cb, args);
     Py_DECREF(args);
     Py_DECREF(pkgObj);
 
     if (!result) {
-	cbInfo->pythonError = 1;
-	cbInfo->_save = PyEval_SaveThread();
-	return NULL;
+	rpmts_Die(cbInfo->cb);
+	/*@notreached@*/
     }
 
     if (what == RPMCALLBACK_INST_OPEN_FILE) {
 	int fdno;
 
         if (!PyArg_Parse(result, "i", &fdno)) {
-	    cbInfo->pythonError = 1;
-	    cbInfo->_save = PyEval_SaveThread();
-	    return NULL;
+	    rpmts_Die(cbInfo->cb);
+	    /*@notreached@*/
 	}
 	Py_DECREF(result);
 	cbInfo->_save = PyEval_SaveThread();
@@ -299,7 +320,7 @@ fprintf(stderr, "\tFclose(%p)\n", fd);
 	Fclose (fd);
     } else {
 if (_rpmts_debug)
-fprintf(stderr, "\t%lu:%lu key %p\n", oamount, ototal, pkgKey);
+fprintf(stderr, "\t%llu:%llu key %p\n", (unsigned long long)amount, (unsigned long long)total, pkgKey);
     }
 
     Py_DECREF(result);
@@ -439,7 +460,7 @@ rpmts_AddErase(rpmtsObject * s, PyObject * args, PyObject * kwds)
 {
     PyObject * o;
     int count;
-    rpmdbMatchIterator mi;
+    rpmmi mi;
     char * kwlist[] = {"name", NULL};
 
 if (_rpmts_debug)
@@ -452,39 +473,39 @@ fprintf(stderr, "*** rpmts_AddErase(%p) ts %p\n", s, s->ts);
 	char * name = PyString_AsString(o);
 
 	mi = rpmtsInitIterator(s->ts, RPMDBI_LABEL, name, 0);
-	count = rpmdbGetIteratorCount(mi);
+	count = rpmmiCount(mi);
 	if (count <= 0) {
-	    mi = rpmdbFreeIterator(mi);
+	    mi = rpmmiFree(mi);
 	    PyErr_SetString(pyrpmError, "package not installed");
 	    return NULL;
 	} else { /* XXX: Note that we automatically choose to remove all matches */
 	    Header h;
-	    while ((h = rpmdbNextIterator(mi)) != NULL) {
-		unsigned int recOffset = rpmdbGetIteratorOffset(mi);
+	    while ((h = rpmmiNext(mi)) != NULL) {
+		unsigned int recOffset = rpmmiInstance(mi);
 		if (recOffset)
 		    rpmtsAddEraseElement(s->ts, h, recOffset);
 	    }
 	}
-	mi = rpmdbFreeIterator(mi);
+	mi = rpmmiFree(mi);
     } else
     if (PyInt_Check(o)) {
 	uint32_t instance = PyInt_AsLong(o);
 
 	mi = rpmtsInitIterator(s->ts, RPMDBI_PACKAGES, &instance, sizeof(instance));
 	if (instance == 0 || mi == NULL) {
-	    mi = rpmdbFreeIterator(mi);
+	    mi = rpmmiFree(mi);
 	    PyErr_SetString(pyrpmError, "package not installed");
 	    return NULL;
 	} else {
 	    Header h;
-	    while ((h = rpmdbNextIterator(mi)) != NULL) {
-		uint32_t recOffset = rpmdbGetIteratorOffset(mi);
+	    while ((h = rpmmiNext(mi)) != NULL) {
+		uint32_t recOffset = rpmmiInstance(mi);
 		if (recOffset)
 		    rpmtsAddEraseElement(s->ts, h, recOffset);
 		break;
 	    }
 	}
-	mi = rpmdbFreeIterator(mi);
+	mi = rpmmiFree(mi);
     }
 
     Py_INCREF(Py_None);
@@ -525,7 +546,6 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
 
     cbInfo.tso = s;
     cbInfo.dso = NULL;		/* XXX perhaps persistent? */
-    cbInfo.pythonError = 0;
     cbInfo._save = PyEval_SaveThread();
 
     xx = rpmtsCheck(s->ts);
@@ -920,7 +940,8 @@ fprintf(stderr, "*** rpmts_HdrFromFdno(%p) ts %p rc %d\n", s, s->ts, rpmrc);
     case RPMRC_OK:
 	if (h)
 	    result = Py_BuildValue("N", hdr_Wrap(h));
-	h = headerFree(h);	/* XXX ref held by result */
+	(void)headerFree(h);	/* XXX ref held by result */
+	h = NULL;
 	break;
 
     case RPMRC_NOKEY:
@@ -978,7 +999,7 @@ fprintf(stderr, "*** rpmts_HdrCheck(%p) ts %p\n", s, s->ts);
 
     dig = pgpDigNew(rpmtsVSFlags(s->ts));
     rpmrc = headerCheck(dig, uh, uc, &msg);
-    dig = pgpDigFree(dig);
+    dig = pgpDigFree(dig, "rpmts_HdrCheck");
 
     switch (rpmrc) {
     case RPMRC_OK:
@@ -1126,7 +1147,7 @@ fprintf(stderr, "*** rpmts_PgpImportPubkey(%p) ts %p\n", s, s->ts);
     pkt = (unsigned char *) PyString_AsString(blob);
     pktlen = PyString_Size(blob);
 
-    rc = rpmtsImportPubkey(s->ts, pkt, pktlen);
+    rc = rpmcliImportPubkey(s->ts, pkt, pktlen);
 
     return Py_BuildValue("i", rc);
 }
@@ -1182,7 +1203,7 @@ rpmts_SetProbFilter(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@modifies s @*/
 {
     rpmprobFilterFlags ignoreSet = 0;
-    rpmprobFilterFlags oignoreSet;
+    rpmprobFilterFlags oignoreSet = 0;
     char * kwlist[] = {"ignoreSet", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:ProbFilter", kwlist,
@@ -1232,7 +1253,6 @@ rpmts_Run(rpmtsObject * s, PyObject * args, PyObject * kwds)
 
     cbInfo.tso = s;
     cbInfo.dso = NULL;
-    cbInfo.pythonError = 0;
     cbInfo._save = PyEval_SaveThread();
 
     if (cbInfo.cb != NULL) {
@@ -1243,16 +1263,6 @@ rpmts_Run(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	(void) rpmtsSetNotifyCallback(s->ts, rpmtsCallback, (void *) &cbInfo);
     }
 
-    /* Initialize security context patterns (if not already done). */
-    if (rpmtsSELinuxEnabled(s->ts)
-     && !(rpmtsFlags(s->ts) & RPMTRANS_FLAG_NOCONTEXTS))
-    {
-	const char *fn = rpmGetPath("%{?_install_file_context_path}", NULL);
-	if (fn != NULL && *fn != '\0')
-	    rc = matchpathcon_init(fn);
-	    fn = _free(fn);
-    }
-
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Run(%p) ts %p ignore %x\n", s, s->ts, s->ignoreSet);
 
@@ -1264,11 +1274,6 @@ fprintf(stderr, "*** rpmts_Run(%p) ts %p ignore %x\n", s, s->ts, s->ignoreSet);
 
     PyEval_RestoreThread(cbInfo._save);
 
-    if (cbInfo.pythonError) {
-	ps = rpmpsFree(ps);
-	return NULL;
-    }
-
     if (rc < 0) {
 	list = PyList_New(0);
 	return list;
@@ -1281,11 +1286,10 @@ fprintf(stderr, "*** rpmts_Run(%p) ts %p ignore %x\n", s, s->ts, s->ignoreSet);
     psi = rpmpsInitIterator(ps);
     while (rpmpsNextIterator(psi) >= 0) {
 	rpmProblem p = rpmpsProblem(psi);
-	unsigned long ulong1 = p->ulong1;
 	PyObject * prob = Py_BuildValue("s(isN)", rpmProblemString(p),
 			     rpmProblemGetType(p),
-			     p->str1,
-			     PyLong_FromLongLong(ulong1));
+			     rpmProblemGetStr(p),
+			     PyLong_FromLongLong(rpmProblemGetDiskNeed(p)));
 	PyList_Append(list, prob);
 	Py_DECREF(prob);
     }
@@ -1520,7 +1524,8 @@ static void rpmts_dealloc(/*@only@*/ rpmtsObject * s)
 
 if (_rpmts_debug)
 fprintf(stderr, "%p -- ts %p db %p\n", s, s->ts, rpmtsGetRdb(s->ts));
-    s->ts = rpmtsFree(s->ts);
+    (void)rpmtsFree(s->ts);
+    s->ts = NULL;
 
     if (s->scriptFd) Fclose(s->scriptFd);
     /* this will free the keyList, and decrement the ref count of all
@@ -1567,28 +1572,7 @@ static int rpmts_init(rpmtsObject * s, PyObject *args, PyObject *kwds)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
-    char * rootDir = "/";
-    int vsflags = rpmExpandNumeric("%{?_vsflags}");
-    char * kwlist[] = {"rootdir", "vsflags", 0};
-
-if (_rpmts_debug < 0)
-fprintf(stderr, "*** rpmts_init(%p,%p,%p)\n", s, args, kwds);
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|si:rpmts_init", kwlist,
-	    &rootDir, &vsflags))
-	return -1;
-
-    s->ts = rpmtsCreate();
-    /* XXX: Why is there no rpmts_SetRootDir() ? */
-    (void) rpmtsSetRootDir(s->ts, rootDir);
-    /* XXX: make this use common code with rpmts_SetVSFlags() to check the
-     *      python objects */
-    (void) rpmtsSetVSFlags(s->ts, vsflags);
-    s->keyList = PyList_New(0);
-    s->scriptFd = NULL;
-    s->tsi = NULL;
-    s->tsiFilter = 0;
-
+    /* nothing to do atm... */
     return 0;
 }
 
@@ -1599,7 +1583,8 @@ static void rpmts_free(/*@only@*/ rpmtsObject * s)
 {
 if (_rpmts_debug)
 fprintf(stderr, "%p -- ts %p db %p\n", s, s->ts, rpmtsGetRdb(s->ts));
-    s->ts = rpmtsFree(s->ts);
+    (void)rpmtsFree(s->ts);
+    s->ts = NULL;
 
     if (s->scriptFd)
 	Fclose(s->scriptFd);
@@ -1631,11 +1616,28 @@ static PyObject * rpmts_new(PyTypeObject * subtype, PyObject *args, PyObject *kw
 {
     rpmtsObject * s = (void *) PyObject_New(rpmtsObject, subtype);
 
-    /* Perform additional initialization. */
-    if (rpmts_init(s, args, kwds) < 0) {
-	rpmts_free(s);
+    char * rootDir = "/";
+    int vsflags = rpmExpandNumeric("%{?_vsflags}");
+    char * kwlist[] = {"rootdir", "vsflags", 0};
+
+if (_rpmts_debug < 0)
+fprintf(stderr, "*** rpmts_init(%p,%p,%p)\n", s, args, kwds);
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|si:rpmts_init", kwlist,
+	    &rootDir, &vsflags))
 	return NULL;
-    }
+
+    s->ts = rpmtsCreate();
+    /* XXX: Why is there no rpmts_SetRootDir() ? */
+    (void) rpmtsSetRootDir(s->ts, rootDir);
+    /* XXX: make this use common code with rpmts_SetVSFlags() to check the
+     *      python objects */
+    (void) rpmtsSetVSFlags(s->ts, vsflags);
+    s->keyList = PyList_New(0);
+    s->ignoreSet = 0;
+    s->scriptFd = NULL;
+    s->tsi = NULL;
+    s->tsiFilter = 0;
 
 if (_rpmts_debug)
 fprintf(stderr, "%p ++ ts %p db %p\n", s, s->ts, rpmtsGetRdb(s->ts));
@@ -1701,34 +1703,9 @@ PyTypeObject rpmts_Type = {
 
 /**
  */
-/* XXX: This should use the same code as rpmts_init */
-rpmtsObject *
-rpmts_Create(/*@unused@*/ PyObject * self, PyObject * args, PyObject * kwds)
+PyObject *
+rpmts_Create(/*@unused@*/ PyObject * s, PyObject * args,
+		PyObject * kwds)
 {
-    rpmtsObject * o;
-    char * rootDir = "/";
-    int vsflags = rpmExpandNumeric("%{?_vsflags}");
-    char * kwlist[] = {"rootdir", "vsflags", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|si:Create", kwlist,
-	    &rootDir, &vsflags))
-	return NULL;
-
-    o = (void *) PyObject_New(rpmtsObject, &rpmts_Type);
-
-    o->ts = rpmtsCreate();
-    /* XXX: Why is there no rpmts_SetRootDir() ? */
-    (void) rpmtsSetRootDir(o->ts, rootDir);
-    /* XXX: make this use common code with rpmts_SetVSFlags() to check the
-     *      python objects */
-    (void) rpmtsSetVSFlags(o->ts, vsflags);
-
-    o->keyList = PyList_New(0);
-    o->scriptFd = NULL;
-    o->tsi = NULL;
-    o->tsiFilter = 0;
-
-if (_rpmts_debug)
-fprintf(stderr, "%p ++ ts %p db %p\n", o, o->ts, rpmtsGetRdb(o->ts));
-    return o;
+    return PyObject_Call((PyObject *) &rpmts_Type, args, kwds);
 }
