@@ -1,32 +1,31 @@
 /* File automatically generate by ./rpmh2tbl
  * Olivier Thauvin <thauvin at aerov.jussieu.fr>
  * Parsed files:
- * ../lib/cpio.h
+ * ../lib/filetriggers.h
  * ../lib/fs.h
  * ../lib/fsm.h
  * ../lib/genpgp.h
  * ../lib/manifest.h
  * ../lib/misc.h
  * ../lib/psm.h
+ * ../lib/rpm46compat.h
  * ../lib/rpm4compat.h
  * ../lib/rpmal.h
  * ../lib/rpmcli.h
- * ../lib/rpmdpkg.h
  * ../lib/rpmds.h
- * ../lib/rpmevr.h
  * ../lib/rpmfc.h
  * ../lib/rpmfi.h
  * ../lib/rpmgi.h
  * ../lib/rpmlib.h
  * ../lib/rpmlock.h
  * ../lib/rpmluaext.h
- * ../lib/rpmns.h
  * ../lib/rpmps.h
+ * ../lib/rpmrc.h
+ * ../lib/rpmrollback.h
  * ../lib/rpmsx.h
  * ../lib/rpmte.h
  * ../lib/rpmts.h
  * ../lib/rpmversion.h
- * ../lib/tar.h
  * ../build/buildio.h
  * ../build/rpmbuild.h
  * ../build/rpmspec.h
@@ -36,52 +35,82 @@
  * ../rpmdb/legacy.h
  * ../rpmdb/pkgio.h
  * ../rpmdb/rpmdb.h
+ * ../rpmdb/rpmdpkg.h
+ * ../rpmdb/rpmevr.h
+ * ../rpmdb/rpmns.h
  * ../rpmdb/rpmtag.h
+ * ../rpmdb/rpmtd.h
+ * ../rpmdb/rpmtypes.h
  * ../rpmdb/rpmwf.h
  * ../rpmdb/signature.h
- * ../rpmio/LzmaDecode.h
  * ../rpmio/argv.h
+ * ../rpmio/ar.h
+ * ../rpmio/cpio.h
+ * ../rpmio/crc.h
  * ../rpmio/envvar.h
+ * ../rpmio/fnmatch.h
  * ../rpmio/fts.h
  * ../rpmio/gengpg.h
+ * ../rpmio/glob.h
  * ../rpmio/html-parse.h
+ * ../rpmio/iosm.h
  * ../rpmio/md2.h
  * ../rpmio/md4.h
  * ../rpmio/mire.h
+ * ../rpmio/poptIO.h
  * ../rpmio/rmd128.h
  * ../rpmio/rmd160.h
  * ../rpmio/rmd256.h
  * ../rpmio/rmd320.h
  * ../rpmio/rpmbc.h
+ * ../rpmio/rpmbz.h
  * ../rpmio/rpmcb.h
+ * ../rpmio/rpmcpio.h
  * ../rpmio/rpmdav.h
+ * ../rpmio/rpmficl.h
  * ../rpmio/rpmgc.h
  * ../rpmio/rpmhash.h
  * ../rpmio/rpmhook.h
- * ../rpmio/rpmio-stub.h
+ * ../rpmio/rpmiob.h
  * ../rpmio/rpmio.h
  * ../rpmio/rpmio_internal.h
+ * ../rpmio/rpmio-stub.h
+ * ../rpmio/rpmiotypes.h
+ * ../rpmio/rpmjs.h
+ * ../rpmio/rpmkeyring.h
+ * ../rpmio/rpmku.h
  * ../rpmio/rpmlog.h
  * ../rpmio/rpmlua.h
  * ../rpmio/rpmmacro.h
  * ../rpmio/rpmmg.h
  * ../rpmio/rpmnss.h
+ * ../rpmio/rpmperl.h
  * ../rpmio/rpmpgp.h
+ * ../rpmio/rpmpython.h
+ * ../rpmio/rpmruby.h
  * ../rpmio/rpmsq.h
  * ../rpmio/rpmssl.h
  * ../rpmio/rpmsw.h
+ * ../rpmio/rpmsyck.h
+ * ../rpmio/rpmtar.h
+ * ../rpmio/rpmtcl.h
  * ../rpmio/rpmurl.h
+ * ../rpmio/rpmuuid.h
  * ../rpmio/rpmxar.h
+ * ../rpmio/rpmz.h
+ * ../rpmio/rpmzlog.h
+ * ../rpmio/rpmzq.h
  * ../rpmio/salsa10.h
  * ../rpmio/salsa20.h
  * ../rpmio/sha224.h
- * ../rpmio/stringbuf.h
+ * ../rpmio/tar.h
  * ../rpmio/tiger.h
  * ../rpmio/ugid.h
+ * ../rpmio/yarn.h
+ * ../misc/asprintf.h
  * ../misc/err.h
  * ../misc/error.h
- * ../misc/fnmatch.h
- * ../misc/glob.h
+ * ../misc/tempname.h
  */
 
 #include "system.h"
@@ -92,6 +121,7 @@
 /* From rpmcli.h
  * prefix tag: VERIFY_
  * table: rpmVerifyFlags */
+#include "rpmcli.h"
 static const struct rpmconstant_s rpmVerifyFlagsctbl[] = {
 #ifdef H_RPMCLI
 	{ "VERIFY_DEFAULT", VERIFY_DEFAULT }, /*!< */
@@ -127,6 +157,7 @@ const struct rpmconstant_s * rpmVerifyFlagsctable = rpmVerifyFlagsctbl;
 /* From rpmcli.h
  * prefix tag: RPMSIGN_
  * table: rpmSignFlags */
+#include "rpmcli.h"
 static const struct rpmconstant_s rpmSignFlagsctbl[] = {
 #ifdef H_RPMCLI
 	{ "RPMSIGN_NONE", RPMSIGN_NONE }, 
@@ -141,41 +172,10 @@ static const struct rpmconstant_s rpmSignFlagsctbl[] = {
 const struct rpmconstant_s * rpmSignFlagsctable = rpmSignFlagsctbl;
 
 
-/* From rpmlib.h
- * prefix tag: RPM_MACHTABLE_
- * table: rpm_machtable */
-static const struct rpmconstant_s rpm_machtablectbl[] = {
-#ifdef H_RPMLIB
-	{ "RPM_MACHTABLE_INSTARCH", RPM_MACHTABLE_INSTARCH }, /*!< Install platform architecture. */
-	{ "RPM_MACHTABLE_INSTOS", RPM_MACHTABLE_INSTOS }, /*!< Install platform operating system. */
-	{ "RPM_MACHTABLE_BUILDARCH", RPM_MACHTABLE_BUILDARCH }, /*!< Build platform architecture. */
-	{ "RPM_MACHTABLE_BUILDOS", RPM_MACHTABLE_BUILDOS }, /*!< Build platform operating system. */
-	{ "RPM_MACHTABLE_COUNT", RPM_MACHTABLE_COUNT }, /*!< No. of arch/os tables. */
-#endif /* H_RPMLIB */
-	{ NULL, 0 } /* NULL terminated (rpm_machtable) */
-};
-const struct rpmconstant_s * rpm_machtablectable = rpm_machtablectbl;
-
-
-/* From rpmlib.h
- * prefix tag: RPMRC_
- * table: rpmRC */
-static const struct rpmconstant_s rpmRCctbl[] = {
-#ifdef H_RPMLIB
-	{ "RPMRC_OK", RPMRC_OK }, /*!< Generic success code */
-	{ "RPMRC_NOTFOUND", RPMRC_NOTFOUND }, /*!< Generic not found code. */
-	{ "RPMRC_FAIL", RPMRC_FAIL }, /*!< Generic failure code. */
-	{ "RPMRC_NOTTRUSTED", RPMRC_NOTTRUSTED }, /*!< Signature is OK, but key is not trusted. */
-	{ "RPMRC_NOKEY", RPMRC_NOKEY }, /*!< Public key is unavailable. */
-#endif /* H_RPMLIB */
-	{ NULL, 0 } /* NULL terminated (rpmRC) */
-};
-const struct rpmconstant_s * rpmRCctable = rpmRCctbl;
-
-
 /* From rpmps.h
  * prefix tag: RPMPROB_
  * table: rpmProblemType */
+#include "rpmps.h"
 static const struct rpmconstant_s rpmProblemTypectbl[] = {
 #ifdef H_RPMPS
 	{ "RPMPROB_FILTER_NONE", RPMPROB_FILTER_NONE }, 
@@ -212,6 +212,7 @@ const struct rpmconstant_s * rpmProblemTypectable = rpmProblemTypectbl;
 /* From rpmte.h
  * prefix tag: TR_
  * table: rpmElementType */
+#include "rpmte.h"
 static const struct rpmconstant_s rpmElementTypectbl[] = {
 #ifdef H_RPMTE
 	{ "TR_ADDED", TR_ADDED }, 
@@ -225,6 +226,7 @@ const struct rpmconstant_s * rpmElementTypectable = rpmElementTypectbl;
 /* From rpmts.h
  * prefix tag: RPMTS_OP_
  * table: rpmtsOpX */
+#include "rpmts.h"
 static const struct rpmconstant_s rpmtsOpXctbl[] = {
 #ifdef H_RPMTS
 	{ "RPMTS_OP_TOTAL", RPMTS_OP_TOTAL }, 
@@ -258,6 +260,7 @@ const struct rpmconstant_s * rpmtsOpXctable = rpmtsOpXctbl;
 /* From rpmts.h
  * prefix tag: RPMTRANS_TYPE_
  * table: rpmtsType */
+#include "rpmts.h"
 static const struct rpmconstant_s rpmtsTypectbl[] = {
 #ifdef H_RPMTS
 	{ "RPMTRANS_TYPE_NORMAL", RPMTRANS_TYPE_NORMAL }, 
@@ -272,6 +275,7 @@ const struct rpmconstant_s * rpmtsTypectable = rpmtsTypectbl;
 /* From rpmbuild.h
  * prefix tag: RPMBUILD_
  * table: rpmBuildFlags */
+#include "rpmbuild.h"
 static const struct rpmconstant_s rpmBuildFlagsctbl[] = {
 #ifdef _H_RPMBUILD_
 	{ "RPMBUILD_NONE", RPMBUILD_NONE }, 
@@ -298,6 +302,7 @@ const struct rpmconstant_s * rpmBuildFlagsctable = rpmBuildFlagsctbl;
 /* From rpmbuild.h
  * prefix tag: PART_
  * table: rpmParseState */
+#include "rpmbuild.h"
 static const struct rpmconstant_s rpmParseStatectbl[] = {
 #ifdef _H_RPMBUILD_
 	{ "PART_SUBNAME", PART_SUBNAME }, 
@@ -337,6 +342,7 @@ const struct rpmconstant_s * rpmParseStatectable = rpmParseStatectbl;
 /* From rpmtag.h
  * prefix tag: RPM_
  * table: rpmTagType */
+#include "rpmtag.h"
 static const struct rpmconstant_s rpmTagTypectbl[] = {
 #ifdef H_RPMTAG
 	{ "RPM_UINT8", RPM_UINT8_TYPE }, 
@@ -373,6 +379,7 @@ const struct rpmconstant_s * rpmTagTypectable = rpmTagTypectbl;
 /* From rpmtag.h
  * prefix tag: RPMTAG_
  * table: rpmTag */
+#include "rpmtag.h"
 static const struct rpmconstant_s rpmTagctbl[] = {
 #ifdef H_RPMTAG
 	{ "RPMTAG_HEADERIMAGE", RPMTAG_HEADERIMAGE }, /*!< internal Current image. */
@@ -408,7 +415,7 @@ static const struct rpmconstant_s rpmTagctbl[] = {
 	{ "RPMTAG_DESCRIPTION", RPMTAG_DESCRIPTION }, /* s{} */
 	{ "RPMTAG_BUILDTIME", RPMTAG_BUILDTIME }, /* i */
 	{ "RPMTAG_BUILDHOST", RPMTAG_BUILDHOST }, /* s */
-	{ "RPMTAG_INSTALLTIME", RPMTAG_INSTALLTIME }, /* i */
+	{ "RPMTAG_INSTALLTIME", RPMTAG_INSTALLTIME }, /* i[] */
 	{ "RPMTAG_SIZE", RPMTAG_SIZE }, /* i */
 	{ "RPMTAG_DISTRIBUTION", RPMTAG_DISTRIBUTION }, /* s */
 	{ "RPMTAG_VENDOR", RPMTAG_VENDOR }, /* s */
@@ -527,8 +534,8 @@ static const struct rpmconstant_s rpmTagctbl[] = {
 	{ "RPMTAG_PAYLOADCOMPRESSOR", RPMTAG_PAYLOADCOMPRESSOR }, /* s */
 	{ "RPMTAG_PAYLOADFLAGS", RPMTAG_PAYLOADFLAGS }, /* s */
 	{ "RPMTAG_INSTALLCOLOR", RPMTAG_INSTALLCOLOR }, /* i transaction color when installed */
-	{ "RPMTAG_INSTALLTID", RPMTAG_INSTALLTID }, /* i */
-	{ "RPMTAG_REMOVETID", RPMTAG_REMOVETID }, /* i */
+	{ "RPMTAG_INSTALLTID", RPMTAG_INSTALLTID }, /* i[] */
+	{ "RPMTAG_REMOVETID", RPMTAG_REMOVETID }, /* i[] */
 	{ "RPMTAG_SHA1RHN", RPMTAG_SHA1RHN }, /* internal - obsolete */
 	{ "RPMTAG_RHNPLATFORM", RPMTAG_RHNPLATFORM }, /* s deprecated */
 	{ "RPMTAG_PLATFORM", RPMTAG_PLATFORM }, /* s */
@@ -604,12 +611,39 @@ static const struct rpmconstant_s rpmTagctbl[] = {
 	{ "RPMTAG_RPMLIBVERSION", RPMTAG_RPMLIBVERSION }, /* i */
 	{ "RPMTAG_RPMLIBTIMESTAMP", RPMTAG_RPMLIBTIMESTAMP }, /* i */
 	{ "RPMTAG_RPMLIBVENDOR", RPMTAG_RPMLIBVENDOR }, /* i */
-	{ "RPMTAG_CLASS", RPMTAG_CLASS }, /* s */
+	{ "RPMTAG_CLASS", RPMTAG_CLASS }, /* s arbitrary */
 	{ "RPMTAG_TRACK", RPMTAG_TRACK }, /* s internal arbitrary */
 	{ "RPMTAG_TRACKPROG", RPMTAG_TRACKPROG }, /* s internal arbitrary */
 	{ "RPMTAG_SANITYCHECK", RPMTAG_SANITYCHECK }, /* s */
 	{ "RPMTAG_SANITYCHECKPROG", RPMTAG_SANITYCHECKPROG }, /* s */
+	{ "RPMTAG_FILESTAT", RPMTAG_FILESTAT }, /* s[] stat(2) from metadata extension*/
+	{ "RPMTAG_STAT", RPMTAG_STAT }, /* s[] stat(2) from disk extension */
+	{ "RPMTAG_ORIGINTID", RPMTAG_ORIGINTID }, /* i[] */
+	{ "RPMTAG_ORIGINTIME", RPMTAG_ORIGINTIME }, /* i[] */
+	{ "RPMTAG_HEADERSTARTOFF", RPMTAG_HEADERSTARTOFF }, /* l */
+	{ "RPMTAG_HEADERENDOFF", RPMTAG_HEADERENDOFF }, /* l */
+	{ "RPMTAG_PACKAGETIME", RPMTAG_PACKAGETIME }, /* l */
+	{ "RPMTAG_PACKAGESIZE", RPMTAG_PACKAGESIZE }, /* l */
+	{ "RPMTAG_PACKAGEDIGEST", RPMTAG_PACKAGEDIGEST }, /* s */
+	{ "RPMTAG_PACKAGESTAT", RPMTAG_PACKAGESTAT }, /* x */
+	{ "RPMTAG_PACKAGEBASEURL", RPMTAG_PACKAGEBASEURL }, /* s */
+	{ "RPMTAG_DISTEPOCH", RPMTAG_DISTEPOCH }, /* s */
+	{ "RPMTAG_D", RPMTAG_D }, /* s */
+	{ "RPMTAG_FILEDIGESTALGO", RPMTAG_FILEDIGESTALGO }, /* i file checksum algorithm */
 	{ "RPMTAG_FIRSTFREE_TAG", RPMTAG_FIRSTFREE_TAG }, /*!< internal */
+	{ "RPMTAG_PACKAGETRANSFLAGS", RPMTAG_PACKAGETRANSFLAGS }, /* s[] arbitrary */
+	{ "RPMTAG_PACKAGEDEPFLAGS", RPMTAG_PACKAGEDEPFLAGS }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDPREPPROG", RPMTAG_BUILDPREPPROG }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDPREP", RPMTAG_BUILDPREP }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDBUILDPROG", RPMTAG_BUILDBUILDPROG }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDBUILD", RPMTAG_BUILDBUILD }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDINSTALLPROG", RPMTAG_BUILDINSTALLPROG }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDINSTALL", RPMTAG_BUILDINSTALL }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDCHECKPROG", RPMTAG_BUILDCHECKPROG }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDCHECK", RPMTAG_BUILDCHECK }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDCLEANPROG", RPMTAG_BUILDCLEANPROG }, /* s[] arbitrary */
+	{ "RPMTAG_BUILDCLEAN", RPMTAG_BUILDCLEAN }, /* s[] arbitrary */
+	{ "RPMTAG_LASTARBITRARY_TAG", RPMTAG_LASTARBITRARY_TAG }, /*!< internal */
 	{ "RPMTAG_EXTERNAL_TAG", RPMTAG_EXTERNAL_TAG }, 
 #endif /* H_RPMTAG */
 	{ NULL, 0 } /* NULL terminated (rpmTag) */
@@ -620,6 +654,7 @@ const struct rpmconstant_s * rpmTagctable = rpmTagctbl;
 /* From rpmlog.h
  * prefix tag: RPMLOG_
  * table: rpmlog */
+#include "rpmlog.h"
 static const struct rpmconstant_s rpmlogctbl[] = {
 #ifdef H_RPMLOG
 	{ "RPMLOG_EMERG", RPMLOG_EMERG }, /*!< system is unusable */
@@ -671,6 +706,7 @@ const struct rpmconstant_s * rpmlogctable = rpmlogctbl;
 /* From rpmmacro.h
  * prefix tag: RMIL_
  * table: rpmMacrosContext */
+#include "rpmmacro.h"
 static const struct rpmconstant_s rpmMacrosContextctbl[] = {
 #ifdef _H_MACRO_
 	{ "RMIL_DEFAULT", RMIL_DEFAULT }, 
@@ -692,12 +728,10 @@ static const struct rpmconstantlist_s rpmconstanttp[] = {
 	{ (void *)rpmMacrosContextctbl, "rpmmacroscontext", "RMIL_" },
 	{ (void *)rpmParseStatectbl, "rpmparsestate", "PART_" },
 	{ (void *)rpmProblemTypectbl, "rpmproblemtype", "RPMPROB_" },
-	{ (void *)rpmRCctbl, "rpmrc", "RPMRC_" },
 	{ (void *)rpmSignFlagsctbl, "rpmsignflags", "RPMSIGN_" },
 	{ (void *)rpmTagctbl, "rpmtag", "RPMTAG_" },
 	{ (void *)rpmTagTypectbl, "rpmtagtype", "RPM_" },
 	{ (void *)rpmVerifyFlagsctbl, "rpmverifyflags", "VERIFY_" },
-	{ (void *)rpm_machtablectbl, "rpm_machtable", "RPM_MACHTABLE_" },
 	{ (void *)rpmlogctbl, "rpmlog", "RPMLOG_" },
 	{ (void *)rpmtsOpXctbl, "rpmtsopx", "RPMTS_OP_" },
 	{ (void *)rpmtsTypectbl, "rpmtstype", "RPMTRANS_TYPE_" },
