@@ -12,7 +12,6 @@ static int _debug = 1;	/* XXX if < 0 debugging, > 0 unusual error returns */
 #include <sys/ipc.h>
 #endif
 
-#include <rpmio.h>
 #include <rpmlog.h>
 #include <rpmmacro.h>
 #include <rpmurl.h>	/* XXX urlPath proto */
@@ -153,7 +152,8 @@ static const char * dbiModeFlags =
 
 
 /*@-globuse -mustmod @*/	/* FIX: rpmError not annotated yet. */
-static int cvtdberr(/*@unused@*/ dbiIndex dbi, const char * msg, int error, int printit)
+static int cvtdberr(/*@unused@*/ dbiIndex dbi, const char * msg,
+		int error, int printit)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
@@ -185,7 +185,7 @@ static const char * mapTagName(rpmdb rpmdb, dbiIndex dbi)
 	/*@*/
 {
     tagStore_t dbiTags = rpmdb->db_tags;
-    int dbix = 0;
+    size_t dbix = 0;
 
     if (dbiTags != NULL)
     while (dbix < rpmdb->db_ndbi) {
@@ -257,8 +257,9 @@ static int db3_fsync_disable(/*@unused@*/ int fd)
  * @param flags		0 or DB_MUTEX_PROCESS_ONLY
  * @return		
  */
-static int db3is_alive(/*@unused@*/ DB_ENV *dbenv, pid_t pid, /*@unused@*/ db_threadid_t tid,
-		uint32_t flags)
+static int db3is_alive(/*@unused@*/ DB_ENV *dbenv, pid_t pid,
+		/*@unused@*/ db_threadid_t tid,
+		rpmuint32_t flags)
 	/*@*/
 {
     int is_alive = 1;	/* assume all processes are alive */
@@ -280,8 +281,8 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 		/*@unused@*/ /*@null@*/ const char * dbsubfile,
 		/*@out@*/ DB_ENV ** dbenvp)
 	/*@globals rpmGlobalMacroContext, h_errno,
-		fileSystem @*/
-	/*@modifies dbi, *dbenvp, fileSystem @*/
+		fileSystem, internalState @*/
+	/*@modifies dbi, *dbenvp, fileSystem, internalState @*/
 {
     static int oneshot = 0;
     rpmdb rpmdb = dbi->dbi_rpmdb;
@@ -504,7 +505,7 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 	xx = dbenv->set_tx_max(dbenv, dbi->dbi_tx_max);
 	xx = cvtdberr(dbi, "dbenv->set_tx_max", xx, _debug);
     }
-/* XXX dbenv->txn_checkpoint */        
+/* XXX dbenv->txn_checkpoint */
 /* XXX dbenv->txn_recover */
 /* XXX dbenv->txn_stat */
  /* 4.1 dbenv->set_timeout(???) */
@@ -764,6 +765,8 @@ static int db3cget(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 	rc = dbcursor->get(dbcursor, key, data, flags);
 	/* XXX DB_NOTFOUND can be returned */
 	_printit = (rc == DB_NOTFOUND ? 0 : _debug);
+	/* XXX Permit DB_BUFFER_SMALL to be returned (more restrictive?) */
+	_printit = (rc == DB_BUFFER_SMALL ? 0 : _printit);
 	rc = cvtdberr(dbi, "dbcursor->get", rc, _printit);
 #else
 	/* XXX db3 does DB_FIRST on uninitialized cursor */
@@ -838,7 +841,7 @@ static int db3byteswapped(dbiIndex dbi)	/*@*/
     int rc = 0;
 
     if (db != NULL) {
-#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH == 11) \
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH >= 11) \
  || (DB_VERSION_MAJOR == 4)
 	int isswapped = 0;
 	rc = db->get_byteswapped(db, &isswapped);
@@ -932,8 +935,8 @@ assert(db != NULL);
 /*@-moduncon@*/ /* FIX: annotate db3 methods */
 static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	/*@globals rpmGlobalMacroContext, h_errno,
-		fileSystem @*/
-	/*@modifies dbi, fileSystem @*/
+		fileSystem, internalState @*/
+	/*@modifies dbi, fileSystem, internalState @*/
 {
     rpmdb rpmdb = dbi->dbi_rpmdb;
     const char * urlfn = NULL;
@@ -1061,7 +1064,7 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 			(dbhome ? dbhome : ""),
 			(dbfile ? dbfile : dbiBN));
 
-	        /*
+		/*
 		 * The DB handle may not be accessed again after
 		 * DB->verify is called, regardless of its return.
 		 */
@@ -1084,6 +1087,13 @@ exit:
 }
 /*@=moduncon@*/
 
+/**
+ * Return handle for an index database.
+ * @param rpmdb         rpm database
+ * @param rpmtag        rpm tag
+ * @retval *dbip	index database handle
+ * @return              0 on success
+ */
 static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
@@ -1109,7 +1119,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
     DB_TXN * txnid = NULL;
 #endif
     DBTYPE dbi_type = DB_UNKNOWN;
-    uint32_t oflags;
+    rpmuint32_t oflags;
     int _printit;
 
     if (dbip)
@@ -1295,14 +1305,20 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
     if (dbi->dbi_use_dbenv) {
 	/*@-mods@*/
 	if (rpmdb->db_dbenv == NULL) {
+	    static int runrecoverycount = 0;
 	    rc = db_init(dbi, dbhome, dbfile, dbsubfile, &dbenv);
 	    switch (rc) {
 	    default:
 		break;
 
 	    case DB_RUNRECOVERY:
+		if (runrecoverycount++ >= 1) {
+		    rpmlog(RPMLOG_ERR, _("RUNRECOVERY failed, exiting ...\n"));
+		   exit(EXIT_FAILURE);
+		}
 		rpmlog(RPMLOG_ERR, _("Runnning db->verify ...\n"));
 		rpmdb = rpmdbLink(rpmdb, "DB_RUNRECOVERY");
+assert(rpmdb != NULL);
 		rpmdb->db_remove_env = 1;
 		rpmdb->db_verifying = 1;
 		xx = rpmdbVerifyAllDBI(rpmdb);
@@ -1551,7 +1567,7 @@ assert(rpmdb && rpmdb->db_dbenv);
 #endif
 
 		if (rc == 0 && dbi_type == DB_UNKNOWN) {
-#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH == 11) \
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH >= 11) \
  || (DB_VERSION_MAJOR == 4)
 		    xx = db->get_type(db, &dbi_type);
 		    if (xx == 0)
