@@ -4,18 +4,25 @@
 
 #include "system.h"
 
+#include <rpmiotypes.h>	/* XXX rpmRC codes. */
+#include <rpmio.h>	/* XXX Realpath(). */
 #include <rpmmacro.h>	/* XXX for rpmCleanPath */
+
+#include <rpmtag.h>
 #include <rpmdb.h>
 
 #include "fprint.h"
 #include "debug.h"
 
+/*@access hashTable @*/
+
 fingerPrintCache fpCacheCreate(int sizeHint)
 {
     fingerPrintCache fpc;
 
     fpc = xmalloc(sizeof(*fpc));
     fpc->ht = htCreate(sizeHint * 2, 0, 1, NULL, NULL);
+assert(fpc->ht != NULL);
     return fpc;
 }
 
@@ -54,7 +61,8 @@ static /*@null@*/ const struct fprintCacheEntry_s * cacheContainsDirectory(
  */
 static fingerPrint doLookup(fingerPrintCache cache,
 		const char * dirName, const char * baseName, int scareMem)
-	/*@modifies cache @*/
+	/*@globals fileSystem, internalState @*/
+	/*@modifies cache, fileSystem, internalState @*/
 {
     char dir[PATH_MAX];
     const char * cleanDirName;
@@ -87,7 +95,7 @@ static fingerPrint doLookup(fingerPrintCache cache,
 	/* if the current directory doesn't exist, we might fail. 
 	   oh well. likewise if it's too long.  */
 	dir[0] = '\0';
-	if (realpath(".", dir) != NULL) {
+	if (Realpath(".", dir) != NULL) {
 	    end = dir + strlen(dir);
 	    if (end[-1] != '/')	*end++ = '/';
 	    end = stpncpy(end, cleanDirName, sizeof(dir) - (end - dir));
@@ -131,8 +139,8 @@ static fingerPrint doLookup(fingerPrintCache cache,
 	    /*@-usereleased@*/	/* LCL: contiguous malloc confusion */
 	    dn += sizeof(*newEntry);
 	    strcpy(dn, (*buf != '\0' ? buf : "/"));
-	    newEntry->ino = sb.st_ino;
-	    newEntry->dev = sb.st_dev;
+	    newEntry->ino = (ino_t)sb.st_ino;
+	    newEntry->dev = (dev_t)sb.st_dev;
 	    newEntry->dirName = dn;
 	    fp.entry = newEntry;
 
@@ -183,7 +191,8 @@ fingerPrint fpLookup(fingerPrintCache cache, const char * dirName,
     return doLookup(cache, dirName, baseName, scareMem);
 }
 
-uint32_t fpHashFunction(uint32_t h, const void * data, /*@unused@*/ size_t size)
+rpmuint32_t fpHashFunction(rpmuint32_t h, const void * data,
+		/*@unused@*/ size_t size)
 {
     const fingerPrint * fp = data;
     const char * chptr = fp->baseName;
@@ -217,8 +226,8 @@ int fpEqual(const void * key1, const void * key2)
 }
 
 void fpLookupList(fingerPrintCache cache, const char ** dirNames, 
-		  const char ** baseNames, const uint32_t * dirIndexes, 
-		  uint32_t fileCount, fingerPrint * fpList)
+		  const char ** baseNames, const rpmuint32_t * dirIndexes, 
+		  rpmuint32_t fileCount, fingerPrint * fpList)
 {
     unsigned i;
 
@@ -236,7 +245,7 @@ void fpLookupList(fingerPrintCache cache, const char ** dirNames,
     }
 }
 
-#ifdef	UNUSED
+#ifdef	NOTUSED
 /**
  * Return finger prints of all file names in header.
  * @warning: scareMem is assumed!
@@ -246,14 +255,14 @@ void fpLookupList(fingerPrintCache cache, const char ** dirNames,
  */
 static
 void fpLookupHeader(fingerPrintCache cache, Header h, fingerPrint * fpList)
-	/*@modifies h, cache, *fpList @*/;
+	/*@modifies h, cache, *fpList @*/
 {
     rpmTagData he_p = { .ptr = NULL };
     HE_s he_s = { .tag = 0, .t = 0, .p = &he_p, .c = 0, .freeData = 0 };
     HE_t he = &he_s;
     const char ** baseNames;
     const char ** dirNames;
-    uint32_t * dirIndexes;
+    rpmuint32_t * dirIndexes;
     rpmTagCount fileCount;
     int xx;
 
