@@ -3,7 +3,8 @@
  */
 
 #include "system.h"
-#include <rpmio.h>
+
+#include <rpmiotypes.h>
 #include <rpmmacro.h>
 
 #define	_RPMTAG_INTERNAL
@@ -24,6 +25,7 @@ static struct tagMacro {
     { "name",		RPMTAG_NAME },
     { "version",	RPMTAG_VERSION },
     { "release",	RPMTAG_RELEASE },
+    { "distepoch",	RPMTAG_DISTEPOCH },
     { "epoch",		RPMTAG_EPOCH },
     { "arch",		RPMTAG_ARCH },
     { "os",		RPMTAG_OS },
@@ -39,7 +41,7 @@ int headerMacrosLoad(Header h)
     struct tagMacro * tagm;
     char numbuf[64];
     const char * val;
-    uint64_t ival;
+    rpmuint64_t ival;
     int xx;
 
     numbuf[0] = '\0';
@@ -64,15 +66,15 @@ int headerMacrosLoad(Header h)
 	ival = 0;
 	switch (he->t) {
 	case RPM_UINT8_TYPE:
-	    ival = he->p.ui8p[0];
+	    ival = (rpmuint64_t)he->p.ui8p[0];
 	    val = numbuf;
 	    /*@switchbreak@*/ break;
 	case RPM_UINT16_TYPE:
-	    ival = he->p.ui16p[0];
+	    ival = (rpmuint64_t)he->p.ui16p[0];
 	    val = numbuf;
 	    /*@switchbreak@*/ break;
 	case RPM_UINT32_TYPE:
-	    ival = he->p.ui32p[0];
+	    ival = (rpmuint64_t)he->p.ui32p[0];
 	    val = numbuf;
 	    /*@switchbreak@*/ break;
 	case RPM_UINT64_TYPE:
@@ -151,9 +153,8 @@ int headerMacrosUnload(Header h)
 }
 /*@=globs =mods =incondefs@*/
 
-int headerNEVRA(Header h, const char **np,
-		/*@unused@*/ const char **ep, const char **vp, const char **rp,
-		const char **ap)
+int headerNEVRA(Header h, const char **np, /*@unused@*/ const char **ep,
+		const char **vp, const char **rp, const char **ap)
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
 
@@ -213,10 +214,10 @@ int headerNEVRA(Header h, const char **np,
     return 0;
 }
 
-uint32_t hGetColor(Header h)
+rpmuint32_t hGetColor(Header h)
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
-    uint32_t hcolor = 0;
+    rpmuint32_t hcolor = 0;
     int xx;
 
     he->tag = RPMTAG_FILECOLORS;
