@@ -14,6 +14,11 @@
 # undef _FILE_OFFSET_BITS
 # define _FILE_OFFSET_BITS 32
 #endif
+#if defined(__LCLINT__)
+/*@-incondefs@*/
+typedef long long loff_t;
+/*@=incondefs@*/
+#endif
 #include <gelf.h>
 
 #if !defined(DT_GNU_PRELINKED)
@@ -26,7 +31,6 @@
 #endif
 
 #include "rpmio_internal.h"
-#include <rpmlib.h>	/* XXX rpmMkdirPath */
 #include <rpmmacro.h>
 #include "misc.h"
 #include "legacy.h"
@@ -63,7 +67,7 @@ static int open_dso(const char * path, /*@null@*/ pid_t * pidp, /*@null@*/ size_
 	struct stat sb, * st = &sb;
 	if (stat(path, st) < 0)
 	    return -1;
-	*fsizep = st->st_size;
+	*fsizep = (size_t)st->st_size;
     }
 
     fdno = open(path, O_RDONLY);
@@ -239,7 +243,9 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
     /* Reap the prelink -y helper. */
     if (pid) {
 	int status;
+/*@+longunsignedintegral@*/
 	(void) waitpid(pid, &status, 0);
+/*@=longunsignedintegral@*/
 	if (!WIFEXITED(status) || WEXITSTATUS(status))
 	    rc = 1;
     }
@@ -253,8 +259,3 @@ exit:
 
     return rc;
 }
-
-int domd5(const char * fn, unsigned char * digest, int asAscii, size_t *fsizep)
-{
-    return dodigest(PGPHASHALGO_MD5, fn, digest, asAscii, fsizep);
-}
