@@ -11,14 +11,13 @@
 
 #include <netinet/in.h>
 
-#if defined(HAVE_KEYUTILS_H)
-#include <keyutils.h>
-#endif
-
+#define	_RPMIOB_INTERNAL
+#include <rpmiotypes.h>
 #include <rpmio_internal.h>
+#include <rpmcb.h>
 #include <rpmbc.h>		/* XXX beecrypt base64 */
-#include <rpmcb.h>		/* XXX fnpyKey */
 #include <rpmmacro.h>
+#include <rpmku.h>
 
 #define	_RPMTAG_INTERNAL
 #include "header_internal.h"
@@ -35,12 +34,14 @@
 #include "debug.h"
 
 /*@access rpmts @*/
+/*@access rpmxar @*/
 /*@access pgpDig @*/
 /*@access pgpDigParams @*/
 /*@access Header @*/            /* XXX compared with NULL */
 /*@access entryInfo @*/
 /*@access indexEntry @*/
-/*@access FD_t @*/              /* XXX stealing digests */
+/*@access FD_t @*/		/* XXX stealing digests */
+/*@access FDSTAT_t @*/		/* XXX stealing digests */
 
 /*@unchecked@*/
 int _pkgio_debug = 0;
@@ -62,15 +63,16 @@ static int _print_pkts = 0;
  * Write (with unload) header to file handle.
  * @param fd		file handle
  * @param h		header
+ * @retval *msg		failure msg
  * @return		RPMRC_OK on success
  */
 static
 rpmRC rpmWriteHeader(FD_t fd, /*@null@*/ Header h, /*@null@*/ const char ** msg)
-	/*@globals fileSystem @*/
-	/*@modifies fd, h, *msg, fileSystem @*/
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fd, h, *msg, fileSystem, internalState @*/
 {
     const void * uh = NULL;
-    ssize_t nb;
+    size_t nb;
     size_t length;
     rpmRC rc = RPMRC_FAIL;	/* assume failure */
 
@@ -127,7 +129,7 @@ rpmop rpmtsOp(rpmts ts, rpmtsOpX opx)
 {
     rpmop op = NULL;
 
-    if (ts != NULL && opx >= 0 && opx < RPMTS_OP_MAX)
+    if (ts != NULL && (int)opx >= 0 && (int)opx < RPMTS_OP_MAX)
 	op = ts->ops + opx;
 /*@-usereleased -compdef @*/
     return op;
@@ -160,15 +162,16 @@ rpmRC rpmtsFindPubkey(rpmts ts, void * _dig)
     pgpDigParams pubp = pgpGetPubkey(dig);
     rpmRC res = RPMRC_NOKEY;
     const char * pubkeysource = NULL;
-#if defined(HAVE_KEYUTILS_H)
+    rpmiob iob = NULL;
     int krcache = 1;	/* XXX assume pubkeys are cached in keyutils keyring. */
-#endif
     int xx;
 
 assert(dig != NULL);
 assert(sigp != NULL);
 assert(pubp != NULL);
+/*@-sefparams@*/
 assert(rpmtsDig(ts) == dig);
+/*@=sefparams@*/
 
 #if 0
 fprintf(stderr, "==> find sig id %08x %08x ts pubkey id %08x %08x\n",
@@ -186,54 +189,52 @@ fprintf(stderr, "*** free pkt %p[%d] id %08x %08x\n", ts->pkpkt, ts->pkpktlen, p
 	memset(ts->pksignid, 0, sizeof(ts->pksignid));
     }
 
-#if defined(HAVE_KEYUTILS_H)
-	/* Try keyutils keyring lookup. */
-    if (krcache && ts->pkpkt == NULL) {
-	key_serial_t keyring = KEY_SPEC_PROCESS_KEYRING;
-	const char * krprefix = "rpm:gpg:pubkey:";
-	char krfp[32];
-	char * krn = alloca(strlen(krprefix) + sizeof("12345678"));
-	long key;
-
-	(void) snprintf(krfp, sizeof(krfp), "%08X", pgpGrab(sigp->signid+4, 4));
-	krfp[sizeof(krfp)-1] = '\0';
-	*krn = '\0';
-	(void) stpcpy( stpcpy(krn, krprefix), krfp);
-
-/*@-moduncon@*/
-	key = keyctl_search(keyring, "user", krn, 0);
-	xx = keyctl_read(key, NULL, 0);
-	if (xx > 0) {
-	    ts->pkpktlen = xx;
-	    ts->pkpkt = NULL;
-	    xx = keyctl_read_alloc(key, (void **)&ts->pkpkt);
-	    if (xx > 0) {
-		pubkeysource = xstrdup(krn);
-		krcache = 0;	/* XXX don't bother caching. */
-	    } else {
-		ts->pkpkt = _free(ts->pkpkt);
-		ts->pkpktlen = 0;
-	    }
+    /* Try keyutils keyring lookup. */
+    if (ts->pkpkt == NULL) {
+	iob = NULL;
+	switch (rpmkuFindPubkey(sigp, &iob)) {
+	case RPMRC_NOTFOUND:
+	case RPMRC_FAIL:
+	case RPMRC_NOTTRUSTED:
+	case RPMRC_NOKEY:
+	    break;
+	case RPMRC_OK:
+	    pubkeysource = xstrdup("keyutils");
+	    krcache = 0;	/* XXX don't bother caching. */
+	    ts->pkpkt = memcpy(xmalloc(iob->blen), iob->b, iob->blen);
+	    ts->pkpktlen = iob->blen;
+	    break;
 	}
-/*@=moduncon@*/
     }
-#endif
 
     /* Try rpmdb keyring lookup. */
     if (ts->pkpkt == NULL) {
 	unsigned hx = 0xffffffff;
 	unsigned ix = 0xffffffff;
-	rpmdbMatchIterator mi;
+	rpmmi mi;
 	Header h;
 
+	/* XXX Do a lazy open if not done already. */
+	if (ts->rdb == NULL) {
+	    xx = rpmdbOpen(ts->rootDir, &ts->rdb, ts->dbmode, 0644);
+	    if (xx) {
+		const char * dn = rpmGetPath(ts->rootDir, "%{_dbpath}", NULL);
+		rpmlog(RPMLOG_ERR,
+			_("cannot open Packages database in %s\n"), dn);
+		dn = _free(dn);
+	    }
+	}
+
 	/* Retrieve the pubkey that matches the signature. */
 	he->tag = RPMTAG_PUBKEYS;
-	mi = rpmdbInitIterator(rpmtsGetRdb(ts), RPMTAG_PUBKEYS, sigp->signid, sizeof(sigp->signid));
-	while ((h = rpmdbNextIterator(mi)) != NULL) {
+/*@-nullstate@*/
+	mi = rpmmiInit(rpmtsGetRdb(ts), RPMTAG_PUBKEYS, sigp->signid, sizeof(sigp->signid));
+/*@=nullstate@*/
+	while ((h = rpmmiNext(mi)) != NULL) {
 	    if (!headerGet(h, he, 0))
 		continue;
-	    hx = rpmdbGetIteratorOffset(mi);
-	    ix = rpmdbGetIteratorFileNum(mi);
+	    hx = rpmmiInstance(mi);
+	    ix = rpmmiFilenum(mi);
 /*@-moduncon -nullstate @*/
 	    if (ix >= (unsigned) he->c
 	     || b64decode(he->p.argv[ix], (void **) &ts->pkpkt, &ts->pkpktlen))
@@ -242,7 +243,7 @@ fprintf(stderr, "*** free pkt %p[%d] id %08x %08x\n", ts->pkpkt, ts->pkpktlen, p
 	    he->p.ptr = _free(he->p.ptr);
 	    break;
 	}
-	mi = rpmdbFreeIterator(mi);
+	mi = rpmmiFree(mi);
 
 	if (ix < 0xffffffff) {
 	    char hnum[32];
@@ -261,7 +262,7 @@ fprintf(stderr, "*** free pkt %p[%d] id %08x %08x\n", ts->pkpkt, ts->pkpktlen, p
 
 	xx = 0;
 	if (fn && *fn != '%') {
-	    xx = (pgpReadPkts(fn, (const uint8_t **)&ts->pkpkt, &ts->pkpktlen) != PGPARMOR_PUBKEY);
+	    xx = (pgpReadPkts(fn, &ts->pkpkt, &ts->pkpktlen) != PGPARMOR_PUBKEY);
 	}
 	fn = _free(fn);
 	if (xx) {
@@ -296,7 +297,7 @@ fprintf(stderr, "*** free pkt %p[%d] id %08x %08x\n", ts->pkpkt, ts->pkpktlen, p
 	goto exit;
 
     /* Retrieve parameters from pubkey packet(s). */
-    xx = pgpPrtPkts((uint8_t *)ts->pkpkt, ts->pkpktlen, dig, 0);
+    xx = pgpPrtPkts((rpmuint8_t *)ts->pkpkt, ts->pkpktlen, dig, 0);
 
     /* Do the parameters match the signature? */
     if (sigp->pubkey_algo == pubp->pubkey_algo
@@ -308,31 +309,23 @@ fprintf(stderr, "*** free pkt %p[%d] id %08x %08x\n", ts->pkpkt, ts->pkpktlen, p
 
 	/* XXX Verify any pubkey signatures. */
 
-#if defined(HAVE_KEYUTILS_H)
 	/* Save the pubkey in the keyutils keyring. */
 	if (krcache) {
-	    key_serial_t keyring = KEY_SPEC_PROCESS_KEYRING;
-	    const char * krprefix = "rpm:gpg:pubkey:";
-	    char krfp[32];
-	    char * krn = alloca(strlen(krprefix) + sizeof("12345678"));
-
-	    (void) snprintf(krfp, sizeof(krfp), "%08X", pgpGrab(sigp->signid+4, 4));
-	    krfp[sizeof(krfp)-1] = '\0';
-	    *krn = '\0';
-	    (void) stpcpy( stpcpy(krn, krprefix), krfp);
-/*@-moduncon -noeffectuncon @*/
-	    (void) add_key("user", krn, ts->pkpkt, ts->pkpktlen, keyring);
-/*@=moduncon =noeffectuncon @*/
+	    if (iob == NULL) {
+		iob = rpmiobNew(ts->pkpktlen);
+		iob->b = memcpy(iob->b, ts->pkpkt, iob->blen);
+	    }
+	    (void) rpmkuStorePubkey(sigp, iob);
 	}
-#endif
 
 	/* Pubkey packet looks good, save the signer id. */
 	memcpy(ts->pksignid, pubp->signid, sizeof(ts->pksignid));
 
 	if (pubkeysource)
 	    rpmlog(RPMLOG_DEBUG, "========== %s pubkey id %08x %08x (%s)\n",
-		(sigp->pubkey_algo == PGPPUBKEYALGO_DSA ? "DSA" :
-		(sigp->pubkey_algo == PGPPUBKEYALGO_RSA ? "RSA" : "???")),
+		(sigp->pubkey_algo == (rpmuint8_t)PGPPUBKEYALGO_DSA ? "DSA" :
+		(sigp->pubkey_algo == (rpmuint8_t)PGPPUBKEYALGO_RSA ? "RSA" :
+			"???")),
 		pgpGrab(sigp->signid, 4), pgpGrab(sigp->signid+4, 4),
 		pubkeysource);
 
@@ -345,7 +338,9 @@ exit:
 	ts->pkpkt = _free(ts->pkpkt);
 	ts->pkpktlen = 0;
     }
+/*@-nullstate@*/
     return res;
+/*@=nullstate@*/
 }
 
 pgpDig rpmtsDig(rpmts ts)
@@ -372,7 +367,8 @@ void rpmtsCleanDig(rpmts ts)
 	opx = RPMTS_OP_SIGNATURE;
 	(void) rpmswAdd(rpmtsOp(ts, opx), pgpStatsAccumulator(ts->dig, opx));
 /*@-onlytrans@*/
-	ts->dig = pgpDigFree(ts->dig);
+	(void) pgpDigFree(ts->dig, "rpmtsCleanDig");
+	ts->dig = NULL;		/* XXX make sure the ptr is __REALLY__ gone */
 /*@=onlytrans@*/
     }
 }
@@ -414,7 +410,7 @@ static unsigned char lead_magic[] = {
  * @param *msg		name to include in lead (or NULL)
  * @return		RPMRC_OK on success, RPMRC_FAIL on error
  */
-static rpmRC wrLead(FD_t fd, const void * ptr, const char ** msg)
+static rpmRC wrLead(FD_t fd, const void * ptr, /*@null@*/ const char ** msg)
 	/*@globals fileSystem @*/
 	/*@modifies fd, fileSystem @*/
 {
@@ -453,7 +449,7 @@ fprintf(stderr, "--> wrLead(%p, %p, %p)\n", fd, ptr, msg);
  * @return		rpmRC return code
  */
 static rpmRC rdLead(FD_t fd, /*@out@*/ /*@null@*/ void * ptr,
-		const char ** msg)
+		/*@null@*/ const char ** msg)
 	/*@globals fileSystem @*/
 	/*@modifies fd, *ptr, *msg, fileSystem @*/
 {
@@ -491,9 +487,10 @@ fprintf(stderr, "--> rdLead(%p, %p, %p)\n", fd, ptr, msg);
 	unsigned char * bh = (unsigned char *)l;
 	if (bh[0] == 'x' && bh[1] == 'a' && bh[2] == 'r' && bh[3] == '!') {
 	    const char * fn = fdGetOPath(fd);
+assert(fn != NULL);
 	    xar = rpmxarNew(fn, "r");
 	    fdSetXAR(fd, xar);
-	    (void) rpmxarFree(xar);
+	    (void) rpmxarFree(xar, "rdLead");
 	}
     }
 
@@ -560,7 +557,7 @@ exit:
     if (rc == RPMRC_OK && leadp != NULL)
 	*leadp = l;
     else
-	l = _free(l);
+	/*@-dependenttrans@*/ l = _free(l); /*@=dependenttrans@*/
 	
     if (msg != NULL && buf[0] != '\0') {
 	buf[sizeof(buf)-1] = '\0';
@@ -578,9 +575,10 @@ exit:
  * @retval *msg		failure msg
  * @return		rpmRC return code
  */
-static rpmRC wrSignature(FD_t fd, void * ptr, /*@unused@*/ const char ** msg)
-	/*@globals fileSystem @*/
-	/*@modifies fd, ptr, *msg, fileSystem @*/
+static rpmRC wrSignature(FD_t fd, void * ptr,
+		/*@unused@*/ /*@null@*/ const char ** msg)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fd, ptr, *msg, fileSystem, internalState @*/
 {
     Header sigh = ptr;
     static unsigned char zero[8]
@@ -608,7 +606,6 @@ fprintf(stderr, "--> wrSignature(%p, %p, %p)\n", fd, ptr, msg);
 
 /**
  * Print package size.
- * @todo rpmio: use fdSize rather than fstat(2) to get file size.
  * @param fd			package file handle
  * @param siglen		signature header size
  * @param pad			signature padding
@@ -616,20 +613,24 @@ fprintf(stderr, "--> wrSignature(%p, %p, %p)\n", fd, ptr, msg);
  * @return 			rpmRC return code
  */
 static inline rpmRC printSize(FD_t fd, size_t siglen, size_t pad, size_t datalen)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/
 {
-    int fdno = Fileno(fd);
     struct stat sb, * st = &sb;
     size_t expected;
     size_t nl = rpmpkgSizeof("Lead", NULL);
 
+#ifndef	DYING	/* XXX Fstat(2) contentLength not gud enuf yet. */
+    int fdno = Fileno(fd);
     /* HACK: workaround for davRead wiring. */
     if (fdno == 123456789) {
+/*@-type@*/
 	st->st_size = 0;
 	st->st_size -= nl + siglen + pad + datalen;
+/*@=type@*/
     } else
-    if (fstat(fdno, st) < 0)
+#endif
+    if (Fstat(fd, st) < 0)
 	return RPMRC_FAIL;
 
     expected = nl + siglen + pad + datalen;
@@ -652,21 +653,22 @@ static inline rpmRC printSize(FD_t fd, size_t siglen, size_t pad, size_t datalen
  * @return		rpmRC return code
  */
 static rpmRC rdSignature(FD_t fd, /*@out@*/ /*@null@*/ void * ptr,
-		const char ** msg)
-	/*@globals fileSystem @*/
-	/*@modifies *ptr, *msg, fileSystem @*/
+		/*@null@*/ const char ** msg)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies *ptr, *msg, fileSystem, internalState @*/
 {
 rpmxar xar = fdGetXAR(fd);
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     Header * sighp = ptr;
     char buf[BUFSIZ];
-    uint32_t block[4];
-    uint32_t il;
-    uint32_t dl;
-    uint32_t * ei = NULL;
+    rpmuint32_t block[4];
+    rpmuint32_t il;
+    rpmuint32_t dl;
+    rpmuint32_t * ei = NULL;
     entryInfo pe;
+    size_t startoff;
     size_t nb;
-    uint32_t ril = 0;
+    rpmuint32_t ril = 0;
     indexEntry entry = memset(alloca(sizeof(*entry)), 0, sizeof(*entry));
     entryInfo info = memset(alloca(sizeof(*info)), 0, sizeof(*info));
     unsigned char * dataStart;
@@ -674,7 +676,8 @@ rpmxar xar = fdGetXAR(fd);
     Header sigh = NULL;
     rpmRC rc = RPMRC_FAIL;		/* assume failure */
     int xx;
-    uint32_t i;
+    rpmuint32_t i;
+    static int map = 1;
 
 if (_pkgio_debug)
 fprintf(stderr, "--> rdSignature(%p, %p, %p)\n", fd, ptr, msg);
@@ -693,6 +696,7 @@ fprintf(stderr, "--> rdSignature(%p, %p, %p)\n", fd, ptr, msg);
 	    goto exit;
 	}
     }
+    startoff = fd->stats->ops[FDSTAT_READ].bytes;
     if ((xx = (int) timedRead(fd, (void *)block, sizeof(block))) != (int) sizeof(block)) {
 	(void) snprintf(buf, sizeof(buf),
 		_("sigh size(%d): BAD, read returned %d"), (int)sizeof(block), xx);
@@ -712,13 +716,13 @@ fprintf(stderr, "--> rdSignature(%p, %p, %p)\n", fd, ptr, msg);
 	    goto exit;
 	}
     }
-    il = (uint32_t) ntohl(block[2]);
+    il = (rpmuint32_t) ntohl(block[2]);
     if (il > 32) {
 	(void) snprintf(buf, sizeof(buf),
 		_("sigh tags: BAD, no. of tags(%u) out of range"), (unsigned) il);
 	goto exit;
     }
-    dl = (uint32_t) ntohl(block[3]);
+    dl = (rpmuint32_t) ntohl(block[3]);
     if (dl > 8192) {
 	(void) snprintf(buf, sizeof(buf),
 		_("sigh data: BAD, no. of bytes(%u) out of range"), (unsigned) dl);
@@ -728,7 +732,24 @@ fprintf(stderr, "--> rdSignature(%p, %p, %p)\n", fd, ptr, msg);
 /*@-sizeoftype@*/
     nb = (il * sizeof(struct entryInfo_s)) + dl;
 /*@=sizeoftype@*/
-    ei = xmalloc(sizeof(il) + sizeof(dl) + nb);
+    if (map) {
+	size_t pvlen = (sizeof(il) + sizeof(dl) + nb);
+        static const int prot = PROT_READ | PROT_WRITE;
+        static const int flags = MAP_PRIVATE| MAP_ANONYMOUS;
+        static const int fdno = -1;
+        static const off_t off = 0;
+
+	ei = mmap(NULL, pvlen, prot, flags, fdno, off);
+	if (ei == NULL || ei == (void *)-1)
+            fprintf(stderr,
+                "==> mmap(%p[%u], 0x%x, 0x%x, %d, 0x%x) error(%d): %s\n",
+                NULL, pvlen, prot, flags, fdno, (unsigned)off,
+                errno, strerror(errno));
+    } else {
+	size_t pvlen = (sizeof(il) + sizeof(dl) + nb);
+	ei = xmalloc(pvlen);
+    }
+
     if ((xx = (int) timedRead(fd, (void *)&ei[2], nb)) != (int) nb) {
 	(void) snprintf(buf, sizeof(buf),
 		_("sigh blob(%u): BAD, read returned %d"), (unsigned) nb, xx);
@@ -736,6 +757,15 @@ fprintf(stderr, "--> rdSignature(%p, %p, %p)\n", fd, ptr, msg);
     }
     ei[0] = block[2];
     ei[1] = block[3];
+
+    if (map) {
+	size_t pvlen = (sizeof(il) + sizeof(dl) + nb);
+        if (mprotect(ei, pvlen, PROT_READ) != 0)
+            fprintf(stderr, "==> mprotect(%p[%u],0x%x) error(%d): %s\n",
+                        ei, pvlen, PROT_READ,
+                        errno, strerror(errno));
+    }
+
     pe = (entryInfo) &ei[2];
     dataStart = (unsigned char *) (pe + il);
     
@@ -753,12 +783,16 @@ fprintf(stderr, "--> rdSignature(%p, %p, %p)\n", fd, ptr, msg);
 /*@-sizeoftype@*/
     if (entry->info.tag == RPMTAG_HEADERSIGNATURES
        && entry->info.type == RPM_BIN_TYPE
-       && entry->info.count == REGION_TAG_COUNT)
+       && entry->info.count == (rpmTagCount)REGION_TAG_COUNT)
     {
 /*@=sizeoftype@*/
 
-assert(entry->info.offset > 0);	/* XXX insurance */
-	if (entry->info.offset >= (int32_t)dl) {
+/*
+ * XXX http://mysql.mirrors.pair.com/Downloads/MySQL-5.0/MySQL-client-community-5.0.51a-0.rhel4.i386.rpm
+ * built by rpm-4.3.3 (from REL4) has entry->info.offset == 0.
+ */
+assert(entry->info.offset >= 0);	/* XXX insurance */
+	if (entry->info.offset >= (rpmint32_t)dl) {
 	    (void) snprintf(buf, sizeof(buf),
 		_("region offset: BAD, tag %u type %u offset %d count %u"),
 		(unsigned) entry->info.tag, (unsigned) entry->info.type,
@@ -771,8 +805,8 @@ assert(entry->info.offset > 0);	/* XXX insurance */
 /*@-sizeoftype@*/
 	(void) memcpy(info, dataEnd, REGION_TAG_COUNT);
 	/* XXX Really old packages have HEADER_IMAGE, not HEADER_SIGNATURES. */
-	if (info->tag == (uint32_t) htonl(RPMTAG_HEADERIMAGE)) {
-	    uint32_t stag = (uint32_t) htonl(RPMTAG_HEADERSIGNATURES);
+	if (info->tag == (rpmuint32_t) htonl(RPMTAG_HEADERIMAGE)) {
+	    rpmuint32_t stag = (rpmuint32_t) htonl(RPMTAG_HEADERSIGNATURES);
 	    info->tag = stag;
 	    memcpy(dataEnd, &stag, sizeof(stag));
 	}
@@ -782,7 +816,7 @@ assert(entry->info.offset > 0);	/* XXX insurance */
 	if (xx != -1 ||
 	    !(entry->info.tag == RPMTAG_HEADERSIGNATURES
 	   && entry->info.type == RPM_BIN_TYPE
-	   && entry->info.count == REGION_TAG_COUNT))
+	   && entry->info.count == (rpmTagCount)REGION_TAG_COUNT))
 	{
 	    (void) snprintf(buf, sizeof(buf),
 		_("region trailer: BAD, tag %u type %u offset %d count %u"),
@@ -794,7 +828,7 @@ assert(entry->info.offset > 0);	/* XXX insurance */
 	memset(info, 0, sizeof(*info));
 
 	/* Is the no. of tags in the region less than the total no. of tags? */
-	ril = (uint32_t) entry->info.offset/sizeof(*pe);
+	ril = (rpmuint32_t) (entry->info.offset/sizeof(*pe));
 	if ((entry->info.offset % sizeof(*pe)) || ril > il) {
 	    (void) snprintf(buf, sizeof(buf),
 		_("region size: BAD, ril(%u) > il(%u)"), (unsigned) ril, (unsigned) il);
@@ -821,7 +855,11 @@ assert(entry->info.offset > 0);	/* XXX insurance */
 	(void) snprintf(buf, sizeof(buf), _("sigh load: BAD"));
 	goto exit;
     }
-    sigh->flags |= HEADERFLAG_ALLOCATED;
+    if (map) {
+	sigh->flags |= HEADERFLAG_MAPPED;
+	sigh->flags |= HEADERFLAG_RDONLY;
+    } else
+	sigh->flags |= HEADERFLAG_ALLOCATED;
     sigh->flags |= HEADERFLAG_SIGNATURE;
 
     {	size_t sigSize = headerSizeof(sigh);
@@ -844,15 +882,18 @@ assert(entry->info.offset > 0);	/* XXX insurance */
 	    rc = printSize(fd, sigSize, pad, datasize);
 	    if (rc != RPMRC_OK)
 		(void) snprintf(buf, sizeof(buf),
-			_("sigh sigSize(%u): BAD, fstat(2) failed"), (unsigned) sigSize);
+			_("sigh sigSize(%u): BAD, Fstat(2) failed"), (unsigned) sigSize);
 	}
 	he->p.ptr = _free(he->p.ptr);
     }
+    (void) headerSetStartOff(sigh, (rpmuint32_t)startoff);
+    (void) headerSetEndOff(sigh, fd->stats->ops[FDSTAT_READ].bytes);
 
 exit:
     if (sighp && sigh && rc == RPMRC_OK)
 	*sighp = headerLink(sigh);
-    sigh = headerFree(sigh);
+    (void)headerFree(sigh);
+    sigh = NULL;
 
     if (msg != NULL) {
 	buf[sizeof(buf)-1] = '\0';
@@ -871,7 +912,7 @@ exit:
  * header-only digest or signature to verify the blob. If found,
  * the digest or signature is verified.
  *
- * @param ts		transaction set
+ * @param dig		signature parameters container
  * @param uh		unloaded header blob
  * @param uc		no. of bytes in blob (or 0 to disable)
  * @retval *msg		signature verification msg
@@ -880,13 +921,13 @@ exit:
 rpmRC headerCheck(pgpDig dig, const void * uh, size_t uc, const char ** msg)
 {
     char buf[8*BUFSIZ];
-    uint32_t * ei = (uint32_t *) uh;
-    uint32_t il = (uint32_t) ntohl(ei[0]);
-    uint32_t dl = (uint32_t) ntohl(ei[1]);
+    rpmuint32_t * ei = (rpmuint32_t *) uh;
+    rpmuint32_t il = (rpmuint32_t) ntohl(ei[0]);
+    rpmuint32_t dl = (rpmuint32_t) ntohl(ei[1]);
 /*@-castexpose@*/
     entryInfo pe = (entryInfo) &ei[2];
 /*@=castexpose@*/
-    uint32_t ildl[2];
+    rpmuint32_t ildl[2];
     size_t pvlen = sizeof(ildl) + (il * sizeof(*pe)) + dl;
     unsigned char * dataStart = (unsigned char *) (pe + il);
     indexEntry entry = memset(alloca(sizeof(*entry)), 0, sizeof(*entry));
@@ -898,11 +939,11 @@ rpmRC headerCheck(pgpDig dig, const void * uh, size_t uc, const char ** msg)
     size_t siglen = 0;
     int blen;
     size_t nb;
-    uint32_t ril = 0;
+    rpmuint32_t ril = 0;
     unsigned char * regionEnd = NULL;
     rpmRC rc = RPMRC_FAIL;	/* assume failure */
     int xx;
-    uint32_t i;
+    rpmuint32_t i;
 
 if (_pkgio_debug)
 fprintf(stderr, "--> headerCheck(%p, %p[%u], %p)\n", dig, uh, (unsigned) uc, msg);
@@ -931,7 +972,7 @@ fprintf(stderr, "--> headerCheck(%p, %p[%u], %p)\n", dig, uh, (unsigned) uc, msg
 /*@-sizeoftype@*/
     if (!(entry->info.tag == RPMTAG_HEADERIMMUTABLE
        && entry->info.type == RPM_BIN_TYPE
-       && entry->info.count == REGION_TAG_COUNT))
+       && entry->info.count == (rpmTagCount)REGION_TAG_COUNT))
     {
 	rc = RPMRC_NOTFOUND;
 	goto exit;
@@ -939,7 +980,7 @@ fprintf(stderr, "--> headerCheck(%p, %p[%u], %p)\n", dig, uh, (unsigned) uc, msg
 /*@=sizeoftype@*/
 
     /* Is the offset within the data area? */
-    if (entry->info.offset >= (unsigned) dl) {
+    if (entry->info.offset >= (int) dl) {
 	(void) snprintf(buf, sizeof(buf),
 		_("region offset: BAD, tag %u type %u offset %d count %u"),
 		(unsigned) entry->info.tag, (unsigned) entry->info.type,
@@ -957,7 +998,7 @@ fprintf(stderr, "--> headerCheck(%p, %p[%u], %p)\n", dig, uh, (unsigned) uc, msg
     if (xx != -1 ||
 	!(entry->info.tag == RPMTAG_HEADERIMMUTABLE
        && entry->info.type == RPM_BIN_TYPE
-       && entry->info.count == REGION_TAG_COUNT))
+       && entry->info.count == (rpmTagCount)REGION_TAG_COUNT))
     {
 	(void) snprintf(buf, sizeof(buf),
 		_("region trailer: BAD, tag %u type %u offset %d count %u"),
@@ -969,7 +1010,7 @@ fprintf(stderr, "--> headerCheck(%p, %p[%u], %p)\n", dig, uh, (unsigned) uc, msg
     memset(info, 0, sizeof(*info));
 
     /* Is the no. of tags in the region less than the total no. of tags? */
-    ril = (uint32_t) entry->info.offset/sizeof(*pe);
+    ril = (rpmuint32_t) (entry->info.offset/sizeof(*pe));
     if ((entry->info.offset % sizeof(*pe)) || ril > il) {
 	(void) snprintf(buf, sizeof(buf),
 		_("region size: BAD, ril(%u) > il(%u)"), (unsigned) ril, (unsigned)il);
@@ -1038,6 +1079,8 @@ exit:
     if (rc != RPMRC_NOTFOUND) {
 	buf[sizeof(buf)-1] = '\0';
 	if (msg) *msg = xstrdup(buf);
+if (_pkgio_debug)
+fprintf(stderr, "--> headerCheck #1: rc %d \"%s\"\n", rc, (msg ? *msg: ""));
 	return rc;
     }
 
@@ -1056,6 +1099,8 @@ exit:
 	}
 	buf[sizeof(buf)-1] = '\0';
 	if (msg) *msg = xstrdup(buf);
+if (_pkgio_debug)
+fprintf(stderr, "--> headerCheck #2: rc %d \"%s\"\n", rc, (msg ? *msg: ""));
 	return rc;
     }
 
@@ -1076,7 +1121,9 @@ assert(dig != NULL);
     case RPMTAG_RSAHEADER:
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, info->count, dig, (_print_pkts & rpmIsDebug()));
-	if (dig->signature.version != 3 && dig->signature.version != 4) {
+	if (dig->signature.version != (rpmuint8_t)3
+	 && dig->signature.version != (rpmuint8_t)4)
+	{
 	    rpmlog(RPMLOG_ERR,
 		_("skipping header with unverifiable V%u signature\n"),
 		(unsigned) dig->signature.version);
@@ -1084,34 +1131,34 @@ assert(dig != NULL);
 	    goto exit;
 	}
 
-	ildl[0] = (uint32_t) htonl(ril);
-	ildl[1] = (uint32_t) (regionEnd - dataStart);
-	ildl[1] = (uint32_t) htonl(ildl[1]);
+	ildl[0] = (rpmuint32_t) htonl(ril);
+	ildl[1] = (rpmuint32_t) (regionEnd - dataStart);
+	ildl[1] = (rpmuint32_t) htonl(ildl[1]);
 
 	op = pgpStatsAccumulator(dig, 10);	/* RPMTS_OP_DIGEST */
 	(void) rpmswEnter(op, 0);
-	dig->hdrmd5ctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
+	dig->hdrctx = rpmDigestInit((pgpHashAlgo)dig->signature.hash_algo, RPMDIGEST_NONE);
 
 	b = NULL; nb = 0;
 	(void) headerGetMagic(NULL, &b, &nb);
 	if (b && nb > 0) {
-	    (void) rpmDigestUpdate(dig->hdrmd5ctx, b, nb);
+	    (void) rpmDigestUpdate(dig->hdrctx, b, nb);
 	    dig->nbytes += nb;
 	}
 
 	b = (unsigned char *) ildl;
 	nb = sizeof(ildl);
-	(void) rpmDigestUpdate(dig->hdrmd5ctx, b, nb);
+	(void) rpmDigestUpdate(dig->hdrctx, b, nb);
 	dig->nbytes += nb;
 
 	b = (unsigned char *) pe;
 	nb = (size_t) (htonl(ildl[0]) * sizeof(*pe));
-	(void) rpmDigestUpdate(dig->hdrmd5ctx, b, nb);
+	(void) rpmDigestUpdate(dig->hdrctx, b, nb);
 	dig->nbytes += nb;
 
 	b = (unsigned char *) dataStart;
 	nb = (size_t) htonl(ildl[1]);
-	(void) rpmDigestUpdate(dig->hdrmd5ctx, b, nb);
+	(void) rpmDigestUpdate(dig->hdrctx, b, nb);
 	dig->nbytes += nb;
 	(void) rpmswExit(op, dig->nbytes);
 
@@ -1119,7 +1166,9 @@ assert(dig != NULL);
     case RPMTAG_DSAHEADER:
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, info->count, dig, (_print_pkts & rpmIsDebug()));
-	if (dig->signature.version != 3 && dig->signature.version != 4) {
+	if (dig->signature.version != (rpmuint8_t)3
+	 && dig->signature.version != (rpmuint8_t)4)
+	{
 	    rpmlog(RPMLOG_ERR,
 		_("skipping header with unverifiable V%u signature\n"),
 		(unsigned) dig->signature.version);
@@ -1128,9 +1177,9 @@ assert(dig != NULL);
 	}
 	/*@fallthrough@*/
     case RPMTAG_SHA1HEADER:
-	ildl[0] = (uint32_t) htonl(ril);
-	ildl[1] = (uint32_t) (regionEnd - dataStart);
-	ildl[1] = (uint32_t) htonl(ildl[1]);
+	ildl[0] = (rpmuint32_t) htonl(ril);
+	ildl[1] = (rpmuint32_t) (regionEnd - dataStart);
+	ildl[1] = (rpmuint32_t) htonl(ildl[1]);
 
 	op = pgpStatsAccumulator(dig, 10);	/* RPMTS_OP_DIGEST */
 	(void) rpmswEnter(op, 0);
@@ -1171,6 +1220,8 @@ assert(dig != NULL);
     buf[sizeof(buf)-1] = '\0';
     if (msg) *msg = xstrdup(buf);
 
+if (_pkgio_debug)
+fprintf(stderr, "--> headerCheck #3: rc %d \"%s\"\n", rc, (msg ? *msg: ""));
     return rc;
 }
 
@@ -1182,7 +1233,7 @@ assert(dig != NULL);
 static size_t szHeader(/*@null@*/ const void * ptr)
 	/*@*/
 {
-    uint32_t p[4];
+    rpmuint32_t p[4];
 assert(ptr != NULL);
     memcpy(p, ptr, sizeof(p));
     return (8 + 8 + 16 * ntohl(p[2]) + ntohl(p[3]));
@@ -1197,9 +1248,9 @@ assert(ptr != NULL);
  * @return		rpmRC return code
  */
 static rpmRC ckHeader(/*@unused@*/ FD_t fd, const void * ptr,
-		/*@unused@*/ const char ** msg)
-	/*@globals fileSystem @*/
-	/*@modifies ptr, fileSystem @*/
+		/*@unused@*/ /*@null@*/ const char ** msg)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies ptr, fileSystem, internalState @*/
 {
     rpmRC rc = RPMRC_OK;
     Header h;
@@ -1207,14 +1258,14 @@ static rpmRC ckHeader(/*@unused@*/ FD_t fd, const void * ptr,
     h = headerLoad((void *)ptr);
     if (h == NULL)
 	rc = RPMRC_FAIL;
-    h = headerFree(h);
+    (void)headerFree(h);
+    h = NULL;
 
     return rc;
 }
 
 /** 
  * Return checked and loaded header.
- * @param dig		signature parameters container
  * @param fd		file handle
  * @retval hdrp		address of header (or NULL)
  * @retval *msg		verification error message (or NULL)
@@ -1228,17 +1279,19 @@ static rpmRC rpmReadHeader(FD_t fd, /*@null@*/ Header * hdrp,
 rpmxar xar = fdGetXAR(fd);
     pgpDig dig = pgpDigLink(fdGetDig(fd), "rpmReadHeader");
     char buf[BUFSIZ];
-    uint32_t block[4];
-    uint32_t il;
-    uint32_t dl;
-    uint32_t * ei = NULL;
-    size_t uc;
+    rpmuint32_t block[4];
+    rpmuint32_t il;
+    rpmuint32_t dl;
+    rpmuint32_t * ei = NULL;
+    size_t uc = 0;
     unsigned char * b;
+    size_t startoff;
     size_t nb;
     Header h = NULL;
     const char * origin = NULL;
     rpmRC rc = RPMRC_FAIL;		/* assume failure */
     int xx;
+    static int map = 1;
 
 if (_pkgio_debug)
 fprintf(stderr, "--> rpmReadHeader(%p, %p, %p)\n", fd, hdrp, msg);
@@ -1265,6 +1318,7 @@ fprintf(stderr, "--> rpmReadHeader(%p, %p, %p)\n", fd, hdrp, msg);
 	}
     }
 
+    startoff = fd->stats->ops[FDSTAT_READ].bytes;
     if ((xx = (int) timedRead(fd, (char *)block, sizeof(block))) != (int)sizeof(block)) {
 	/* XXX Handle EOF's as RPMRC_NOTFOUND, not RPMRC_FAIL, returns. */
 	if (xx == 0)
@@ -1286,14 +1340,14 @@ fprintf(stderr, "--> rpmReadHeader(%p, %p, %p)\n", fd, hdrp, msg);
 	goto exit;
     }
 
-    il = ntohl(block[2]);
+    il = (rpmuint32_t)ntohl(block[2]);
     if (hdrchkTags(il)) {
 	(void) snprintf(buf, sizeof(buf),
 		_("hdr tags: BAD, no. of tags(%u) out of range"), (unsigned) il);
 
 	goto exit;
     }
-    dl = ntohl(block[3]);
+    dl = (rpmuint32_t)ntohl(block[3]);
     if (hdrchkData(dl)) {
 	(void) snprintf(buf, sizeof(buf),
 		_("hdr data: BAD, no. of bytes(%u) out of range\n"), (unsigned) dl);
@@ -1304,7 +1358,22 @@ fprintf(stderr, "--> rpmReadHeader(%p, %p, %p)\n", fd, hdrp, msg);
     nb = (il * sizeof(struct entryInfo_s)) + dl;
 /*@=sizeoftype@*/
     uc = sizeof(il) + sizeof(dl) + nb;
-    ei = (uint32_t *) xmalloc(uc);
+    if (map) {
+        static const int prot = PROT_READ | PROT_WRITE;
+        static const int flags = MAP_PRIVATE| MAP_ANONYMOUS;
+        static const int fdno = -1;
+        static const off_t off = 0;
+
+	ei = mmap(NULL, uc, prot, flags, fdno, off);
+	if (ei == NULL || ei == (void *)-1)
+            fprintf(stderr,
+                "==> mmap(%p[%u], 0x%x, 0x%x, %d, 0x%x) error(%d): %s\n",
+                NULL, uc, prot, flags, fdno, (unsigned)off,
+                errno, strerror(errno));
+    } else {
+	ei = (rpmuint32_t *) xmalloc(uc);
+    }
+
     if ((xx = (int) timedRead(fd, (char *)&ei[2], nb)) != (int) nb) {
 	(void) snprintf(buf, sizeof(buf),
 		_("hdr blob(%u): BAD, read returned %d"), (unsigned)nb, xx);
@@ -1313,6 +1382,13 @@ fprintf(stderr, "--> rpmReadHeader(%p, %p, %p)\n", fd, hdrp, msg);
     ei[0] = block[2];
     ei[1] = block[3];
 
+    if (map) {
+        if (mprotect(ei, uc, PROT_READ) != 0)
+            fprintf(stderr, "==> mprotect(%p[%u],0x%x) error(%d): %s\n",
+                        ei, uc, PROT_READ,
+                        errno, strerror(errno));
+    }
+
     /* Sanity check header tags */
     rc = headerCheck(dig, ei, uc, msg);
     if (rc != RPMRC_OK)
@@ -1324,26 +1400,60 @@ fprintf(stderr, "--> rpmReadHeader(%p, %p, %p)\n", fd, hdrp, msg);
 	(void) snprintf(buf, sizeof(buf), _("hdr load: BAD\n"));
         goto exit;
     }
-    h->flags |= HEADERFLAG_ALLOCATED;
+    if (map) {
+	h->flags |= HEADERFLAG_MAPPED;
+	h->flags |= HEADERFLAG_RDONLY;
+    } else
+	h->flags |= HEADERFLAG_ALLOCATED;
     ei = NULL;	/* XXX will be freed with header */
 
     /* Save the opened path as the header origin. */
+    /* XXX TODO: push the Realpath() underneath fdGetOPath(). */
     origin = fdGetOPath(fd);
-    if (origin != NULL)
-	(void) headerSetOrigin(h, origin);
+    if (origin != NULL) {
+	const char * lpath = NULL;
+	int ut = urlPath(origin, &lpath);
+	ut = ut;	/* XXX keep gcc quiet. */
+	if (lpath && *lpath != '/') {
+	    char * rpath = Realpath(origin, NULL);
+	    (void) headerSetOrigin(h, rpath);
+	    rpath = _free(rpath);
+	} else
+	    (void) headerSetOrigin(h, origin);
+    }
+/*@-mods@*/
+    {	struct stat * st = headerGetStatbuf(h);
+	int saveno = errno;
+	(void) Fstat(fd, st);
+	errno = saveno;
+    }
+/*@=mods@*/
+    (void) headerSetStartOff(h, (rpmuint32_t)startoff);
+    (void) headerSetEndOff(h, fd->stats->ops[FDSTAT_READ].bytes);
     
 exit:
     if (hdrp && h && rc == RPMRC_OK)
 	*hdrp = headerLink(h);
-    ei = _free(ei);
-    dig = pgpDigFree(dig);
-    h = headerFree(h);
+    if (ei != NULL && uc > 0) {
+	if (map) {
+            if (munmap(ei, uc) != 0)
+                fprintf(stderr, "==> munmap(%p[%u]) error(%d): %s\n",
+                ei, uc, errno, strerror(errno));
+	    ei = NULL;
+	} else
+	    ei = _free(ei);
+    }
+    dig = pgpDigFree(dig, "rpmReadHeader");
+    (void)headerFree(h);
+    h = NULL;
 
     if (msg != NULL && *msg == NULL && buf[0] != '\0') {
 	buf[sizeof(buf)-1] = '\0';
 	*msg = xstrdup(buf);
     }
 
+if (_pkgio_debug)
+fprintf(stderr, "--> rpmReadHeader: rc %d \"%s\"\n", rc, (msg ? *msg: ""));
     return rc;
 }
 
@@ -1355,9 +1465,9 @@ exit:
  * @return		rpmRC return code
  */
 static rpmRC rdHeader(FD_t fd, /*@out@*/ /*@null@*/ void * ptr,
-		const char ** msg)
-	/*@globals fileSystem @*/
-	/*@modifies fd, *ptr, *msg, fileSystem @*/
+		/*@null@*/ const char ** msg)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fd, *ptr, *msg, fileSystem, internalState @*/
 {
     Header * hdrp = ptr;
 /*@-compdef@*/
@@ -1372,9 +1482,10 @@ static rpmRC rdHeader(FD_t fd, /*@out@*/ /*@null@*/ void * ptr,
  * @retval *msg		failure msg
  * @return		rpmRC return code
  */
-static rpmRC wrHeader(FD_t fd, void * ptr, /*@unused@*/ const char ** msg)
-	/*@globals fileSystem @*/
-	/*@modifies fd, ptr, *msg, fileSystem @*/
+static rpmRC wrHeader(FD_t fd, void * ptr,
+		/*@unused@*/ /*@null@*/ const char ** msg)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fd, ptr, *msg, fileSystem, internalState @*/
 {
     Header h = ptr;
     return rpmWriteHeader(fd, h, msg);
