@@ -5,11 +5,11 @@
 
 #include "system.h"
 
-#include <rpmio.h>
-#include <rpmcb.h>		/* XXX fnpyKey */
+#include <rpmiotypes.h>
+#include <rpmtag.h>
+#include <rpmtypes.h>
 #include <rpmcli.h>
 
-
 #include "debug.h"
 
 struct rpmQVKArguments_s rpmDBArgs;
