@@ -8,13 +8,19 @@
  */
 
 #include <assert.h>
-#include <rpmlib.h>
+#include <rpmtypes.h>
 #include <mire.h>
+#if defined(_RPMDB_INTERNAL)
 #if defined(WITH_DB)
 #include "db.h"
 #else
 #include "db_emu.h"
 #endif
+#endif
+
+#if defined(_RPMDB_INTERNAL)
+#define DBT_INIT /*@-fullinitblock@*/ {0} /*@-fullinitblock@*/ 	/* -Wno-missing-field-initializers */
+#endif
 
 /*@-exportlocal@*/
 /*@unchecked@*/
@@ -30,7 +36,7 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmdb_s * rpmdb;
 /** \ingroup rpmdb
  * Database iterator.
  */
-typedef /*@abstract@*/ struct rpmdbMatchIterator_s * rpmdbMatchIterator;
+typedef /*@abstract@*/ struct rpmmi_s * rpmmi;
 #endif
 
 /**
@@ -47,17 +53,17 @@ typedef /*@abstract@*/ struct _dbiIndexSet * dbiIndexSet;
 typedef /*@abstract@*/ struct _dbiIndex * dbiIndex;
 
 #if defined(_RPMDB_INTERNAL)
+#include <rpmio.h>
 #include <rpmsw.h>
+
 #if !defined(SWIG)	/* XXX inline dbiFoo() need */
 /** \ingroup dbi
  * A single item from an index database (i.e. the "data returned").
- * Note: In rpm-3.0.4 and earlier, this structure was passed by value,
- * and was identical to the "data saved" structure below.
  */
 struct _dbiIndexItem {
-    uint32_t hdrNum;			/*!< header instance in db */
-    uint32_t tagNum;			/*!< tag index in header */
-    uint32_t fpNum;			/*!< finger print index */
+    rpmuint32_t hdrNum;			/*!< header instance in db */
+    rpmuint32_t tagNum;			/*!< tag index in header */
+    rpmuint32_t fpNum;			/*!< finger print index */
 };
 
 /** \ingroup dbi
@@ -79,10 +85,10 @@ struct _dbiVec {
 /** \ingroup dbi
  * Return handle for an index database.
  * @param rpmdb		rpm database
- * @param rpmtag	rpm tag
+ * @param tag		rpm tag
  * @return		0 on success
  */
-    int (*open) (rpmdb rpmdb, rpmTag rpmtag, /*@out@*/ dbiIndex * dbip)
+    int (*open) (rpmdb rpmdb, rpmTag tag, /*@out@*/ dbiIndex * dbip)
 	/*@globals fileSystem @*/
 	/*@modifies *dbip, fileSystem @*/;
 
@@ -278,7 +284,7 @@ struct _dbiIndex {
 
     int	dbi_ecflags;		/*!< db_env_create flags */
     int	dbi_cflags;		/*!< db_create flags */
-    int	dbi_oeflags;		/*!< common (db,dbenv}->open flags */
+    int	dbi_oeflags;		/*!< common (db,dbenv)->open flags */
     int	dbi_eflags;		/*!< dbenv->open flags */
     int	dbi_oflags;		/*!< db->open flags */
     int	dbi_tflags;		/*!< dbenv->txn_begin flags */
@@ -413,6 +419,7 @@ struct _dbiIndex {
  * Describes the collection of index databases used by rpm.
  */
 struct rpmdb_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
 /*@owned@*/ /*@relnull@*/
     const char * db_root;	/*!< rpmdb path prefix */
 /*@owned@*/
@@ -460,8 +467,10 @@ struct rpmdb_s {
     struct rpmop_s db_putops;	/*!< dbiPut statistics. */
     struct rpmop_s db_delops;	/*!< dbiDel statistics. */
 
+#if defined(__LCLINT__)
 /*@refs@*/
-    int nrefs;			/*!< Reference count. */
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 #endif	/* defined(_RPMDB_INTERNAL) */
 
@@ -487,11 +496,11 @@ extern "C" {
 /** \ingroup db3
  * Return new configured index database handle instance.
  * @param rpmdb		rpm database
- * @param rpmtag	rpm tag
+ * @param tag		rpm tag
  * @return		index database handle
  */
 /*@unused@*/ /*@only@*/ /*@null@*/
-dbiIndex db3New(rpmdb rpmdb, rpmTag rpmtag)
+dbiIndex db3New(rpmdb rpmdb, rpmTag tag)
 	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
 	/*@modifies rpmGlobalMacroContext, internalState @*/;
 
@@ -520,14 +529,14 @@ extern const char * prDbiOpenFlags(int dbflags, int print_dbenv_flags)
 /** \ingroup dbi
  * Return handle for an index database.
  * @param db		rpm database
- * @param rpmtag	rpm tag
+ * @param tag		rpm tag
  * @param flags		(unused)
  * @return		index database handle
  */
-/*@only@*/ /*@null@*/ dbiIndex dbiOpen(/*@null@*/ rpmdb db, rpmTag rpmtag,
+/*@only@*/ /*@null@*/ dbiIndex dbiOpen(/*@null@*/ rpmdb db, rpmTag tag,
 		unsigned int flags)
-	/*@globals rpmGlobalMacroContext, errno, h_errno @*/
-	/*@modifies db, rpmGlobalMacroContext, errno @*/;
+	/*@globals rpmGlobalMacroContext, errno, h_errno, internalState @*/
+	/*@modifies db, rpmGlobalMacroContext, errno, internalState @*/;
 
 /**
  * Return dbiStats accumulator structure.
@@ -868,15 +877,8 @@ unsigned int dbiIndexRecordFileNumber(dbiIndexSet set, int recno)
 /*@unused@*/ /*@null@*/
 rpmdb rpmdbUnlink (/*@killref@*/ /*@only@*/ rpmdb db, const char * msg)
 	/*@modifies db @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@-exportlocal@*/
-/*@null@*/
-rpmdb XrpmdbUnlink (/*@killref@*/ /*@only@*/ rpmdb db, const char * msg,
-		const char * fn, unsigned ln)
-	/*@modifies db @*/;
-/*@=exportlocal@*/
-#define	rpmdbUnlink(_db, _msg)	XrpmdbUnlink(_db, _msg, __FILE__, __LINE__)
+#define	rpmdbUnlink(_db, _msg)	\
+	((rpmdb)rpmioUnlinkPoolItem((rpmioItem)(_db), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmdb
  * Reference a database instance.
@@ -884,24 +886,20 @@ rpmdb XrpmdbUnlink (/*@killref@*/ /*@only@*/ rpmdb db, const char * msg,
  * @param msg
  * @return		new rpm database reference
  */
-/*@unused@*/
+/*@unused@*/ /*@newref@*/
 rpmdb rpmdbLink (rpmdb db, const char * msg)
 	/*@modifies db @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@-exportlocal@*/
-rpmdb XrpmdbLink (rpmdb db, const char * msg,
-		const char * fn, unsigned ln)
-        /*@modifies db @*/;
-/*@=exportlocal@*/
-#define	rpmdbLink(_db, _msg)	XrpmdbLink(_db, _msg, __FILE__, __LINE__)
+#define	rpmdbLink(_db, _msg)	\
+	((void *)rpmioLinkPoolItem((rpmioItem)(_db), _msg, __FILE__, __LINE__))
 
 /** @todo document rpmdbNew
  */
 /*@only@*/ /*@null@*/
 rpmdb rpmdbNew(/*@kept@*/ /*@null@*/ const char * root,
 		/*@kept@*/ /*@null@*/ const char * home,
-		int mode, int perms, int flags);
+		int mode, int perms, int flags)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
 
 /** @todo document rpmdbOpenDatabase
  */
@@ -958,19 +956,19 @@ int rpmdbVerify(/*@null@*/ const char * prefix)
 /**
  * Block access to a single database index.
  * @param db		rpm database
- * @param rpmtag	rpm tag (negative to block)
+ * @param tag		rpm tag (negative to block)
  * @return              0 on success
  */
-int rpmdbBlockDBI(/*@null@*/ rpmdb db, int rpmtag)
+int rpmdbBlockDBI(/*@null@*/ rpmdb db, int tag)
 	/*@modifies db @*/;
 
 /**
  * Close a single database index.
  * @param db		rpm database
- * @param rpmtag	rpm tag
+ * @param tag		rpm tag
  * @return              0 on success
  */
-int rpmdbCloseDBI(/*@null@*/ rpmdb db, int rpmtag)
+int rpmdbCloseDBI(/*@null@*/ rpmdb db, int tag)
 	/*@globals fileSystem @*/
 	/*@modifies db, fileSystem @*/;
 
@@ -999,12 +997,25 @@ int rpmdbSync (/*@null@*/ rpmdb db)
  */
 /*@-exportlocal@*/
 int rpmdbOpenAll (/*@null@*/ rpmdb db)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies db, rpmGlobalMacroContext @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies db, rpmGlobalMacroContext, internalState @*/;
 /*@=exportlocal@*/
 
 /** \ingroup rpmdb
- * Return number of instances of package in rpm database.
+ * Return number of instances of key in a tag index.
+ * @param db		rpm database
+ * @param tag		rpm tag
+ * @param keyp		key data
+ * @param keylen	key data length (0 will use strlen(keyp))
+ * @return		number of instances
+ */
+int rpmdbCount(/*@null@*/ rpmdb db, rpmTag tag,
+		const void * keyp, size_t keylen)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies db, rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/** \ingroup rpmdb
+ * Return number of instances of package in Name index.
  * @param db		rpm database
  * @param name		rpm package name
  * @return		number of instances
@@ -1018,14 +1029,14 @@ int rpmdbCountPackages(/*@null@*/ rpmdb db, const char * name)
  * @param mi		rpm database iterator
  * @return		current header join key
  */
-unsigned int rpmdbGetIteratorOffset(/*@null@*/ rpmdbMatchIterator mi)
+unsigned int rpmmiInstance(/*@null@*/ rpmmi mi)
 	/*@*/;
 
 /** \ingroup rpmdb
  * Return header tag index join key for current position of rpmdb iterator.
  * @param mi		rpm database iterator
  */
-unsigned int rpmdbGetIteratorFileNum(rpmdbMatchIterator mi)
+unsigned int rpmmiFilenum(rpmmi mi)
 	/*@*/;
 
 /** \ingroup rpmdb
@@ -1033,7 +1044,7 @@ unsigned int rpmdbGetIteratorFileNum(rpmdbMatchIterator mi)
  * @param mi		rpm database iterator
  * @return		number of elements
  */
-int rpmdbGetIteratorCount(/*@null@*/ rpmdbMatchIterator mi)
+int rpmmiCount(/*@null@*/ rpmmi mi)
 	/*@*/;
 
 /** \ingroup rpmdb
@@ -1043,7 +1054,7 @@ int rpmdbGetIteratorCount(/*@null@*/ rpmdbMatchIterator mi)
  * @param nHdrNums	number of elements in array
  * @return		0 on success, 1 on failure (bad args)
  */
-int rpmdbAppendIterator(/*@null@*/ rpmdbMatchIterator mi,
+int rpmmiGrow(/*@null@*/ rpmmi mi,
 		/*@null@*/ const int * hdrNums, int nHdrNums)
 	/*@modifies mi @*/;
 
@@ -1056,7 +1067,7 @@ int rpmdbAppendIterator(/*@null@*/ rpmdbMatchIterator mi,
  * @param sorted	is the array sorted? (array will be sorted on return)
  * @return		0 on success, 1 on failure (bad args)
  */
-int rpmdbPruneIterator(/*@null@*/ rpmdbMatchIterator mi,
+int rpmmiPrune(/*@null@*/ rpmmi mi,
 		/*@null@*/ int * hdrNums, int nHdrNums, int sorted)
 	/*@modifies mi, hdrNums @*/;
 
@@ -1068,19 +1079,19 @@ int rpmdbPruneIterator(/*@null@*/ rpmdbMatchIterator mi,
  * @param pattern	pattern to match
  * @return		0 on success
  */
-int rpmdbSetIteratorRE(/*@null@*/ rpmdbMatchIterator mi, rpmTag tag,
+int rpmmiAddPattern(/*@null@*/ rpmmi mi, rpmTag tag,
 		rpmMireMode mode, /*@null@*/ const char * pattern)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies mi, mode, rpmGlobalMacroContext @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies mi, mode, rpmGlobalMacroContext, internalState @*/;
 
 /** \ingroup rpmdb
  * Prepare iterator for lazy writes.
- * @note Must be called before rpmdbNextIterator() with CDB model database.
+ * @note Must be called before rpmmiNext() with CDB model database.
  * @param mi		rpm database iterator
  * @param rewrite	new value of rewrite
  * @return		previous value
  */
-int rpmdbSetIteratorRewrite(/*@null@*/ rpmdbMatchIterator mi, int rewrite)
+int rpmmiSetRewrite(/*@null@*/ rpmmi mi, int rewrite)
 	/*@modifies mi @*/;
 
 /** \ingroup rpmdb
@@ -1089,7 +1100,7 @@ int rpmdbSetIteratorRewrite(/*@null@*/ rpmdbMatchIterator mi, int rewrite)
  * @param modified	new value of modified
  * @return		previous value
  */
-int rpmdbSetIteratorModified(/*@null@*/ rpmdbMatchIterator mi, int modified)
+int rpmmiSetModified(/*@null@*/ rpmmi mi, int modified)
 	/*@modifies mi @*/;
 
 /** \ingroup rpmdb
@@ -1098,19 +1109,19 @@ int rpmdbSetIteratorModified(/*@null@*/ rpmdbMatchIterator mi, int modified)
  * @param ts		transaction set
  * @return		0 always
  */
-int rpmdbSetHdrChk(/*@null@*/ rpmdbMatchIterator mi, /*@null@*/ rpmts ts)
+int rpmmiSetHdrChk(/*@null@*/ rpmmi mi, /*@null@*/ rpmts ts)
 	/*@modifies mi @*/;
 
 /** \ingroup rpmdb
  * Return database iterator.
  * @param db		rpm database
- * @param rpmtag	rpm tag
+ * @param tag		rpm tag
  * @param keyp		key data (NULL for sequential access)
  * @param keylen	key data length (0 will use strlen(keyp))
  * @return		NULL on failure
  */
 /*@only@*/ /*@null@*/
-rpmdbMatchIterator rpmdbInitIterator(/*@null@*/ rpmdb db, rpmTag rpmtag,
+rpmmi rpmmiInit(/*@null@*/ rpmdb db, rpmTag tag,
 			/*@null@*/ const void * keyp, size_t keylen)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies db, rpmGlobalMacroContext, fileSystem, internalState @*/;
@@ -1121,7 +1132,7 @@ rpmdbMatchIterator rpmdbInitIterator(/*@null@*/ rpmdb db, rpmTag rpmtag,
  * @return		NULL on end of iteration.
  */
 /*@null@*/
-Header rpmdbNextIterator(/*@null@*/ rpmdbMatchIterator mi)
+Header rpmmiNext(/*@null@*/ rpmmi mi)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies mi, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
@@ -1147,15 +1158,56 @@ int rpmdbCheckSignals(void)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
 
+/** \ingroup rpmdb
+ * Unreference a rpm database iterator.
+ * @param mi		rpm database iterator
+ * @return		NULL on last dereference
+ */
+/*@null@*/
+rpmmi rpmmiUnlink(/*@only@*/ /*@null@*/rpmmi mi)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies mi, rpmGlobalMacroContext, fileSystem, internalState @*/;
+#define rpmmiUnlink(_mi)  \
+    ((rpmmi)rpmioUnlinkPoolItem((rpmioItem)(_mi), __FUNCTION__, __FILE__, __LINE__))
+
+/** \ingroup rpmdb
+ * Reference a rpm database iterator.
+ * @param mi		rpm database iterator
+ * @return		NULL on last dereference
+ */
+/*@null@*/
+rpmmi rpmmiLink(/*@only@*/ /*@null@*/rpmmi mi)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies mi, rpmGlobalMacroContext, fileSystem, internalState @*/;
+#define rpmmiLink(_mi)  \
+    ((rpmmi)rpmioLinkPoolItem((rpmioItem)(_mi), __FUNCTION__, __FILE__, __LINE__))
+
 /** \ingroup rpmdb
  * Destroy rpm database iterator.
  * @param mi		rpm database iterator
  * @return		NULL always
  */
 /*@null@*/
-rpmdbMatchIterator rpmdbFreeIterator(/*@only@*/ /*@null@*/rpmdbMatchIterator mi)
+rpmmi rpmmiFree(/*@only@*/ /*@null@*/rpmmi mi)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies mi, rpmGlobalMacroContext, fileSystem, internalState @*/;
+#define rpmmiFree(_mi)  \
+    ((rpmmi)rpmioFreePoolItem((rpmioItem)(_mi), __FUNCTION__, __FILE__, __LINE__))
+
+/** \ingroup rpmdb
+ * Return array of keys matching a pattern.
+ * @param db		rpm database
+ * @param tag		rpm tag
+ * @param mode		type of pattern match
+ * @param pat		pattern to match
+ * @retval *argvp	array of keys that match
+ * @return		0 on success
+ */
+int rpmdbMireApply(rpmdb db, rpmTag tag, rpmMireMode mode, const char * pat,
+		const char *** argvp)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies db, *argvp,
+		rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmdb
  * Add package header to rpm database and indices.
