@@ -5,8 +5,8 @@
  * \file rpmdb/rpmtag.h
  */
 
+#include <rpmiotypes.h>
 #include <rpmsw.h>
-#include <stringbuf.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -51,7 +51,7 @@ typedef union rpmDataType_u rpmTagData;	/*!< tag data. */
 
 /** \ingroup header
  */
-typedef uint32_t rpmTagCount;	/*!< tag data element count. */
+typedef rpmuint32_t rpmTagCount;	/*!< tag data element count. */
 
 /** \ingroup header
  */
@@ -64,10 +64,10 @@ typedef struct _HE_s * HE_t;	/*!< tag container. */
 union rpmDataType_u {
 /*@null@*/
     void * ptr;
-    uint8_t * ui8p;		/*!< RPM_UINT8_TYPE | RPM_CHAR_TYPE */
-    uint16_t * ui16p;		/*!< RPM_UINT16_TYPE */
-    uint32_t * ui32p;		/*!< RPM_UINT32_TYPE */
-    uint64_t * ui64p;		/*!< RPM_UINT64_TYPE */
+    rpmuint8_t * ui8p;		/*!< RPM_UINT8_TYPE | RPM_CHAR_TYPE */
+    rpmuint16_t * ui16p;	/*!< RPM_UINT16_TYPE */
+    rpmuint32_t * ui32p;	/*!< RPM_UINT32_TYPE */
+    rpmuint64_t * ui64p;	/*!< RPM_UINT64_TYPE */
 /*@relnull@*/
     const char * str;		/*!< RPM_STRING_TYPE */
     unsigned char * blob;	/*!< RPM_BIN_TYPE */
@@ -193,7 +193,7 @@ enum rpmTag_e {
     RPMTAG_DESCRIPTION		= 1005,	/* s{} */
     RPMTAG_BUILDTIME		= 1006,	/* i */
     RPMTAG_BUILDHOST		= 1007,	/* s */
-    RPMTAG_INSTALLTIME		= 1008,	/* i */
+    RPMTAG_INSTALLTIME		= 1008,	/* i[] */
     RPMTAG_SIZE			= 1009,	/* i */
     RPMTAG_DISTRIBUTION		= 1010,	/* s */
     RPMTAG_VENDOR		= 1011,	/* s */
@@ -326,8 +326,8 @@ enum rpmTag_e {
     RPMTAG_PAYLOADCOMPRESSOR	= 1125,	/* s */
     RPMTAG_PAYLOADFLAGS		= 1126,	/* s */
     RPMTAG_INSTALLCOLOR		= 1127, /* i transaction color when installed */
-    RPMTAG_INSTALLTID		= 1128,	/* i */
-    RPMTAG_REMOVETID		= 1129,	/* i */
+    RPMTAG_INSTALLTID		= 1128,	/* i[] */
+    RPMTAG_REMOVETID		= 1129,	/* i[] */
 /*@-enummemuse@*/
     RPMTAG_SHA1RHN		= 1130, /* internal - obsolete */
 /*@=enummemuse@*/
@@ -405,15 +405,46 @@ enum rpmTag_e {
     RPMTAG_RPMLIBVERSION	= 1199, /* i */
     RPMTAG_RPMLIBTIMESTAMP	= 1200, /* i */
     RPMTAG_RPMLIBVENDOR		= 1201, /* i */
-    RPMTAG_CLASS		= 1202, /* s */
+    RPMTAG_CLASS		= 1202, /* s arbitrary */
     RPMTAG_TRACK		= 1203, /* s internal arbitrary */
     RPMTAG_TRACKPROG		= 1204, /* s internal arbitrary */
     RPMTAG_SANITYCHECK		= 1205, /* s */
     RPMTAG_SANITYCHECKPROG	= 1206, /* s */
+    RPMTAG_FILESTAT		= 1207, /* s[] stat(2) from metadata extension*/
+    RPMTAG_STAT			= 1208, /* s[] stat(2) from disk extension */
+    RPMTAG_ORIGINTID		= 1209,	/* i[] */
+    RPMTAG_ORIGINTIME		= 1210,	/* i[] */
+    RPMTAG_HEADERSTARTOFF	= 1211,	/* l */
+    RPMTAG_HEADERENDOFF		= 1212,	/* l */
+    RPMTAG_PACKAGETIME		= 1213,	/* l */
+    RPMTAG_PACKAGESIZE		= 1214,	/* l */
+    RPMTAG_PACKAGEDIGEST	= 1215,	/* s */
+    RPMTAG_PACKAGESTAT		= 1216,	/* x */
+    RPMTAG_PACKAGEBASEURL	= 1217,	/* s */
+    RPMTAG_DISTEPOCH		= 1218, /* s */
+#define	RPMTAG_D RPMTAG_DISTEPOCH	/* s */
+
+    RPMTAG_FILEDIGESTALGO       = 5011, /* i file checksum algorithm */
 
 /*@-enummemuse@*/
-    RPMTAG_FIRSTFREE_TAG	/*!< internal */
+    RPMTAG_FIRSTFREE_TAG,	/*!< internal */
 /*@=enummemuse@*/
+
+    RPMTAG_PACKAGETRANSFLAGS	= 0x4efaafd9, /* s[] arbitrary */
+    RPMTAG_PACKAGEDEPFLAGS	= 0x748a8314, /* s[] arbitrary */
+
+    RPMTAG_BUILDPREPPROG	= 0x4ba37c9e, /* s[] arbitrary */
+    RPMTAG_BUILDPREP		= 0x799c0b4d, /* s[] arbitrary */
+    RPMTAG_BUILDBUILDPROG	= 0x6fb46014, /* s[] arbitrary */
+    RPMTAG_BUILDBUILD		= 0x5bae1a5a, /* s[] arbitrary */
+    RPMTAG_BUILDINSTALLPROG	= 0x70d4ab6f, /* s[] arbitrary */
+    RPMTAG_BUILDINSTALL		= 0x567f5983, /* s[] arbitrary */
+    RPMTAG_BUILDCHECKPROG	= 0x488a60ce, /* s[] arbitrary */
+    RPMTAG_BUILDCHECK		= 0x7f3b97b5, /* s[] arbitrary */
+    RPMTAG_BUILDCLEANPROG	= 0x42c93d41, /* s[] arbitrary */
+    RPMTAG_BUILDCLEAN		= 0x566042bf, /* s[] arbitrary */
+
+    RPMTAG_LASTARBITRARY_TAG	= 0x80000000  /*!< internal */
 };
 
 #define	RPMTAG_EXTERNAL_TAG		1000000
@@ -434,7 +465,8 @@ enum rpmSigTag_e {
     RPMSIGTAG_BADSHA1_2	= RPMTAG_BADSHA1_2,	/*!< internal Broken SHA1, take 2. */
     RPMSIGTAG_SHA1	= RPMTAG_SHA1HEADER,	/*!< internal sha1 header digest. */
     RPMSIGTAG_DSA	= RPMTAG_DSAHEADER,	/*!< internal DSA header signature. */
-    RPMSIGTAG_RSA	= RPMTAG_RSAHEADER	/*!< internal RSA header signature. */
+    RPMSIGTAG_RSA	= RPMTAG_RSAHEADER,	/*!< internal RSA header signature. */
+    RPMSIGTAG_PADDING	= 0x3fffffff		/*!< signature header padding */
 };
 
 /** \ingroup header
@@ -495,7 +527,7 @@ struct tagStore_s {
 /*@only@*/
     const char * str;           /*!< Tag string (might be arbitrary). */
     rpmTag tag;                 /*!< Tag number. */
-    StringBuf val;		/*!< Tag contents. */
+    rpmiob iob;			/*!< Tag contents. */
 };  
 #endif	/* _RPMTAG_INTERNAL */
 
@@ -615,7 +647,7 @@ void tagClean(/*@null@*/ headerTagIndices _rpmTags)
  * @return		NULL always
  */
 /*@null@*/
-tagStore_t tagStoreFree(/*@only@*/ tagStore_t dbiTags, size_t dbiNTags)
+tagStore_t tagStoreFree(/*@only@*//*@null@*/tagStore_t dbiTags, size_t dbiNTags)
 	/*@modifies dbiTags @*/;
 
 #if defined(_RPMTAG_INTERNAL)
@@ -632,9 +664,10 @@ typedef enum headerSprintfExtensionType_e {
  * HEADER_EXT_TAG format function prototype.
  *
  * @param he		tag container
+ * @param av		parameter array (or NULL)
  * @return		formatted string
  */
-typedef /*only@*/ char * (*headerTagFormatFunction) (HE_t he)
+typedef /*only@*/ char * (*headerTagFormatFunction) (HE_t he, /*@null@*/ const char ** av)
 	/*@modifies he @*/;
 
 /** \ingroup header
@@ -707,9 +740,8 @@ char * headerSprintf(Header h, const char * fmt,
 		/*@null@*/ headerTagTableEntry tags,
 		/*@null@*/ headerSprintfExtension exts,
 		/*@null@*/ /*@out@*/ errmsg_t * errmsg)
-	/*@globals headerCompoundFormats @*/
-	/*@modifies h, *errmsg @*/
-	/*@requires maxSet(errmsg) >= 0 @*/;
+	/*@globals headerCompoundFormats, fileSystem, internalState @*/
+	/*@modifies h, *errmsg, fileSystem, internalState @*/;
 
 /** \ingroup header
  * Retrieve extension or tag value from a header.
@@ -720,7 +752,8 @@ char * headerSprintf(Header h, const char * fmt,
  * @return		1 on success, 0 on failure
  */
 int headerGet(Header h, HE_t he, unsigned int flags)
-	/*@modifies he @*/;
+	/*@globals internalState @*/
+	/*@modifies he, internalState @*/;
 #define	HEADERGET_NOEXTENSION	(1 << 0) /*!< Extension search disabler. */
 #define	HEADERGET_NOI18NSTRING	(1 << 1) /*!< Return i18n strings as argv. */
 
@@ -766,7 +799,8 @@ int headerMod(Header h, HE_t he, /*@unused@*/ unsigned int flags)
  */
 /*@null@*/
 HeaderIterator headerFini(/*@only@*/ HeaderIterator hi)
-	/*@modifies hi @*/;
+	/*@globals fileSystem @*/
+	/*@modifies hi, fileSystem */;
 
 /** \ingroup header
  * Create header tag iterator.
@@ -774,7 +808,8 @@ HeaderIterator headerFini(/*@only@*/ HeaderIterator hi)
  * @return		header tag iterator
  */
 HeaderIterator headerInit(Header h)
-	/*@modifies h */;
+	/*@globals fileSystem @*/
+	/*@modifies h, fileSystem */;
 
 /** \ingroup header
  * Return next tag from header.
@@ -784,7 +819,8 @@ HeaderIterator headerInit(Header h)
  * @return		1 on success, 0 on failure
  */
 int headerNext(HeaderIterator hi, HE_t he, /*@unused@*/ unsigned int flags)
-	/*@modifies hi, he @*/;
+	/*@globals internalState @*/
+	/*@modifies hi, he, internalState @*/;
 
 /** \ingroup header
  * Reference a header instance.
@@ -793,30 +829,38 @@ int headerNext(HeaderIterator hi, HE_t he, /*@unused@*/ unsigned int flags)
  */
 Header headerLink(Header h)
 	/*@modifies h @*/;
+#define headerLink(_h)        \
+    ((Header)rpmioLinkPoolItem((rpmioItem)(_h), __FUNCTION__, __FILE__, __LINE__))
 
 /** \ingroup header
  * Dereference a header instance.
  * @param h		header
- * @return		NULL always
+ * @return		NULL on last dereference
  */
+/*@null@*/
 Header headerUnlink(/*@killref@*/ /*@null@*/ Header h)
 	/*@modifies h @*/;
+#define headerUnlink(_h)        \
+    ((Header)rpmioUnlinkPoolItem((rpmioItem)(_h), __FUNCTION__, __FILE__, __LINE__))
 
 /** \ingroup header
  * Dereference a header instance.
  * @param h		header
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@null@*/
 Header headerFree(/*@killref@*/ /*@null@*/ Header h)
 	/*@modifies h @*/;
+#define headerFree(_h)        \
+    ((Header)rpmioFreePoolItem((rpmioItem)(_h), __FUNCTION__, __FILE__, __LINE__))
 
 /** \ingroup header
  * Create new (empty) header instance.
  * @return		header
  */
 Header headerNew(void)
-	/*@*/;
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/;
 
 /** \ingroup header
  * Return size of on-disk header representation in bytes.
@@ -827,24 +871,28 @@ size_t headerSizeof(/*@null@*/ Header h)
 	/*@modifies h @*/;
 
 /** \ingroup header
+ * headerUnload.
  * @param h		header
  * @retval *lenp	no. bytes in unloaded header blob
  * @return		unloaded header blob (NULL on error)
  */
 /*@only@*/ /*@null@*/
 void * headerUnload(Header h, /*@out@*/ /*@null@*/ size_t * lenp)
-	/*@modifies h, *lenp @*/;
+	/*@globals internalState @*/
+	/*@modifies h, *lenp, internalState @*/;
 
 /** \ingroup header
  * Convert header to on-disk representation, and then reload.
- * This is used to insure that all header data is in one chunk.
+ * This is used to insure that all header data is in a single
+ * contiguous memory allocation.
  * @param h		header (with pointers)
  * @param tag		region tag
  * @return		on-disk header (with offsets)
  */
 /*@null@*/
 Header headerReload(/*@only@*/ Header h, int tag)
-	/*@modifies h @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies h, fileSystem, internalState @*/;
 
 /** \ingroup header
  * Duplicate a header.
@@ -853,7 +901,8 @@ Header headerReload(/*@only@*/ Header h, int tag)
  */
 /*@null@*/
 Header headerCopy(Header h)
-	/*@modifies h @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies h, fileSystem, internalState @*/;
 
 /** \ingroup header
  * Convert header to in-memory representation.
@@ -862,7 +911,8 @@ Header headerCopy(Header h)
  */
 /*@null@*/
 Header headerLoad(/*@kept@*/ void * uh)
-	/*@modifies uh @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies uh, fileSystem, internalState @*/;
 
 /** \ingroup header
  * Make a copy and convert header to in-memory representation.
@@ -871,7 +921,8 @@ Header headerLoad(/*@kept@*/ void * uh)
  */
 /*@null@*/
 Header headerCopyLoad(const void * uh)
-	/*@*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
 
 /** \ingroup header
  * Check if tag is in header.
@@ -913,7 +964,8 @@ int headerAddI18NString(Header h, rpmTag tag, const char * string,
  * @param tagstocopy	array of tags that are copied
  */
 void headerCopyTags(Header headerFrom, Header headerTo, rpmTag * tagstocopy)
-	/*@modifies headerTo @*/;
+	/*@globals internalState @*/
+	/*@modifies headerTo, internalState @*/;
 
 /** \ingroup header
  * Return header magic.
@@ -953,21 +1005,127 @@ const char * headerGetOrigin(/*@null@*/ Header h)
 int headerSetOrigin(/*@null@*/ Header h, const char * origin)
 	/*@modifies h @*/;
 
+/** \ingroup header
+ * Return header base URL (e.g path or URL).
+ * @param h		header
+ * @return		header origin
+ */
+/*@observer@*/ /*@null@*/
+const char * headerGetBaseURL(/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Store header base URL (e.g path or URL).
+ * @param h		header
+ * @param baseurl	new header baseurl
+ * @return		0 always
+ */
+int headerSetBaseURL(/*@null@*/ Header h, const char * baseurl)
+	/*@modifies h @*/;
+
+/** \ingroup header
+ * Return header stat(2) buffer (of origin *.rpm file).
+ * @param h		header
+ * @return		header stat(2) buffer
+ */
+struct stat * headerGetStatbuf(/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Copy into header stat(2) buffer (of origin *.rpm file).
+ * @param h		header
+ * @param st		new header stat(2) buffer
+ * @return		0 always
+ */
+int headerSetStatbuf(/*@null@*/ Header h, struct stat * st)
+	/*@modifies h @*/;
+
+/** \ingroup header
+ * Return digest of origin *.rpm file.
+ * @param h		header
+ * @return		header digest
+ */
+/*@null@*/
+const char * headerGetDigest(/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Store digest of origin *.rpm file.
+ * @param h		header
+ * @param st		new header digest
+ * @return		0 always
+ */
+int headerSetDigest(/*@null@*/ Header h, const char * digest)
+	/*@modifies h @*/;
+
+/** \ingroup header
+ * Return rpmdb pointer.
+ * @param h		header
+ * @return		rpmdb pointer
+ */
+/*@null@*/
+void * headerGetRpmdb(/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Store rpmdb pointer.
+ * @param h		header
+ * @param rpmdb		new rpmdb pointer (or NULL to unset)
+ * @return		NULL always
+ */
+/*@null@*/
+void * headerSetRpmdb(/*@null@*/ Header h, /*@null@*/ void * rpmdb)
+	/*@modifies h @*/;
+
 /** \ingroup header
  * Return header instance (if from rpmdb).
  * @param h		header
  * @return		header instance
  */
-uint32_t headerGetInstance(/*@null@*/ Header h)
+rpmuint32_t headerGetInstance(/*@null@*/ Header h)
 	/*@*/;
 
 /** \ingroup header
  * Store header instance (e.g path or URL).
  * @param h		header
- * @param origin	new header instance
+ * @param instance	new header instance
  * @return		0 always
  */
-uint32_t headerSetInstance(/*@null@*/ Header h, uint32_t instance)
+rpmuint32_t headerSetInstance(/*@null@*/ Header h, rpmuint32_t instance)
+	/*@modifies h @*/;
+
+/** \ingroup header
+ * Return header starting byte offset.
+ * @param h		header
+ * @return		header starting byte offset 
+ */
+rpmuint32_t headerGetStartOff(/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Store header starting byte offset.
+ * @param h		header
+ * @param startoff	new header starting byte offset
+ * @return		0 always
+ */
+rpmuint32_t headerSetStartOff(/*@null@*/ Header h, rpmuint32_t startoff)
+	/*@modifies h @*/;
+
+/** \ingroup header
+ * Return header ending byte offset.
+ * @param h		header
+ * @return		header ending byte offset 
+ */
+rpmuint32_t headerGetEndOff(/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Store header ending byte offset.
+ * @param h		header
+ * @param startoff	new header ending byte offset
+ * @return		0 always
+ */
+rpmuint32_t headerSetEndOff(/*@null@*/ Header h, rpmuint32_t endoff)
 	/*@modifies h @*/;
 
 /** \ingroup header
@@ -1014,15 +1172,17 @@ int headerNEVRA(Header h,
 		/*@null@*/ /*@out@*/ const char ** vp,
 		/*@null@*/ /*@out@*/ const char ** rp,
 		/*@null@*/ /*@out@*/ const char ** ap)
-	/*@modifies h, *np, *vp, *rp, *ap @*/;
+	/*@globals internalState @*/
+	/*@modifies h, *np, *vp, *rp, *ap, internalState @*/;
 
 /**
  * Return header color.
  * @param h		header
  * @return		header color
  */
-uint32_t hGetColor(Header h)
-	/*@modifies h @*/;
+rpmuint32_t hGetColor(Header h)
+	/*@globals internalState @*/
+	/*@modifies h, internalState @*/;
 
 /** \ingroup header
  * Translate and merge legacy signature tags into header.
@@ -1031,7 +1191,8 @@ uint32_t hGetColor(Header h)
  * @param sigh		signature header
  */
 void headerMergeLegacySigs(Header h, const Header sigh)
-	/*@modifies h, sigh @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies h, sigh, fileSystem, internalState @*/;
 
 /** \ingroup header
  * Regenerate signature header.
@@ -1041,7 +1202,9 @@ void headerMergeLegacySigs(Header h, const Header sigh)
  * @return		regenerated signature header
  */
 Header headerRegenSigHeader(const Header h, int noArchiveSize)
-	/*@modifies h @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies h, fileSystem, internalState @*/;
+
 #ifdef __cplusplus
 }
 #endif
