@@ -28,7 +28,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * and GNU Library Public License along with this program;
- * if not, write to the Free Software Foundation, Inc., 
+ * if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  */
@@ -48,15 +48,26 @@
 
 #include "debug.h"
 
+#if defined(__LCLINT__)
+#define	UINT32_T	u_int32_t
+#else
+#define	UINT32_T	rpmuint32_t
+#endif
+
 /*@access rpmdb @*/
 /*@access dbiIndex @*/
 
+/* XXX retrofit the *BSD typedef for the deprived. */
+#if defined(__QNXNTO__)
+typedef rpmuint32_t       u_int32_t;
+#endif
+
 /*@unchecked@*/
 static int _debug = 0;
 
 /* Define the things normally in a header... */
 struct _sql_db_s;	typedef struct _sql_db_s	SQL_DB;
-struct _sql_dbcursor_s;	typedef struct _sql_dbcursor_s *SCP_t; 
+struct _sql_dbcursor_s;	typedef struct _sql_dbcursor_s *SCP_t;
 
 struct _sql_db_s {
     sqlite3 * db;		/* Database pointer */
@@ -91,7 +102,9 @@ struct _sql_dbcursor_s {
 
     int count;
 
+/*@null@*/
     void * lkey;		/* Last key returned */
+/*@null@*/
     void * ldata;		/* Last data returned */
 
     int used;
@@ -99,7 +112,7 @@ struct _sql_dbcursor_s {
 
 /*@-redef@*/
 union _dbswap {
-    uint32_t ui;
+    rpmuint32_t ui;
     unsigned char uc[4];
 };
 /*@=redef@*/
@@ -122,7 +135,7 @@ static void enterChroot(dbiIndex dbi)
 	/*@globals sqlCwd, sqlInRoot, internalState @*/
 	/*@modifies sqlCwd, sqlInRoot, internalState @*/
 {
-    char * currDir = NULL; 
+    char * currDir = NULL;
     int xx;
 
     if ((dbi->dbi_root[0] == '/' && dbi->dbi_root[1] == '\0') || dbi->dbi_rpmdb->db_chrootDone || sqlInRoot)
@@ -215,7 +228,7 @@ if (!_debug) return;
 	dbg_scp(dbcursor);
 }
 
-/*@only@*/ 
+/*@only@*/
 static SCP_t scpResetKeys(/*@only@*/ SCP_t scp)
 	/*@modifies scp @*/
 {
@@ -237,7 +250,7 @@ dbg_scp(scp);
     return scp;
 }
 
-/*@only@*/ 
+/*@only@*/
 static SCP_t scpResetAv(/*@only@*/ SCP_t scp)
 	/*@modifies scp @*/
 {
@@ -276,7 +289,7 @@ dbg_scp(scp);
     return scp;
 }
 
-/*@only@*/ 
+/*@only@*/
 static SCP_t scpReset(/*@only@*/ SCP_t scp)
 	/*@modifies scp @*/
 {
@@ -359,7 +372,7 @@ static int sql_step(dbiIndex dbi, SCP_t scp)
     /* XXX scp->nc = need = scp->nalloc = 0 case forces + 1 here */
     if (!scp->ac && !need && !scp->nalloc)
 	need++;
-   
+
     if (scp->ac + need >= scp->nalloc) {
 	/* XXX +4 is bogus, was +1 */
 	scp->nalloc = 2 * scp->nalloc + need + 4;
@@ -428,7 +441,7 @@ assert(swapped == 0); /* Byte swap?! */
 		    }
 		} else
 		if (!strcmp(vtype, "int")) {
-		    int32_t v = sqlite3_column_int(scp->pStmt, i);
+		    rpmint32_t v = sqlite3_column_int(scp->pStmt, i);
 		    nb = sizeof(v);
 if (_debug)
 fprintf(stderr, "\t%d %s %s %d\n", i, cname, vtype, (int) v);
@@ -476,7 +489,7 @@ assert(scp->ac <= scp->nalloc);
 	    /*@switchbreak@*/ break;
 	case SQLITE_ERROR:
 	    fprintf(stderr, "sqlite3_step: ERROR %d -- %s\n", rc, scp->cmd);
-	    fprintf(stderr, "              %s (%d)\n", 
+	    fprintf(stderr, "              %s (%d)\n",
 			sqlite3_errmsg(((SQL_DB*)dbi->dbi_db)->db), sqlite3_errcode(((SQL_DB*)dbi->dbi_db)->db));
 /*@-nullpass@*/
 	    fprintf(stderr, "              cwd '%s'\n", getcwd(NULL,0));
@@ -512,7 +525,7 @@ assert(key->data != NULL);
     switch (dbi->dbi_rpmtag) {
     case RPMDBI_PACKAGES:
 	{   unsigned int hnum;
-/*@i@*/ assert(key->size == sizeof(uint32_t));
+/*@i@*/ assert(key->size == sizeof(rpmuint32_t));
 	    memcpy(&hnum, key->data, sizeof(hnum));
 
 if (swapped == 1) {
@@ -538,7 +551,7 @@ assert(swapped == 0); /* Byte swap?! */
 	} /*@innerbreak@*/ break;
 	case RPM_UINT16_TYPE:
 	{	unsigned short i;
-/*@i@*/ assert(key->size == sizeof(uint16_t));
+/*@i@*/ assert(key->size == sizeof(rpmuint16_t));
 assert(swapped == 0); /* Byte swap?! */
 	    memcpy(&i, key->data, sizeof(i));
 	    rc = sqlite3_bind_int(scp->pStmt, pos, (int) i);
@@ -549,7 +562,7 @@ assert(0);	/* borken */
         case RPM_UINT32_TYPE:
 	default:
 	{   unsigned int i;
-/*@i@*/ assert(key->size == sizeof(uint32_t));
+/*@i@*/ assert(key->size == sizeof(rpmuint32_t));
 	    memcpy(&i, key->data, sizeof(i));
 
 if (swapped == 1)
@@ -573,7 +586,8 @@ if (swapped == 1)
     return rc;
 }
 
-static int sql_bind_data(dbiIndex dbi, SCP_t scp, int pos, DBT * data)
+static int sql_bind_data(/*@unused@*/ dbiIndex dbi, SCP_t scp,
+		int pos, DBT * data)
 	/*@modifies scp @*/
 {
     int rc;
@@ -683,7 +697,8 @@ static int sql_busy_handler(void * dbi_void, int time)
  * Create the table.. create the db_info
  */
 static int sql_initDB(dbiIndex dbi)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies internalState @*/
 {
     SQL_DB * sqldb = (SQL_DB *) dbi->dbi_db;
     SCP_t scp = scpNew(dbi->dbi_db);
@@ -796,13 +811,13 @@ exit:
     return rc;
 }
 
-/**   
+/**
  * Close database cursor.
  * @param dbi           index database handle
  * @param dbcursor      database cursor
  * @param flags         (unused)
  * @return              0 on success
- */   
+ */
 static int sql_cclose (dbiIndex dbi, /*@only@*/ DBC * dbcursor,
 		unsigned int flags)
 	/*@globals fileSystem, internalState @*/
@@ -887,6 +902,7 @@ leaveChroot(dbi);
  * Return handle for an index database.
  * @param rpmdb         rpm database
  * @param rpmtag        rpm tag
+ * @retval *dbip	index database handle
  * @return              0 on success
  */
 static int sql_open(rpmdb rpmdb, rpmTag rpmtag, /*@out@*/ dbiIndex * dbip)
@@ -896,12 +912,12 @@ static int sql_open(rpmdb rpmdb, rpmTag rpmtag, /*@out@*/ dbiIndex * dbip)
 /*@-nestedextern -shadow @*/
     extern struct _dbiVec sqlitevec;
 /*@=nestedextern -shadow @*/
-   
+
     const char * urlfn = NULL;
     const char * root;
     const char * home;
     const char * dbhome;
-    const char * dbfile;  
+    const char * dbfile;
     const char * dbfname;
     const char * sql_errcode;
     mode_t umask_safed = 0002;
@@ -910,7 +926,7 @@ static int sql_open(rpmdb rpmdb, rpmTag rpmtag, /*@out@*/ dbiIndex * dbip)
     size_t len;
     int rc = 0;
     int xx;
-    
+
     if (dbip)
 	*dbip = NULL;
 
@@ -929,10 +945,10 @@ static int sql_open(rpmdb rpmdb, rpmTag rpmtag, /*@out@*/ dbiIndex * dbip)
      */
     root = rpmdb->db_root;
     home = rpmdb->db_home;
-    
+
     dbi->dbi_root = root;
     dbi->dbi_home = home;
-      
+
     dbfile = tagName(dbi->dbi_rpmtag);
 
 enterChroot(dbi);
@@ -953,7 +969,7 @@ enterChroot(dbi);
     }
 
     dbi->dbi_mode = O_RDWR;
-       
+
     /*
      * Either the root or directory components may be a URL. Concatenate,
      * convert the URL to a path, and add the name of the file.
@@ -963,11 +979,11 @@ enterChroot(dbi);
     /*@=mods@*/
     (void) urlPath(urlfn, &dbhome);
 
-    /* 
+    /*
      * Create the /var/lib/rpm directory if it doesn't exist (root only).
      */
     (void) rpmioMkpath(dbhome, 0755, getuid(), getgid());
-       
+
     if (dbi->dbi_eflags & DB_PRIVATE)
         dbfname = xstrdup(":memory:");
     else
@@ -978,18 +994,22 @@ enterChroot(dbi);
 
     /* Open the Database */
     sqldb = xcalloc(1, sizeof(*sqldb));
-       
+
     sql_errcode = NULL;
+/*@+longunsignedintegral@*/
     if (dbi->dbi_perms)
         /* mask-out permission bits which are not requested (security) */
         umask_safed = umask(~((mode_t)(dbi->dbi_perms)));
+/*@=longunsignedintegral@*/
     xx = sqlite3_open(dbfname, &sqldb->db);
     if (dbi->dbi_perms) {
         if ((0644 /* = SQLite hard-coded default */ & dbi->dbi_perms) != dbi->dbi_perms) {
             /* add requested permission bits which are still missing (semantic) */
             (void) Chmod(dbfname, dbi->dbi_perms);
         }
+/*@+longunsignedintegral@*/
         (void) umask(umask_safed);
+/*@=longunsignedintegral@*/
     }
     if (xx != SQLITE_OK)
 	sql_errcode = sqlite3_errmsg(sqldb->db);
@@ -1016,14 +1036,14 @@ enterChroot(dbi);
     } else {
 	(void) sql_close(dbi, 0);
     }
- 
+
     urlfn = _free(urlfn);
     dbfname = _free(dbfname);
 
 /*@-usereleased@*/
 leaveChroot(dbi);
 /*@=usereleased@*/
-   
+
     return rc;
 }
 
@@ -1033,7 +1053,7 @@ leaveChroot(dbi);
  * @param flags         (unused)
  * @return              0 on success
  */
-static int sql_sync (dbiIndex dbi, unsigned int flags)
+static int sql_sync (dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
@@ -1054,7 +1074,8 @@ leaveChroot(dbi);
  * @param flags		DB_WRITECURSOR or 0
  * @return		0 on success
  */
-static int sql_copen (dbiIndex dbi, /*@null@*/ DB_TXN * txnid,
+static int sql_copen (dbiIndex dbi,
+		/*@unused@*/ /*@null@*/ DB_TXN * txnid,
 		/*@out@*/ DBC ** dbcp, unsigned int flags)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *txnid, *dbcp, fileSystem, internalState @*/
@@ -1078,14 +1099,14 @@ enterChroot(dbi);
 	/*@-kepttrans -nullstate @*/ (void) sql_cclose(dbi, dbcursor, 0); /*@=kepttrans =nullstate @*/
 
 leaveChroot(dbi);
-     
+
     return rc;
 }
 
 /**
  * Delete (key,data) pair(s) using db->del or dbcursor->c_del.
  * @param dbi           index database handle
- * @param dbcursor      database cursor (NULL will use db->del)   
+ * @param dbcursor      database cursor (NULL will use db->del)
  * @param key           delete key value/length/flags
  * @param data          delete data value/length/flags
  * @param flags         (unused)
@@ -1166,13 +1187,15 @@ fprintf(stderr, "\tcget(%s) scp %p rc %d flags %d av %p\n",
         if ( key->size == 0) {
 	    scp->all = 1;
 
-/* 
+/*
  * The only condition not dealt with is if there are multiple identical keys.  This can lead
  * to later iteration confusion.  (It may return the same value for the multiple keys.)
  */
 
+#ifdef	DYING
 /* Only RPMDBI_PACKAGES is supposed to be iterating, and this is guarenteed to be unique */
 assert(dbi->dbi_rpmtag == RPMDBI_PACKAGES);
+#endif
 
 	    switch (dbi->dbi_rpmtag) {
 	    case RPMDBI_PACKAGES:
@@ -1195,7 +1218,7 @@ assert(dbi->dbi_rpmtag == RPMDBI_PACKAGES);
 	    scp->keys = xcalloc(scp->nkeys, sizeof(*scp->keys));
 	    for (ix = 0; ix < scp->nkeys; ix++) {
 		scp->keys[ix] = xmalloc(sizeof(*scp->keys[0]));
-		scp->keys[ix]->size = (u_int32_t) scp->avlen[ix+1];
+		scp->keys[ix]->size = (UINT32_T) scp->avlen[ix+1];
 		scp->keys[ix]->data = xmalloc(scp->keys[ix]->size);
 		memcpy(scp->keys[ix]->data, scp->av[ix+1], scp->avlen[ix+1]);
 	    }
@@ -1274,7 +1297,7 @@ assert(scp->nr == 1);
 	    scp->ldata = _free(scp->ldata);
 	}
 
-	data->size = (u_int32_t) scp->avlen[1];
+	data->size = (UINT32_T) scp->avlen[1];
         data->data = xmalloc(data->size);
 	if (! (data->flags & DB_DBT_MALLOC) )
 	    scp->ldata = data->data;
@@ -1351,7 +1374,7 @@ leaveChroot(dbi);
 
 /**
  * Is database byte swapped?
- * @param dbi           index database handle   
+ * @param dbi           index database handle
  * @return              0 no
  */
 static int sql_byteswapped (dbiIndex dbi)
@@ -1409,9 +1432,10 @@ leaveChroot(dbi);
  * @param flags         DB_CREATE or 0
  * @return              0 on success
  */
-static int sql_associate (dbiIndex dbi, dbiIndex dbisecondary,
-		int (*callback) (DB *, const DBT *, const DBT *, DBT *),
-		unsigned int flags)
+static int sql_associate (/*@unused@*/ dbiIndex dbi,
+		/*@unused@*/ dbiIndex dbisecondary,
+    /*@unused@*/int (*callback) (DB *, const DBT *, const DBT *, DBT *),
+		/*@unused@*/ unsigned int flags)
 	/*@*/
 {
 if (_debug)
@@ -1427,8 +1451,10 @@ fprintf(stderr, "*** sql_associate:\n");
  * @param flags         DB_JOIN_NOSORT or 0
  * @return              0 on success
  */
-static int sql_join (dbiIndex dbi, DBC ** curslist, /*@out@*/ DBC ** dbcp,
-		unsigned int flags)
+static int sql_join (/*@unused@*/ dbiIndex dbi,
+		/*@unused@*/ DBC ** curslist,
+		/*@unused@*/ /*@out@*/ DBC ** dbcp,
+		/*@unused@*/ unsigned int flags)
 	/*@globals fileSystem @*/
 	/*@modifies dbi, *dbcp, fileSystem @*/
 {
@@ -1438,15 +1464,17 @@ fprintf(stderr, "*** sql_join:\n");
 }
 
 /**
- * Duplicate a database cursor.   
+ * Duplicate a database cursor.
  * @param dbi           index database handle
  * @param dbcursor      database cursor
  * @retval dbcp         address of new database cursor
  * @param flags         DB_POSITION for same position, 0 for uninitialized
  * @return              0 on success
  */
-static int sql_cdup (dbiIndex dbi, DBC * dbcursor, /*@out@*/ DBC ** dbcp,
-		unsigned int flags)
+static int sql_cdup (/*@unused@*/ dbiIndex dbi,
+		/*@unused@*/ DBC * dbcursor,
+		/*@unused@*/ /*@out@*/ DBC ** dbcp,
+		/*@unused@*/ unsigned int flags)
 	/*@globals fileSystem @*/
 	/*@modifies dbi, *dbcp, fileSystem @*/
 {
@@ -1461,12 +1489,16 @@ fprintf(stderr, "*** sql_cdup:\n");
  * @param dbcursor      database cursor
  * @param key           secondary retrieve key value/length/flags
  * @param pkey          primary retrieve key value/length/flags
- * @param data          primary retrieve data value/length/flags 
+ * @param data          primary retrieve data value/length/flags
  * @param flags         DB_NEXT, DB_SET, or 0
  * @return              0 on success
  */
-static int sql_cpget (dbiIndex dbi, /*@null@*/ DBC * dbcursor,
-		DBT * key, DBT * pkey, DBT * data, unsigned int flags)
+static int sql_cpget (/*@unused@*/ dbiIndex dbi,
+		/*@unused@*/ /*@null@*/ DBC * dbcursor,
+		/*@unused@*/ DBT * key,
+		/*@unused@*/ DBT * pkey,
+		/*@unused@*/ DBT * data,
+		/*@unused@*/ unsigned int flags)
 	/*@globals fileSystem @*/
 	/*@modifies *dbcursor, *key, *pkey, *data, fileSystem @*/
 {
@@ -1483,7 +1515,8 @@ fprintf(stderr, "*** sql_cpget:\n");
  * @param flags         (unused)
  * @return              0 on success
  */
-static int sql_ccount (dbiIndex dbi, /*@unused@*/ DBC * dbcursor,   
+static int sql_ccount (/*@unused@*/ dbiIndex dbi,
+		/*@unused@*/ DBC * dbcursor,
 		/*@unused@*/ /*@out@*/ unsigned int * countp,
 		/*@unused@*/ unsigned int flags)
 	/*@globals fileSystem @*/
@@ -1500,7 +1533,7 @@ fprintf(stderr, "*** sql_ccount:\n");
  * @param flags         retrieve statistics that don't require traversal?
  * @return              0 on success
  */
-static int sql_stat (dbiIndex dbi, unsigned int flags)
+static int sql_stat (dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, fileSystem, internalState @*/
 {
@@ -1555,15 +1588,15 @@ leaveChroot(dbi);
 /* db_bytewapped, stat */
 /*@observer@*/ /*@unchecked@*/
 struct _dbiVec sqlitevec = {
-    0, 0, 0, 
-    sql_open, 
+    0, 0, 0,
+    sql_open,
     sql_close,
-    sql_sync,  
-    sql_associate,  
+    sql_sync,
+    sql_associate,
     sql_join,
     sql_copen,
     sql_cclose,
-    sql_cdup, 
+    sql_cdup,
     sql_cdel,
     sql_cget,
     sql_cpget,
