@@ -21,8 +21,8 @@
  */
 static int tagLoadATags(/*@null@*/ ARGV_t * argvp,
 		int (*cmp) (const void * avp, const void * bvp))
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies *argvp, rpmGlobalMacroContext @*/
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies *argvp, rpmGlobalMacroContext, internalState @*/
 {
     ARGV_t aTags = NULL;
     char * s = rpmExpand("%{?_arbitrary_tags}", NULL);
@@ -97,7 +97,7 @@ static int tagLoadIndex(headerTagTableEntry ** ipp, size_t * np,
 	ip[n] = tte;
 	n++;
     }
-assert(n == rpmTagTableSize);
+assert(n == (size_t)rpmTagTableSize);
 /*@=dependenttrans@*/ /*@=observertrans@*/ /*@=castexpose@*/ /*@=mods@*/ /*@=modobserver@*/
 
     if (n > 1)
@@ -157,14 +157,14 @@ static rpmTag _tagGenerate(const char *s)
 
 /* forward refs */
 static const char * _tagName(rpmTag tag)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies rpmGlobalMacroContext @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies rpmGlobalMacroContext, internalState @*/;
 static unsigned int _tagType(rpmTag tag)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies rpmGlobalMacroContext @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies rpmGlobalMacroContext, internalState @*/;
 static rpmTag _tagValue(const char * tagstr)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies rpmGlobalMacroContext @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies rpmGlobalMacroContext, internalState @*/;
 
 /*@unchecked@*/
 static struct headerTagIndices_s _rpmTags = {
@@ -237,6 +237,11 @@ static const char * _tagName(rpmTag tag)
 	strncpy(nameBuf, "Sha1header", nameBufLen);
 	break;
 
+    /* XXX make sure that h.['filenames'] in python "works". */
+    case 0x54aafb71:
+	strncpy(nameBuf, "Filenames", nameBufLen);
+	break;
+
     default:
 	if (_rpmTags.byValue == NULL)
 	    break;
@@ -256,9 +261,8 @@ static const char * _tagName(rpmTag tag)
 		/* Make sure that the bsearch retrieve is stable. */
 		while (i > 0 && tag == _rpmTags.byValue[i-1]->val) {
 		    i--;
-		    t--;
+		    t = _rpmTags.byValue[i];
 		}
-		t = _rpmTags.byValue[i];
 		s = (*_rpmTags.tagCanonicalize) (t->name);
 		strncpy(nameBuf, s, nameBufLen);
 		s = _free(s);
@@ -317,7 +321,7 @@ static unsigned int _tagType(rpmTag tag)
 		/* Make sure that the bsearch retrieve is stable. */
 		while (i > 0 && t->val == _rpmTags.byValue[i-1]->val) {
 		    i--;
-		    t--;
+		    t = _rpmTags.byValue[i];
 		}
 		return t->type;
 	    }
@@ -431,7 +435,7 @@ tagStore_t tagStoreFree(tagStore_t dbiTags, size_t dbiNTags)
 	size_t i;
 	for (i = 0; i < dbiNTags; i++) {
 	    dbiTags[i].str = _free(dbiTags[i].str);
-	    dbiTags[i].val = freeStringBuf(dbiTags[i].val);
+	    dbiTags[i].iob = rpmiobFree(dbiTags[i].iob);
 	}
 	dbiTags = _free(dbiTags);
     }
