@@ -14,10 +14,20 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_BLINKNEVRA",	1166,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BLINKPKGID",	1164,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BUILDARCHS",	1089,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDBUILD",	0x5bae1a5a,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDBUILDPROG",	0x6fb46014,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDCHECK",	0x7f3b97b5,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDCHECKPROG",	0x488a60ce,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDCLEAN",	0x566042bf,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDCLEANPROG",	0x42c93d41,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BUILDCPUCLOCK",	1176,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_BUILDHOST",	1007,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_BUILDINSTALL",	0x567f5983,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDINSTALLPROG",	0x70d4ab6f,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BUILDMACROS",	1111,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BUILDPLATFORMS",	1183,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDPREP",	0x799c0b4d,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_BUILDPREPPROG",	0x4ba37c9e,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BUILDTIME",	1006,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_C",	RPMTAG_CONFLICTNAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
 	{ "RPMTAG_CACHECTIME",	1136,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
@@ -37,12 +47,14 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_CONFLICTVERSION",	1055,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_COOKIE",	1094,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_CVSID",	1163,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_D",	RPMTAG_DISTEPOCH,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE  },
 	{ "RPMTAG_DBINSTANCE",	1195,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_DEPATTRSDICT",	1188,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_DEPENDSDICT",	1145,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_DESCRIPTION",	1005,	RPM_I18NSTRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_DIRINDEXES",	1116,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_DIRNAMES",	1118,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_DISTEPOCH",	1218,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_DISTRIBUTION",	1010,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_DISTTAG",	1155,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_DISTURL",	1123,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
@@ -63,6 +75,7 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_FILEDEPENDSN",	1144,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEDEPENDSX",	1143,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEDEVICES",	1095,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_FILEDIGESTALGO",	5011,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_FILEDIGESTALGOS",	1177,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEDIGESTS",	1035,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEFLAGS",	1037,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
@@ -76,6 +89,7 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_FILEPATHS",	1197,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILERDEVS",	1033,	RPM_UINT16_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILESIZES",	1028,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_FILESTAT",	1207,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILESTATES",	1029,	RPM_UINT8_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEUSERNAME",	1039,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEVERIFYFLAGS",	1045,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
@@ -87,12 +101,14 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_GIF",	1012,	RPM_BIN_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_GROUP",	1016,	RPM_I18NSTRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_HDRID",	RPMTAG_SHA1HEADER,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE  },
+	{ "RPMTAG_HEADERENDOFF",	1212,	RPM_UINT64_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_HEADERI18NTABLE",	HEADER_I18NTABLE,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_HEADERIMMUTABLE",	HEADER_IMMUTABLE,	RPM_BIN_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_HEADERSTARTOFF",	1211,	RPM_UINT64_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_ICON",	1043,	RPM_BIN_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_INSTALLCOLOR",	1127,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
-	{ "RPMTAG_INSTALLTID",	1128,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
-	{ "RPMTAG_INSTALLTIME",	1008,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_INSTALLTID",	1128,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_INSTALLTIME",	1008,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_INSTPREFIXES",	1099,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_KEYWORDS",	1182,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_LICENSE",	1014,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
@@ -110,13 +126,22 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_ORIGBASENAMES",	1120,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_ORIGDIRINDEXES",	1119,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_ORIGDIRNAMES",	1121,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_ORIGINTID",	1209,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_ORIGINTIME",	1210,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_ORIGPATHS",	1198,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_OS",	1021,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_P",	RPMTAG_PROVIDENAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
+	{ "RPMTAG_PACKAGEBASEURL",	1217,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PACKAGECOLOR",	1184,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_PACKAGEDEPFLAGS",	0x748a8314,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_PACKAGEDIGEST",	1215,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PACKAGEORIGIN",	1170,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PACKAGEPREFCOLOR",	1185,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PACKAGER",	1015,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_PACKAGESIZE",	1214,	RPM_UINT64_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_PACKAGESTAT",	1216,	RPM_BIN_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_PACKAGETIME",	1213,	RPM_UINT64_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_PACKAGETRANSFLAGS",	0x4efaafd9,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_PATCH",	1019,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_PATCHESFLAGS",	1134,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_PATCHESNAME",	1133,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
@@ -150,7 +175,7 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_R",	RPMTAG_RELEASE,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE  },
 	{ "RPMTAG_RECONTEXTS",	1149,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_RELEASE",	1002,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
-	{ "RPMTAG_REMOVETID",	1129,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_REMOVETID",	1129,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_REPOTAG",	1181,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_REQUIREATTRSX",	1192,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_REQUIREFLAGS",	1048,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
@@ -177,6 +202,7 @@ static const struct headerTagTableEntry_s _rpmTagTable[] = {
 	{ "RPMTAG_SOURCEPACKAGE",	1106,	RPM_UINT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_SOURCEPKGID",	1146,	RPM_BIN_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_SOURCERPM",	1044,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_STAT",	1208,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_SUGGESTS",	RPMTAG_SUGGESTSNAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
 	{ "RPMTAG_SUGGESTSFLAGS",	1158,	RPM_UINT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_SUGGESTSNAME",	1156,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
