@@ -9,6 +9,8 @@
 
 #include <db.h>
 
+#include <rpmtag.h>
+
 #define RPMDB_HOME	"/tmp/rpm"
 #define	RPMDB_Packages	"Packages"
 #define	RPMDB_Dirnames	"Dirnames"
@@ -33,7 +35,7 @@ static int
 db_init(const char * home, DB_ENV ** dbenvp)
 {
     DB_ENV * dbenv;
-    uint32_t flags;
+    rpmuint32_t flags;
     int ret;
 
     if ((ret = db_env_create(&dbenv, 0)) != 0)
