@@ -22,7 +22,7 @@ void argvPrint(const char * msg, ARGV_t argv, FILE * fp)
 
     if (argv)
     for (av = argv; *av; av++)
-	fprintf(fp, "%s\n", *av);
+	fprintf(fp, "\t%s\n", *av);
 
 }
 
@@ -79,6 +79,14 @@ ARGV_t argvData(ARGV_t argv)
 /*@=retalias =temptrans @*/
 }
 
+int argiCmp(const void * a, const void * b)
+{
+    unsigned aint = *(ARGint_t)a;
+    unsigned bint = *(ARGint_t)b;
+    return ((aint < bint) ? -1 :
+	    (aint > bint) ? +1 : 0);
+}
+
 int argvCmp(const void * a, const void * b)
 {
     ARGstr_t astr = *(ARGV_t)a;
@@ -109,6 +117,17 @@ int argvFnmatchCasefold(const void * a, const void * b)
 }
 #endif
 
+int argiSort(ARGI_t argi, int (*compar)(const void *, const void *))
+{
+    unsigned nvals = argiCount(argi);
+    ARGint_t vals = argiData(argi);
+    if (compar == NULL)
+	compar = argiCmp;
+    if (nvals > 1)
+	qsort(vals, nvals, sizeof(*vals), compar);
+    return 0;
+}
+
 int argvSort(ARGV_t argv, int (*compar)(const void *, const void *))
 {
     if (compar == NULL)
@@ -159,7 +178,7 @@ int argiAdd(/*@out@*/ ARGI_t * argip, int ix, int val)
     argi = *argip;
     if (ix < 0)
 	ix = argi->nvals;
-    if (ix >= argi->nvals) {
+    if (ix >= (int)argi->nvals) {
 	argi->vals = xrealloc(argi->vals, (ix + 1) * sizeof(*argi->vals));
 	memset(argi->vals + argi->nvals, 0,
 		(ix - argi->nvals) * sizeof(*argi->vals));
@@ -186,18 +205,20 @@ int argvAdd(/*@out@*/ ARGV_t * argvp, ARGstr_t val)
     return 0;
 }
 
-int argvAppend(/*@out@*/ ARGV_t * argvp, const ARGV_t av)
+int argvAppend(/*@out@*/ ARGV_t * argvp, ARGV_t av)
 {
-    ARGV_t argv = *argvp;
-    int argc = argvCount(argv);
     int ac = argvCount(av);
-    int i;
 
-    argv = xrealloc(argv, (argc + ac + 1) * sizeof(*argv));
-    for (i = 0; i < ac; i++)
-	argv[argc + i] = xstrdup(av[i]);
-    argv[argc + ac] = NULL;
-    *argvp = argv;
+    if (av != NULL && ac > 0) {
+	ARGV_t argv = *argvp;
+	int argc = argvCount(argv);
+
+	argv = xrealloc(argv, (argc + ac + 1) * sizeof(*argv));
+	while (*av++)
+	    argv[argc++] = xstrdup(av[-1]);
+	argv[argc] = NULL;
+	*argvp = argv;
+    }
     return 0;
 }
 
@@ -232,7 +253,11 @@ int argvSplit(ARGV_t * argvp, const char * str, const char * seps)
 	argv[c++] = xstrdup(s);
     }
     argv[c] = NULL;
-    *argvp = argv;
+    if (argvp)
+	*argvp = argv;
+    else
+	argv = argvFree(argv);
+    dest = _free(dest);
 /*@-nullstate@*/
     return 0;
 /*@=nullstate@*/
@@ -262,6 +287,7 @@ char * argvJoin(ARGV_t argv)
     return t;
 }
 
+/*@-mustmod@*/
 int argvFgets(ARGV_t * argvp, void * fd)
 {
     FILE * fp = (fd ? fdGetFILE(fd) : stdin);
@@ -274,7 +300,8 @@ int argvFgets(ARGV_t * argvp, void * fd)
 	return -2;
     while (!rc && (b = fgets(buf, (int)sizeof(buf), fp)) != NULL) {
 	buf[sizeof(buf)-1] = '\0';
-	be = b + strlen(buf) - 1;
+	be = b + strlen(buf);
+	if (be > b) be--;
 	while (strchr("\r\n", *be) != NULL)
 	    *be-- = '\0';
 	rc = argvAdd(&av, b);
@@ -293,3 +320,4 @@ int argvFgets(ARGV_t * argvp, void * fd)
     return rc;
 /*@=nullstate@*/
 }
+/*@=mustmod@*/
