@@ -4,12 +4,11 @@
 
 #include "system.h"
 
-#include <zlib.h>
-
 #include "rpmio_internal.h"
 
 #include <rpmbc.h>
 
+#include "crc.h"
 #include "md2.h"
 #include "md4.h"
 #include "sha224.h"
@@ -29,278 +28,10 @@
 #define	DPRINTF(_a)
 #endif
 
-#if !defined(ZLIB_H) || defined(__LCLINT__)
-/**
- */
-/*@-shadow@*/
-static uint32_t crc32(uint32_t crc, const byte * data, size_t size)
-	/*@*/
-{
-    static uint32_t polynomial = 0xedb88320;    /* reflected 0x04c11db7 */
-    static uint32_t xorout = 0xffffffff;
-    static uint32_t table[256];
-
-    crc ^= xorout;
-
-    if (data == NULL) {
-	/* generate the table of CRC remainders for all possible bytes */
-	uint32_t c;
-	uint32_t i, j;
-	for (i = 0;  i < 256;  i++) {
-	    c = i;
-	    for (j = 0;  j < 8;  j++) {
-		if (c & 1)
-		    c = polynomial ^ (c >> 1);
-		else
-		    c = (c >> 1);
-	    }
-	    table[i] = c;
-	}
-    } else
-    while (size) {
-	crc = table[(crc ^ *data) & 0xff] ^ (crc >> 8);
-	size--;
-	data++;
-    }
-
-    crc ^= xorout;
-
-    return crc;
-
-}
-/*@=shadow@*/
-#endif
-
 /* Include Bob Jenkins lookup3 hash */
 #define	_JLU3_jlu32l
 #include "lookup3.c"
 
-/**
- */
-typedef struct {
-	uint32_t crc;
-	uint32_t (*update)  (uint32_t crc, const byte * data, size_t size);
-	uint32_t (*combine) (uint32_t crc1, uint32_t crc2, size_t len2);
-} sum32Param;
-
-/**
- */
-static int sum32Reset(register sum32Param* mp)
-	/*@modifies *mp @*/
-{
-    if (mp->update)
-	mp->crc = (*mp->update) (0, NULL, 0);
-    return 0;
-}
-
-/**
- */
-static int sum32Update(sum32Param* mp, const byte* data, size_t size)
-	/*@modifies *mp @*/
-{
-    if (mp->update)
-	mp->crc = (*mp->update) (mp->crc, data, size);
-    return 0;
-}
-
-/**
- */
-static int sum32Digest(sum32Param* mp, byte* data)
-	/*@modifies *mp, data @*/
-{
-	uint32_t c = mp->crc;
-
-	data[ 0] = (byte)(c >> 24);
-	data[ 1] = (byte)(c >> 16);
-	data[ 2] = (byte)(c >>  8);
-	data[ 3] = (byte)(c      );
-
-	(void) sum32Reset(mp);
-
-	return 0;
-}
-
-/*
- * ECMA-182 polynomial, see
- *     http://www.ecma-international.org/publications/files/ECMA-ST/Ecma-182.pdf
- */
-/**
- */
-static uint64_t crc64(uint64_t crc, const byte * data, size_t size)
-	/*@*/
-{
-    static uint64_t polynomial =
-	0xc96c5795d7870f42ULL;	/* reflected 0x42f0e1eba9ea3693ULL */
-    static uint64_t xorout = 0xffffffffffffffffULL;
-    static uint64_t table[256];
-
-    crc ^= xorout;
-
-    if (data == NULL) {
-	/* generate the table of CRC remainders for all possible bytes */
-	uint64_t c;
-	uint32_t i, j;
-	for (i = 0;  i < 256;  i++) {
-	    c = i;
-	    for (j = 0;  j < 8;  j++) {
-		if (c & 1)
-		    c = polynomial ^ (c >> 1);
-		else
-		    c = (c >> 1);
-	    }
-	    table[i] = c;
-	}
-    } else
-    while (size) {
-	crc = table[(crc ^ *data) & 0xff] ^ (crc >> 8);
-	size--;
-	data++;
-    }
-
-    crc ^= xorout;
-
-    return crc;
-
-}
-
-/*
- * Swiped from zlib, using uint64_t rather than unsigned long computation.
- * Use at your own risk, uint64_t problems with compilers may exist.
- */
-#define	GF2_DIM	64
-
-/**
- */
-static uint64_t gf2_matrix_times(uint64_t *mat, uint64_t vec)
-	/*@*/
-{
-    uint64_t sum;
-
-    sum = 0;
-    while (vec) {
-        if (vec & 1)
-            sum ^= *mat;
-        vec >>= 1;
-        mat++;
-    }
-    return sum;
-}
-
-/**
- */
-static void gf2_matrix_square(/*@out@*/ uint64_t *square, uint64_t *mat)
-	/*@modifies square @*/
-{
-    int n;
-
-    for (n = 0; n < GF2_DIM; n++)
-        square[n] = gf2_matrix_times(mat, mat[n]);
-}
-
-/**
- */
-static uint64_t crc64_combine(uint64_t crc1, uint64_t crc2, size_t len2)
-	/*@*/
-{
-    int n;
-    uint64_t row;
-    uint64_t even[GF2_DIM];    /* even-power-of-two zeros operator */
-    uint64_t odd[GF2_DIM];     /* odd-power-of-two zeros operator */
-
-    /* degenerate case */
-    if (len2 == 0)
-        return crc1;
-
-    /* put operator for one zero bit in odd */
-    odd[0] = 0xc96c5795d7870f42ULL;	/* reflected 0x42f0e1eba9ea3693ULL */
-    row = 1;
-    for (n = 1; n < GF2_DIM; n++) {
-        odd[n] = row;
-        row <<= 1;
-    }
-
-    /* put operator for two zero bits in even */
-    gf2_matrix_square(even, odd);
-
-    /* put operator for four zero bits in odd */
-    gf2_matrix_square(odd, even);
-
-    /* apply len2 zeros to crc1 (first square will put the operator for one
-       zero byte, eight zero bits, in even) */
-    do {
-        /* apply zeros operator for this bit of len2 */
-        gf2_matrix_square(even, odd);
-        if (len2 & 1)
-            crc1 = gf2_matrix_times(even, crc1);
-        len2 >>= 1;
-
-        /* if no more bits set, then done */
-        if (len2 == 0)
-            break;
-
-        /* another iteration of the loop with odd and even swapped */
-        gf2_matrix_square(odd, even);
-        if (len2 & 1)
-            crc1 = gf2_matrix_times(odd, crc1);
-        len2 >>= 1;
-
-        /* if no more bits set, then done */
-    } while (len2 != 0);
-
-    /* return combined crc */
-    crc1 ^= crc2;
-    return crc1;
-}
-
-/**
- */
-typedef struct {
-	uint64_t crc;
-	uint64_t (*update)  (uint64_t crc, const byte * data, size_t size);
-	uint64_t (*combine) (uint64_t crc1, uint64_t crc2, size_t len2);
-} sum64Param;
-
-/**
- */
-static int sum64Reset(register sum64Param* mp)
-	/*@modifies *mp @*/
-{
-    if (mp->update)
-	mp->crc = (*mp->update) (0, NULL, 0);
-    return 0;
-}
-
-/**
- */
-static int sum64Update(sum64Param* mp, const byte* data, size_t size)
-	/*@modifies *mp @*/
-{
-    if (mp->update)
-	mp->crc = (*mp->update) (mp->crc, data, size);
-    return 0;
-}
-
-/**
- */
-static int sum64Digest(sum64Param* mp, byte* data)
-	/*@modifies *mp, data @*/
-{
-	uint64_t c = mp->crc;
-
-	data[ 0] = (byte)(c >> 56);
-	data[ 1] = (byte)(c >> 48);
-	data[ 2] = (byte)(c >> 40);
-	data[ 3] = (byte)(c >> 32);
-	data[ 4] = (byte)(c >> 24);
-	data[ 5] = (byte)(c >> 16);
-	data[ 6] = (byte)(c >>  8);
-	data[ 7] = (byte)(c      );
-
-	(void) sum64Reset(mp);
-
-	return 0;
-}
-
 /*@access DIGEST_CTX@*/
 
 /**
@@ -318,10 +49,29 @@ struct DIGEST_CTX_s {
 	/*@modifies param @*/;	/*!< Digest transform. */
     int (*Digest) (void * param, /*@out@*/ byte * digest)
 	/*@modifies param, digest @*/;	/*!< Digest finish. */
+    pgpHashAlgo hashalgo;	/*!< RFC 2440/4880 hash algorithm id. */
     rpmDigestFlags flags;	/*!< Bit(s) to control digest operation. */
+/*@observer@*/ /*@null@*/
+    const char * asn1;		/*!< RFC 3447 ASN1 oid string (in hex). */
     void * param;		/*!< Digest parameters. */
 };
 
+pgpHashAlgo rpmDigestAlgo(DIGEST_CTX ctx)
+{
+    return (ctx != NULL ? ctx->hashalgo : PGPHASHALGO_NONE);
+}
+
+const char * rpmDigestName(DIGEST_CTX ctx)
+{
+    return (ctx != NULL ? ctx->name : "UNKNOWN");
+}
+
+const char * rpmDigestASN1(DIGEST_CTX ctx)
+{
+    return (ctx != NULL ? ctx->asn1 : NULL);
+}
+
+
 DIGEST_CTX
 rpmDigestDup(DIGEST_CTX octx)
 {
@@ -337,6 +87,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
     DIGEST_CTX ctx = xcalloc(1, sizeof(*ctx));
     int xx;
 
+    ctx->hashalgo = hashalgo;
     ctx->flags = flags;
 
     switch (hashalgo) {
@@ -353,9 +104,10 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) md5Update;
 	ctx->Digest = (int (*)(void *, byte *)) md5Digest;
 /*@=type@*/
+	ctx->asn1 = "3020300c06082a864886f70d020505000410";
 	break;
     case PGPHASHALGO_SHA1:
-	ctx->name = "SHA-1";
+	ctx->name = "SHA1";
 	ctx->digestsize = 160/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -367,9 +119,10 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) sha1Update;
 	ctx->Digest = (int (*)(void *, byte *)) sha1Digest;
 /*@=type@*/
+	ctx->asn1 = "3021300906052b0e03021a05000414";
 	break;
     case PGPHASHALGO_RIPEMD128:
-	ctx->name = "RIPEMD-128";
+	ctx->name = "RIPEMD128";
 	ctx->digestsize = 128/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -383,7 +136,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_RIPEMD160:
-	ctx->name = "RIPEMD-160";
+	ctx->name = "RIPEMD160";
 	ctx->digestsize = 160/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -395,9 +148,10 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) rmd160Update;
 	ctx->Digest = (int (*)(void *, byte *)) rmd160Digest;
 /*@=type@*/
+	ctx->asn1 = "3021300906052b2403020105000414";
 	break;
     case PGPHASHALGO_RIPEMD256:
-	ctx->name = "RIPEMD-256";
+	ctx->name = "RIPEMD256";
 	ctx->digestsize = 256/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -411,7 +165,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_RIPEMD320:
-	ctx->name = "RIPEMD-320";
+	ctx->name = "RIPEMD320";
 	ctx->digestsize = 320/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -425,7 +179,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_SALSA10:
-	ctx->name = "SALSA-10";
+	ctx->name = "SALSA10";
 	ctx->digestsize = 512/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -439,7 +193,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_SALSA20:
-	ctx->name = "SALSA-20";
+	ctx->name = "SALSA20";
 	ctx->digestsize = 512/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -453,7 +207,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_TIGER192:
-	ctx->name = "TIGER-192";
+	ctx->name = "TIGER192";
 	ctx->digestsize = 192/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -465,6 +219,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) tigerUpdate;
 	ctx->Digest = (int (*)(void *, byte *)) tigerDigest;
 /*@=type@*/
+	ctx->asn1 = "3029300d06092b06010401da470c0205000418";
 	break;
     case PGPHASHALGO_MD2:
 	ctx->name = "MD2";
@@ -479,6 +234,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) md2Update;
 	ctx->Digest = (int (*)(void *, byte *)) md2Digest;
 /*@=type@*/
+	ctx->asn1 = "3020300c06082a864886f70d020205000410";
 	break;
     case PGPHASHALGO_MD4:
 	ctx->name = "MD4";
@@ -495,15 +251,13 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_CRC32:
-	ctx->name = "CRC-32";
+	ctx->name = "CRC32";
 	ctx->digestsize = 32/8;
 	ctx->datasize = 8;
 	{   sum32Param * mp = xcalloc(1, sizeof(*mp));
 /*@-type @*/
-	    mp->update = (uint32_t (*)(uint32_t, const byte *, size_t)) crc32;
-#if defined(ZLIB_H)
-	    mp->combine = (uint32_t (*)(uint32_t, uint32_t, size_t)) crc32_combine;
-#endif
+	    mp->update = (rpmuint32_t (*)(rpmuint32_t, const byte *, size_t)) __crc32;
+	    mp->combine = (rpmuint32_t (*)(rpmuint32_t, rpmuint32_t, size_t)) __crc32_combine;
 /*@=type @*/
 	    ctx->paramsize = sizeof(*mp);
 	    ctx->param = mp;
@@ -515,15 +269,13 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_ADLER32:
-	ctx->name = "ADLER-32";
+	ctx->name = "ADLER32";
 	ctx->digestsize = 32/8;
 	ctx->datasize = 8;
 	{   sum32Param * mp = xcalloc(1, sizeof(*mp));
 /*@-type @*/
-#if defined(ZLIB_H)
-	    mp->update = (uint32_t (*)(uint32_t, const byte *, size_t)) adler32;
-	    mp->combine = (uint32_t (*)(uint32_t, uint32_t, size_t)) adler32_combine;
-#endif
+	    mp->update = (rpmuint32_t (*)(rpmuint32_t, const byte *, size_t)) __adler32;
+	    mp->combine = (rpmuint32_t (*)(rpmuint32_t, rpmuint32_t, size_t)) __adler32_combine;
 /*@=type @*/
 	    ctx->paramsize = sizeof(*mp);
 	    ctx->param = mp;
@@ -535,12 +287,12 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_JLU32:
-	ctx->name = "JLU-32";
+	ctx->name = "JLU32";
 	ctx->digestsize = 32/8;
 	ctx->datasize = 8;
 	{   sum32Param * mp = xcalloc(1, sizeof(*mp));
 /*@-type @*/
-	    mp->update = (uint32_t (*)(uint32_t, const byte *, size_t)) jlu32l;
+	    mp->update = (rpmuint32_t (*)(rpmuint32_t, const byte *, size_t)) jlu32l;
 /*@=type @*/
 	    ctx->paramsize = sizeof(*mp);
 	    ctx->param = mp;
@@ -552,13 +304,13 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_CRC64:
-	ctx->name = "CRC-64";
+	ctx->name = "CRC64";
 	ctx->digestsize = 64/8;
 	ctx->datasize = 8;
 	{   sum64Param * mp = xcalloc(1, sizeof(*mp));
 /*@-type@*/
-	    mp->update = (uint64_t (*)(uint64_t, const byte *, size_t)) crc64;
-	    mp->combine = (uint64_t (*)(uint64_t, uint64_t, size_t)) crc64_combine;
+	    mp->update = (rpmuint64_t (*)(rpmuint64_t, const byte *, size_t)) __crc64;
+	    mp->combine = (rpmuint64_t (*)(rpmuint64_t, rpmuint64_t, size_t)) __crc64_combine;
 /*@=type@*/
 	    ctx->paramsize = sizeof(*mp);
 	    ctx->param = mp;
@@ -571,7 +323,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	break;
 #if defined(HAVE_BEECRYPT_API_H)
     case PGPHASHALGO_SHA224:
-	ctx->name = "SHA-224";
+	ctx->name = "SHA224";
 	ctx->digestsize = 224/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -583,9 +335,10 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) sha224Update;
 	ctx->Digest = (int (*)(void *, byte *)) sha224Digest;
 /*@=type@*/
+	ctx->asn1 = "302d300d06096086480165030402040500041C";
 	break;
     case PGPHASHALGO_SHA256:
-	ctx->name = "SHA-256";
+	ctx->name = "SHA256";
 	ctx->digestsize = 256/8;
 	ctx->datasize = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -597,9 +350,10 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) sha256Update;
 	ctx->Digest = (int (*)(void *, byte *)) sha256Digest;
 /*@=type@*/
+	ctx->asn1 = "3031300d060960864801650304020105000420";
 	break;
     case PGPHASHALGO_SHA384:
-	ctx->name = "SHA-384";
+	ctx->name = "SHA384";
 	ctx->digestsize = 384/8;
 	ctx->datasize = 128;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -611,9 +365,10 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) sha384Update;
 	ctx->Digest = (int (*)(void *, byte *)) sha384Digest;
 /*@=type@*/
+	ctx->asn1 = "3041300d060960864801650304020205000430";
 	break;
     case PGPHASHALGO_SHA512:
-	ctx->name = "SHA-512";
+	ctx->name = "SHA512";
 	ctx->digestsize = 512/8;
 	ctx->datasize = 128;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
@@ -625,6 +380,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Update = (int (*)(void *, const byte *, size_t)) sha512Update;
 	ctx->Digest = (int (*)(void *, byte *)) sha512Digest;
 /*@=type@*/
+	ctx->asn1 = "3051300d060960864801650304020305000440";
 	break;
 #endif
     case PGPHASHALGO_HAVAL_5_160:
@@ -699,50 +455,3 @@ DPRINTF((stderr, "*** Final(%p,%p,%p,%d) param %p digest %p\n", ctx, datap, lenp
     free(ctx);
     return 0;
 }
-
-pgpHashAlgo rpmDigestHashAlgo = PGPHASHALGO_MD5;
-
-/**
- * Digest options using popt.
- */
-struct poptOption rpmDigestPoptTable[] = {
- { "md2", '\0', POPT_ARG_VAL, 	&rpmDigestHashAlgo, PGPHASHALGO_MD2,
-	NULL, NULL },
- { "md4", '\0', POPT_ARG_VAL, 	&rpmDigestHashAlgo, PGPHASHALGO_MD4,
-	NULL, NULL },
- { "md5", '\0', POPT_ARG_VAL, 	&rpmDigestHashAlgo, PGPHASHALGO_MD5,
-	NULL, NULL },
- { "sha1",'\0', POPT_ARG_VAL, 	&rpmDigestHashAlgo, PGPHASHALGO_SHA1,
-	NULL, NULL },
- { "sha224",'\0', POPT_ARG_VAL, &rpmDigestHashAlgo, PGPHASHALGO_SHA224,
-	NULL, NULL },
- { "sha256",'\0', POPT_ARG_VAL, &rpmDigestHashAlgo, PGPHASHALGO_SHA256,
-	NULL, NULL },
- { "sha384",'\0', POPT_ARG_VAL, &rpmDigestHashAlgo, PGPHASHALGO_SHA384,
-	NULL, NULL },
- { "sha512",'\0', POPT_ARG_VAL, &rpmDigestHashAlgo, PGPHASHALGO_SHA512,
-	NULL, NULL },
- { "salsa10",'\0', POPT_ARG_VAL, &rpmDigestHashAlgo, PGPHASHALGO_SALSA10,
-	NULL, NULL },
- { "salsa20",'\0', POPT_ARG_VAL, &rpmDigestHashAlgo, PGPHASHALGO_SALSA20,
-	NULL, NULL },
- { "rmd128",'\0', POPT_ARG_VAL,	&rpmDigestHashAlgo, PGPHASHALGO_RIPEMD128,
-	NULL, NULL },
- { "rmd160",'\0', POPT_ARG_VAL,	&rpmDigestHashAlgo, PGPHASHALGO_RIPEMD160,
-	NULL, NULL },
- { "rmd256",'\0', POPT_ARG_VAL,	&rpmDigestHashAlgo, PGPHASHALGO_RIPEMD256,
-	NULL, NULL },
- { "rmd320",'\0', POPT_ARG_VAL,	&rpmDigestHashAlgo, PGPHASHALGO_RIPEMD320,
-	NULL, NULL },
- { "tiger",'\0', POPT_ARG_VAL,	&rpmDigestHashAlgo, PGPHASHALGO_TIGER192,
-	NULL, NULL },
- { "crc32",'\0', POPT_ARG_VAL,	&rpmDigestHashAlgo, PGPHASHALGO_CRC32,
-	NULL, NULL },
- { "crc64",'\0', POPT_ARG_VAL,	&rpmDigestHashAlgo, PGPHASHALGO_CRC64,
-	NULL, NULL },
- { "adler32",'\0', POPT_ARG_VAL,&rpmDigestHashAlgo, PGPHASHALGO_ADLER32,
-	NULL, NULL },
- { "jlu32",'\0', POPT_ARG_VAL,&rpmDigestHashAlgo, PGPHASHALGO_JLU32,
-	NULL, NULL },
-    POPT_TABLEEND
-};
