@@ -9,6 +9,19 @@
 extern "C" {
 #endif
 
+#if !defined(HAVE_CLEARENV) || defined(__LCLINT__)
+/*@-globstate@*/	/*@ XXX annotate environ. @*/
+static inline
+int clearenv(void)
+	/*@globals environ @*/
+	/*@modifies environ @*/
+{
+    environ = NULL;
+    return 0;
+}
+/*@=globstate@*/
+#endif
+
 /*@unused@*/
 static inline
 /*@observer@*/ /*@null@*/
@@ -24,7 +37,9 @@ static inline
 int envPut(/*@null@*/ const char *name, /*@null@*/ const char *value)
 	/*@*/
 {
-    char *t, *te;
+/*@kept@*/
+    char *t;
+    char *te;
     size_t nb = 0;
     int rc;
 
