@@ -1,5 +1,4 @@
-/*@-sysunrecog -noeffectuncon -nullpass -sizeoftype -unrecog -usereleased @*/
-/*@-compdef -compmempass -dependenttrans -retalias @*/
+/*@-dependenttrans -nullpass -retalias -usereleased @*/
 /*-
  * Copyright (c) 1990, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -46,6 +45,9 @@ static char sccsid[] = "@(#)fts.c	8.6 (Berkeley) 8/14/94";
 #include <string.h>
 #include <unistd.h>
 #else
+#if defined(__UCLIBC__)
+#   define __fxstat64(_stat_ver, _fd, _sbp)    fstat((_fd), (_sbp))
+#endif
 #if defined(hpux) || defined(__hpux)
 # define        _INCLUDE_POSIX_SOURCE
 #   define __errno_location() 	(&errno)
@@ -155,6 +157,9 @@ static char sccsid[] = "@(#)fts.c	8.6 (Berkeley) 8/14/94";
 #define	ALIGN(p)	(((unsigned long int) (p) + ALIGNBYTES) & ~ALIGNBYTES)
 #endif
 
+/*@unchecked@*/
+int _fts_debug = 0;
+
 /*@only@*/ /*@null@*/
 static FTSENT *	fts_alloc(FTS * sp, const char * name, int namelen)
 	/*@*/;
@@ -205,9 +210,16 @@ Fts_open(char * const * argv, int options,
 	FTSENT *tmp = NULL;
 	size_t len;
 
+/*@-formattype -modfilesys@*/
+if (_fts_debug)
+fprintf(stderr, "*** Fts_open(%p, 0x%x, %p)\n", argv, options, compar);
+/*@=formattype =modfilesys@*/
+
 	/* Options check. */
 	if (options & ~FTS_OPTIONMASK) {
+/*@-sysunrecog@*/
 		__set_errno (EINVAL);
+/*@=sysunrecog@*/
 		return (NULL);
 	}
 
@@ -365,6 +377,9 @@ Fts_close(FTS * sp)
 	register FTSENT *freep, *p;
 	int saved_errno;
 
+if (_fts_debug)
+fprintf(stderr, "*** Fts_close(%p)\n", sp);
+
 	if (sp == NULL)
 		return 0;
 
@@ -425,6 +440,9 @@ Fts_read(FTS * sp)
 	register char *t;
 	int saved_errno;
 
+if (_fts_debug)
+fprintf(stderr, "*** Fts_read(%p)\n", sp);
+
 	/* If finished or unrecoverable error, return NULL. */
 	if (sp == NULL || sp->fts_cur == NULL || ISSET(FTS_STOP))
 		return (NULL);
@@ -615,6 +633,11 @@ name:		t = sp->fts_path + NAPPEND(p->fts_parent);
 int
 Fts_set(/*@unused@*/ FTS * sp, FTSENT * p, int instr)
 {
+/*@-modfilesys@*/
+if (_fts_debug)
+fprintf(stderr, "*** Fts_set(%p, %p, 0x%x)\n", sp, p, instr);
+/*@=modfilesys@*/
+
 	if (instr != 0 && instr != FTS_AGAIN && instr != FTS_FOLLOW &&
 	    instr != FTS_NOINSTR && instr != FTS_SKIP) {
 		__set_errno (EINVAL);
@@ -630,6 +653,11 @@ Fts_children(FTS * sp, int instr)
 	register FTSENT *p;
 	int fd;
 
+/*@-modfilesys@*/
+if (_fts_debug)
+fprintf(stderr, "*** Fts_children(%p, 0x%x)\n", sp, instr);
+/*@=modfilesys@*/
+
 	if (instr != 0 && instr != FTS_NAMEONLY) {
 		__set_errno (EINVAL);
 		return (NULL);
@@ -780,7 +808,9 @@ fts_build(FTS * sp, int type)
 	 */
 	cderrno = 0;
 	if (nlinks || type == BREAD) {
+/*@-unrecog@*/
 		if (fts_safe_changedir(sp, cur, dirfd(dirp), NULL)) {
+/*@=unrecog@*/
 			if (nlinks && type == BREAD)
 				cur->fts_errno = errno;
 			cur->fts_flags |= FTS_DONTCHDIR;
@@ -1230,12 +1260,23 @@ fts_safe_changedir(FTS * sp, FTSENT * p, int fd, const char * path)
 	newfd = fd;
 	if (ISSET(FTS_NOCHDIR))
 		return (0);
+
+	/* Permit open(2) on file:// prefixed URI paths. */
+	/* XXX todo: use Open(2), which is Chroot(2) path invariant. */
+	/* XXX todo: add Fts(3) options to disable the hackery? */
+	{	const char * lpath = NULL;
+		int ut = urlPath(path, &lpath);
+		if (ut == URL_IS_PATH) path = lpath;
+	}
+
 	if (fd < 0 && (newfd = __open(path, O_RDONLY, 0)) < 0)
 		return (-1);
+/*@-sysunrecog -unrecog @*/
 	if (__fxstat64(_STAT_VER, newfd, &sb)) {
 		ret = -1;
 		goto bail;
 	}
+/*@=sysunrecog =unrecog @*/
 	if (p->fts_dev != sb.st_dev || p->fts_ino != sb.st_ino) {
 		__set_errno (ENOENT);		/* disinformation */
 		ret = -1;
@@ -1249,5 +1290,4 @@ bail:
 	__set_errno (oerrno);
 	return (ret);
 }
-/*@=compdef =compmempass =dependenttrans =retalias @*/
-/*@=sysunrecog =noeffectuncon =nullpass =sizeoftype =unrecog =usereleased @*/
+/*@=dependenttrans =nullpass =retalias =usereleased @*/
