@@ -22,8 +22,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <popt.h>
+
 #define rpmlog fprintf
 #define RPMLOG_ERR stderr
+#define RPMLOG_WARNING stderr
 #undef	_
 #define	_(x)	x
 
@@ -34,6 +37,7 @@
 #define	xisalnum(_c)		isalnum(_c)
 #define	xisalpha(_c)		isalpha(_c)
 #define	xisdigit(_c)		isdigit(_c)
+#define	xisspace(_c)		isspace(_c)
 
 typedef	FILE * FD_t;
 #define Fopen(_path, _fmode)	fopen(_path, "r");
@@ -59,29 +63,46 @@ const char * rpmMacrofiles = MACROFILES;
 
 #include <rpmio_internal.h>
 #include <rpmlog.h>
+#include <mire.h>
 
 #ifdef	WITH_LUA
 #define	_RPMLUA_INTERNAL	/* XXX lua->printbuf access */
 #include <rpmlua.h>
 #endif
 
+#include <rpmficl.h>
+#include <rpmjs.h>
+#include <rpmperl.h>
+#include <rpmpython.h>
+#include <rpmruby.h>
+#include <rpmtcl.h>
+
 #endif
 
+#include <rpmuuid.h>
+
 #define	_MACRO_INTERNAL
 #include <rpmmacro.h>
 
 #include "debug.h"
 
+/*@unchecked@*/
+#if defined(WITH_FICL) || defined(WITH_JS) || defined(WITH_PERLEMBED) || defined(WITH_PYTHONEMBED) || defined(WITH_RUBYEMBED) || defined(WITH_TCL)
+static int _globalI = 1;
+#endif
+
 #if defined(__LCLINT__)
 /*@-exportheader@*/
 extern const unsigned short int **__ctype_b_loc (void) /*@*/;
 /*@=exportheader@*/
 #endif
 
-/*@access FD_t@*/		/* XXX compared with NULL */
-/*@access MacroContext@*/
-/*@access MacroEntry@*/
+/*@access FD_t @*/		/* XXX compared with NULL */
+/*@access miRE @*/		/* XXX cast */
+/*@access MacroContext @*/
+/*@access MacroEntry@ */
 /*@access rpmlua @*/
+/*@access rpmtcl @*/
 
 static struct MacroContext_s rpmGlobalMacroContext_s;
 /*@-compmempass@*/
@@ -131,6 +152,7 @@ int print_expand_trace = _PRINT_EXPAND_TRACE;
 #define	MACRO_CHUNK_SIZE	16
 
 /* Size of expansion buffers. */
+/*@unchecked@*/
 static size_t _macro_BUFSIZ = 16 * 1024;
 
 /* forward ref */
@@ -213,6 +235,7 @@ sortMacroTable(MacroContext mc)
     }
 }
 
+#if !defined(DEBUG_MACROS)
 /*@only@*/
 static char * dupMacroEntry(MacroEntry me)
 	/*@*/
@@ -239,6 +262,7 @@ assert(me != NULL);
 
     return t;
 }
+#endif
 
 void
 rpmDumpMacroTable(MacroContext mc, FILE * fp)
@@ -273,10 +297,14 @@ rpmDumpMacroTable(MacroContext mc, FILE * fp)
 		nactive, nempty);
 }
 
+#if !defined(DEBUG_MACROS)
 int
-rpmGetMacroEntries(MacroContext mc, miRE mire, int used,
+rpmGetMacroEntries(MacroContext mc, void * _mire, int used,
 		const char *** avp)
 {
+/*@-assignexpose -castexpose @*/
+    miRE mire = (miRE) _mire;
+/*@=assignexpose =castexpose @*/
     const char ** av;
     int ac = 0;
     int i;
@@ -297,7 +325,7 @@ rpmGetMacroEntries(MacroContext mc, miRE mire, int used,
 	if (used == 0 && me->used != 0)
 	    continue;
 #if !defined(DEBUG_MACROS)	/* XXX preserve standalone build */
-	if (mire != NULL && mireRegexec(mire, me->name))
+	if (mire != NULL && mireRegexec(mire, me->name, 0) < 0)
 	    continue;
 #endif
 	av[ac++] = dupMacroEntry(me);
@@ -307,6 +335,7 @@ rpmGetMacroEntries(MacroContext mc, miRE mire, int used,
     
     return ac;
 }
+#endif
 
 /**
  * Find entry in macro table.
@@ -984,20 +1013,20 @@ freeArgs(MacroBuf mb)
 /*@dependent@*/ static const char *
 grabArgs(MacroBuf mb, const MacroEntry me, /*@returned@*/ const char * se,
 		const char * lastc)
-	/*@globals rpmGlobalMacroContext, internalState @*/
-	/*@modifies mb, rpmGlobalMacroContext, internalState @*/
+	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@modifies mb, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
+    poptContext optCon;
+    struct poptOption *optTbl;
     size_t bufn = _macro_BUFSIZ;
     char *buf = alloca(bufn);
     char *b, *be;
     char aname[16];
-    const char *opts, *o;
+    const char *opts;
     int argc = 0;
     const char **argv;
     int c;
-#ifdef __GLIBC__
-    char *posixly_correct;
-#endif
+    unsigned int popt_flags;
 
     /* Copy macro name as argv[0], save beginning of args.  */
     buf[0] = '\0';
@@ -1049,6 +1078,7 @@ grabArgs(MacroBuf mb, const MacroEntry me, /*@returned@*/ const char * se,
     argv = (const char **) alloca((argc + 1) * sizeof(*argv));
     be[-1] = ' '; /* assert((be - 1) == (b + strlen(b) == buf + strlen(buf))) */
     be[0] = '\0';
+
     b = buf;
     for (c = 0; c < argc; c++) {
 	argv[c] = b;
@@ -1058,107 +1088,93 @@ grabArgs(MacroBuf mb, const MacroEntry me, /*@returned@*/ const char * se,
     /* assert(b == be);  */
     argv[argc] = NULL;
 
-    /* Citation from glibc/posix/getopt.c:
-     *    Index in ARGV of the next element to be scanned.
-     *    This is used for communication to and from the caller
-     *    and for communication between successive calls to `getopt'.
-     *
-     *    On entry to `getopt', zero means this is the first call; initialize.
-     *
-     *    When `getopt' returns -1, this is the index of the first of the
-     *    non-option elements that the caller should itself scan.
-     *
-     *    Otherwise, `optind' communicates from one call to the next
-     *    how much of ARGV has been scanned so far.
-     */
-    /* 1003.2 says this must be 1 before any call.  */
-
-#ifdef __GLIBC__
-    /*@-mods@*/
-    optind = 0;		/* XXX but posix != glibc */
-    /*@=mods@*/
-#else
-    optind = 1;
+    /* '+' as the first character means that options are recognized
+     * only before positional arguments, as POSIX requires.
+    */
+    popt_flags = POPT_CONTEXT_NO_EXEC;
+#if defined(RPM_VENDOR_OPENPKG) /* always-strict-posix-option-parsing */
+    popt_flags |= POPT_CONTEXT_POSIXMEHARDER;
 #endif
+    if (me->opts[0] == '+') popt_flags |= POPT_CONTEXT_POSIXMEHARDER;
 
+    /* Count the number of short options. */
     opts = me->opts;
+    if (*opts == '+') opts++;
+    for (c = 0; *opts != '\0'; opts++)
+	if (*opts != ':') c++;
 
-#ifdef __GLIBC__
-    /*
-     *  Ensure option parsing is done without allowing option/argument permutations
-     *  to avoid accidentally picking up and complaining about unknown options.
-     *
-     *  Required standard POSIX getopt(3) behavior:
-     *  $ rpm --define '%foo() <%*>' --eval '%{foo bar %(echo -n "quux") baz}'
-     *  <bar quux baz>
-     *
-     *  Unexpected non-standard Linux GLIBC getopt(3) behavior:
-     *  $ rpm --define '%foo() <%*>' --eval '%{foo bar %(echo -n "quux") baz}'
-     *  foo: invalid option -- n
-     *  error: Unknown option ? in foo()
-     *  <%*>
-     *
-     *  Fixed standard POSIX getopt(3) behavior also under Linux GLIBC:
-     *  $ POSIXLY_CORRECT=1 rpm --define '%foo() <%*>' --eval '%{foo bar %(echo -n "quux") baz}'
-     *  <bar quux baz>
-     */
-    posixly_correct = getenv("POSIXLY_CORRECT");
-    setenv("POSIXLY_CORRECT", "1", 1);
-#endif
+    /* Set up popt option table. */
+    optTbl = xcalloc(sizeof(*optTbl), (c + 1));
+    opts = me->opts;
+    if (*opts == '+') opts++;
+    for (c = 0; *opts != '\0'; opts++) {
+	if (*opts == ':') continue;
+	optTbl[c].shortName = opts[0];
+	optTbl[c].val = (int) opts[0];
+	if (opts[1] == ':')
+	    optTbl[c].argInfo = POPT_ARG_STRING;
+	c++;
+    }
 
-    /* Define option macros. */
-/*@-nullstate@*/ /* FIX: argv[] can be NULL */
-    while((c = getopt(argc, (char **)argv, opts)) != -1)
+    /* Parse the options, defining option macros. */
+/*@-nullstate@*/
+    optCon = poptGetContext(argv[0], argc, argv, optTbl, popt_flags);
 /*@=nullstate@*/
-    {
-	if (c == (int) '?' || (o = strchr(opts, c)) == NULL) {
-	    rpmlog(RPMLOG_ERR, _("Unknown option %c in %s(%s)\n"),
-			(char)c, me->name, opts);
-	    return se;
-	}
+    while ((c = poptGetNextOpt(optCon)) > 0) {
+	const char * optArg = poptGetOptArg(optCon);
 	*be++ = '-';
 	*be++ = (char) c;
-	if (o[1] == ':') {
+	if (optArg != NULL) {
 	    *be++ = ' ';
-	    be = stpcpy(be, optarg);
+	    be = stpcpy(be, optArg);
 	}
 	*be++ = '\0';
 	aname[0] = '-'; aname[1] = (char)c; aname[2] = '\0';
 	addMacro(mb->mc, aname, NULL, b, mb->depth);
-	if (o[1] == ':') {
+	if (optArg != NULL) {
 	    aname[0] = '-'; aname[1] = (char)c; aname[2] = '*'; aname[3] = '\0';
-	    addMacro(mb->mc, aname, NULL, optarg, mb->depth);
+	    addMacro(mb->mc, aname, NULL, optArg, mb->depth);
 	}
 	be = b; /* reuse the space */
+/*@-dependenttrans -modobserver -observertrans @*/
+	optArg = _free(optArg);
+/*@=dependenttrans =modobserver =observertrans @*/
+    }
+    if (c < -1) {
+	rpmlog(RPMLOG_ERR, _("Unknown option in macro %s(%s): %s: %s\n"),
+		me->name, me->opts,
+		poptBadOption(optCon, POPT_BADOPTION_NOALIAS), poptStrerror(c));
+	goto exit;
     }
 
-#ifdef __GLIBC__
-    if (posixly_correct != NULL)
-        setenv("POSIXLY_CORRECT", posixly_correct, 1);
-    else
-        unsetenv("POSIXLY_CORRECT");
-#endif
-
+    argv = poptGetArgs(optCon);
+    argc = 0;
+    if (argv != NULL)
+    for (c = 0; argv[c] != NULL; c++)
+	argc++;
+    
     /* Add arg count as macro. */
-    sprintf(aname, "%d", (argc - optind));
+    sprintf(aname, "%d", argc);
     addMacro(mb->mc, "#", NULL, aname, mb->depth);
 
     /* Add macro for each arg. Concatenate args for %*. */
     if (be) {
 	*be = '\0';
-	for (c = optind; c < argc; c++) {
-	    sprintf(aname, "%d", (c - optind + 1));
+	if (argv != NULL)
+	for (c = 0; c < argc; c++) {
+	    sprintf(aname, "%d", (c + 1));
 	    addMacro(mb->mc, aname, NULL, argv[c], mb->depth);
 	    if (be != b) *be++ = ' '; /* Add space between args */
-/*@-nullpass@*/ /* FIX: argv[] can be NULL */
 	    be = stpcpy(be, argv[c]);
-/*@=nullpass@*/
 	}
     }
 
     /* Add unexpanded args as macro. */
     addMacro(mb->mc, "*", NULL, b, mb->depth);
 
+exit:
+    optCon = poptFreeContext(optCon);
+    optTbl = _free(optTbl);
     return se;
 }
 
@@ -1214,7 +1230,7 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
     }
     if (fn > 5 && STREQ("patch", f, 5) && xisdigit((int)f[5])) {
 	/* Skip leading zeros */
-	for (c = 5; c < fn-1 && f[c] == '0' && xisdigit((int)f[c+1]);)
+	for (c = 5; c < (int)(fn-1) && f[c] == '0' && xisdigit((int)f[c+1]);)
 	    c++;
 	b = buf;
 	be = stpncpy( stpcpy(b, "%patch -P "), f+c, fn-c);
@@ -1229,41 +1245,45 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	if ((b = strrchr(buf, '/')) != NULL)
 	    *b = '\0';
 	b = buf;
+#if !defined(__LCLINT__) /* XXX LCL: realpath(3) annotations are buggy. */
     } else if (STREQ("realpath", f, fn)) {
-        char rp[PATH_MAX];
-        char *cp;
-        size_t l;
-        if ((cp = realpath(buf, rp)) != NULL) {
-            l = strlen(cp);
-            if ((size_t)(l+1) <= bufn) {
-                memcpy(buf, cp, l+1);
-                b = buf;
-            }
-        }
+	char rp[PATH_MAX];
+	char *cp;
+	size_t l;
+	if ((cp = realpath(buf, rp)) != NULL) {
+	    l = strlen(cp);
+	    if ((size_t)(l+1) <= bufn) {
+		memcpy(buf, cp, l+1);
+		b = buf;
+	    }
+	}
+#endif
     } else if (STREQ("getenv", f, fn)) {
-        char *cp;
-        if ((cp = getenv(buf)) != NULL)
-            b = cp;
+	char *cp;
+	if ((cp = getenv(buf)) != NULL)
+	    b = cp;
     } else if (STREQ("shrink", f, fn)) {
-        /* shrink body by removing all leading and trailing whitespaces and
-           reducing intermediate whitespaces to a single space character */
-        int i, j, k, was_space = 0;
-        for (i = 0, j = 0, k = strlen(buf); i < k; ) {
-            if (xisspace((int)(buf[i]))) {
-                was_space = 1;
-                i++;
-                continue;
-            }
-            else if (was_space) {
-                was_space = 0;
-                if (j > 0) /* remove leading blanks at all */
-                    buf[j++] = ' ';
-                /* fallthrough */
-            }
-            buf[j++] = buf[i++];
-        }
-        buf[j] = '\0';
-        b = buf;
+	/*
+	 * shrink body by removing all leading and trailing whitespaces and
+	 * reducing intermediate whitespaces to a single space character.
+	 */
+	int i, j, k, was_space = 0;
+	for (i = 0, j = 0, k = (int)strlen(buf); i < k; ) {
+	    if (xisspace((int)(buf[i]))) {
+		was_space = 1;
+		i++;
+		continue;
+	    }
+	    else if (was_space) {
+		was_space = 0;
+		if (j > 0) /* remove leading blanks at all */
+		    buf[j++] = ' ';
+		/* fallthrough */
+	    }
+	    buf[j++] = buf[i++];
+	}
+	buf[j] = '\0';
+	b = buf;
     } else if (STREQ("suffix", f, fn)) {
 	if ((b = strrchr(buf, '.')) != NULL)
 	    b++;
@@ -1314,6 +1334,9 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	case 5:	/* COMPRESSED_LZMA */
 	    sprintf(be, "%%__lzma -dc '%s'", b);
 	    break;
+	case 6:	/* COMPRESSED_XZ */
+	    sprintf(be, "%%__xz -dc '%s'", b);
+	    break;
 	}
 	b = be;
     } else if (STREQ("mkstemp", f, fn)) {
@@ -1325,10 +1348,62 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	    be++;
 /*@=globs@*/
 #if defined(HAVE_MKSTEMP)
-        (void) close(mkstemp(b));
+	(void) close(mkstemp(b));
 #else
-        (void) mktemp(b);
+	(void) mktemp(b);
 #endif
+    } else if (STREQ("mkdtemp", f, fn)) {
+/*@-globs@*/
+	for (b = buf; (c = (int)*b) && isblank(c);)
+	    b++;
+	/* XXX FIXME: file paths with embedded white space needs rework. */
+	for (be = b; (c = (int)*be) && !isblank(c);)
+	    be++;
+/*@=globs@*/
+#if defined(HAVE_MKDTEMP) && !defined(__LCLINT__)
+	if (mkdtemp(b) == NULL)
+	    perror("mkdtemp");
+#else
+	if ((b = tmpnam(b)) != NULL)
+	    (void) mkdir(b, 0700);	/* XXX S_IWRSXU is not included. */
+#endif
+    } else if (STREQ("uuid", f, fn)) {
+	int uuid_version;
+	const char *uuid_ns;
+	const char *uuid_data;
+	char *cp;
+	size_t n;
+
+	uuid_version = 1;
+	uuid_ns = NULL;
+	uuid_data = NULL;
+	cp = buf;
+	if ((n = strspn(cp, " \t\n")) > 0)
+	    cp += n;
+	if ((n = strcspn(cp, " \t\n")) > 0) {
+	    uuid_version = (int)strtol(cp, (char **)NULL, 10);
+	    cp += n;
+	    if ((n = strspn(cp, " \t\n")) > 0)
+		cp += n;
+	    if ((n = strcspn(cp, " \t\n")) > 0) {
+		uuid_ns = cp;
+		cp += n;
+		*cp++ = '\0';
+		if ((n = strspn(cp, " \t\n")) > 0)
+		    cp += n;
+		if ((n = strcspn(cp, " \t\n")) > 0) {
+		    uuid_data = cp;
+		    cp += n;
+		    *cp++ = '\0';
+		}
+	    }
+	}
+/*@-nullpass@*/	/* FIX: uuid_ns may be NULL */
+	if (rpmuuidMake(uuid_version, uuid_ns, uuid_data, buf, NULL))
+	    rpmlog(RPMLOG_ERR, "failed to create UUID\n");
+	else
+	    b = buf;
+/*@=nullpass@*/
     } else if (STREQ("S", f, fn)) {
 	for (b = buf; (c = (int)*b) && xisdigit(c);)
 	    b++;
@@ -1356,7 +1431,7 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 }
 
 static int expandFIFO(MacroBuf mb, MacroEntry me, const char *g, size_t gn)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies mb, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     int rc = 0;
@@ -1371,6 +1446,167 @@ static int expandFIFO(MacroBuf mb, MacroEntry me, const char *g, size_t gn)
     return rc;
 }
 
+#if !defined(DEBUG_MACROS)
+/* =============================================================== */
+/* XXX dupe'd to avoid change in linkage conventions. */
+
+#define POPT_ERROR_NOARG        -10     /*!< missing argument */
+#define POPT_ERROR_BADQUOTE     -15     /*!< error in paramter quoting */
+#define POPT_ERROR_MALLOC       -21     /*!< memory allocation failed */
+
+#define POPT_ARGV_ARRAY_GROW_DELTA 5
+
+static int XpoptDupArgv(int argc, const char **argv,
+		int * argcPtr, const char *** argvPtr)
+	/*@modifies *argcPtr, *argvPtr @*/
+{
+    size_t nb = (argc + 1) * sizeof(*argv);
+    const char ** argv2;
+    char * dst;
+    int i;
+
+    if (argc <= 0 || argv == NULL)	/* XXX can't happen */
+	return POPT_ERROR_NOARG;
+    for (i = 0; i < argc; i++) {
+	if (argv[i] == NULL)
+	    return POPT_ERROR_NOARG;
+	nb += strlen(argv[i]) + 1;
+    }
+	
+    dst = xmalloc(nb);
+    if (dst == NULL)			/* XXX can't happen */
+	return POPT_ERROR_MALLOC;
+    argv2 = (void *) dst;
+    dst += (argc + 1) * sizeof(*argv);
+
+    for (i = 0; i < argc; i++) {
+	argv2[i] = dst;
+	dst += strlen(strcpy(dst, argv[i])) + 1;
+    }
+    argv2[argc] = NULL;
+
+    if (argvPtr) {
+	*argvPtr = argv2;
+    } else {
+	free(argv2);
+	argv2 = NULL;
+    }
+    if (argcPtr)
+	*argcPtr = argc;
+    return 0;
+}
+
+static int XpoptParseArgvString(const char * s, int * argcPtr, const char *** argvPtr)
+	/*@modifies *argcPtr, *argvPtr @*/
+{
+    const char * src;
+    char quote = '\0';
+    int argvAlloced = POPT_ARGV_ARRAY_GROW_DELTA;
+    const char ** argv = xmalloc(sizeof(*argv) * argvAlloced);
+    int argc = 0;
+    size_t buflen = strlen(s) + 1;
+    char * buf = memset(alloca(buflen), 0, buflen);
+    int rc = POPT_ERROR_MALLOC;
+
+    if (argv == NULL) return rc;
+    argv[argc] = buf;
+
+    for (src = s; *src != '\0'; src++) {
+	if (quote == *src) {
+	    quote = '\0';
+	} else if (quote != '\0') {
+	    if (*src == '\\') {
+		src++;
+		if (!*src) {
+		    rc = POPT_ERROR_BADQUOTE;
+		    goto exit;
+		}
+		if (*src != quote) *buf++ = '\\';
+	    }
+	    *buf++ = *src;
+	} else if (isspace(*src)) {
+	    if (*argv[argc] != '\0') {
+		buf++, argc++;
+		if (argc == argvAlloced) {
+		    argvAlloced += POPT_ARGV_ARRAY_GROW_DELTA;
+		    argv = realloc(argv, sizeof(*argv) * argvAlloced);
+		    if (argv == NULL) goto exit;
+		}
+		argv[argc] = buf;
+	    }
+	} else switch (*src) {
+	  case '"':
+	  case '\'':
+	    quote = *src;
+	    /*@switchbreak@*/ break;
+	  case '\\':
+	    src++;
+	    if (!*src) {
+		rc = POPT_ERROR_BADQUOTE;
+		goto exit;
+	    }
+	    /*@fallthrough@*/
+	  default:
+	    *buf++ = *src;
+	    /*@switchbreak@*/ break;
+	}
+    }
+
+    if (strlen(argv[argc])) {
+	argc++, buf++;
+    }
+
+    rc = XpoptDupArgv(argc, argv, argcPtr, argvPtr);
+
+exit:
+    if (argv) free(argv);
+    return rc;
+}
+#endif	/* !defined(DEBUG_MACROS) */
+
+/**
+ * Parse args and string for PHP like %{foo <args> : <string> } syntax.
+ * @param s		"{ ... }" construct to parse
+ * @param nb		no. of bytes
+ * @retval *avp		invocation args
+ * @return		script string
+ */
+#if defined(WITH_FICL) || defined(WITH_JS) || defined(WITH_PERLEMBED) || defined(WITH_PYTHONEMBED) || defined(WITH_RUBYEMBED) || defined(WITH_TCL)
+static char * parseEmbedded(const char * s, size_t nb, const char *** avp)
+	/*@*/
+{
+    char * script = NULL;
+    const char * se;
+
+    /* XXX FIXME: args might have embedded : too. */
+    for (se = s + 1; se < (s+nb); se++)
+    switch (*se) {
+    default:	continue;	/*@notreached@*/ break;
+    case ':':	goto bingo;	/*@notreached@*/ break;
+    }
+
+bingo:
+    {	size_t na = (size_t)(se-s-1);
+	char * args = NULL;
+	int ac;
+	int rc;
+
+	args = memcpy(xmalloc(na+1), s+1, na);
+	args[na] = '\0';
+
+	ac = 0;
+	rc = XpoptParseArgvString(args, &ac, avp);
+	args = _free(args);
+	nb -= na;
+    }
+
+    nb -= 3;
+    script = memcpy(xmalloc(nb+1), se+1, nb+1);
+    script[nb] = '\0';
+    return script;
+}
+#endif
+
 /**
  * The main macro recursion loop.
  * @todo Dynamically reallocate target buffer.
@@ -1554,6 +1790,9 @@ expandMacro(MacroBuf mb)
 		    int xx;
 		    mfn[gn] = '\0';
 		    xx = rpmLoadMacroFile(NULL, mfn);
+		    /* Print failure iff %{load:...} or %{!?load:...} */
+		    if (xx != 0 && chkexist == negate)
+			rpmlog(RPMLOG_ERR, _("%s: load macros failed\n"), mfn);
 		}
 		s = se;
 		continue;
@@ -1645,6 +1884,171 @@ expandMacro(MacroBuf mb)
 	}
 #endif
 
+#ifdef	WITH_FICL
+	if (STREQ("ficl", f, fn)) {
+		const char ** av = NULL;
+		char * script = parseEmbedded(s, (size_t)(se-s), &av);
+		rpmficl ficl = rpmficlNew(av, _globalI);
+		const char * result = NULL;
+
+		if (rpmficlRun(ficl, script, &result) != RPMRC_OK)
+		    rc = 1;
+		else {
+		  if (result == NULL) result = "FIXME";
+		  if (result != NULL && *result != '\0') {
+		    size_t len = strlen(result);
+		    if (len > mb->nb)
+			len = mb->nb;
+		    memcpy(mb->t, result, len);
+		    mb->t += len;
+		    mb->nb -= len;
+		 }
+		}
+		ficl = rpmficlFree(ficl);
+		av = _free(av);
+		script = _free(script);
+		s = se;
+		continue;
+	}
+#endif
+
+#ifdef	WITH_JS
+	if (STREQ("js", f, fn)) {
+		const char ** av = NULL;
+		char * script = parseEmbedded(s, (size_t)(se-s), &av);
+		rpmjs js = rpmjsNew(av, _globalI);
+		const char * result = NULL;
+
+		if (rpmjsRun(js, script, &result) != RPMRC_OK)
+		    rc = 1;
+		else {
+		  if (result == NULL) result = "FIXME";
+		  if (result != NULL && *result != '\0') {
+		    size_t len = strlen(result);
+		    if (len > mb->nb)
+			len = mb->nb;
+		    memcpy(mb->t, result, len);
+		    mb->t += len;
+		    mb->nb -= len;
+		 }
+		}
+		js = rpmjsFree(js);
+		av = _free(av);
+		script = _free(script);
+		s = se;
+		continue;
+	}
+#endif
+
+#ifdef	WITH_PERLEMBED
+	if (STREQ("perl", f, fn)) {
+		const char ** av = NULL;
+		char * script = parseEmbedded(s, (size_t)(se-s), &av);
+		rpmperl perl = rpmperlNew(av, _globalI);
+		const char * result = NULL;
+
+		if (rpmperlRun(perl, script, &result) != RPMRC_OK)
+		    rc = 1;
+		else {
+		  if (result == NULL) result = "FIXME";
+		  if (result != NULL && *result != '\0') {
+		    size_t len = strlen(result);
+		    if (len > mb->nb)
+			len = mb->nb;
+		    memcpy(mb->t, result, len);
+		    mb->t += len;
+		    mb->nb -= len;
+		 }
+		}
+		perl = rpmperlFree(perl);
+		av = _free(av);
+		script = _free(script);
+		s = se;
+		continue;
+	}
+#endif
+
+#ifdef	WITH_PYTHONEMBED
+	if (STREQ("python", f, fn)) {
+		const char ** av = NULL;
+		char * script = parseEmbedded(s, (size_t)(se-s), &av);
+		rpmpython python = rpmpythonNew(av, _globalI);
+		const char * result = NULL;
+
+		if (rpmpythonRun(python, script, &result) != RPMRC_OK)
+		    rc = 1;
+		else {
+		  if (result == NULL) result = "FIXME";
+		  if (result != NULL && *result != '\0') {
+		    size_t len = strlen(result);
+		    if (len > mb->nb)
+			len = mb->nb;
+		    memcpy(mb->t, result, len);
+		    mb->t += len;
+		    mb->nb -= len;
+		  }
+		}
+		python = rpmpythonFree(python);
+		av = _free(av);
+		script = _free(script);
+		s = se;
+		continue;
+	}
+#endif
+
+#ifdef	WITH_RUBYEMBED
+	if (STREQ("ruby", f, fn)) {
+		const char ** av = NULL;
+		char * script = parseEmbedded(s, (size_t)(se-s), &av);
+		rpmruby ruby = rpmrubyNew(av, _globalI);
+		const char * result = NULL;
+
+		if (rpmrubyRun(ruby, script, &result) != RPMRC_OK)
+		    rc = 1;
+		else {
+		  if (result == NULL) result = "FIXME";
+		  if (result != NULL && *result != '\0') {
+		    size_t len = strlen(result);
+		    if (len > mb->nb)
+			len = mb->nb;
+		    memcpy(mb->t, result, len);
+		    mb->t += len;
+		    mb->nb -= len;
+		  }
+		}
+		ruby = rpmrubyFree(ruby);
+		av = _free(av);
+		script = _free(script);
+		s = se;
+		continue;
+	}
+#endif
+
+#ifdef	WITH_TCL
+	if (STREQ("tcl", f, fn)) {
+		const char ** av = NULL;
+		char * script = parseEmbedded(s, (size_t)(se-s), &av);
+		rpmtcl tcl = rpmtclNew(av, _globalI);
+		const char * result = NULL;
+
+		if (rpmtclRun(tcl, script, &result) != RPMRC_OK)
+		    rc = 1;
+		else if (result != NULL && *result != '\0') {
+		    size_t len = strlen(result);
+		    if (len > mb->nb)
+			len = mb->nb;
+		    memcpy(mb->t, result, len);
+		    mb->t += len;
+		    mb->nb -= len;
+		}
+		tcl = rpmtclFree(tcl);
+		av = _free(av);
+		script = _free(script);
+		s = se;
+		continue;
+	}
+#endif
+
 	/* Rewrite "%patchNN ..." as "%patch -P NN ..." and expand. */
 	if (lastc && fn > 5 && STREQ("patch", f, 5) && xisdigit((int)f[5])) {
 		/*@-internalglobs@*/ /* FIX: verbose may be set */
@@ -1665,6 +2069,8 @@ expandMacro(MacroBuf mb)
 	    STREQ("verbose", f, fn) ||
 	    STREQ("uncompress", f, fn) ||
 	    STREQ("mkstemp", f, fn) ||
+	    STREQ("mkdtemp", f, fn) ||
+	    STREQ("uuid", f, fn) ||
 	    STREQ("url2path", f, fn) ||
 	    STREQ("u2p", f, fn) ||
 	    STREQ("S", f, fn) ||
@@ -1719,7 +2125,7 @@ expandMacro(MacroBuf mb)
 	}
 
 	if (me == NULL) {	/* leave unknown %... as is */
-#ifndef HACK
+#if !defined(RPM_VENDOR_WINDRIVER_DEBUG)	/* XXX usually disabled */
 #if DEAD
 		/* XXX hack to skip over empty arg list */
 		if (fn == 1 && *f == '*') {
@@ -1791,142 +2197,27 @@ expandMacro(MacroBuf mb)
     return rc;
 }
 
-#if defined(RPM_VENDOR_OPENPKG) /* security-sanity-check-rpmpopt-and-rpmmacros */
+#if defined(RPM_VENDOR_OPENPKG) /* stick-with-rpm-file-sanity-checking */ || \
+    !defined(POPT_ERROR_BADCONFIG)	/* XXX POPT 1.15 retrofit */
 int rpmSecuritySaneFile(const char *filename)
 {
     struct stat sb;
     uid_t uid;
 
     if (stat(filename, &sb) == -1)
-        return (errno == ENOENT ? 1 : 0);
+	return 1;
     uid = getuid();
     if (sb.st_uid != uid)
-        return 0;
+	return 0;
     if (!S_ISREG(sb.st_mode))
-        return 0;
+	return 0;
     if (sb.st_mode & (S_IWGRP|S_IWOTH))
-        return 0;
+	return 0;
     return 1;
 }
 #endif
 
 #if !defined(DEBUG_MACROS)
-/* =============================================================== */
-/* XXX dupe'd to avoid change in linkage conventions. */
-
-#define POPT_ERROR_NOARG        -10     /*!< missing argument */
-#define POPT_ERROR_BADQUOTE     -15     /*!< error in paramter quoting */
-#define POPT_ERROR_MALLOC       -21     /*!< memory allocation failed */
-
-#define POPT_ARGV_ARRAY_GROW_DELTA 5
-
-static int XpoptDupArgv(int argc, const char **argv,
-		int * argcPtr, const char *** argvPtr)
-	/*@modifies *argcPtr, *argvPtr @*/
-{
-    size_t nb = (argc + 1) * sizeof(*argv);
-    const char ** argv2;
-    char * dst;
-    int i;
-
-    if (argc <= 0 || argv == NULL)	/* XXX can't happen */
-	return POPT_ERROR_NOARG;
-    for (i = 0; i < argc; i++) {
-	if (argv[i] == NULL)
-	    return POPT_ERROR_NOARG;
-	nb += strlen(argv[i]) + 1;
-    }
-	
-    dst = malloc(nb);
-    if (dst == NULL)			/* XXX can't happen */
-	return POPT_ERROR_MALLOC;
-    argv2 = (void *) dst;
-    dst += (argc + 1) * sizeof(*argv);
-
-    for (i = 0; i < argc; i++) {
-	argv2[i] = dst;
-	dst += strlen(strcpy(dst, argv[i])) + 1;
-    }
-    argv2[argc] = NULL;
-
-    if (argvPtr) {
-	*argvPtr = argv2;
-    } else {
-	free(argv2);
-	argv2 = NULL;
-    }
-    if (argcPtr)
-	*argcPtr = argc;
-    return 0;
-}
-
-static int XpoptParseArgvString(const char * s, int * argcPtr, const char *** argvPtr)
-	/*@modifies *argcPtr, *argvPtr @*/
-{
-    const char * src;
-    char quote = '\0';
-    int argvAlloced = POPT_ARGV_ARRAY_GROW_DELTA;
-    const char ** argv = malloc(sizeof(*argv) * argvAlloced);
-    int argc = 0;
-    size_t buflen = strlen(s) + 1;
-    char * buf = memset(alloca(buflen), 0, buflen);
-    int rc = POPT_ERROR_MALLOC;
-
-    if (argv == NULL) return rc;
-    argv[argc] = buf;
-
-    for (src = s; *src != '\0'; src++) {
-	if (quote == *src) {
-	    quote = '\0';
-	} else if (quote != '\0') {
-	    if (*src == '\\') {
-		src++;
-		if (!*src) {
-		    rc = POPT_ERROR_BADQUOTE;
-		    goto exit;
-		}
-		if (*src != quote) *buf++ = '\\';
-	    }
-	    *buf++ = *src;
-	} else if (isspace(*src)) {
-	    if (*argv[argc] != '\0') {
-		buf++, argc++;
-		if (argc == argvAlloced) {
-		    argvAlloced += POPT_ARGV_ARRAY_GROW_DELTA;
-		    argv = realloc(argv, sizeof(*argv) * argvAlloced);
-		    if (argv == NULL) goto exit;
-		}
-		argv[argc] = buf;
-	    }
-	} else switch (*src) {
-	  case '"':
-	  case '\'':
-	    quote = *src;
-	    /*@switchbreak@*/ break;
-	  case '\\':
-	    src++;
-	    if (!*src) {
-		rc = POPT_ERROR_BADQUOTE;
-		goto exit;
-	    }
-	    /*@fallthrough@*/
-	  default:
-	    *buf++ = *src;
-	    /*@switchbreak@*/ break;
-	}
-    }
-
-    if (strlen(argv[argc])) {
-	argc++, buf++;
-    }
-
-    rc = XpoptDupArgv(argc, argv, argcPtr, argvPtr);
-
-exit:
-    if (argv) free(argv);
-    return rc;
-}
-
 /* =============================================================== */
 /*@unchecked@*/
 static int _debug = 0;
@@ -1944,7 +2235,8 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
     const char * t;
 #endif
     size_t maxb, nb;
-    int i, j;
+    size_t i;
+    int j;
     int rc;
 
     rc = XpoptParseArgvString(patterns, &ac, &av);
@@ -2057,8 +2349,8 @@ exit:
     if (rc || argvPtr == NULL) {
 /*@-dependenttrans -unqualifiedtrans@*/
 	if (argv != NULL)
-	for (i = 0; i < argc; i++)
-	    argv[i] = _free(argv[i]);
+	for (j = 0; j < argc; j++)
+	    argv[j] = _free(argv[j]);
 	argv = _free(argv);
 /*@=dependenttrans =unqualifiedtrans@*/
     }
@@ -2080,7 +2372,7 @@ expandMacros(void * spec, MacroContext mc, char * sbuf, size_t slen)
     if (mc == NULL) mc = rpmGlobalMacroContext;
 
     tbuf = alloca(slen + 1);
-    memset(tbuf, 0, (slen + 1));
+    tbuf[0] = '\0';
 
     mb->s = sbuf;
     mb->t = tbuf;
@@ -2211,10 +2503,10 @@ static void expand_macrosfile_macro(const char *file_name, const char *buf, size
     l = strlen(macro_name);
     k = strlen(file_name);
     while ((cp = strstr(buf, macro_name)) != NULL) {
-        if (((strlen(buf) - l) + k) < bufn) {
-            memmove(cp+k, cp+l, strlen(cp+l)+1);
-            memcpy(cp, file_name, k);
-        }
+	if (((strlen(buf) - l) + k) < bufn) {
+	    memmove(cp+k, cp+l, strlen(cp+l)+1);
+	    memcpy(cp, file_name, k);
+	}
     }
     return;
 }
@@ -2223,6 +2515,7 @@ static void expand_macrosfile_macro(const char *file_name, const char *buf, size
 int
 rpmLoadMacroFile(MacroContext mc, const char * fn)
 {
+    /* XXX TODO: teach rdcl() to read through a URI, eliminate ".fpio". */
     FD_t fd = Fopen(fn, "r.fpio");
     size_t bufn = _macro_BUFSIZ;
     char *buf = alloca(bufn);
@@ -2286,16 +2579,6 @@ rpmInitMacros(MacroContext mc, const char * macrofiles)
 	else
 	    me = m + strlen(m);
 
-#if defined(RPM_VENDOR_OPENPKG) /* security-sanity-check-rpmpopt-and-rpmmacros */
-        if (m[0] == '@' /* attention */) {
-            m++;
-            if (!rpmSecuritySaneFile(m)) {
-                rpmlog(RPMLOG_WARNING, "existing RPM macros file \"%s\" considered INSECURE -- not loaded\n", m);
-                continue;
-            }
-        }
-#endif
-
 	/* Glob expand the macro file path element, expanding ~ to $HOME. */
 	ac = 0;
 	av = NULL;
@@ -2306,7 +2589,7 @@ rpmInitMacros(MacroContext mc, const char * macrofiles)
 	av[1] = NULL;
 #else
 	i = rpmGlob(m, &ac, &av);
-        if (i != 0)
+	if (i != 0)
 	    continue;
 #endif
 
@@ -2314,16 +2597,31 @@ rpmInitMacros(MacroContext mc, const char * macrofiles)
 
 	for (i = 0; i < ac; i++) {
 	    size_t slen = strlen(av[i]);
+	    const char *fn = av[i];
+
+	if (fn[0] == '@' /* attention */) {
+	    fn++;
+#if defined(RPM_VENDOR_OPENPKG) /* stick-with-rpm-file-sanity-checking */ || \
+    !defined(POPT_ERROR_BADCONFIG)	/* XXX POPT 1.15 retrofit */
+	    if (!rpmSecuritySaneFile(fn))
+#else
+	    if (!poptSaneFile(fn))
+#endif
+	    {
+		rpmlog(RPMLOG_WARNING, "existing RPM macros file \"%s\" considered INSECURE -- not loaded\n", fn);
+		/*@innercontinue@*/ continue;
+	    }
+	}
 
 	/* Skip backup files and %config leftovers. */
 #define	_suffix(_s, _x) \
     (slen >= sizeof(_x) && !strcmp((_s)+slen-(sizeof(_x)-1), (_x)))
-	    if (!(_suffix(av[i], "~")
-	       || _suffix(av[i], ".rpmnew")
-	       || _suffix(av[i], ".rpmorig")
-	       || _suffix(av[i], ".rpmsave"))
+	    if (!(_suffix(fn, "~")
+	       || _suffix(fn, ".rpmnew")
+	       || _suffix(fn, ".rpmorig")
+	       || _suffix(fn, ".rpmsave"))
 	       )
-		   (void) rpmLoadMacroFile(mc, av[i]);
+		   (void) rpmLoadMacroFile(mc, fn);
 #undef _suffix
 
 	    av[i] = _free(av[i]);
@@ -2373,9 +2671,48 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
     ssize_t nb;
     int rc = -1;
     unsigned char magic[13];
+#if defined(RPM_VENDOR_OPENPKG) || defined(RPM_VENDOR_FEDORA) || defined(RPM_VENDOR_MANDRIVA) /* extension-based-compression-detection */
+    size_t file_len;
+#endif
 
     *compressed = COMPRESSED_NOT;
 
+#if defined(RPM_VENDOR_OPENPKG) || defined(RPM_VENDOR_FEDORA) || defined(RPM_VENDOR_MANDRIVA) /* extension-based-compression-detection */
+    file_len = strlen(file);
+    if ((file_len > 4 && strcasecmp(file+file_len-4, ".tbz") == 0)
+     || (file_len > 4 && strcasecmp(file+file_len-4, ".bz2") == 0)) {
+	*compressed = COMPRESSED_BZIP2;
+	return 0;
+    } else
+    if (file_len > 4 && strcasecmp(file+file_len-4, ".zip") == 0) {
+	*compressed = COMPRESSED_ZIP;
+	return 0;
+    } else
+    if ((file_len > 4 && strcasecmp(file+file_len-4, ".tlz") == 0)
+     || (file_len > 5 && strcasecmp(file+file_len-5, ".lzma") == 0)) {
+	*compressed = COMPRESSED_LZMA;
+	return 0;
+    } else
+    if (file_len > 4 && strcasecmp(file+file_len-3, ".xz") == 0) {
+	*compressed = COMPRESSED_XZ;
+	return 0;
+    } else
+    if ((file_len > 4 && strcasecmp(file+file_len-4, ".tgz") == 0)
+     || (file_len > 3 && strcasecmp(file+file_len-3, ".gz") == 0)
+     || (file_len > 2 && strcasecmp(file+file_len-2, ".Z") == 0)) {
+	*compressed = COMPRESSED_OTHER;
+	return 0;
+    } else
+    if (file_len > 5 && strcasecmp(file+file_len-5, ".cpio") == 0) {
+	*compressed = COMPRESSED_NOT;
+	return 0;
+    } else
+    if (file_len > 4 && strcasecmp(file+file_len-4, ".tar") == 0) {
+	*compressed = COMPRESSED_NOT;
+	return 0;
+    }
+#endif
+
     fd = Fopen(file, "r");
     if (fd == NULL || Ferror(fd)) {
 	/* XXX Fstrerror */
@@ -2384,10 +2721,10 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
 	return 1;
     }
     nb = Fread(magic, sizeof(magic[0]), sizeof(magic), fd);
-    if (nb < 0) {
+    if (nb < (ssize_t)0) {
 	rpmlog(RPMLOG_ERR, _("File %s: %s\n"), file, Fstrerror(fd));
 	rc = 1;
-    } else if (nb < sizeof(magic)) {
+    } else if (nb < (ssize_t)sizeof(magic)) {
 	rpmlog(RPMLOG_ERR, _("File %s is smaller than %u bytes\n"),
 		file, (unsigned)sizeof(magic));
 	rc = 0;
@@ -2409,11 +2746,22 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
      &&	magic[2] == 'Z' && magic[3] == 'O')	/* lzop */
 	*compressed = COMPRESSED_LZOP;
     else
+#if !defined(RPM_VENDOR_OPENPKG) && !defined(RPM_VENDOR_FEDORA) && !defined(RPM_VENDOR_MANDRIVA) /* extension-based-compression-detection */
     /* XXX Ick, LZMA has no magic. See http://lkml.org/lkml/2005/6/13/285 */
     if (magic[ 9] == (unsigned char) 0x00 && magic[10] == (unsigned char) 0x00 &&
 	magic[11] == (unsigned char) 0x00 && magic[12] == (unsigned char) 0x00)	/* lzmash */
 	*compressed = COMPRESSED_LZMA;
     else
+#endif
+#if defined(RPM_VENDOR_OPENSUSE)
+    if (magic[0] == 0135 && magic[1] == 0 && magic[2] == 0) {		/* lzma */
+	*compressed = COMPRESSED_LZMA;
+    else
+#endif
+    if (magic[0] == (unsigned char) 0xFD && magic[1] == 0x37 &&	magic[2] == 0x7A
+     && magic[3] == 0x58 && magic[4] == 0x5A && magic[5] == 0x00)		/* xz */
+	*compressed = COMPRESSED_XZ;
+    else
     if ((magic[0] == (unsigned char) 0037 && magic[1] == (unsigned char) 0213)	/* gzip */
      ||	(magic[0] == (unsigned char) 0037 && magic[1] == (unsigned char) 0236)	/* old gzip */
      ||	(magic[0] == (unsigned char) 0037 && magic[1] == (unsigned char) 0036)	/* pack */
@@ -2427,8 +2775,46 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
 /* =============================================================== */
 
 /*@-modfilesys@*/
+/* XXX TODO: merge rpmExpand and rpmMCExpand. gud enuf for now ... */
 char * 
 rpmExpand(const char *arg, ...)
+{
+    MacroContext mc = NULL;
+    const char *s;
+    char *t, *te;
+    size_t sn, tn;
+    size_t bufn = 8 * _macro_BUFSIZ;
+
+    va_list ap;
+
+    if (arg == NULL)
+	return xstrdup("");
+
+    t = xmalloc(bufn + strlen(arg) + 1);
+    *t = '\0';
+    te = stpcpy(t, arg);
+
+    va_start(ap, arg);
+    while ((s = va_arg(ap, const char *)) != NULL) {
+	sn = strlen(s);
+	tn = (te - t);
+	t = xrealloc(t, tn + sn + bufn + 1);
+	te = t + tn;
+	te = stpcpy(te, s);
+    }
+    va_end(ap);
+
+    *te = '\0';
+    tn = (te - t);
+    (void) expandMacros(NULL, mc, t, tn + bufn + 1);
+    t[tn + bufn] = '\0';
+    t = xrealloc(t, strlen(t) + 1);
+    
+    return t;
+}
+
+char * 
+rpmMCExpand(MacroContext mc, const char *arg, ...)
 {
     const char *s;
     char *t, *te;
@@ -2456,7 +2842,7 @@ rpmExpand(const char *arg, ...)
 
     *te = '\0';
     tn = (te - t);
-    (void) expandMacros(NULL, NULL, t, tn + bufn + 1);
+    (void) expandMacros(NULL, mc, t, tn + bufn + 1);
     t[tn + bufn] = '\0';
     t = xrealloc(t, strlen(t) + 1);
     
@@ -2546,10 +2932,11 @@ char *rpmCleanPath(char * path)
 	    if (begin && s[1] == '\0') {
 		/*@switchbreak@*/ break;
 	    }
-	    /* Trim embedded ./ , trailing /. */
-	    if ((t[-1] == '/' && s[1] == '\0') || (t > path && t[-1] == '/' && s[1] == '/')) {
-		s++;
-		continue;
+	    if (t > path && t[-1] == '/')
+	    switch (s[1]) {
+	    case '/':	s++;	/*@fallthrough@*/	/* Trim embedded ./ */
+	    case '\0':	s++;	continue;		/* Trim trailing /. */
+	    default:	/*@innerbreak@*/ break;
 	    }
 	    /* Trim embedded /../ and trailing /.. */
 	    if (!begin && t > path && t[-1] == '/' && s[1] == '.' && (s[2] == '/' || s[2] == '\0')) {
