@@ -44,8 +44,8 @@
 #include "system.h"
 
 #include "rmd128.h"      
-#include "mp.h"
-#include "endianness.h"
+#include <beecrypt/mp.h>
+#include <beecrypt/endianness.h>
 
 #include "debug.h"
 
@@ -137,12 +137,12 @@ void rmd128Process(rmd128Param* mp)
 	register uint32_t aa,bb,cc,dd;
 	register uint32_t aaa,bbb,ccc,ddd;
 	register uint32_t* X;
-	#if WORDS_BIGENDIAN
+	#ifdef WORDS_BIGENDIAN
 	register byte t;
 	#endif
 
 	X = mp->data;
-	#if WORDS_BIGENDIAN
+	#ifdef WORDS_BIGENDIAN
 	t = 16;
 	while (t--)
 	{
