@@ -42,8 +42,8 @@
 #include "system.h"
 
 #include "rmd160.h"      
-#include "mp.h"
-#include "endianness.h"
+#include <beecrypt/mp.h>
+#include <beecrypt/endianness.h>
 
 #include "debug.h"
 
@@ -154,12 +154,12 @@ void rmd160Process(rmd160Param* mp)
 	register uint32_t aa,bb,cc,dd,ee;
 	register uint32_t aaa,bbb,ccc,ddd,eee;
 	register uint32_t* X;
-	#if WORDS_BIGENDIAN
+	#ifdef WORDS_BIGENDIAN
 	register byte t;
 	#endif
 
 	X = mp->data;
-	#if WORDS_BIGENDIAN
+	#ifdef WORDS_BIGENDIAN
 	t = 16;
 	while (t--)
 	{
