@@ -5,6 +5,19 @@
  * \file rpmio/rpmdav.h
  */
 
+#include <argv.h>
+
+/**
+ */
+/*@unchecked@*/
+extern int rpmioHttpReadTimeoutSecs;
+/*@unchecked@*/
+extern int rpmioHttpConnectTimeoutSecs;
+/*@unchecked@*/ /*@null@*/
+extern const char * rpmioHttpAccept;
+/*@unchecked@*/ /*@null@*/
+extern const char * rpmioHttpUserAgent;
+
 #if defined(_RPMDAV_INTERNAL)
 struct __dirstream {
     int fd;			/* File descriptor.  */
@@ -13,7 +26,7 @@ struct __dirstream {
     size_t size;		/* Total valid data in the block.  */
     size_t offset;		/* Current offset into the block.  */
     off_t filepos;		/* Position of next entry to read.  */
-#if defined(HAVE_PTHREAD_H)
+#if defined(WITH_PTHREADS)
     pthread_mutex_t lock;	/* Mutex lock for this structure.  */
 #endif
 };
@@ -32,7 +45,7 @@ struct __dirstream {
 typedef struct __dirstream *	AVDIR;
 typedef struct __dirstream *	DAVDIR;
 #else
-# if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__DragonFly__)
+# if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__DragonFly__) || defined(__OpenBSD__)
 typedef struct __dirstream *	AVDIR;
 typedef struct __dirstream *	DAVDIR;
 #else	/* __FreeBSD__ */
@@ -42,6 +55,15 @@ typedef DIR *			DAVDIR;
 #endif
 
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#if defined(_RPMAV_INTERNAL)
+/**
+ */
+typedef struct avContext_s * avContext;
+
 /**
  */
 /*@unchecked@*/
@@ -50,25 +72,42 @@ extern int avmagicdir;
 
 /**
  */
-/*@unchecked@*/
-extern int davmagicdir;
-#define ISDAVMAGIC(_dir) (!memcmp((_dir), &davmagicdir, sizeof(davmagicdir)))
-
-#ifdef __cplusplus
-extern "C" {
-#endif
+struct avContext_s {
+/*@relnull@*/ /*@dependent@*/
+    void ** resrock;
+    const char *uri;
+/*@refcounted@*/
+    urlinfo u;
+    int ac;
+    int nalloced;
+    ARGV_t av;
+/*@relnull@*/ /*@shared@*/
+    struct stat *st;
+    rpmuint16_t * modes;	/* XXX sizeof(mode_t) != sizeof(rpmmode_t) */
+    size_t * sizes;
+    time_t * mtimes;
+};
 
 /**
  */
-int davFree(urlinfo u)
-	/*@globals internalState @*/
-	/*@modifies u, internalState @*/;
+/*@null@*/
+void * avContextDestroy(/*@only@*/ /*@null@*/ avContext ctx)
+        /*@globals internalState @*/
+        /*@modifies ctx, internalState @*/;
 
 /**
  */
-void davDestroy(void)
-	/*@globals internalState @*/
-	/*@modifies internalState @*/;
+/*@null@*/
+void * avContextCreate(const char *uri, /*@null@*/ struct stat *st)
+        /*@globals internalState @*/
+        /*@modifies *st, internalState @*/;
+
+/**
+ */
+int avContextAdd(avContext ctx, const char * path,
+		mode_t mode, size_t size, time_t mtime)
+        /*@globals internalState @*/
+        /*@modifies ctx, internalState @*/;
 
 /**
  * Close an argv directory.
@@ -90,18 +129,47 @@ struct dirent * avReaddir(DIR * dir)
 	/*@modifies fileSystem @*/;
 
 /**
- * Create an argv directory from URL collection.
- * @param path		URL for collection path
+ * Create an argv directory from an argv array.
+ * @param path		directory path
+ * @param av		argv array
+ * @param modes		element modes (NULL will use pesky trailing '/')
  * @return 		argv DIR
  */
 /*@null@*/
-DIR * avOpendir(const char * path)
+DIR * avOpendir(const char * path,
+		/*@null@*/ const char ** av, /*@null@*/ rpmuint16_t * modes)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
+#endif
+
+/**
+ * Close active neon transfer(s) (if any).
+ * @param _u		URL container
+ * @return		0 on sucess
+ */
+int davDisconnect(void * _u)
+	/*@globals internalState @*/
+	/*@modifies _u, internalState @*/;
+
+/**
+ * Free persistent neon session state.
+ * @param u		URL container
+ * @return		0 on success
+ */
+int davFree(urlinfo u)
+	/*@globals internalState @*/
+	/*@modifies u, internalState @*/;
+
+/**
+ * Free global neon+openSSL state memory.
+ */
+void davDestroy(void)
+	/*@globals internalState @*/
+	/*@modifies internalState @*/;
 
 /**
  * Send a http request.
- * @param ctrl		
+ * @param ctrl		connection stream
  * @param httpCmd	http command
  * @param httpArg	http command argument (NULL if none)
  * @returns		0 on success
@@ -112,8 +180,8 @@ int davReq(FD_t ctrl, const char * httpCmd, /*@null@*/ const char * httpArg)
 
 /**
  * Read a http response.
- * @param u
- * @param cntl		
+ * @param u		URL container
+ * @param ctrl		connection stream
  * @retval *str		error msg		
  * @returns		0 on success
  */
@@ -124,6 +192,12 @@ int davResp(urlinfo u, FD_t ctrl, /*@out@*/ /*@null@*/ char *const * str)
 /*@=exportlocal@*/
 
 /**
+ * Open a URL.
+ * @param url
+ * @param flags
+ * @param mode
+ * @retval *uret
+ * @return
  */
 /*@null@*/
 FD_t davOpen(const char * url, /*@unused@*/ int flags,
@@ -139,13 +213,9 @@ FD_t httpOpen(const char * url, /*@unused@*/ int flags,
 
 /**
  */
-/*@-incondefs@*/
 ssize_t davRead(void * cookie, /*@out@*/ char * buf, size_t count)
-        /*@globals fileSystem, internalState @*/
-        /*@modifies buf, fileSystem, internalState @*/
-	/*@requires maxSet(buf) >= (count - 1) @*/
-	/*@ensures maxRead(buf) == result @*/;
-/*@=incondefs@*/
+        /*@globals errno, fileSystem, internalState @*/
+        /*@modifies buf, errno, fileSystem, internalState @*/;
 
 /**
  */
@@ -215,22 +285,28 @@ struct dirent * davReaddir(DIR * dir)
  */
 /*@null@*/
 DIR * davOpendir(const char * path)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fileSystem, internalState @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * stat(2) clone.
  */
 int davStat(const char * path, /*@out@*/ struct stat * st)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, fileSystem, internalState @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies *st, errno, fileSystem, internalState @*/;
 
 /**
  * lstat(2) clone.
  */
 int davLstat(const char * path, /*@out@*/ struct stat * st)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, fileSystem, internalState @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies *st, errno, fileSystem, internalState @*/;
+
+/**
+ * realpath(3) clone.
+ */
+char * davRealpath(const char * path, /*@out@*/ /*@null@*/ char * resolved_path)
+	/*@modifies *resolved_path @*/;
 
 #ifdef __cplusplus
 }
