@@ -4,9 +4,14 @@
  */
 
 #include "system.h"
+#include <rpmiotypes.h>
+#include <rpmio.h>
 #include <rpmhash.h>
 #include "debug.h"
 
+/*@unchecked@*/
+int _ht_debug = 0;
+
 typedef /*@owned@*/ const void * voidptr;
 
 typedef	struct hashBucket_s * hashBucket;
@@ -23,26 +28,21 @@ struct hashBucket_s {
 /**
  */
 struct hashTable_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
     int numBuckets;			/*!< number of hash buckets */
     size_t keySize;			/*!< size of key (0 if unknown) */
     int freeData;	/*!< should data be freed when table is destroyed? */
     hashBucket * buckets;		/*!< hash bucket array */
+/*@relnull@*/
     hashFunctionType fn;		/*!< generate hash value for key */
+/*@relnull@*/
     hashEqualityType eq;		/*!< compare hash keys for equality */
+#if defined(__LCLINT__)
+/*@refs@*/
+    int nrefs;				/*!< (unused) keep splint happy */
+#endif
 };
 
-/**
- * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
- * @param p		memory to free
- * @retval		NULL always
- */
-/*@unused@*/ static inline /*@null@*/ void *
-_free(/*@only@*/ /*@null@*/ const void * p) /*@modifies p@*/
-{
-    if (p != NULL)	free((void *)p);
-    return NULL;
-}
-
 /**
  * Find entry in hash table.
  * @param ht            pointer to hash table
@@ -53,7 +53,7 @@ static /*@shared@*/ /*@null@*/
 hashBucket findEntry(hashTable ht, const void * key)
 	/*@*/
 {
-    uint32_t hash = 0;
+    rpmuint32_t hash = 0;
     hashBucket b;
 
     /*@-modunconnomods@*/
@@ -67,15 +67,14 @@ hashBucket findEntry(hashTable ht, const void * key)
     return b;
 }
 
-/**
- * Return hash value of a string.
- * @param h		hash initial value
- * @param data		data on which to calculate hash value
- * @param size		size of data in bytes
- * @return		hash value
- */
-static uint32_t hashFunctionString(uint32_t h, const void * data, size_t size)
-	/*@*/
+int hashEqualityString(const void * key1, const void * key2)
+{
+    const char *k1 = (const char *)key1;
+    const char *k2 = (const char *)key2;
+    return strcmp(k1, k2);
+}
+
+rpmuint32_t hashFunctionString(rpmuint32_t h, const void * data, size_t size)
 {
     const char *key = data;
 
@@ -119,64 +118,32 @@ static uint32_t hashFunctionString(uint32_t h, const void * data, size_t size)
     if (h == 0)
 	h = 5381;
     for (; size >= 8; size -= 8) {
-	h = ((h << 5) + h) + (uint32_t)*key++;
-	h = ((h << 5) + h) + (uint32_t)*key++;
-	h = ((h << 5) + h) + (uint32_t)*key++;
-	h = ((h << 5) + h) + (uint32_t)*key++;
-	h = ((h << 5) + h) + (uint32_t)*key++;
-	h = ((h << 5) + h) + (uint32_t)*key++;
-	h = ((h << 5) + h) + (uint32_t)*key++;
-	h = ((h << 5) + h) + (uint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
+	h = ((h << 5) + h) + (rpmuint32_t)*key++;
     }
     switch (size) {
-	case 7: h = ((h << 5) + h) + (uint32_t)*key++; /*@fallthrough@*/
-	case 6: h = ((h << 5) + h) + (uint32_t)*key++; /*@fallthrough@*/
-	case 5: h = ((h << 5) + h) + (uint32_t)*key++; /*@fallthrough@*/
-	case 4: h = ((h << 5) + h) + (uint32_t)*key++; /*@fallthrough@*/
-	case 3: h = ((h << 5) + h) + (uint32_t)*key++; /*@fallthrough@*/
-	case 2: h = ((h << 5) + h) + (uint32_t)*key++; /*@fallthrough@*/
-	case 1: h = ((h << 5) + h) + (uint32_t)*key++; break;
+	case 7: h = ((h << 5) + h) + (rpmuint32_t)*key++; /*@fallthrough@*/
+	case 6: h = ((h << 5) + h) + (rpmuint32_t)*key++; /*@fallthrough@*/
+	case 5: h = ((h << 5) + h) + (rpmuint32_t)*key++; /*@fallthrough@*/
+	case 4: h = ((h << 5) + h) + (rpmuint32_t)*key++; /*@fallthrough@*/
+	case 3: h = ((h << 5) + h) + (rpmuint32_t)*key++; /*@fallthrough@*/
+	case 2: h = ((h << 5) + h) + (rpmuint32_t)*key++; /*@fallthrough@*/
+	case 1: h = ((h << 5) + h) + (rpmuint32_t)*key++; break;
 	default: /* case 0: */ break;
     }
 
     return h;
 }
 
-/**
- * Compare two hash table entries for equality.
- * @param key1          entry 1
- * @param key2          entry 2
- * @return		0 if entries are equal
- */
-static int hashEqualityString(const void * key1, const void * key2)
-	/*@*/
-{
-    const char *k1 = (const char *)key1;
-    const char *k2 = (const char *)key2;
-    return strcmp(k1, k2);
-}
-
-hashTable htCreate(int numBuckets, size_t keySize, int freeData,
-		hashFunctionType fn, hashEqualityType eq)
-{
-    hashTable ht;
-
-    ht = xmalloc(sizeof(*ht));
-    ht->numBuckets = numBuckets;
-    ht->buckets = xcalloc(numBuckets, sizeof(*ht->buckets));
-    ht->keySize = keySize;
-    ht->freeData = freeData;
-    /*@-assignexpose@*/
-    ht->fn = (fn != NULL ? fn : hashFunctionString);
-    ht->eq = (eq != NULL ? eq : hashEqualityString);
-    /*@=assignexpose@*/
-
-    return ht;
-}
-
 void htAddEntry(hashTable ht, const void * key, const void * data)
 {
-    uint32_t hash = 0;
+    rpmuint32_t hash = 0;
     hashBucket b;
 
     hash = ht->fn(hash, key, 0) % ht->numBuckets;
@@ -204,34 +171,6 @@ void htAddEntry(hashTable ht, const void * key, const void * data)
     b->data[b->dataCount++] = data;
 }
 
-hashTable htFree(hashTable ht)
-{
-    hashBucket b, n;
-    int i;
-
-    for (i = 0; i < ht->numBuckets; i++) {
-	b = ht->buckets[i];
-	if (b == NULL)
-	    continue;
-	ht->buckets[i] = NULL;
-	if (ht->keySize > 0)
-	    b->key = _free(b->key);
-	do {
-	    n = b->next;
-	    if (b->data) {
-		if (ht->freeData)
-		    *b->data = _free(*b->data);
-		b->data = _free(b->data);
-	    }
-	    b = _free(b);
-	} while ((b = n) != NULL);
-    }
-
-    ht->buckets = _free(ht->buckets);
-    ht = _free(ht);
-    return NULL;
-}
-
 int htHasEntry(hashTable ht, const void * key)
 {
     hashBucket b;
@@ -256,3 +195,90 @@ int htGetEntry(hashTable ht, const void * key, const void * data,
 
     return 0;
 }
+
+const void ** htGetKeys(hashTable ht)
+{
+    const void ** keys = xcalloc(ht->numBuckets+1, sizeof(const void*));
+    const void ** keypointer = keys;
+    hashBucket b, n;
+    int i;
+
+    for (i = 0; i < ht->numBuckets; i++) {
+	b = ht->buckets[i];
+	if (b == NULL)
+	    continue;
+	if (b->data)
+	    *(keys++) = b->key;
+	do {
+	    n = b->next;
+	    if(n != NULL)
+    		*(keys++) = n->key;
+	} while ((b = n) != NULL);
+    }
+
+    return keypointer;
+}
+
+/*@-mustmod@*/	/* XXX splint on crack */
+static void htFini(void * _ht)
+	/*@modifies _ht @*/
+{
+    hashTable ht = _ht;
+    hashBucket b, n;
+    int i;
+
+    for (i = 0; i < ht->numBuckets; i++) {
+	b = ht->buckets[i];
+	if (b == NULL)
+	    continue;
+	ht->buckets[i] = NULL;
+	if (ht->keySize > 0)
+	    b->key = _free(b->key);
+	do {
+	    n = b->next;
+	    if (b->data) {
+		if (ht->freeData)
+		    *b->data = _free(*b->data);
+		b->data = _free(b->data);
+	    }
+	    b = _free(b);
+	} while ((b = n) != NULL);
+    }
+
+    ht->buckets = _free(ht->buckets);
+}
+/*@=mustmod@*/
+
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _htPool;
+
+static hashTable htGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _htPool, fileSystem @*/
+	/*@modifies pool, _htPool, fileSystem @*/
+{
+    hashTable ht;
+
+    if (_htPool == NULL) {
+	_htPool = rpmioNewPool("ht", sizeof(*ht), -1, _ht_debug,
+			NULL, NULL, htFini);
+	pool = _htPool;
+    }
+    return (hashTable) rpmioGetPool(pool, sizeof(*ht));
+}
+
+hashTable htCreate(int numBuckets, size_t keySize, int freeData,
+		hashFunctionType fn, hashEqualityType eq)
+{
+    hashTable ht = htGetPool(_htPool);
+
+    ht->numBuckets = numBuckets;
+    ht->buckets = xcalloc(numBuckets, sizeof(*ht->buckets));
+    ht->keySize = keySize;
+    ht->freeData = freeData;
+    /*@-assignexpose@*/
+    ht->fn = (fn != NULL ? fn : hashFunctionString);
+    ht->eq = (eq != NULL ? eq : hashEqualityString);
+    /*@=assignexpose@*/
+
+    return htLink(ht);
+}
