@@ -9,13 +9,13 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
-/*@-noparams@*/
-#include "glob.h"
-/*@=noparams@*/
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <rpmiotypes.h>
+#include <rpmzlog.h>
+
 /** \ingroup rpmio
  * Hide libio API lossage.
  * The libio interface changed after glibc-2.1.3 to pass the seek offset
@@ -23,7 +23,7 @@
  * typedefs to isolate the lossage.
  */
 /*@{*/
-#if !defined(__LCLINT__) && defined(__GLIBC__) && \
+#if !defined(__LCLINT__) && !defined(__UCLIBC__) && defined(__GLIBC__) && \
 	(__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 2))
 #define USE_COOKIE_SEEK_POINTER 1
 typedef _IO_off64_t 	_libio_off_t;
@@ -77,117 +77,24 @@ typedef int (*fdio_close_function_t) (void *cookie)
 	/*@globals errno, fileSystem, systemState @*/
 	/*@modifies *cookie, errno, fileSystem, systemState @*/;
 
-
-/**
- */
-typedef /*@only@*/ /*@null@*/ FD_t (*fdio_ref_function_t) ( /*@only@*/ void * cookie,
-		const char * msg, const char * file, unsigned line)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-
-/**
- */
-typedef /*@only@*/ /*@null@*/ FD_t (*fdio_deref_function_t) ( /*@only@*/ FD_t fd,
-		const char * msg, const char * file, unsigned line)
-	/*@globals fileSystem @*/
-	/*@modifies fd, fileSystem @*/;
-
-
-/**
- */
-typedef /*@only@*/ /*@null@*/ FD_t (*fdio_new_function_t) (const char * msg,
-		const char * file, unsigned line)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-
-
-/**
- */
-typedef int (*fdio_fileno_function_t) (void * cookie)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, fileSystem @*/;
-
-
-/**
- */
-typedef FD_t (*fdio_open_function_t) (const char * path, int flags, mode_t mode)
-	/*@globals errno, fileSystem @*/
-	/*@modifies errno, fileSystem @*/;
-
 /**
  */
 typedef FD_t (*fdio_fopen_function_t) (const char * path, const char * fmode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-
-/**
- */
-typedef void * (*fdio_ffileno_function_t) (FD_t fd)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-
-/**
- */
-typedef int (*fdio_fflush_function_t) (FD_t fd)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-/*@}*/
-
-
-/** \ingroup rpmrpc
- * \name RPMRPC Vectors.
- */
-/*@{*/
-
-/**
- */
-typedef int (*fdio_mkdir_function_t) (const char * path, mode_t mode)
 	/*@globals errno, fileSystem @*/
 	/*@modifies errno, fileSystem @*/;
 
 /**
  */
-typedef int (*fdio_chdir_function_t) (const char * path)
+typedef FD_t (*fdio_fdopen_function_t) (void * cookie, const char * fmode)
 	/*@globals errno, fileSystem @*/
 	/*@modifies errno, fileSystem @*/;
 
 /**
  */
-typedef int (*fdio_rmdir_function_t) (const char * path)
+typedef int (*fdio_flush_function_t) (void * cookie)
 	/*@globals errno, fileSystem @*/
 	/*@modifies errno, fileSystem @*/;
 
-/**
- */
-typedef int (*fdio_rename_function_t) (const char * oldpath, const char * newpath)
-	/*@globals errno, fileSystem @*/
-	/*@modifies errno, fileSystem @*/;
-
-/**
- */
-typedef int (*fdio_unlink_function_t) (const char * path)
-	/*@globals errno, fileSystem @*/
-	/*@modifies errno, fileSystem @*/;
-/*@-typeuse@*/
-
-/**
- */
-typedef int (*fdio_stat_function_t) (const char * path, /*@out@*/ struct stat * st)
-	/*@globals errno, fileSystem @*/
-	/*@modifies *st, errno, fileSystem @*/;
-
-/**
- */
-typedef int (*fdio_lstat_function_t) (const char * path, /*@out@*/ struct stat * st)
-	/*@globals errno, fileSystem @*/
-	/*@modifies *st, errno, fileSystem @*/;
-
-/**
- */
-typedef int (*fdio_access_function_t) (const char * path, int amode)
-	/*@globals errno, fileSystem @*/
-	/*@modifies errno, fileSystem @*/;
-/*@=typeuse@*/
 /*@}*/
 
 
@@ -198,22 +105,12 @@ struct FDIO_s {
   fdio_write_function_t		write;
   fdio_seek_function_t		seek;
   fdio_close_function_t		close;
-
-  fdio_ref_function_t		_fdref;
-  fdio_deref_function_t		_fdderef;
-  fdio_new_function_t		_fdnew;
-  fdio_fileno_function_t	_fileno;
-
-  fdio_open_function_t		_open;
+/*@null@*/
   fdio_fopen_function_t		_fopen;
-  fdio_ffileno_function_t	_ffileno;
-  fdio_fflush_function_t	_fflush;
-
-  fdio_mkdir_function_t		_mkdir;
-  fdio_chdir_function_t		_chdir;
-  fdio_rmdir_function_t		_rmdir;
-  fdio_rename_function_t	_rename;
-  fdio_unlink_function_t	_unlink;
+/*@null@*/
+  fdio_fdopen_function_t	_fdopen;
+/*@null@*/
+  fdio_flush_function_t		_flush;
 };
 
 
@@ -235,8 +132,7 @@ struct FDIO_s {
 size_t Fread(/*@out@*/ void * buf, size_t size, size_t nmemb, FD_t fd)
 	/*@globals fileSystem @*/
 	/*@modifies fd, *buf, fileSystem @*/
-	/*@requires maxSet(buf) >= (nmemb - 1) @*/
-	/*@ensures maxRead(buf) == result @*/;
+	/*@requires maxSet(buf) >= (nmemb - 1) @*/;
 /*@=incondefs@*/
 
 /**
@@ -388,6 +284,13 @@ int Lstat(const char * path, /*@out@*/ struct stat * st)
 	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies *st, errno, fileSystem, internalState @*/;
 
+/**
+ * fstat(2) clone.
+ */
+int Fstat(FD_t fd, /*@out@*/ struct stat * st)
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies *st, errno, fileSystem, internalState @*/;
+
 /**
  * chown(2) clone.
  * @todo Implement remotely.
@@ -396,6 +299,14 @@ int Chown(const char * path, uid_t owner, gid_t group)
 	/*@globals errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
+/**
+ * fchown(2) clone.
+ * @todo Implement remotely.
+ */
+int Fchown(FD_t fd, uid_t owner, gid_t group)
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
+
 /**
  * lchown(2) clone.
  * @todo Implement remotely.
@@ -412,6 +323,14 @@ int Chmod(const char * path, mode_t mode)
 	/*@globals errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
+/**
+ * fchmod(2) clone.
+ * @todo Implement remotely.
+ */
+int Fchmod(FD_t fd, mode_t mode)
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
+
 /**
  * mkfifo(3) clone.
  * @todo Implement remotely.
@@ -460,9 +379,7 @@ int Symlink(const char * oldpath, const char * newpath)
 /*@-incondefs@*/
 int Readlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
 	/*@globals errno, h_errno, fileSystem, internalState @*/
-	/*@modifies *buf, errno, fileSystem, internalState @*/
-	/*@requires maxSet(buf) >= (bufsiz - 1) @*/
-	/*@ensures maxRead(buf) <= bufsiz @*/;
+	/*@modifies *buf, errno, fileSystem, internalState @*/;
 /*@=incondefs@*/
 
 /**
@@ -515,16 +432,16 @@ int Glob_error(const char * epath, int eerrno)
  */
 int Glob(const char * pattern, int flags,
 		int errfunc(const char * epath, int eerrno),
-		/*@out@*/ glob_t * pglob)
+		/*@out@*/ void * _pglob)
 	/*@globals fileSystem @*/
-	/*@modifies *pglob, fileSystem @*/;
+	/*@modifies *_pglob, fileSystem @*/;
 
 /**
  * globfree(3) clone.
  */
-void Globfree( /*@only@*/ glob_t * pglob)
+void Globfree( /*@only@*/ void * _pglob)
 	/*@globals fileSystem @*/
-	/*@modifies *pglob, fileSystem @*/;
+	/*@modifies *_pglob, fileSystem @*/;
 
 
 /**
@@ -550,6 +467,16 @@ int Closedir(/*@only@*/ DIR * dir)
 	/*@globals errno, fileSystem @*/
 	/*@modifies *dir, errno, fileSystem @*/;
 
+/**
+ * realpath(3) clone.
+ */
+/*@-globuse@*/
+/*@null@*/
+char * Realpath(const char * path, /*@out@*/ /*@null@*/ char * resolved_path)
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies resolved_path, errno, fileSystem, internalState @*/;
+/*@=globuse@*/
+
 /**
  * lseek(2) clone.
  * @todo Implement SEEK_HOLE/SEEK_DATA.
@@ -566,50 +493,19 @@ off_t Lseek(int fdno, off_t offset, int whence)
  */
 /*@{*/
 
-/**
- */
-off_t	fdSize(FD_t fd)
-	/*@globals fileSystem @*/
-	/*@modifies fd, fileSystem@*/;
-
 /**
  */
 /*@null@*/ FD_t fdDup(int fdno)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
 
-#ifdef UNUSED
-/*@null@*/ FILE *fdFdopen( /*@only@*/ void * cookie, const char * mode);
-#endif
-
-/* XXX Legacy interfaces needed by gnorpm, rpmfind et al */
-
 /*@-exportlocal@*/
 /**
  */
-#ifndef H_RPMIO_INTERNAL	/* XXX avoid gcc warning */
-/*@unused@*/ int fdFileno(void * cookie)
-	/*@*/;
-#define	fdFileno(_fd)		fdio->_fileno(_fd)
-#endif
-
-/**
- */
-/*@null@*/ FD_t fdOpen(const char *path, int flags, mode_t mode)
-	/*@globals errno, fileSystem, internalState @*/
-	/*@modifies errno, fileSystem, internalState @*/;
-#define	fdOpen(_path, _flags, _mode)	fdio->_open((_path), (_flags), (_mode))
-
-/**
- */
-/*@-incondefs@*/
 ssize_t fdRead(void * cookie, /*@out@*/ char * buf, size_t count)
 	/*@globals errno, fileSystem, internalState @*/
-	/*@modifies *cookie, *buf, errno, fileSystem, internalState @*/
-	/*@requires maxSet(buf) >= (count - 1) @*/
-	/*@ensures maxRead(buf) == result @*/ ;
+	/*@modifies *cookie, *buf, errno, fileSystem, internalState @*/;
 #define	fdRead(_fd, _buf, _count)	fdio->read((_fd), (_buf), (_count))
-/*@=incondefs@*/
 
 /**
  */
@@ -625,32 +521,44 @@ int fdClose( /*@only@*/ void * cookie)
 	/*@modifies *cookie, errno, fileSystem, systemState, internalState @*/;
 #define	fdClose(_fd)		fdio->close(_fd)
 
+/**
+ */
+/*@null@*/ FD_t fdOpen(const char *path, int flags, mode_t mode)
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
+#define	fdOpen(_path, _flags, _mode)	fdio->_open((_path), (_flags), (_mode))
+
 /**
  */
 /*@unused@*/
-/*@only@*/ /*@null@*/
-FD_t fdLink (/*@only@*/ void * cookie, const char * msg)
+/*@newref@*/ /*@null@*/
+FD_t fdLink (void * cookie, const char * msg)
 	/*@globals fileSystem @*/
 	/*@modifies *cookie, fileSystem @*/;
-#define	fdLink(_fd, _msg)	fdio->_fdref(_fd, _msg, __FILE__, __LINE__)
+#define	fdLink(_fd, _msg)	\
+	((FD_t)rpmioLinkPoolItem((rpmioItem)(_fd), _msg, __FILE__, __LINE__))
 
 /**
  */
-/*@unused@*/
-/*@only@*/ /*@null@*/
-FD_t fdFree(/*@only@*/ FD_t fd, const char * msg)
+/*@unused@*/ /*@null@*/
+FD_t fdFree(/*@killref@*/ FD_t fd, const char * msg)
 	/*@globals fileSystem @*/
 	/*@modifies fd, fileSystem @*/;
-#define	fdFree(_fd, _msg)	fdio->_fdderef(_fd, _msg, __FILE__, __LINE__)
+#define	fdFree(_fd, _msg)	\
+	((FD_t)rpmioFreePoolItem((rpmioItem)(_fd), _msg, __FILE__, __LINE__))
 
 /**
  */
 /*@unused@*/
-/*@only@*/ /*@null@*/
+/*@newref@*/ /*@null@*/
 FD_t fdNew (const char * msg)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
-#define	fdNew(_msg)		fdio->_fdnew(_msg, __FILE__, __LINE__)
+/*@newref@*/ /*@null@*/
+FD_t XfdNew (const char * msg, const char * fn, unsigned ln)
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/;
+#define	fdNew(_msg)		XfdNew(_msg, __FILE__, __LINE__)
 
 /**
  */
@@ -733,21 +641,25 @@ typedef enum ftperrCode_e {
 /**
  */
 /*@-redecl@*/
-/*@observer@*/ const char * ftpStrerror(int errorNumber)
+/*@observer@*/
+const char * ftpStrerror(int errorNumber)
 	/*@*/;
 /*@=redecl@*/
 
 /**
  */
 /*@unused@*/
-/*@dependent@*/ /*@null@*/ void * ufdGetUrlinfo(FD_t fd)
-	/*@modifies fd @*/;
+/*@dependent@*/ /*@null@*/
+void * ufdGetUrlinfo(FD_t fd)
+	/*@globals fileSystem @*/
+	/*@modifies fd, fileSystem @*/;
 
 /**
  */
 /*@-redecl@*/
 /*@unused@*/
-/*@observer@*/ const char * urlStrerror(const char * url)
+/*@observer@*/
+const char * urlStrerror(const char * url)
 	/*@globals h_errno, internalState @*/
 	/*@modifies internalState @*/;
 /*@=redecl@*/
@@ -793,96 +705,111 @@ int ufdGetFile( /*@killref@*/ FD_t sfd, FD_t tfd)
 
 /**
  */
-/*@observer@*/ /*@unchecked@*/ extern FDIO_t fadio;
+/*@observer@*/ /*@unchecked@*/ extern FDIO_t xzdio;
+
 /*@=exportlocal@*/
 /*@}*/
 
-/*@unused@*/ static inline int xislower(int c) /*@*/ {
-    return (c >= (int)'a' && c <= (int)'z');
-}
-/*@unused@*/ static inline int xisupper(int c) /*@*/ {
-    return (c >= (int)'A' && c <= (int)'Z');
-}
-/*@unused@*/ static inline int xisalpha(int c) /*@*/ {
-    return (xislower(c) || xisupper(c));
-}
-/*@unused@*/ static inline int xisdigit(int c) /*@*/ {
-    return (c >= (int)'0' && c <= (int)'9');
-}
-/*@unused@*/ static inline int xisalnum(int c) /*@*/ {
-    return (xisalpha(c) || xisdigit(c));
-}
-/*@unused@*/ static inline int xisblank(int c) /*@*/ {
-    return (c == (int)' ' || c == (int)'\t');
-}
-/*@unused@*/ static inline int xisspace(int c) /*@*/ {
-    return (xisblank(c) || c == (int)'\n' || c == (int)'\r' || c == (int)'\f' || c == (int)'\v');
-}
-/*@unused@*/ static inline int xiscntrl(int c) /*@*/ {
-    return (c < (int)' ');
-}
-/*@unused@*/ static inline int xisascii(int c) /*@*/ {
-    return ((c & 0x80) != 0x80);
-}
-/*@unused@*/ static inline int xisprint(int c) /*@*/ {
-    return (c >= (int)' ' && xisascii(c));
-}
-/*@unused@*/ static inline int xisgraph(int c) /*@*/ {
-    return (c > (int)' ' && xisascii(c));
-}
-/*@unused@*/ static inline int xispunct(int c) /*@*/ {
-    return (xisgraph(c) && !xisalnum(c));
-}
-
-/*@unused@*/ static inline int xtolower(int c) /*@*/ {
-    return ((xisupper(c)) ? (c | ('a' - 'A')) : c);
-}
-/*@unused@*/ static inline int xtoupper(int c) /*@*/ {
-    return ((xislower(c)) ? (c & ~('a' - 'A')) : c);
-}
-
-/** \ingroup rpmio
- * Locale insensitive strcasecmp(3).
- */
-int xstrcasecmp(const char * s1, const char * s2)		/*@*/;
-
-/** \ingroup rpmio
- * Locale insensitive strncasecmp(3).
- */
-int xstrncasecmp(const char *s1, const char * s2, size_t n)	/*@*/;
-
-/** \ingroup rpmio
- * Force encoding of string.
- */
-/*@only@*/ /*@null@*/
-const char * xstrtolocale(/*@only@*/ const char *str)
-	/*@modifies *str @*/;
-
-#if !defined(SWIG)
-/**
- * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
- * @param p		memory to free
- * @return		NULL always
- */
-#if defined(WITH_DMALLOC)
-#define _free(p) ((p) != NULL ? free((void *)(p)) : (void)0, NULL)
-#else
-/*@unused@*/ static inline /*@null@*/
-void * _free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
-	/*@modifies p @*/
-{
-    if (p != NULL)	free((void *)p);
-    return NULL;
-}
-#endif
-#endif
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+extern rpmioPool _fdPool;
 
 /**
  * Free all memory allocated by rpmio usage.
  */
 void rpmioClean(void)
-	/*@globals internalState, fileSystem @*/
-	/*@modifies internalState, fileSystem @*/;
+	/*@globals _fdPool, fileSystem, internalState @*/
+	/*@modifies _fdPool, fileSystem, internalState @*/;
+
+/**
+ * Reclaim memory pool items.
+ * @param pool		memory pool (NULL uses global rpmio pool)
+ * @return		NULL always
+ */
+/*@null@*/
+rpmioPool rpmioFreePool(/*@only@*//*@null@*/ rpmioPool pool)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies pool, fileSystem, internalState @*/;
+
+/**
+ * Create a memory pool.
+ * @param name		pool name
+ * @param size		item size
+ * @param limit		no. of items permitted (-1 for unlimited)
+ * @param flags		debugging flags
+ * @param (*dbg)()	generate string for Unlink/Link/Free debugging
+ * @param (*init)()	create item contents
+ * @param (*fini)()	destroy item contents
+ * @return		memory pool
+ */
+rpmioPool rpmioNewPool(/*@observer@*/ const char * name,
+		size_t size, int limit, int flags,
+		/*@null@*/ const char * (*dbg) (void *item), 
+		/*@null@*/ void (*init) (void *item),
+		/*@null@*/ void (*fini) (void *item))
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/;
+
+/**
+ * Decrement a pool item refcount.
+ * @param item		pool item
+ * @param msg		debugging msg (NULL disables debugging)
+ * @param fn		usually __FILE__
+ * @param ln		usually __LINE__
+ * @return		pool item (NULL on last dereference)
+ */
+/*@null@*/
+rpmioItem rpmioUnlinkPoolItem(/*@killref@*/ /*@null@*/ rpmioItem item,
+		const char * msg, const char * fn, unsigned ln)
+	/*@globals fileSystem @*/
+	/*@modifies item, fileSystem @*/;
+
+/**
+ * Increment a pool item refcount.
+ * @param item		pool item
+ * @param msg		debugging msg (NULL disables debugging)
+ * @param fn		usually __FILE__
+ * @param ln		usually __LINE__
+ * @return		pool item
+ */
+/*@newref@*/ /*@null@*/
+rpmioItem rpmioLinkPoolItem(/*@returned@*/ /*@null@*/ rpmioItem item,
+		const char * msg, const char * fn, unsigned ln)
+	/*@globals fileSystem @*/
+	/*@modifies item, fileSystem @*/;
+
+/**
+ * Free a pool item.
+ * @param item		pool item
+ * @param msg		debugging msg (NULL disables debugging)
+ * @param fn		usually __FILE__
+ * @param ln		usually __LINE__
+ * @return		pool item (NULL on last dereference)
+ */
+/*@null@*/
+void * rpmioFreePoolItem(/*@killref@*/ /*@null@*/ rpmioItem item,
+		const char * msg, const char * fn, unsigned ln)
+	/*@globals fileSystem @*/
+	/*@modifies item, fileSystem @*/;
+
+/**
+ * Get unused item from pool, or alloc a new item.
+ * @param pool		memory pool (NULL will always alloc a new item)
+ * @param size		item size
+ * @return		new item
+ */
+rpmioItem rpmioGetPool(/*@kept@*/ /*@null@*/ rpmioPool pool, size_t size)
+	/*@globals fileSystem @*/
+        /*@modifies pool, fileSystem @*/;
+
+/**
+ * Put unused item into pool (or free).
+ * @param _item		unused item
+ * @return		NULL always
+ */
+/*@null@*/
+rpmioItem rpmioPutPool(rpmioItem item)
+	/*@globals fileSystem @*/
+        /*@modifies item, fileSystem @*/;
 
 #ifdef __cplusplus
 }
