@@ -5,6 +5,8 @@
  * \file rpmio/rpmio_internal.h
  */
 
+#include <rpmiotypes.h>
+#include <rpmlog.h>
 #include <rpmio.h>
 #include <rpmurl.h>
 
@@ -13,6 +15,9 @@
 
 #include <rpmxar.h>
 
+/*@access pgpDig @*/	/* XXX FIXME: (by refactoring to foo.c) */
+/*@access rpmxar @*/	/* XXX FIXME: (by refactoring to foo.c) */
+
 /** \ingroup rpmio
  */
 typedef struct _FDSTACK_s {
@@ -53,8 +58,7 @@ typedef struct _FDDIGEST_s {
  * The FD_t File Handle data structure.
  */
 struct _FD_s {
-/*@refs@*/
-    int		nrefs;
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
     int		flags;
 #define	RPMIO_DEBUG_IO		0x40000000
 #define	RPMIO_DEBUG_REFS	0x20000000
@@ -64,7 +68,7 @@ struct _FD_s {
     FDSTACK_t	fps[8];
     int		urlType;	/* ufdio: */
 
-/*@dependent@*/
+/*@dependent@*/ /*@relnull@*/
     void *	url;		/* ufdio: URL info */
 /*@relnull@*/
     void *	req;		/* ufdio: HTTP request */
@@ -92,15 +96,24 @@ struct _FD_s {
     FDSTAT_t	stats;		/* I/O statistics */
 
     int		ndigests;
-#define	FDDIGEST_MAX	4
+#define	FDDIGEST_MAX	32
     struct _FDDIGEST_s	digests[FDDIGEST_MAX];
 
+/*null@*/
+    const char *contentType;	/* ufdio: (HTTP) */
+/*null@*/
+    const char *contentDisposition;	/* ufdio: (HTTP) */
+    time_t	lastModified;	/* ufdio: (HTTP) */
     int		ftpFileDoneNeeded; /* ufdio: (FTP) */
     unsigned long long	fd_cpioPos;	/* cpio: */
+#if defined(__LCLINT__)
+/*@refs@*/
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 /*@access FD_t@*/
 
-#define	FDSANE(fd)	assert(fd && fd->magic == FDMAGIC)
+#define	FDSANE(fd)	assert(fd != NULL && fd->magic == FDMAGIC)
 
 /*@-redecl@*/
 /*@unchecked@*/
@@ -139,6 +152,11 @@ extern int _dav_debug;
 extern "C" {
 #endif
 
+/** \ingroup rpmio
+ */
+/*@observer@*/ const char * fdbg(/*@null@*/ FD_t fd)
+        /*@*/;
+
 /** \ingroup rpmio
  */
 int fdFgets(FD_t fd, char * buf, size_t len)
@@ -220,11 +238,14 @@ mode_t fdGetOMode(FD_t fd)
  */
 /*@unused@*/ static inline
 void fdSetDig(FD_t fd, pgpDig dig)
-	/*@modifies fd, dig @*/
+	/*@globals fileSystem @*/
+	/*@modifies fd, dig, fileSystem @*/
 {
     FDSANE(fd);
-    fd->dig = pgpDigFree(fd->dig);
+/*@-assignexpose -castexpose @*/
+    fd->dig = pgpDigFree(fd->dig, "fdSetDig");
     fd->dig = pgpDigLink(dig, "fdSetDig");
+/*@=assignexpose =castexpose @*/
 }
 
 /** \ingroup rpmio
@@ -243,10 +264,13 @@ void fdSetDig(FD_t fd, pgpDig dig)
  */
 /*@unused@*/ static inline
 void fdSetXAR(FD_t fd, rpmxar xar)
-	/*@modifies fd, xar @*/
+	/*@globals fileSystem @*/
+	/*@modifies fd, xar, fileSystem @*/
 {
     FDSANE(fd);
+/*@-assignexpose -castexpose @*/
     fd->xar = rpmxarLink(xar, "fdSetXAR");
+/*@=assignexpose =castexpose @*/
 }
 
 /** \ingroup rpmio
@@ -358,7 +382,7 @@ void fdPush(FD_t fd, FDIO_t io, void * fp, int fdno)
 	/*@modifies fd @*/
 {
     FDSANE(fd);
-    if (fd->nfps >= (sizeof(fd->fps)/sizeof(fd->fps[0]) - 1))
+    if (fd->nfps >= (int)(sizeof(fd->fps)/sizeof(fd->fps[0]) - 1))
 	return;
     fd->nfps++;
     fdSetIo(fd, io);
@@ -388,7 +412,7 @@ rpmop fdstat_op(/*@null@*/ FD_t fd, fdOpX opx)
 {
     rpmop op = NULL;
 
-    if (fd != NULL && fd->stats != NULL && opx >= 0 && opx < FDSTAT_MAX)
+    if (fd != NULL && fd->stats != NULL && (int)opx >= 0 && opx < FDSTAT_MAX)
         op = fd->stats->ops + opx;
     return op;
 }
@@ -444,13 +468,13 @@ void fdstat_print(/*@null@*/ FD_t fd, const char * msg, FILE * fp)
 	if (op->count <= 0) continue;
 	switch (opx) {
 	case FDSTAT_READ:
-	    if (msg) fprintf(fp, "%s:", msg);
+	    if (msg != NULL) fprintf(fp, "%s:", msg);
 	    fprintf(fp, "%8d reads, %8lu total bytes in %d.%06d secs\n",
 		op->count, (unsigned long)op->bytes,
 		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
 	    /*@switchbreak@*/ break;
 	case FDSTAT_WRITE:
-	    if (msg) fprintf(fp, "%s:", msg);
+	    if (msg != NULL) fprintf(fp, "%s:", msg);
 	    fprintf(fp, "%8d writes, %8lu total bytes in %d.%06d secs\n",
 		op->count, (unsigned long)op->bytes,
 		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
@@ -585,8 +609,8 @@ void fdFiniDigest(FD_t fd, pgpHashAlgo hashalgo,
 	break;
     }
     if (i < 0) {
-	if (datap) *(void **)datap = NULL;
-	if (lenp) *lenp = 0;
+	if (datap != NULL) *(void **)datap = NULL;
+	if (lenp != NULL) *lenp = 0;
     }
 
     fd->ndigests = imax;
@@ -649,18 +673,6 @@ int fdFileno(/*@null@*/ void * cookie)
 }
 /*@=shadow@*/
 
-/**
- * Read an entire file into a buffer.
- * @param fn		file name to read
- * @retval *bp		(malloc'd) buffer address
- * @retval *blenp	(malloc'd) buffer length
- * @return		0 on success
- */
-int rpmioSlurp(const char * fn,
-                /*@out@*/ uint8_t ** bp, /*@out@*/ ssize_t * blenp)
-        /*@globals h_errno, fileSystem, internalState @*/
-        /*@modifies *bp, *blenp, fileSystem, internalState @*/;
-
 #ifdef __cplusplus
 }
 #endif
