@@ -1,7 +1,9 @@
-/*@-realcompare -sizeoftype -protoparammatch @*/
+/*@-moduncon -mustmod -realcompare -sizeoftype @*/
 #include "system.h"
 
 #ifdef	WITH_LUA
+#define	_RPMIOB_INTERNAL
+#include <rpmiotypes.h>
 #include <rpmio.h>
 #include <rpmmacro.h>
 #include <rpmlog.h>
@@ -9,14 +11,28 @@
 #include <rpmhook.h>
 #include <rpmcb.h>
 #include <argv.h>
+#include <popt.h>		/* XXX poptSaneFile test */
 
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
+#ifdef	WITH_SYCK
+LUALIB_API int luaopen_syck(lua_State *L)
+	/*@modifies L @*/;
+#endif	/* WITH_SYCK */
 #ifdef WITH_LUA_INTERNAL
 #include <llocal.h>
 #include <lposix.h>
 #include <lrexlib.h>
+#include <luuid.h>
+#include <lwrs.h>
+#ifdef	USE_LUA_CRYPTO		/* XXX external lua modules instead. */
+#include <lcrypto.h>
+#include <lxplib.h>
+#endif
+#ifdef	USE_LUA_SOCKET		/* XXX external lua modules instead. */
+#include <luasocket.h>
+#endif
 #endif
 
 #include <unistd.h>
@@ -27,8 +43,25 @@
 
 #include "debug.h"
 
+/*@access rpmiob @*/
+
+#else /* WITH_LUA */
+#include <rpmio.h>
+#endif
+
+/*@unchecked@*/
+int _rpmlua_debug = 0;
+
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _rpmluaPool = NULL;
+
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _rpmluavPool = NULL;
+
+#ifdef	WITH_LUA
+
 #if !defined(HAVE_VSNPRINTF)
-static inline int vsnprintf(char * buf, /*@unused@*/ int nb,
+static inline int vsnprintf(char * buf, /*@unused@*/ size_t nb,
 			    const char * fmt, va_list ap)
 {
     return vsprintf(buf, fmt, ap);
@@ -43,8 +76,8 @@ static inline int vsnprintf(char * buf, /*@unused@*/ int nb,
 			/*@=mods@*/ \
 	    )
 
-/*@only@*/ /*@unchecked@*/ /*@null@*/
-static rpmlua globalLuaState = NULL;
+/*@only@*/ /*@unchecked@*/ /*@relnull@*/
+static rpmlua globalLuaState;
 
 static int luaopen_rpm(lua_State *L)
 	/*@modifies L @*/;
@@ -52,17 +85,58 @@ static int rpm_print(lua_State *L)
 	/*@globals fileSystem @*/
 	/*@modifies L, fileSystem @*/;
 
+/*@unchecked@*/ /*@observer@*/
+const char * rpmluaFiles = RPMLUAFILES;
+
+/*@unchecked@*/ /*@observer@*/
+const char * rpmluaPath = "%{?_rpmhome}%{!?_rpmhome:" USRLIBRPM "}/lua/?.lua";
+
 rpmlua rpmluaGetGlobalState(void)
 {
+/*@-globstate@*/
     return globalLuaState;
+/*@=globstate@*/
 }
 
-/*@-mods@*/	/* XXX hide rpmGlobalMacroContext mods for now. */
-rpmlua rpmluaNew()
+static void rpmluaFini(void * _lua)
+	/*@globals globalLuaState @*/
+	/*@modifies globalLuaState @*/
 {
-    rpmlua lua = (rpmlua) xcalloc(1, sizeof(*lua));
+    rpmlua lua = _lua;
+
+    if (lua->L) lua_close(lua->L);
+    lua->L = NULL;
+    lua->printbuf = _free(lua->printbuf);
+}
+
+static rpmlua rpmluaGetPool(/*@null@*/ rpmioPool pool)
+        /*@globals _rpmluaPool, fileSystem @*/
+        /*@modifies pool, _rpmluaPool, fileSystem @*/
+{
+    rpmlua lua;
+
+    if (_rpmluaPool == NULL) {
+        _rpmluaPool = rpmioNewPool("lua", sizeof(*lua), -1, _rpmlua_debug,
+                        NULL, NULL, rpmluaFini);
+        pool = _rpmluaPool;
+    }
+    return (rpmlua) rpmioGetPool(pool, sizeof(*lua));
+}
+
+void *rpmluaFree(rpmlua lua)
+{
+    if (lua == NULL) lua = globalLuaState;
+    (void)rpmioFreePoolItem((rpmioItem)lua, __FUNCTION__, __FILE__, __LINE__);
+    if (lua == globalLuaState) globalLuaState = NULL;
+    return NULL;
+}
+
+/*@-globs -mods@*/	/* XXX hide rpmGlobalMacroContext mods for now. */
+rpmlua rpmluaNew(void)
+{
+    rpmlua lua = rpmluaGetPool(_rpmluaPool);
     lua_State *L = lua_open();
-    /*@-readonlytrans@*/
+    /*@-readonlytrans -nullassign @*/
     /*@observer@*/ /*@unchecked@*/
     static const luaL_reg lualibs[] = {
 	/* standard LUA libraries */
@@ -74,18 +148,34 @@ rpmlua rpmluaNew()
 	{LUA_STRLIBNAME, luaopen_string},
 	{LUA_MATHLIBNAME, luaopen_math},
 	{LUA_DBLIBNAME, luaopen_debug},
+#ifdef	WITH_SYCK
+	{"lsyck", luaopen_syck},
+#endif	/* WITH_SYCK */
 	/* local LUA libraries (RPM only) */
 #ifdef WITH_LUA_INTERNAL
 	{"posix", luaopen_posix},
-	{"rex", luaopen_rex},
+	{"rex_posix", luaopen_rex_posix},
+	{"rex_pcre", luaopen_rex_pcre},
+	{"uuid", luaopen_uuid},
+	{"wrs", luaopen_wrs},
+#ifdef	USE_LUA_CRYPTO		/* XXX external lua modules instead. */
+	{"crypto", luaopen_crypto},
+	{"lxp", luaopen_lxp},
+#endif
+#ifdef	USE_LUA_SOCKET		/* XXX external lua modules instead. */
+	{"socket", luaopen_socket_core},
+#endif
 	{"local", luaopen_local},
 #endif
 	{"rpm", luaopen_rpm},
 	{NULL, NULL},
     };
+    /*@=readonlytrans =nullassign @*/
     /*@observer@*/ /*@unchecked@*/
     const luaL_reg *lib = lualibs;
-    /*@=readonlytrans@*/
+    char *path_buf;
+    char *path_next;
+    char *path;
 
     lua->L = L;
     for (; lib->name; lib++) {
@@ -95,9 +185,8 @@ rpmlua rpmluaNew()
 	lua_call(L, 1, 0);
 /*@=noeffectuncon@*/
     }
-#define	_LUADOTDIR	"%{?_rpmhome}%{!?_rpmhome:" USRLIBRPM "}"
-    {	const char * _lua_path = rpmGetPath(_LUADOTDIR, "/lua/?.lua", NULL);
-	if (_lua_path != NULL) {
+    {	const char * _lua_path = rpmGetPath(rpmluaPath, NULL);
+ 	if (_lua_path != NULL) {
 	    lua_pushliteral(L, "LUA_PATH");
 	    lua_pushstring(L, _lua_path);
 	    _lua_path = _free(_lua_path);
@@ -108,34 +197,54 @@ rpmlua rpmluaNew()
     lua_pushcfunction(L, rpm_print);
     lua_rawset(L, LUA_GLOBALSINDEX);
     rpmluaSetData(lua, "lua", lua);
-    {	const char * _lua_init = rpmGetPath(_LUADOTDIR, "/init.lua", NULL);
-	if (_lua_init != NULL) {
-	    struct stat st;
-	    if (Stat(_lua_init, &st) != -1)
-		(void)rpmluaRunScriptFile(lua, _lua_init);
-	    _lua_init = _free(_lua_init);
-	}
-    }
-#undef	_LUADOTDIR
-    return lua;
-}
-/*@=mods@*/
 
-void *rpmluaFree(rpmlua lua)
-	/*@globals globalLuaState @*/
-	/*@modifies globalLuaState @*/
-{
-    if (lua == NULL) {
-	lua = globalLuaState;
-	globalLuaState = NULL;
+    /* load all standard RPM Lua script files */
+    path_buf = xstrdup(rpmluaFiles);
+    for (path = path_buf; path != NULL && *path != '\0'; path = path_next) {
+        const char **av;
+        struct stat st;
+        int ac, i;
+
+        /* locate start of next path element */
+        path_next = strchr(path, ':');
+        if (path_next != NULL && *path_next == ':')
+            *path_next++ = '\0';
+        else
+            path_next = path + strlen(path);
+
+        /* glob-expand the path element */
+        ac = 0;
+        av = NULL;
+        if ((i = rpmGlob(path, &ac, &av)) != 0)
+            continue;
+
+        /* work-off each resulting file from the path element */
+        for (i = 0; i < ac; i++) {
+            const char *fn = av[i];
+            if (fn[0] == '@' /* attention */) {
+                fn++;
+#if defined(RPM_VENDOR_OPENPKG) /* stick-with-rpm-file-sanity-checking */ || \
+    !defined(POPT_ERROR_BADCONFIG)	/* XXX POPT 1.15 retrofit */
+		if (!rpmSecuritySaneFile(fn))
+#else
+		if (!poptSaneFile(fn))
+#endif
+		{
+                    rpmlog(RPMLOG_WARNING, "existing RPM Lua script file \"%s\" considered INSECURE -- not loaded\n", fn);
+                    /*@innercontinue@*/ continue;
+                }
+            }
+            if (Stat(fn, &st) != -1)
+                (void)rpmluaRunScriptFile(lua, fn);
+            av[i] = _free(av[i]);
+        }
+        av = _free(av);
     }
-    if (lua) {
-	if (lua->L) lua_close(lua->L);
-	lua->printbuf = _free(lua->printbuf);
-	lua = _free(lua);
-    }
-    return NULL;
+    path_buf = _free(path_buf);
+
+    return ((rpmlua)rpmioLinkPoolItem((rpmioItem)lua, __FUNCTION__, __FILE__, __LINE__));
 }
+/*@=globs =mods@*/
 
 void rpmluaSetData(rpmlua _lua, const char *key, const void *data)
 {
@@ -151,6 +260,7 @@ void rpmluaSetData(rpmlua _lua, const char *key, const void *data)
     lua_rawset(L, LUA_REGISTRYINDEX);
 }
 
+/*@null@*/
 static void *getdata(lua_State *L, const char *key)
 	/*@modifies L @*/
 {
@@ -283,7 +393,7 @@ static int findkey(lua_State *L, int oper, const char *key, va_list va)
     char buf[BUFSIZ];
     const char *s, *e;
     int ret = 0;
-    (void) vsnprintf(buf, BUFSIZ, key, va);
+    (void) vsnprintf(buf, sizeof(buf), key, va);
     s = e = buf;
     lua_pushvalue(L, LUA_GLOBALSINDEX);
     for (;;) {
@@ -374,18 +484,32 @@ void rpmluaPop(rpmlua _lua)
     lua_pop(lua->L, 1);
 }
 
-rpmluav rpmluavNew(void)
-{
-    rpmluav var = (rpmluav) xcalloc(1, sizeof(*var));
-    return var;
-}
-
 void *rpmluavFree(rpmluav var)
 {
-    var = _free(var);
+    (void)rpmioFreePoolItem((rpmioItem)var, __FUNCTION__, __FILE__, __LINE__);
     return NULL;
 }
 
+static rpmluav rpmluavGetPool(/*@null@*/ rpmioPool pool)
+        /*@globals _rpmluavPool, fileSystem @*/
+        /*@modifies pool, _rpmluavPool, fileSystem @*/
+{
+    rpmluav luav;
+
+    if (_rpmluavPool == NULL) {
+        _rpmluavPool = rpmioNewPool("luav", sizeof(*luav), -1, _rpmlua_debug,
+                        NULL, NULL, NULL);
+        pool = _rpmluavPool;
+    }
+    return (rpmluav) rpmioGetPool(pool, sizeof(*luav));
+}
+
+rpmluav rpmluavNew(void)
+{
+    rpmluav var = rpmluavGetPool(_rpmluavPool);
+    return ((rpmluav)rpmioLinkPoolItem((rpmioItem)var, __FUNCTION__, __FILE__, __LINE__));
+}
+
 void rpmluavSetListMode(rpmluav var, int flag)
 {
     var->listmode = flag;
@@ -633,6 +757,7 @@ void rpmluaInteractive(rpmlua _lua)
 /* Lua API */
 
 static int rpm_macros(lua_State *L)
+	/*@modifies L @*/
 {
     const char ** av = NULL;
     int ac = 0;
@@ -642,7 +767,9 @@ static int rpm_macros(lua_State *L)
     lua_newtable(L);
 /*@=modunconnomods@*/
 
+/*@-globs@*/
     ac = rpmGetMacroEntries(NULL, NULL, -1, &av);
+/*@=globs@*/
 
     if (av != NULL)
     for (i = 0; i < ac; i++) {
@@ -702,7 +829,7 @@ static int rpm_define(lua_State *L)
 }
 
 static int rpm_undefine(lua_State *L)
-	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@globals rpmGlobalMacroContext, internalState @*/
 	/*@modifies L, rpmGlobalMacroContext, internalState @*/
 {
     const char *str = luaL_checkstring(L, 1);
@@ -726,7 +853,7 @@ typedef struct rpmluaHookData_s {
 } * rpmluaHookData;
 
 static int rpmluaHookWrapper(rpmhookArgs args, void *data)
-    /*@*/
+	/*@*/
 {
     rpmluaHookData hookdata = (rpmluaHookData)data;
     lua_State *L = hookdata->L;
@@ -885,7 +1012,7 @@ static int rpm_print (lua_State *L)
 	    return luaL_error(L, "`tostring' must return a string to `print'");
 	if (lua->storeprint) {
 	    size_t sl = lua_strlen(L, -1);
-	    if (lua->printbufused+sl+1 > lua->printbufsize) {
+	    if ((size_t)(lua->printbufused+sl+1) > lua->printbufsize) {
 		lua->printbufsize += sl+512;
 		lua->printbuf = xrealloc(lua->printbuf, lua->printbufsize);
 	    }
@@ -903,7 +1030,7 @@ static int rpm_print (lua_State *L)
     if (!lua->storeprint) {
 	(void) fputs("\n", stdout);
     } else {
-	if (lua->printbufused+1 > lua->printbufsize) {
+	if ((size_t)(lua->printbufused+1) > lua->printbufsize) {
 	    lua->printbufsize += 512;
 	    lua->printbuf = xrealloc(lua->printbuf, lua->printbufsize);
 	}
@@ -913,8 +1040,8 @@ static int rpm_print (lua_State *L)
 }
 
 static int rpm_source(lua_State *L)
-	/*@globals internalState @*/
-	/*@modifies L, internalState @*/
+	/*@globals fileSystem, internalState @*/
+	/*@modifies L, fileSystem, internalState @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void)luaL_argerror(L, 1, "filename expected");
@@ -927,14 +1054,16 @@ static int rpm_source(lua_State *L)
 }
 
 static int rpm_load(lua_State *L)
-	/*@globals internalState @*/
-	/*@modifies L, internalState @*/
+	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@modifies L, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void)luaL_argerror(L, 1, "filename expected");
     } else {
 	const char *filename = lua_tostring(L, 1);
+/*@-globs@*/
 	(void)rpmLoadMacroFile(NULL, filename);
+/*@=globs@*/
     }
     return 0;
 }
@@ -947,7 +1076,98 @@ static int rpm_verbose(lua_State *L)
     return 1;
 }
 
-/*@-readonlytrans@*/
+static int rpm_debug(lua_State *L)
+	/*@globals internalState @*/
+	/*@modifies L, internalState @*/
+{
+    lua_pushboolean(L, rpmIsDebug());
+    return 1;
+}
+
+static int rpm_slurp(lua_State *L)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies L, fileSystem, internalState @*/
+{
+    rpmiob iob = NULL;
+    const char *fn;
+    int rc;
+
+    if (lua_isstring(L, 1))
+        fn = lua_tostring(L, 1);
+    else {
+        (void)luaL_argerror(L, 1, "filename");
+        return 0;
+    }
+/*@-globs@*/
+    rc = rpmiobSlurp(fn, &iob);
+/*@=globs@*/
+    if (rc || iob == NULL) {
+        (void)luaL_error(L, "failed to slurp data");
+        return 0;
+    }
+    lua_pushlstring(L, (const char *)rpmiobStr(iob), rpmiobLen(iob));
+    iob = rpmiobFree(iob);
+    return 1;
+}
+
+static int rpm_sleep(lua_State *L)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies L, fileSystem, internalState @*/
+{
+    unsigned sec;
+
+    if (lua_isnumber(L, 1))
+        sec = (unsigned) lua_tonumber(L, 1);
+    else {
+        (void)luaL_argerror(L, 1, "seconds");
+        return 0;
+    }
+    (void) sleep(sec);
+    return 0;
+}
+
+static int rpm_realpath(lua_State *L)
+    /*@globals fileSystem, internalState @*/
+    /*@modifies L, fileSystem, internalState @*/
+{
+    const char *pn;
+    char rp_buf[PATH_MAX];
+    char *rp = "";
+
+    if (lua_isstring(L, 1))
+        pn = lua_tostring(L, 1);
+    else {
+        (void)luaL_argerror(L, 1, "pathname");
+        return 0;
+    }
+    if ((rp = Realpath(pn, rp_buf)) == NULL) {
+        (void)luaL_error(L, "failed to resolve path via realpath(3): %s", strerror(errno));
+        return 0;
+    }
+    lua_pushstring(L, (const char *)rp);
+    return 1;
+}
+
+static int rpm_hostname(lua_State *L)
+	/*@globals h_errno, internalState @*/
+	/*@modifies L, h_errno, internalState @*/
+{
+    char hostname[1024];
+    struct hostent *hbn;
+    char *h;
+
+/*@-multithreaded@*/
+    (void)gethostname(hostname, sizeof(hostname));
+    if ((hbn = gethostbyname(hostname)) != NULL)
+        h = hbn->h_name;
+    else
+        h = hostname;
+/*@=multithreaded@*/
+    lua_pushstring(L, (const char *)h);
+    return 1;
+}
+
+/*@-readonlytrans -nullassign @*/
 /*@observer@*/ /*@unchecked@*/
 static const luaL_reg rpmlib[] = {
     {"macros", rpm_macros},
@@ -961,9 +1181,14 @@ static const luaL_reg rpmlib[] = {
     {"source", rpm_source},
     {"load", rpm_load},
     {"verbose", rpm_verbose},
+    {"debug", rpm_debug},
+    {"slurp", rpm_slurp},
+    {"sleep", rpm_sleep},
+    {"realpath", rpm_realpath},
+    {"hostname", rpm_hostname},
     {NULL, NULL}
 };
-/*@=readonlytrans@*/
+/*@=readonlytrans =nullassign @*/
 
 static int luaopen_rpm(lua_State *L)
 	/*@modifies L @*/
@@ -974,4 +1199,4 @@ static int luaopen_rpm(lua_State *L)
 }
 #endif	/* WITH_LUA */
 
-/*@=realcompare =sizeoftype =protoparammatch @*/
+/*@=moduncon =mustmod =realcompare =sizeoftype @*/
