@@ -1,8 +1,6 @@
 #ifndef _H_MACRO_
 #define	_H_MACRO_
 
-#include <mire.h>
-
 /** \ingroup rpmio
  * \file rpmio/rpmmacro.h
  */
@@ -75,16 +73,16 @@ void rpmDumpMacroTable(/*@null@*/ MacroContext mc, /*@null@*/ FILE * fp)
 /**
  * Return macro entries as string array.
  * @param mc		macro context (NULL uses global context)
- * @param mire		pattern to match (NULL disables)
+ * @param _mire		pattern to match (NULL disables)
  * @param used		macro usage (<0 all, =0 unused, >=1 used count)
  * @retval *avp		macro definitions
  * @return		no. of entries
  */
 int
-rpmGetMacroEntries(/*@null@*/ MacroContext mc, /*@null@*/ miRE mire,
+rpmGetMacroEntries(/*@null@*/ MacroContext mc, /*@null@*/ void * _mire,
 		int used, /*@null@*/ const char *** avp)
 	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies mire, *avp @*/;
+	/*@modifies _mire, *avp @*/;
 
 /**
  * Check whether configuration file is moderately secure to load.
@@ -92,7 +90,7 @@ rpmGetMacroEntries(/*@null@*/ MacroContext mc, /*@null@*/ miRE mire,
  * @return		1 on success, 0 on failure
  */
 int rpmSecuritySaneFile(const char *filename)
-	/*@globals fileSystem, internalState @*/;
+	/*@globals internalState @*/;
 
 /**
  * Return URL path(s) from a (URL prefixed) pattern glob.
@@ -208,7 +206,8 @@ typedef enum rpmCompressedMagic_e {
     COMPRESSED_BZIP2		= 2,	/*!< bzip2 can handle */
     COMPRESSED_ZIP		= 3,	/*!< unzip can handle */
     COMPRESSED_LZOP		= 4,	/*!< lzop can handle */
-    COMPRESSED_LZMA		= 5	/*!< lzma can handle */
+    COMPRESSED_LZMA		= 5,	/*!< lzma can handle */
+    COMPRESSED_XZ		= 6	/*!< xz can handle */
 } rpmCompressedMagic;
 
 /**
@@ -227,6 +226,24 @@ int isCompressed(const char * file, /*@out@*/ rpmCompressedMagic * compressed)
  * @return		macro expansion (malloc'ed)
  */
 char * rpmExpand(/*@null@*/ const char * arg, ...)
+#if defined(__GNUC__) && __GNUC__ >= 4
+	/* issue a warning if the list is not  NULL-terminated */
+	__attribute__((sentinel))
+#endif
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies rpmGlobalMacroContext, internalState @*/;
+
+/**
+ * Return (malloc'ed) concatenated macro expansion(s) in a context.
+ * @param mc		macro context
+ * @param arg		macro(s) to expand (NULL terminates list)
+ * @return		macro expansion (malloc'ed)
+ */
+char * rpmMCExpand(/*@null@*/ MacroContext mc, /*@null@*/ const char * arg, ...)
+#if defined(__GNUC__) && __GNUC__ >= 4
+	/* issue a warning if the list is not  NULL-terminated */
+	__attribute__((sentinel))
+#endif
 	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
 	/*@modifies rpmGlobalMacroContext, internalState @*/;
 
@@ -246,6 +263,10 @@ char * rpmCleanPath(/*@returned@*/ /*@null@*/ char * path)
  */
 /*@-redecl@*/ /* LCL: shrug */
 const char * rpmGetPath(/*@null@*/ const char * path, ...)
+#if defined(__GNUC__) && __GNUC__ >= 4
+	/* issue a warning if the list is not  NULL-terminated */
+	 __attribute__((sentinel))
+#endif
 	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
 	/*@modifies rpmGlobalMacroContext, internalState @*/;
 /*@=redecl@*/
