@@ -5,10 +5,14 @@
  * \file rpmio/rpmnss.h
  */
 
+#include <rpmiotypes.h>
 #include <rpmpgp.h>
 #include <rpmsw.h>
 
 #if defined(_RPMNSS_INTERNAL)
+#if defined(__LCLINT__)
+#define	__i386__
+#endif
 #include <nss.h>
 #include <sechash.h>
 #include <keyhi.h>
