@@ -4,7 +4,12 @@
  */
 
 #include "system.h"
-#include "rpmio_internal.h"
+
+#define	_RPMIOB_INTERNAL
+#include <rpmiotypes.h>
+
+#include <rpmio.h>
+
 #define	_RPMPGP_INTERNAL
 #include <rpmbc.h>	/* XXX still needs base64 goop */
 #if defined(WITH_NSS)
@@ -15,6 +20,7 @@
 /*@access pgpDig @*/
 /*@access pgpDigParams @*/
 /*@access pgpPkt @*/
+/*@access rpmiob @*/
 
 /*@unchecked@*/
 int _pgp_debug = 0;
@@ -24,10 +30,26 @@ int _pgp_print = 0;
 
 /*@unchecked@*/
 pgpImplVecs_t * pgpImplVecs =
-#if defined(WITH_NSS)
-	&rpmnssImplVecs;
-#else
+    /* explicit selection (order DOES NOT matter here) */
+#if defined(USE_CRYPTO_BEECRYPT) && defined(WITH_BEECRYPT)
 	&rpmbcImplVecs;
+#elif defined(USE_CRYPTO_GCRYPT) && defined(WITH_GCRYPT)
+	&rpmgcImplVecs;
+#elif defined(USE_CRYPTO_NSS) && defined(WITH_NSS)
+	&rpmnssImplVecs;
+#elif defined(USE_CRYPTO_OPENSSL) && defined(WITH_SSL)
+	&rpmsslImplVecs;
+    /* implict selection (order DOES matter) */
+#elif defined(WITH_BEECRYPT)
+	&rpmbcImplVecs;
+#elif defined(WITH_GCRYPT)
+	&rpmgcImplVecs;
+#elif defined(WITH_NSS)
+	&rpmnssImplVecs;
+#elif defined(WITH_SSL)
+	&rpmsslImplVecs;
+#else
+#error INTERNAL ERROR: no suitable Cryptography library available
 #endif
 
 /*@unchecked@*/ /*@refcounted@*/ /*@relnull@*/
@@ -39,7 +61,7 @@ static pgpDigParams _digp = NULL;
 struct pgpPkt_s {
     pgpTag tag;
     unsigned int pktlen;
-    const uint8_t * h;
+    const rpmuint8_t * h;
     unsigned int hlen;
 };
 
@@ -57,6 +79,7 @@ struct pgpValTbl_s pgpSigTypeTbl[] = {
     { PGPSIGTYPE_SUBKEY_REVOKE,	"Subkey revocation signature" },
     { PGPSIGTYPE_CERT_REVOKE,	"Certification revocation signature" },
     { PGPSIGTYPE_TIMESTAMP,	"Timestamp signature" },
+    { PGPSIGTYPE_CONFIRM,	"Third-Party Confirmation signature" },
     { -1,			"Unknown signature type" },
 };
 
@@ -104,6 +127,7 @@ struct pgpValTbl_s pgpHashTbl[] = {
     { PGPHASHALGO_MD2,		"MD2" },
     { PGPHASHALGO_TIGER192,	"TIGER192" },
     { PGPHASHALGO_HAVAL_5_160,	"HAVAL-5-160" },
+    { PGPHASHALGO_SHA224,	"SHA224" },
     { PGPHASHALGO_SHA256,	"SHA256" },
     { PGPHASHALGO_SHA384,	"SHA384" },
     { PGPHASHALGO_SHA512,	"SHA512" },
@@ -141,6 +165,7 @@ struct pgpValTbl_s pgpSubTypeTbl[] = {
     { PGPSUBTYPE_SIGNER_USERID,	"signer's user id" },
     { PGPSUBTYPE_REVOKE_REASON,	"reason for revocation" },
     { PGPSUBTYPE_FEATURES,	"features" },
+    { PGPSUBTYPE_SIG_TARGET,	"signature target" },
     { PGPSUBTYPE_EMBEDDED_SIG,	"embedded signature" },
 
     { PGPSUBTYPE_INTERNAL_100,	"internal subpkt type 100" },
@@ -231,7 +256,7 @@ static void pgpPrtStr(const char *pre, const char *s)
     fprintf(stderr, " %s", s);
 }
 
-static void pgpPrtHex(const char *pre, const uint8_t * p, size_t plen)
+static void pgpPrtHex(const char *pre, const rpmuint8_t * p, size_t plen)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
@@ -241,7 +266,7 @@ static void pgpPrtHex(const char *pre, const uint8_t * p, size_t plen)
     fprintf(stderr, " %s", pgpHexStr(p, plen));
 }
 
-void pgpPrtVal(const char * pre, pgpValTbl vs, uint8_t val)
+void pgpPrtVal(const char * pre, pgpValTbl vs, rpmuint8_t val)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
@@ -251,11 +276,11 @@ void pgpPrtVal(const char * pre, pgpValTbl vs, uint8_t val)
     fprintf(stderr, "%s(%u)", pgpValStr(vs, val), (unsigned)val);
 }
 
-int pgpPrtSubType(const uint8_t * h, size_t hlen, pgpSigType sigtype)
+int pgpPrtSubType(const rpmuint8_t * h, size_t hlen, pgpSigType sigtype)
 {
-    const uint8_t * p = h;
+    const rpmuint8_t * p = h;
     unsigned plen;
-    int i;
+    unsigned i;
 
     while (hlen > 0) {
 	i = pgpLen(p, &plen);
@@ -263,7 +288,7 @@ int pgpPrtSubType(const uint8_t * h, size_t hlen, pgpSigType sigtype)
 	hlen -= i;
 
 	pgpPrtVal("    ", pgpSubTypeTbl, (p[0]&(~PGPSUBTYPE_CRITICAL)));
-	if (p[0] & PGPSUBTYPE_CRITICAL)
+	if ((p[0] & PGPSUBTYPE_CRITICAL) != (rpmuint8_t)0)
 	    if (_pgp_print)
 		fprintf(stderr, " *CRITICAL*");
 	switch (*p) {
@@ -327,6 +352,7 @@ int pgpPrtSubType(const uint8_t * h, size_t hlen, pgpSigType sigtype)
 	case PGPSUBTYPE_SIGNER_USERID:
 	case PGPSUBTYPE_REVOKE_REASON:
 	case PGPSUBTYPE_FEATURES:
+	case PGPSUBTYPE_SIG_TARGET:
 	case PGPSUBTYPE_EMBEDDED_SIG:
 	case PGPSUBTYPE_INTERNAL_100:
 	case PGPSUBTYPE_INTERNAL_101:
@@ -350,7 +376,7 @@ int pgpPrtSubType(const uint8_t * h, size_t hlen, pgpSigType sigtype)
     return 0;
 }
 
-/*@-varuse =readonlytrans @*/
+/*@-varuse =readonlytrans -nullassign @*/
 /*@observer@*/ /*@unchecked@*/
 static const char * pgpSigRSA[] = {
     " m**d =",
@@ -363,14 +389,14 @@ static const char * pgpSigDSA[] = {
     "    s =",
     NULL,
 };
-/*@=varuse =readonlytrans @*/
+/*@=varuse =readonlytrans =nullassign @*/
 
-static int pgpPrtSigParams(const pgpPkt pp, uint8_t pubkey_algo,
-		uint8_t sigtype, const uint8_t * p)
+static int pgpPrtSigParams(const pgpPkt pp, pgpPubkeyAlgo pubkey_algo,
+		pgpSigType sigtype, const rpmuint8_t * p)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
-    const uint8_t * pend = pp->h + pp->hlen;
+    const rpmuint8_t * pend = pp->h + pp->hlen;
     int xx;
     int i;
 
@@ -427,8 +453,8 @@ int pgpPrtSig(const pgpPkt pp)
 	/*@globals _digp @*/
 	/*@modifies *_digp @*/
 {
-    uint8_t version = pp->h[0];
-    uint8_t * p;
+    rpmuint8_t version = pp->h[0];
+    rpmuint8_t * p;
     unsigned plen;
     int rc;
 
@@ -437,10 +463,10 @@ int pgpPrtSig(const pgpPkt pp)
     {   pgpPktSigV3 v = (pgpPktSigV3)pp->h;
 	time_t t;
 
-	if (v->hashlen != 5)
+	if (v->hashlen != (rpmuint8_t)5)
 	    return 1;
 
-	pgpPrtVal("V3 ", pgpTagTbl, pp->tag);
+	pgpPrtVal("V3 ", pgpTagTbl, (rpmuint8_t)pp->tag);
 	pgpPrtVal(" ", pgpPubkeyTbl, v->pubkey_algo);
 	pgpPrtVal(" ", pgpHashTbl, v->hash_algo);
 	pgpPrtVal(" ", pgpSigTypeTbl, v->sigtype);
@@ -454,11 +480,11 @@ int pgpPrtSig(const pgpPkt pp)
 	pgpPrtHex(" signhash16", v->signhash16, sizeof(v->signhash16));
 	pgpPrtNL();
 
-	if (_digp && _digp->pubkey_algo == 0) {
+	if (_digp && _digp->pubkey_algo == (rpmuint8_t)0) {
 	    _digp->version = v->version;
-	    _digp->hashlen = v->hashlen;
+	    _digp->hashlen = (size_t) v->hashlen;
 	    _digp->sigtype = v->sigtype;
-	    _digp->hash = memcpy(xmalloc(v->hashlen), &v->sigtype, v->hashlen);
+	    _digp->hash = memcpy(xmalloc(_digp->hashlen), &v->sigtype, _digp->hashlen);
 	    memcpy(_digp->time, v->time, sizeof(_digp->time));
 	    memcpy(_digp->signid, v->signid, sizeof(_digp->signid));
 	    _digp->pubkey_algo = v->pubkey_algo;
@@ -466,13 +492,14 @@ int pgpPrtSig(const pgpPkt pp)
 	    memcpy(_digp->signhash16, v->signhash16, sizeof(_digp->signhash16));
 	}
 
-	p = ((uint8_t *)v) + sizeof(*v);
-	rc = pgpPrtSigParams(pp, v->pubkey_algo, v->sigtype, p);
+	p = ((rpmuint8_t *)v) + sizeof(*v);
+	rc = pgpPrtSigParams(pp, (pgpPubkeyAlgo)v->pubkey_algo,
+			(pgpSigType)v->sigtype, p);
     }	break;
     case 4:
     {   pgpPktSigV4 v = (pgpPktSigV4)pp->h;
 
-	pgpPrtVal("V4 ", pgpTagTbl, pp->tag);
+	pgpPrtVal("V4 ", pgpTagTbl, (rpmuint8_t)pp->tag);
 	pgpPrtVal(" ", pgpPubkeyTbl, v->pubkey_algo);
 	pgpPrtVal(" ", pgpHashTbl, v->hash_algo);
 	pgpPrtVal(" ", pgpSigTypeTbl, v->sigtype);
@@ -487,11 +514,11 @@ int pgpPrtSig(const pgpPkt pp)
 
 if (_pgp_debug && _pgp_print)
 fprintf(stderr, "   hash[%u] -- %s\n", plen, pgpHexStr(p, plen));
-	if (_digp && _digp->pubkey_algo == 0) {
+	if (_digp && _digp->pubkey_algo == (rpmuint8_t)0) {
 	    _digp->hashlen = sizeof(*v) + plen;
 	    _digp->hash = memcpy(xmalloc(_digp->hashlen), v, _digp->hashlen);
 	}
-	(void) pgpPrtSubType(p, plen, v->sigtype);
+	(void) pgpPrtSubType(p, plen, (pgpSigType)v->sigtype);
 	p += plen;
 
 	plen = pgpGrab(p,2);
@@ -502,14 +529,14 @@ fprintf(stderr, "   hash[%u] -- %s\n", plen, pgpHexStr(p, plen));
 
 if (_pgp_debug && _pgp_print)
 fprintf(stderr, " unhash[%u] -- %s\n", plen, pgpHexStr(p, plen));
-	(void) pgpPrtSubType(p, plen, v->sigtype);
+	(void) pgpPrtSubType(p, plen, (pgpSigType)v->sigtype);
 	p += plen;
 
 	plen = pgpGrab(p,2);
 	pgpPrtHex(" signhash16", p, 2);
 	pgpPrtNL();
 
-	if (_digp && _digp->pubkey_algo == 0) {
+	if (_digp && _digp->pubkey_algo == (rpmuint8_t)0) {
 	    _digp->version = v->version;
 	    _digp->sigtype = v->sigtype;
 	    _digp->pubkey_algo = v->pubkey_algo;
@@ -521,7 +548,8 @@ fprintf(stderr, " unhash[%u] -- %s\n", plen, pgpHexStr(p, plen));
 	if (p > (pp->h + pp->hlen))
 	    return 1;
 
-	rc = pgpPrtSigParams(pp, v->pubkey_algo, v->sigtype, p);
+	rc = pgpPrtSigParams(pp, (pgpPubkeyAlgo)v->pubkey_algo,
+			(pgpSigType)v->sigtype, p);
     }	break;
     default:
 	rc = 1;
@@ -530,7 +558,7 @@ fprintf(stderr, " unhash[%u] -- %s\n", plen, pgpHexStr(p, plen));
     return rc;
 }
 
-/*@-varuse =readonlytrans @*/
+/*@-varuse =readonlytrans -nullassign @*/
 /*@observer@*/ /*@unchecked@*/
 static const char * pgpPublicRSA[] = {
     "    n =",
@@ -581,10 +609,10 @@ static const char * pgpSecretELGAMAL[] = {
     NULL,
 };
 #endif
-/*@=varuse =readonlytrans @*/
+/*@=varuse =readonlytrans =nullassign @*/
 
-static const uint8_t * pgpPrtPubkeyParams(const pgpPkt pp, uint8_t pubkey_algo,
-		/*@returned@*/ const uint8_t * p)
+static const rpmuint8_t * pgpPrtPubkeyParams(const pgpPkt pp,
+		pgpPubkeyAlgo pubkey_algo, /*@returned@*/ const rpmuint8_t * p)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
@@ -641,8 +669,9 @@ static const uint8_t * pgpPrtPubkeyParams(const pgpPkt pp, uint8_t pubkey_algo,
     return p;
 }
 
-static const uint8_t * pgpPrtSeckeyParams(const pgpPkt pp, /*@unused@*/ uint8_t pubkey_algo,
-		/*@returned@*/ const uint8_t *p)
+static const rpmuint8_t * pgpPrtSeckeyParams(const pgpPkt pp,
+		/*@unused@*/ rpmuint8_t pubkey_algo,
+		/*@returned@*/ const rpmuint8_t *p)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
@@ -667,9 +696,7 @@ static const uint8_t * pgpPrtSeckeyParams(const pgpPkt pp, /*@unused@*/ uint8_t
 	    /*@innerbreak@*/ break;
 	case 0x03:
 	    pgpPrtVal(" iterated/salted ", pgpHashTbl, p[2]);
-	    /*@-shiftnegative -shiftimplementation @*/ /* FIX: unsigned cast */
-	    i = (int)(16 + (p[11] & 0xf)) << ((p[11] >> 4) + 6);
-	    /*@=shiftnegative =shiftimplementation @*/
+	    i = (16 + ((unsigned)p[11] & 0xf)) << (((unsigned)p[11] >> 4U) + 6);
 	    pgpPrtHex("", p+3, 8);
 	    pgpPrtInt(" iter", i);
 	    p += 11;
@@ -719,8 +746,8 @@ int pgpPrtKey(const pgpPkt pp)
 	/*@globals _digp @*/
 	/*@modifies *_digp @*/
 {
-    uint8_t version = pp->h[0];
-    const uint8_t * p;
+    rpmuint8_t version = pp->h[0];
+    const rpmuint8_t * p;
     unsigned plen;
     time_t t;
     int rc;
@@ -728,7 +755,7 @@ int pgpPrtKey(const pgpPkt pp)
     switch (version) {
     case 3:
     {   pgpPktKeyV3 v = (pgpPktKeyV3)pp->h;
-	pgpPrtVal("V3 ", pgpTagTbl, pp->tag);
+	pgpPrtVal("V3 ", pgpTagTbl, (rpmuint8_t)pp->tag);
 	pgpPrtVal(" ", pgpPubkeyTbl, v->pubkey_algo);
 	t = pgpGrab(v->time, sizeof(v->time));
 	if (_pgp_print)
@@ -738,33 +765,33 @@ int pgpPrtKey(const pgpPkt pp)
 	    fprintf(stderr, " valid %u days", plen);
 	pgpPrtNL();
 
-	if (_digp && _digp->tag == pp->tag) {
+	if (_digp && _digp->tag == (rpmuint8_t)pp->tag) {
 	    _digp->version = v->version;
 	    memcpy(_digp->time, v->time, sizeof(_digp->time));
 	    _digp->pubkey_algo = v->pubkey_algo;
 	}
 
-	p = ((uint8_t *)v) + sizeof(*v);
-	p = pgpPrtPubkeyParams(pp, v->pubkey_algo, p);
+	p = ((rpmuint8_t *)v) + sizeof(*v);
+	p = pgpPrtPubkeyParams(pp, (pgpPubkeyAlgo)v->pubkey_algo, p);
 	rc = 0;
     }	break;
     case 4:
     {   pgpPktKeyV4 v = (pgpPktKeyV4)pp->h;
-	pgpPrtVal("V4 ", pgpTagTbl, pp->tag);
+	pgpPrtVal("V4 ", pgpTagTbl, (rpmuint8_t)pp->tag);
 	pgpPrtVal(" ", pgpPubkeyTbl, v->pubkey_algo);
 	t = pgpGrab(v->time, sizeof(v->time));
 	if (_pgp_print)
 	    fprintf(stderr, " %-24.24s(0x%08x)", ctime(&t), (unsigned)t);
 	pgpPrtNL();
 
-	if (_digp && _digp->tag == pp->tag) {
+	if (_digp && _digp->tag == (rpmuint8_t)pp->tag) {
 	    _digp->version = v->version;
 	    memcpy(_digp->time, v->time, sizeof(_digp->time));
 	    _digp->pubkey_algo = v->pubkey_algo;
 	}
 
-	p = ((uint8_t *)v) + sizeof(*v);
-	p = pgpPrtPubkeyParams(pp, v->pubkey_algo, p);
+	p = ((rpmuint8_t *)v) + sizeof(*v);
+	p = pgpPrtPubkeyParams(pp, (pgpPubkeyAlgo)v->pubkey_algo, p);
 	if (!(pp->tag == PGPTAG_PUBLIC_KEY || pp->tag == PGPTAG_PUBLIC_SUBKEY))
 	    p = pgpPrtSeckeyParams(pp, v->pubkey_algo, p);
 	rc = 0;
@@ -780,7 +807,7 @@ int pgpPrtUserID(const pgpPkt pp)
 	/*@globals _digp @*/
 	/*@modifies *_digp @*/
 {
-    pgpPrtVal("", pgpTagTbl, pp->tag);
+    pgpPrtVal("", pgpTagTbl, (rpmuint8_t)pp->tag);
     if (_pgp_print)
 	fprintf(stderr, " \"%.*s\"", (int)pp->hlen, (const char *)pp->h);
     pgpPrtNL();
@@ -795,19 +822,19 @@ int pgpPrtUserID(const pgpPkt pp)
 
 int pgpPrtComment(const pgpPkt pp)
 {
-    const uint8_t * h = pp->h;
+    const rpmuint8_t * h = pp->h;
     int i = pp->hlen;
 
-    pgpPrtVal("", pgpTagTbl, pp->tag);
+    pgpPrtVal("", pgpTagTbl, (rpmuint8_t)pp->tag);
     if (_pgp_print)
 	fprintf(stderr, " ");
     while (i > 0) {
 	int j;
-	if (*h >= (uint8_t)' ' && *h <= (uint8_t)'z') {
+	if (*h >= (rpmuint8_t)' ' && *h <= (rpmuint8_t)'z') {
 	    j = 0;
-	    while (j < i && h[j] != (uint8_t)'\0')
+	    while (j < i && h[j] != (rpmuint8_t)'\0')
 		j++;
-	    while (j < i && h[j] == (uint8_t)'\0')
+	    while (j < i && h[j] == (rpmuint8_t)'\0')
 		j++;
 	    if (_pgp_print && j)
 		fprintf(stderr, "%.*s", (int)strlen((const char *)h), (const char *)h);
@@ -822,7 +849,7 @@ int pgpPrtComment(const pgpPkt pp)
     return 0;
 }
 
-int pgpPktLen(const uint8_t *pkt, size_t pleft, pgpPkt pp)
+int pgpPktLen(const rpmuint8_t *pkt, size_t pleft, pgpPkt pp)
 {
     unsigned int val = (unsigned int)*pkt;
     unsigned int plen;
@@ -852,11 +879,11 @@ int pgpPktLen(const uint8_t *pkt, size_t pleft, pgpPkt pp)
     return pp->pktlen;
 }
 
-int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen, uint8_t * keyid)
+int pgpPubkeyFingerprint(const rpmuint8_t * pkt, size_t pktlen, rpmuint8_t * keyid)
 {
     pgpPkt pp = alloca(sizeof(*pp));
     int rc = pgpPktLen(pkt, pktlen, pp);
-    const uint8_t * se;
+    const rpmuint8_t * se;
     int i;
 
     /* Pubkeys only please. */
@@ -868,7 +895,7 @@ int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen, uint8_t * keyid)
     default:	return -1;
     case 3:
       {	pgpPktKeyV3 v = (pgpPktKeyV3) (pp->h);
-	se = (uint8_t *)(v + 1);
+	se = (rpmuint8_t *)(v + 1);
 	switch (v->pubkey_algo) {
 	default:	return -1;
 	case PGPPUBKEYALGO_RSA:
@@ -879,10 +906,10 @@ int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen, uint8_t * keyid)
       } break;
     case 4:
       {	pgpPktKeyV4 v = (pgpPktKeyV4) (pp->h);
-	uint8_t * d = NULL;
+	rpmuint8_t * d = NULL;
 	size_t dlen = 0;
 
-	se = (uint8_t *)(v + 1);
+	se = (rpmuint8_t *)(v + 1);
 	switch (v->pubkey_algo) {
 	default:	return -1;
 	case PGPPUBKEYALGO_RSA:
@@ -907,9 +934,9 @@ int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen, uint8_t * keyid)
     return rc;
 }
 
-int pgpExtractPubkeyFingerprint(const char * b64pkt, uint8_t * keyid)
+int pgpExtractPubkeyFingerprint(const char * b64pkt, rpmuint8_t * keyid)
 {
-    const uint8_t * pkt;
+    const rpmuint8_t * pkt;
     size_t pktlen;
 
     if (b64decode(b64pkt, (void **)&pkt, &pktlen))
@@ -919,7 +946,7 @@ int pgpExtractPubkeyFingerprint(const char * b64pkt, uint8_t * keyid)
     return 8;	/* no. of bytes of pubkey signid */
 }
 
-int pgpPrtPkt(const uint8_t * pkt, size_t pleft)
+int pgpPrtPkt(const rpmuint8_t * pkt, size_t pleft)
 {
     pgpPkt pp = alloca(sizeof(*pp));
     int rc = pgpPktLen(pkt, pleft, pp);
@@ -972,43 +999,19 @@ int pgpPrtPkt(const uint8_t * pkt, size_t pleft)
     case PGPTAG_PRIVATE_62:
     case PGPTAG_CONTROL:
     default:
-	pgpPrtVal("", pgpTagTbl, pp->tag);
+	pgpPrtVal("", pgpTagTbl, (rpmuint8_t)pp->tag);
 	pgpPrtHex("", pp->h, pp->hlen);
 	pgpPrtNL();
 	rc = 0;
 	break;
     }
 
-    return (rc ? -1 : pp->pktlen);
+    return (rc ? -1 : (int)pp->pktlen);
 }
 
 /*@unchecked@*/
 pgpVSFlags pgpDigVSFlags;
 
-pgpDig XpgpDigUnlink(pgpDig dig, const char * msg, const char * fn, unsigned ln)
-{
-    if (dig == NULL) return NULL;
-/*@-modfilesys@*/
-if (_pgp_debug && msg != NULL)
-fprintf(stderr, "--> dig %p -- %d %s at %s:%u\n", dig, dig->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-    dig->nrefs--;
-    return NULL;
-}
-
-pgpDig XpgpDigLink(pgpDig dig, const char * msg, const char * fn, unsigned ln)
-{
-    if (dig == NULL) return NULL;
-    dig->nrefs++;
-
-/*@-modfilesys@*/
-if (_pgp_debug && msg != NULL)
-fprintf(stderr, "--> dig %p ++ %d %s at %s:%u\n", dig, dig->nrefs, msg, fn, ln);
-/*@=modfilesys@*/
-
-    /*@-refcounttrans@*/ return dig; /*@=refcounttrans@*/
-}
-
 void pgpDigClean(pgpDig dig)
 {
     if (dig != NULL) {
@@ -1042,54 +1045,68 @@ void pgpDigClean(pgpDig dig)
 /*@=nullstate@*/
 }
 
-pgpDig pgpDigFree(pgpDig dig)
+static void pgpDigFini(void * __dig)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies __dig, fileSystem, internalState @*/
 {
-    if (dig != NULL) {
+    pgpDig dig = __dig;
 
-/*@-onlytrans@*/
-	if (dig->nrefs > 1)
-	    return pgpDigUnlink(dig, "pgpDigFree");
+    /* Lose the header tag data. */
+    /* XXX this free should be done somewhere else. */
+    dig->sig = _free(dig->sig);
 
-	/* Lose the header tag data. */
-	/* XXX this free should be done somewhere else. */
-	dig->sig = _free(dig->sig);
-
-	/* Dump the signature/pubkey data. */
-	pgpDigClean(dig);
-
-	if (dig->hdrsha1ctx != NULL)
-	    (void) rpmDigestFinal(dig->hdrsha1ctx, NULL, NULL, 0);
-	dig->hdrsha1ctx = NULL;
-
-	if (dig->sha1ctx != NULL)
-	    (void) rpmDigestFinal(dig->sha1ctx, NULL, NULL, 0);
-	dig->sha1ctx = NULL;
-
-#ifdef	NOTYET
-	if (dig->hdrmd5ctx != NULL)
-	    (void) rpmDigestFinal(dig->hdrmd5ctx, NULL, NULL, 0);
-	dig->hdrmd5ctx = NULL;
+    /* XXX there's a recursion here ... release and reacquire the lock */
+#ifndef	BUGGY
+    yarnRelease(dig->_item.use);
+#endif
+    /* Dump the signature/pubkey data. */
+    pgpDigClean(dig);
+#ifndef	BUGGY
+    yarnPossess(dig->_item.use);
 #endif
 
-	if (dig->md5ctx != NULL)
-	    (void) rpmDigestFinal(dig->md5ctx, NULL, NULL, 0);
-	dig->md5ctx = NULL;
+    if (dig->hdrsha1ctx != NULL)
+	(void) rpmDigestFinal(dig->hdrsha1ctx, NULL, NULL, 0);
+    dig->hdrsha1ctx = NULL;
 
-	dig->impl = pgpImplFree(dig->impl);
+    if (dig->sha1ctx != NULL)
+	(void) rpmDigestFinal(dig->sha1ctx, NULL, NULL, 0);
+    dig->sha1ctx = NULL;
 
-	(void) pgpDigUnlink(dig, "pgpDigFree");
-/*@=onlytrans@*/
-	/*@-refcounttrans -usereleased@*/
-	memset(dig, 0, sizeof(*dig));         /* XXX trash and burn */
-	dig = _free(dig);
-	/*@=refcounttrans =usereleased@*/
+#ifdef	NOTYET
+    if (dig->hdrmd5ctx != NULL)
+	(void) rpmDigestFinal(dig->hdrmd5ctx, NULL, NULL, 0);
+    dig->hdrmd5ctx = NULL;
+#endif
+
+    if (dig->md5ctx != NULL)
+	(void) rpmDigestFinal(dig->md5ctx, NULL, NULL, 0);
+    dig->md5ctx = NULL;
+
+    dig->impl = pgpImplFree(dig->impl);
+
+}
+
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _digPool;
+
+static pgpDig digGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _digPool, fileSystem @*/
+	/*@modifies pool, _digPool, fileSystem @*/
+{
+    pgpDig dig;
+
+    if (_digPool == NULL) {
+	_digPool = rpmioNewPool("dig", sizeof(*dig), -1, _pgp_debug,
+			NULL, NULL, pgpDigFini);
+	pool = _digPool;
     }
-    return NULL;
+    return (pgpDig) rpmioGetPool(pool, sizeof(*dig));
 }
 
 pgpDig pgpDigNew(/*@unused@*/ pgpVSFlags vsflags)
 {
-    pgpDig dig = xcalloc(1, sizeof(*dig));
+    pgpDig dig = digGetPool(_digPool);
     dig->vsflags = pgpDigVSFlags;
     dig->impl = pgpImplInit();
     return pgpDigLink(dig, "pgpDigNew");
@@ -1105,12 +1122,12 @@ pgpDigParams pgpGetPubkey(pgpDig dig)
     return (dig ? &dig->pubkey : NULL);
 }
 
-uint32_t pgpGetSigtag(pgpDig dig)
+rpmuint32_t pgpGetSigtag(pgpDig dig)
 {
     return (dig ? dig->sigtag : 0);
 }
 
-uint32_t pgpGetSigtype(pgpDig dig)
+rpmuint32_t pgpGetSigtype(pgpDig dig)
 {
     return (dig ? dig->sigtype : 0);
 }
@@ -1120,13 +1137,13 @@ const void * pgpGetSig(pgpDig dig)
     return (dig ? dig->sig : NULL);
 }
 
-uint32_t pgpGetSiglen(pgpDig dig)
+rpmuint32_t pgpGetSiglen(pgpDig dig)
 {
     return (dig ? dig->siglen : 0);
 }
 
 int pgpSetSig(pgpDig dig,
-	uint32_t sigtag, uint32_t sigtype, const void * sig, uint32_t siglen)
+	rpmuint32_t sigtag, rpmuint32_t sigtype, const void * sig, rpmuint32_t siglen)
 {
     if (dig != NULL) {
 	dig->sigtag = sigtag;
@@ -1175,16 +1192,16 @@ int pgpFindPubkey(pgpDig dig)
     return rc;
 }
 
-static int pgpGrabPkts(const uint8_t * pkts, size_t pktlen,
-		/*@out@*/ uint8_t *** pppkts, /*@out@*/ int * pnpkts)
+static int pgpGrabPkts(const rpmuint8_t * pkts, size_t pktlen,
+		/*@out@*/ rpmuint8_t *** pppkts, /*@out@*/ int * pnpkts)
 	/*@modifies *pppkts, *pnpkts @*/
 {
     pgpPkt pp = alloca(sizeof(*pp));
-    const uint8_t * p;
+    const rpmuint8_t * p;
     size_t pleft;
     size_t len;
     int npkts = 0;
-    uint8_t ** ppkts;
+    rpmuint8_t ** ppkts;
 
     for (p = pkts, pleft = pktlen; p < (pkts + pktlen); p += len, pleft -= len) {
 	if (pgpPktLen(p, pleft, pp) < 0)
@@ -1203,7 +1220,7 @@ static int pgpGrabPkts(const uint8_t * pkts, size_t pktlen,
 	if (pgpPktLen(p, pleft, pp) < 0)
 	    return -1;
 	len = pp->pktlen;
-	ppkts[npkts++] = (uint8_t *) p;
+	ppkts[npkts++] = (rpmuint8_t *) p;
     }
 
     if (pppkts != NULL)
@@ -1218,7 +1235,7 @@ static int pgpGrabPkts(const uint8_t * pkts, size_t pktlen,
 }
 
 /*@-globstate -incondefs -nullderef @*/	/* _dig annotations are not correct. */
-int pgpPrtPkts(const uint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
+int pgpPrtPkts(const rpmuint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
 	/*@globals _dig, _digp, _pgp_print @*/
 	/*@modifies _dig, _digp, *_digp, _pgp_print @*/
 {
@@ -1226,7 +1243,7 @@ int pgpPrtPkts(const uint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
     unsigned int val = (unsigned int)*pkts;
     size_t pleft;
     int len;
-    uint8_t ** ppkts = NULL;
+    rpmuint8_t ** ppkts = NULL;
     int npkts;
     int i;
 
@@ -1235,12 +1252,12 @@ int pgpPrtPkts(const uint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
     if (dig != NULL && (val & 0x80)) {
 	pgpTag tag = (val & 0x40) ? (val & 0x3f) : ((val >> 2) & 0xf);
 	_digp = (tag == PGPTAG_SIGNATURE) ? &_dig->signature : &_dig->pubkey;
-	_digp->tag = tag;
+	_digp->tag = (rpmuint8_t)tag;
     } else
 	_digp = NULL;
 
     if (pgpGrabPkts(pkts, pktlen, &ppkts, &npkts) || ppkts == NULL) {
-	_dig = pgpDigFree(_dig);
+	_dig = pgpDigFree(_dig, "pgpPrtPkts");
 	return -1;
     }
 
@@ -1257,22 +1274,21 @@ int pgpPrtPkts(const uint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
     } else
 	ppkts = _free(ppkts);
 
-    _dig = pgpDigFree(_dig);
+    _dig = pgpDigFree(_dig, "pgpPrtPkts");
     return 0;
 }
 /*@=globstate =incondefs =nullderef @*/
 
-pgpArmor pgpReadPkts(const char * fn, const uint8_t ** pkt, size_t * pktlen)
+pgpArmor pgpReadPkts(const char * fn, rpmuint8_t ** pkt, size_t * pktlen)
 {
-    uint8_t * b = NULL;
-    ssize_t blen;
+    rpmiob iob = NULL;
     const char * enc = NULL;
     const char * crcenc = NULL;
-    uint8_t * dec;
-    uint8_t * crcdec;
+    rpmuint8_t * dec;
+    rpmuint8_t * crcdec;
     size_t declen;
     size_t crclen;
-    uint32_t crcpkt, crc;
+    rpmuint32_t crcpkt, crc;
     const char * armortype = NULL;
     char * t, * te;
     int pstate = 0;
@@ -1280,13 +1296,12 @@ pgpArmor pgpReadPkts(const char * fn, const uint8_t ** pkt, size_t * pktlen)
     pgpTag tag = 0;
     int rc;
 
-    rc = rpmioSlurp(fn, &b, &blen);
-    if (rc || b == NULL || blen <= 0) {
+    rc = rpmiobSlurp(fn, &iob);
+    if (rc || iob == NULL)
 	goto exit;
-    }
 
     /* Read unarmored packets. */
-    if (pgpIsPkt(b, &tag)) {
+    if (pgpIsPkt(iob->b, &tag)) {
 	switch (tag) {
 	default:		ec = PGPARMOR_NONE;	break;
 	case PGPTAG_PUBLIC_KEY:	ec = PGPARMOR_PUBKEY;	break;
@@ -1302,7 +1317,7 @@ pgpArmor pgpReadPkts(const char * fn, const uint8_t ** pkt, size_t * pktlen)
 	/* Truncate blen to actual no. of octets in packet. */
 	if (ec != PGPARMOR_NONE) {
 	    pgpPkt pp = alloca(sizeof(*pp));
-	    blen = pgpPktLen(b, blen, pp);
+	    iob->blen = pgpPktLen(iob->b, iob->blen, pp);
 	}
 	goto exit;
     }
@@ -1310,7 +1325,7 @@ pgpArmor pgpReadPkts(const char * fn, const uint8_t ** pkt, size_t * pktlen)
 #define	TOKEQ(_s, _tok)	(!strncmp((_s), (_tok), sizeof(_tok)-1))
 
     /* Read armored packets, converting to binary. */
-    for (t = (char *)b; t && *t; t = te) {
+    for (t = (char *)iob->b; t && *t; t = te) {
 	if ((te = strchr(t, '\n')) == NULL)
 	    te = t + strlen(t);
 	else
@@ -1412,9 +1427,9 @@ pgpArmor pgpReadPkts(const char * fn, const uint8_t ** pkt, size_t * pktlen)
 		ec = PGPARMOR_ERR_CRC_CHECK;
 		goto exit;
 	    }
-	    b = _free(b);
-	    b = dec;
-	    blen = declen;
+	    iob->b = _free(iob->b);
+	    iob->b = dec;
+	    iob->blen = declen;
 	    goto exit;
 	    /*@notreached@*/ /*@switchbreak@*/ break;
 	}
@@ -1422,16 +1437,19 @@ pgpArmor pgpReadPkts(const char * fn, const uint8_t ** pkt, size_t * pktlen)
     ec = PGPARMOR_NONE;
 
 exit:
-    if (ec > PGPARMOR_NONE && pkt)
-	*pkt = b;
-    else if (b != NULL)
-	b = _free(b);
-    if (pktlen)
-	*pktlen = blen;
+    if (ec > PGPARMOR_NONE) {
+	if (pkt)	*pkt = iob->b;
+	if (pktlen)	*pktlen = iob->blen;
+	iob->b = NULL;	/* XXX iob->b has been stolen */
+    } else {
+	if (pkt)	*pkt = NULL;
+	if (pktlen)	*pktlen = 0;
+    }
+    iob = rpmiobFree(iob);
     return ec;
 }
 
-char * pgpArmorWrap(int atype, const unsigned char * s, size_t ns)
+char * pgpArmorWrap(rpmuint8_t atype, const unsigned char * s, size_t ns)
 {
     const char * enc;
     char * t;
@@ -1480,7 +1498,7 @@ char * pgpArmorWrap(int atype, const unsigned char * s, size_t ns)
 /*@=globstate@*/
 }
 
-int pgpHashAlgoStringToNumber(const char *name, size_t name_len)
+pgpHashAlgo pgpHashAlgoStringToNumber(const char *name, size_t name_len)
 {
     size_t i;
 
@@ -1491,6 +1509,6 @@ int pgpHashAlgoStringToNumber(const char *name, size_t name_len)
     for (i = 0; i < sizeof(pgpHashTbl)/sizeof(pgpHashTbl[0]); i++)
         if (xstrncasecmp(name, pgpHashTbl[i].str, name_len) == 0)
             return pgpHashTbl[i].val;
-    return -1;
+    return PGPHASHALGO_ERROR;
 }
 
