@@ -12,66 +12,8 @@
 
 #include <string.h>
 #include <popt.h>
-
-/** \ingroup rpmpgp
- */
-typedef /*@abstract@*/ struct DIGEST_CTX_s * DIGEST_CTX;
-
-/** \ingroup rpmpgp
- */
-typedef /*@abstract@*/ struct pgpPkt_s * pgpPkt;
-
-/**
- */
-typedef /*@abstract@*/ /*@refcounted@*/ struct pgpDig_s * pgpDig;
-
-/**
- */
-typedef /*@abstract@*/ struct pgpDigParams_s * pgpDigParams;
-
-/** \ingroup rpmpgp
- * Bit(s) to control digest and signature verification.
- */
-typedef enum pgpVSFlags_e {
-    RPMVSF_DEFAULT	= 0,
-    RPMVSF_NOHDRCHK	= (1 <<  0),
-    RPMVSF_NEEDPAYLOAD	= (1 <<  1),
-    /* bit(s) 2-7 unused */
-    RPMVSF_NOSHA1HEADER	= (1 <<  8),
-    RPMVSF_NOMD5HEADER	= (1 <<  9),	/* unimplemented */
-    RPMVSF_NODSAHEADER	= (1 << 10),
-    RPMVSF_NORSAHEADER	= (1 << 11),
-    /* bit(s) 12-15 unused */
-    RPMVSF_NOSHA1	= (1 << 16),	/* unimplemented */
-    RPMVSF_NOMD5	= (1 << 17),
-    RPMVSF_NODSA	= (1 << 18),
-    RPMVSF_NORSA	= (1 << 19)
-    /* bit(s) 20-31 unused */
-} pgpVSFlags;
-
-#define	_RPMVSF_NODIGESTS	\
-  ( RPMVSF_NOSHA1HEADER |	\
-    RPMVSF_NOMD5HEADER |	\
-    RPMVSF_NOSHA1 |		\
-    RPMVSF_NOMD5 )
-
-#define	_RPMVSF_NOSIGNATURES	\
-  ( RPMVSF_NODSAHEADER |	\
-    RPMVSF_NORSAHEADER |	\
-    RPMVSF_NODSA |		\
-    RPMVSF_NORSA )
-
-#define	_RPMVSF_NOHEADER	\
-  ( RPMVSF_NOSHA1HEADER |	\
-    RPMVSF_NOMD5HEADER |	\
-    RPMVSF_NODSAHEADER |	\
-    RPMVSF_NORSAHEADER )
-
-#define	_RPMVSF_NOPAYLOAD	\
-  ( RPMVSF_NOSHA1 |		\
-    RPMVSF_NOMD5 |		\
-    RPMVSF_NODSA |		\
-    RPMVSF_NORSA )
+#include <rpmiotypes.h>
+#include <yarn.h>
 
 #if defined(_RPMPGP_INTERNAL)
 #include <rpmsw.h>
@@ -83,20 +25,20 @@ struct pgpDigParams_s {
 /*@only@*/ /*@null@*/
     const char * userid;
 /*@only@*/ /*@null@*/
-    const uint8_t * hash;
+    const rpmuint8_t * hash;
     const char * params[4];
-    uint8_t tag;
+    rpmuint8_t tag;
 
-    uint8_t version;		/*!< version number. */
-    uint8_t time[4];		/*!< time that the key was created. */
-    uint8_t pubkey_algo;	/*!< public key algorithm. */
+    rpmuint8_t version;		/*!< version number. */
+    rpmuint8_t time[4];		/*!< time that the key was created. */
+    rpmuint8_t pubkey_algo;	/*!< public key algorithm. */
 
-    uint8_t hash_algo;
-    uint8_t sigtype;
-    uint8_t hashlen;
-    uint8_t signhash16[2];
-    uint8_t signid[8];
-    uint8_t saved;
+    rpmuint8_t hash_algo;
+    rpmuint8_t sigtype;
+    size_t hashlen;
+    rpmuint8_t signhash16[2];
+    rpmuint8_t signid[8];
+    rpmuint8_t saved;
 #define	PGPDIG_SAVED_TIME	(1 << 0)
 #define	PGPDIG_SAVED_ID		(1 << 1)
 
@@ -106,14 +48,15 @@ struct pgpDigParams_s {
  * Container for values parsed from an OpenPGP signature and public key.
  */
 struct pgpDig_s {
+    struct rpmioItem_s _item;	/*!< usage mutex and pool identifier. */
     struct pgpDigParams_s signature;
     struct pgpDigParams_s pubkey;
 
-    uint32_t sigtag;		/*!< Package signature tag. */
-    uint32_t sigtype;		/*!< Package signature data type. */
+    rpmuint32_t sigtag;		/*!< Package signature tag. */
+    rpmuint32_t sigtype;	/*!< Package signature data type. */
 /*@relnull@*/
     const void * sig;		/*!< Package signature. */
-    uint32_t siglen;		/*!< Package signature length. */
+    rpmuint32_t siglen;		/*!< Package signature length. */
 
     pgpVSFlags vsflags;		/*!< Digest/signature operation disablers. */
     struct rpmop_s dops;	/*!< Digest operation statistics. */
@@ -123,10 +66,8 @@ struct pgpDig_s {
 	/*@modifies *_ts, *_dig @*/;/*!< Find pubkey, i.e. rpmtsFindPubkey(). */
 /*@null@*/
     void * _ts;			/*!< Find pubkey argument, i.e. rpmts. */
-/*@refs@*/
-    int nrefs;			/*!< Reference count. */
 
-    uint8_t ** ppkts;
+    rpmuint8_t ** ppkts;
     int npkts;
     size_t nbytes;		/*!< No. bytes of plain text. */
 
@@ -141,12 +82,16 @@ struct pgpDig_s {
 /*@only@*/ /*@null@*/
     DIGEST_CTX md5ctx;		/*!< (rsa) md5 hash context. */
 /*@only@*/ /*@null@*/
-    DIGEST_CTX hdrmd5ctx;	/*!< (rsa) header md5 hash context. */
+    DIGEST_CTX hdrctx;		/*!< (rsa) header hash context. */
 /*@only@*/ /*@null@*/
     void * md5;			/*!< (rsa) V3 signature hash. */
     size_t md5len;		/*!< (rsa) V3 signature hash length. */
 /*@owned@*/ /*@relnull@*/
     void * impl;		/*!< Implementation data */
+#if defined(__LCLINT__)
+/*@refs@*/
+    int nrefs;			/*!< (unused) keep splint happy */
+#endif
 };
 #endif
 
@@ -154,7 +99,8 @@ struct pgpDig_s {
  */
 typedef const struct pgpValTbl_s {
     int val;
-/*@observer@*/ const char * str;
+/*@observer@*/
+    const char * str;
 } * pgpValTbl;
  
 /** \ingroup rpmpgp
@@ -230,12 +176,11 @@ extern struct pgpValTbl_s pgpTagTbl[];
  *   - MPI of Elgamal (Diffie-Hellman) value m * y**k mod p.
  */
 typedef struct pgpPktPubkey_s {
-    uint8_t version;	/*!< version number (generate 3, accept 2). */
-    uint8_t keyid[8];	/*!< key ID of the public key for session key. */
-    uint8_t algo;	/*!< public key algorithm used. */
+    rpmuint8_t version;		/*!< version number (generate 3, accept 2). */
+    rpmuint8_t keyid[8];	/*!< key ID of the public key for session. */
+    rpmuint8_t algo;		/*!< public key algorithm used. */
 } pgpPktPubkey;
 
-
 /** \ingroup rpmpgp
  * 5.2.1. Signature Types
  * 
@@ -260,7 +205,8 @@ typedef enum pgpSigType_e {
     PGPSIGTYPE_KEY_REVOKE	 = 0x20, /*!< Key revocation */
     PGPSIGTYPE_SUBKEY_REVOKE	 = 0x28, /*!< Subkey revocation */
     PGPSIGTYPE_CERT_REVOKE	 = 0x30, /*!< Certification revocation */
-    PGPSIGTYPE_TIMESTAMP	 = 0x40  /*!< Timestamp */
+    PGPSIGTYPE_TIMESTAMP	 = 0x40,  /*!< Timestamp */
+    PGPSIGTYPE_CONFIRM		 = 0x50  /*!< Third-Party confirmation */
 } pgpSigType;
 /*@=typeuse@*/
 
@@ -325,9 +271,10 @@ extern struct pgpValTbl_s pgpPubkeyTbl[];
        4          - Blowfish (128 bit key, 16 rounds) [BLOWFISH]
        5          - SAFER-SK128 (13 rounds) [SAFER]
        6          - Reserved for DES/SK
-       7          - Reserved for AES with 128-bit key
-       8          - Reserved for AES with 192-bit key
-       9          - Reserved for AES with 256-bit key
+       7          - AES with 128-bit key
+       8          - AES with 192-bit key
+       9          - AES with 256-bit key
+       10         - Twofish with 256-bit key
        100 to 110 - Private/Experimental algorithm.
 \endverbatim
  *
@@ -388,52 +335,6 @@ typedef enum pgpCompressAlgo_e {
 /*@observer@*/ /*@unchecked@*/ /*@unused@*/
 extern struct pgpValTbl_s pgpCompressionTbl[];
 
-/** \ingroup rpmpgp
- * 9.4. Hash Algorithms
- *
-\verbatim
-       ID           Algorithm                              Text Name
-       --           ---------                              ---- ----
-       1          - MD5                                    "MD5"
-       2          - SHA-1                                  "SHA1"
-       3          - RIPE-MD/160                            "RIPEMD160"
-       4          - Reserved for double-width SHA (experimental)
-       5          - MD2                                    "MD2"
-       6          - Reserved for TIGER/192                 "TIGER192"
-       7          - Reserved for HAVAL (5 pass, 160-bit)
-       "HAVAL-5-160"
-       100 to 110 - Private/Experimental algorithm.
-\endverbatim
- *
- * Implementations MUST implement SHA-1. Implementations SHOULD
- * implement MD5.
- * @todo Add SHA256.
- */
-typedef enum pgpHashAlgo_e {
-    PGPHASHALGO_MD5		=  1,	/*!< MD5 */
-    PGPHASHALGO_SHA1		=  2,	/*!< SHA-1 */
-    PGPHASHALGO_RIPEMD160	=  3,	/*!< RIPEMD-160 */
-    PGPHASHALGO_MD2		=  5,	/*!< MD2 */
-    PGPHASHALGO_TIGER192	=  6,	/*!< TIGER-192 */
-    PGPHASHALGO_HAVAL_5_160	=  7,	/*!< HAVAL-5-160 */
-    PGPHASHALGO_SHA256		=  8,	/*!< SHA-256 */
-    PGPHASHALGO_SHA384		=  9,	/*!< SHA-384 */
-    PGPHASHALGO_SHA512		= 10,	/*!< SHA-512 */
-
-    PGPHASHALGO_MD4		= 104,	/*!< (private) MD4 */
-    PGPHASHALGO_RIPEMD128	= 105,	/*!< (private) RIPEMD-128 */
-    PGPHASHALGO_CRC32		= 106,	/*!< (private) CRC-32 */
-    PGPHASHALGO_ADLER32		= 107,	/*!< (private) ADLER-32 */
-    PGPHASHALGO_CRC64		= 108,	/*!< (private) CRC-64 */
-    PGPHASHALGO_JLU32		= 109,	/*!< (private) Jenkins lookup3.c */
-    PGPHASHALGO_SHA224		= 110,	/*!< (private) SHA-224 */
-    PGPHASHALGO_RIPEMD256	= 111,	/*!< (private) RIPEMD-256 */
-    PGPHASHALGO_RIPEMD320	= 112,	/*!< (private) RIPEMD-320 */
-    PGPHASHALGO_SALSA10		= 113,	/*!< (private) SALSA-10 */
-    PGPHASHALGO_SALSA20		= 114,	/*!< (private) SALSA-20 */
-
-} pgpHashAlgo;
-
 /** \ingroup rpmpgp
  * Hash (string, value) pairs.
  */
@@ -462,14 +363,14 @@ extern struct pgpValTbl_s pgpHashTbl[];
  *   - MPI of DSA value s.
  */
 typedef struct pgpPktSigV3_s {
-    uint8_t version;	/*!< version number (3). */
-    uint8_t hashlen;	/*!< length of following hashed material. MUST be 5. */
-    uint8_t sigtype;	/*!< signature type. */
-    uint8_t time[4];	/*!< 4 byte creation time. */
-    uint8_t signid[8];	/*!< key ID of signer. */
-    uint8_t pubkey_algo;/*!< public key algorithm. */
-    uint8_t hash_algo;	/*!< hash algorithm. */
-    uint8_t signhash16[2];	/*!< left 16 bits of signed hash value. */
+    rpmuint8_t version;	/*!< version number (3). */
+    rpmuint8_t hashlen;	/*!< length of following hashed material. MUST be 5. */
+    rpmuint8_t sigtype;	/*!< signature type. */
+    rpmuint8_t time[4];	/*!< 4 byte creation time. */
+    rpmuint8_t signid[8];	/*!< key ID of signer. */
+    rpmuint8_t pubkey_algo;	/*!< public key algorithm. */
+    rpmuint8_t hash_algo;	/*!< hash algorithm. */
+    rpmuint8_t signhash16[2];	/*!< left 16 bits of signed hash value. */
 } * pgpPktSigV3;
 
 /** \ingroup rpmpgp
@@ -494,11 +395,11 @@ typedef struct pgpPktSigV3_s {
  *   - One or more multi-precision integers comprising the signature.
  */
 typedef struct pgpPktSigV4_s {
-    uint8_t version;	/*!< version number (4). */
-    uint8_t sigtype;	/*!< signature type. */
-    uint8_t pubkey_algo;/*!< public key algorithm. */
-    uint8_t hash_algo;	/*!< hash algorithm. */
-    uint8_t hashlen[2];	/*!< length of following hashed material. */
+    rpmuint8_t version;		/*!< version number (4). */
+    rpmuint8_t sigtype;		/*!< signature type. */
+    rpmuint8_t pubkey_algo;	/*!< public key algorithm. */
+    rpmuint8_t hash_algo;	/*!< hash algorithm. */
+    rpmuint8_t hashlen[2];	/*!< length of following hashed material. */
 } * pgpPktSigV4;
 
 /** \ingroup rpmpgp
@@ -534,17 +435,26 @@ typedef struct pgpPktSigV4_s {
  * The value of the subpacket type octet may be:
  *
 \verbatim
+       0 = reserved
+       1 = reserved
        2 = signature creation time
        3 = signature expiration time
        4 = exportable certification
        5 = trust signature
        6 = regular expression
        7 = revocable
+       8 = reserved
        9 = key expiration time
        10 = placeholder for backward compatibility
        11 = preferred symmetric algorithms
        12 = revocation key
+       13 = reserved
+       14 = reserved
+       15 = reserved
        16 = issuer key ID
+       17 = reserved
+       18 = reserved
+       19 = reserved
        20 = notation data
        21 = preferred hash algorithms
        22 = preferred compression algorithms
@@ -555,6 +465,9 @@ typedef struct pgpPktSigV4_s {
        27 = key flags
        28 = signer's user id
        29 = reason for revocation
+       30 = features
+       31 = signature target
+       32 = embedded signature
        100 to 110 = internal or user-defined
 \endverbatim
  *
@@ -591,8 +504,9 @@ typedef enum pgpSubType_e {
     PGPSUBTYPE_KEY_FLAGS	=  27, /*!< key flags */
     PGPSUBTYPE_SIGNER_USERID	=  28, /*!< signer's user id */
     PGPSUBTYPE_REVOKE_REASON	=  29, /*!< reason for revocation */
-    PGPSUBTYPE_FEATURES		=  30, /*!< feature flags (gpg) */
-    PGPSUBTYPE_EMBEDDED_SIG	=  32, /*!< embedded signature (gpg) */
+    PGPSUBTYPE_FEATURES		=  30, /*!< feature flags */
+    PGPSUBTYPE_SIG_TARGET	=  31, /*!< signature target */
+    PGPSUBTYPE_EMBEDDED_SIG	=  32, /*!< embedded signature */
 
     PGPSUBTYPE_INTERNAL_100	= 100, /*!< internal or user-defined */
     PGPSUBTYPE_INTERNAL_101	= 101, /*!< internal or user-defined */
@@ -670,9 +584,9 @@ typedef union pgpPktSig_u {
  *
  */
 typedef struct pgpPktSymkey_s {
-    uint8_t version;	/*!< version number (4). */
-    uint8_t symkey_algo;
-    uint8_t s2k[1];
+    rpmuint8_t version;	/*!< version number (4). */
+    rpmuint8_t symkey_algo;
+    rpmuint8_t s2k[1];
 } pgpPktSymkey;
 
 /** \ingroup rpmpgp
@@ -705,12 +619,12 @@ typedef struct pgpPktSymkey_s {
  * pass packet.
  */
 typedef struct pgpPktOnepass_s {
-    uint8_t version;	/*!< version number (3). */
-    uint8_t sigtype;	/*!< signature type. */
-    uint8_t hash_algo;	/*!< hash algorithm. */
-    uint8_t pubkey_algo;/*!< public key algorithm. */
-    uint8_t signid[8];	/*!< key ID of signer. */
-    uint8_t nested;
+    rpmuint8_t version;		/*!< version number (3). */
+    rpmuint8_t sigtype;		/*!< signature type. */
+    rpmuint8_t hash_algo;	/*!< hash algorithm. */
+    rpmuint8_t pubkey_algo;	/*!< public key algorithm. */
+    rpmuint8_t signid[8];	/*!< key ID of signer. */
+    rpmuint8_t nested;
 } * pgpPktOnepass;
 
 /** \ingroup rpmpgp
@@ -786,10 +700,10 @@ typedef struct pgpPktOnepass_s {
  *
  */
 typedef struct pgpPktKeyV3_s {
-    uint8_t version;	/*!< version number (3). */
-    uint8_t time[4];	/*!< time that the key was created. */
-    uint8_t valid[2];	/*!< time in days that this key is valid. */
-    uint8_t pubkey_algo;/*!< public key algorithm. */
+    rpmuint8_t version;		/*!< version number (3). */
+    rpmuint8_t time[4];		/*!< time that the key was created. */
+    rpmuint8_t valid[2];	/*!< time in days that this key is valid. */
+    rpmuint8_t pubkey_algo;	/*!< public key algorithm. */
 } * pgpPktKeyV3;
 
 /** \ingroup rpmpgp
@@ -824,9 +738,9 @@ typedef struct pgpPktKeyV3_s {
  *
  */
 typedef struct pgpPktKeyV4_s {
-    uint8_t version;	/*!< version number (4). */
-    uint8_t time[4];	/*!< time that the key was created. */
-    uint8_t pubkey_algo;/*!< public key algorithm. */
+    rpmuint8_t version;		/*!< version number (4). */
+    rpmuint8_t time[4];		/*!< time that the key was created. */
+    rpmuint8_t pubkey_algo;	/*!< public key algorithm. */
 } * pgpPktKeyV4;
 
 /** \ingroup rpmpgp
@@ -923,8 +837,8 @@ typedef union pgpPktKey_u {
  * blocks.
  */
 typedef struct pgpPktCdata_s {
-    uint8_t compressalgo;
-    uint8_t data[1];
+    rpmuint8_t compressalgo;
+    rpmuint8_t data[1];
 } pgpPktCdata;
 
 /** \ingroup rpmpgp
@@ -962,7 +876,7 @@ typedef struct pgpPktCdata_s {
  * session key is incorrect.
  */
 typedef struct pgpPktEdata_s {
-    uint8_t data[1];
+    rpmuint8_t data[1];
 } pgpPktEdata;
 
 /** \ingroup rpmpgp
@@ -1013,9 +927,9 @@ typedef struct pgpPktEdata_s {
  * the receiving software.
  */
 typedef struct pgpPktLdata_s {
-    uint8_t format;
-    uint8_t filenamelen;
-    uint8_t filename[1];
+    rpmuint8_t format;
+    rpmuint8_t filenamelen;
+    rpmuint8_t filename[1];
 } pgpPktLdata;
 
 /** \ingroup rpmpgp
@@ -1032,7 +946,7 @@ typedef struct pgpPktLdata_s {
  * other than local keyring files.
  */
 typedef struct pgpPktTrust_s {
-    uint8_t flag;
+    rpmuint8_t flag;
 } pgpPktTrust;
 
 /** \ingroup rpmpgp
@@ -1046,7 +960,7 @@ typedef struct pgpPktTrust_s {
  *
  */
 typedef struct pgpPktUid_s {
-    uint8_t userid[1];
+    rpmuint8_t userid[1];
 } pgpPktUid;
 
 /** \ingroup rpmpgp
@@ -1112,23 +1026,6 @@ typedef enum pgpArmorKey_e {
 /*@observer@*/ /*@unchecked@*/ /*@unused@*/
 extern struct pgpValTbl_s pgpArmorKeyTbl[];
 
-/** \ingroup rpmpgp
- * Bit(s) to control digest operation.
- */
-typedef enum rpmDigestFlags_e {
-    RPMDIGEST_NONE	= 0
-} rpmDigestFlags;
-
-/** \ingroup rpmpgp
- */
-/*@unchecked@*/
-extern pgpHashAlgo rpmDigestHashAlgo;
-
-/** \ingroup rpmpgp
- */
-/*@unchecked@*/ /*@observer@*/
-extern struct poptOption rpmDigestPoptTable[];
-
 /*@-fcnuse@*/
 #ifdef __cplusplus
 extern "C" {
@@ -1141,7 +1038,7 @@ extern "C" {
  * @return		native-endian integer
  */
 /*@unused@*/ static inline
-unsigned int pgpGrab(const uint8_t * s, size_t nbytes)
+unsigned int pgpGrab(const rpmuint8_t * s, size_t nbytes)
 	/*@*/
 {
     unsigned int i = 0;
@@ -1158,14 +1055,14 @@ unsigned int pgpGrab(const uint8_t * s, size_t nbytes)
  * @return		no. of bytes in length prefix
  */
 /*@unused@*/ static inline
-int pgpLen(const uint8_t * s, /*@out@*/ unsigned int * lenp)
+unsigned int pgpLen(const rpmuint8_t * s, /*@out@*/ unsigned int * lenp)
 	/*@modifies *lenp @*/
 {
-    if (*s < 192) {
+    if (*s < (rpmuint8_t)192) {
 	*lenp = (unsigned int) *s++;
 	return 1;
-    } else if (*s < 255) {
-	*lenp = (unsigned int) ((((unsigned)s[0]) - 192) << 8) + s[1] + 192;
+    } else if (*s < (rpmuint8_t)255) {
+	*lenp = (unsigned int) ((((unsigned)s[0]) - 192) << 8) + (unsigned)s[1] + 192;
 	return 2;
     } else {
 	*lenp = pgpGrab(s+1, 4);
@@ -1179,7 +1076,7 @@ int pgpLen(const uint8_t * s, /*@out@*/ unsigned int * lenp)
  * @return		no. of bits
  */
 /*@unused@*/ static inline
-unsigned int pgpMpiBits(const uint8_t * p)
+unsigned int pgpMpiBits(const rpmuint8_t * p)
 	/*@requires maxRead(p) >= 1 @*/
 	/*@*/
 {
@@ -1192,7 +1089,7 @@ unsigned int pgpMpiBits(const uint8_t * p)
  * @return		no. of bytes
  */
 /*@unused@*/ static inline
-unsigned int pgpMpiLen(const uint8_t * p)
+unsigned int pgpMpiLen(const rpmuint8_t * p)
 	/*@requires maxRead(p) >= 1 @*/
 	/*@*/
 {
@@ -1207,7 +1104,7 @@ unsigned int pgpMpiLen(const uint8_t * p)
  * @return		target buffer
  */
 /*@unused@*/ static inline
-char * pgpHexCvt(/*@returned@*/ char * t, const uint8_t * s, size_t nbytes)
+char * pgpHexCvt(/*@returned@*/ char * t, const rpmuint8_t * s, size_t nbytes)
 	/*@modifies *t @*/
 {
     static char hex[] = "0123456789abcdef";
@@ -1229,7 +1126,7 @@ char * pgpHexCvt(/*@returned@*/ char * t, const uint8_t * s, size_t nbytes)
  * @return		hex formatted string
  */
 /*@unused@*/ static inline /*@observer@*/
-char * pgpHexStr(const uint8_t * p, size_t plen)
+char * pgpHexStr(const rpmuint8_t * p, size_t plen)
 	/*@*/
 {
     static char prbuf[8*BUFSIZ];	/* XXX ick */
@@ -1245,7 +1142,7 @@ char * pgpHexStr(const uint8_t * p, size_t plen)
  * @return		hex formatted string
  */
 /*@unused@*/ static inline /*@observer@*/
-const char * pgpMpiStr(const uint8_t * p)
+const char * pgpMpiStr(const rpmuint8_t * p)
 	/*@requires maxRead(p) >= 3 @*/
 	/*@*/
 {
@@ -1264,7 +1161,7 @@ const char * pgpMpiStr(const uint8_t * p)
  * @return		string value of byte
  */
 /*@unused@*/ static inline /*@observer@*/
-const char * pgpValStr(pgpValTbl vs, uint8_t val)
+const char * pgpValStr(pgpValTbl vs, rpmuint8_t val)
 	/*@*/
 {
     do {
@@ -1287,7 +1184,7 @@ int pgpValTok(pgpValTbl vs, const char * s, const char * se)
 {
     do {
 	size_t vlen = strlen(vs->str);
-	if (vlen <= (se-s) && !strncmp(s, vs->str, vlen))
+	if (vlen <= (size_t)(se-s) && !strncmp(s, vs->str, vlen))
 	    break;
     } while ((++vs)->val != -1);
     return vs->val;
@@ -1300,7 +1197,7 @@ int pgpValTok(pgpValTbl vs, const char * s, const char * se)
  * @param val		byte value to print
  */
 /*@-exportlocal@*/
-void pgpPrtVal(const char * pre, pgpValTbl vs, uint8_t val)
+void pgpPrtVal(const char * pre, pgpValTbl vs, rpmuint8_t val)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
 /*@=exportlocal@*/
@@ -1313,7 +1210,7 @@ void pgpPrtVal(const char * pre, pgpValTbl vs, uint8_t val)
  * @return		0 on success
  */
 /*@-exportlocal@*/
-int pgpPrtSubType(const uint8_t * h, size_t hlen, pgpSigType sigtype)
+int pgpPrtSubType(const rpmuint8_t * h, size_t hlen, pgpSigType sigtype)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
 /*@=exportlocal@*/
@@ -1369,8 +1266,8 @@ int pgpPrtComment(const pgpPkt pp)
  * @return		0 on sucess, else -1
  */
 /*@-exportlocal@*/
-int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen,
-		/*@out@*/ uint8_t * keyid)
+int pgpPubkeyFingerprint(const rpmuint8_t * pkt, size_t pktlen,
+		/*@out@*/ rpmuint8_t * keyid)
 	/*@modifies *keyid @*/;
 /*@=exportlocal@*/
 
@@ -1381,7 +1278,8 @@ int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen,
  * @retval keyid[8]	public key fingerprint
  * @return		8 (no. of bytes) on success, < 0 on error
  */
-int pgpExtractPubkeyFingerprint(const char * b64pkt, /*@out@*/ uint8_t * keyid)
+int pgpExtractPubkeyFingerprint(const char * b64pkt,
+		/*@out@*/ rpmuint8_t * keyid)
 	/*@modifies *keyid @*/;
 
 /** \ingroup rpmpgp
@@ -1391,7 +1289,7 @@ int pgpExtractPubkeyFingerprint(const char * b64pkt, /*@out@*/ uint8_t * keyid)
  * @retval pp		packet tag/ptr/len
  * @return		packet length, <0 on error.
  */
-int pgpPktLen(const uint8_t * pkt, size_t pleft, /*@out@*/ pgpPkt pp)
+int pgpPktLen(const rpmuint8_t * pkt, size_t pleft, /*@out@*/ pgpPkt pp)
 	/*@modifies pp @*/;
 
 /** \ingroup rpmpgp
@@ -1401,7 +1299,7 @@ int pgpPktLen(const uint8_t * pkt, size_t pleft, /*@out@*/ pgpPkt pp)
  * @return		-1 on error, otherwise this packet length
  */
 /*@-exportlocal@*/
-int pgpPrtPkt(const uint8_t * pkt, size_t pleft)
+int pgpPrtPkt(const rpmuint8_t * pkt, size_t pleft)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
 /*@=exportlocal@*/
@@ -1414,7 +1312,7 @@ int pgpPrtPkt(const uint8_t * pkt, size_t pleft)
  * @param printing	should packets be printed?
  * @return		-1 on error, 0 on success
  */
-int pgpPrtPkts(const uint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
+int pgpPrtPkts(const rpmuint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dig, fileSystem, internalState @*/;
 
@@ -1426,7 +1324,7 @@ int pgpPrtPkts(const uint8_t * pkts, size_t pktlen, pgpDig dig, int printing)
  * @return		type of armor found
  */
 pgpArmor pgpReadPkts(const char * fn,
-		/*@out@*/ const uint8_t ** pkt, /*@out@*/ size_t * pktlen)
+		/*@out@*/ rpmuint8_t ** pkt, /*@out@*/ size_t * pktlen)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies *pkt, *pktlen, fileSystem, internalState @*/;
 
@@ -1437,16 +1335,16 @@ pgpArmor pgpReadPkts(const char * fn,
  * @param ns		binary pkt data length
  * @return		formatted string
  */
-char * pgpArmorWrap(int atype, const unsigned char * s, size_t ns)
+char * pgpArmorWrap(rpmuint8_t atype, const unsigned char * s, size_t ns)
 	/*@*/;
 
 /** \ingroup rpmpgp
- * Convert a hash algorithm <name> to the internal PGPHASHALGO_<name> number
+ * Convert a hash algorithm "foo" to the internal PGPHASHALGO_FOO number.
  * @param name		name of hash algorithm
  * @param name_len		length of name or 0 for strlen(name)
  * @return		PGPHASHALGO_<name> or -1 in case of error
  */
-int pgpHashAlgoStringToNumber(const char *name, size_t name_len)
+pgpHashAlgo pgpHashAlgoStringToNumber(const char *name, size_t name_len)
 	/*@*/;
 
 /**
@@ -1459,21 +1357,14 @@ extern pgpVSFlags pgpDigVSFlags;
  * Unreference a signature parameters instance.
  * @param dig		signature parameters
  * @param msg
- * @return		NULL always
+ * @return		NULL on last dereference
  */
 /*@unused@*/ /*@null@*/
 pgpDig pgpDigUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ pgpDig dig,
 		/*@null@*/ const char * msg)
 	/*@modifies dig @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@-exportlocal@*/
-/*@null@*/
-pgpDig XpgpDigUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ pgpDig dig,
-		/*@null@*/ const char * msg, const char * fn, unsigned ln)
-	/*@modifies dig @*/;
-/*@=exportlocal@*/
-#define	pgpDigUnlink(_dig, _msg) XpgpDigUnlink(_dig, _msg, __FILE__, __LINE__)
+#define	pgpDigUnlink(_dig, _msg)	\
+    ((pgpDig)rpmioUnlinkPoolItem((rpmioItem)(_dig), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmpgp
  * Reference a signature parameters instance.
@@ -1484,22 +1375,19 @@ pgpDig XpgpDigUnlink (/*@killref@*/ /*@only@*/ /*@null@*/ pgpDig dig,
 /*@unused@*/ /*@newref@*/ /*@null@*/
 pgpDig pgpDigLink (/*@null@*/ pgpDig dig, /*@null@*/ const char * msg)
 	/*@modifies dig @*/;
-
-/** @todo Remove debugging entry from the ABI. */
-/*@newref@*/ /*@null@*/
-pgpDig XpgpDigLink (/*@null@*/ pgpDig dig, /*@null@*/ const char * msg,
-		const char * fn, unsigned ln)
-	/*@modifies dig @*/;
-#define	pgpDigLink(_dig, _msg)	XpgpDigLink(_dig, _msg, __FILE__, __LINE__)
+#define	pgpDigLink(_dig, _msg)	\
+    ((pgpDig)rpmioLinkPoolItem((rpmioItem)(_dig), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmpgp
  * Destroy a container for parsed OpenPGP packates.
  * @param dig		signature parameters container
- * @return		NULL always
+ * @return		NULL on last dereference
  */
-/*@null@*/
+/*@unused@*/ /*@null@*/
 pgpDig pgpDigFree(/*@killref@*/ /*@only@*/ /*@null@*/ pgpDig dig)
 	/*@modifies dig @*/;
+#define pgpDigFree(_dig, _msg)       \
+    ((pgpDig)rpmioFreePoolItem((rpmioItem)(_dig), _msg, __FILE__, __LINE__))
 
 /** \ingroup rpmpgp
  * Create a container for parsed OpenPGP packates.
@@ -1507,7 +1395,8 @@ pgpDig pgpDigFree(/*@killref@*/ /*@only@*/ /*@null@*/ pgpDig dig)
  */
 /*@relnull@*/
 pgpDig pgpDigNew(/*@unused@*/ pgpVSFlags vsflags)
-	/*@*/;
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/;
 
 /** \ingroup rpmpgp
  * Release (malloc'd) data from container.
@@ -1539,16 +1428,15 @@ pgpDigParams pgpGetSignature(const pgpDig dig)
  * @param dig		signature parameters container
  * @return		signature tag
  */
-uint32_t pgpGetSigtag(const pgpDig dig)
+rpmuint32_t pgpGetSigtag(const pgpDig dig)
 	/*@*/;
 
 /** \ingroup rpmpgp
  * Get signature tag type.
  * @param dig		signature parameters container
- * @param ts		transaction set
  * @return		signature tag type
  */
-uint32_t pgpGetSigtype(const pgpDig dig)
+rpmuint32_t pgpGetSigtype(const pgpDig dig)
 	/*@*/;
 
 /** \ingroup rpmpgp
@@ -1565,7 +1453,7 @@ extern const void * pgpGetSig(const pgpDig dig)
  * @param dig		signature parameters container
  * @return		signature tag data length
  */
-uint32_t pgpGetSiglen(const pgpDig dig)
+rpmuint32_t pgpGetSiglen(const pgpDig dig)
 	/*@*/;
 
 /** \ingroup rpmpgp
@@ -1578,8 +1466,8 @@ uint32_t pgpGetSiglen(const pgpDig dig)
  * @return		0 always
  */
 int pgpSetSig(pgpDig dig,
-		uint32_t sigtag, uint32_t sigtype,
-		/*@kept@*/ /*@null@*/ const void * sig, uint32_t siglen)
+		rpmuint32_t sigtag, rpmuint32_t sigtype,
+		/*@kept@*/ /*@null@*/ const void * sig, rpmuint32_t siglen)
 	/*@modifies dig @*/;
 
 /** \ingroup rpmpgp
@@ -1588,6 +1476,7 @@ int pgpSetSig(pgpDig dig,
  * @param opx		per-container accumulator index (aka rpmtsOpX)
  * @return		per-container accumulator pointer
  */
+/*@null@*/
 void * pgpStatsAccumulator(pgpDig dig, int opx)
 	/*@*/;
 
@@ -1595,7 +1484,7 @@ void * pgpStatsAccumulator(pgpDig dig, int opx)
  * Set find pubkey vector.
  * @param dig		signature parameters container
  * @param findPubkey	routine to find a pubkey.
- * @param ts		argument to (*findPubkey) (ts)
+ * @param _ts		argument to (*findPubkey) (ts, ...)
  * @return		0 always
  */
 int pgpSetFindPubkey(pgpDig dig,
@@ -1614,10 +1503,11 @@ int pgpFindPubkey(pgpDig dig)
 /** \ingroup rpmpgp
  * Is buffer at beginning of an OpenPGP packet?
  * @param p		buffer
+ * @retval *tagp	OpenPGP tag
  * @return		1 if an OpenPGP packet, 0 otherwise
  */
 /*@unused@*/ static inline
-int pgpIsPkt(const uint8_t * p, /*@null@*/ pgpTag * tagp)
+int pgpIsPkt(const rpmuint8_t * p, /*@null@*/ pgpTag * tagp)
 	/*@modifies *tagp @*/
 {
     unsigned int val = (unsigned int) *p++;
@@ -1663,7 +1553,7 @@ int pgpIsPkt(const uint8_t * p, /*@null@*/ pgpTag * tagp)
 	rc = 0;
 	break;
     }
-    if (tagp)
+    if (tagp != NULL)
 	*tagp = tag;
     return rc;
 }
@@ -1678,7 +1568,7 @@ int pgpIsPkt(const uint8_t * p, /*@null@*/ pgpTag * tagp)
  * @return		crc of buffer
  */
 /*@unused@*/ static inline
-unsigned int pgpCRC(const uint8_t * octets, size_t len)
+unsigned int pgpCRC(const rpmuint8_t * octets, size_t len)
 	/*@*/
 {
     unsigned int crc = CRC24_INIT;
@@ -1695,52 +1585,6 @@ unsigned int pgpCRC(const uint8_t * octets, size_t len)
     return crc & 0xffffff;
 }
 
-/** \ingroup rpmpgp
- * Duplicate a digest context.
- * @param octx		existing digest context
- * @return		duplicated digest context
- */
-/*@only@*/
-DIGEST_CTX rpmDigestDup(DIGEST_CTX octx)
-	/*@*/;
-
-/** \ingroup rpmpgp
- * Initialize digest.
- * Set bit count to 0 and buffer to mysterious initialization constants.
- * @param hashalgo	type of digest
- * @param flags		bit(s) to control digest operation
- * @return		digest context
- */
-/*@only@*/ /*@null@*/
-DIGEST_CTX rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
-	/*@*/;
-
-/** \ingroup rpmpgp
- * Update context with next plain text buffer.
- * @param ctx		digest context
- * @param data		next data buffer
- * @param len		no. bytes of data
- * @return		0 on success
- */
-int rpmDigestUpdate(/*@null@*/ DIGEST_CTX ctx, const void * data, size_t len)
-	/*@modifies ctx @*/;
-
-/** \ingroup rpmpgp
- * Return digest and destroy context.
- * Final wrapup - pad to 64-byte boundary with the bit pattern 
- * 1 0* (64-bit count of bits processed, MSB-first)
- *
- * @param ctx		digest context
- * @retval datap	address of returned digest
- * @retval lenp		address of digest length
- * @param asAscii	return digest as ascii string?
- * @return		0 on success
- */
-int rpmDigestFinal(/*@only@*/ /*@null@*/ DIGEST_CTX ctx,
-	/*@null@*/ /*@out@*/ void * datap,
-	/*@null@*/ /*@out@*/ size_t * lenp, int asAscii)
-		/*@modifies *datap, *lenp @*/;
-
 /**
  */
 typedef int (*pgpImplSet_t) (/*@only@*/ DIGEST_CTX ctx, pgpDig dig, pgpDigParams sigp)
@@ -1754,7 +1598,7 @@ typedef int (*pgpImplVerify_t) (pgpDig dig)
 /**
  */
 typedef int (*pgpImplMpiItem_t) (const char * pre, pgpDig dig, int itemno,
-		const uint8_t * p, /*@null@*/ const uint8_t * pend)
+		const rpmuint8_t * p, /*@null@*/ const rpmuint8_t * pend)
 	/*@globals fileSystem @*/
 	/*@modifies dig, fileSystem @*/;
 
@@ -1781,6 +1625,8 @@ typedef struct pgpImplVecs_s {
     pgpImplVerify_t	_pgpVerifyRSA;
     pgpImplSet_t	_pgpSetDSA;
     pgpImplVerify_t	_pgpVerifyDSA;
+    pgpImplSet_t	_pgpSetECDSA;
+    pgpImplVerify_t	_pgpVerifyECDSA;
     pgpImplMpiItem_t	_pgpMpiItem;
     pgpImplClean_t	_pgpClean;
     pgpImplFree_t	_pgpFree;
@@ -1829,11 +1675,29 @@ int pgpImplVerifyDSA(pgpDig dig)
     return (*pgpImplVecs->_pgpVerifyDSA) (dig);
 }
 
+/**
+ */
+/*@unused@*/ static inline
+int pgpImplSetECDSA(/*@only@*/ DIGEST_CTX ctx, pgpDig dig, pgpDigParams sigp)
+	/*@modifies ctx, dig @*/
+{
+    return (*pgpImplVecs->_pgpSetECDSA) (ctx, dig, sigp);
+}
+
+/**
+ */
+/*@unused@*/ static inline
+int pgpImplVerifyECDSA(pgpDig dig)
+	/*@*/
+{
+    return (*pgpImplVecs->_pgpVerifyECDSA) (dig);
+}
+
 /**
  */
 /*@unused@*/ static inline
 int pgpImplMpiItem(const char * pre, pgpDig dig, int itemno,
-		const uint8_t * p, /*@null@*/ const uint8_t * pend)
+		const rpmuint8_t * p, /*@null@*/ const rpmuint8_t * pend)
 	/*@modifies dig @*/
 {
     return (*pgpImplVecs->_pgpMpiItem) (pre, dig, itemno, p, pend);
