@@ -4,13 +4,16 @@
 
 #include "system.h"
 
-#include "rpmio_internal.h"
-#include "rpmmacro.h"
+#include <rpmio_internal.h>
+#include <rpmmacro.h>
 
+#define	_RPMAV_INTERNAL
 #define	_RPMDAV_INTERNAL
-#include "rpmdav.h"
+#include <rpmdav.h>
+
+#include <rpmhash.h>
+#include <ugid.h>
 
-#include "ugid.h"
 #include "debug.h"
 
 /*@access DIR @*/
@@ -71,6 +74,8 @@ int Mkdir (const char * path, mode_t mode)
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
+if (_rpmio_debug)
+fprintf(stderr, "*** Mkdir(%s, 0%o)\n", path, (unsigned)mode);
     switch (ut) {
     case URL_IS_FTP:
 	return ftpMkdir(path, mode);
@@ -100,6 +105,8 @@ int Chdir (const char * path)
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
+if (_rpmio_debug)
+fprintf(stderr, "*** Chdir(%s)\n", path);
     switch (ut) {
     case URL_IS_FTP:
 	return ftpChdir(path);
@@ -133,6 +140,8 @@ int Rmdir (const char * path)
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
+if (_rpmio_debug)
+fprintf(stderr, "*** Rmdir(%s)\n", path);
     switch (ut) {
     case URL_IS_FTP:
 	return ftpRmdir(path);
@@ -160,7 +169,6 @@ int Rmdir (const char * path)
 /*@unchecked@*/
 const char * _chroot_prefix = NULL;
 
-/*@-mods@*/	/* XXX hide rpmGlobalMacroContext mods for now. */
 int Chroot(const char * path)
 {
     const char * lpath;
@@ -188,8 +196,10 @@ fprintf(stderr, "*** Chroot(%s)\n", path);
 /*@-dependenttrans -modobserver -observertrans @*/
     _chroot_prefix = _free(_chroot_prefix);
 /*@=dependenttrans =modobserver =observertrans @*/
+/*@-globs -mods@*/	/* XXX hide rpmGlobalMacroContext mods for now. */
     if (strcmp(path, "."))
 	_chroot_prefix = rpmGetPath(path, NULL);
+/*@=globs =mods@*/
 
 /*@-superuser@*/
     return chroot(path);
@@ -253,6 +263,8 @@ int Rename (const char * oldpath, const char * newpath)
     const char *ne = NULL;
     int oldut, newut;
 
+if (_rpmio_debug)
+fprintf(stderr, "*** Rename(%s, %s)\n", oldpath, newpath);
     /* XXX lib/install.c used to rely on this behavior. */
     if (!strcmp(oldpath, newpath)) return 0;
 
@@ -308,6 +320,8 @@ int Link (const char * oldpath, const char * newpath)
     const char *ne = NULL;
     int oldut, newut;
 
+if (_rpmio_debug)
+fprintf(stderr, "*** Link(%s, %s)\n", oldpath, newpath);
     oldut = urlPath(oldpath, &oe);
     switch (oldut) {
     case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
@@ -354,6 +368,8 @@ int Unlink(const char * path) {
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
+if (_rpmio_debug)
+fprintf(stderr, "*** Unlink(%s)\n", path);
     switch (ut) {
     case URL_IS_FTP:
 	return ftpUnlink(path);
@@ -652,9 +668,9 @@ static int vfs_parse_filedate(int idx, /*@out@*/ time_t *t)
 
 	/* Here just this special case with MM-DD-YY */
         if (is_dos_date(p)){
-	    /*@-mods@*/
+/*@-mods@*/
             p[2] = p[5] = '-';
-	    /*@=mods@*/
+/*@=mods@*/
 
 	    memset(d, 0, sizeof(d));
 	    if (sscanf(p, "%2d-%2d-%2d", &d[0], &d[1], &d[2]) == 3){
@@ -856,13 +872,8 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 	char *t;
 
 	if (filename){
-#ifdef HACK
-	    t = g_strndup (p_copy + column_ptr [idx], column_ptr [idx2] - column_ptr [idx] - 1);
-#else
-	    int nb = column_ptr [idx2] - column_ptr [idx] - 1;
-	    t = xmalloc(nb+1);
-	    strncpy(t, p_copy + column_ptr [idx], nb);
-#endif
+	    size_t nb = column_ptr [idx2] - column_ptr [idx] - 1;
+	    t = strncpy(xcalloc(1, nb+1), p_copy + column_ptr [idx], nb);
 	    *filename = t;
 	}
 	if (linkname){
@@ -931,13 +942,12 @@ typedef enum {
 
 /**
  */
-/*@unchecked@*/
-static size_t ftpBufAlloced = 0;
+static size_t ftpBufAlloced;
 
 /**
  */
-/*@unchecked@*/
-static /*@only@*/ char * ftpBuf = NULL;
+/*@only@*/ /*@relnull@*/
+static char * ftpBuf;
 	
 #define alloca_strdup(_s)       strcpy(alloca(strlen(_s)+1), (_s))
 
@@ -1075,7 +1085,7 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 		    /*@innerbreak@*/ break;
 	    }
 
-	    if (nbn != (ne - n))	/* Same name length? */
+	    if (nbn != (size_t)(ne - n))	/* Same name length? */
 		/*@innercontinue@*/ continue;
 	    if (strncmp(n, bn, nbn))	/* Same name? */
 		/*@innercontinue@*/ continue;
@@ -1136,33 +1146,41 @@ static const char * statstr(const struct stat * st,
 		/*@returned@*/ /*@out@*/ char * buf)
 	/*@modifies *buf @*/
 {
-    sprintf(buf,
-	"*** dev %x ino %x mode %0o nlink %d uid %d gid %d rdev %x size %x\n",
-	(unsigned int)st->st_dev,
-	(unsigned int)st->st_ino,
-	(unsigned int)st->st_mode,
-	(unsigned int)st->st_nlink,
-	(unsigned int)st->st_uid,
-	(unsigned int)st->st_gid,
-	(unsigned int)st->st_rdev,
-	(unsigned int)st->st_size);
+    char * t = buf;
+    sprintf(t, "*** dev %x", (unsigned int)st->st_dev);
+	t += strlen(t);
+    sprintf(t, " ino %x", (unsigned int)st->st_ino);
+	t += strlen(t);
+    sprintf(t, " mode %0o", (unsigned int)st->st_mode);
+	t += strlen(t);
+    sprintf(t, " nlink %d", (unsigned int)st->st_nlink);
+	t += strlen(t);
+    sprintf(t, " uid %d", (unsigned int)st->st_uid);
+	t += strlen(t);
+    sprintf(t, " gid %d", (unsigned int)st->st_gid);
+	t += strlen(t);
+    sprintf(t, " rdev %x", (unsigned int)st->st_rdev);
+	t += strlen(t);
+    sprintf(t, " size %x", (unsigned int)st->st_size);
+	t += strlen(t);
+    sprintf(t, "\n");
     return buf;
 }
 
-/*@unchecked@*/
-static int ftp_st_ino = 0xdead0000;
-
 /* FIXME: borked for path with trailing '/' */
 static int ftpStat(const char * path, /*@out@*/ struct stat *st)
-	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
-	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
+	/*@globals ftpBufAlloced, ftpBuf, h_errno, fileSystem, internalState @*/
+	/*@modifies ftpBufAlloced, ftpBuf, *st, fileSystem, internalState @*/
 {
     char buf[1024];
     int rc;
     rc = ftpNLST(path, DO_FTP_STAT, st, NULL, 0);
-    /* XXX fts(3) needs/uses st_ino, make something up for now. */
+
+    /* XXX fts(3) needs/uses st_ino. */
+    /* Hash the path to generate a st_ino analogue. */
     if (st->st_ino == 0)
-	st->st_ino = ftp_st_ino++;
+	st->st_ino = hashFunctionString(0, path, 0);
+
 if (_ftp_debug)
 fprintf(stderr, "*** ftpStat(%s) rc %d\n%s", path, rc, statstr(st, buf));
     return rc;
@@ -1170,23 +1188,26 @@ fprintf(stderr, "*** ftpStat(%s) rc %d\n%s", path, rc, statstr(st, buf));
 
 /* FIXME: borked for path with trailing '/' */
 static int ftpLstat(const char * path, /*@out@*/ struct stat *st)
-	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
-	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
+	/*@globals ftpBufAlloced, ftpBuf, h_errno, fileSystem, internalState @*/
+	/*@modifies ftpBufAlloced, ftpBuf, *st, fileSystem, internalState @*/
 {
     char buf[1024];
     int rc;
     rc = ftpNLST(path, DO_FTP_LSTAT, st, NULL, 0);
-    /* XXX fts(3) needs/uses st_ino, make something up for now. */
+
+    /* XXX fts(3) needs/uses st_ino. */
+    /* Hash the path to generate a st_ino analogue. */
     if (st->st_ino == 0)
-	st->st_ino = ftp_st_ino++;
+	st->st_ino = hashFunctionString(0, path, 0);
+
 if (_ftp_debug)
 fprintf(stderr, "*** ftpLstat(%s) rc %d\n%s\n", path, rc, statstr(st, buf));
     return rc;
 }
 
 static int ftpReadlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
-	/*@globals h_errno, fileSystem, internalState @*/
-	/*@modifies *buf, fileSystem, internalState @*/
+	/*@globals ftpBufAlloced, ftpBuf, h_errno, fileSystem, internalState @*/
+	/*@modifies ftpBufAlloced, ftpBuf, *buf, fileSystem, internalState @*/
 {
     int rc;
     rc = ftpNLST(path, DO_FTP_READLINK, NULL, buf, bufsiz);
@@ -1197,36 +1218,37 @@ fprintf(stderr, "*** ftpReadlink(%s) rc %d\n", path, rc);
 
 /*@null@*/
 static DIR * ftpOpendir(const char * path)
-	/*@globals h_errno, fileSystem, internalState @*/
-	/*@modifies fileSystem, internalState @*/
+	/*@globals ftpBufAlloced, ftpBuf, h_errno, errno,
+		fileSystem, internalState @*/
+	/*@modifies ftpBufAlloced, ftpBuf, errno,
+		fileSystem, internalState @*/
 {
     AVDIR avdir;
-    struct dirent * dp;
-    size_t nb;
+    avContext ctx;
+    struct stat * st = NULL;
     const char * s, * sb, * se;
-    const char ** av;
-    unsigned char * dt;
-    char * t;
-    int ac;
+    int nac;
     int c;
     int rc;
 
 if (_ftp_debug)
 fprintf(stderr, "*** ftpOpendir(%s)\n", path);
+
+    /* Load FTP collection into argv. */
+    ctx = avContextCreate(path, st);
+    if (ctx == NULL) {
+        errno = ENOENT;         /* Note: ctx is NULL iff urlSplit() fails. */
+        return NULL;
+    }
+
     rc = ftpNLST(path, DO_FTP_GLOB, NULL, NULL, 0);
     if (rc)
 	return NULL;
 
-    /*
-     * ftpBuf now contains absolute paths, newline terminated.
-     * Calculate the number of bytes to hold the directory info.
-     */
-    nb = sizeof(".") + sizeof("..");
-    ac = 2;
+    nac = 0;
     sb = NULL;
     s = se = ftpBuf;
-    while ((c = (int) *se) != (int) '\0') {
-	se++;
+    while ((c = (int) *se++) != (int) '\0') {
 	switch (c) {
 	case '/':
 	    sb = se;
@@ -1236,8 +1258,7 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 		for (sb = se; sb > s && sb[-1] != ' '; sb--)
 		    {};
 	    }
-	    ac++;
-	    nb += (se - sb);
+	    nac++;
 
 	    if (*se == '\n') se++;
 	    sb = NULL;
@@ -1248,35 +1269,10 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 	}
     }
 
-    nb += sizeof(*avdir) + sizeof(*dp) + ((ac + 1) * sizeof(*av)) + (ac + 1);
-    avdir = xcalloc(1, nb);
-    /*@-abstract@*/
-    dp = (struct dirent *) (avdir + 1);
-    av = (const char **) (dp + 1);
-    dt = (unsigned char *) (av + (ac + 1));
-    t = (char *) (dt + ac + 1);
-    /*@=abstract@*/
+    ctx->av = xcalloc(nac+1, sizeof(*ctx->av));
+    ctx->modes = xcalloc(nac, sizeof(*ctx->modes));
 
-    avdir->fd = avmagicdir;
-/*@-usereleased@*/
-    avdir->data = (char *) dp;
-/*@=usereleased@*/
-    avdir->allocation = nb;
-    avdir->size = ac;
-    avdir->offset = -1;
-    avdir->filepos = 0;
-
-#if defined(HAVE_PTHREAD_H)
-/*@-moduncon -noeffectuncon@*/
-    (void) pthread_mutex_init(&avdir->lock, NULL);
-/*@=moduncon =noeffectuncon@*/
-#endif
-
-    ac = 0;
-    /*@-dependenttrans -unrecog@*/
-    dt[ac] = (unsigned char)DT_DIR; av[ac++] = t; t = stpcpy(t, ".");	t++;
-    dt[ac] = (unsigned char)DT_DIR; av[ac++] = t; t = stpcpy(t, "..");	t++;
-    /*@=dependenttrans =unrecog@*/
+    nac = 0;
     sb = NULL;
     s = se = ftpBuf;
     while ((c = (int) *se) != (int) '\0') {
@@ -1286,44 +1282,24 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 	    sb = se;
 	    /*@switchbreak@*/ break;
 	case '\r':
-	    /*@-dependenttrans@*/
-	    av[ac] = t;
-	    /*@=dependenttrans@*/
 	    if (sb == NULL) {
+		ctx->modes[nac] = (*s == 'd' ? 0755 : 0644);
 		/*@-unrecog@*/
 		switch(*s) {
-		case 'p':
-		    dt[ac] = (unsigned char) DT_FIFO;
-		    /*@innerbreak@*/ break;
-		case 'c':
-		    dt[ac] = (unsigned char) DT_CHR;
-		    /*@innerbreak@*/ break;
-		case 'd':
-		    dt[ac] = (unsigned char) DT_DIR;
-		    /*@innerbreak@*/ break;
-		case 'b':
-		    dt[ac] = (unsigned char) DT_BLK;
-		    /*@innerbreak@*/ break;
-		case '-':
-		    dt[ac] = (unsigned char) DT_REG;
-		    /*@innerbreak@*/ break;
-		case 'l':
-		    dt[ac] = (unsigned char) DT_LNK;
-		    /*@innerbreak@*/ break;
-		case 's':
-		    dt[ac] = (unsigned char) DT_SOCK;
-		    /*@innerbreak@*/ break;
-		default:
-		    dt[ac] = (unsigned char) DT_UNKNOWN;
-		    /*@innerbreak@*/ break;
+		case 'p': ctx->modes[nac] |= S_IFIFO; /*@innerbreak@*/ break;
+		case 'c': ctx->modes[nac] |= S_IFCHR; /*@innerbreak@*/ break;
+		case 'd': ctx->modes[nac] |= S_IFDIR; /*@innerbreak@*/ break;
+		case 'b': ctx->modes[nac] |= S_IFBLK; /*@innerbreak@*/ break;
+		case '-': ctx->modes[nac] |= S_IFREG; /*@innerbreak@*/ break;
+		case 'l': ctx->modes[nac] |= S_IFLNK; /*@innerbreak@*/ break;
+		case 's': ctx->modes[nac] |= S_IFSOCK; /*@innerbreak@*/ break;
+		default:  ctx->modes[nac] |= S_IFREG; /*@innerbreak@*/ break;
 		}
 		/*@=unrecog@*/
 		for (sb = se; sb > s && sb[-1] != ' '; sb--)
 		    {};
 	    }
-	    ac++;
-	    t = stpncpy(t, sb, (se - sb));
-	    t[-1] = '\0';
+	    ctx->av[nac++] = strncpy(xcalloc(1, (se-sb-1)+1), sb, (se-sb-1));
 	    if (*se == '\n') se++;
 	    sb = NULL;
 	    s = se;
@@ -1332,14 +1308,27 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 	    /*@switchbreak@*/ break;
 	}
     }
-    av[ac] = NULL;
+
+    avdir = (AVDIR) avOpendir(path, ctx->av, ctx->modes);
+
+    ctx = avContextDestroy(ctx);
 
 /*@-kepttrans@*/
     return (DIR *) avdir;
 /*@=kepttrans@*/
 }
 
+static char * ftpRealpath(const char * path, /*@null@*/ char * resolved_path)
+	/*@*/
+{
+assert(resolved_path == NULL);	/* XXX no POSIXly broken realpath(3) here. */
+    /* XXX TODO: handle redirects. For now, just dupe the path. */
+    return xstrdup(path);
+}
+
 int Stat(const char * path, struct stat * st)
+	/*@globals ftpBufAlloced, ftpBuf @*/
+	/*@modifies ftpBufAlloced, ftpBuf @*/
 {
     const char * lpath;
     int ut = urlPath(path, &lpath);
@@ -1364,7 +1353,7 @@ fprintf(stderr, "*** Stat(%s,%p)\n", path, st);
     case URL_IS_DASH:
     case URL_IS_HKP:
     default:
-	errno = EINVAL;		/* XXX W2DO? */
+	errno = ENOENT;	
 	return -2;
 	/*@notreached@*/ break;
     }
@@ -1372,6 +1361,8 @@ fprintf(stderr, "*** Stat(%s,%p)\n", path, st);
 }
 
 int Lstat(const char * path, struct stat * st)
+	/*@globals ftpBufAlloced, ftpBuf @*/
+	/*@modifies ftpBufAlloced, ftpBuf @*/
 {
     const char * lpath;
     int ut = urlPath(path, &lpath);
@@ -1396,20 +1387,70 @@ fprintf(stderr, "*** Lstat(%s,%p)\n", path, st);
     case URL_IS_DASH:
     case URL_IS_HKP:
     default:
-	errno = EINVAL;		/* XXX W2DO? */
+	errno = ENOENT;	
 	return -2;
 	/*@notreached@*/ break;
     }
     return lstat(path, st);
 }
 
+int Fstat(FD_t fd, struct stat * st)
+{
+    const char * path = fdGetOPath(fd);
+    const char * lpath;
+    int ut = urlPath(path, &lpath);
+
+if (_rpmio_debug)
+fprintf(stderr, "*** Fstat(%p,%p) path %s\n", fd, st, path);
+    if (fd == NULL || path == NULL || *path == '\0' || st == NULL) {
+	errno = ENOENT;
+	return -2;
+    }
+
+    switch (ut) {
+    case URL_IS_DASH:
+    case URL_IS_PATH:
+    case URL_IS_UNKNOWN:
+	break;
+    case URL_IS_FTP:
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+    case URL_IS_HKP:
+	if (fd->contentLength < 0) {
+	   errno = ENOENT;
+	   return -2;
+	}
+	memset(st, 0, sizeof(*st));
+	if (path[strlen(path)-1] == '/') {
+	    st->st_nlink = 2;
+	    st->st_mode = (S_IFDIR | 0755);
+	} else {
+	    st->st_nlink = 1;
+	    st->st_mode = (S_IFREG | 0644);
+	}
+	st->st_ino = hashFunctionString(0, path, 0);;
+	st->st_size = fd->contentLength;
+	st->st_mtime = fd->lastModified;
+
+	st->st_atime = st->st_ctime = st->st_mtime;
+	st->st_blksize = 4 * 1024;  /* HACK correct for linux ext */
+	st->st_blocks = (st->st_size + 511)/512;
+	break;
+    default:
+	errno = ENOENT;	
+	return -2;
+	/*@notreached@*/ break;
+    }
+    return fstat(Fileno(fd), st);
+}
+
 int Chown(const char * path, uid_t owner, gid_t group)
 {
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
 if (_rpmio_debug)
-fprintf(stderr, "*** Chown(%s,%d,%d)\n", path, (int)owner, (int)group);
+fprintf(stderr, "*** Chown(%s,%u,%u)\n", path, (unsigned)owner, (unsigned)group);
     switch (ut) {
     case URL_IS_PATH:
 	path = lpath;
@@ -1429,13 +1470,40 @@ fprintf(stderr, "*** Chown(%s,%d,%d)\n", path, (int)owner, (int)group);
     return chown(path, owner, group);
 }
 
+int Fchown(FD_t fd, uid_t owner, gid_t group)
+{
+    const char * path = fdGetOPath(fd);
+    const char * lpath;
+    int ut = urlPath(path, &lpath);
+
+if (_rpmio_debug)
+fprintf(stderr, "*** Fchown(%p,%u,%u) path %s\n", fd, (unsigned)owner, (unsigned)group, path);
+    switch (ut) {
+    case URL_IS_PATH:
+	path = lpath;
+	/*@fallthrough@*/
+    case URL_IS_UNKNOWN:
+	break;
+    case URL_IS_DASH:
+    case URL_IS_HKP:
+    case URL_IS_FTP:		/* XXX TODO: implement. */
+    case URL_IS_HTTPS:		/* XXX TODO: implement. */
+    case URL_IS_HTTP:		/* XXX TODO: implement. */
+    default:
+	errno = EINVAL;		/* XXX W2DO? */
+	return -2;
+	/*@notreached@*/ break;
+    }
+    return fchown(Fileno(fd), owner, group);
+}
+
 int Lchown(const char * path, uid_t owner, gid_t group)
 {
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
 if (_rpmio_debug)
-fprintf(stderr, "*** Lchown(%s,%d,%d)\n", path, (int)owner, (int)group);
+fprintf(stderr, "*** Lchown(%s,%u,%u)\n", path, (unsigned)owner, (unsigned)group);
     switch (ut) {
     case URL_IS_PATH:
 	path = lpath;
@@ -1481,6 +1549,33 @@ fprintf(stderr, "*** Chmod(%s,%0o)\n", path, (int)mode);
     return chmod(path, mode);
 }
 
+int Fchmod(FD_t fd, mode_t mode)
+{
+    const char * path = fdGetOPath(fd);
+    const char * lpath;
+    int ut = urlPath(path, &lpath);
+
+if (_rpmio_debug)
+fprintf(stderr, "*** Fchmod(%p,%0o) path %s\n", fd, (int)mode, path);
+    switch (ut) {
+    case URL_IS_PATH:
+	path = lpath;
+	/*@fallthrough@*/
+    case URL_IS_UNKNOWN:
+	break;
+    case URL_IS_DASH:
+    case URL_IS_HKP:
+    case URL_IS_FTP:		/* XXX TODO: implement. */
+    case URL_IS_HTTPS:		/* XXX TODO: implement. */
+    case URL_IS_HTTP:		/* XXX TODO: implement. */
+    default:
+	errno = EINVAL;		/* XXX W2DO? */
+	return -2;
+	/*@notreached@*/ break;
+    }
+    return fchmod(Fileno(fd), mode);
+}
+
 int Mkfifo(const char * path, mode_t mode)
 {
     const char * lpath;
@@ -1620,10 +1715,14 @@ fprintf(stderr, "*** Symlink(%s,%s)\n", oldpath, newpath);
 }
 
 int Readlink(const char * path, char * buf, size_t bufsiz)
+	/*@globals ftpBufAlloced, ftpBuf @*/
+	/*@modifies ftpBufAlloced, ftpBuf @*/
 {
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
+if (_rpmio_debug)
+fprintf(stderr, "*** Readlink(%s,%p[%u])\n", path, buf, (unsigned)bufsiz);
     switch (ut) {
     case URL_IS_FTP:
 	return ftpReadlink(path, buf, bufsiz);
@@ -1724,14 +1823,16 @@ int Glob_pattern_p (const char * pattern, int quote)
     return (0);
 }
 
-int Glob_error(/*@unused@*/const char * epath, /*@unused@*/ int eerrno)
+int Glob_error(/*@unused@*/ const char * epath,
+		/*@unused@*/ int eerrno)
 {
     return 1;
 }
 
 int Glob(const char *pattern, int flags,
-	int errfunc(const char * epath, int eerrno), glob_t *pglob)
+	int errfunc(const char * epath, int eerrno), void *_pglob)
 {
+    glob_t *pglob = _pglob;
     const char * lpath;
     int ut = urlPath(pattern, &lpath);
     const char *home = getenv("HOME");
@@ -1772,14 +1873,17 @@ fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)e
     return glob(pattern, flags, errfunc, pglob);
 }
 
-void Globfree(glob_t *pglob)
+void Globfree(void *_pglob)
 {
+    glob_t *pglob = _pglob;
 if (_rpmio_debug)
 fprintf(stderr, "*** Globfree(%p)\n", pglob);
     globfree(pglob);
 }
 
 DIR * Opendir(const char * path)
+	/*@globals ftpBufAlloced, ftpBuf @*/
+	/*@modifies ftpBufAlloced, ftpBuf @*/
 {
     const char * lpath;
     int ut = urlPath(path, &lpath);
@@ -1820,10 +1924,6 @@ fprintf(stderr, "*** Readdir(%p)\n", (void *)dir);
 	return NULL;
     if (ISAVMAGIC(dir))
 	return avReaddir(dir);
-#ifdef WITH_NEON
-    if (ISDAVMAGIC(dir))
-	return davReaddir(dir);
-#endif
     return readdir(dir);
 }
 
@@ -1835,13 +1935,95 @@ fprintf(stderr, "*** Closedir(%p)\n", (void *)dir);
 	return 0;
     if (ISAVMAGIC(dir))
 	return avClosedir(dir);
-#ifdef WITH_NEON
-    if (ISDAVMAGIC(dir))
-	return davClosedir(dir);
-#endif
     return closedir(dir);
 }
 
+char * Realpath(const char * path, /*@null@*/ char * resolved_path)
+{
+    const char * lpath;
+    int ut = urlPath(path, &lpath);
+    char * rpath = NULL;
+
+if (_rpmio_debug)
+fprintf(stderr, "*** Realpath(%s, %s)\n", path, (resolved_path ? resolved_path : "NULL"));
+#if !defined(__LCLINT__) /* XXX LCL: realpath(3) annotations are buggy. */
+/*@-nullpass@*/
+    /* XXX if POSIXly broken realpath(3) is desired, do that. */
+    /* XXX note: preserves current rpmlib realpath(3) usage cases. */
+    if (path == NULL || resolved_path != NULL)
+	return realpath(path, resolved_path);
+/*@=nullpass@*/
+#endif	/* !__LCLINT__ */
+
+    switch (ut) {
+    case URL_IS_FTP:
+	return ftpRealpath(path, resolved_path);
+	/*@notreached@*/ break;
+    case URL_IS_HTTPS:	
+    case URL_IS_HTTP:
+    case URL_IS_HKP:
+#ifdef WITH_NEON
+	return davRealpath(path, resolved_path);
+	/*@notreached@*/ break;
+#endif
+	/*@fallthrough@*/
+    default:
+	return xstrdup(path);
+	/*@notreached@*/ break;
+    case URL_IS_DASH:
+	/* non-GLIBC systems => EINVAL. non-linux systems => EINVAL */
+#if defined(__linux__)
+	lpath = "/dev/stdin";
+#else
+	lpath = NULL;
+#endif
+	break;
+    case URL_IS_PATH:		/* XXX note: file:/// prefix is dropped. */
+    case URL_IS_UNKNOWN:
+	path = lpath;
+	break;
+    }
+
+#if !defined(__LCLINT__) /* XXX LCL: realpath(3) annotations are buggy. */
+    if (lpath == NULL || *lpath == '/')
+/*@-nullpass@*/	/* XXX glibc extension */
+	rpath = realpath(lpath, resolved_path);
+/*@=nullpass@*/
+    else {
+	char * t;
+#if defined(__GLIBC__)
+	char * dn = NULL;
+#else
+	char dn[PATH_MAX];
+	dn[0] = '\0';
+#endif
+	/*
+	 * Using realpath on lpath isn't correct if the lpath is a symlink,
+	 * especially if the symlink is a dangling link.  What we 
+	 * do instead is use realpath() on `.' and then append lpath to
+	 * the result.
+	 */
+	if ((t = realpath(".", dn)) != NULL) {
+/*@-globs -mods@*/	/* XXX no rpmGlobalMacroContext mods please. */
+	    rpath = (char *) rpmGetPath(t, "/", lpath, NULL);
+	    /* XXX preserve the pesky trailing '/' */
+	    if (lpath[strlen(lpath)-1] == '/') {
+		char * s = rpath;
+		rpath = rpmExpand(s, "/", NULL);
+		s = _free(s);
+	    }
+/*@=globs =mods@*/
+	} else
+	    rpath = NULL;
+#if defined(__GLIBC__)
+	t = _free(t);
+#endif
+    }
+#endif	/* !__LCLINT__ */
+
+    return rpath;
+}
+
 off_t Lseek(int fdno, off_t offset, int whence)
 {
 if (_rpmio_debug)
