@@ -119,7 +119,7 @@ extern int pthread_cond_signal(pthread_cond_t *cond)
 
 #include <signal.h>
 #if !defined(__QNX__)
-#include <sys/signal.h>
+#  include <sys/signal.h>
 #endif
 #include <sys/wait.h>
 #include <search.h>
@@ -190,13 +190,13 @@ static	void __remque(struct qelem * elem)
 #define	remque(_e)	__remque(_e)
 #endif
 
-#if defined(HAVE_PTHREAD_H)
+#if defined(WITH_PTHREADS)
 
 # if !defined(__QNX__)
 /* XXX suggested in bugzilla #159024 */
-#if PTHREAD_MUTEX_DEFAULT != PTHREAD_MUTEX_NORMAL
-  #error RPM expects PTHREAD_MUTEX_DEFAULT == PTHREAD_MUTEX_NORMAL
-#endif
+#  if PTHREAD_MUTEX_DEFAULT != PTHREAD_MUTEX_NORMAL
+#    error RPM expects PTHREAD_MUTEX_DEFAULT == PTHREAD_MUTEX_NORMAL
+#  endif
 # endif
 
 #ifndef PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
@@ -209,7 +209,8 @@ static pthread_mutex_t rpmsigTbl_lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
 /*@=type@*/
 #endif
 
-#define	DO_LOCK()	pthread_mutex_lock(&rpmsigTbl_lock);
+/*@-macromatchname@*/
+#define	DO_LOCK()	/*@-retvalint@*/pthread_mutex_lock(&rpmsigTbl_lock)/*@=retvalint@*/;
 #define	DO_UNLOCK()	pthread_mutex_unlock(&rpmsigTbl_lock);
 #define	INIT_LOCK()	\
     {	pthread_mutexattr_t attr; \
@@ -229,6 +230,7 @@ static pthread_mutex_t rpmsigTbl_lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
     (void) pthread_setcanceltype ((__oldtype), &(__oldtype));
 
 #define	SAME_THREAD(_a, _b)	pthread_equal(((pthread_t)_a), ((pthread_t)_b))
+/*@-macromatchname@*/
 
 #define ME() __tid2vp(pthread_self())
 /*@shared@*/
@@ -242,15 +244,17 @@ static void *__tid2vp(pthread_t tid)
 
 #else
 
+/*@-macromatchname@*/
 #define	DO_LOCK() (0)
 #define	DO_UNLOCK() (0)
-#define	INIT_LOCK() ((void)0)
+#define	INIT_LOCK()
 #define	ADD_REF(__tbl)	/*@-noeffect@*/ (0) /*@=noeffect@*/
 #define	SUB_REF(__tbl)	/*@-noeffect@*/ (0) /*@=noeffect@*/
-#define	CLEANUP_HANDLER(__handler, __arg, __oldtypeptr) ((void)0)
-#define	CLEANUP_RESET(__execute, __oldtype) ((void)0)
+#define	CLEANUP_HANDLER(__handler, __arg, __oldtypeptr)
+#define	CLEANUP_RESET(__execute, __oldtype)
 
 #define	SAME_THREAD(_a, _b)	(42)
+/*@=macromatchname@*/
 
 #define ME() __pid2vp(getpid())
 /*@shared@*/
@@ -262,7 +266,7 @@ static void *__pid2vp(pid_t pid)
     return u.vp;
 }
 
-#endif	/* HAVE_PTHREAD_H */
+#endif	/* WITH_PTHREADS */
 
 #define	_RPMSQ_INTERNAL
 #include <rpmsq.h>
@@ -273,8 +277,11 @@ static void *__pid2vp(pid_t pid)
 /*@unchecked@*/
 int _rpmsq_debug = _RPMSQ_DEBUG;
 
+/* XXX __OpenBSD__ insque(3) needs rock->q_forw initialized. */
 /*@unchecked@*/
-static struct rpmsqElem rpmsqRock;
+/*@-fullinitblock @*/
+static struct rpmsqElem rpmsqRock = { .q_forw = &rpmsqRock };
+/*@=fullinitblock @*/
 
 /*@-compmempass@*/
 /*@unchecked@*/
@@ -359,17 +366,29 @@ static struct rpmsig_s {
     { SIGCHLD,	rpmsqAction },
 #define	rpmsigTbl_sigchld	(&rpmsigTbl[2])
     { SIGHUP,	rpmsqAction },
-#define	rpmsigTbl_sighup	(&rpmsigTbl[3])
+#define	rpmsigTbl_sighup	(&rpmsigTbl[3])		/* XXX unused */
     { SIGTERM,	rpmsqAction },
-#define	rpmsigTbl_sigterm	(&rpmsigTbl[4])
+#define	rpmsigTbl_sigterm	(&rpmsigTbl[4])		/* XXX unused */
     { SIGPIPE,	rpmsqAction },
-#define	rpmsigTbl_sigpipe	(&rpmsigTbl[5])
+#define	rpmsigTbl_sigpipe	(&rpmsigTbl[5])		/* XXX unused */
+
+#ifdef	NOTYET		/* XXX todo++ */
+#if defined(SIGXCPU)
+    { SIGXCPU,	rpmsqAction },
+#define	rpmsigTbl_sigxcpu	(&rpmsigTbl[6])		/* XXX unused */
+#endif
+#if defined(SIGXFSZ)
+    { SIGXFSZ,	rpmsqAction },
+#define	rpmsigTbl_sigxfsz	(&rpmsigTbl[7])		/* XXX unused */
+#endif
+#endif
+
     { -1,	NULL },
 };
 /*@=fullinitblock@*/
 
-void rpmsqAction(int signum,
-		/*@unused@*/ void * info, /*@unused@*/ void * context)
+void rpmsqAction(int signum, /*@unused@*/ void * info,
+		/*@unused@*/ void * context)
 {
     int save = errno;
     rpmsig tbl;
@@ -425,8 +444,9 @@ int rpmsqEnable(int signum, /*@null@*/ rpmsqAction_t handler)
     struct sigaction sa;
     rpmsig tbl;
     int ret = -1;
+    int xx;
 
-    (void) DO_LOCK ();
+    xx = DO_LOCK ();
     if (rpmsqQueue->id == NULL)
 	rpmsqQueue->id = ME();
     for (tbl = rpmsigTbl; tbl->signum >= 0; tbl++) {
@@ -450,7 +470,7 @@ int rpmsqEnable(int signum, /*@null@*/ rpmsqAction_t handler)
 		sa.sa_sigaction = (void *) (handler != NULL ? handler : tbl->handler);
 #endif
 		if (sigaction(tbl->signum, &sa, &tbl->oact) < 0) {
-		    (void)SUB_REF(tbl);
+		    xx = SUB_REF(tbl);
 		    break;
 		}
 		tbl->active = 1;		/* XXX just in case */
@@ -468,7 +488,7 @@ int rpmsqEnable(int signum, /*@null@*/ rpmsqAction_t handler)
 	ret = tbl->active;
 	break;
     }
-    (void) DO_UNLOCK ();
+    xx = DO_UNLOCK ();
     return ret;
 }
 
@@ -635,36 +655,40 @@ fprintf(stderr, "      Fini(%p): %p child %d status 0x%x\n", ME(), sq, (int)sq->
 
 void * rpmsqThread(void * (*start) (void * arg), void * arg)
 {
-#if defined(HAVE_PTHREAD_H)
+#if defined(WITH_PTHREADS)
     pthread_t pth;
     int ret;
 
     ret = pthread_create(&pth, NULL, start, arg);
     return (ret == 0 ? (void *)pth : NULL);
 #else
+    (void) start;
+    (void) arg; 
     return NULL;
 #endif
 }
 
 int rpmsqJoin(void * thread)
 {
-#if defined(HAVE_PTHREAD_H)
+#if defined(WITH_PTHREADS)
     pthread_t pth = (pthread_t) thread;
     if (thread == NULL)
 	return EINVAL;
     return pthread_join(pth, NULL);
 #else
+    (void) thread;
     return EINVAL;
 #endif
 }
 
 int rpmsqThreadEqual(void * thread)
 {
-#if defined(HAVE_PTHREAD_H)
+#if defined(WITH_PTHREADS)
     pthread_t t1 = (pthread_t) thread;
     pthread_t t2 = pthread_self();
     return pthread_equal(t1, t2);
 #else
+    (void) thread;
     return 0;
 #endif
 }
@@ -672,7 +696,7 @@ int rpmsqThreadEqual(void * thread)
 /**
  * SIGCHLD cancellation handler.
  */
-#if defined(HAVE_PTHREAD_H)
+#if defined(WITH_PTHREADS)
 static void
 sigchld_cancel (void *arg)
 	/*@globals rpmsigTbl, fileSystem, internalState @*/
@@ -680,19 +704,20 @@ sigchld_cancel (void *arg)
 {
     pid_t child = *(pid_t *) arg;
     pid_t result;
+    int xx;
 
-    (void) kill(child, SIGKILL);
+    xx = kill(child, SIGKILL);
 
     do {
 	result = waitpid(child, NULL, 0);
     } while (result == (pid_t)-1 && errno == EINTR);
 
-    (void) DO_LOCK ();
+    xx = DO_LOCK ();
     if (SUB_REF (rpmsigTbl_sigchld) == 0) {
-	(void) rpmsqEnable(-SIGQUIT, NULL);
-	(void) rpmsqEnable(-SIGINT, NULL);
+	xx = rpmsqEnable(-SIGQUIT, NULL);
+	xx = rpmsqEnable(-SIGINT, NULL);
     }
-    (void) DO_UNLOCK ();
+    xx = DO_UNLOCK ();
 }
 #endif
 
@@ -704,7 +729,7 @@ rpmsqExecve (const char ** argv)
 	/*@globals rpmsigTbl @*/
 	/*@modifies rpmsigTbl @*/
 {
-#if defined(HAVE_PTHREAD_H)
+#if defined(WITH_PTHREADS)
     int oldtype;
 #endif
     int status = -1;
@@ -712,28 +737,29 @@ rpmsqExecve (const char ** argv)
     pid_t result;
     sigset_t newMask, oldMask;
     rpmsq sq = memset(alloca(sizeof(*sq)), 0, sizeof(*sq));
+    int xx;
 
 #ifndef PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
 	INIT_LOCK ();
 #endif
 
-    (void) DO_LOCK ();
+    xx = DO_LOCK ();
     if (ADD_REF (rpmsigTbl_sigchld) == 0) {
 	if (rpmsqEnable(SIGINT, NULL) < 0) {
-	    (void) SUB_REF (rpmsigTbl_sigchld);
+	    xx = SUB_REF (rpmsigTbl_sigchld);
 	    goto out;
 	}
 	if (rpmsqEnable(SIGQUIT, NULL) < 0) {
-	    (void) SUB_REF (rpmsigTbl_sigchld);
+	    xx = SUB_REF (rpmsigTbl_sigchld);
 	    goto out_restore_sigint;
 	}
     }
-    (void) DO_UNLOCK ();
+    xx = DO_UNLOCK ();
 
     (void) sigemptyset (&newMask);
     (void) sigaddset (&newMask, SIGCHLD);
     if (sigprocmask (SIG_BLOCK, &newMask, &oldMask) < 0) {
-	(void) DO_LOCK ();
+	xx = DO_LOCK ();
 	if (SUB_REF (rpmsigTbl_sigchld) == 0)
 	    goto out_restore_sigquit_and_sigint;
 	goto out;
@@ -768,7 +794,7 @@ rpmsqExecve (const char ** argv)
 
     CLEANUP_RESET(0, oldtype);
 
-    (void) DO_LOCK ();
+    xx = DO_LOCK ();
     if ((SUB_REF (rpmsigTbl_sigchld) == 0 &&
         (rpmsqEnable(-SIGINT, NULL) < 0 || rpmsqEnable (-SIGQUIT, NULL) < 0))
       || sigprocmask (SIG_SETMASK, &oldMask, NULL) != 0)
@@ -778,10 +804,10 @@ rpmsqExecve (const char ** argv)
     goto out;
 
 out_restore_sigquit_and_sigint:
-    (void) rpmsqEnable(-SIGQUIT, NULL);
+    xx = rpmsqEnable(-SIGQUIT, NULL);
 out_restore_sigint:
-    (void) rpmsqEnable(-SIGINT, NULL);
+    xx = rpmsqEnable(-SIGINT, NULL);
 out:
-    (void) DO_UNLOCK ();
+    xx = DO_UNLOCK ();
     return status;
 }
