@@ -5,6 +5,7 @@
  * \file rpmio/rpmssl.h
  */
 
+#include <rpmiotypes.h>
 #include <rpmpgp.h>
 #include <rpmsw.h>
 
@@ -35,8 +36,6 @@ struct rpmssl_s {
     BIGNUM * rsahm;
 
     BIGNUM * c;
-
-    int type;
 };
 #endif
 
