@@ -3,6 +3,7 @@
  */
 
 #include "system.h"
+#include <rpmiotypes.h>
 #include <rpmsw.h>
 #include "debug.h"
 
@@ -15,6 +16,9 @@ extern int nanosleep(const struct timespec *__requested_time,
 /*@=exportheader@*/
 #endif
 
+/*@unchecked@*/
+int _rpmsw_stats = 0;
+
 /*@unchecked@*/
 static rpmtime_t rpmsw_overhead = 0;
 
@@ -27,7 +31,7 @@ static int rpmsw_type = 0;
 /*@unchecked@*/
 static int rpmsw_initialized = 0;
 
-#if defined(__linux__) && defined(__i386__)
+#if defined(__linux__) && defined(__i386__) && !defined(RPM_VENDOR_PLD)
 /* Swiped from glibc-2.3.2 sysdeps/i386/i686/hp-timing.h */
 
 #define	HP_TIMING_ZERO(Var)	(Var) = (0)
@@ -118,7 +122,7 @@ rpmtime_t tvsub(/*@null@*/ const struct timeval * etv,
 
 rpmtime_t rpmswDiff(rpmsw end, rpmsw begin)
 {
-    uint64_t ticks = 0;
+    rpmuint64_t ticks = 0;
 
     if (end == NULL || begin == NULL)
 	return 0;
@@ -184,7 +188,7 @@ rpmtime_t rpmswInit(void)
 #if defined(HP_TIMING_NOW)
     rpmtime_t cycles;
     rpmtime_t sum_usecs = 0;
-    uint64_t sum_cycles = 0;
+    rpmuint64_t sum_cycles = 0;
 #endif
     int i;
 
@@ -222,7 +226,8 @@ rpmtime_t rpmswInit(void)
 	rpmsw_type = 1;
 
 	/* Compute cycles/usec */
-	rpmsw_cycles = sum_cycles/sum_usecs;
+	if (sum_usecs > 0)	/* XXX insure that time has passed. */
+	    rpmsw_cycles = sum_cycles/sum_usecs;
 #else
 	rpmsw_type = 0;
 #endif
@@ -295,3 +300,15 @@ rpmtime_t rpmswSub(rpmop to, rpmop from)
     }
     return usecs;
 }
+
+void rpmswPrint(const char * name, /*@null@*/ rpmop op)
+        /*@globals fileSystem @*/
+        /*@modifies fileSystem @*/
+{
+    static unsigned int scale = (1000 * 1000);
+    if (op != NULL && op->count > 0)
+        fprintf(stderr, "   %s %8d %6lu.%06lu MB %6lu.%06lu secs\n",
+                name, op->count,
+                (unsigned long)op->bytes/scale, (unsigned long)op->bytes%scale,
+                op->usecs/scale, op->usecs%scale);
+}
