@@ -1,12 +1,8 @@
 #include "system.h"
 
-#if HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include "tiger.h"
-#include "mp.h"
-#include "endianness.h"
+#include <beecrypt/mp.h>
+#include <beecrypt/endianness.h>
 
 #include "debug.h"
 
@@ -628,7 +624,7 @@ void  tigerProcess(tigerParam *mp)
 	uint64_t * w, x[8];
 
 	memcpy(x, mp->data, sizeof(x));
-	#if WORDS_BIGENDIAN
+	#ifdef WORDS_BIGENDIAN
 	{	register int t = 8;
 		w = x;
 		while (t--) {
@@ -847,7 +843,7 @@ Hash of "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-ABCDEFG
         00B83EB4E53440C5 76AC6AAEE0A74858 25FD15E70A59FFE4
 */
 
-int main() {
+int main(int argc, char *argv[]) {
     int rc = tiger_test();
     switch(rc) {
     case 0:	printf("OK\n");		break;
