@@ -31,6 +31,18 @@
 #define	IPPORT_PGPKEYSERVER	11371
 #endif
 
+/**
+ */
+/*@-redecl@*/
+int (*urlNotify) (const urlinfo u, unsigned status)
+        /*@*/;
+/*@=redecl@*/
+
+/**
+ */
+/*@unchecked@*/ /*@null@*/
+void * urlNotifyArg;
+
 /**
  */
 /*@unchecked@*/
@@ -52,92 +64,61 @@ int _url_debug = 0;
 /*@only@*/ /*@null@*/
 urlinfo *_url_cache = NULL;
 
-/**
- */
-/*@unchecked@*/
-int _url_count = 0;
-
-urlinfo XurlLink(urlinfo u, const char *msg, const char *file, unsigned line)
-{
-    URLSANE(u);
-    u->nrefs++;
-/*@-modfilesys@*/
-URLDBGREFS(0, (stderr, "--> url %p ++ %d %s at %s:%u\n", u, u->nrefs, msg, file, line));
-/*@=modfilesys@*/
-    /*@-refcounttrans@*/ return u; /*@=refcounttrans@*/
-}
-
-urlinfo XurlNew(const char *msg, const char *file, unsigned line)
-{
-    urlinfo u;
-    if ((u = xmalloc(sizeof(*u))) == NULL)
-	return NULL;
-    memset(u, 0, sizeof(*u));
-    u->proxyp = -1;
-    u->port = -1;
-    u->urltype = URL_IS_UNKNOWN;
-    u->ctrl = NULL;
-    u->data = NULL;
-    u->bufAlloced = 0;
-    u->buf = NULL;
-    u->allow = RPMURL_SERVER_HASRANGE;
-    u->httpVersion = 0;
-    u->nrefs = 0;
-    u->magic = URLMAGIC;
-    return XurlLink(u, msg, file, line);
-}
-
-urlinfo XurlFree(urlinfo u, const char *msg, const char *file, unsigned line)
+static void urlFini(void * _u)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies _u, fileSystem, internalState @*/
 {
+    urlinfo u =_u;
     int xx;
 
-    URLSANE(u);
-URLDBGREFS(0, (stderr, "--> url %p -- %d %s at %s:%u\n", u, u->nrefs, msg, file, line));
-    if (--u->nrefs > 0)
-	/*@-refcounttrans -retalias@*/ return u; /*@=refcounttrans =retalias@*/
     if (u->ctrl) {
 #ifndef	NOTYET
 	void * fp = fdGetFp(u->ctrl);
 	if (fp) {
 	    fdPush(u->ctrl, fpio, fp, -1);   /* Push fpio onto stack */
-	    (void) Fclose(u->ctrl);
-	} else if (fdio->_fileno(u->ctrl) >= 0)
+	    xx = Fclose(u->ctrl);
+	} else if (fdFileno(u->ctrl) >= 0)
 	    xx = fdio->close(u->ctrl);
 #else
-	(void) Fclose(u->ctrl);
+	xx = Fclose(u->ctrl);
 #endif
 
-	u->ctrl = fdio->_fdderef(u->ctrl, "persist ctrl (urlFree)", file, line);
-	/*@-usereleased@*/
+/*@-usereleased@*/
+	u->ctrl = (FD_t)rpmioFreePoolItem((rpmioItem)u->ctrl, "persist ctrl (urlFree)", __FILE__, __LINE__);
 	if (u->ctrl)
 	    fprintf(stderr, _("warning: u %p ctrl %p nrefs != 0 (%s %s)\n"),
 			u, u->ctrl, (u->host ? u->host : ""),
 			(u->scheme ? u->scheme : ""));
-	/*@=usereleased@*/
+/*@=usereleased@*/
     }
     if (u->data) {
 #ifndef	NOTYET
 	void * fp = fdGetFp(u->data);
 	if (fp) {
 	    fdPush(u->data, fpio, fp, -1);   /* Push fpio onto stack */
-	    (void) Fclose(u->data);
-	} else if (fdio->_fileno(u->data) >= 0)
+	    xx = Fclose(u->data);
+	} else if (fdFileno(u->data) >= 0)
 	    xx = fdio->close(u->data);
 #else
-	(void) Fclose(u->ctrl);
+	xx = Fclose(u->ctrl);
 #endif
 
-	u->data = fdio->_fdderef(u->data, "persist data (urlFree)", file, line);
-	/*@-usereleased@*/
+/*@-usereleased@*/
+	u->data = (FD_t)rpmioFreePoolItem((rpmioItem)u->data, "persist data (urlFree)", __FILE__, __LINE__);
 	if (u->data)
 	    fprintf(stderr, _("warning: u %p data %p nrefs != 0 (%s %s)\n"),
 			u, u->data, (u->host ? u->host : ""),
 			(u->scheme ? u->scheme : ""));
-	/*@=usereleased@*/
+/*@=usereleased@*/
     }
 #ifdef WITH_NEON
     xx = davFree(u);
 #endif
+    u->etag = _free(u->etag);
+    u->location = _free(u->location);
+    u->rop = _free(u->rop);
+    u->sop = _free(u->sop);
+    u->top = _free(u->top);
     u->buf = _free(u->buf);
     u->url = _free(u->url);
     u->scheme = _free((void *)u->scheme);
@@ -145,11 +126,58 @@ URLDBGREFS(0, (stderr, "--> url %p -- %d %s at %s:%u\n", u, u->nrefs, msg, file,
     u->password = _free((void *)u->password);
     u->host = _free((void *)u->host);
     u->portstr = _free((void *)u->portstr);
+    u->query = _free(u->query);
+    u->fragment = _free(u->fragment);
     u->proxyu = _free((void *)u->proxyu);
     u->proxyh = _free((void *)u->proxyh);
+}
 
-    /*@-refcounttrans@*/ u = _free(u); /*@-refcounttrans@*/
-    return NULL;
+/**
+ */
+/*@unchecked@*/
+int _url_count = 0;
+
+/*@unchecked@*/ /*@only@*/ /*@null@*/
+rpmioPool _urlPool;
+
+static urlinfo urlGetPool(/*@null@*/ rpmioPool pool)
+	/*@globals _urlPool, fileSystem @*/
+	/*@modifies pool, _urlPool, fileSystem @*/
+{
+    urlinfo u;
+
+    if (_urlPool == NULL) {
+	_urlPool = rpmioNewPool("u", sizeof(*u), -1, _url_debug,
+			NULL, NULL, urlFini);
+	pool = _urlPool;
+    }
+    return (urlinfo) rpmioGetPool(pool, sizeof(*u));
+}
+
+urlinfo XurlNew(const char *msg, const char *fn, unsigned ln)
+{
+    urlinfo u = urlGetPool(_urlPool);
+
+    u->proxyp = -1;
+    u->port = -1;
+    u->urltype = URL_IS_UNKNOWN;
+    u->ctrl = NULL;
+    u->data = NULL;
+    u->location = NULL;
+    u->etag = NULL;
+    u->notify = urlNotify;
+/*@-assignexpose@*/
+    u->arg = urlNotifyArg;
+/*@=assignexpose@*/
+    u->rop = xcalloc(1, sizeof(*u->rop));
+    u->sop = xcalloc(1, sizeof(*u->sop));
+    u->top = xcalloc(1, sizeof(*u->top));
+    u->bufAlloced = 0;
+    u->buf = NULL;
+    u->allow = RPMURL_SERVER_HASRANGE;
+    u->httpVersion = 0;
+    u->magic = URLMAGIC;
+    return (urlinfo) rpmioLinkPoolItem((rpmioItem)u, msg, fn, ln);
 }
 
 void urlFreeCache(void)
@@ -159,12 +187,15 @@ void urlFreeCache(void)
 	for (i = 0; i < _url_count; i++) {
 	    if (_url_cache[i] == NULL) continue;
 	    _url_cache[i] = urlFree(_url_cache[i], "_url_cache");
-	    if (_url_cache[i])
-		fprintf(stderr,
-			_("warning: _url_cache[%d] %p nrefs(%d) != 1 (%s %s)\n"),
-			i, _url_cache[i], _url_cache[i]->nrefs,
-			(_url_cache[i]->host ? _url_cache[i]->host : ""),
-			(_url_cache[i]->scheme ? _url_cache[i]->scheme : ""));
+	    if (_url_cache[i] == NULL)
+		continue;
+	    yarnPossess(_url_cache[i]->_item.use);
+	    fprintf(stderr,
+		_("warning: _url_cache[%d] %p nrefs(%ld) != 1 (%s %s)\n"),
+		i, _url_cache[i], yarnPeekLock(_url_cache[i]->_item.use),
+		(_url_cache[i]->host ? _url_cache[i]->host : ""),
+		(_url_cache[i]->scheme ? _url_cache[i]->scheme : ""));
+	    yarnRelease(_url_cache[i]->_item.use);
 	}
     }
     _url_cache = _free(_url_cache);
@@ -242,6 +273,7 @@ static void urlFind(/*@null@*/ /*@in@*/ /*@out@*/ urlinfo * uret, int mustAsk)
     /*@-usereleased@*/
     u = urlFree(u, "_url_cache (urlFind)");
     /*@=usereleased@*/
+assert(u != NULL);
 
     /* Zap proxy host and port in case they have been reset */
     u->proxyp = -1;
@@ -405,11 +437,26 @@ urltype urlPath(const char * url, const char ** pathp)
     return urltype;
 }
 
+/**
+ * Copy a URL, adding extra byte for the pesky trailing '/'.
+ */
+static const char * urlStrdup(const char * url)
+	/*@*/
+{
+    size_t nb = strlen(url);
+    char * t = xmalloc(nb + 1 + 1);
+    const char * nurl = t;
+    while (*url != '\0')
+	*t++ = *url++;
+    *t = '\0';
+    return nurl;
+}
+
 /*
  * Split URL into components. The URL can look like
  *	scheme://user:password@host:port/path
   * or as in RFC2732 for IPv6 address
-  *    service://user:password@[ip:v6:ad:dr:es:s]:port/path
+  *    service://user:password@[ip:v6:ad:dr:es:s]:port/path?query#fragment
  */
 /*@-modfilesys@*/
 int urlSplit(const char * url, urlinfo *uret)
@@ -423,14 +470,20 @@ int urlSplit(const char * url, urlinfo *uret)
     if ((u = urlNew("urlSplit")) == NULL)
 	return -1;
 
-    if ((se = s = myurl = xstrdup(url)) == NULL) {
-	u = urlFree(u, "urlSplit (error #1)");
-	return -1;
+    myurl = xstrdup(url);
+    if ((se = strrchr(myurl, '#')) != NULL) {
+	*se++ = '\0';
+	u->fragment = xstrdup(se);
+    }
+    if ((se = strrchr(myurl, '?')) != NULL) {
+	*se++ = '\0';
+	u->query = xstrdup(se);
     }
 
-    u->url = xstrdup(url);
-    u->urltype = urlIsURL(url);
+    u->url = urlStrdup(myurl);		/* XXX +1 byte for pesky trailing '/' */
+    u->urltype = urlIsURL(myurl);
 
+    se = s = myurl;
     while (1) {
 	/* Point to end of next item */
 	while (*se && *se != '/') se++;
@@ -465,12 +518,12 @@ int urlSplit(const char * url, urlinfo *uret)
 
     /* Look for ...host:port or [v6addr]:port*/
     fe = f = s;
-    if (strchr(fe, '[') && strchr(fe, ']'))
-    {
-	    fe = strchr(f, ']');
-	    *f++ = '\0';
-	    *fe++ = '\0';
+    if (strchr(fe, '[') && strchr(fe, ']')) {
+	fe = strchr(f, ']');
+	*f++ = '\0';
+	*fe++ = '\0';
     }
+assert(fe != NULL);	/* XXX can't happen */
     while (*fe && *fe != ':') fe++;
     if (*fe == ':') {
 	*fe++ = '\0';
@@ -524,9 +577,7 @@ int urlGetFile(const char * url, const char * dest)
     FD_t tfd = NULL;
     const char * sfuPath = NULL;
     int urlType = urlPath(url, &sfuPath);
-#if defined(RPM_VENDOR_OPENPKG) /* support-external-download-command */
     char *result;
-#endif
 
     if (*sfuPath == '\0')
 	return FTPERR_UNKNOWN;
@@ -540,10 +591,10 @@ int urlGetFile(const char * url, const char * dest)
     if (dest == NULL)
 	return FTPERR_UNKNOWN;
 
-#if defined(RPM_VENDOR_OPENPKG) /* support-external-download-command */
+/*@-globs -mods@*/	/* Avoid including <rpmmacro.h> everywhere for now */
     if (rpmExpandNumeric("%{?__urlgetfile:1}%{!?__urlgetfile:0}")) {
-        result = rpmExpand("%(%{__urlgetfile ", url, " ", dest, "}; echo $?)", NULL);
-        if (result != NULL && strcmp(result, "0") == 0)
+        result = rpmExpand("%{__urlgetfile ", url, " ", dest, "}", NULL);
+        if (result != NULL && strcmp(result, "OK") == 0)
             rc = 0;
         else {
             rpmlog(RPMLOG_DEBUG, D_("failed to fetch URL %s via external command\n"), url);
@@ -552,9 +603,9 @@ int urlGetFile(const char * url, const char * dest)
         result = _free(result);
         goto exit;
     }
-#endif
+/*@=globs =mods@*/
 
-    sfd = Fopen(url, "r");
+    sfd = Fopen(url, "r.ufdio");
     if (sfd == NULL || Ferror(sfd)) {
 	rpmlog(RPMLOG_DEBUG, D_("failed to open %s: %s\n"), url, Fstrerror(sfd));
 	rc = FTPERR_UNKNOWN;
