@@ -14,12 +14,21 @@ extern const char *__progname;
 #endif
 
 #if defined(RPM_VENDOR_OPENPKG) /* integrity-checking */
+#define	_RPMIOB_INTERNAL	/* XXX rpmiobSlurp */
 #include "rpmio_internal.h"
 #endif
 
 #include <rpmio.h>
-#include <rpmcli.h>
-#include <rpmbuild.h>
+#include <rpmiotypes.h>
+#include <poptIO.h>
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
+#include "rpmdb.h"
+
+#if defined(IAM_RPMBT) || defined(IAM_RPMK)
+#include "signature.h"
+#endif
 
 #if defined(RPM_VENDOR_OPENPKG) /* integrity-checking */
 #include "rpmns.h"
@@ -28,21 +37,22 @@ extern const char *__progname;
 #include "rpmluaext.h"
 #endif
 
-#include "rpmdb.h"
+#include "rpmversion.h"
 #include "rpmps.h"
 #include "rpmts.h"
 
 #include "fs.h"		/* XXX for rpmFreeFilesystems() */
 
+#include <rpmbuild.h>
+
 #ifdef	IAM_RPMBT
 #include "build.h"
 #define GETOPT_REBUILD		1003
 #define GETOPT_RECOMPILE	1004
 #endif
 
-#if defined(IAM_RPMBT) || defined(IAM_RPMK)
-#include "signature.h"
-#endif
+#include <rpmcli.h>
+#include <rpmrollback.h>
 
 #include "debug.h"
 
@@ -105,7 +115,7 @@ static struct poptOption optionsTable[] = {
 #endif	/* IAM_RPMQV */
 
 #ifdef	IAM_RPMQV
- { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmcliFtsPoptTable, 0,
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmioFtsPoptTable, 0,
         N_("File tree walk options (with --ftswalk):"),
         NULL },
 #endif	/* IAM_RPMQV */
@@ -211,6 +221,8 @@ static void integrity_check(const char *progname, enum modes progmode_num)
     char *pkey_fn = NULL;
     char *spec = NULL;
     char *proc = NULL;
+    rpmiob spec_iob = NULL;
+    rpmiob proc_iob = NULL;
     const char *result = NULL;
     const char *error = NULL;
     int xx;
@@ -256,22 +268,22 @@ static void integrity_check(const char *progname, enum modes progmode_num)
     }
 
     /* load integrity configuration specification file */
-    spec = NULL;
-	xx = rpmioSlurp(spec_fn, (uint8_t **)&spec, NULL);
-	if (!(xx == 0 && spec != NULL)) {
+    xx = rpmiobSlurp(spec_fn, &spec_iob);
+    if (!(xx == 0 && spec_iob != NULL)) {
         integrity_check_message("ERROR: Unable to load Integrity Configuration Specification file.\n"
             "rpm: HINT: Check file \"%s\".\n", spec_fn);
         goto failure;
     }
+    spec = rpmiobStr(spec_iob);
 
     /* load integrity validation processor file */
-    proc = NULL;
-	xx = rpmioSlurp(proc_fn, (uint8_t **)&proc, NULL);
-	if (!(xx == 0 && proc != NULL)) {
+    xx = rpmiobSlurp(proc_fn, &proc_iob);
+    if (!(xx == 0 && proc_iob != NULL)) {
         integrity_check_message("ERROR: Unable to load Integrity Validation Processor file.\n"
             "rpm: HINT: Check file \"%s\".\n", proc_fn);
         goto failure;
     }
+    proc = rpmiobStr(proc_iob);
 
     /* provision program name and mode */
     if (progname == NULL || progname[0] == '\0')
@@ -342,17 +354,19 @@ static void integrity_check(const char *progname, enum modes progmode_num)
     /* cleanup processing */
     failure:
     if (lua != NULL)
-        rpmluaFree(lua);
+	rpmluaFree(lua);
     if (ts != NULL)
-        ts = rpmtsFree(ts);
-    if (spec != NULL)
-        spec = _free(spec);
-    if (proc != NULL)
-        proc = _free(proc);
+	(void)rpmtsFree(ts);
+    ts = NULL;
+    if (spec_iob != NULL)
+	spec_iob = rpmiobFree(spec_iob);
+    if (proc_iob != NULL)
+	proc_iob = rpmiobFree(proc_iob);
 
     /* final result handling */
     if (rc != INTEGRITY_OK) {
-        sleep(4);
+        if (isatty(STDIN_FILENO) || isatty(STDOUT_FILENO))
+            sleep(4);
         if (rc == INTEGRITY_ERROR)
             exit(42);
     }
@@ -406,7 +420,7 @@ int main(int argc, const char ** argv)
     int status;
     int p[2];
 #ifdef	IAM_RPMEIU
-    int i;
+    int xx;
 #endif
 	
 #if !defined(__GLIBC__) && !defined(__LCLINT__)
@@ -641,19 +655,19 @@ int main(int argc, const char ** argv)
 		 "erasure, and building"));
 #endif	/* IAM_RPMEIU */
 
-    if (rpmcliRootDir && rpmcliRootDir[1] && (bigMode & ~MODES_FOR_ROOT))
+    if (rpmioRootDir && rpmioRootDir[1] && (bigMode & ~MODES_FOR_ROOT))
 	argerror(_("--root (-r) may only be specified during "
 		 "installation, erasure, querying, and "
 		 "database rebuilds"));
 
-    if (rpmcliRootDir) {
-	switch (urlIsURL(rpmcliRootDir)) {
+    if (rpmioRootDir) {
+	switch (urlIsURL(rpmioRootDir)) {
 	default:
 	    if (bigMode & MODES_FOR_ROOT)
 		break;
 	    /*@fallthrough@*/
 	case URL_IS_UNKNOWN:
-	    if (rpmcliRootDir[0] != '/')
+	    if (rpmioRootDir[0] != '/')
 		argerror(_("arguments to --root (-r) must begin with a /"));
 	    break;
 	}
@@ -698,7 +712,15 @@ int main(int argc, const char ** argv)
 		goto exit;
 	    }
 
-            if (poptPeekArg(optCon)) {
+            if (poptPeekArg(optCon)
+#if defined(IAM_RPMBT)
+	    && !ba->nopassword 
+#endif
+#if defined(IAM_RPMK)
+	    && !ka->nopassword
+#endif
+	    )
+	    {
 		passPhrase = Getpass(_("Enter pass phrase: "));
 		if (rpmCheckPassPhrase(passPhrase)) {
 		    fprintf(stderr, _("Pass phrase check failed\n"));
@@ -714,14 +736,17 @@ int main(int argc, const char ** argv)
     /*@=branchstate@*/
 #endif	/* IAM_RPMBT || IAM_RPMK */
 
-    if (rpmcliPipeOutput) {
-	(void) pipe(p);
+    if (rpmioPipeOutput) {
+	if (pipe(p) < 0) {
+	    fprintf(stderr, _("creating a pipe for --pipe failed: %m\n"));
+	    goto exit;
+	}
 
 	if (!(pipeChild = fork())) {
 	    (void) close(p[1]);
 	    (void) dup2(p[0], STDIN_FILENO);
 	    (void) close(p[0]);
-	    (void) execl("/bin/sh", "/bin/sh", "-c", rpmcliPipeOutput, NULL);
+	    (void) execl("/bin/sh", "/bin/sh", "-c", rpmioPipeOutput, NULL);
 	    fprintf(stderr, _("exec failed\n"));
 	}
 
@@ -731,7 +756,7 @@ int main(int argc, const char ** argv)
     }
 	
     ts = rpmtsCreate();
-    (void) rpmtsSetRootDir(ts, rpmcliRootDir);
+    (void) rpmtsSetRootDir(ts, rpmioRootDir);
     switch (bigMode) {
 #ifdef	IAM_RPMDB
     case MODE_INITDB:
@@ -761,6 +786,7 @@ int main(int argc, const char ** argv)
     case MODE_REBUILD:
     case MODE_RECOMPILE:
     {	const char * pkg;
+	int nbuilds = 0;
 
         while (!rpmIsVerbose())
 	    rpmIncreaseVerbosity();
@@ -781,10 +807,15 @@ int main(int argc, const char ** argv)
 	while ((pkg = poptGetArg(optCon))) {
 	    const char * specFile = NULL;
 
+	    if (nbuilds++ > 0) {
+		rpmFreeMacros(NULL);
+		rpmFreeRpmrc();
+		(void) rpmReadConfigFiles(NULL, NULL);
+	    }
 	    ba->cookie = NULL;
 	    ec = rpmInstallSource(ts, pkg, &specFile, &ba->cookie);
 	    if (ec == 0) {
-		ba->rootdir = rpmcliRootDir;
+		ba->rootdir = rpmioRootDir;
 		ba->passPhrase = passPhrase;
 		ec = build(ts, specFile, ba, NULL);
 	    }
@@ -800,6 +831,8 @@ int main(int argc, const char ** argv)
     case MODE_BUILD:
     case MODE_TARBUILD:
     {	const char * pkg;
+	int nbuilds = 0;
+
 #if defined(RPM_VENDOR_OPENPKG) /* no-auto-verbose-increase-for-track-and-fetch */
 	if (ba->buildChar != 't' && ba->buildChar != 'f')
 #endif
@@ -836,7 +869,7 @@ int main(int argc, const char ** argv)
 	    /*@innerbreak@*/ break;
 	case 's':
 	    ba->buildAmount |= RPMBUILD_PACKAGESOURCE;
-#if defined(RPM_VENDOR_OPENPKG) /* no-deps-on-building-srpms */
+#if defined(RPM_VENDOR_OPENPKG) || defined(RPM_VENDOR_MANDRIVA) || defined(RPM_VENDOR_ARK) /* no-deps-on-building-srpms */
 	    /* enforce no dependency checking when rolling a source RPM */
 	    ba->noDeps = 1;
 #endif
@@ -849,12 +882,10 @@ int main(int argc, const char ** argv)
 	    rpmDefineMacro(NULL, "patch #", RMIL_CMDLINE);
 	    rpmDefineMacro(NULL, "prep %%prep", RMIL_CMDLINE);
 	    /*@innerbreak@*/ break;
-#if defined(RPM_VENDOR_OPENPKG) /* explicit-source-fetch-cli-option */
 	case 'f':
 	    ba->buildAmount |= RPMBUILD_FETCHSOURCE;
 	    ba->noDeps = 1;
 	    /*@innerbreak@*/ break;
-#endif
 	}
 
 	if (!poptPeekArg(optCon)) {
@@ -865,7 +896,12 @@ int main(int argc, const char ** argv)
 	}
 
 	while ((pkg = poptGetArg(optCon))) {
-	    ba->rootdir = rpmcliRootDir;
+	    if (nbuilds++ > 0) {
+		rpmFreeMacros(NULL);
+		rpmFreeRpmrc();
+		(void) rpmReadConfigFiles(NULL, NULL);
+	    }
+	    ba->rootdir = rpmioRootDir;
 	    ba->passPhrase = passPhrase;
 	    ba->cookie = NULL;
 	    ec = build(ts, pkg, ba, NULL);
@@ -911,16 +947,13 @@ ia->rbRun = rpmcliInstallRun;
 	/* we've already ensured !(!ia->prefix && !ia->relocations) */
 	/*@-branchstate@*/
 	if (ia->qva_prefix) {
-	    ia->relocations = xmalloc(2 * sizeof(*ia->relocations));
-	    ia->relocations[0].oldPath = NULL;   /* special case magic */
-	    ia->relocations[0].newPath = ia->qva_prefix;
-	    ia->relocations[1].oldPath = NULL;
-	    ia->relocations[1].newPath = NULL;
+	    xx = rpmfiAddRelocation(&ia->relocations, &ia->nrelocations,
+			NULL, ia->qva_prefix);
+	    xx = rpmfiAddRelocation(&ia->relocations, &ia->nrelocations,
+			NULL, NULL);
 	} else if (ia->relocations) {
-	    ia->relocations = xrealloc(ia->relocations, 
-			sizeof(*ia->relocations) * (ia->numRelocations + 1));
-	    ia->relocations[ia->numRelocations].oldPath = NULL;
-	    ia->relocations[ia->numRelocations].newPath = NULL;
+	    xx = rpmfiAddRelocation(&ia->relocations, &ia->nrelocations,
+			NULL, NULL);
 	}
 	/*@=branchstate@*/
 
@@ -1019,7 +1052,8 @@ ia->rbRun = rpmcliInstallRun;
 exit:
 #endif	/* IAM_RPMBT || IAM_RPMK */
 
-    ts = rpmtsFree(ts);
+    (void)rpmtsFree(ts); 
+    ts = NULL;
 
     if (pipeChild) {
 	(void) fclose(stdout);
@@ -1035,16 +1069,14 @@ exit:
 #endif
 
 #ifdef	IAM_RPMEIU
-    if (ia->relocations != NULL)
-    for (i = 0; i < ia->numRelocations; i++)
-	ia->relocations[i].oldPath = _free(ia->relocations[i].oldPath);
-    ia->relocations = _free(ia->relocations);
+    ia->relocations = rpmfiFreeRelocations(ia->relocations);
 #endif
 
     optCon = rpmcliFini(optCon);
 
     /* XXX don't overflow single byte exit status */
-    if (ec > 255) ec = 255;
+    /* XXX status 255 is special to xargs(1) */
+    if (ec > 254) ec = 254;
 
     /*@-globstate@*/
     return ec;
