@@ -399,8 +399,8 @@ syck_emit( SyckEmitter *e, st_data_t n )
 
     /* Look for anchor */
     if ( e->anchors != NULL &&
-        st_lookup( e->markers, n, (st_data_t *)&oid ) &&
-        st_lookup( e->anchors, (st_data_t)oid, (st_data_t *)&anchor_name ) )
+        st_lookup( e->markers, n, (void *)&oid ) &&
+        st_lookup( e->anchors, (st_data_t)oid, (void *)&anchor_name ) )
     {
         if ( e->anchored == NULL )
         {
@@ -409,7 +409,7 @@ syck_emit( SyckEmitter *e, st_data_t n )
 assert(e->anchored != NULL);
 assert(anchor_name != NULL);
 
-        if ( ! st_lookup( e->anchored, (st_data_t)anchor_name, (st_data_t *)&x ) )
+        if ( ! st_lookup( e->anchored, (st_data_t)anchor_name, (void *)&x ) )
         {
             char *an = S_ALLOC_N( char, strlen( anchor_name ) + 3 );
             sprintf( an, "&%s ", anchor_name );
@@ -829,7 +829,7 @@ void syck_emit_1quoted( SyckEmitter *e, int width, char *str, long len )
     char *end = str;
     syck_emitter_write( e, "'", 1 );
     while ( mark < str + len ) {
-        if ( do_indent ) {
+        if ( do_indent != 0 ) {
             syck_emit_indent( e );
             do_indent = 0;
         }
@@ -1244,7 +1244,7 @@ assert(e->markers != NULL);
      * object.  Doesn't yet create an anchor, simply notes the
      * position.
      */
-    if ( ! st_lookup( e->markers, n, (st_data_t *)&oid ) )
+    if ( ! st_lookup( e->markers, n, (void *)&oid ) )
     {
         /*
          * Store all markers
@@ -1260,7 +1260,7 @@ assert(e->markers != NULL);
         }
 assert(e->anchors != NULL);
 
-        if ( ! st_lookup( e->anchors, (st_data_t)oid, (st_data_t *)&anchor_name ) )
+        if ( ! st_lookup( e->anchors, (st_data_t)oid, (void *)&anchor_name ) )
         {
             int idx = 0;
             char *anc = ( e->anchor_format == NULL ? DEFAULT_ANCHOR_FORMAT : e->anchor_format );
