@@ -9,13 +9,11 @@
 #include "config.h"
 #endif
 
+#if HAVE_SYSLOG_H
+#include <syslog.h>
+#endif
+
 #include <sys/types.h>
-#if defined(HAVE_STDINT_H)
-#include <stdint.h>
-#endif
-#if defined(HAVE_INTTYPES_H)
-#include <inttypes.h>
-#endif
 
 #if defined(__LCLINT__)
 /*@-redef@*/
@@ -64,7 +62,7 @@ extern char ** environ;
 #endif
 #endif
 
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
@@ -262,6 +260,10 @@ extern int _tolower(int) __THROW	/*@*/;
 #include <sys/mman.h>
 #endif
 
+#if !defined(MAP_ANONYMOUS) && defined(MAP_ANON)
+#define MAP_ANONYMOUS MAP_ANON
+#endif
+
 /* XXX FIXME: popt on sunos4.1.3: <sys/resource.h> requires <sys/time.h> */
 #if defined(HAVE_SYS_RESOURCE_H) && defined(HAVE_SYS_TIME_H)
 #include <sys/resource.h>
@@ -302,7 +304,7 @@ extern int _tolower(int) __THROW	/*@*/;
 #else
 typedef	char * security_context_t;
 
-#define	freecon(_c)
+#define	freecon(_c) do {} while(0)
 
 #define	getfilecon(_fn, _c)	(-1)
 #define	lgetfilecon(_fn, _c)	(-1)
@@ -320,7 +322,7 @@ typedef	char * security_context_t;
 
 #define matchpathcon_init(_fn)	(-1)
 #define matchpathcon(_fn, _fmode, _s)	(-1)
-#define	matchpathcon_fini()
+#define	matchpathcon_fini() do {} while(0)
 #endif
 
 #if defined(WITH_SELINUX) && defined(__LCLINT__)
@@ -504,8 +506,8 @@ extern const char *__progname;
 /* ============== from misc/miscfn.h */
 
 /*@-noparams@*/
-#include "misc/glob.h"
-#include "misc/fnmatch.h"
+#include "rpmio/glob.h"
+#include "rpmio/fnmatch.h"
 /*@=noparams@*/
 
 #if defined(__LCLINT__)
@@ -725,10 +727,6 @@ static inline const char *rcsid(const char *p) { \
 #include <search.h>
 #endif
 
-#if defined(HAVE_PTHREAD_H) && !defined(__LCLINT__)
-#include <pthread.h>
-#endif
-
 /**
  * makedev() on QNX takes three parameters
  * the additional one (first place) specifies the node for QNET
@@ -741,6 +739,12 @@ static inline const char *rcsid(const char *p) { \
 #define Makedev(x,y)   makedev((x),(y))
 #endif
 
+#if defined(WITH_PTHREADS)
+#if defined(HAVE_PTHREAD_H) && !defined(__LCLINT__)
+#include <pthread.h>
+#endif
+#endif
+
 /**
  * Mark --initdb and --verifydb for destruction.
  */
@@ -752,4 +756,9 @@ static inline const char *rcsid(const char *p) { \
  */
 #undef SUPPORT_IMPLICIT_TAG_DATA_TYPES	/* XXX postpone to rpm-5.1 */
 
+/**
+ * Permit ar(1) payloads. Disabled while rpmio/iosm.c is under development.
+ */
+#undef	SUPPORT_AR_PAYLOADS
+
 #endif	/* H_SYSTEM */
