@@ -9,22 +9,29 @@ const char *__progname;
 #include <fts.h>
 
 #include <rpmio.h>
-#include <rpmcli.h>
+#include <rpmiotypes.h>
+#include <poptIO.h>
+
+#include <rpmtypes.h>
+#include <rpmtag.h>
+
+#include "rpmdb.h"
+
+#include "rpmps.h"
+
+#include "misc.h"	/* XXX rpmMkdirPath */
 
 #define	_RPMGI_INTERNAL
 #include <rpmgi.h>
 
-#include "rpmps.h"
-#include "rpmdb.h"
-
-#include "misc.h"	/* XXX rpmMkdirPath */
+#include <rpmcli.h>
 
 #include "debug.h"
 
 static int _debug = 0;
 
 /* XXX should be flag in ts */
-static int noCache = 0;
+static int noCache = -1;
 
 static ARGV_t ftsSet;
 
@@ -56,8 +63,10 @@ static int nitems = 0;
 static inline Item freeItem(Item item) {
     if (item != NULL) {
 	item->path = _free(item->path);
-	item->this = rpmdsFree(item->this);
-	item->h = headerFree(item->h);
+	(void)rpmdsFree(item->this);
+	item->this = NULL;
+	(void)headerFree(item->h);
+	item->h = NULL;
 	item = _free(item);
     }
     return NULL;
@@ -107,7 +116,7 @@ static int ftsCacheUpdate(rpmts ts)
 {
     HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     uint32_t tid = rpmtsGetTid(ts);
-    rpmdbMatchIterator mi;
+    rpmmi mi;
     unsigned char * md5;
     int rc = 0;
     int xx;
@@ -139,8 +148,8 @@ static int ftsCacheUpdate(rpmts ts)
 	}
         mi = rpmtsInitIterator(ts, RPMTAG_SIGMD5, md5, 16);
 	md5 = _free(md5);
-	rc = rpmdbGetIteratorCount(mi);
-        mi = rpmdbFreeIterator(mi);
+	rc = rpmmiCount(mi);
+        mi = rpmmiFree(mi);
 	if (rc) {
 	    rc = 0;
 	    continue;
@@ -202,6 +211,7 @@ static rpmRC cacheStashLatest(rpmgi gi, Header h)
     struct stat sb, * st;
     int ec = -1;	/* assume not found */
     int i = 0;
+    int xx;
 
     rpmlog(RPMLOG_DEBUG, "============== %s\n", fts->fts_accpath);
 
@@ -252,8 +262,11 @@ static rpmRC cacheStashLatest(rpmgi gi, Header h)
     items[i] = newItem();
     items[i]->path = xstrdup(fts->fts_path);
     st = fts->fts_statp;
-    if (st == NULL && Stat(fts->fts_accpath, &sb) == 0)
+    if (st == NULL || ((long)st & 0xffff0000) == 0L) {
 	st = &sb;
+	memset(st, 0, sizeof(*st));
+	xx = Stat(fts->fts_accpath, &sb);
+    }
 
     if (st != NULL) {
 	items[i]->size = st->st_size;
@@ -273,7 +286,8 @@ static rpmRC cacheStashLatest(rpmgi gi, Header h)
 #endif
 
 exit:
-    add = rpmdsFree(add);
+    (void)rpmdsFree(add);
+    add = NULL;
     return (ec ? RPMRC_NOTFOUND : RPMRC_OK);
 }
 
@@ -446,7 +460,7 @@ static void initGlobs(/*@unused@*/ rpmts ts, const char ** argv)
 	    t = stpcpy(t, arg);
 	    *t++ = '|';
 	}
-	t[-1] = (single ? '\0' : ')');
+	t[-1] = (char)(single ? '\0' : ')');
 	*t = '\0';
     }
 
@@ -516,10 +530,12 @@ static struct poptOption optionsTable[] = {
  { "nolegacy", '\0', POPT_BIT_SET,      &vsflags, RPMVSF_NEEDPAYLOAD,
 	N_("don't verify header+payload signature"), NULL },
 
+ { "cache", '\0', POPT_ARG_VAL,   &noCache, 0,
+	N_("update cache database"), NULL },
  { "nocache", '\0', POPT_ARG_VAL,   &noCache, -1,
 	N_("don't update cache database, only print package paths"), NULL },
 
- { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmcliFtsPoptTable, 0,
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmioFtsPoptTable, 0,
         N_("File tree walk options:"),
         NULL },
 
@@ -535,15 +551,14 @@ static struct poptOption optionsTable[] = {
 int
 main(int argc, char *argv[])
 {
+    poptContext optCon = rpmcliInit(argc, argv, optionsTable);
     rpmts ts = NULL;
     rpmgi gi = NULL;
-    poptContext optCon;
     const char * s;
     int ec = 1;
     rpmRC rpmrc;
     int xx;
 
-    optCon = rpmcliInit(argc, argv, optionsTable);
     if (optCon == NULL)
         exit(EXIT_FAILURE);
 
@@ -580,15 +595,15 @@ main(int argc, char *argv[])
 
     gi = rpmgiNew(ts, RPMDBI_FTSWALK, NULL, 0);
 
-    if (ftsOpts == 0)
-	ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+    if (rpmioFtsOpts == 0)
+	rpmioFtsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
 
     if (noCache)
-	ftsOpts |= FTS_NOSTAT;
+	rpmioFtsOpts |= FTS_NOSTAT;
     else
-	ftsOpts &= ~FTS_NOSTAT;
+	rpmioFtsOpts &= ~FTS_NOSTAT;
 
-    xx = rpmgiSetArgs(gi, ftsSet, ftsOpts, giFlags);
+    xx = rpmgiSetArgs(gi, ftsSet, rpmioFtsOpts, giFlags);
 
     gi->walkPathFilter = cacheWalkPathFilter;
     gi->stash = cacheStashLatest;
@@ -608,7 +623,8 @@ main(int argc, char *argv[])
     freeGlobs();
 
     gi = rpmgiFree(gi);
-    ts = rpmtsFree(ts);
+    (void)rpmtsFree(ts); 
+    ts = NULL;
     optCon = rpmcliFini(optCon);
 
     return ec;
