@@ -62,6 +62,7 @@ static char *Toccksum = NULL;
 static char *Compression = NULL;
 static char *Rsize = NULL;
 static char *CompressionArg = NULL;
+static char *Chdir = NULL;
 
 static int Err = 0;
 static int List = 0;
@@ -302,6 +303,7 @@ static int extract(const char *filename, int arglen, char *args[]) {
 	struct lnode *extract_files = NULL;
 	struct lnode *extract_tail = NULL;
 	struct lnode *lnodei = NULL;
+	struct lnode *dirs = NULL;
 
 	for(argi = 0; args[argi]; argi++) {
 		struct lnode *tmp;
@@ -350,6 +352,13 @@ static int extract(const char *filename, int arglen, char *args[]) {
 		exit(1);
 	}
 
+	if(Chdir) {
+		if( chdir(Chdir) != 0 ) {
+			fprintf(stderr, "Unable to chdir to %s\n", Chdir);
+			exit(1);
+		}
+	}
+
 	xar_register_errhandler(x, err_callback, NULL);
 
 	if( Perms == SYMBOLIC ) {
@@ -409,13 +418,31 @@ static int extract(const char *filename, int arglen, char *args[]) {
 			if( NoOverwrite && (lstat(path, &sb) == 0) ) {
 				printf("%s already exists, not overwriting\n", path);
 			} else {
-				files_extracted++;
-				print_file(x, f);
-				xar_extract(x, f);
+				const char *prop = NULL;
+				int deferred = 0;
+				if( xar_prop_get(f, "type", &prop) == 0 ) {
+					if( strcmp(prop, "directory") == 0 ) {
+						struct lnode *tmpl = calloc(sizeof(struct lnode),1);
+						tmpl->str = (char *)f;
+						tmpl->next = dirs;
+						dirs = tmpl;
+						deferred = 1;
+					}
+				}
+				if( ! deferred ) {
+					files_extracted++;
+					print_file(x, f);
+					xar_extract(x, f);
+				}
 			}
 		}
 		free(path);
 	}
+	for(lnodei = dirs; lnodei; lnodei = lnodei->next) {
+		files_extracted++;
+		print_file(x,(xar_file_t)lnodei->str);
+		xar_extract(x, (xar_file_t)lnodei->str);
+	}
 	if( args[0] && (files_extracted == 0) ) {
 		fprintf(stderr, "No files matched extraction criteria.\n");
 		Err = 3;
@@ -652,6 +679,7 @@ static void usage(const char *prog) {
 	fprintf(stderr, "\t-t               Lists an archive\n");
 	fprintf(stderr, "\t-f <filename>    Specifies an archive to operate on [REQUIRED!]\n");
 	fprintf(stderr, "\t-v               Print filenames as they are archived\n");
+	fprintf(stderr, "\t-C <path>        On extract, chdir to this location\n");
 	fprintf(stderr, "\t-n name          Provides a name for a subdocument\n");
 	fprintf(stderr, "\t-s <filename>    On extract, specifies the file to extract\n");
 	fprintf(stderr, "\t                      subdocuments to.\n");
@@ -669,7 +697,7 @@ static void usage(const char *prog) {
 	fprintf(stderr, "\t                      specified file.\n");
 	fprintf(stderr, "\t--dump-header    Prints out the xar binary header information\n");
 	fprintf(stderr, "\t--compression    Specifies the compression type to use.\n");
-	fprintf(stderr, "\t                      Valid values: none, gzip, bzip2, lzma\n");
+	fprintf(stderr, "\t                      Valid values: none, gzip, bzip2, lzma, xz\n");
 	fprintf(stderr, "\t                      Default: gzip\n");
 	fprintf(stderr, "\t-a               Synonym for \"--compression=lzma\"\n");
 	fprintf(stderr, "\t-j               Synonym for \"--compression=bzip2\"\n");
@@ -688,7 +716,7 @@ static void usage(const char *prog) {
 	fprintf(stderr, "\t                      is not streamable\n");
 	fprintf(stderr, "\t--link-same      Hardlink identical files\n");
 	fprintf(stderr, "\t--no-compress    POSIX regular expression of files\n");
-	fprintf(stderr, "\t                      not to archive, but not compress.\n");
+	fprintf(stderr, "\t                      to archive, but not compress.\n");
 	fprintf(stderr, "\t--prop-include   File properties to include in archive\n");
 	fprintf(stderr, "\t--prop-exclude   File properties to exclude in archive\n");
 	fprintf(stderr, "\t--distribution   Only includes a subset of file properties\n");
@@ -745,7 +773,7 @@ int main(int argc, char *argv[]) {
 		exit(1);
 	}
 
-	while( (c = getopt_long(argc, argv, "axcvtjzf:hpPln:s:d:vk", o, &loptind)) != -1 ) {
+	while( (c = getopt_long(argc, argv, "axcC:vtjzf:hpPln:s:d:vk", o, &loptind)) != -1 ) {
 		switch(c) {
 		case  1 : if( !optarg ) {
 		          	usage(argv[0]);
@@ -767,7 +795,8 @@ int main(int argc, char *argv[]) {
 		          if( (strcmp(optarg, XAR_OPT_VAL_NONE) != 0) &&
 		              (strcmp(optarg, XAR_OPT_VAL_GZIP) != 0) &&
 		              (strcmp(optarg, XAR_OPT_VAL_BZIP) != 0) &&
-		              (strcmp(optarg, XAR_OPT_VAL_LZMA) != 0) ) {
+		              (strcmp(optarg, XAR_OPT_VAL_LZMA) != 0) &&
+		              (strcmp(optarg, XAR_OPT_VAL_XZ) != 0) ) {
 		          	usage(argv[0]);
 		          	exit(1);
 		          }
@@ -901,6 +930,12 @@ int main(int argc, char *argv[]) {
 		case 17 :
 			CompressionArg = optarg;
 			break;
+		case 'C': if( !optarg ) {
+		          	usage(argv[0]);
+		          	exit(1);
+		          }
+		          Chdir = optarg;
+		          break;
 		case 'c':
 		case 'x':
 		case 't':
